
import java.util.HashMap;

/**
 * #################################################################################################
 * #  SpolLineages: simple algorithms for the identification of Mycobacterium tuberculosis complex
 * #   lineages using classical genotyping methods and updated rules
 * # 
 * #  Copyright (C) 2019 - SpolLineages team
 * #  (Institut Pasteur de la Guadeloupe, Universite des Antilles)
 * #
 * #  See the COPYRIGHT file for details.
 * #
 * #  SpolLineages is distributed under the terms of the GNU General Public License
 * #  (GPLv3). See the COPYING file for details.  
 * #################################################################################################
 */

/*
 * Spoligotype43.java

 *
 * Class allowing to manage spoligotyping data for SpolLineages
 * Date: 08/11/2019
 */
/**
 * @author dcouvin
 * SpolLineage version 1.0.1
 * Date: 08/11/2019
 */


public class Spoligotype43 {

	public static String octToBin(String valOctale){
        if (valOctale.length()!=15) return "unknown";
        String res="";
        for(int i=0;i<14;i++){
            String oct=String.valueOf(valOctale.charAt(i));
            if(oct.equals("7")){res+=("nnn");}
            else if(oct.equals("6")){res+=("nno");}
            else if(oct.equals("5")){res+=("non");} 
            else if(oct.equals("4")){res+=("noo");} 
            else if(oct.equals("3")){res+=("onn");} 
            else if(oct.equals("2")){res+=("ono");} 
            else if(oct.equals("1")){res+=("oon");} 
            else if(oct.equals("0")){res+=("ooo");} 
            else{ res+=("?");}
        }
        String last=(String.valueOf(valOctale.charAt(14)));
        String c="";
        if (last.equals("0")) c="o";
        else if (last.equals("1")) c="n";
        else c="?";
        res+=c;
        return res;
    }
    
    public static String binToOct(String valBinaire) throws Exception{
        valBinaire.toLowerCase();
        String oct="";
        for(int i=0;i<14;i++){
            String bin=valBinaire.substring(i*3,(i*3+3));
            if (bin.equals("111") || bin.equals("nnn")) oct+="7";
            else if(bin.equals("110") || bin.equals("nno")) oct+="6" ;
            else if(bin.equals("101") || bin.equals("non")) oct+="5";
            else if(bin.equals("100") || bin.equals("noo")) oct+="4";
            else if(bin.equals("011") || bin.equals("onn")) oct+="3";
            else if(bin.equals("010") || bin.equals("ono")) oct+="2";
            else if(bin.equals("001") || bin.equals("oon")) oct+="1";
            else if(bin.equals("000") || bin.equals("ooo")) oct+="0";
            else if(bin.equals("...")) oct+="[0-7]";
            else if(bin.equals("1..") || bin.equals("n..")) oct+="[4-7]";
            else if(bin.equals(".1.") || bin.equals(".n.")) oct+="[2357]"; // oct+="[2-3]" replaced by oct+="[2357]"
            else if(bin.equals("..1") || bin.equals("..n")) oct+="[1357]"; // oct+="[1347]" replaced by oct+="[1357]"
            else if(bin.equals("0..") || bin.equals("o..")) oct+="[0-3]";
            else if(bin.equals(".0.") || bin.equals(".o.")) oct+="[0145]";
            else if(bin.equals("..0") || bin.equals("..o")) oct+="[0246]";
            else if(bin.equals("11.") || bin.equals("nn.")) oct+="[67]";
            else if(bin.equals("1.1") || bin.equals("n.n")) oct+="[57]";
            else if(bin.equals(".11") || bin.equals(".nn")) oct+="[37]";
            else if(bin.equals("00.") || bin.equals("oo.")) oct+="[01]";
            else if(bin.equals("0.0") || bin.equals("o.o")) oct+="[02]";
            else if(bin.equals(".00") || bin.equals(".oo")) oct+="[04]";
            else if(bin.equals("01.") || bin.equals("on.")) oct+="[23]";
            else if(bin.equals("0.1") || bin.equals("o.n")) oct+="[13]";
            else if(bin.equals(".01") || bin.equals(".on")) oct+="[15]";
            else if(bin.equals("10.") || bin.equals("no.")) oct+="[45]";
            else if(bin.equals("1.0") || bin.equals("n.o")) oct+="[46]";
            else if(bin.equals(".10") || bin.equals(".no")) oct+="[26]";
            
            else throw new Exception("This spoligotype can not be converted to an octal value");
        }
        char last=valBinaire.charAt(42);
        if(last=='1' || last=='n') oct+=1;
        else if(last=='0' || last =='o') oct+=0;
        else if(last=='.') oct+=".";
        else throw new Exception("This spoligotype can not be converted to an octal value");
        return oct;
    }
    
    /**
	 * Spoligotyping binary Rules corresponding to lineages
	 * 
	 */
	public static HashMap<String,String> getLineage() throws Exception{
		HashMap<String,String> myH = new HashMap<String,String>();
		
		myH.put(".......oo.............................on...","AFRI");
		myH.put(".....nooon...........................non...","AFRI_1");
		myH.put("......nooooon......noooon..........nooon...","AFRI_2");
		myH.put("......nooooon......................nooon...","AFRI_3");
		myH.put("ooooooooooooooooooooooooooooooooooooooooooo","ATYPICAL");
		myH.put("oooooooooooooooooooooooooooooooooo.........","Beijing");
		myH.put("..o.....o.............................ooooo","BOV");
		myH.put(".no....non....non.....................ooooo","BOV_1");
		myH.put(".nonnonooooonnnon.....................ooooo","BOV_2");
		myH.put(".nonoooooooooonon.....................ooooo","BOV_3");
		myH.put("o.o............o...........o..........ooooo","BOV_4-CAPRAE");
		myH.put(".....................nooon....n.oooo.......","Cameroon");
		myH.put("ooooooooooooooooooooooooooooonooooonooooooo","CANETTII");
		myH.put("..noooo...............oooooooooooo.........","CAS");
		myH.put("..noooon.............noooooooooooon........","CAS1-Delhi");
		myH.put("..noooonnon.......noooooooooooooooon.......","CAS1-Kili");
		myH.put("..nooooooo...........noooooooooooon........","CAS2");
		myH.put("............................oooono....non..","EAI1-SOM");
		myH.put("............................oooono..oo.on..","EAI1-MYS");
		myH.put(".non...............oo.......oooono.........","EAI2");
		myH.put(".non..............noon......oooono.........","EAI2-Manila");
		myH.put(".non...oooooooooooooooooo...oooono.........","EAI2-Nonthaburi");
		myH.put(".oo.........................oooono.nooon...","EAI3-IND");
		myH.put("........................noonoooono.........","EAI4-VNM");
		myH.put("............................oooono.........","EAI5");
		myH.put(".....................no....noooono.........","EAI6-BGD1");
		myH.put(".......................noooooooooonnon.....","EAI7-BGD2");
		myH.put(".oon.............non........oooono.........","EAI8-MDG");
		myH.put("nnnooooooooooooooooooooon..oo......n.......","Ethiopian");
		myH.put("........................noooooonoooo.......","H1");
		myH.put("oooooooooooooooooooooooonoooooonoooo.......","H2");
		myH.put("..............................onoooo.......","H3");
		myH.put("..o.................oooon.....n.oooo.......","LAM1");
		myH.put("....................oooonnoooonnoooo.......","LAM11-ZWE");
		myH.put(".......noooooon.....oooon.....n.oooo.......","LAM12-Madrid1");
		myH.put("..o.........o.......oooon.....n.oooo.......","LAM2");
		myH.put("........ooo.........oooon.....n.oooo.......","LAM3");
		myH.put("....................oooon.....n.oooo...o...","LAM4");
		myH.put("............o.......oooon.....n.oooo.......","LAM5");
		myH.put("....................oooon..nonnnoooo.......","LAM6");
		myH.put(".............noooooooooo......n.oooo.......","LAM-RUS");
		myH.put("...................ooooonoon..n.oooo.......","Turkey");
		myH.put("....................oooonnon..n.oooo.......","LAM8");
		myH.put("....................oooon.....n.oooo.......","LAM9");
		myH.put("................................non........","Manu1");
		myH.put("...............................noon........","Manu2");
		myH.put("................................nooon......","Manu3");
		myH.put("nnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnn","Manu_ancestor");
		myH.put("oooooooooooooooooooooooooooooooooooonnooooo","microti");
		myH.put("..o...o.o..ooooooooooooooooooooooooo..o....","mungi");
		myH.put("ooo....ooooooooooooooo................ooooo","PINI");
		myH.put("ooo...nooooooooooooooon...............ooooo","PINI1");
		myH.put("oooooooooooooooooooooooon.............ooooo","PINI2");
		myH.put(".......noon...................n.oooo.......","S");
		myH.put("..................noon........nnoooo.......","T-H37Rv");
		myH.put(".............nooooooooon......n.oooo.......","T-Tuscany");
		myH.put("..............................n.oooo.......","T1");
		myH.put(".....noooooooooooon...........n.oooo.......","T1-RUS2");
		myH.put("..............................n.oooo..non..","T2");
		myH.put("..............................n.oooo..nonno","T2-Uganda");
		myH.put("...........non................n.oooo.......","T3");
		myH.put("........noooooooooon..........n.oooo.......","T3-ETH");
		myH.put("...noooon..non................n.oooo.......","T3-OSA");
		myH.put(".................non..........n.oooo.......","T4");
		myH.put(".................non.noon.....n.oooonoon...","T4-CEU1");
		myH.put(".....................non......n.oooo.......","T5");
		myH.put("..................nonnon......n.oooo.......","T5-Madrid2");
		myH.put(".n.........................nooonoooo.......","Ural-1");
		myH.put(".o.........................nooonoooo.......","NEW-1 (misnamed as Ural-2)");
		myH.put("................non...........n.oooo.......","X1");
		myH.put("................non...........n.oooo..oooo.","X2");
		myH.put("..nooooooooo....non...........n.oooo.......","X3");
		myH.put(".................nooooooooooooooooooooooonn","Zero-copy");
		
		return myH;
	}
	/**
	 * Method allowing to get lineage information from sublineage definition
	 * @param subLineage
	 * @return
	 * @throws Exception
	 */
	public static String getLineageFromSublineage(String subLineage) throws Exception{
		HashMap<String,String> myHSub = new HashMap<String,String>();
		
		myHSub.put("AFRI","AFRI");
		myHSub.put("AFRI_1","AFRI");
		myHSub.put("AFRI_2","AFRI");
		myHSub.put("AFRI_3","AFRI");
		myHSub.put("ATYPICAL","ATYPICAL");
		myHSub.put("Beijing","Beijing");
		myHSub.put("BOV","BOV");
		myHSub.put("BOV_1","BOV");
		myHSub.put("BOV_2","BOV");
		myHSub.put("BOV_3","BOV");
		myHSub.put("BOV_4-CAPRAE","BOV");
		myHSub.put("Cameroon","Cameroon");
		myHSub.put("CANETTII","CANETTII");
		myHSub.put("CAS","CAS");
		myHSub.put("CAS1-Delhi","CAS");
		myHSub.put("CAS1-Kili","CAS");
		myHSub.put("CAS2","CAS");
		myHSub.put("EAI1-MYS","EAI");
		myHSub.put("EAI1-SOM","EAI");
		myHSub.put("EAI2","EAI");
		myHSub.put("EAI2-Manila","EAI");
		myHSub.put("EAI2-Nonthaburi","EAI");
		myHSub.put("EAI3-IND","EAI");
		myHSub.put("EAI4-VNM","EAI");
		myHSub.put("EAI5","EAI");
		myHSub.put("EAI6-BGD1","EAI");
		myHSub.put("EAI7-BGD2","EAI");
		myHSub.put("EAI8-MDG","EAI");
		myHSub.put("Ethiopian","Ethiopian");
		myHSub.put("H1","Haarlem");
		myHSub.put("H2","Haarlem");
		myHSub.put("H3","Haarlem");
		myHSub.put("LAM","LAM");
		myHSub.put("LAM1","LAM");
		myHSub.put("LAM11-ZWE","LAM");
		myHSub.put("LAM12-Madrid1","LAM");
		myHSub.put("LAM2","LAM");
		myHSub.put("LAM3","LAM");
		myHSub.put("LAM4","LAM");
		myHSub.put("LAM5","LAM");
		myHSub.put("LAM6","LAM");
		myHSub.put("LAM8","LAM");
		myHSub.put("LAM9","LAM");
		myHSub.put("Manu_ancestor","Manu");
		myHSub.put("Manu1","Manu");
		myHSub.put("Manu2","Manu");
		myHSub.put("Manu3","Manu");
		myHSub.put("microti","microti");
		myHSub.put("mungi","mungi");
		myHSub.put("PINI","PINI");
		myHSub.put("PINI1","PINI");
		myHSub.put("PINI2","PINI");
		myHSub.put("S","S");
		myHSub.put("T","T");
		myHSub.put("T1","T");
		myHSub.put("T1-RUS2","T");
		myHSub.put("T2","T");
		myHSub.put("T2-Uganda","T");
		myHSub.put("T3","T");
		myHSub.put("T3-ETH","T");
		myHSub.put("T3-OSA","T");
		myHSub.put("T4","T");
		myHSub.put("T4-CEU1","T");
		myHSub.put("T5","T");
		myHSub.put("T5-Madrid2","T");
		myHSub.put("LAM-RUS","LAM");
		myHSub.put("T-H37Rv","T");
		myHSub.put("T-Tuscany","T");
		myHSub.put("Turkey","Turkey");
		myHSub.put("Ural-1","Ural");
		myHSub.put("NEW-1 (misnamed as Ural-2)","NEW-1 (misnamed as Ural-2)");
		myHSub.put("X1","X");
		myHSub.put("X2","X");
		myHSub.put("X3","X");
		myHSub.put("Zero-copy","Zero-copy");
		myHSub.put("Unknown","Unknown");
		
		return myHSub.get(subLineage);
		
	}
	
	/**
	 * Method allowing to get SNP-based lineage information from lineage
	 * @param subLineage
	 * @return
	 * @throws Exception
	 */
	public static String getSNPbasedLFromLineage(String subLineage) throws Exception{
		HashMap<String,String> myHSNP = new HashMap<String,String>();
		
		myHSNP.put("S", "Euro-American (L4.4.1.1)");
		myHSNP.put("Beijing", "East-Asian (L2)");
		myHSNP.put("T", "Euro-American (L4)");
		myHSNP.put("Haarlem", "Euro-American (L4.1.2)");
		myHSNP.put("LAM", "Euro-American (L4.3)");
		myHSNP.put("BOV", "M. bovis");
		myHSNP.put("EAI", "Indo-Oceanic (L1)");
		myHSNP.put("CAS", "East-African-Indian (L3)");
		myHSNP.put("Cameroon", "Euro-American (L4.6.2.2)");
		myHSNP.put("X", "Euro-American (L4.1.1)");
		myHSNP.put("Turkey", "Euro-American (L4.2.2.1)");
		myHSNP.put("Ural", "Euro-American (L4.2.1)");
		myHSNP.put("Ethiopian", "Ethiopian (L7)");
		myHSNP.put("AFRI", "West-Africa (L5 and L6)");
		myHSNP.put("NEW-1 (misnamed as Ural-2)", "Euro-American (L4.5)");
		//
		myHSNP.put("ATYPIC", "Unknown");
		myHSNP.put("CANETTII", "M. canettii");
		myHSNP.put("mungi", "M. mungi");
		myHSNP.put("PINI", "M. pinnipedii");
		myHSNP.put("Manu", "Unknown");
		myHSNP.put("microti", "M. microti");
		myHSNP.put("", "Unknown");
		myHSNP.put(null, "Unknown");
		myHSNP.put("null", "Unknown");
		myHSNP.put("Unknown", "Unknown");
		
		return myHSNP.get(subLineage);
		
	}
	
	/**
	 * Method allowing to get SIT from spoligotype
	 * @param spoligotyping pattern
	 * @return
	 * @throws Exception
	 */
	public static String getSITFromSpoligo(String spoligo) throws Exception{
		HashMap<String,String> myHSIT = new HashMap<String,String>();
		
		myHSIT.put("000000000003771","1");
		myHSIT.put("000000004020771","2");
		myHSIT.put("000000007720771","3");
		myHSIT.put("000000007760771","4");
		myHSIT.put("000677777760771","5");
		myHSIT.put("077777777413731","6");
		myHSIT.put("377777777760771","7");
		myHSIT.put("400037777413771","8");
		myHSIT.put("401776777760601","9");
		myHSIT.put("477777277413771","10");
		myHSIT.put("477777777413071","11");
		myHSIT.put("500036777760771","12");
		myHSIT.put("500066777760771","13");
		myHSIT.put("500076777760771","14");
		myHSIT.put("577777600360771","15");
		myHSIT.put("577777777413071","16");
		myHSIT.put("677737607760771","17");
		myHSIT.put("677776777760601","18");
		myHSIT.put("677777477413771","19");
		myHSIT.put("677777607760771","20");
		myHSIT.put("703377400001771","21");
		myHSIT.put("703777400001771","22");
		myHSIT.put("703777740001471","23");
		myHSIT.put("703777740003031","24");
		myHSIT.put("703777740003171","25");
		myHSIT.put("703777740003771","26");
		myHSIT.put("703777747770371","27");
		myHSIT.put("737777777413371","28");
		myHSIT.put("760001400000171","29");
		myHSIT.put("773776607760771","30");
		myHSIT.put("774037777760700","31");
		myHSIT.put("776000000000171","32");
		myHSIT.put("776177607760771","33");
		myHSIT.put("776377777760771","34");
		myHSIT.put("777737777420771","35");
		myHSIT.put("777737777720771","36");
		myHSIT.put("777737777760771","37");
		myHSIT.put("777776407760601","38");
		myHSIT.put("777777347760471","39");
		myHSIT.put("777777377760771","40");
		myHSIT.put("777777404760771","41");
		myHSIT.put("777777607760771","42");
		myHSIT.put("777777747413771","43");
		myHSIT.put("777777757760771","44");
		myHSIT.put("777777764020771","45");
		myHSIT.put("777777770000000","46");
		myHSIT.put("777777774020771","47");
		myHSIT.put("777777777413731","48");
		myHSIT.put("777777777720731","49");
		myHSIT.put("777777777720771","50");
		myHSIT.put("777777777760700","51");
		myHSIT.put("777777777760731","52");
		myHSIT.put("777777777760771","53");
		myHSIT.put("777777777763771","54");
		myHSIT.put("776377775540771","55");
		myHSIT.put("777737770000000","56");
		myHSIT.put("777777143760771","57");
		myHSIT.put("777777557760771","58");
		myHSIT.put("777777606060771","59");
		myHSIT.put("777777607760731","60");
		myHSIT.put("777777743760771","61");
		myHSIT.put("777777774020731","62");
		myHSIT.put("377777737760771","63");
		myHSIT.put("777777607560771","64");
		myHSIT.put("777777777760471","65");
		myHSIT.put("777777037720671","66");
		myHSIT.put("777777037720771","67");
		myHSIT.put("775777757760771","68");
		myHSIT.put("777207747760731","69");
		myHSIT.put("700076777760671","70");
		myHSIT.put("776337777760771","71");
		myHSIT.put("777577777003731","72");
		myHSIT.put("777737777760731","73");
		myHSIT.put("777777600160771","74");
		myHSIT.put("777767777720771","75");
		myHSIT.put("474001377413771","76");
		myHSIT.put("743777777760771","77");
		myHSIT.put("777777777760711","78");
		myHSIT.put("517347606060661","79");
		myHSIT.put("767777744020771","80");
		myHSIT.put("777377607760771","81");
		myHSIT.put("777757606060731","82");
		myHSIT.put("777757777760671","83");
		myHSIT.put("777773604060731","84");
		myHSIT.put("557757606060771","85");
		myHSIT.put("777777737760771","86");
		myHSIT.put("577757606060771","87");
		myHSIT.put("776337777760731","88");
		myHSIT.put("674000003413771","89");
		myHSIT.put("677777776000371","90");
		myHSIT.put("700036777760771","91");
		myHSIT.put("700076777760771","92");
		myHSIT.put("777737607760771","93");
		myHSIT.put("777777347560471","94");
		myHSIT.put("777777607560731","95");
		myHSIT.put("777777750003371","96");
		myHSIT.put("777777760760771","97");
		myHSIT.put("170003607760771","98");
		myHSIT.put("757777777720771","99");
		myHSIT.put("777777777773771","100");
		myHSIT.put("774077606767071","101");
		myHSIT.put("777703777760771","102");
		myHSIT.put("777777754020731","103");
		myHSIT.put("576167400000171","104");
		myHSIT.put("776160000000071","105");
		myHSIT.put("776177400000171","106");
		myHSIT.put("036377777760771","107");
		myHSIT.put("377777777720731","108");
		myHSIT.put("477777377413771","109");
		myHSIT.put("770777775720771","110");
		myHSIT.put("776167607760771","111");
		myHSIT.put("777737761760771","112");
		myHSIT.put("477777776413071","113");
		myHSIT.put("777767741760771","114");
		myHSIT.put("777767743760771","115");
		myHSIT.put("777767775720771","116");
		myHSIT.put("777767777760731","117");
		myHSIT.put("777767777760771","118");
		myHSIT.put("777776777760771","119");
		myHSIT.put("777777577760771","120");
		myHSIT.put("777777775720771","121");
		myHSIT.put("777777775760771","122");
		myHSIT.put("777777776360771","123");
		myHSIT.put("777777777700771","124");
		myHSIT.put("000000007760731","125");
		myHSIT.put("477777777413771","126");
		myHSIT.put("577777777420771","127");
		myHSIT.put("637775777760730","128");
		myHSIT.put("700777747413771","129");
		myHSIT.put("776177607760731","130");
		myHSIT.put("777717777760771","131");
		myHSIT.put("777777606000031","132");
		myHSIT.put("777603405760430","133");
		myHSIT.put("777777777720631","134");
		myHSIT.put("777777777760730","135");
		myHSIT.put("777603405760471","136");
		myHSIT.put("777776777760601","137");
		myHSIT.put("777777777413700","138");
		myHSIT.put("777777774413771","139");
		myHSIT.put("702777740003171","140");
		myHSIT.put("703767740003771","141");
		myHSIT.put("703777700003771","142");
		myHSIT.put("757400004020771","143");
		myHSIT.put("770000003760771","144");
		myHSIT.put("770007000003071","145");
		myHSIT.put("774000077777031","146");
		myHSIT.put("774777616000371","147");
		myHSIT.put("774777716000371","148");
		myHSIT.put("777000377760771","149");
		myHSIT.put("777767607760771","150");
		myHSIT.put("777767774020771","151");
		myHSIT.put("777777776413771","152");
		myHSIT.put("757777777760731","153");
		myHSIT.put("757777777760771","154");
		myHSIT.put("776157777760771","155");
		myHSIT.put("776177777760771","156");
		myHSIT.put("777737777760471","157");
		myHSIT.put("777737777760601","158");
		myHSIT.put("777740017760771","159");
		myHSIT.put("777777477400001","160");
		myHSIT.put("777777607740771","161");
		myHSIT.put("777777607760671","162");
		myHSIT.put("777777607760700","163");
		myHSIT.put("777777760060771","164");
		myHSIT.put("777777774720731","165");
		myHSIT.put("777777777320571","166");
		myHSIT.put("777777777660771","167");
		myHSIT.put("777777777720671","168");
		myHSIT.put("777777777740700","169");
		myHSIT.put("776017777760400","170");
		myHSIT.put("776017777760771","171");
		myHSIT.put("777777777740771","172");
		myHSIT.put("777767677760771","173");
		myHSIT.put("777777037760771","174");
		myHSIT.put("777777677760731","175");
		myHSIT.put("777737607560771","176");
		myHSIT.put("377777607760771","177");
		myHSIT.put("477737607760731","178");
		myHSIT.put("677737607760711","179");
		myHSIT.put("677777777720771","180");
		myHSIT.put("770777777777671","181");
		myHSIT.put("777777004020771","182");
		myHSIT.put("777777377720771","183");
		myHSIT.put("777417606060731","184");
		myHSIT.put("777766777760601","185");
		myHSIT.put("777767404760771","186");
		myHSIT.put("770777777767671","187");
		myHSIT.put("770777777777631","188");
		myHSIT.put("777741777760771","189");
		myHSIT.put("000000000003731","190");
		myHSIT.put("177777777760771","191");
		myHSIT.put("400003777413771","192");
		myHSIT.put("677737607660771","193");
		myHSIT.put("677737607760731","194");
		myHSIT.put("677777607660771","195");
		myHSIT.put("677777777760771","196");
		myHSIT.put("700036777760731","197");
		myHSIT.put("700040777760771","198");
		myHSIT.put("700076760000011","199");
		myHSIT.put("700076777760700","200");
		myHSIT.put("700377000003771","201");
		myHSIT.put("701776777760601","202");
		myHSIT.put("703777740001771","203");
		myHSIT.put("737777777413771","204");
		myHSIT.put("737777777760771","205");
		myHSIT.put("740777607760771","206");
		myHSIT.put("767777777720771","207");
		myHSIT.put("774077774413771","208");
		myHSIT.put("776017607760771","209");
		myHSIT.put("776037777760771","210");
		myHSIT.put("776137607760771","211");
		myHSIT.put("776377776760771","212");
		myHSIT.put("776777607760731","213");
		myHSIT.put("777703757760771","214");
		myHSIT.put("777703777360771","215");
		myHSIT.put("777717607760771","216");
		myHSIT.put("777736777760771","217");
		myHSIT.put("777737774020771","218");
		myHSIT.put("777740777760771","219");
		myHSIT.put("777742777760601","220");
		myHSIT.put("777766777760771","221");
		myHSIT.put("777774077560771","222");
		myHSIT.put("777776377760601","223");
		myHSIT.put("777776776360711","224");
		myHSIT.put("777776777760611","225");
		myHSIT.put("777776777763771","226");
		myHSIT.put("777777370020771","227");
		myHSIT.put("777777603560771","228");
		myHSIT.put("777777603760371","229");
		myHSIT.put("777777603760771","230");
		myHSIT.put("777777757760700","231");
		myHSIT.put("777777760000171","232");
		myHSIT.put("777777770060731","233");
		myHSIT.put("777777777413371","234");
		myHSIT.put("777777777413531","235");
		myHSIT.put("777777777413771","236");
		myHSIT.put("777777777700000","237");
		myHSIT.put("777777777700171","238");
		myHSIT.put("777777777760031","239");
		myHSIT.put("777777777760371","240");
		myHSIT.put("777777777760411","241");
		myHSIT.put("777777777760431","242");
		myHSIT.put("777777777760600","243");
		myHSIT.put("777777777760601","244");
		myHSIT.put("777777777760671","245");
		myHSIT.put("777777777777731","246");
		myHSIT.put("703777740003471","247");
		myHSIT.put("763777774413771","248");
		myHSIT.put("777640007760771","249");
		myHSIT.put("000000000000371","250");
		myHSIT.put("774000000760771","251");
		myHSIT.put("777477607760771","252");
		myHSIT.put("777777663760771","253");
		myHSIT.put("777760007760771","254");
		myHSIT.put("000000000003671","255");
		myHSIT.put("777777777413671","256");
		myHSIT.put("077777777413631","257");
		myHSIT.put("000000177760771","258");
		myHSIT.put("775777377760771","259");
		myHSIT.put("000000000003171","260");
		myHSIT.put("737777773760771","261");
		myHSIT.put("774777777420771","262");
		myHSIT.put("777600003760771","263");
		myHSIT.put("777740003760771","264");
		myHSIT.put("000000000003371","265");
		myHSIT.put("775740003760771","266");
		myHSIT.put("777700003760771","267");
		myHSIT.put("777777677720771","268");
		myHSIT.put("000000000000771","269");
		myHSIT.put("337777777413731","270");
		myHSIT.put("776000000000111","271");
		myHSIT.put("037777777760771","272");
		myHSIT.put("777617607760771","273");
		myHSIT.put("777777600000371","274");
		myHSIT.put("777777600020771","275");
		myHSIT.put("777777770060771","276");
		myHSIT.put("777777777560700","277");
		myHSIT.put("777777777760761","278");
		myHSIT.put("677677477413771","279");
		myHSIT.put("770000777760771","280");
		myHSIT.put("777775777760771","281");
		myHSIT.put("777776747760601","282");
		myHSIT.put("777777704020771","283");
		myHSIT.put("037637777760771","284");
		myHSIT.put("637777777760771","285");
		myHSIT.put("677777470000000","286");
		myHSIT.put("677777477413751","287");
		myHSIT.put("700377740003771","288");
		myHSIT.put("703777740003571","289");
		myHSIT.put("777777606760771","290");
		myHSIT.put("777777677760771","291");
		myHSIT.put("777777757413371","292");
		myHSIT.put("777777777720751","293");
		myHSIT.put("577777777720771","294");
		myHSIT.put("776377763760771","295");
		myHSIT.put("477567777413071","296");
		myHSIT.put("477767377413671","297");
		myHSIT.put("477767777413071","298");
		myHSIT.put("477776077411171","299");
		myHSIT.put("577727674413671","300");
		myHSIT.put("577767777413631","301");
		myHSIT.put("777756777760771","302");
		myHSIT.put("777767777413331","303");
		myHSIT.put("540777757760771","304");
		myHSIT.put("777777477760631","305");
		myHSIT.put("777777601560771","306");
		myHSIT.put("077777777720731","307");
		myHSIT.put("674001777677600","308");
		myHSIT.put("703767740003171","309");
		myHSIT.put("777727770020720","310");
		myHSIT.put("777767570020731","311");
		myHSIT.put("777767770020731","312");
		myHSIT.put("777774003760731","313");
		myHSIT.put("777777570020731","314");
		myHSIT.put("777777764020731","315");
		myHSIT.put("777777770020731","316");
		myHSIT.put("777777777360731","317");
		myHSIT.put("570777777777671","318");
		myHSIT.put("574077607777071","319");
		myHSIT.put("770003606377071","320");
		myHSIT.put("770737004177671","321");
		myHSIT.put("770767777737671","322");
		myHSIT.put("770767777767671","323");
		myHSIT.put("770767777777671","324");
		myHSIT.put("770777507777671","325");
		myHSIT.put("770777707777671","326");
		myHSIT.put("774067606727071","327");
		myHSIT.put("774067606767071","328");
		myHSIT.put("774067607777071","329");
		myHSIT.put("774077607777031","330");
		myHSIT.put("774077607777071","331");
		myHSIT.put("776707777737071","332");
		myHSIT.put("777727607760771","333");
		myHSIT.put("577777777760771","334");
		myHSIT.put("777777607720771","335");
		myHSIT.put("777776777760731","336");
		myHSIT.put("777777777013731","337");
		myHSIT.put("077777777413071","338");
		myHSIT.put("437776777760601","339");
		myHSIT.put("474377777413771","340");
		myHSIT.put("477777775413031","341");
		myHSIT.put("677777777413771","342");
		myHSIT.put("700000007175771","343");
		myHSIT.put("700077777760771","344");
		myHSIT.put("777000377760731","345");
		myHSIT.put("777377700003531","346");
		myHSIT.put("777736777760601","347");
		myHSIT.put("777736777760671","348");
		myHSIT.put("777737777413731","349");
		myHSIT.put("777776617760601","350");
		myHSIT.put("777776777600171","351");
		myHSIT.put("777776777760711","352");
		myHSIT.put("777777774760771","353");
		myHSIT.put("777777777403731","354");
		myHSIT.put("477777777413031","355");
		myHSIT.put("703777600001771","356");
		myHSIT.put("703777740000771","357");
		myHSIT.put("717777777760771","358");
		myHSIT.put("037677774020771","359");
		myHSIT.put("540377777720771","360");
		myHSIT.put("577737777420771","361");
		myHSIT.put("675777757760771","362");
		myHSIT.put("716377777760771","363");
		myHSIT.put("757777776760771","364");
		myHSIT.put("777000003760771","365");
		myHSIT.put("777577777760471","366");
		myHSIT.put("777737404760771","367");
		myHSIT.put("777777007720731","368");
		myHSIT.put("777777574020771","369");
		myHSIT.put("777777747760471","370");
		myHSIT.put("777777007720771","371");
		myHSIT.put("777777752000000","372");
		myHSIT.put("777777767760771","373");
		myHSIT.put("777777776000771","374");
		myHSIT.put("077737607760771","375");
		myHSIT.put("376177607760771","376");
		myHSIT.put("657737607760771","377");
		myHSIT.put("777777667760771","378");
		myHSIT.put("777741017760771","379");
		myHSIT.put("577777777413771","380");
		myHSIT.put("703777740003071","381");
		myHSIT.put("757777774020771","382");
		myHSIT.put("776377774020771","383");
		myHSIT.put("777777674020771","384");
		myHSIT.put("777736377760571","385");
		myHSIT.put("000000007360771","386");
		myHSIT.put("400017600560731","387");
		myHSIT.put("737777607760771","388");
		myHSIT.put("677767607760771","389");
		myHSIT.put("777777777620771","390");
		myHSIT.put("743777607760731","391");
		myHSIT.put("777777600160731","392");
		myHSIT.put("777757777760771","393");
		myHSIT.put("777777777620731","394");
		myHSIT.put("777777777760400","395");
		myHSIT.put("777777607560371","396");
		myHSIT.put("777777600000771","397");
		myHSIT.put("777777607760631","398");
		myHSIT.put("777777777420410","399");
		myHSIT.put("757777774020731","400");
		myHSIT.put("776377403760771","401");
		myHSIT.put("777777600000000","402");
		myHSIT.put("777777743760731","403");
		myHSIT.put("777647000000011","404");
		myHSIT.put("777777000000011","405");
		myHSIT.put("000000000000731","406");
		myHSIT.put("000036040003771","407");
		myHSIT.put("157756777760771","408");
		myHSIT.put("264073777777600","409");
		myHSIT.put("377774077760771","410");
		myHSIT.put("407777777760771","411");
		myHSIT.put("437777606060600","412");
		myHSIT.put("477777477413771","413");
		myHSIT.put("477777767413071","414");
		myHSIT.put("477777774137771","415");
		myHSIT.put("616777677777600","416");
		myHSIT.put("577777770060771","417");
		myHSIT.put("617777777720771","418");
		myHSIT.put("630000207760771","419");
		myHSIT.put("637774777760730","420");
		myHSIT.put("676600000000371","421");
		myHSIT.put("676773667777600","422");
		myHSIT.put("677774207760771","423");
		myHSIT.put("677777477413711","424");
		myHSIT.put("700076774160771","425");
		myHSIT.put("700377600003771","426");
		myHSIT.put("703707740003771","427");
		myHSIT.put("703777740003371","428");
		myHSIT.put("703777740003731","429");
		myHSIT.put("704003347760471","430");
		myHSIT.put("717777774020771","431");
		myHSIT.put("735777777760731","432");
		myHSIT.put("757777777320771","433");
		myHSIT.put("760777777740371","434");
		myHSIT.put("763777607760771","435");
		myHSIT.put("770376777760601","436");
		myHSIT.put("774077777775071","437");
		myHSIT.put("774077777777071","438");
		myHSIT.put("777376777760601","439");
		myHSIT.put("777607607760771","440");
		myHSIT.put("777700277760771","441");
		myHSIT.put("777737377760771","442");
		myHSIT.put("777737607700771","443");
		myHSIT.put("777740001760771","444");
		myHSIT.put("777756777760601","445");
		myHSIT.put("777760016760601","446");
		myHSIT.put("777760377760771","447");
		myHSIT.put("777767777720731","448");
		myHSIT.put("777776757760771","449");
		myHSIT.put("777776770000000","450");
		myHSIT.put("777777477760771","451");
		myHSIT.put("777777606360771","452");
		myHSIT.put("777777677560771","453");
		myHSIT.put("777777736760731","454");
		myHSIT.put("777777760360771","455");
		myHSIT.put("777777774413711","456");
		myHSIT.put("777777777320771","457");
		myHSIT.put("777777777403771","458");
		myHSIT.put("777777777410331","459");
		myHSIT.put("777777777413751","460");
		myHSIT.put("777777777416761","461");
		myHSIT.put("777777777560771","462");
		myHSIT.put("777777777720571","463");
		myHSIT.put("777777777747771","464");
		myHSIT.put("777417737700000","465");
		myHSIT.put("776377377760771","466");
		myHSIT.put("000000000020771","467");
		myHSIT.put("067777777413031","468");
		myHSIT.put("077777607760771","469");
		myHSIT.put("177777777413771","470");
		myHSIT.put("303777740003771","471");
		myHSIT.put("377777777720771","472");
		myHSIT.put("401777777413071","473");
		myHSIT.put("467777777413031","474");
		myHSIT.put("477776077411771","475");
		myHSIT.put("577776777760601","476");
		myHSIT.put("577777777413731","477");
		myHSIT.put("617776777760601","478");
		myHSIT.put("640013777777600","479");
		myHSIT.put("674777477411771","480");
		myHSIT.put("676773677777600","481");
		myHSIT.put("676773777777600","482");
		myHSIT.put("677777477413701","483");
		myHSIT.put("700036777740171","484");
		myHSIT.put("703777400003771","485");
		myHSIT.put("703777740000371","486");
		myHSIT.put("740000004020771","487");
		myHSIT.put("760777774020771","488");
		myHSIT.put("767776777760601","489");
		myHSIT.put("767776777760771","490");
		myHSIT.put("773776777760601","491");
		myHSIT.put("773777607760771","492");
		myHSIT.put("774177777413731","493");
		myHSIT.put("776377777760711","494");
		myHSIT.put("776760003760771","495");
		myHSIT.put("777460007760771","496");
		myHSIT.put("777677777413771","497");
		myHSIT.put("777677777760771","498");
		myHSIT.put("777700000060771","499");
		myHSIT.put("777700001760771","500");
		myHSIT.put("777700377760771","501");
		myHSIT.put("777727737760771","502");
		myHSIT.put("777737377760731","503");
		myHSIT.put("777737737760771","504");
		myHSIT.put("777737777760700","505");
		myHSIT.put("777760003760771","506");
		myHSIT.put("777776037760771","507");
		myHSIT.put("777776377760771","508");
		myHSIT.put("777777605760771","509");
		myHSIT.put("777777700003531","510");
		myHSIT.put("777777700020771","511");
		myHSIT.put("777777707720771","512");
		myHSIT.put("777777717760731","513");
		myHSIT.put("777777774413731","514");
		myHSIT.put("777777776760731","515");
		myHSIT.put("777777777360771","516");
		myHSIT.put("777777777413711","517");
		myHSIT.put("777777777660031","518");
		myHSIT.put("777777777740371","519");
		myHSIT.put("777777777760571","520");
		myHSIT.put("777777777760611","521");
		myHSIT.put("777777777760770","522");
		myHSIT.put("777777777777771","523");
		myHSIT.put("777777777720711","524");
		myHSIT.put("570777637743671","525");
		myHSIT.put("677777777403671","526");
		myHSIT.put("677777777403771","527");
		myHSIT.put("677777777413671","528");
		myHSIT.put("677777777413731","529");
		myHSIT.put("770777477775671","530");
		myHSIT.put("737777774020771","531");
		myHSIT.put("770777767777671","532");
		myHSIT.put("777400777720771","533");
		myHSIT.put("777777607400000","534");
		myHSIT.put("777777707760771","535");
		myHSIT.put("700000377776671","536");
		myHSIT.put("770777777777601","537");
		myHSIT.put("777637777413731","538");
		myHSIT.put("000000000000600","539");
		myHSIT.put("000000000003031","540");
		myHSIT.put("000000000003711","541");
		myHSIT.put("077757607760731","542");
		myHSIT.put("577777677561771","543");
		myHSIT.put("600000000760731","544");
		myHSIT.put("675737607760771","545");
		myHSIT.put("700036777560771","546");
		myHSIT.put("700036777760031","547");
		myHSIT.put("700076700000071","548");
		myHSIT.put("700076757760771","549");
		myHSIT.put("717777477413731","550");
		myHSIT.put("737777777760610","551");
		myHSIT.put("740076777760601","552");
		myHSIT.put("741737777760731","553");
		myHSIT.put("757700777760771","554");
		myHSIT.put("757776777760571","555");
		myHSIT.put("773603774360771","556");
		myHSIT.put("773777774020771","557");
		myHSIT.put("776377770760731","558");
		myHSIT.put("776377770760771","559");
		myHSIT.put("777000000000371","560");
		myHSIT.put("777500003760771","561");
		myHSIT.put("777737600000000","562");
		myHSIT.put("777737770000071","563");
		myHSIT.put("777737774413771","564");
		myHSIT.put("777737777760671","565");
		myHSIT.put("777740777760731","566");
		myHSIT.put("777760007763771","567");
		myHSIT.put("777767477760771","568");
		myHSIT.put("777767760000000","569");
		myHSIT.put("777773776760771","570");
		myHSIT.put("777775774020771","571");
		myHSIT.put("777776407360601","572");
		myHSIT.put("777777663560771","573");
		myHSIT.put("777777777740071","574");
		myHSIT.put("777777777760561","575");
		myHSIT.put("777776777360771","576");
		myHSIT.put("477767277413771","577");
		myHSIT.put("637777607760771","578");
		myHSIT.put("777737607700171","579");
		myHSIT.put("676077607760771","580");
		myHSIT.put("777777677560671","581");
		myHSIT.put("777002777760731","582");
		myHSIT.put("777737777763771","583");
		myHSIT.put("777775777760731","584");
		myHSIT.put("000000000000031","585");
		myHSIT.put("000000007720631","586");
		myHSIT.put("700740007774671","587");
		myHSIT.put("703777744003771","588");
		myHSIT.put("776161000000071","589");
		myHSIT.put("777777403760730","590");
		myHSIT.put("777777757413771","591");
		myHSIT.put("000000000101000","592");
		myHSIT.put("074000037777600","593");
		myHSIT.put("656573777777600","594");
		myHSIT.put("676777777777600","595");
		myHSIT.put("457777777420771","596");
		myHSIT.put("567777777420771","597");
		myHSIT.put("703777400000371","598");
		myHSIT.put("703777400000771","599");
		myHSIT.put("703777400003171","600");
		myHSIT.put("700377400000771","601");
		myHSIT.put("777777770000771","602");
		myHSIT.put("777777777320731","603");
		myHSIT.put("677737607760751","604");
		myHSIT.put("777737607700031","605");
		myHSIT.put("777737607760000","606");
		myHSIT.put("777777467400000","607");
		myHSIT.put("776177770000000","608");
		myHSIT.put("777600774020771","609");
		myHSIT.put("777767774020731","610");
		myHSIT.put("777767777560771","611");
		myHSIT.put("777777777760751","612");
		myHSIT.put("777177777760771","613");
		myHSIT.put("777777607760431","614");
		myHSIT.put("777777777720770","615");
		myHSIT.put("400000000003771","616");
		myHSIT.put("777737776413771","617");
		myHSIT.put("777737777413771","618");
		myHSIT.put("777777002000011","619");
		myHSIT.put("777777744020771","620");
		myHSIT.put("000000000002771","621");
		myHSIT.put("777601774413771","622");
		myHSIT.put("777777757777771","623");
		myHSIT.put("477775777413071","624");
		myHSIT.put("477777777412071","625");
		myHSIT.put("477777777412771","626");
		myHSIT.put("741737777760771","627");
		myHSIT.put("777777777760760","628");
		myHSIT.put("477777777410071","629");
		myHSIT.put("771777737760771","630");
		myHSIT.put("777777767720771","631");
		myHSIT.put("000000000003571","632");
		myHSIT.put("657777607760731","633");
		myHSIT.put("000000004177600","634");
		myHSIT.put("000000007560771","635");
		myHSIT.put("000000007777000","636");
		myHSIT.put("000000007777600","637");
		myHSIT.put("001777607760771","638");
		myHSIT.put("040000000000600","639");
		myHSIT.put("074000007177200","640");
		myHSIT.put("074000030000600","641");
		myHSIT.put("074000032000600","642");
		myHSIT.put("077777377000371","643");
		myHSIT.put("200003377207600","644");
		myHSIT.put("200003717207600","645");
		myHSIT.put("200003777207600","646");
		myHSIT.put("200003777377600","647");
		myHSIT.put("200103717207600","648");
		myHSIT.put("377777423560771","649");
		myHSIT.put("436773677774600","650");
		myHSIT.put("464063777777600","651");
		myHSIT.put("477577777413071","652");
		myHSIT.put("477777604020771","653");
		myHSIT.put("477777737413071","654");
		myHSIT.put("477777777720771","655");
		myHSIT.put("577777377420771","656");
		myHSIT.put("600000016177600","657");
		myHSIT.put("600000017777600","658");
		myHSIT.put("602733777777600","659");
		myHSIT.put("602773607777600","660");
		myHSIT.put("602773777777600","661");
		myHSIT.put("614073777777600","662");
		myHSIT.put("616773677777600","663");
		myHSIT.put("616773776377600","664");
		myHSIT.put("616773777777600","665");
		myHSIT.put("640013777177600","666");
		myHSIT.put("644073777777600","667");
		myHSIT.put("656473737777600","668");
		myHSIT.put("656573377777200","669");
		myHSIT.put("656573377777600","670");
		myHSIT.put("664043777777600","671");
		myHSIT.put("664063777777600","672");
		myHSIT.put("664070377777600","673");
		myHSIT.put("664071777777600","674");
		myHSIT.put("664073377777600","675");
		myHSIT.put("664073774037600","676");
		myHSIT.put("664073777740600","677");
		myHSIT.put("664073777757600","678");
		myHSIT.put("664073777763600","679");
		myHSIT.put("664073777767600","680");
		myHSIT.put("664073777776600","681");
		myHSIT.put("664073777777200","682");
		myHSIT.put("664073777777600","683");
		myHSIT.put("666773677777600","684");
		myHSIT.put("674073777777600","685");
		myHSIT.put("674153777777600","686");
		myHSIT.put("674573777767600","687");
		myHSIT.put("676473777777600","688");
		myHSIT.put("676573777017600","689");
		myHSIT.put("676573777637600","690");
		myHSIT.put("676573777777600","691");
		myHSIT.put("676673677777600","692");
		myHSIT.put("676743677777600","693");
		myHSIT.put("676743777777600","694");
		myHSIT.put("676763677777600","695");
		myHSIT.put("676773674177600","696");
		myHSIT.put("676773676177600","697");
		myHSIT.put("676773677777200","698");
		myHSIT.put("677777777720571","699");
		myHSIT.put("700741007764671","700");
		myHSIT.put("700741007774671","701");
		myHSIT.put("700775747413771","702");
		myHSIT.put("703177400001771","703");
		myHSIT.put("703777740003111","704");
		myHSIT.put("717377777760771","705");
		myHSIT.put("730177577760771","706");
		myHSIT.put("736377777760771","707");
		myHSIT.put("737776777760771","708");
		myHSIT.put("737777607760700","709");
		myHSIT.put("737777700000031","710");
		myHSIT.put("737777777413731","711");
		myHSIT.put("747777777760731","712");
		myHSIT.put("763777777760771","713");
		myHSIT.put("777777006720771","714");
		myHSIT.put("770477777777671","715");
		myHSIT.put("772777400001671","716");
		myHSIT.put("774177777760771","717");
		myHSIT.put("776000000000771","718");
		myHSIT.put("776177407760771","719");
		myHSIT.put("776177607760371","720");
		myHSIT.put("776377777760371","721");
		myHSIT.put("777700007413731","722");
		myHSIT.put("777700074413771","723");
		myHSIT.put("777701002001731","724");
		myHSIT.put("777737607760770","725");
		myHSIT.put("777737747413771","726");
		myHSIT.put("777737774020731","727");
		myHSIT.put("777737777760431","728");
		myHSIT.put("657777607760771","729");
		myHSIT.put("777757777413731","730");
		myHSIT.put("777761607760771","731");
		myHSIT.put("777763777760771","732");
		myHSIT.put("777775747413771","733");
		myHSIT.put("777776777411731","734");
		myHSIT.put("777776777413731","735");
		myHSIT.put("777777577760731","736");
		myHSIT.put("777777607760571","737");
		myHSIT.put("777777607760760","738");
		myHSIT.put("777777707413771","739");
		myHSIT.put("777777747720771","740");
		myHSIT.put("777777757720771","741");
		myHSIT.put("777777770020771","742");
		myHSIT.put("777777776000071","743");
		myHSIT.put("777777777411731","744");
		myHSIT.put("777777777413131","745");
		myHSIT.put("777777777520771","746");
		myHSIT.put("777777777720471","747");
		myHSIT.put("777777777720760","748");
		myHSIT.put("777777677777771","749");
		myHSIT.put("037677500020771","750");
		myHSIT.put("077777777760771","751");
		myHSIT.put("440013777777600","752");
		myHSIT.put("477777607760771","753");
		myHSIT.put("503777740003771","754");
		myHSIT.put("577777677720771","755");
		myHSIT.put("673777477413771","756");
		myHSIT.put("676573777677600","757");
		myHSIT.put("677777477413700","758");
		myHSIT.put("077637777760771","759");
		myHSIT.put("774037000020700","760");
		myHSIT.put("774077007777071","761");
		myHSIT.put("774777777420731","762");
		myHSIT.put("777700777413700","763");
		myHSIT.put("777757777720771","764");
		myHSIT.put("777760077760771","765");
		myHSIT.put("777761007760771","766");
		myHSIT.put("777773777413700","767");
		myHSIT.put("777777407720771","768");
		myHSIT.put("777777474020771","769");
		myHSIT.put("777677607760771","770");
		myHSIT.put("777777717760771","771");
		myHSIT.put("777777743460771","772");
		myHSIT.put("777777760000031","773");
		myHSIT.put("777777773760771","774");
		myHSIT.put("777777777000371","775");
		myHSIT.put("777777777410771","776");
		myHSIT.put("777777777420771","777");
		myHSIT.put("676573777077600","778");
		myHSIT.put("777777717720771","779");
		myHSIT.put("777777777600771","780");
		myHSIT.put("000000004020731","781");
		myHSIT.put("417777607760771","782");
		myHSIT.put("477777777760731","783");
		myHSIT.put("776377777760731","784");
		myHSIT.put("777777554760771","785");
		myHSIT.put("777777760000000","786");
		myHSIT.put("777777777760071","787");
		myHSIT.put("577777002060771","788");
		myHSIT.put("676377777760771","789");
		myHSIT.put("776377777740771","790");
		myHSIT.put("777777760020771","791");
		myHSIT.put("763777777413771","792");
		myHSIT.put("577736777760601","793");
		myHSIT.put("703757740003771","794");
		myHSIT.put("000000006760771","795");
		myHSIT.put("000000000001771","796");
		myHSIT.put("676773777777200","797");
		myHSIT.put("437777777760771","798");
		myHSIT.put("770767677777671","799");
		myHSIT.put("774000370020771","800");
		myHSIT.put("774777777760771","801");
		myHSIT.put("777603000000011","802");
		myHSIT.put("777740007760771","803");
		myHSIT.put("477777777760771","804");
		myHSIT.put("737417606060731","805");
		myHSIT.put("757777777413731","806");
		myHSIT.put("770077606060731","807");
		myHSIT.put("777747606060771","808");
		myHSIT.put("777775606060771","809");
		myHSIT.put("777777602060771","810");
		myHSIT.put("777777604060731","811");
		myHSIT.put("777777606060571","812");
		myHSIT.put("777777606060631","813");
		myHSIT.put("777777606060671","814");
		myHSIT.put("777777606060731","815");
		myHSIT.put("077777606060771","816");
		myHSIT.put("777777777420731","817");
		myHSIT.put("200003770000000","818");
		myHSIT.put("676763677777400","819");
		myHSIT.put("676763777777600","820");
		myHSIT.put("674000000000171","821");
		myHSIT.put("757777607760771","822");
		myHSIT.put("776000003760771","823");
		myHSIT.put("777777457760771","824");
		myHSIT.put("777777600000171","825");
		myHSIT.put("677737207760771","826");
		myHSIT.put("776337770760771","827");
		myHSIT.put("377777607760731","828");
		myHSIT.put("640003777777600","829");
		myHSIT.put("000000007720731","830");
		myHSIT.put("776367777760771","831");
		myHSIT.put("777727777760731","832");
		myHSIT.put("757777777760700","833");
		myHSIT.put("777767777740771","834");
		myHSIT.put("777767777720671","835");
		myHSIT.put("777777777700131","836");
		myHSIT.put("777777777020731","837");
		myHSIT.put("777777743760751","838");
		myHSIT.put("777777743700171","839");
		myHSIT.put("777775777720771","840");
		myHSIT.put("777767643760771","841");
		myHSIT.put("777766770000000","842");
		myHSIT.put("777177777720771","843");
		myHSIT.put("761777743760731","844");
		myHSIT.put("761767743760731","845");
		myHSIT.put("760777777760731","846");
		myHSIT.put("760001743760771","847");
		myHSIT.put("737777777760731","848");
		myHSIT.put("637777777720771","849");
		myHSIT.put("437777743760771","850");
		myHSIT.put("403777702000371","851");
		myHSIT.put("400003743760771","852");
		myHSIT.put("377777777760731","853");
		myHSIT.put("177777403760771","854");
		myHSIT.put("037767777720731","855");
		myHSIT.put("774077770005071","856");
		myHSIT.put("774067770007071","857");
		myHSIT.put("774067770005071","858");
		myHSIT.put("770777777747071","859");
		myHSIT.put("574067607777071","860");
		myHSIT.put("574067606767071","861");
		myHSIT.put("777767777360731","862");
		myHSIT.put("000000007400000","863");
		myHSIT.put("703740000000760","864");
		myHSIT.put("477777677413071","865");
		myHSIT.put("577777607760771","866");
		myHSIT.put("777737607560731","867");
		myHSIT.put("777676777760771","868");
		myHSIT.put("777774047560771","869");
		myHSIT.put("777767776760731","870");
		myHSIT.put("737777777720771","871");
		myHSIT.put("773777623760771","872");
		myHSIT.put("703400000000000","873");
		myHSIT.put("776377777760431","874");
		myHSIT.put("777717777760731","875");
		myHSIT.put("703777740000731","876");
		myHSIT.put("770000770000000","877");
		myHSIT.put("777776777760571","878");
		myHSIT.put("777767777760671","879");
		myHSIT.put("777777000760771","880");
		myHSIT.put("776377770000731","881");
		myHSIT.put("577777757413771","882");
		myHSIT.put("777777754020771","883");
		myHSIT.put("376377777760771","884");
		myHSIT.put("777737607460771","885");
		myHSIT.put("777776776760601","886");
		myHSIT.put("557767777720771","887");
		myHSIT.put("777777777760631","888");
		myHSIT.put("757777757413771","889");
		myHSIT.put("741774047560771","890");
		myHSIT.put("777777607660771","891");
		myHSIT.put("777777477413771","892");
		myHSIT.put("777777017760771","893");
		myHSIT.put("677777477413731","894");
		myHSIT.put("677777477013771","895");
		myHSIT.put("777734077560771","896");
		myHSIT.put("676003477413771","897");
		myHSIT.put("677777774020771","898");
		myHSIT.put("670000777760771","899");
		myHSIT.put("677777607740011","900");
		myHSIT.put("700036777740031","901");
		myHSIT.put("776137607760631","902");
		myHSIT.put("777772777760601","903");
		myHSIT.put("775775757760771","904");
		myHSIT.put("677737607700771","905");
		myHSIT.put("777774177720771","906");
		myHSIT.put("777777677760770","907");
		myHSIT.put("677737607760700","908");
		myHSIT.put("740777676000011","909");
		myHSIT.put("700000007177771","910");
		myHSIT.put("777777600060771","911");
		myHSIT.put("677777777413371","912");
		myHSIT.put("777743777760771","913");
		myHSIT.put("776377777720771","914");
		myHSIT.put("761777777720731","915");
		myHSIT.put("777777607760411","916");
		myHSIT.put("777577777760771","917");
		myHSIT.put("770000607760771","918");
		myHSIT.put("777777774020760","919");
		myHSIT.put("777633777420771","920");
		myHSIT.put("437737777420771","921");
		myHSIT.put("774747377420771","922");
		myHSIT.put("676377777720771","923");
		myHSIT.put("777600007413371","924");
		myHSIT.put("777717777720771","925");
		myHSIT.put("773777777760771","926");
		myHSIT.put("440060000000771","927");
		myHSIT.put("777777750060771","928");
		myHSIT.put("777737777620771","929");
		myHSIT.put("677777404760771","930");
		myHSIT.put("774000007760771","931");
		myHSIT.put("777777663760731","932");
		myHSIT.put("777777607760071","933");
		myHSIT.put("777777777413071","934");
		myHSIT.put("775777777720771","935");
		myHSIT.put("740737677760731","936");
		myHSIT.put("777777777013771","937");
		myHSIT.put("474177777413771","938");
		myHSIT.put("775777777413771","939");
		myHSIT.put("000000000003401","940");
		myHSIT.put("000000000003751","941");
		myHSIT.put("777777770760731","942");
		myHSIT.put("777777757760731","943");
		myHSIT.put("777777774000071","944");
		myHSIT.put("763743777413771","945");
		myHSIT.put("777777740020771","946");
		myHSIT.put("777777777413631","947");
		myHSIT.put("777777760020611","948");
		myHSIT.put("777777017720771","949");
		myHSIT.put("700037607760771","950");
		myHSIT.put("774177777413771","951");
		myHSIT.put("603777740003771","952");
		myHSIT.put("777377577360771","953");
		myHSIT.put("703677740003771","954");
		myHSIT.put("000000007777771","955");
		myHSIT.put("777777777760011","956");
		myHSIT.put("676501777177600","957");
		myHSIT.put("777777776013731","958");
		myHSIT.put("777741777413731","959");
		myHSIT.put("477737607760771","960");
		myHSIT.put("407777607760771","961");
		myHSIT.put("777777777413031","962");
		myHSIT.put("777777737720771","963");
		myHSIT.put("437777607760771","964");
		myHSIT.put("437777407760771","965");
		myHSIT.put("775777777760771","966");
		myHSIT.put("000000006177600","967");
		myHSIT.put("000001705777600","968");
		myHSIT.put("000001707777000","969");
		myHSIT.put("000001707777600","970");
		myHSIT.put("000001747777000","971");
		myHSIT.put("000001747777600","972");
		myHSIT.put("016773777777600","973");
		myHSIT.put("046773777764600","974");
		myHSIT.put("046773777767600","975");
		myHSIT.put("076773777777400","976");
		myHSIT.put("076773777777600","977");
		myHSIT.put("200003777357600","978");
		myHSIT.put("200003777777600","979");
		myHSIT.put("400000077777600","980");
		myHSIT.put("416773777777000","981");
		myHSIT.put("416773777777600","982");
		myHSIT.put("600373777777600","983");
		myHSIT.put("602753377777600","984");
		myHSIT.put("606773677747600","985");
		myHSIT.put("606773777777600","986");
		myHSIT.put("614173777777600","987");
		myHSIT.put("616601777777600","988");
		myHSIT.put("616733777777600","989");
		myHSIT.put("616772777777600","990");
		myHSIT.put("616773737777600","991");
		myHSIT.put("616773777677600","992");
		myHSIT.put("616773777757600","993");
		myHSIT.put("616773777763600","994");
		myHSIT.put("616773777775600","995");
		myHSIT.put("636373677777600","996");
		myHSIT.put("636773777777600","997");
		myHSIT.put("644773777767600","998");
		myHSIT.put("646770077767600","999");
		myHSIT.put("646773776167600","1000");
		myHSIT.put("646773777767200","1001");
		myHSIT.put("646773777767600","1002");
		myHSIT.put("656773777777600","1003");
		myHSIT.put("662773777777600","1004");
		myHSIT.put("672773777775600","1005");
		myHSIT.put("674373777774000","1006");
		myHSIT.put("674373777777600","1007");
		myHSIT.put("676563777777600","1008");
		myHSIT.put("676713677767600","1009");
		myHSIT.put("676733677777600","1010");
		myHSIT.put("676743377777600","1011");
		myHSIT.put("676760000007600","1012");
		myHSIT.put("676760006067600","1013");
		myHSIT.put("676763637767600","1014");
		myHSIT.put("676770077777600","1015");
		myHSIT.put("676772777767600","1016");
		myHSIT.put("676773017737600","1017");
		myHSIT.put("676773377757600","1018");
		myHSIT.put("676773377777600","1019");
		myHSIT.put("676773417777600","1020");
		myHSIT.put("676773674037600","1021");
		myHSIT.put("676773677743600","1022");
		myHSIT.put("676773677757600","1023");
		myHSIT.put("676773677761600","1024");
		myHSIT.put("676773677763600","1025");
		myHSIT.put("676773677774600","1026");
		myHSIT.put("676773677775600","1027");
		myHSIT.put("676773677777000","1028");
		myHSIT.put("676773737766600","1029");
		myHSIT.put("676773737777600","1030");
		myHSIT.put("676773764146600","1031");
		myHSIT.put("676773776003600","1032");
		myHSIT.put("676773777075600","1033");
		myHSIT.put("676773777600600","1034");
		myHSIT.put("676773777627600","1035");
		myHSIT.put("676773777637600","1036");
		myHSIT.put("676773777677600","1037");
		myHSIT.put("676773777735600","1038");
		myHSIT.put("676773777736600","1039");
		myHSIT.put("676773777737600","1040");
		myHSIT.put("676773777743600","1041");
		myHSIT.put("676773777753600","1042");
		myHSIT.put("676773777766600","1043");
		myHSIT.put("676773777767600","1044");
		myHSIT.put("676773777770600","1045");
		myHSIT.put("676773777771600","1046");
		myHSIT.put("676773777773600","1047");
		myHSIT.put("676773777775600","1048");
		myHSIT.put("777637547013771","1049");
		myHSIT.put("777730000000771","1050");
		myHSIT.put("777777437760771","1051");
		myHSIT.put("776367377760771","1052");
		myHSIT.put("777767637760771","1053");
		myHSIT.put("777737770020731","1054");
		myHSIT.put("777767770020720","1055");
		myHSIT.put("777477777760731","1056");
		myHSIT.put("000003743760771","1057");
		myHSIT.put("037777777760731","1058");
		myHSIT.put("777777774417771","1059");
		myHSIT.put("777777776160771","1060");
		myHSIT.put("757776777760771","1061");
		myHSIT.put("700774007413771","1062");
		myHSIT.put("776377777660771","1063");
		myHSIT.put("771777607760771","1064");
		myHSIT.put("777717743760771","1065");
		myHSIT.put("777767607560771","1066");
		myHSIT.put("774037707760700","1067");
		myHSIT.put("776337377760771","1068");
		myHSIT.put("771777777760771","1069");
		myHSIT.put("777777607760371","1070");
		myHSIT.put("777771607760771","1071");
		myHSIT.put("777731607760771","1072");
		myHSIT.put("777637777760771","1073");
		myHSIT.put("777777607760761","1074");
		myHSIT.put("777770607760771","1075");
		myHSIT.put("007777607760771","1076");
		myHSIT.put("777777377760731","1077");
		myHSIT.put("377703377760731","1078");
		myHSIT.put("577777703760771","1079");
		myHSIT.put("777776777760701","1080");
		myHSIT.put("777736777760731","1081");
		myHSIT.put("777777777413311","1082");
		myHSIT.put("777700000001731","1083");
		myHSIT.put("600000000360771","1084");
		myHSIT.put("777777774620731","1085");
		myHSIT.put("740737607760731","1086");
		myHSIT.put("177777603760771","1087");
		myHSIT.put("777767777763771","1088");
		myHSIT.put("403777740003771","1089");
		myHSIT.put("077777777413771","1090");
		myHSIT.put("703777740002771","1091");
		myHSIT.put("702777740003771","1092");
		myHSIT.put("703777600003771","1093");
		myHSIT.put("777777777563771","1094");
		myHSIT.put("000076777760771","1095");
		myHSIT.put("577777777763771","1096");
		myHSIT.put("400177777413071","1097");
		myHSIT.put("777777777760331","1098");
		myHSIT.put("776000017760471","1099");
		myHSIT.put("037776777760601","1100");
		myHSIT.put("000076700000031","1101");
		myHSIT.put("617776777760401","1102");
		myHSIT.put("770000757760771","1103");
		myHSIT.put("777777757660771","1104");
		myHSIT.put("777773777760771","1105");
		myHSIT.put("777777607760400","1106");
		myHSIT.put("777700007760771","1107");
		myHSIT.put("200003370000000","1108");
		myHSIT.put("200003400037600","1109");
		myHSIT.put("376037777760771","1110");
		myHSIT.put("377777777760760","1111");
		myHSIT.put("403777777760771","1112");
		myHSIT.put("476377777760771","1113");
		myHSIT.put("477377777760731","1114");
		myHSIT.put("577777777700000","1115");
		myHSIT.put("607777777720771","1116");
		myHSIT.put("674777777420771","1117");
		myHSIT.put("676673757777600","1118");
		myHSIT.put("700036777760471","1119");
		myHSIT.put("703760000000331","1120");
		myHSIT.put("717777777760611","1121");
		myHSIT.put("767777777760771","1122");
		myHSIT.put("770000777740171","1123");
		myHSIT.put("770003777720771","1124");
		myHSIT.put("773737774020771","1125");
		myHSIT.put("775677757760771","1126");
		myHSIT.put("776377577760771","1127");
		myHSIT.put("776377730360771","1128");
		myHSIT.put("776777777760771","1129");
		myHSIT.put("777377777700000","1130");
		myHSIT.put("777617347760471","1131");
		myHSIT.put("777637777720771","1132");
		myHSIT.put("777737347760471","1133");
		myHSIT.put("777737777420731","1134");
		myHSIT.put("777737777720731","1135");
		myHSIT.put("777740004720771","1136");
		myHSIT.put("777740015760771","1137");
		myHSIT.put("777767774760771","1138");
		myHSIT.put("777777374020771","1139");
		myHSIT.put("777777404120771","1140");
		myHSIT.put("777777741760771","1141");
		myHSIT.put("777777777720071","1142");
		myHSIT.put("777777777720371","1143");
		myHSIT.put("777777600760771","1144");
		myHSIT.put("677777607060771","1145");
		myHSIT.put("741737607560771","1146");
		myHSIT.put("777777600560771","1147");
		myHSIT.put("777734377760771","1148");
		myHSIT.put("777777777777331","1149");
		myHSIT.put("700036776360771","1150");
		myHSIT.put("700377740001771","1151");
		myHSIT.put("000037777760770","1152");
		myHSIT.put("772700000003631","1153");
		myHSIT.put("777777607760751","1154");
		myHSIT.put("577777774020771","1155");
		myHSIT.put("636377607760771","1156");
		myHSIT.put("700076777760770","1157");
		myHSIT.put("640013377777600","1158");
		myHSIT.put("437777777720771","1159");
		myHSIT.put("777737207760771","1160");
		myHSIT.put("777740007760531","1161");
		myHSIT.put("000000000002171","1162");
		myHSIT.put("677737777760771","1163");
		myHSIT.put("637777777700371","1164");
		myHSIT.put("777737757420771","1165");
		myHSIT.put("777377777760771","1166");
		myHSIT.put("777703737760771","1167");
		myHSIT.put("000000000003631","1168");
		myHSIT.put("677737477413771","1169");
		myHSIT.put("676177477413771","1170");
		myHSIT.put("677777477412771","1171");
		myHSIT.put("000000000403771","1172");
		myHSIT.put("770000777760731","1173");
		myHSIT.put("774777757420771","1174");
		myHSIT.put("774777777420371","1175");
		myHSIT.put("777757607760771","1176");
		myHSIT.put("777777767700000","1177");
		myHSIT.put("777771000000731","1178");
		myHSIT.put("777776776360771","1179");
		myHSIT.put("763777777413700","1180");
		myHSIT.put("676773757777600","1181");
		myHSIT.put("477777777413731","1182");
		myHSIT.put("777777777413331","1183");
		myHSIT.put("000002000003771","1184");
		myHSIT.put("676673777777600","1185");
		myHSIT.put("777777777403671","1186");
		myHSIT.put("674000003403771","1187");
		myHSIT.put("777777777003771","1188");
		myHSIT.put("677777477403771","1189");
		myHSIT.put("777700776413771","1190");
		myHSIT.put("777737776760731","1191");
		myHSIT.put("777777677763771","1192");
		myHSIT.put("777777677773771","1193");
		myHSIT.put("777777677770771","1194");
		myHSIT.put("777767477763771","1195");
		myHSIT.put("777777777761771","1196");
		myHSIT.put("703777740203171","1197");
		myHSIT.put("703737740003171","1198");
		myHSIT.put("703777740001171","1199");
		myHSIT.put("703777747777771","1200");
		myHSIT.put("677636207760771","1201");
		myHSIT.put("000000003760771","1202");
		myHSIT.put("703777640003171","1203");
		myHSIT.put("777770000000731","1204");
		myHSIT.put("777777774020671","1205");
		myHSIT.put("437777777400171","1206");
		myHSIT.put("737777777700771","1207");
		myHSIT.put("000000104020771","1208");
		myHSIT.put("000000377760771","1209");
		myHSIT.put("017777607760771","1210");
		myHSIT.put("576377777760771","1211");
		myHSIT.put("600000000060751","1212");
		myHSIT.put("647737607760771","1213");
		myHSIT.put("777617777760771","1214");
		myHSIT.put("677735607760771","1215");
		myHSIT.put("677737607700371","1216");
		myHSIT.put("677777477413151","1217");
		myHSIT.put("700076777740771","1218");
		myHSIT.put("737777423560771","1219");
		myHSIT.put("754377607760700","1220");
		myHSIT.put("757767777760771","1221");
		myHSIT.put("770177607760771","1222");
		myHSIT.put("776077777760771","1223");
		myHSIT.put("776177607760700","1224");
		myHSIT.put("776377700160771","1225");
		myHSIT.put("777617777777771","1226");
		myHSIT.put("777737557760771","1227");
		myHSIT.put("777740017760731","1228");
		myHSIT.put("777743777720771","1229");
		myHSIT.put("777757774020771","1230");
		myHSIT.put("777774057760771","1231");
		myHSIT.put("777774077560731","1232");
		myHSIT.put("777777077760771","1233");
		myHSIT.put("777777644720771","1234");
		myHSIT.put("777777744720771","1235");
		myHSIT.put("777777760000611","1236");
		myHSIT.put("777777763660771","1237");
		myHSIT.put("777777776720771","1238");
		myHSIT.put("777777777160031","1239");
		myHSIT.put("777760001760771","1240");
		myHSIT.put("777777607700771","1241");
		myHSIT.put("776377777420771","1242");
		myHSIT.put("777377777720771","1243");
		myHSIT.put("776377777720731","1244");
		myHSIT.put("776377777400071","1245");
		myHSIT.put("777776777720771","1246");
		myHSIT.put("777777607763771","1247");
		myHSIT.put("760003777760771","1248");
		myHSIT.put("003777607760771","1249");
		myHSIT.put("777777737740771","1250");
		myHSIT.put("777767777413731","1251");
		myHSIT.put("777777177760771","1252");
		myHSIT.put("676377737760771","1253");
		myHSIT.put("777776777060771","1254");
		myHSIT.put("177777657760771","1255");
		myHSIT.put("770177774020771","1256");
		myHSIT.put("777736000000000","1257");
		myHSIT.put("707777347760471","1258");
		myHSIT.put("777740000160771","1259");
		myHSIT.put("000000007620771","1260");
		myHSIT.put("777777404760731","1261");
		myHSIT.put("000000007000771","1262");
		myHSIT.put("477777377410771","1263");
		myHSIT.put("703777740000000","1264");
		myHSIT.put("776777777760731","1265");
		myHSIT.put("700777740003771","1266");
		myHSIT.put("774037777760300","1267");
		myHSIT.put("777760057760771","1268");
		myHSIT.put("574777777420771","1269");
		myHSIT.put("757776777760601","1270");
		myHSIT.put("776377677760771","1271");
		myHSIT.put("577737777760571","1272");
		myHSIT.put("700076777760731","1273");
		myHSIT.put("777777760000071","1274");
		myHSIT.put("777777774000371","1275");
		myHSIT.put("777017777420771","1276");
		myHSIT.put("777777207760771","1277");
		myHSIT.put("037677777760771","1278");
		myHSIT.put("737777757760771","1279");
		myHSIT.put("000000007760770","1280");
		myHSIT.put("777537777420771","1281");
		myHSIT.put("777777400020771","1282");
		myHSIT.put("777777777020771","1283");
		myHSIT.put("741703777760771","1284");
		myHSIT.put("577776755412731","1285");
		myHSIT.put("474177777613771","1286");
		myHSIT.put("400037777613771","1287");
		myHSIT.put("777477607763771","1288");
		myHSIT.put("777777717763771","1289");
		myHSIT.put("777777607723771","1290");
		myHSIT.put("777737777423771","1291");
		myHSIT.put("674777377420771","1292");
		myHSIT.put("676177607760771","1293");
		myHSIT.put("774177407760771","1294");
		myHSIT.put("776163607760771","1295");
		myHSIT.put("776371777760771","1296");
		myHSIT.put("664073077777600","1297");
		myHSIT.put("642217777777600","1298");
		myHSIT.put("440003777777600","1299");
		myHSIT.put("001776777760601","1300");
		myHSIT.put("577777777760601","1301");
		myHSIT.put("577777777760731","1302");
		myHSIT.put("624073777777600","1303");
		myHSIT.put("640013737777600","1304");
		myHSIT.put("064063777777600","1305");
		myHSIT.put("264063777777600","1306");
		myHSIT.put("064061777777600","1307");
		myHSIT.put("664063777700600","1308");
		myHSIT.put("064263777777600","1309");
		myHSIT.put("464223777774600","1310");
		myHSIT.put("000000000003700","1311");
		myHSIT.put("703777740003131","1312");
		myHSIT.put("777777775413731","1313");
		myHSIT.put("703377740003171","1314");
		myHSIT.put("700066777760671","1315");
		myHSIT.put("717777777413731","1316");
		myHSIT.put("577767777760671","1317");
		myHSIT.put("577767777760771","1318");
		myHSIT.put("500367740003771","1319");
		myHSIT.put("700177740003771","1320");
		myHSIT.put("677777607760731","1321");
		myHSIT.put("777703077760771","1322");
		myHSIT.put("503767740003671","1323");
		myHSIT.put("777760047760771","1324");
		myHSIT.put("777761050000771","1325");
		myHSIT.put("777037777413700","1326");
		myHSIT.put("703637740003771","1327");
		myHSIT.put("777777034020771","1328");
		myHSIT.put("777776777560771","1329");
		myHSIT.put("377777757760771","1330");
		myHSIT.put("776175407760771","1331");
		myHSIT.put("677777777760731","1332");
		myHSIT.put("776370001760771","1333");
		myHSIT.put("776370001760731","1334");
		myHSIT.put("776177607760400","1335");
		myHSIT.put("676377717760471","1336");
		myHSIT.put("777700607760771","1337");
		myHSIT.put("777777777777671","1338");
		myHSIT.put("677737777760731","1339");
		myHSIT.put("777775757413371","1340");
		myHSIT.put("777776757760601","1341");
		myHSIT.put("477777177413071","1342");
		myHSIT.put("703737740003771","1343");
		myHSIT.put("703777440003771","1344");
		myHSIT.put("703777700000371","1345");
		myHSIT.put("703777640003770","1346");
		myHSIT.put("777700777760771","1347");
		myHSIT.put("774037777760701","1348");
		myHSIT.put("057777777760771","1349");
		myHSIT.put("777017777700000","1350");
		myHSIT.put("776377677700771","1351");
		myHSIT.put("777677770000000","1352");
		myHSIT.put("777777774000000","1353");
		myHSIT.put("576177607760771","1354");
		myHSIT.put("777777407560731","1355");
		myHSIT.put("776377777760751","1356");
		myHSIT.put("703777400003031","1357");
		myHSIT.put("777417760060771","1358");
		myHSIT.put("477774374020771","1359");
		myHSIT.put("577777777720631","1360");
		myHSIT.put("757737777760731","1361");
		myHSIT.put("677777700000371","1362");
		myHSIT.put("777777777400171","1363");
		myHSIT.put("000000000003471","1364");
		myHSIT.put("000000007413771","1365");
		myHSIT.put("000076777760671","1366");
		myHSIT.put("377737607760771","1367");
		myHSIT.put("701777747770371","1368");
		myHSIT.put("477777777413671","1369");
		myHSIT.put("757777757760771","1370");
		myHSIT.put("767777606760771","1371");
		myHSIT.put("777577777413771","1372");
		myHSIT.put("777677777413731","1373");
		myHSIT.put("777777700003771","1374");
		myHSIT.put("777777753413771","1375");
		myHSIT.put("777777775413371","1376");
		myHSIT.put("777777777412671","1377");
		myHSIT.put("777777777770771","1378");
		myHSIT.put("773777777770771","1379");
		myHSIT.put("774777777410771","1380");
		myHSIT.put("774777775410771","1381");
		myHSIT.put("777777776720671","1382");
		myHSIT.put("773777777760731","1383");
		myHSIT.put("000000277420771","1384");
		myHSIT.put("537767776760631","1385");
		myHSIT.put("777777777403701","1386");
		myHSIT.put("407760007760771","1387");
		myHSIT.put("777777743413331","1388");
		myHSIT.put("177777777413731","1389");
		myHSIT.put("777777747413371","1390");
		myHSIT.put("777777700003371","1391");
		myHSIT.put("777777777413431","1392");
		myHSIT.put("777777777405700","1393");
		myHSIT.put("777776777760671","1394");
		myHSIT.put("474377767413771","1395");
		myHSIT.put("777777750002371","1396");
		myHSIT.put("777777761413700","1397");
		myHSIT.put("577777740003771","1398");
		myHSIT.put("077777777413700","1399");
		myHSIT.put("700000007413371","1400");
		myHSIT.put("703777740003671","1401");
		myHSIT.put("577770050000000","1402");
		myHSIT.put("777777757003371","1403");
		myHSIT.put("777777767413731","1404");
		myHSIT.put("703743740003771","1405");
		myHSIT.put("777777757413171","1406");
		myHSIT.put("775377777413771","1407");
		myHSIT.put("000000007413371","1408");
		myHSIT.put("777777757413271","1409");
		myHSIT.put("777777700000000","1410");
		myHSIT.put("777477757413371","1411");
		myHSIT.put("777377757413271","1412");
		myHSIT.put("177743777413731","1413");
		myHSIT.put("777757757413371","1414");
		myHSIT.put("077743777413731","1415");
		myHSIT.put("777747757413771","1416");
		myHSIT.put("777777757413071","1417");
		myHSIT.put("577777750003771","1418");
		myHSIT.put("717777700003371","1419");
		myHSIT.put("777777757410071","1420");
		myHSIT.put("777777757403371","1421");
		myHSIT.put("703777000000771","1422");
		myHSIT.put("477774777413771","1423");
		myHSIT.put("777777010003371","1424");
		myHSIT.put("777777757413331","1425");
		myHSIT.put("777737777760711","1426");
		myHSIT.put("457001777413771","1427");
		myHSIT.put("774707777413731","1428");
		myHSIT.put("703777750003771","1429");
		myHSIT.put("757777700003371","1430");
		myHSIT.put("174000007413731","1431");
		myHSIT.put("777743300000000","1432");
		myHSIT.put("577737757413771","1433");
		myHSIT.put("773777740000171","1434");
		myHSIT.put("774777777413731","1435");
		myHSIT.put("577777757403771","1436");
		myHSIT.put("703737740003731","1437");
		myHSIT.put("777776757413371","1438");
		myHSIT.put("674373677777600","1439");
		myHSIT.put("676777677777600","1440");
		myHSIT.put("076777777777600","1441");
		myHSIT.put("614077777777600","1442");
		myHSIT.put("616777777777600","1443");
		myHSIT.put("777400000060771","1444");
		myHSIT.put("407757607760771","1445");
		myHSIT.put("777757677420771","1446");
		myHSIT.put("777637777420771","1447");
		myHSIT.put("777637777420731","1448");
		myHSIT.put("777777677421771","1449");
		myHSIT.put("777777670000771","1450");
		myHSIT.put("777760000000000","1451");
		myHSIT.put("774777677420771","1452");
		myHSIT.put("776177637763771","1453");
		myHSIT.put("774000677760371","1454");
		myHSIT.put("774377677420771","1455");
		myHSIT.put("777637677420771","1456");
		myHSIT.put("777757777420771","1457");
		myHSIT.put("773737777400771","1458");
		myHSIT.put("777776777760700","1459");
		myHSIT.put("777777377763771","1460");
		myHSIT.put("773737677420771","1461");
		myHSIT.put("777777770003771","1462");
		myHSIT.put("670000777760731","1463");
		myHSIT.put("777377770000771","1464");
		myHSIT.put("774077770001071","1465");
		myHSIT.put("757777606060771","1466");
		myHSIT.put("477777777410771","1467");
		myHSIT.put("077777606060671","1468");
		myHSIT.put("017776607760771","1469");
		myHSIT.put("777776607760771","1470");
		myHSIT.put("777737606060771","1471");
		myHSIT.put("477677777410771","1472");
		myHSIT.put("700777746013771","1473");
		myHSIT.put("677777627760771","1474");
		myHSIT.put("777357777760771","1475");
		myHSIT.put("736177607700171","1476");
		myHSIT.put("677777463413701","1477");
		myHSIT.put("675777477413771","1478");
		myHSIT.put("674777470001771","1479");
		myHSIT.put("774776000420771","1480");
		myHSIT.put("774777777423771","1481");
		myHSIT.put("775740003763771","1482");
		myHSIT.put("777717377760771","1483");
		myHSIT.put("777777774023771","1484");
		myHSIT.put("777740007763771","1485");
		myHSIT.put("300036777760771","1486");
		myHSIT.put("000000007777731","1487");
		myHSIT.put("401777777410071","1488");
		myHSIT.put("637777477413750","1489");
		myHSIT.put("677767477413771","1490");
		myHSIT.put("740000007760731","1491");
		myHSIT.put("777700000001131","1492");
		myHSIT.put("777777603413700","1493");
		myHSIT.put("777777676760771","1494");
		myHSIT.put("777777737413771","1495");
		myHSIT.put("777777756413771","1496");
		myHSIT.put("777777775413671","1497");
		myHSIT.put("777777776000371","1498");
		myHSIT.put("677003477413771","1499");
		myHSIT.put("677777477400771","1500");
		myHSIT.put("677777477413671","1501");
		myHSIT.put("477757777013071","1502");
		myHSIT.put("777747757413271","1503");
		myHSIT.put("777747777413671","1504");
		myHSIT.put("777757607760671","1505");
		myHSIT.put("777757757413271","1506");
		myHSIT.put("777757757413771","1507");
		myHSIT.put("777757777413771","1508");
		myHSIT.put("677777472000000","1509");
		myHSIT.put("677777476003771","1510");
		myHSIT.put("677577477413771","1511");
		myHSIT.put("657777477413701","1512");
		myHSIT.put("777577757413771","1513");
		myHSIT.put("757777637760771","1514");
		myHSIT.put("477777777403771","1515");
		myHSIT.put("777377777761771","1516");
		myHSIT.put("477777377417771","1517");
		myHSIT.put("477777377403771","1518");
		myHSIT.put("000037777403771","1519");
		myHSIT.put("477777277403771","1520");
		myHSIT.put("700774007403771","1521");
		myHSIT.put("777737606063771","1522");
		myHSIT.put("777777667763771","1523");
		myHSIT.put("700777747403771","1524");
		myHSIT.put("376137607760771","1525");
		myHSIT.put("700037177760771","1526");
		myHSIT.put("777776774020771","1527");
		myHSIT.put("774377607760771","1528");
		myHSIT.put("777777777720171","1529");
		myHSIT.put("777777607760711","1530");
		myHSIT.put("777776777760071","1531");
		myHSIT.put("777776775760771","1532");
		myHSIT.put("777763777720771","1533");
		myHSIT.put("777721607560731","1534");
		myHSIT.put("777577607760771","1535");
		myHSIT.put("776377607760771","1536");
		myHSIT.put("776137607760731","1537");
		myHSIT.put("774137600020771","1538");
		myHSIT.put("773777777720771","1539");
		myHSIT.put("377377607760771","1540");
		myHSIT.put("376377607760771","1541");
		myHSIT.put("177776777760601","1542");
		myHSIT.put("076160000000071","1543");
		myHSIT.put("777777777760531","1544");
		myHSIT.put("601775607760771","1545");
		myHSIT.put("077777777413011","1546");
		myHSIT.put("777727777760771","1547");
		myHSIT.put("777703400000360","1548");
		myHSIT.put("775777606060731","1549");
		myHSIT.put("377777777660760","1550");
		myHSIT.put("701777740003771","1551");
		myHSIT.put("777777774020631","1552");
		myHSIT.put("775777401760771","1553");
		myHSIT.put("402777607760731","1554");
		myHSIT.put("700001403760371","1555");
		myHSIT.put("775777777660771","1556");
		myHSIT.put("777677774020771","1557");
		myHSIT.put("777777775660771","1558");
		myHSIT.put("777777400000171","1559");
		myHSIT.put("703777777760771","1560");
		myHSIT.put("777777774020371","1561");
		myHSIT.put("777777377400171","1562");
		myHSIT.put("777777636760771","1563");
		myHSIT.put("777776737760571","1564");
		myHSIT.put("777740077760771","1565");
		myHSIT.put("747777777760771","1566");
		myHSIT.put("777777707560771","1567");
		myHSIT.put("774777377420771","1568");
		myHSIT.put("700037000020700","1569");
		myHSIT.put("777740000000000","1570");
		myHSIT.put("777775774413771","1571");
		myHSIT.put("677777777760730","1572");
		myHSIT.put("777777177413371","1573");
		myHSIT.put("760000007760771","1574");
		myHSIT.put("777777347760400","1575");
		myHSIT.put("777774063560771","1576");
		myHSIT.put("776376003760771","1577");
		myHSIT.put("767777777760731","1578");
		myHSIT.put("677777777740771","1579");
		myHSIT.put("777777747760771","1580");
		myHSIT.put("577477777420771","1581");
		myHSIT.put("777777377760700","1582");
		myHSIT.put("777777777760740","1583");
		myHSIT.put("757777630760771","1584");
		myHSIT.put("777777764120771","1585");
		myHSIT.put("177777764020771","1586");
		myHSIT.put("636377770760771","1587");
		myHSIT.put("037637607760771","1588");
		myHSIT.put("776017404760771","1589");
		myHSIT.put("703777340003771","1590");
		myHSIT.put("700377740003371","1591");
		myHSIT.put("774077602000031","1592");
		myHSIT.put("740000000020771","1593");
		myHSIT.put("777400007760771","1594");
		myHSIT.put("674363677777600","1595");
		myHSIT.put("200003777377200","1596");
		myHSIT.put("707777777760771","1597");
		myHSIT.put("262773777772600","1598");
		myHSIT.put("000003777377600","1599");
		myHSIT.put("200003770003600","1600");
		myHSIT.put("676773677777400","1601");
		myHSIT.put("674023777777600","1602");
		myHSIT.put("666773777677600","1603");
		myHSIT.put("200003377377600","1604");
		myHSIT.put("416773677777600","1605");
		myHSIT.put("603777740000771","1606");
		myHSIT.put("601777606060771","1607");
		myHSIT.put("777417607760700","1608");
		myHSIT.put("777637777413771","1609");
		myHSIT.put("763777607560771","1610");
		myHSIT.put("776777777720771","1611");
		myHSIT.put("077577757003371","1612");
		myHSIT.put("177777777760731","1613");
		myHSIT.put("777776607760700","1614");
		myHSIT.put("777417777760731","1615");
		myHSIT.put("003777740003771","1616");
		myHSIT.put("700036776760771","1617");
		myHSIT.put("577777000000011","1618");
		myHSIT.put("000002777760731","1619");
		myHSIT.put("777777604360771","1620");
		myHSIT.put("737737777760731","1621");
		myHSIT.put("761777777760731","1622");
		myHSIT.put("777777677760700","1623");
		myHSIT.put("776177607560771","1624");
		myHSIT.put("664073777700600","1625");
		myHSIT.put("777777776760771","1626");
		myHSIT.put("777137777760731","1627");
		myHSIT.put("077737777413771","1628");
		myHSIT.put("777777774020751","1629");
		myHSIT.put("777777601760771","1630");
		myHSIT.put("757777777760571","1631");
		myHSIT.put("577777776000371","1632");
		myHSIT.put("637403607760671","1633");
		myHSIT.put("777777777723771","1634");
		myHSIT.put("200003777363600","1635");
		myHSIT.put("200003774075600","1636");
		myHSIT.put("676773777577600","1637");
		myHSIT.put("577767777763771","1638");
		myHSIT.put("743777607760771","1639");
		myHSIT.put("777777777720740","1640");
		myHSIT.put("777737177760771","1641");
		myHSIT.put("700017777760771","1642");
		myHSIT.put("777777423560771","1643");
		myHSIT.put("777777404020771","1644");
		myHSIT.put("377767777760771","1645");
		myHSIT.put("777737607020771","1646");
		myHSIT.put("777777477760731","1647");
		myHSIT.put("400017777413771","1648");
		myHSIT.put("757677777413731","1649");
		myHSIT.put("720001400000771","1650");
		myHSIT.put("000000000003661","1651");
		myHSIT.put("477777774020771","1652");
		myHSIT.put("730000000000171","1653");
		myHSIT.put("777674077413671","1654");
		myHSIT.put("777723777760771","1655");
		myHSIT.put("777767770000000","1656");
		myHSIT.put("777777377000371","1657");
		myHSIT.put("777777714020771","1658");
		myHSIT.put("777703700000000","1659");
		myHSIT.put("777777600060731","1660");
		myHSIT.put("677737607760671","1661");
		myHSIT.put("600000000060731","1662");
		myHSIT.put("777777607520771","1663");
		myHSIT.put("777777600560731","1664");
		myHSIT.put("377400000000131","1665");
		myHSIT.put("664063777776600","1666");
		myHSIT.put("616773777777200","1667");
		myHSIT.put("676763677776200","1668");
		myHSIT.put("676763677776600","1669");
		myHSIT.put("064073777777600","1670");
		myHSIT.put("377767607760771","1671");
		myHSIT.put("776177775760771","1672");
		myHSIT.put("614373777777600","1673");
		myHSIT.put("000000000003761","1674");
		myHSIT.put("703367400001771","1675");
		myHSIT.put("477777377613071","1676");
		myHSIT.put("703377400203771","1677");
		myHSIT.put("777774377720771","1678");
		myHSIT.put("777767737760771","1679");
		myHSIT.put("475777777413071","1680");
		myHSIT.put("377777774020771","1681");
		myHSIT.put("662773677777600","1682");
		myHSIT.put("677777777660771","1683");
		myHSIT.put("776000000000000","1684");
		myHSIT.put("776077607760771","1685");
		myHSIT.put("777740004020771","1686");
		myHSIT.put("777767775760771","1687");
		myHSIT.put("777777403760771","1688");
		myHSIT.put("777777607600031","1689");
		myHSIT.put("777777777762771","1690");
		myHSIT.put("677727607760771","1691");
		myHSIT.put("417136777760731","1692");
		myHSIT.put("737737607760771","1693");
		myHSIT.put("277737607760771","1694");
		myHSIT.put("677737607760770","1695");
		myHSIT.put("707737607760771","1696");
		myHSIT.put("775767607760671","1697");
		myHSIT.put("777727607700031","1698");
		myHSIT.put("777347607760671","1699");
		myHSIT.put("753777776760771","1700");
		myHSIT.put("777317607760771","1701");
		myHSIT.put("037737607760771","1702");
		myHSIT.put("677737607560771","1703");
		myHSIT.put("477727607760771","1704");
		myHSIT.put("777703777460771","1705");
		myHSIT.put("717777607760771","1706");
		myHSIT.put("677777400760771","1707");
		myHSIT.put("776777607760771","1708");
		myHSIT.put("777737606760771","1709");
		myHSIT.put("777737607760731","1710");
		myHSIT.put("677337607760771","1711");
		myHSIT.put("677317607760771","1712");
		myHSIT.put("663777607760771","1713");
		myHSIT.put("777740607760771","1714");
		myHSIT.put("437777607760731","1715");
		myHSIT.put("767737675560771","1716");
		myHSIT.put("777776777460771","1717");
		myHSIT.put("677776607760771","1718");
		myHSIT.put("677337607760671","1719");
		myHSIT.put("777767774413771","1720");
		myHSIT.put("777746777760771","1721");
		myHSIT.put("664073677777200","1722");
		myHSIT.put("664073770037600","1723");
		myHSIT.put("435773037604771","1724");
		myHSIT.put("717777757413771","1725");
		myHSIT.put("777577777413371","1726");
		myHSIT.put("776377703760771","1727");
		myHSIT.put("776377603760771","1728");
		myHSIT.put("700000004177771","1729");
		myHSIT.put("676773777747600","1730");
		myHSIT.put("577777774413771","1731");
		myHSIT.put("577767777413731","1732");
		myHSIT.put("376774737760771","1733");
		myHSIT.put("077767777413731","1734");
		myHSIT.put("777777774320771","1735");
		myHSIT.put("777777700000371","1736");
		myHSIT.put("777760017760771","1737");
		myHSIT.put("777760063760771","1738");
		myHSIT.put("577777377760700","1739");
		myHSIT.put("467777774000071","1740");
		myHSIT.put("600000077413771","1741");
		myHSIT.put("775777774020771","1742");
		myHSIT.put("767777777720631","1743");
		myHSIT.put("577776770000000","1744");
		myHSIT.put("773737777760771","1745");
		myHSIT.put("476377737760771","1746");
		myHSIT.put("717777757760771","1747");
		myHSIT.put("777757777620771","1748");
		myHSIT.put("777777734720771","1749");
		myHSIT.put("777767607760731","1750");
		myHSIT.put("700066777760771","1751");
		myHSIT.put("677777607740771","1752");
		myHSIT.put("700037777760771","1753");
		myHSIT.put("777767777760601","1754");
		myHSIT.put("677777607560771","1755");
		myHSIT.put("700026777760771","1756");
		myHSIT.put("757777600000000","1757");
		myHSIT.put("777777607540771","1758");
		myHSIT.put("772177606760771","1759");
		myHSIT.put("476177607760771","1760");
		myHSIT.put("677767777760771","1761");
		myHSIT.put("477777637413071","1762");
		myHSIT.put("437777774320731","1763");
		myHSIT.put("400037777405771","1764");
		myHSIT.put("703337400001771","1765");
		myHSIT.put("760777607760771","1766");
		myHSIT.put("777770007760771","1767");
		myHSIT.put("777777207560771","1768");
		myHSIT.put("677761607760771","1769");
		myHSIT.put("000000004020571","1770");
		myHSIT.put("757777744020771","1771");
		myHSIT.put("776377774020071","1772");
		myHSIT.put("777777547760471","1773");
		myHSIT.put("777777376760771","1774");
		myHSIT.put("277777477413771","1775");
		myHSIT.put("776377760000171","1776");
		myHSIT.put("703776000000011","1777");
		myHSIT.put("777770000000371","1778");
		myHSIT.put("776477777760771","1779");
		myHSIT.put("000000000004771","1780");
		myHSIT.put("677777477413770","1781");
		myHSIT.put("770777607777671","1782");
		myHSIT.put("776377743760771","1783");
		myHSIT.put("676760777777600","1784");
		myHSIT.put("770000004760771","1785");
		myHSIT.put("037647777760771","1786");
		myHSIT.put("703777740002171","1787");
		myHSIT.put("777703400000361","1788");
		myHSIT.put("703777740000171","1789");
		myHSIT.put("777777000360771","1790");
		myHSIT.put("677777607160771","1791");
		myHSIT.put("577777607720771","1792");
		myHSIT.put("777777777760000","1793");
		myHSIT.put("277777737760731","1794");
		myHSIT.put("777777740000371","1795");
		myHSIT.put("000077777760771","1796");
		myHSIT.put("777777777660731","1797");
		myHSIT.put("577777744020771","1798");
		myHSIT.put("777777347760431","1799");
		myHSIT.put("777777407760771","1800");
		myHSIT.put("777777777413730","1801");
		myHSIT.put("777767777720631","1802");
		myHSIT.put("617777607760771","1803");
		myHSIT.put("777777577720771","1804");
		myHSIT.put("777777777740671","1805");
		myHSIT.put("677747757760771","1806");
		myHSIT.put("777777774020571","1807");
		myHSIT.put("677777757760771","1808");
		myHSIT.put("707777347740471","1809");
		myHSIT.put("200001377376600","1810");
		myHSIT.put("677300004760731","1811");
		myHSIT.put("777777637720771","1812");
		myHSIT.put("757767774020771","1813");
		myHSIT.put("777777617560571","1814");
		myHSIT.put("677777606760771","1815");
		myHSIT.put("676773677603600","1816");
		myHSIT.put("474563777777600","1817");
		myHSIT.put("640013777763600","1818");
		myHSIT.put("476763677777200","1819");
		myHSIT.put("676773277777200","1820");
		myHSIT.put("777347777760771","1821");
		myHSIT.put("777777774020770","1822");
		myHSIT.put("777776777760770","1823");
		myHSIT.put("760377777760701","1824");
		myHSIT.put("000000004220771","1825");
		myHSIT.put("777716777760771","1826");
		myHSIT.put("777703600020771","1827");
		myHSIT.put("777776207760771","1828");
		myHSIT.put("775767777760671","1829");
		myHSIT.put("776177207760771","1830");
		myHSIT.put("777777663740771","1831");
		myHSIT.put("777773607760771","1832");
		myHSIT.put("777705077560571","1833");
		myHSIT.put("377737777760771","1834");
		myHSIT.put("777703377760731","1835");
		myHSIT.put("777777007660751","1836");
		myHSIT.put("000000000203771","1837");
		myHSIT.put("777367607760771","1838");
		myHSIT.put("777327607760770","1839");
		myHSIT.put("776327777760771","1840");
		myHSIT.put("776177607740771","1841");
		myHSIT.put("476763777777600","1842");
		myHSIT.put("740777777760731","1843");
		myHSIT.put("777763607760771","1844");
		myHSIT.put("000173777777600","1845");
		myHSIT.put("640013776777600","1846");
		myHSIT.put("676773677377600","1847");
		myHSIT.put("676740677777600","1848");
		myHSIT.put("674573777777600","1849");
		myHSIT.put("664073776177600","1850");
		myHSIT.put("464073777777600","1851");
		myHSIT.put("676773770003600","1852");
		myHSIT.put("674773674377200","1853");
		myHSIT.put("676773770000600","1854");
		myHSIT.put("664073777773600","1855");
		myHSIT.put("664072777777600","1856");
		myHSIT.put("676573777177600","1857");
		myHSIT.put("264071777777600","1858");
		myHSIT.put("624073777700600","1859");
		myHSIT.put("664073777741600","1860");
		myHSIT.put("664073777707600","1861");
		myHSIT.put("640000037777600","1862");
		myHSIT.put("664073777577600","1863");
		myHSIT.put("400027777413671","1864");
		myHSIT.put("757767777413631","1865");
		myHSIT.put("477776377413771","1866");
		myHSIT.put("770777707777651","1867");
		myHSIT.put("677777477413761","1868");
		myHSIT.put("777700000000000","1869");
		myHSIT.put("077777607760730","1870");
		myHSIT.put("700037750060771","1871");
		myHSIT.put("777777774060771","1872");
		myHSIT.put("777774606060731","1873");
		myHSIT.put("776370001760331","1874");
		myHSIT.put("477001777413771","1875");
		myHSIT.put("740077777777671","1876");
		myHSIT.put("737377777760771","1877");
		myHSIT.put("703777740003600","1878");
		myHSIT.put("777737400413731","1879");
		myHSIT.put("037603607400011","1880");
		myHSIT.put("077777377413771","1881");
		myHSIT.put("703776600003771","1882");
		myHSIT.put("703777740003760","1883");
		myHSIT.put("777774777000171","1884");
		myHSIT.put("037637377760771","1885");
		myHSIT.put("637777477413771","1886");
		myHSIT.put("703777707770371","1887");
		myHSIT.put("677777707760771","1888");
		myHSIT.put("007777707760771","1889");
		myHSIT.put("777777707760731","1890");
		myHSIT.put("777777773720771","1891");
		myHSIT.put("777777660000171","1892");
		myHSIT.put("777777577620771","1893");
		myHSIT.put("747777607760771","1894");
		myHSIT.put("777777607460731","1895");
		myHSIT.put("777637777770711","1896");
		myHSIT.put("777777770203371","1897");
		myHSIT.put("777777770003371","1898");
		myHSIT.put("677640007760771","1899");
		myHSIT.put("577777777413770","1900");
		myHSIT.put("777777714413771","1901");
		myHSIT.put("707777774413731","1902");
		myHSIT.put("737777774413771","1903");
		myHSIT.put("037676000000371","1904");
		myHSIT.put("777777777460771","1905");
		myHSIT.put("777777605560771","1906");
		myHSIT.put("777777777774771","1907");
		myHSIT.put("777777776020771","1908");
		myHSIT.put("775777777760700","1909");
		myHSIT.put("677737707760771","1910");
		myHSIT.put("677737707760770","1911");
		myHSIT.put("775767757760771","1912");
		myHSIT.put("777767757760771","1913");
		myHSIT.put("377777607560771","1914");
		myHSIT.put("776377737760771","1915");
		myHSIT.put("777701777760771","1916");
		myHSIT.put("577777777760770","1917");
		myHSIT.put("776377777700771","1918");
		myHSIT.put("777777377720731","1919");
		myHSIT.put("703777770001771","1920");
		myHSIT.put("577777407560731","1921");
		myHSIT.put("777702777760771","1922");
		myHSIT.put("757777777700771","1923");
		myHSIT.put("577747404760671","1924");
		myHSIT.put("754100777677761","1925");
		myHSIT.put("777777777760701","1926");
		myHSIT.put("777737764020771","1927");
		myHSIT.put("757777776000771","1928");
		myHSIT.put("777777777760300","1929");
		myHSIT.put("477777737760771","1930");
		myHSIT.put("767000007720731","1931");
		myHSIT.put("676733777777600","1932");
		myHSIT.put("777777607760770","1933");
		myHSIT.put("777777724020771","1934");
		myHSIT.put("677767607760671","1935");
		myHSIT.put("577677077760771","1936");
		myHSIT.put("777777404760760","1937");
		myHSIT.put("776377777760770","1938");
		myHSIT.put("777777775720731","1939");
		myHSIT.put("503777000003771","1940");
		myHSIT.put("000000000003071","1941");
		myHSIT.put("703377740003771","1942");
		myHSIT.put("477777777613071","1943");
		myHSIT.put("400077777413071","1944");
		myHSIT.put("503767740003171","1945");
		myHSIT.put("777777777412700","1946");
		myHSIT.put("703771740003771","1947");
		myHSIT.put("477777777403071","1948");
		myHSIT.put("703777700000771","1949");
		myHSIT.put("703777747770771","1950");
		myHSIT.put("703777000003771","1951");
		myHSIT.put("777777774000771","1952");
		myHSIT.put("777776774020601","1953");
		myHSIT.put("770377777777671","1954");
		myHSIT.put("777737777761771","1955");
		myHSIT.put("467777777413071","1956");
		myHSIT.put("477777777013771","1957");
		myHSIT.put("477777777513471","1958");
		myHSIT.put("703773740003771","1959");
		myHSIT.put("777177607760771","1960");
		myHSIT.put("417777777413071","1961");
		myHSIT.put("700000000003771","1962");
		myHSIT.put("703774340003771","1963");
		myHSIT.put("477763777413071","1964");
		myHSIT.put("703777740003770","1965");
		myHSIT.put("477777777413011","1966");
		myHSIT.put("703777740000031","1967");
		myHSIT.put("703777740003711","1968");
		myHSIT.put("777777776413700","1969");
		myHSIT.put("777776757413771","1970");
		myHSIT.put("400000040003771","1971");
		myHSIT.put("703777740000600","1972");
		myHSIT.put("703737740002771","1973");
		myHSIT.put("777771070000771","1974");
		myHSIT.put("577777763420771","1975");
		myHSIT.put("703777747763771","1976");
		myHSIT.put("477777037413071","1977");
		myHSIT.put("700373740003771","1978");
		myHSIT.put("777777777400771","1979");
		myHSIT.put("700377740000771","1980");
		myHSIT.put("103777740003171","1981");
		myHSIT.put("777777777400031","1982");
		myHSIT.put("474000377413031","1983");
		myHSIT.put("400343436742200","1984");
		myHSIT.put("577746616560601","1985");
		myHSIT.put("777777770360771","1986");
		myHSIT.put("777777777760231","1987");
		myHSIT.put("777777604020731","1988");
		myHSIT.put("656773377777600","1989");
		myHSIT.put("077777774020731","1990");
		myHSIT.put("754175417672361","1991");
		myHSIT.put("677777477413000","1992");
		myHSIT.put("006377777760771","1993");
		myHSIT.put("414343436742200","1994");
		myHSIT.put("776377701760771","1995");
		myHSIT.put("777400037760771","1996");
		myHSIT.put("554347436400261","1997");
		myHSIT.put("777777607740071","1998");
		myHSIT.put("777701607760771","1999");
		myHSIT.put("776377777760700","2000");
		myHSIT.put("576337777760771","2001");
		myHSIT.put("777417777700000","2002");
		myHSIT.put("001677777760771","2003");
		myHSIT.put("776377777760071","2004");
		myHSIT.put("017777600000371","2005");
		myHSIT.put("400377607560771","2006");
		myHSIT.put("777737677760771","2007");
		myHSIT.put("577377774020771","2008");
		myHSIT.put("677177477413771","2009");
		myHSIT.put("400037777413741","2010");
		myHSIT.put("467777777413771","2011");
		myHSIT.put("677777761760771","2012");
		myHSIT.put("776177607760740","2013");
		myHSIT.put("776177607360771","2014");
		myHSIT.put("736177607760771","2015");
		myHSIT.put("737776777760601","2016");
		myHSIT.put("677777606060771","2017");
		myHSIT.put("760760047700771","2018");
		myHSIT.put("777776777760740","2019");
		myHSIT.put("700016777760771","2020");
		myHSIT.put("777777777700031","2021");
		myHSIT.put("077776775760731","2022");
		myHSIT.put("777737777720171","2023");
		myHSIT.put("676363777777600","2024");
		myHSIT.put("777777777700371","2025");
		myHSIT.put("777777777740171","2026");
		myHSIT.put("777777777000731","2027");
		myHSIT.put("777777604020771","2028");
		myHSIT.put("777774376000711","2029");
		myHSIT.put("777777704020751","2030");
		myHSIT.put("740000377760771","2031");
		myHSIT.put("037627777760771","2032");
		myHSIT.put("777777714160371","2033");
		myHSIT.put("577677777760771","2034");
		myHSIT.put("277777077413771","2035");
		myHSIT.put("777777774520771","2036");
		myHSIT.put("777777774360771","2037");
		myHSIT.put("603777607560771","2038");
		myHSIT.put("777776777560601","2039");
		myHSIT.put("777737777760371","2040");
		myHSIT.put("777777760000771","2041");
		myHSIT.put("000000007660771","2042");
		myHSIT.put("777777737760740","2043");
		myHSIT.put("577777604020771","2044");
		myHSIT.put("776037607760771","2045");
		myHSIT.put("776377377760731","2046");
		myHSIT.put("776163000000071","2047");
		myHSIT.put("777737777320771","2048");
		myHSIT.put("777770777760771","2049");
		myHSIT.put("575777707760771","2050");
		myHSIT.put("777777760000140","2051");
		myHSIT.put("737737777760771","2052");
		myHSIT.put("776377775760771","2053");
		myHSIT.put("677777607760371","2054");
		myHSIT.put("776377777720071","2055");
		myHSIT.put("770637777767671","2056");
		myHSIT.put("774767777420771","2057");
		myHSIT.put("774677607777071","2058");
		myHSIT.put("700000000000371","2059");
		myHSIT.put("477017777413071","2060");
		myHSIT.put("740177777013731","2061");
		myHSIT.put("777776777600000","2062");
		myHSIT.put("700377707776671","2063");
		myHSIT.put("770757777767671","2064");
		myHSIT.put("776017407760771","2065");
		myHSIT.put("777777400060771","2066");
		myHSIT.put("777771777760771","2067");
		myHSIT.put("377777763760771","2068");
		myHSIT.put("777777707760371","2069");
		myHSIT.put("777777637760771","2070");
		myHSIT.put("777720774760771","2071");
		myHSIT.put("000000004000171","2072");
		myHSIT.put("776000607760771","2073");
		myHSIT.put("774177677413771","2074");
		myHSIT.put("740617777760771","2075");
		myHSIT.put("777776777420771","2076");
		myHSIT.put("757377777720771","2077");
		myHSIT.put("771347777740771","2078");
		myHSIT.put("777777776000740","2079");
		myHSIT.put("777777776760571","2080");
		myHSIT.put("767777777720571","2081");
		myHSIT.put("577767777760731","2082");
		myHSIT.put("677777437413771","2083");
		myHSIT.put("677776477413771","2084");
		myHSIT.put("674377477413771","2085");
		myHSIT.put("747777770020771","2086");
		myHSIT.put("777737600020771","2087");
		myHSIT.put("777777737760731","2088");
		myHSIT.put("777773777720771","2089");
		myHSIT.put("777777761720771","2090");
		myHSIT.put("777637770020771","2091");
		myHSIT.put("777737770020771","2092");
		myHSIT.put("757777770020771","2093");
		myHSIT.put("777773700020731","2094");
		myHSIT.put("777037777760031","2095");
		myHSIT.put("077777477413771","2096");
		myHSIT.put("617777477413771","2097");
		myHSIT.put("647777477413731","2098");
		myHSIT.put("647777477413771","2099");
		myHSIT.put("700377700003771","2100");
		myHSIT.put("000000000003000","2101");
		myHSIT.put("703760000000771","2102");
		myHSIT.put("074000077777600","2103");
		myHSIT.put("400017777413071","2104");
		myHSIT.put("776176607760771","2105");
		myHSIT.put("777777400000371","2106");
		myHSIT.put("776177400000131","2107");
		myHSIT.put("100076777760771","2108");
		myHSIT.put("777775077560700","2109");
		myHSIT.put("777777607000771","2110");
		myHSIT.put("701777777720771","2111");
		myHSIT.put("777717777760730","2112");
		myHSIT.put("777177777760731","2113");
		myHSIT.put("777761777760771","2114");
		myHSIT.put("777777434760771","2115");
		myHSIT.put("777777774020701","2116");
		myHSIT.put("776377777763771","2117");
		myHSIT.put("777737347760411","2118");
		myHSIT.put("777600774413771","2119");
		myHSIT.put("007777707760351","2120");
		myHSIT.put("741737777760371","2121");
		myHSIT.put("741736777760771","2122");
		myHSIT.put("777577777777771","2123");
		myHSIT.put("737777777777771","2124");
		myHSIT.put("777777404760571","2125");
		myHSIT.put("743777700003371","2126");
		myHSIT.put("777760007760751","2127");
		myHSIT.put("777707777760771","2128");
		myHSIT.put("676773677377200","2129");
		myHSIT.put("674773677777200","2130");
		myHSIT.put("676573477777600","2131");
		myHSIT.put("776277777760771","2132");
		myHSIT.put("676001677777200","2133");
		myHSIT.put("740000007320771","2134");
		myHSIT.put("776367677760771","2135");
		myHSIT.put("777777000320771","2136");
		myHSIT.put("177777774003731","2137");
		myHSIT.put("774777770020771","2138");
		myHSIT.put("740637777760771","2139");
		myHSIT.put("777777417760771","2140");
		myHSIT.put("636773677777600","2141");
		myHSIT.put("677777777560771","2142");
		myHSIT.put("177777677760771","2143");
		myHSIT.put("773777776760731","2144");
		myHSIT.put("703777640003771","2145");
		myHSIT.put("077577747003371","2146");
		myHSIT.put("703777740003011","2147");
		myHSIT.put("700000000000000","2148");
		myHSIT.put("703757740000771","2149");
		myHSIT.put("774077407777071","2150");
		myHSIT.put("777777347760771","2151");
		myHSIT.put("777777702003371","2152");
		myHSIT.put("664033777777600","2153");
		myHSIT.put("777777434020771","2154");
		myHSIT.put("777777756013371","2155");
		myHSIT.put("777777757410000","2156");
		myHSIT.put("770777774377671","2157");
		myHSIT.put("777777757410371","2158");
		myHSIT.put("777777757411371","2159");
		myHSIT.put("777737777700171","2160");
		myHSIT.put("000000007740771","2161");
		myHSIT.put("777776004760771","2162");
		myHSIT.put("757377777700000","2163");
		myHSIT.put("777777777740731","2164");
		myHSIT.put("777747607760771","2165");
		myHSIT.put("777776404360601","2166");
		myHSIT.put("703777740002331","2167");
		myHSIT.put("557747606760771","2168");
		myHSIT.put("777776677760601","2169");
		myHSIT.put("777773777760571","2170");
		myHSIT.put("576777777720771","2171");
		myHSIT.put("370177607760771","2172");
		myHSIT.put("377777606060771","2173");
		myHSIT.put("774037607777071","2174");
		myHSIT.put("760000003760771","2175");
		myHSIT.put("376147607760771","2176");
		myHSIT.put("774177406760771","2177");
		myHSIT.put("776376777760601","2178");
		myHSIT.put("776377760360771","2179");
		myHSIT.put("777676777760601","2180");
		myHSIT.put("776371001760771","2181");
		myHSIT.put("777377677763771","2182");
		myHSIT.put("764137607720771","2183");
		myHSIT.put("766177407760771","2184");
		myHSIT.put("774377003740771","2185");
		myHSIT.put("777016777760671","2186");
		myHSIT.put("777777347763771","2187");
		myHSIT.put("777377777760731","2188");
		myHSIT.put("777777761720740","2189");
		myHSIT.put("000177777420771","2190");
		myHSIT.put("777741007760771","2191");
		myHSIT.put("500076777560771","2192");
		myHSIT.put("777637763660771","2193");
		myHSIT.put("740377777760731","2194");
		myHSIT.put("477777376013771","2195");
		myHSIT.put("777775606060731","2196");
		myHSIT.put("477776077413771","2197");
		myHSIT.put("417777777620771","2198");
		myHSIT.put("637737607760771","2199");
		myHSIT.put("777777607540761","2200");
		myHSIT.put("177777607760771","2201");
		myHSIT.put("777743557760771","2202");
		myHSIT.put("707777777720771","2203");
		myHSIT.put("760121400200771","2204");
		myHSIT.put("000000007700371","2205");
		myHSIT.put("676077477413771","2206");
		myHSIT.put("777377777413731","2207");
		myHSIT.put("761777777760771","2208");
		myHSIT.put("676600000000771","2209");
		myHSIT.put("747773777460771","2210");
		myHSIT.put("757777707720671","2211");
		myHSIT.put("777777404720571","2212");
		myHSIT.put("775777777720751","2213");
		myHSIT.put("776777577720771","2214");
		myHSIT.put("767777770000771","2215");
		myHSIT.put("777777147720771","2216");
		myHSIT.put("777777757760751","2217");
		myHSIT.put("777737677420771","2218");
		myHSIT.put("774003774020771","2219");
		myHSIT.put("000000007160771","2220");
		myHSIT.put("757737604020771","2221");
		myHSIT.put("770020607760771","2222");
		myHSIT.put("777000777760731","2223");
		myHSIT.put("776337777763771","2224");
		myHSIT.put("700076774360771","2225");
		myHSIT.put("777476777760771","2226");
		myHSIT.put("777777730020771","2227");
		myHSIT.put("776777757760771","2228");
		myHSIT.put("776177607763771","2229");
		myHSIT.put("777700077760771","2230");
		myHSIT.put("700000004020711","2231");
		myHSIT.put("777737777700000","2232");
		myHSIT.put("777763770000000","2233");
		myHSIT.put("777737760000151","2234");
		myHSIT.put("777777377740371","2235");
		myHSIT.put("477477777413071","2236");
		myHSIT.put("702377740003771","2237");
		myHSIT.put("471777777413071","2238");
		myHSIT.put("741737777760671","2239");
		myHSIT.put("737777777000731","2240");
		myHSIT.put("477776377413071","2241");
		myHSIT.put("777777743440771","2242");
		myHSIT.put("774003777775071","2243");
		myHSIT.put("777577777760731","2244");
		myHSIT.put("774067600000071","2245");
		myHSIT.put("777760007760731","2246");
		myHSIT.put("777740003763771","2247");
		myHSIT.put("777774607760771","2248");
		myHSIT.put("777600000060771","2249");
		myHSIT.put("737776777760731","2250");
		myHSIT.put("477740001760771","2251");
		myHSIT.put("777735777423771","2252");
		myHSIT.put("670177607760731","2253");
		myHSIT.put("730000777760771","2254");
		myHSIT.put("707777770060731","2255");
		myHSIT.put("377700000020731","2256");
		myHSIT.put("777700004020771","2257");
		myHSIT.put("777777777700011","2258");
		myHSIT.put("577637777420771","2259");
		myHSIT.put("777017577420771","2260");
		myHSIT.put("777737776000131","2261");
		myHSIT.put("637760007760771","2262");
		myHSIT.put("767777607760771","2263");
		myHSIT.put("777737607760711","2264");
		myHSIT.put("577777606060771","2265");
		myHSIT.put("000037607760771","2266");
		myHSIT.put("701377400001771","2267");
		myHSIT.put("777767606060771","2268");
		myHSIT.put("703377400000771","2269");
		myHSIT.put("742377777760771","2270");
		myHSIT.put("677717607760771","2271");
		myHSIT.put("740777777760771","2272");
		myHSIT.put("777777607720731","2273");
		myHSIT.put("037777407560731","2274");
		myHSIT.put("740000007760771","2275");
		myHSIT.put("777777777743771","2276");
		myHSIT.put("703377600001771","2277");
		myHSIT.put("777777606061771","2278");
		myHSIT.put("777777777473771","2279");
		myHSIT.put("777727606060771","2280");
		myHSIT.put("777775607760771","2281");
		myHSIT.put("776377777560771","2282");
		myHSIT.put("777761377760731","2283");
		myHSIT.put("776177407760731","2284");
		myHSIT.put("777774777760771","2285");
		myHSIT.put("300076777760771","2286");
		myHSIT.put("000000007760711","2287");
		myHSIT.put("477777774000071","2288");
		myHSIT.put("777717761720771","2289");
		myHSIT.put("777777607600171","2290");
		myHSIT.put("777417777400000","2291");
		myHSIT.put("677777607760071","2292");
		myHSIT.put("777737037760771","2293");
		myHSIT.put("777763774020771","2294");
		myHSIT.put("737776777560771","2295");
		myHSIT.put("737777777760601","2296");
		myHSIT.put("777763007760771","2297");
		myHSIT.put("777777647760771","2298");
		myHSIT.put("776107607760771","2299");
		myHSIT.put("743760047700771","2300");
		myHSIT.put("776167607760671","2301");
		myHSIT.put("776117607760771","2302");
		myHSIT.put("743776777760601","2303");
		myHSIT.put("617776777760771","2304");
		myHSIT.put("600076777760771","2305");
		myHSIT.put("777737770000771","2306");
		myHSIT.put("777703777560731","2307");
		myHSIT.put("777777757403771","2308");
		myHSIT.put("774377003740371","2309");
		myHSIT.put("776177606760771","2310");
		myHSIT.put("776177607760471","2311");
		myHSIT.put("77377777760611","2312");
		myHSIT.put("776321777760771","2313");
		myHSIT.put("737637400020771","2314");
		myHSIT.put("677777207760771","2315");
		myHSIT.put("000000000003740","2316");
		myHSIT.put("777377347760400","2317");
		myHSIT.put("674000000000071","2318");
		myHSIT.put("777770177413700","2319");
		myHSIT.put("777376777760771","2320");
		myHSIT.put("017777777320771","2321");
		myHSIT.put("777777760160731","2322");
		myHSIT.put("477774777413071","2323");
		myHSIT.put("770737777777671","2324");
		myHSIT.put("777737777413701","2325");
		myHSIT.put("677777607760711","2326");
		myHSIT.put("777776677760771","2327");
		myHSIT.put("777737757700011","2328");
		myHSIT.put("777777776120771","2329");
		myHSIT.put("600077743760171","2330");
		myHSIT.put("777777605760471","2331");
		myHSIT.put("774037600760700","2332");
		myHSIT.put("777777774020711","2333");
		myHSIT.put("777735777760730","2334");
		myHSIT.put("000000777760730","2335");
		myHSIT.put("777637607760771","2336");
		myHSIT.put("777377774020771","2337");
		myHSIT.put("777600777720771","2338");
		myHSIT.put("767777777760730","2339");
		myHSIT.put("637777777760730","2340");
		myHSIT.put("777733607760771","2341");
		myHSIT.put("037777607760771","2342");
		myHSIT.put("776177607400771","2343");
		myHSIT.put("776373001760771","2344");
		myHSIT.put("000000007760760","2345");
		myHSIT.put("777737776413731","2346");
		myHSIT.put("776000003660771","2347");
		myHSIT.put("577761777740371","2348");
		myHSIT.put("637777637760771","2349");
		myHSIT.put("600017607760771","2350");
		myHSIT.put("657777477413771","2351");
		myHSIT.put("747377777760771","2352");
		myHSIT.put("776177600000000","2353");
		myHSIT.put("777737737420771","2354");
		myHSIT.put("777777345760471","2355");
		myHSIT.put("717776777760771","2356");
		myHSIT.put("000000174020771","2357");
		myHSIT.put("777777777000071","2358");
		myHSIT.put("703677740003171","2359");
		myHSIT.put("700076777760271","2360");
		myHSIT.put("703776040000771","2361");
		myHSIT.put("674000017000771","2362");
		myHSIT.put("737700777413700","2363");
		myHSIT.put("703601740003771","2364");
		myHSIT.put("777337577360771","2365");
		myHSIT.put("777777777760171","2366");
		myHSIT.put("766377777760771","2367");
		myHSIT.put("777737206060771","2368");
		myHSIT.put("777776777560731","2369");
		myHSIT.put("276773777777600","2370");
		myHSIT.put("400777777620731","2371");
		myHSIT.put("600777607760771","2372");
		myHSIT.put("700337740003771","2373");
		myHSIT.put("717737777760771","2374");
		myHSIT.put("767777774020771","2375");
		myHSIT.put("770000000020771","2376");
		myHSIT.put("770777777420711","2377");
		myHSIT.put("776162000000071","2378");
		myHSIT.put("777600000000000","2379");
		myHSIT.put("777623405760471","2380");
		myHSIT.put("777677777720771","2381");
		myHSIT.put("777723400000360","2382");
		myHSIT.put("757737607760771","2383");
		myHSIT.put("777740003760731","2384");
		myHSIT.put("777777606060471","2385");
		myHSIT.put("777777607400171","2386");
		myHSIT.put("777777774000331","2387");
		myHSIT.put("746177607760771","2388");
		myHSIT.put("000202000002771","2389");
		myHSIT.put("577737747420771","2390");
		myHSIT.put("703377400003771","2391");
		myHSIT.put("703777700003171","2392");
		myHSIT.put("777037777760771","2393");
		myHSIT.put("767700777760731","2394");
		myHSIT.put("777777767413771","2395");
		myHSIT.put("377776777760711","2396");
		myHSIT.put("700076377760771","2397");
		myHSIT.put("703777744037771","2398");
		myHSIT.put("700036777760671","2399");
		myHSIT.put("437777774020771","2400");
		myHSIT.put("777777607600371","2401");
		myHSIT.put("776177607760711","2402");
		myHSIT.put("677737601760771","2403");
		myHSIT.put("577777750000000","2404");
		myHSIT.put("600377477413771","2405");
		myHSIT.put("777777737760471","2406");
		myHSIT.put("770003400003071","2407");
		myHSIT.put("777774000000771","2408");
		myHSIT.put("777737757760771","2409");
		myHSIT.put("376177607760731","2410");
		myHSIT.put("700076777760711","2411");
		myHSIT.put("777737607703771","2412");
		myHSIT.put("000000000003760","2413");
		myHSIT.put("777737674020771","2414");
		myHSIT.put("776177607700771","2415");
		myHSIT.put("770777407560731","2416");
		myHSIT.put("777777544320771","2417");
		myHSIT.put("767777767760771","2418");
		myHSIT.put("703000000000371","2419");
		myHSIT.put("777777777720700","2420");
		myHSIT.put("777437737700000","2421");
		myHSIT.put("774037777360700","2422");
		myHSIT.put("777777437760731","2423");
		myHSIT.put("777775777760700","2424");
		myHSIT.put("772174607760771","2425");
		myHSIT.put("777777777000271","2426");
		myHSIT.put("377777777000031","2427");
		myHSIT.put("674017607760771","2428");
		myHSIT.put("777777743560771","2429");
		myHSIT.put("664013777777600","2430");
		myHSIT.put("677777607760671","2431");
		myHSIT.put("646773777700600","2432");
		myHSIT.put("676563677477600","2433");
		myHSIT.put("000033777777600","2434");
		myHSIT.put("777777777677731","2435");
		myHSIT.put("776177607020771","2436");
		myHSIT.put("777077777760771","2437");
		myHSIT.put("776777606020771","2438");
		myHSIT.put("776171000000071","2439");
		myHSIT.put("770777777740671","2440");
		myHSIT.put("770777777600671","2441");
		myHSIT.put("770773777777671","2442");
		myHSIT.put("760000377777671","2443");
		myHSIT.put("740377777777671","2444");
		myHSIT.put("477777770000000","2445");
		myHSIT.put("060000777777671","2446");
		myHSIT.put("777777767760760","2447");
		myHSIT.put("774337607760700","2448");
		myHSIT.put("747777600000000","2449");
		myHSIT.put("777777774412071","2450");
		myHSIT.put("577777777413711","2451");
		myHSIT.put("477774377413071","2452");
		myHSIT.put("777776775413700","2453");
		myHSIT.put("414367777413771","2454");
		myHSIT.put("777716777760601","2455");
		myHSIT.put("646573777777600","2456");
		myHSIT.put("407777777413071","2457");
		myHSIT.put("776036777760771","2458");
		myHSIT.put("476000377413071","2459");
		myHSIT.put("777774777760601","2460");
		myHSIT.put("500076000000000","2461");
		myHSIT.put("613776777760601","2462");
		myHSIT.put("777037777413731","2463");
		myHSIT.put("700036777760730","2464");
		myHSIT.put("477777757413771","2465");
		myHSIT.put("000036777760771","2466");
		myHSIT.put("477777775413760","2467");
		myHSIT.put("777777757760711","2468");
		myHSIT.put("777773676760730","2469");
		myHSIT.put("477777077413771","2470");
		myHSIT.put("600000177760711","2471");
		myHSIT.put("037777777000371","2472");
		myHSIT.put("037775770060771","2473");
		myHSIT.put("777740777760531","2474");
		myHSIT.put("676753777677600","2475");
		myHSIT.put("700076777760600","2476");
		myHSIT.put("617777774020771","2477");
		myHSIT.put("776177730000000","2478");
		myHSIT.put("700036776060771","2479");
		myHSIT.put("777767777413600","2480");
		myHSIT.put("776267777760771","2481");
		myHSIT.put("776377777413731","2482");
		myHSIT.put("777777606060770","2483");
		myHSIT.put("737377677760771","2484");
		myHSIT.put("603777740003171","2485");
		myHSIT.put("701577600023171","2486");
		myHSIT.put("777777606060751","2487");
		myHSIT.put("741777606060771","2488");
		myHSIT.put("777707606060771","2489");
		myHSIT.put("776007606060731","2490");
		myHSIT.put("777777204060731","2491");
		myHSIT.put("777677606060771","2492");
		myHSIT.put("777773606060731","2493");
		myHSIT.put("377737606060771","2494");
		myHSIT.put("777777606000771","2495");
		myHSIT.put("777367777760771","2496");
		myHSIT.put("677677607760771","2497");
		myHSIT.put("777760077620771","2498");
		myHSIT.put("777777617760771","2499");
		myHSIT.put("776377770000771","2500");
		myHSIT.put("777777774220771","2501");
		myHSIT.put("777777407560771","2502");
		myHSIT.put("777777777420611","2503");
		myHSIT.put("777741077760771","2504");
		myHSIT.put("777677607560771","2505");
		myHSIT.put("677757607760771","2506");
		myHSIT.put("777777400600000","2507");
		myHSIT.put("777577600000000","2508");
		myHSIT.put("777777777750371","2509");
		myHSIT.put("774737777760731","2510");
		myHSIT.put("777736770000000","2511");
		myHSIT.put("657757607760771","2512");
		myHSIT.put("777767777760471","2513");
		myHSIT.put("774377607760700","2514");
		myHSIT.put("777737607760400","2515");
		myHSIT.put("756377777760771","2516");
		myHSIT.put("777600167760771","2517");
		myHSIT.put("763763607560771","2518");
		myHSIT.put("700000007560771","2519");
		myHSIT.put("771600000000011","2520");
		myHSIT.put("737777607560771","2521");
		myHSIT.put("677777607760740","2522");
		myHSIT.put("777740017160771","2523");
		myHSIT.put("757777605560771","2524");
		myHSIT.put("677737607620771","2525");
		myHSIT.put("007777606000031","2526");
		myHSIT.put("776177607660771","2527");
		myHSIT.put("374377607760700","2528");
		myHSIT.put("777777607740371","2529");
		myHSIT.put("777700017760771","2530");
		myHSIT.put("677777777760601","2531");
		myHSIT.put("777777634020771","2532");
		myHSIT.put("777760007560771","2533");
		myHSIT.put("777377607700771","2534");
		myHSIT.put("777777207600331","2535");
		myHSIT.put("676777607760771","2536");
		myHSIT.put("777617777720771","2537");
		myHSIT.put("777741003760771","2538");
		myHSIT.put("673777607760771","2539");
		myHSIT.put("777700000760771","2540");
		myHSIT.put("777740000760771","2541");
		myHSIT.put("777400167760771","2542");
		myHSIT.put("773601757013371","2543");
		myHSIT.put("677737607760371","2544");
		myHSIT.put("703660000000331","2545");
		myHSIT.put("777600073760771","2546");
		myHSIT.put("777740000660771","2547");
		myHSIT.put("777777607600331","2548");
		myHSIT.put("777677207760771","2549");
		myHSIT.put("777737743760771","2550");
		myHSIT.put("777777607760701","2551");
		myHSIT.put("677637607760771","2552");
		myHSIT.put("776776777760601","2553");
		myHSIT.put("577777207760771","2554");
		myHSIT.put("776777777760601","2555");
		myHSIT.put("777776607760601","2556");
		myHSIT.put("777777777540771","2557");
		myHSIT.put("000000037760771","2558");
		myHSIT.put("777737777460771","2559");
		myHSIT.put("777737400600171","2560");
		myHSIT.put("777777000020771","2561");
		myHSIT.put("777377600000000","2562");
		myHSIT.put("777777607700731","2563");
		myHSIT.put("377777600000000","2564");
		myHSIT.put("377776777760601","2565");
		myHSIT.put("777777607160771","2566");
		myHSIT.put("777771374020771","2567");
		myHSIT.put("737776607760771","2568");
		myHSIT.put("747777667560771","2569");
		myHSIT.put("777777337760771","2570");
		myHSIT.put("776340000000171","2571");
		myHSIT.put("667777607760771","2572");
		myHSIT.put("777777600000731","2573");
		myHSIT.put("777776777760600","2574");
		myHSIT.put("774777747420731","2575");
		myHSIT.put("777723607760771","2576");
		myHSIT.put("777477777760771","2577");
		myHSIT.put("777737777760740","2578");
		myHSIT.put("776000000760771","2579");
		myHSIT.put("777740016360771","2580");
		myHSIT.put("770023206371071","2581");
		myHSIT.put("737777377760731","2582");
		myHSIT.put("477777277413571","2583");
		myHSIT.put("777770677760771","2584");
		myHSIT.put("703763740003171","2585");
		myHSIT.put("077577717003351","2586");
		myHSIT.put("777737777777731","2587");
		myHSIT.put("676023477413771","2588");
		myHSIT.put("477677376003741","2589");
		myHSIT.put("677777470000071","2590");
		myHSIT.put("703767740000000","2591");
		myHSIT.put("677377477413771","2592");
		myHSIT.put("400037777413761","2593");
		myHSIT.put("767777777413731","2594");
		myHSIT.put("757777606760631","2595");
		myHSIT.put("777737777760760","2596");
		myHSIT.put("777777777730771","2597");
		myHSIT.put("777777557770771","2598");
		myHSIT.put("677777744030771","2599");
		myHSIT.put("776177607770771","2600");
		myHSIT.put("577777777770771","2601");
		myHSIT.put("777777607770771","2602");
		myHSIT.put("776177607770731","2603");
		myHSIT.put("677777607770771","2604");
		myHSIT.put("777777557010771","2605");
		myHSIT.put("776017607770771","2606");
		myHSIT.put("776177400010171","2607");
		myHSIT.put("477777777730771","2608");
		myHSIT.put("777777607770770","2609");
		myHSIT.put("000000000003770","2610");
		myHSIT.put("757777777030771","2611");
		myHSIT.put("757777774030771","2612");
		myHSIT.put("777777777400001","2613");
		myHSIT.put("777367777720671","2614");
		myHSIT.put("777367777720771","2615");
		myHSIT.put("776167777760771","2616");
		myHSIT.put("777367577760771","2617");
		myHSIT.put("777367777760671","2618");
		myHSIT.put("777367777720661","2619");
		myHSIT.put("777367607760661","2620");
		myHSIT.put("703775740003771","2621");
		myHSIT.put("777377777760671","2622");
		myHSIT.put("700000004020771","2623");
		myHSIT.put("400000007760771","2624");
		myHSIT.put("776163400000171","2625");
		myHSIT.put("776177607700171","2626");
		myHSIT.put("773767777720771","2627");
		myHSIT.put("777377777520771","2628");
		myHSIT.put("774377777420771","2629");
		myHSIT.put("676773477777600","2630");
		myHSIT.put("676773675777600","2631");
		myHSIT.put("777777607420771","2632");
		myHSIT.put("000007777760771","2633");
		myHSIT.put("777577777620771","2634");
		myHSIT.put("777767777000271","2635");
		myHSIT.put("777763777777600","2636");
		myHSIT.put("776737777420771","2637");
		myHSIT.put("777737607400071","2638");
		myHSIT.put("776377737560731","2639");
		myHSIT.put("677777777720731","2640");
		myHSIT.put("776376037760751","2641");
		myHSIT.put("777777774020611","2642");
		myHSIT.put("760177777720771","2643");
		myHSIT.put("177777777720771","2644");
		myHSIT.put("777777662020771","2645");
		myHSIT.put("766177607760771","2646");
		myHSIT.put("777077607760771","2647");
		myHSIT.put("757767607760671","2648");
		myHSIT.put("777777607400371","2649");
		myHSIT.put("743737607760731","2650");
		myHSIT.put("777737604760771","2651");
		myHSIT.put("777777604760771","2652");
		myHSIT.put("776377707760771","2653");
		myHSIT.put("777637607400031","2654");
		myHSIT.put("775777760060771","2655");
		myHSIT.put("477377777413071","2656");
		myHSIT.put("703777700002771","2657");
		myHSIT.put("777700000001331","2658");
		myHSIT.put("740027606760771","2659");
		myHSIT.put("777777777600131","2660");
		myHSIT.put("777677577413771","2661");
		myHSIT.put("776773777777600","2662");
		myHSIT.put("777076777760771","2663");
		myHSIT.put("776377770000000","2664");
		myHSIT.put("777777777000771","2665");
		myHSIT.put("367777770020731","2666");
		myHSIT.put("737777757413371","2667");
		myHSIT.put("777777377760770","2668");
		myHSIT.put("000000000000000","2669");
		myHSIT.put("703607740003171","2670");
		myHSIT.put("467777377413771","2671");
		myHSIT.put("777777377413771","2672");
		myHSIT.put("777677777760731","2673");
		myHSIT.put("777757777760731","2674");
		myHSIT.put("477777777413171","2675");
		myHSIT.put("777774377413671","2676");
		myHSIT.put("403637757413771","2677");
		myHSIT.put("777777776023771","2678");
		myHSIT.put("477775777760771","2679");
		myHSIT.put("703740000003771","2680");
		myHSIT.put("477777777413761","2681");
		myHSIT.put("703761000000331","2682");
		myHSIT.put("703777600003171","2683");
		myHSIT.put("703770000000001","2684");
		myHSIT.put("703777000003171","2685");
		myHSIT.put("703767740003471","2686");
		myHSIT.put("577777757473771","2687");
		myHSIT.put("703777742003571","2688");
		myHSIT.put("777775000000171","2689");
		myHSIT.put("703777740000071","2690");
		myHSIT.put("037737777760771","2691");
		myHSIT.put("777737707760771","2692");
		myHSIT.put("703763740003771","2693");
		myHSIT.put("303777740003171","2694");
		myHSIT.put("703767740003571","2695");
		myHSIT.put("703777760003771","2696");
		myHSIT.put("775777777763771","2697");
		myHSIT.put("777777743763771","2698");
		myHSIT.put("777777747760731","2699");
		myHSIT.put("477777770020771","2700");
		myHSIT.put("777777277413771","2701");
		myHSIT.put("676143777777600","2702");
		myHSIT.put("400027777413471","2703");
		myHSIT.put("677777457413771","2704");
		myHSIT.put("777403607760771","2705");
		myHSIT.put("701767777700011","2706");
		myHSIT.put("701767740003671","2707");
		myHSIT.put("701377740003771","2708");
		myHSIT.put("773777607560171","2709");
		myHSIT.put("577777777411731","2710");
		myHSIT.put("477777740173071","2711");
		myHSIT.put("037637777770171","2712");
		myHSIT.put("700277740007001","2713");
		myHSIT.put("703777774000771","2714");
		myHSIT.put("477777760177071","2715");
		myHSIT.put("777777637560771","2716");
		myHSIT.put("573377777420771","2717");
		myHSIT.put("777477777460731","2718");
		myHSIT.put("377777047341771","2719");
		myHSIT.put("677777477413071","2720");
		myHSIT.put("777777700000171","2721");
		myHSIT.put("777737774413731","2722");
		myHSIT.put("577777777760371","2723");
		myHSIT.put("774737777420771","2724");
		myHSIT.put("777777606160771","2725");
		myHSIT.put("703777600000771","2726");
		myHSIT.put("414363777777600","2727");
		myHSIT.put("577767777420771","2728");
		myHSIT.put("614363777777600","2729");
		myHSIT.put("637774777740130","2730");
		myHSIT.put("703777740001071","2731");
		myHSIT.put("777730607760771","2732");
		myHSIT.put("777777760000371","2733");
		myHSIT.put("777777601560751","2734");
		myHSIT.put("777577777720771","2735");
		myHSIT.put("777777777760741","2736");
		myHSIT.put("777400017660731","2737");
		myHSIT.put("700036377760771","2738");
		myHSIT.put("767777607560771","2739");
		myHSIT.put("777377757760771","2740");
		myHSIT.put("757737777760771","2741");
		myHSIT.put("776377400000171","2742");
		myHSIT.put("777777077560771","2743");
		myHSIT.put("777777777700071","2744");
		myHSIT.put("577777607760751","2745");
		myHSIT.put("700001774020771","2746");
		myHSIT.put("763777777720771","2747");
		myHSIT.put("774077600000031","2748");
		myHSIT.put("600000007720771","2749");
		myHSIT.put("377777407760771","2750");
		myHSIT.put("776007607760771","2751");
		myHSIT.put("776177600000171","2752");
		myHSIT.put("677737607760571","2753");
		myHSIT.put("767677777760771","2754");
		myHSIT.put("677777777760671","2755");
		myHSIT.put("703777700000171","2756");
		myHSIT.put("777761057770771","2757");
		myHSIT.put("700377700001771","2758");
		myHSIT.put("703777600000171","2759");
		myHSIT.put("777777607570771","2760");
		myHSIT.put("377777777413700","2761");
		myHSIT.put("670777777477671","2762");
		myHSIT.put("703777707770771","2763");
		myHSIT.put("776167607760731","2764");
		myHSIT.put("770777677777671","2765");
		myHSIT.put("703777300003771","2766");
		myHSIT.put("702177740003171","2767");
		myHSIT.put("777774000000171","2768");
		myHSIT.put("777777757013371","2769");
		myHSIT.put("777777750013371","2770");
		myHSIT.put("577777001413771","2771");
		myHSIT.put("777777755413771","2772");
		myHSIT.put("577775757413771","2773");
		myHSIT.put("734177777413771","2774");
		myHSIT.put("777777757767771","2775");
		myHSIT.put("777777750003771","2776");
		myHSIT.put("777377757413771","2777");
		myHSIT.put("777377757413371","2778");
		myHSIT.put("777777777412071","2779");
		myHSIT.put("777776777740000","2780");
		myHSIT.put("777777777767771","2781");
		myHSIT.put("777777600003271","2782");
		myHSIT.put("703777740003751","2783");
		myHSIT.put("757777607460731","2784");
		myHSIT.put("777347777760371","2785");
		myHSIT.put("777777775760770","2786");
		myHSIT.put("627777607360771","2787");
		myHSIT.put("673737607760771","2788");
		myHSIT.put("737737607160771","2789");
		myHSIT.put("677737607760760","2790");
		myHSIT.put("777777770760771","2791");
		myHSIT.put("103777740003771","2792");
		myHSIT.put("400000757760771","2793");
		myHSIT.put("776357777760731","2794");
		myHSIT.put("777737774760771","2795");
		myHSIT.put("777737777740131","2796");
		myHSIT.put("577777770000000","2797");
		myHSIT.put("777776777620601","2798");
		myHSIT.put("775777777773771","2799");
		myHSIT.put("777377777770771","2800");
		myHSIT.put("715357777760771","2801");
		myHSIT.put("477777777413051","2802");
		myHSIT.put("577777377413771","2803");
		myHSIT.put("770000077757031","2804");
		myHSIT.put("774137777757771","2805");
		myHSIT.put("774177777757771","2806");
		myHSIT.put("776367777760731","2807");
		myHSIT.put("703767740003461","2808");
		myHSIT.put("777377777773771","2809");
		myHSIT.put("777777777677771","2810");
		myHSIT.put("774077606000031","2811");
		myHSIT.put("777770004000731","2812");
		myHSIT.put("676773776277600","2813");
		myHSIT.put("770777777377440","2814");
		myHSIT.put("770777404177671","2815");
		myHSIT.put("400000000000031","2816");
		myHSIT.put("770777577777671","2817");
		myHSIT.put("761777777777671","2818");
		myHSIT.put("770777703017671","2819");
		myHSIT.put("777777767760731","2820");
		myHSIT.put("676777777617600","2821");
		myHSIT.put("777777777641770","2822");
		myHSIT.put("774037777777071","2823");
		myHSIT.put("774077604000031","2824");
		myHSIT.put("777777743760171","2825");
		myHSIT.put("700000000020071","2826");
		myHSIT.put("774077607761031","2827");
		myHSIT.put("700047777740600","2828");
		myHSIT.put("774076007777071","2829");
		myHSIT.put("777777742760771","2830");
		myHSIT.put("761777743760771","2831");
		myHSIT.put("577777743760771","2832");
		myHSIT.put("777777747777771","2833");
		myHSIT.put("377777743760771","2834");
		myHSIT.put("770003602000031","2835");
		myHSIT.put("000001777760771","2836");
		myHSIT.put("770003607777001","2837");
		myHSIT.put("676773776276600","2838");
		myHSIT.put("676701777677600","2839");
		myHSIT.put("646773777677600","2840");
		myHSIT.put("676773776261600","2841");
		myHSIT.put("676773777677200","2842");
		myHSIT.put("674373776277600","2843");
		myHSIT.put("774077602000000","2844");
		myHSIT.put("773777607560771","2845");
		myHSIT.put("777777667560771","2846");
		myHSIT.put("674000003760771","2847");
		myHSIT.put("674000002413771","2848");
		myHSIT.put("000001777760571","2849");
		myHSIT.put("777760017760731","2850");
		myHSIT.put("757777000000700","2851");
		myHSIT.put("677777477413371","2852");
		myHSIT.put("077777606000031","2853");
		myHSIT.put("777777607020771","2854");
		myHSIT.put("777777743760571","2855");
		myHSIT.put("777377777763671","2856");
		myHSIT.put("777770000000000","2857");
		myHSIT.put("777776077020771","2858");
		myHSIT.put("777177600000000","2859");
		myHSIT.put("757777607560771","2860");
		myHSIT.put("777765757413371","2861");
		myHSIT.put("774777777413771","2862");
		myHSIT.put("657777777413071","2863");
		myHSIT.put("700777747413671","2864");
		myHSIT.put("777777607460771","2865");
		myHSIT.put("777717774020771","2866");
		myHSIT.put("777777775760731","2867");
		myHSIT.put("777777363760771","2868");
		myHSIT.put("377777364020771","2869");
		myHSIT.put("377777767760771","2870");
		myHSIT.put("777037777760611","2871");
		myHSIT.put("777777557760700","2872");
		myHSIT.put("676773777700200","2873");
		myHSIT.put("264073737777600","2874");
		myHSIT.put("774067607761031","2875");
		myHSIT.put("774067606767031","2876");
		myHSIT.put("574063670007071","2877");
		myHSIT.put("074000430000600","2878");
		myHSIT.put("074000437777600","2879");
		myHSIT.put("772763000001671","2880");
		myHSIT.put("772763400001671","2881");
		myHSIT.put("677677606000171","2882");
		myHSIT.put("774000017760771","2883");
		myHSIT.put("761777607760771","2884");
		myHSIT.put("777777477720771","2885");
		myHSIT.put("777607777760771","2886");
		myHSIT.put("477767377413771","2887");
		myHSIT.put("477767377613771","2888");
		myHSIT.put("476777777413771","2889");
		myHSIT.put("777777761760771","2890");
		myHSIT.put("777773000000331","2891");
		myHSIT.put("737400004020771","2892");
		myHSIT.put("777777742160731","2893");
		myHSIT.put("777777607760721","2894");
		myHSIT.put("773777777720631","2895");
		myHSIT.put("777723607760571","2896");
		myHSIT.put("723777607760771","2897");
		myHSIT.put("774037600760710","2898");
		myHSIT.put("477377707760731","2899");
		myHSIT.put("777777763760771","2900");
		myHSIT.put("770000007760771","2901");
		myHSIT.put("757775777720771","2902");
		myHSIT.put("777403777720771","2903");
		myHSIT.put("177777607560771","2904");
		myHSIT.put("377717777760771","2905");
		myHSIT.put("777740370000731","2906");
		myHSIT.put("477777777020771","2907");
		myHSIT.put("777777757413671","2908");
		myHSIT.put("477777777413061","2909");
		myHSIT.put("777777777713771","2910");
		myHSIT.put("777737777700771","2911");
		myHSIT.put("777777350060771","2912");
		myHSIT.put("741777777720771","2913");
		myHSIT.put("677776777760771","2914");
		myHSIT.put("757737777413731","2915");
		myHSIT.put("776377777360771","2916");
		myHSIT.put("677777474003771","2917");
		myHSIT.put("000037777413771","2918");
		myHSIT.put("400037777413731","2919");
		myHSIT.put("777767777413771","2920");
		myHSIT.put("400027777413771","2921");
		myHSIT.put("400037777403771","2922");
		myHSIT.put("400367777740771","2923");
		myHSIT.put("503367400001401","2924");
		myHSIT.put("503367400001471","2925");
		myHSIT.put("777367606060771","2926");
		myHSIT.put("777777606063771","2927");
		myHSIT.put("503367400001771","2928");
		myHSIT.put("777700207760771","2929");
		myHSIT.put("777741777720771","2930");
		myHSIT.put("703607740003771","2931");
		myHSIT.put("777000177760771","2932");
		myHSIT.put("677737606360771","2933");
		myHSIT.put("677707477413771","2934");
		myHSIT.put("677707477412771","2935");
		myHSIT.put("777737777000371","2936");
		myHSIT.put("000177760000771","2937");
		myHSIT.put("600000360000011","2938");
		myHSIT.put("776377703740731","2939");
		myHSIT.put("777777607760471","2940");
		myHSIT.put("074377607760700","2941");
		myHSIT.put("677777600760771","2942");
		myHSIT.put("777777774020531","2943");
		myHSIT.put("677777617760771","2944");
		myHSIT.put("776000007760771","2945");
		myHSIT.put("776137400000171","2946");
		myHSIT.put("770000777760571","2947");
		myHSIT.put("555767637600771","2948");
		myHSIT.put("776173400000171","2949");
		myHSIT.put("703700040003771","2950");
		myHSIT.put("777737557560771","2951");
		myHSIT.put("777747777760771","2952");
		myHSIT.put("774037607760771","2953");
		myHSIT.put("677777600060771","2954");
		myHSIT.put("777777740000171","2955");
		myHSIT.put("775760007760771","2956");
		myHSIT.put("777777770000731","2957");
		myHSIT.put("476177400000171","2958");
		myHSIT.put("704023347760471","2959");
		myHSIT.put("777736776000071","2960");
		myHSIT.put("400000071760571","2961");
		myHSIT.put("777777757770771","2962");
		myHSIT.put("703777740013171","2963");
		myHSIT.put("777777607730771","2964");
		myHSIT.put("777777404770771","2965");
		myHSIT.put("703737740013171","2966");
		myHSIT.put("777777770030731","2967");
		myHSIT.put("777777770033771","2968");
		myHSIT.put("777777777771771","2969");
		myHSIT.put("777777343760771","2970");
		myHSIT.put("703777740013771","2971");
		myHSIT.put("777775007760731","2972");
		myHSIT.put("703701740003171","2973");
		myHSIT.put("677777777763771","2974");
		myHSIT.put("776763777677600","2975");
		myHSIT.put("776743777677600","2976");
		myHSIT.put("777777377360771","2977");
		myHSIT.put("777777777060771","2978");
		myHSIT.put("000000000000171","2979");
		myHSIT.put("777767777520771","2980");
		myHSIT.put("776001743760771","2981");
		myHSIT.put("777777737740071","2982");
		myHSIT.put("157756777760731","2983");
		myHSIT.put("770003607777071","2984");
		myHSIT.put("777777740413371","2985");
		myHSIT.put("677777771413731","2986");
		myHSIT.put("777774007413771","2987");
		myHSIT.put("642573777777600","2988");
		myHSIT.put("777377700003371","2989");
		myHSIT.put("477760777413071","2990");
		myHSIT.put("677777750403371","2991");
		myHSIT.put("777777747763771","2992");
		myHSIT.put("700377740001740","2993");
		myHSIT.put("576377777420771","2994");
		myHSIT.put("477777661760771","2995");
		myHSIT.put("720021400000171","2996");
		myHSIT.put("753777777413731","2997");
		myHSIT.put("777763777413731","2998");
		myHSIT.put("777761007760731","2999");
		myHSIT.put("757777407560731","3000");
		myHSIT.put("777774077020771","3001");
		myHSIT.put("001777707560771","3002");
		myHSIT.put("001777607560771","3003");
		myHSIT.put("776377777000000","3004");
		myHSIT.put("603777677560771","3005");
		myHSIT.put("577777607760731","3006");
		myHSIT.put("776377770360771","3007");
		myHSIT.put("776777774020771","3008");
		myHSIT.put("777773777760171","3009");
		myHSIT.put("776377777740731","3010");
		myHSIT.put("743774077560731","3011");
		myHSIT.put("777774070000171","3012");
		myHSIT.put("377777377760771","3013");
		myHSIT.put("677777607700771","3014");
		myHSIT.put("077776777760701","3015");
		myHSIT.put("600000000560771","3016");
		myHSIT.put("777777777120771","3017");
		myHSIT.put("774006777760771","3018");
		myHSIT.put("577737607760771","3019");
		myHSIT.put("777777607360771","3020");
		myHSIT.put("677777747560771","3021");
		myHSIT.put("700077774020731","3022");
		myHSIT.put("716177607760771","3023");
		myHSIT.put("677737707760571","3024");
		myHSIT.put("676773777617600","3025");
		myHSIT.put("703777700000071","3026");
		myHSIT.put("003777747777771","3027");
		myHSIT.put("703777740005471","3028");
		myHSIT.put("777777770014351","3029");
		myHSIT.put("760001400004071","3030");
		myHSIT.put("703777740005071","3031");
		myHSIT.put("757777757370371","3032");
		myHSIT.put("703737740005471","3033");
		myHSIT.put("777600037760771","3034");
		myHSIT.put("777763607560771","3035");
		myHSIT.put("777777777760710","3036");
		myHSIT.put("777777777560031","3037");
		myHSIT.put("617777777760601","3038");
		myHSIT.put("777761377760771","3039");
		myHSIT.put("777367607760671","3040");
		myHSIT.put("677777607760700","3041");
		myHSIT.put("777777674020731","3042");
		myHSIT.put("743777774020771","3043");
		myHSIT.put("777777607700071","3044");
		myHSIT.put("677737600000000","3045");
		myHSIT.put("607677607560771","3046");
		myHSIT.put("777357777720671","3047");
		myHSIT.put("777777606020731","3048");
		myHSIT.put("777767777413631","3049");
		myHSIT.put("720001400000171","3050");
		myHSIT.put("777771000000371","3051");
		myHSIT.put("777767347760471","3052");
		myHSIT.put("700765747413771","3053");
		myHSIT.put("777675757413371","3054");
		myHSIT.put("700076777763771","3055");
		myHSIT.put("777777777560731","3056");
		myHSIT.put("700767747413771","3057");
		myHSIT.put("777765606060731","3058");
		myHSIT.put("774167607760771","3059");
		myHSIT.put("600000606060731","3060");
		myHSIT.put("700076777760661","3061");
		myHSIT.put("776167407760771","3062");
		myHSIT.put("757777607413731","3063");
		myHSIT.put("777777377410071","3064");
		myHSIT.put("757777737413731","3065");
		myHSIT.put("777736607760771","3066");
		myHSIT.put("000000007760751","3067");
		myHSIT.put("603777743760771","3068");
		myHSIT.put("774077607777061","3069");
		myHSIT.put("777777767720731","3070");
		myHSIT.put("774077607777041","3071");
		myHSIT.put("774137600060771","3072");
		myHSIT.put("777437777760700","3073");
		myHSIT.put("773417760060771","3074");
		myHSIT.put("777613405760471","3075");
		myHSIT.put("777777742160771","3076");
		myHSIT.put("500377777720771","3077");
		myHSIT.put("774037777720771","3078");
		myHSIT.put("476363636756200","3079");
		myHSIT.put("777377777763771","3080");
		myHSIT.put("766073777760771","3081");
		myHSIT.put("776071777760771","3082");
		myHSIT.put("376177407760771","3083");
		myHSIT.put("376177607760371","3084");
		myHSIT.put("775777607760771","3085");
		myHSIT.put("741775047560771","3086");
		myHSIT.put("777701003560771","3087");
		myHSIT.put("577777757760771","3088");
		myHSIT.put("337777777760771","3089");
		myHSIT.put("677773777777600","3090");
		myHSIT.put("777777777740031","3091");
		myHSIT.put("777737607420771","3092");
		myHSIT.put("777000001760771","3093");
		myHSIT.put("777773037760771","3094");
		myHSIT.put("077777777760731","3095");
		myHSIT.put("777767777600771","3096");
		myHSIT.put("777640007760731","3097");
		myHSIT.put("717766777760771","3098");
		myHSIT.put("777777607700171","3099");
		myHSIT.put("777477777760711","3100");
		myHSIT.put("756177607760771","3101");
		myHSIT.put("577777607560771","3102");
		myHSIT.put("777777677760671","3103");
		myHSIT.put("760377777743671","3104");
		myHSIT.put("776173607760771","3105");
		myHSIT.put("777377637760771","3106");
		myHSIT.put("677737607720771","3107");
		myHSIT.put("777777770000751","3108");
		myHSIT.put("577777777420731","3109");
		myHSIT.put("777600000760771","3110");
		myHSIT.put("703777700000000","3111");
		myHSIT.put("777637417560771","3112");
		myHSIT.put("777767477720771","3113");
		myHSIT.put("377767607760731","3114");
		myHSIT.put("777777607740031","3115");
		myHSIT.put("774737600160671","3116");
		myHSIT.put("777777777400000","3117");
		myHSIT.put("760377777777671","3118");
		myHSIT.put("617737607760771","3119");
		myHSIT.put("737777700000611","3120");
		myHSIT.put("774077777760771","3121");
		myHSIT.put("776377773740731","3122");
		myHSIT.put("776117400000171","3123");
		myHSIT.put("776147607760771","3124");
		myHSIT.put("777737606560771","3125");
		myHSIT.put("677777604760771","3126");
		myHSIT.put("000000017760771","3127");
		myHSIT.put("777757607560771","3128");
		myHSIT.put("477737777760731","3129");
		myHSIT.put("770777777377671","3130");
		myHSIT.put("777777777000011","3131");
		myHSIT.put("777767677773671","3132");
		myHSIT.put("677777777420731","3133");
		myHSIT.put("777737377720771","3134");
		myHSIT.put("777777470000000","3135");
		myHSIT.put("760000001760771","3136");
		myHSIT.put("776737777760771","3137");
		myHSIT.put("777776777700731","3138");
		myHSIT.put("577737777760771","3139");
		myHSIT.put("776737777760731","3140");
		myHSIT.put("177000377760771","3141");
		myHSIT.put("777347777760661","3142");
		myHSIT.put("777767777760700","3143");
		myHSIT.put("777747607740031","3144");
		myHSIT.put("777347607760661","3145");
		myHSIT.put("777337777760731","3146");
		myHSIT.put("677700000160771","3147");
		myHSIT.put("777767777760600","3148");
		myHSIT.put("177767777720771","3149");
		myHSIT.put("677737607760611","3150");
		myHSIT.put("672600000000671","3151");
		myHSIT.put("770000777700000","3152");
		myHSIT.put("777777717760571","3153");
		myHSIT.put("776177604360771","3154");
		myHSIT.put("777777774720771","3155");
		myHSIT.put("777776774760731","3156");
		myHSIT.put("577637002060771","3157");
		myHSIT.put("777777001413771","3158");
		myHSIT.put("737774077560771","3159");
		myHSIT.put("577002377760771","3160");
		myHSIT.put("703741000000760","3161");
		myHSIT.put("170023607760771","3162");
		myHSIT.put("577777603720771","3163");
		myHSIT.put("777777607760600","3164");
		myHSIT.put("767777757413771","3165");
		myHSIT.put("400000757560771","3166");
		myHSIT.put("777777747613771","3167");
		myHSIT.put("737777777720731","3168");
		myHSIT.put("460060000000771","3169");
		myHSIT.put("676773776777600","3170");
		myHSIT.put("777777417720771","3171");
		myHSIT.put("757777777760711","3172");
		myHSIT.put("770000777360771","3173");
		myHSIT.put("703775740000171","3174");
		myHSIT.put("776020777760771","3175");
		myHSIT.put("777437777720771","3176");
		myHSIT.put("770000207760771","3177");
		myHSIT.put("000000007760471","3178");
		myHSIT.put("757777777720731","3179");
		myHSIT.put("470000777760771","3180");
		myHSIT.put("777777777520731","3181");
		myHSIT.put("777776777760171","3182");
		myHSIT.put("000000077760771","3183");
		myHSIT.put("676773747777600","3184");
		myHSIT.put("761777777720771","3185");
		myHSIT.put("777777764760771","3186");
		myHSIT.put("460060000000371","3187");
		myHSIT.put("700000000000771","3188");
		myHSIT.put("777777765720771","3189");
		myHSIT.put("777777747760431","3190");
		myHSIT.put("400060000000771","3191");
		myHSIT.put("541737777760771","3192");
		myHSIT.put("777777703760771","3193");
		myHSIT.put("700737757760771","3194");
		myHSIT.put("403777477413771","3195");
		myHSIT.put("777777774403771","3196");
		myHSIT.put("777777761520771","3197");
		myHSIT.put("737767607760771","3198");
		myHSIT.put("771777777760731","3199");
		myHSIT.put("077777777763731","3200");
		myHSIT.put("777777770720700","3201");
		myHSIT.put("000000000003611","3202");
		myHSIT.put("777761000000000","3203");
		myHSIT.put("577737777760731","3204");
		myHSIT.put("677777777700771","3205");
		myHSIT.put("777777777760131","3206");
		myHSIT.put("377777777763771","3207");
		myHSIT.put("777777577763771","3208");
		myHSIT.put("577417777760771","3209");
		myHSIT.put("537777777420771","3210");
		myHSIT.put("777743777420771","3211");
		myHSIT.put("777777777740001","3212");
		myHSIT.put("777757637760431","3213");
		myHSIT.put("777777776060771","3214");
		myHSIT.put("577000777760771","3215");
		myHSIT.put("000000000002770","3216");
		myHSIT.put("400000000000011","3217");
		myHSIT.put("760000007720771","3218");
		myHSIT.put("603777600003771","3219");
		myHSIT.put("775767777760771","3220");
		myHSIT.put("777667777760771","3221");
		myHSIT.put("717777777760731","3222");
		myHSIT.put("777777757763771","3223");
		myHSIT.put("777777477763771","3224");
		myHSIT.put("077737777760771","3225");
		myHSIT.put("577717777420771","3226");
		myHSIT.put("760777777760771","3227");
		myHSIT.put("777777777660071","3228");
		myHSIT.put("777777776413731","3229");
		myHSIT.put("777740007760730","3230");
		myHSIT.put("000000000003431","3231");
		myHSIT.put("340000007760731","3232");
		myHSIT.put("700377777760731","3233");
		myHSIT.put("777777777600371","3234");
		myHSIT.put("607777760060731","3235");
		myHSIT.put("000000000002031","3236");
		myHSIT.put("777777777400011","3237");
		myHSIT.put("777777777740300","3238");
		myHSIT.put("777760000000731","3239");
		myHSIT.put("776167777760731","3240");
		myHSIT.put("000000000003600","3241");
		myHSIT.put("777777607760171","3242");
		myHSIT.put("777701000000000","3243");
		myHSIT.put("777777777360700","3244");
		myHSIT.put("774077607757071","3245");
		myHSIT.put("637777774020771","3246");
		myHSIT.put("757777377760771","3247");
		myHSIT.put("677777777413071","3248");
		myHSIT.put("777776077760771","3249");
		myHSIT.put("757777774020700","3250");
		myHSIT.put("776177777720771","3251");
		myHSIT.put("677777777730771","3252");
		myHSIT.put("777701000020771","3253");
		myHSIT.put("707777777760731","3254");
		myHSIT.put("760000074020771","3255");
		myHSIT.put("776017774020771","3256");
		myHSIT.put("777777776740171","3257");
		myHSIT.put("777771000000000","3258");
		myHSIT.put("777417777720771","3259");
		myHSIT.put("740000007720771","3260");
		myHSIT.put("000000037720771","3261");
		myHSIT.put("607777607760771","3262");
		myHSIT.put("776777374020771","3263");
		myHSIT.put("703777740003000","3264");
		myHSIT.put("777777760000001","3265");
		myHSIT.put("777777770000371","3266");
		myHSIT.put("776000000160771","3267");
		myHSIT.put("777771777720771","3268");
		myHSIT.put("603777607760771","3269");
		myHSIT.put("774777737420771","3270");
		myHSIT.put("700377740003751","3271");
		myHSIT.put("777777700000731","3272");
		myHSIT.put("775737607760771","3273");
		myHSIT.put("777777777413701","3274");
		myHSIT.put("700037777760731","3275");
		myHSIT.put("777777770000171","3276");
		myHSIT.put("777777477513771","3277");
		myHSIT.put("700076717760771","3278");
		myHSIT.put("776177607760600","3279");
		myHSIT.put("777777770000031","3280");
		myHSIT.put("777777777570771","3281");
		myHSIT.put("777776777770771","3282");
		myHSIT.put("770777777760731","3283");
		myHSIT.put("703357400001771","3284");
		myHSIT.put("777357607760771","3285");
		myHSIT.put("377357606060771","3286");
		myHSIT.put("743777777001731","3287");
		myHSIT.put("777777663760331","3288");
		myHSIT.put("007777606060771","3289");
		myHSIT.put("777777604000000","3290");
		myHSIT.put("743777777760700","3291");
		myHSIT.put("777377606060771","3292");
		myHSIT.put("377377606060771","3293");
		myHSIT.put("777737606060731","3294");
		myHSIT.put("737777606060731","3295");
		myHSIT.put("577775777760771","3296");
		myHSIT.put("777773777410331","3297");
		myHSIT.put("341577600001771","3298");
		myHSIT.put("777577606060751","3299");
		myHSIT.put("777777607060771","3300");
		myHSIT.put("700076777740371","3301");
		myHSIT.put("777577606060731","3302");
		myHSIT.put("700777777760771","3303");
		myHSIT.put("777777606060331","3304");
		myHSIT.put("700076737760771","3305");
		myHSIT.put("777377774020731","3306");
		myHSIT.put("777777774020630","3307");
		myHSIT.put("777677374020731","3308");
		myHSIT.put("777737777760571","3309");
		myHSIT.put("767777774020731","3310");
		myHSIT.put("006177607760771","3311");
		myHSIT.put("675737607760371","3312");
		myHSIT.put("000000004020631","3313");
		myHSIT.put("006737777760771","3314");
		myHSIT.put("376777737760771","3315");
		myHSIT.put("377000377760771","3316");
		myHSIT.put("477737737760771","3317");
		myHSIT.put("703377407761771","3318");
		myHSIT.put("703417740003771","3319");
		myHSIT.put("743777777520731","3320");
		myHSIT.put("757677777760771","3321");
		myHSIT.put("762737777760771","3322");
		myHSIT.put("776737377720771","3323");
		myHSIT.put("776737377760771","3324");
		myHSIT.put("776737407760771","3325");
		myHSIT.put("776737677700011","3326");
		myHSIT.put("777000377560771","3327");
		myHSIT.put("777000377760761","3328");
		myHSIT.put("777346777760601","3329");
		myHSIT.put("777737374020771","3330");
		myHSIT.put("777737740160771","3331");
		myHSIT.put("777737742412771","3332");
		myHSIT.put("777737747760760","3333");
		myHSIT.put("737777777420731","3334");
		myHSIT.put("737777747413771","3335");
		myHSIT.put("700000007177770","3336");
		myHSIT.put("777737770360771","3337");
		myHSIT.put("707777777760700","3338");
		myHSIT.put("777733777420771","3339");
		myHSIT.put("637737777420771","3340");
		myHSIT.put("777720007760771","3341");
		myHSIT.put("770000000000071","3342");
		myHSIT.put("774775003420771","3343");
		myHSIT.put("774774003420771","3344");
		myHSIT.put("777737777420600","3345");
		myHSIT.put("777767600760771","3346");
		myHSIT.put("000000005720771","3347");
		myHSIT.put("003777740003171","3348");
		myHSIT.put("777737677420731","3349");
		myHSIT.put("000002000003671","3350");
		myHSIT.put("775741003760771","3351");
		myHSIT.put("000002000003661","3352");
		myHSIT.put("437776077411171","3353");
		myHSIT.put("317777777413731","3354");
		myHSIT.put("777717777413731","3355");
		myHSIT.put("477777777411071","3356");
		myHSIT.put("473777077011771","3357");
		myHSIT.put("477777777773771","3358");
		myHSIT.put("776077777413731","3359");
		myHSIT.put("777700000001431","3360");
		myHSIT.put("777761047760771","3361");
		myHSIT.put("777000337760771","3362");
		myHSIT.put("777377777760751","3363");
		myHSIT.put("700000004147740","3364");
		myHSIT.put("703737740001171","3365");
		myHSIT.put("460777777413071","3366");
		myHSIT.put("777777777573771","3367");
		myHSIT.put("700077740003771","3368");
		myHSIT.put("777777740003771","3369");
		myHSIT.put("477777777413471","3370");
		myHSIT.put("703777740003761","3371");
		myHSIT.put("703777600000031","3372");
		myHSIT.put("702037740003771","3373");
		myHSIT.put("777763057760771","3374");
		myHSIT.put("000000007740731","3375");
		myHSIT.put("700777740003711","3376");
		myHSIT.put("700377740002771","3377");
		myHSIT.put("477760017413071","3378");
		myHSIT.put("477777776000071","3379");
		myHSIT.put("703777740002001","3380");
		myHSIT.put("703737740006371","3381");
		myHSIT.put("577747777420771","3382");
		myHSIT.put("474037777413071","3383");
		myHSIT.put("777760047760371","3384");
		myHSIT.put("777777777401731","3385");
		myHSIT.put("743747740003771","3386");
		myHSIT.put("777777707413731","3387");
		myHSIT.put("703777740003411","3388");
		myHSIT.put("477775377413071","3389");
		myHSIT.put("777377702003531","3390");
		myHSIT.put("703741000003771","3391");
		myHSIT.put("343777740003771","3392");
		myHSIT.put("713777740003771","3393");
		myHSIT.put("703777740006371","3394");
		myHSIT.put("703773747770371","3395");
		myHSIT.put("777777743760711","3396");
		myHSIT.put("706377777760771","3397");
		myHSIT.put("776377773760771","3398");
		myHSIT.put("740000000000001","3399");
		myHSIT.put("777770343760771","3400");
		myHSIT.put("567737777760771","3401");
		myHSIT.put("777770000020071","3402");
		myHSIT.put("777557773760771","3403");
		myHSIT.put("500077777760771","3404");
		myHSIT.put("503377400001771","3405");
		myHSIT.put("703377440001771","3406");
		myHSIT.put("703777740003451","3407");
		myHSIT.put("777003777760771","3408");
		myHSIT.put("700020047177771","3409");
		myHSIT.put("777777740000000","3410");
		myHSIT.put("777002377760771","3411");
		myHSIT.put("777003377760771","3412");
		myHSIT.put("477777661760371","3413");
		myHSIT.put("555147437740661","3414");
		myHSIT.put("777707437000771","3415");
		myHSIT.put("777737777000771","3416");
		myHSIT.put("000000007700171","3417");
		myHSIT.put("577767737760771","3418");
		myHSIT.put("000000007720700","3419");
		myHSIT.put("377777747760771","3420");
		myHSIT.put("741777777760771","3421");
		myHSIT.put("777777607740000","3422");
		myHSIT.put("676373777776600","3423");
		myHSIT.put("703637740002771","3424");
		myHSIT.put("702777740002771","3425");
		myHSIT.put("377777777760700","3426");
		myHSIT.put("477777777420771","3427");
		myHSIT.put("577777776420771","3428");
		myHSIT.put("503777740000771","3429");
		myHSIT.put("577734777420771","3430");
		myHSIT.put("777776775720771","3431");
		myHSIT.put("700003607760771","3432");
		myHSIT.put("776177607460771","3433");
		myHSIT.put("777760007760051","3434");
		myHSIT.put("775777777420771","3435");
		myHSIT.put("740377607760771","3436");
		myHSIT.put("777776777770351","3437");
		myHSIT.put("777777777420760","3438");
		myHSIT.put("777737776760771","3439");
		myHSIT.put("277777607760771","3440");
		myHSIT.put("777777037760731","3441");
		myHSIT.put("777737760060771","3442");
		myHSIT.put("237777777760771","3443");
		myHSIT.put("777737777760071","3444");
		myHSIT.put("047777607760771","3445");
		myHSIT.put("774137607720771","3446");
		myHSIT.put("777777177720771","3447");
		myHSIT.put("774177777720771","3448");
		myHSIT.put("774137777720771","3449");
		myHSIT.put("777777405720771","3450");
		myHSIT.put("477775374020771","3451");
		myHSIT.put("676173777777600","3452");
		myHSIT.put("602773761000200","3453");
		myHSIT.put("777737770060771","3454");
		myHSIT.put("777477677760731","3455");
		myHSIT.put("777777400760771","3456");
		myHSIT.put("675777607760771","3457");
		myHSIT.put("777773777760731","3458");
		myHSIT.put("777777700060771","3459");
		myHSIT.put("777777777400371","3460");
		myHSIT.put("757777743760731","3461");
		myHSIT.put("777740377760771","3462");
		myHSIT.put("777777177760731","3463");
		myHSIT.put("777777750000000","3464");
		myHSIT.put("777777360060771","3465");
		myHSIT.put("777777700000771","3466");
		myHSIT.put("777777777400131","3467");
		myHSIT.put("777737777760770","3468");
		myHSIT.put("776177400000071","3469");
		myHSIT.put("777741003760731","3470");
		myHSIT.put("000000001560731","3471");
		myHSIT.put("776177607760770","3472");
		myHSIT.put("740777606000011","3473");
		myHSIT.put("777767777760431","3474");
		myHSIT.put("777761777720771","3475");
		myHSIT.put("770777707777631","3476");
		myHSIT.put("672773777377600","3477");
		myHSIT.put("676773677747600","3478");
		myHSIT.put("776321377760771","3479");
		myHSIT.put("774000377760771","3480");
		myHSIT.put("377703777760671","3481");
		myHSIT.put("776377767760771","3482");
		myHSIT.put("477777776013071","3483");
		myHSIT.put("737777757413771","3484");
		myHSIT.put("000000177720631","3485");
		myHSIT.put("777741377760771","3486");
		myHSIT.put("777777777600731","3487");
		myHSIT.put("757377774020771","3488");
		myHSIT.put("777777756760771","3489");
		myHSIT.put("777377707760771","3490");
		myHSIT.put("776177607760671","3491");
		myHSIT.put("776303777760771","3492");
		myHSIT.put("777777660000371","3493");
		myHSIT.put("477777776413771","3494");
		myHSIT.put("777737777740771","3495");
		myHSIT.put("777777777700731","3496");
		myHSIT.put("703767400001771","3497");
		myHSIT.put("770777707776671","3498");
		myHSIT.put("777777775760700","3499");
		myHSIT.put("003777747762771","3500");
		myHSIT.put("637774760760730","3501");
		myHSIT.put("003777740002771","3502");
		myHSIT.put("637714377760730","3503");
		myHSIT.put("777637607560771","3504");
		myHSIT.put("777777577660771","3505");
		myHSIT.put("777737607000771","3506");
		myHSIT.put("636733777777600","3507");
		myHSIT.put("616763777777600","3508");
		myHSIT.put("436773676777600","3509");
		myHSIT.put("476773677777200","3510");
		myHSIT.put("676773657777600","3511");
		myHSIT.put("606773677777600","3512");
		myHSIT.put("664073777747600","3513");
		myHSIT.put("636573777777600","3514");
		myHSIT.put("676773677077600","3515");
		myHSIT.put("676773664177600","3516");
		myHSIT.put("676673674177600","3517");
		myHSIT.put("676772277777600","3518");
		myHSIT.put("616773777077600","3519");
		myHSIT.put("636733777774600","3520");
		myHSIT.put("664073677777600","3521");
		myHSIT.put("676400657777600","3522");
		myHSIT.put("676773617743600","3523");
		myHSIT.put("676772037777600","3524");
		myHSIT.put("676772077777600","3525");
		myHSIT.put("676433777777400","3526");
		myHSIT.put("076773776077600","3527");
		myHSIT.put("676573777737600","3528");
		myHSIT.put("672773777777600","3529");
		myHSIT.put("676733677777200","3530");
		myHSIT.put("676643677777600","3531");
		myHSIT.put("276773677777600","3532");
		myHSIT.put("676573677777600","3533");
		myHSIT.put("676613677777600","3534");
		myHSIT.put("676761677777600","3535");
		myHSIT.put("674033677777200","3536");
		myHSIT.put("672773677777600","3537");
		myHSIT.put("676763675777600","3538");
		myHSIT.put("676773677677600","3539");
		myHSIT.put("676740000037600","3540");
		myHSIT.put("676773677773600","3541");
		myHSIT.put("676770000000000","3542");
		myHSIT.put("436763676777600","3543");
		myHSIT.put("400000000000000","3544");
		myHSIT.put("616773677777200","3545");
		myHSIT.put("076773677777600","3546");
		myHSIT.put("476773677777600","3547");
		myHSIT.put("676773677767600","3548");
		myHSIT.put("676773677577600","3549");
		myHSIT.put("676573604377600","3550");
		myHSIT.put("676713677777600","3551");
		myHSIT.put("400773777777600","3552");
		myHSIT.put("666763677777600","3553");
		myHSIT.put("676573077777600","3554");
		myHSIT.put("614773777777600","3555");
		myHSIT.put("436773677777600","3556");
		myHSIT.put("006773677777600","3557");
		myHSIT.put("676741377757600","3558");
		myHSIT.put("400000077776600","3559");
		myHSIT.put("636773676377600","3560");
		myHSIT.put("676762077777600","3561");
		myHSIT.put("602773777777400","3562");
		myHSIT.put("676033677777600","3563");
		myHSIT.put("676553777777600","3564");
		myHSIT.put("616773777001600","3565");
		myHSIT.put("266773674017600","3566");
		myHSIT.put("676773677757200","3567");
		myHSIT.put("616773774037600","3568");
		myHSIT.put("676743754777600","3569");
		myHSIT.put("676763777767600","3570");
		myHSIT.put("666773677776600","3571");
		myHSIT.put("616773777774600","3572");
		myHSIT.put("602773674177600","3573");
		myHSIT.put("616763777077600","3574");
		myHSIT.put("676763674177600","3575");
		myHSIT.put("676573777477600","3576");
		myHSIT.put("666773657777600","3577");
		myHSIT.put("676753677777600","3578");
		myHSIT.put("676773676777200","3579");
		myHSIT.put("676773677776000","3580");
		myHSIT.put("400000077477600","3581");
		myHSIT.put("676773037777600","3582");
		myHSIT.put("400000077777400","3583");
		myHSIT.put("076773677777200","3584");
		myHSIT.put("476763677777600","3585");
		myHSIT.put("676773677741600","3586");
		myHSIT.put("446773777767600","3587");
		myHSIT.put("436773437777600","3588");
		myHSIT.put("676773674777600","3589");
		myHSIT.put("676763676377600","3590");
		myHSIT.put("602773677777600","3591");
		myHSIT.put("676473677777600","3592");
		myHSIT.put("612773777777600","3593");
		myHSIT.put("676573677477600","3594");
		myHSIT.put("616773774377600","3595");
		myHSIT.put("646772077777600","3596");
		myHSIT.put("676573777771600","3597");
		myHSIT.put("676473377757600","3598");
		myHSIT.put("676773617743400","3599");
		myHSIT.put("606573777777600","3600");
		myHSIT.put("676561777777600","3601");
		myHSIT.put("676773677776600","3602");
		myHSIT.put("676573417777600","3603");
		myHSIT.put("676343677777600","3604");
		myHSIT.put("666773677776200","3605");
		myHSIT.put("676761377757600","3606");
		myHSIT.put("676373677777600","3607");
		myHSIT.put("676733674177600","3608");
		myHSIT.put("646773777740200","3609");
		myHSIT.put("672773674177600","3610");
		myHSIT.put("636773674177600","3611");
		myHSIT.put("672763777775600","3612");
		myHSIT.put("616753777777600","3613");
		myHSIT.put("676673677777200","3614");
		myHSIT.put("616773777777400","3615");
		myHSIT.put("676673474177600","3616");
		myHSIT.put("676573777777000","3617");
		myHSIT.put("676573377777600","3618");
		myHSIT.put("676773677771200","3619");
		myHSIT.put("466773677777600","3620");
		myHSIT.put("676773777377600","3621");
		myHSIT.put("676763677777000","3622");
		myHSIT.put("676573677774000","3623");
		myHSIT.put("676773277757600","3624");
		myHSIT.put("674773677767600","3625");
		myHSIT.put("676773617747600","3626");
		myHSIT.put("676773677703600","3627");
		myHSIT.put("664033777747600","3628");
		myHSIT.put("674773377757600","3629");
		myHSIT.put("602773777775400","3630");
		myHSIT.put("676763677777200","3631");
		myHSIT.put("676773607777600","3632");
		myHSIT.put("476573677777200","3633");
		myHSIT.put("676773637777200","3634");
		myHSIT.put("666773277777600","3635");
		myHSIT.put("676573777757600","3636");
		myHSIT.put("676773637777600","3637");
		myHSIT.put("676673676177600","3638");
		myHSIT.put("672773677775600","3639");
		myHSIT.put("600000077777600","3640");
		myHSIT.put("676363674177600","3641");
		myHSIT.put("664073777777400","3642");
		myHSIT.put("676600000000000","3643");
		myHSIT.put("660000007777600","3644");
		myHSIT.put("670173677773600","3645");
		myHSIT.put("616773777773600","3646");
		myHSIT.put("476773677777000","3647");
		myHSIT.put("676773676037600","3648");
		myHSIT.put("676661677777600","3649");
		myHSIT.put("676763377757600","3650");
		myHSIT.put("676773677775200","3651");
		myHSIT.put("644063777747600","3652");
		myHSIT.put("676772274177600","3653");
		myHSIT.put("674013674037600","3654");
		myHSIT.put("456773677777600","3655");
		myHSIT.put("676700017777600","3656");
		myHSIT.put("616773777177600","3657");
		myHSIT.put("000000777777600","3658");
		myHSIT.put("636773677777200","3659");
		myHSIT.put("676773670017200","3660");
		myHSIT.put("600373674177600","3661");
		myHSIT.put("476773677776600","3662");
		myHSIT.put("676773277777600","3663");
		myHSIT.put("616343777777600","3664");
		myHSIT.put("616473777777600","3665");
		myHSIT.put("676573777763600","3666");
		myHSIT.put("674353677777600","3667");
		myHSIT.put("676773674377600","3668");
		myHSIT.put("616773777703600","3669");
		myHSIT.put("646763777767600","3670");
		myHSIT.put("676563677774000","3671");
		myHSIT.put("676773666177600","3672");
		myHSIT.put("676773666617600","3673");
		myHSIT.put("676633677777600","3674");
		myHSIT.put("676773774177600","3675");
		myHSIT.put("636773777774600","3676");
		myHSIT.put("276573777777600","3677");
		myHSIT.put("666573677777600","3678");
		myHSIT.put("676363677777600","3679");
		myHSIT.put("676773000037600","3680");
		myHSIT.put("676673677677600","3681");
		myHSIT.put("644073777747600","3682");
		myHSIT.put("616773774017600","3683");
		myHSIT.put("600363777377600","3684");
		myHSIT.put("670373677777200","3685");
		myHSIT.put("616763777774600","3686");
		myHSIT.put("000073777777600","3687");
		myHSIT.put("676573400000000","3688");
		myHSIT.put("646762077777600","3689");
		myHSIT.put("002773677777200","3690");
		myHSIT.put("616773377717600","3691");
		myHSIT.put("676773377740600","3692");
		myHSIT.put("674173777773600","3693");
		myHSIT.put("666733677777600","3694");
		myHSIT.put("676773677437600","3695");
		myHSIT.put("616740004377600","3696");
		myHSIT.put("676753674177600","3697");
		myHSIT.put("600033674177600","3698");
		myHSIT.put("676401677777600","3699");
		myHSIT.put("674773677775600","3700");
		myHSIT.put("676773657577600","3701");
		myHSIT.put("276773677777200","3702");
		myHSIT.put("676773657777200","3703");
		myHSIT.put("676763655777600","3704");
		myHSIT.put("676672677777600","3705");
		myHSIT.put("676433677777600","3706");
		myHSIT.put("676573777775600","3707");
		myHSIT.put("703767740003761","3708");
		myHSIT.put("676771677777600","3709");
		myHSIT.put("676773577777600","3710");
		myHSIT.put("676763777773600","3711");
		myHSIT.put("666773477777600","3712");
		myHSIT.put("676773677737600","3713");
		myHSIT.put("200003367207600","3714");
		myHSIT.put("200003774207600","3715");
		myHSIT.put("200003710000000","3716");
		myHSIT.put("200003357207600","3717");
		myHSIT.put("200003077377600","3718");
		myHSIT.put("676773437637600","3719");
		myHSIT.put("476773777677600","3720");
		myHSIT.put("676773776277400","3721");
		myHSIT.put("674773777637600","3722");
		myHSIT.put("676773437777600","3723");
		myHSIT.put("636773777677600","3724");
		myHSIT.put("676773776237600","3725");
		myHSIT.put("676743776277600","3726");
		myHSIT.put("676343776277600","3727");
		myHSIT.put("676573377677600","3728");
		myHSIT.put("666743717677600","3729");
		myHSIT.put("674002177677600","3730");
		myHSIT.put("676773777647400","3731");
		myHSIT.put("676773776677600","3732");
		myHSIT.put("676343777677600","3733");
		myHSIT.put("676763777674200","3734");
		myHSIT.put("676773776617600","3735");
		myHSIT.put("246773777677600","3736");
		myHSIT.put("676773777657600","3737");
		myHSIT.put("676773776274600","3738");
		myHSIT.put("676773776077600","3739");
		myHSIT.put("676773037613600","3740");
		myHSIT.put("676773776037600","3741");
		myHSIT.put("676763777677600","3742");
		myHSIT.put("676743777677600","3743");
		myHSIT.put("262763777776600","3744");
		myHSIT.put("602762037777600","3745");
		myHSIT.put("602763777777600","3746");
		myHSIT.put("676773777760200","3747");
		myHSIT.put("476773777777600","3748");
		myHSIT.put("402773777757600","3749");
		myHSIT.put("602773777774600","3750");
		myHSIT.put("262773777776600","3751");
		myHSIT.put("602773037777600","3752");
		myHSIT.put("256433777402600","3753");
		myHSIT.put("676773777776600","3754");
		myHSIT.put("676773777777000","3755");
		myHSIT.put("602433777777400","3756");
		myHSIT.put("602773777677600","3757");
		myHSIT.put("602773776277600","3758");
		myHSIT.put("616773775777600","3759");
		myHSIT.put("466773777677600","3760");
		myHSIT.put("377777777760571","3761");
		myHSIT.put("377737777660771","3762");
		myHSIT.put("757400004020731","3763");
		myHSIT.put("577777777420411","3764");
		myHSIT.put("077737777413731","3765");
		myHSIT.put("777737757413371","3766");
		myHSIT.put("401777777413031","3767");
		myHSIT.put("777777743413371","3768");
		myHSIT.put("674020003413771","3769");
		myHSIT.put("674177777413771","3770");
		myHSIT.put("473777777413071","3771");
		myHSIT.put("773777757413371","3772");
		myHSIT.put("773777777763771","3773");
		myHSIT.put("674000001413771","3774");
		myHSIT.put("674377777413771","3775");
		myHSIT.put("037637777760731","3776");
		myHSIT.put("777775077560771","3777");
		myHSIT.put("777775077020771","3778");
		myHSIT.put("767774077020771","3779");
		myHSIT.put("700036777740771","3780");
		myHSIT.put("774777737420731","3781");
		myHSIT.put("777777607660731","3782");
		myHSIT.put("574777777423771","3783");
		myHSIT.put("777777607762771","3784");
		myHSIT.put("676337737760771","3785");
		myHSIT.put("777477607700771","3786");
		myHSIT.put("777723777420771","3787");
		myHSIT.put("777777663760000","3788");
		myHSIT.put("777777740763771","3789");
		myHSIT.put("777767600360771","3790");
		myHSIT.put("703727740003171","3791");
		myHSIT.put("703767640003771","3792");
		myHSIT.put("557767777763771","3793");
		myHSIT.put("703767740000771","3794");
		myHSIT.put("277767607760771","3795");
		myHSIT.put("703767740000371","3796");
		myHSIT.put("777777774120771","3797");
		myHSIT.put("777717777760371","3798");
		myHSIT.put("777767614760771","3799");
		myHSIT.put("703767400003771","3800");
		myHSIT.put("777777770002771","3801");
		myHSIT.put("703774000003771","3802");
		myHSIT.put("477737777413071","3803");
		myHSIT.put("737001777413731","3804");
		myHSIT.put("767777702003371","3805");
		myHSIT.put("737021777413731","3806");
		myHSIT.put("377777777413731","3807");
		myHSIT.put("777737730000000","3808");
		myHSIT.put("777777707413371","3809");
		myHSIT.put("777777607741771","3810");
		myHSIT.put("474377377413771","3811");
		myHSIT.put("776007777760771","3812");
		myHSIT.put("777567777003731","3813");
		myHSIT.put("777777137760771","3814");
		myHSIT.put("703777740023771","3815");
		myHSIT.put("777775007413771","3816");
		myHSIT.put("677777752403371","3817");
		myHSIT.put("777777760360731","3818");
		myHSIT.put("617777777760771","3819");
		myHSIT.put("703777740000001","3820");
		myHSIT.put("703777740003611","3821");
		myHSIT.put("737737737760771","3822");
		myHSIT.put("477757777413071","3823");
		myHSIT.put("700377740000171","3824");
		myHSIT.put("777777777412031","3825");
		myHSIT.put("777761057760771","3826");
		myHSIT.put("737001777411731","3827");
		myHSIT.put("400037777413671","3828");
		myHSIT.put("700377740000011","3829");
		myHSIT.put("703737400001771","3830");
		myHSIT.put("177737777760771","3831");
		myHSIT.put("477777777000000","3832");
		myHSIT.put("735777777413731","3833");
		myHSIT.put("703777740002371","3834");
		myHSIT.put("703177740003771","3835");
		myHSIT.put("677757777760771","3836");
		myHSIT.put("774777777760731","3837");
		myHSIT.put("677777677413771","3838");
		myHSIT.put("077077777413731","3839");
		myHSIT.put("777777470060771","3840");
		myHSIT.put("677777077413701","3841");
		myHSIT.put("674777477413771","3842");
		myHSIT.put("777777577413731","3843");
		myHSIT.put("600003777777600","3844");
		myHSIT.put("600023777777600","3845");
		myHSIT.put("676073777777600","3846");
		myHSIT.put("616763757577600","3847");
		myHSIT.put("676773777402600","3848");
		myHSIT.put("476773777617600","3849");
		myHSIT.put("676773377736600","3850");
		myHSIT.put("674173777777600","3851");
		myHSIT.put("664073777743600","3852");
		myHSIT.put("264073775777600","3853");
		myHSIT.put("276773675777600","3854");
		myHSIT.put("044063777777600","3855");
		myHSIT.put("664063777757600","3856");
		myHSIT.put("664033777776600","3857");
		myHSIT.put("656573377603600","3858");
		myHSIT.put("664031777777600","3859");
		myHSIT.put("664073776377600","3860");
		myHSIT.put("676761757777600","3861");
		myHSIT.put("737777777760700","3862");

		if(!myHSIT.containsKey(spoligo)){
			return "Orphan";
		}
		else{
			return myHSIT.get(spoligo);
		}
		
	}
	
	/**
	 * Method allowing to get Clade_Expert lineage information (contained in SITVIT2)
	 * @param spoligotype
	 * @return
	 * @throws Exception
	 */
	public static String getCladeExpert(String spoligotype) throws Exception{
		HashMap<String,String> myHClade = new HashMap<String,String>();
		
		myHClade.put("000000000000000","ATYPIC");
		myHClade.put("000000000000001","Beijing");
		myHClade.put("000000000000020","Beijing");
		myHClade.put("000000000000031","Beijing");
		myHClade.put("000000000000061","Beijing");
		myHClade.put("000000000000071","Beijing");
		myHClade.put("000000000000171","Beijing");
		myHClade.put("000000000000371","Beijing");
		myHClade.put("000000000000600","microti");
		myHClade.put("000000000000731","Beijing");
		myHClade.put("000000000000771","Beijing");
		myHClade.put("000000000001071","Beijing");
		myHClade.put("000000000001200","Beijing");
		myHClade.put("000000000001771","Beijing");
		myHClade.put("000000000002031","Beijing");
		myHClade.put("000000000002171","Beijing");
		myHClade.put("000000000002371","Beijing");
		myHClade.put("000000000002770","Beijing");
		myHClade.put("000000000002771","Beijing");
		myHClade.put("000000000003000","Beijing");
		myHClade.put("000000000003001","Beijing");
		myHClade.put("000000000003011","Beijing");
		myHClade.put("000000000003020","Beijing");
		myHClade.put("000000000003031","Beijing");
		myHClade.put("000000000003071","Beijing");
		myHClade.put("000000000003151","Beijing");
		myHClade.put("000000000003171","Beijing");
		myHClade.put("000000000003331","Beijing");
		myHClade.put("000000000003371","Beijing");
		myHClade.put("000000000003401","Beijing");
		myHClade.put("000000000003431","Beijing");
		myHClade.put("000000000003471","Beijing");
		myHClade.put("000000000003531","Beijing");
		myHClade.put("000000000003571","Beijing");
		myHClade.put("000000000003600","Beijing");
		myHClade.put("000000000003601","Beijing");
		myHClade.put("000000000003611","Beijing");
		myHClade.put("000000000003630","Beijing");
		myHClade.put("000000000003631","Beijing");
		myHClade.put("000000000003661","Beijing");
		myHClade.put("000000000003671","Beijing");
		myHClade.put("000000000003700","Beijing");
		myHClade.put("000000000003711","Beijing");
		myHClade.put("000000000003731","Beijing");
		myHClade.put("000000000003740","Beijing");
		myHClade.put("000000000003751","Beijing");
		myHClade.put("000000000003760","Beijing");
		myHClade.put("000000000003761","Beijing");
		myHClade.put("000000000003770","Beijing");
		myHClade.put("000000000003771","Beijing");
		myHClade.put("000000000004771","Unknown");
		myHClade.put("000000000007771","Unknown");
		myHClade.put("000000000017771","Unknown");
		myHClade.put("000000000020771","H");
		myHClade.put("000000000060731","T2");
		myHClade.put("000000000060771","T1");
		myHClade.put("000000000100200","PINI");
		myHClade.put("000000000101000","CANETTII");
		myHClade.put("000000000140000","PINI");
		myHClade.put("000000000160771","T1");
		myHClade.put("000000000203771","Unknown");
		myHClade.put("000000000360771","T1");
		myHClade.put("000000000363771","Manu2");
		myHClade.put("000000000403771","Unknown");
		myHClade.put("000000000503771","Unknown");
		myHClade.put("000000000730371","Unknown");
		myHClade.put("000000000760771","T1");
		myHClade.put("000000001560731","T2");
		myHClade.put("000000001560771","T1");
		myHClade.put("000000001760771","T1");
		myHClade.put("000000003737031","Unknown");
		myHClade.put("000000003760771","Unknown");
		myHClade.put("000000004000000","PINI2");
		myHClade.put("000000004000011","Unknown");
		myHClade.put("000000004000171","Unknown");
		myHClade.put("000000004000771","Unknown");
		myHClade.put("000000004003771","Unknown");
		myHClade.put("000000004020571","H2");
		myHClade.put("000000004020631","H2");
		myHClade.put("000000004020721","H2");
		myHClade.put("000000004020731","H2");
		myHClade.put("000000004020771","H2");
		myHClade.put("000000004024771","Unknown");
		myHClade.put("000000004177600","PINI2");
		myHClade.put("000000004220771","H3");
		myHClade.put("000000004320771","H3");
		myHClade.put("000000004367600","PINI2");
		myHClade.put("000000004620731","H3");
		myHClade.put("000000005413671","EAI5");
		myHClade.put("000000005720771","H3");
		myHClade.put("000000006000011","Unknown");
		myHClade.put("000000006003771","Unknown");
		myHClade.put("000000006177600","PINI2");
		myHClade.put("000000006760771","LAM3");
		myHClade.put("000000007000771","Unknown");
		myHClade.put("000000007160771","LAM3");
		myHClade.put("000000007360771","LAM3");
		myHClade.put("000000007377600","PINI2");
		myHClade.put("000000007400000","PINI2");
		myHClade.put("000000007413371","EAI5");
		myHClade.put("000000007413731","EAI1-SOM");
		myHClade.put("000000007413771","EAI5");
		myHClade.put("000000007420731","Ural-2");
		myHClade.put("000000007420771","Ural-2");
		myHClade.put("000000007560731","LAM3");
		myHClade.put("000000007560771","LAM3");
		myHClade.put("000000007600771","Unknown");
		myHClade.put("000000007620671","H3");
		myHClade.put("000000007620771","H3");
		myHClade.put("000000007660771","LAM3");
		myHClade.put("000000007700021","Unknown");
		myHClade.put("000000007700171","Unknown");
		myHClade.put("000000007700371","Unknown");
		myHClade.put("000000007720631","H3");
		myHClade.put("000000007720700","H3");
		myHClade.put("000000007720701","H3");
		myHClade.put("000000007720731","H3");
		myHClade.put("000000007720771","H3");
		myHClade.put("000000007737071","AFRI");
		myHClade.put("000000007740071","LAM3");
		myHClade.put("000000007740661","LAM3");
		myHClade.put("000000007740731","Unknown");
		myHClade.put("000000007740771","LAM3");
		myHClade.put("000000007760331","LAM3");
		myHClade.put("000000007760471","Unknown");
		myHClade.put("000000007760661","LAM3");
		myHClade.put("000000007760701","LAM3");
		myHClade.put("000000007760711","LAM3");
		myHClade.put("000000007760731","S");
		myHClade.put("000000007760751","LAM3");
		myHClade.put("000000007760760","LAM");
		myHClade.put("000000007760761","LAM3");
		myHClade.put("000000007760770","LAM3");
		myHClade.put("000000007760771","Unknown");
		myHClade.put("000000007763771","Manu2");
		myHClade.put("000000007776200","PINI2");
		myHClade.put("000000007777000","PINI2");
		myHClade.put("000000007777600","PINI2");
		myHClade.put("000000007777711","Unknown");
		myHClade.put("000000007777731","Unknown");
		myHClade.put("000000007777771","Unknown");
		myHClade.put("000000017740771","T1");
		myHClade.put("000000017760730","T2-Uganda");
		myHClade.put("000000017760771","Unknown");
		myHClade.put("000000024000000","PINI");
		myHClade.put("000000034003771","Unknown");
		myHClade.put("000000036377371","Unknown");
		myHClade.put("000000037720771","H3");
		myHClade.put("000000037760771","T1");
		myHClade.put("000000074020771","H1");
		myHClade.put("000000076760631","T1");
		myHClade.put("000000077403771","Unknown");
		myHClade.put("000000077413671","EAI5");
		myHClade.put("000000077560731","T2");
		myHClade.put("000000077560771","T1");
		myHClade.put("000000077760771","Unknown");
		myHClade.put("000000077777771","Unknown");
		myHClade.put("000000100003771","Unknown");
		myHClade.put("000000104020771","H1");
		myHClade.put("000000107760731","T2");
		myHClade.put("000000157760771","T5");
		myHClade.put("000000170000000","BOV_4-CAPRAE");
		myHClade.put("000000174020771","H1");
		myHClade.put("000000176760771","T1");
		myHClade.put("000000177720631","H3");
		myHClade.put("000000177760771","T");
		myHClade.put("000000277420771","Ural-2");
		myHClade.put("000000377000010","Unknown");
		myHClade.put("000000377400771","Unknown");
		myHClade.put("000000377620771","H3");
		myHClade.put("000000377737671","AFRI");
		myHClade.put("000000377760771","T1");
		myHClade.put("000000400200000","BOV_4-CAPRAE");
		myHClade.put("000000577760731","T2");
		myHClade.put("000000674364771","Unknown");
		myHClade.put("000000717207600","BOV_4-CAPRAE");
		myHClade.put("000000757670340","Unknown");
		myHClade.put("000000760003771","Unknown");
		myHClade.put("000000777760730","T2-Uganda");
		myHClade.put("000000777777600","BOV");
		myHClade.put("000001705777600","BOV");
		myHClade.put("000001707777000","BOV");
		myHClade.put("000001707777600","BOV");
		myHClade.put("000001740005471","AFRI");
		myHClade.put("000001745777600","BOV");
		myHClade.put("000001747777000","BOV");
		myHClade.put("000001747777600","BOV");
		myHClade.put("000001777207600","BOV_4-CAPRAE");
		myHClade.put("000001777760571","T1");
		myHClade.put("000001777760771","Unknown");
		myHClade.put("000002000000111","Unknown");
		myHClade.put("000002000003661","AFRI");
		myHClade.put("000002000003671","AFRI");
		myHClade.put("000002000003771","Unknown");
		myHClade.put("000002004020731","H1");
		myHClade.put("000002777760731","X1");
		myHClade.put("000003607760771","LAM3");
		myHClade.put("000003743760771","Cameroon");
		myHClade.put("000003770000731","Unknown");
		myHClade.put("000003777377600","BOV_4-CAPRAE");
		myHClade.put("000003777760731","T2");
		myHClade.put("000003777760771","T1");
		myHClade.put("000004000000000","BOV");
		myHClade.put("000006007413731","EAI1-SOM");
		myHClade.put("000007077700020","Unknown");
		myHClade.put("000007374000000","BOV");
		myHClade.put("000007760060771","T1");
		myHClade.put("000007777760771","T1");
		myHClade.put("000016777760671","X1");
		myHClade.put("000017600003771","Unknown");
		myHClade.put("000020000003771","Unknown");
		myHClade.put("000020007760731","LAM-RUS");
		myHClade.put("000020034003671","AFRI");
		myHClade.put("000022000003771","Unknown");
		myHClade.put("000027607020671","H3");
		myHClade.put("000033777777600","BOV");
		myHClade.put("000036040003771","Unknown");
		myHClade.put("000036777760771","X1");
		myHClade.put("000037607760771","LAM3");
		myHClade.put("000037740000000","BOV");
		myHClade.put("000037774001771","Unknown");
		myHClade.put("000037777403771","Unknown");
		myHClade.put("000037777413771","EAI5");
		myHClade.put("000037777760770","T1");
		myHClade.put("000037777760771","T1");
		myHClade.put("000037777763661","AFRI");
		myHClade.put("000046037160000","BOV");
		myHClade.put("000047414020771","H1");
		myHClade.put("000073777777600","BOV");
		myHClade.put("000076700000031","Unknown");
		myHClade.put("000076775760671","X1");
		myHClade.put("000076777760671","X1");
		myHClade.put("000076777760771","X1");
		myHClade.put("000077774020771","H1");
		myHClade.put("000077777760771","T1");
		myHClade.put("000140000000000","BOV_4-CAPRAE");
		myHClade.put("000147620000771","Unknown");
		myHClade.put("000173777777600","BOV");
		myHClade.put("000177600000000","BOV");
		myHClade.put("000177717760771","T1");
		myHClade.put("000177760000771","Unknown");
		myHClade.put("000177777420771","Ural-2");
		myHClade.put("000177777760771","T1");
		myHClade.put("000200000000000","Unknown");
		myHClade.put("000200000002000","BOV_4-CAPRAE");
		myHClade.put("000202000002771","Unknown");
		myHClade.put("000220000000000","BOV_4-CAPRAE");
		myHClade.put("000320000003771","Unknown");
		myHClade.put("000367436560771","T1");
		myHClade.put("000373767501471","AFRI");
		myHClade.put("000377600000000","BOV");
		myHClade.put("000377601560771","T1");
		myHClade.put("000377777420771","Ural-2");
		myHClade.put("000377777560771","T1");
		myHClade.put("000377777760771","T1");
		myHClade.put("000667777760771","T1");
		myHClade.put("000677777760771","T1");
		myHClade.put("000740004460671","Turkey");
		myHClade.put("000773777773600","BOV");
		myHClade.put("000773777777600","BOV");
		myHClade.put("000777606060771","LAM11-ZWE");
		myHClade.put("000777607760771","LAM1");
		myHClade.put("000777774560460","T1");
		myHClade.put("000777777760771","T1");
		myHClade.put("001020000000000","BOV_4-CAPRAE");
		myHClade.put("001677777760771","T1");
		myHClade.put("001741760340071","T1");
		myHClade.put("001767000003071","Unknown");
		myHClade.put("001776777760601","X2");
		myHClade.put("001776777760771","X1");
		myHClade.put("001777207560771","LAM6");
		myHClade.put("001777600000000","Unknown");
		myHClade.put("001777606060771","LAM11-ZWE");
		myHClade.put("001777607560771","LAM6");
		myHClade.put("001777607760771","LAM1");
		myHClade.put("001777707560771","T1");
		myHClade.put("001777777760771","T1");
		myHClade.put("002002000003371","Unknown");
		myHClade.put("002002007000000","BOV_4-CAPRAE");
		myHClade.put("002773677777200","BOV");
		myHClade.put("002777777400771","Unknown");
		myHClade.put("003000001400020","Unknown");
		myHClade.put("003701000000771","Unknown");
		myHClade.put("003767700003771","Unknown");
		myHClade.put("003774777760771","T1");
		myHClade.put("003776777760771","X1");
		myHClade.put("003777400000771","Unknown");
		myHClade.put("003777400010771","EAI5");
		myHClade.put("003777607760771","LAM1");
		myHClade.put("003777740002771","Unknown");
		myHClade.put("003777740003171","Unknown");
		myHClade.put("003777740003771","Unknown");
		myHClade.put("003777747762771","Manu2");
		myHClade.put("003777747777771","Unknown");
		myHClade.put("003777774020771","H1");
		myHClade.put("003777775413031","EAI5");
		myHClade.put("003777777420771","Ural-2");
		myHClade.put("003777777760731","T2");
		myHClade.put("004061427777600","BOV");
		myHClade.put("004267637746600","BOV");
		myHClade.put("006177607760771","LAM3");
		myHClade.put("006363777760771","S");
		myHClade.put("006377777760771","S");
		myHClade.put("006563616000771","Unknown");
		myHClade.put("006737777760771","T3");
		myHClade.put("006773677777600","BOV");
		myHClade.put("006773777777600","BOV");
		myHClade.put("007757606000031","Unknown");
		myHClade.put("007764057777771","Unknown");
		myHClade.put("007776777760771","X1");
		myHClade.put("007777606000031","Unknown");
		myHClade.put("007777606060771","LAM11-ZWE");
		myHClade.put("007777607760711","LAM");
		myHClade.put("007777607760771","LAM1");
		myHClade.put("007777707760351","T1");
		myHClade.put("007777707760771","T1");
		myHClade.put("007777777741371","Unknown");
		myHClade.put("007777777760771","T1");
		myHClade.put("011203777760671","T1");
		myHClade.put("011340607740661","LAM1");
		myHClade.put("014177775740721","T2");
		myHClade.put("015707607740071","LAM2");
		myHClade.put("015747600160760","T1");
		myHClade.put("015777777420771","Ural-2");
		myHClade.put("016337777760731","S");
		myHClade.put("016703677746600","BOV");
		myHClade.put("016761777777600","BOV");
		myHClade.put("016773777777600","BOV");
		myHClade.put("016777606060771","LAM11-ZWE");
		myHClade.put("017467576140771","T1");
		myHClade.put("017677777760771","T1");
		myHClade.put("017700007702400","Unknown");
		myHClade.put("017703777760771","T1");
		myHClade.put("017774001560371","T1");
		myHClade.put("017775001560371","T1");
		myHClade.put("017776607760771","LAM1");
		myHClade.put("017777417760771","T1");
		myHClade.put("017777600000371","Unknown");
		myHClade.put("017777600000771","Unknown");
		myHClade.put("017777607560371","LAM6");
		myHClade.put("017777607760771","LAM1");
		myHClade.put("017777774020771","H1");
		myHClade.put("017777777320771","H3");
		myHClade.put("017777777700000","Unknown");
		myHClade.put("017777777760071","T1");
		myHClade.put("017777777760731","T2");
		myHClade.put("017777777760760","T1");
		myHClade.put("020000000000400","PINI");
		myHClade.put("024041767542600","BOV");
		myHClade.put("024061637777600","BOV");
		myHClade.put("024061777777600","BOV");
		myHClade.put("030771462603471","AFRI");
		myHClade.put("033777477413771","EAI5");
		myHClade.put("034777777420771","Ural-2");
		myHClade.put("035777776400000","Unknown");
		myHClade.put("036377777760771","S");
		myHClade.put("037174002000071","Unknown");
		myHClade.put("037317607000271","Unknown");
		myHClade.put("037377607760731","LAM");
		myHClade.put("037603607400011","Unknown");
		myHClade.put("037621703774371","Unknown");
		myHClade.put("037627777760771","T1");
		myHClade.put("037636377760771","T1");
		myHClade.put("037637037760771","T1");
		myHClade.put("037637377760771","T4");
		myHClade.put("037637377770371","Unknown");
		myHClade.put("037637477720771","H3");
		myHClade.put("037637607760771","LAM2");
		myHClade.put("037637717760771","T1");
		myHClade.put("037637743760771","Cameroon");
		myHClade.put("037637777720771","H3");
		myHClade.put("037637777760731","T2");
		myHClade.put("037637777760771","T1");
		myHClade.put("037637777770171","Unknown");
		myHClade.put("037640367760771","T1");
		myHClade.put("037657777741770","Unknown");
		myHClade.put("037667500020771","H3");
		myHClade.put("037667600020771","H3");
		myHClade.put("037674777620771","H3");
		myHClade.put("037676000000331","Unknown");
		myHClade.put("037676000000360","Unknown");
		myHClade.put("037676000000371","Unknown");
		myHClade.put("037676004360371","LAM1");
		myHClade.put("037677500020731","H3");
		myHClade.put("037677500020771","H3");
		myHClade.put("037677707760771","T1");
		myHClade.put("037677760000171","Unknown");
		myHClade.put("037677774020771","H1");
		myHClade.put("037677777607771","Unknown");
		myHClade.put("037677777620771","H3");
		myHClade.put("037677777720731","H3");
		myHClade.put("037677777760771","T1");
		myHClade.put("037727777760771","T3");
		myHClade.put("037737607760771","LAM2");
		myHClade.put("037737777760771","T3");
		myHClade.put("037741403760731","T2");
		myHClade.put("037766677760601","X2");
		myHClade.put("037767777720731","H3");
		myHClade.put("037767777760471","T1");
		myHClade.put("037775770060771","T1");
		myHClade.put("037776777760601","X2");
		myHClade.put("037777407560731","LAM");
		myHClade.put("037777407760731","LAM1");
		myHClade.put("037777477760711","T-H37Rv");
		myHClade.put("037777606000031","Unknown");
		myHClade.put("037777607760771","LAM1");
		myHClade.put("037777740003771","Unknown");
		myHClade.put("037777740013771","EAI5");
		myHClade.put("037777770060771","T1");
		myHClade.put("037777774020771","H1");
		myHClade.put("037777775720731","H3");
		myHClade.put("037777777000371","Unknown");
		myHClade.put("037777777413700","EAI5");
		myHClade.put("037777777520771","H3");
		myHClade.put("037777777720671","H3");
		myHClade.put("037777777720731","H3");
		myHClade.put("037777777720771","H3");
		myHClade.put("037777777760731","T2");
		myHClade.put("037777777760771","T1");
		myHClade.put("040000000000600","microti");
		myHClade.put("040077200000370","Unknown");
		myHClade.put("042773777777600","BOV");
		myHClade.put("044063777777600","BOV");
		myHClade.put("044263777746600","BOV");
		myHClade.put("046773777764600","BOV");
		myHClade.put("046773777767600","BOV");
		myHClade.put("047777607760771","LAM1");
		myHClade.put("057677606060771","LAM11-ZWE");
		myHClade.put("057777607760771","LAM1");
		myHClade.put("057777777760771","T1");
		myHClade.put("060000777777671","AFRI");
		myHClade.put("061777606060671","LAM11-ZWE");
		myHClade.put("064061767776600","BOV");
		myHClade.put("064061777777600","BOV");
		myHClade.put("064063407777600","BOV");
		myHClade.put("064063767777600","BOV");
		myHClade.put("064063777767600","BOV");
		myHClade.put("064063777777600","BOV");
		myHClade.put("064073777777600","BOV");
		myHClade.put("064263777046600","BOV_4-CAPRAE");
		myHClade.put("064263777746600","BOV");
		myHClade.put("064263777777600","BOV");
		myHClade.put("066263777777600","BOV");
		myHClade.put("067777777413031","EAI5");
		myHClade.put("070000000003600","PINI");
		myHClade.put("070377777767671","AFRI");
		myHClade.put("071777770000021","Unknown");
		myHClade.put("074000003413771","EAI5");
		myHClade.put("074000007177200","PINI");
		myHClade.put("074000007777600","PINI");
		myHClade.put("074000026037600","PINI1");
		myHClade.put("074000030000400","PINI1");
		myHClade.put("074000030000600","microti");
		myHClade.put("074000032000400","PINI1");
		myHClade.put("074000032000600","microti");
		myHClade.put("074000037377600","PINI1");
		myHClade.put("074000037777600","PINI1");
		myHClade.put("074000077777600","BOV");
		myHClade.put("074000430000600","BOV_4-CAPRAE");
		myHClade.put("074000437777600","BOV");
		myHClade.put("074002007177200","BOV_4-CAPRAE");
		myHClade.put("074367607760700","LAM");
		myHClade.put("074377607760700","LAM");
		myHClade.put("075747637403631","Unknown");
		myHClade.put("076160000000071","Unknown");
		myHClade.put("076357777760771","S");
		myHClade.put("076377737760771","S");
		myHClade.put("076737737760771","T3");
		myHClade.put("076753777777600","BOV");
		myHClade.put("076763677777600","BOV");
		myHClade.put("076773677777200","BOV");
		myHClade.put("076773677777600","BOV");
		myHClade.put("076773776077600","BOV_4-CAPRAE");
		myHClade.put("076773777777400","BOV");
		myHClade.put("076773777777600","BOV");
		myHClade.put("076777737760771","T1");
		myHClade.put("076777777777600","BOV");
		myHClade.put("077077777413731","EAI1-SOM");
		myHClade.put("077264577775270","Unknown");
		myHClade.put("077307637740771","T1");
		myHClade.put("077327776413731","EAI1-SOM");
		myHClade.put("077347606060661","LAM11-ZWE");
		myHClade.put("077417773413071","EAI3-IND");
		myHClade.put("077577477413750","EAI5");
		myHClade.put("077577717003351","Unknown");
		myHClade.put("077577747003371","Unknown");
		myHClade.put("077577757000371","Unknown");
		myHClade.put("077577757000771","Unknown");
		myHClade.put("077577757003371","Unknown");
		myHClade.put("077577767403371","Unknown");
		myHClade.put("077603405760471","LAM2");
		myHClade.put("077603405760671","LAM2");
		myHClade.put("077637770000671","Unknown");
		myHClade.put("077637777760771","T1");
		myHClade.put("077677777720771","H3");
		myHClade.put("077701002001731","Unknown");
		myHClade.put("077737604560771","LAM2");
		myHClade.put("077737607760771","LAM2");
		myHClade.put("077737777403771","Unknown");
		myHClade.put("077737777413731","EAI1-SOM");
		myHClade.put("077737777413740","EAI5");
		myHClade.put("077737777413771","EAI5");
		myHClade.put("077737777420771","Ural-2");
		myHClade.put("077737777760771","T3");
		myHClade.put("077743777413731","EAI1-SOM");
		myHClade.put("077757607760731","LAM");
		myHClade.put("077757607760771","LAM1");
		myHClade.put("077757777413771","EAI5");
		myHClade.put("077767777413731","EAI1-SOM");
		myHClade.put("077767777760771","T1");
		myHClade.put("077773477413771","EAI5");
		myHClade.put("077775377413771","EAI8-MDG");
		myHClade.put("077776117413771","EAI5");
		myHClade.put("077776775760731","X1");
		myHClade.put("077776777403671","Unknown");
		myHClade.put("077776777760701","X1");
		myHClade.put("077776777760731","X1");
		myHClade.put("077777077413771","EAI5");
		myHClade.put("077777277403771","Unknown");
		myHClade.put("077777377000371","Unknown");
		myHClade.put("077777377400771","Unknown");
		myHClade.put("077777377413771","EAI8-MDG");
		myHClade.put("077777377417771","Unknown");
		myHClade.put("077777417760730","T2-Uganda");
		myHClade.put("077777474013171","EAI5");
		myHClade.put("077777477412771","EAI5");
		myHClade.put("077777477413771","EAI5");
		myHClade.put("077777606000031","Unknown");
		myHClade.put("077777606060671","LAM11-ZWE");
		myHClade.put("077777606060771","LAM11-ZWE");
		myHClade.put("077777607560771","LAM6");
		myHClade.put("077777607760730","LAM");
		myHClade.put("077777607760731","LAM");
		myHClade.put("077777607760771","LAM1");
		myHClade.put("077777700020771","H3");
		myHClade.put("077777707760731","T2");
		myHClade.put("077777747413731","EAI");
		myHClade.put("077777747413771","EAI6-BGD1");
		myHClade.put("077777774020731","H1");
		myHClade.put("077777774020771","H1");
		myHClade.put("077777774360771","T1");
		myHClade.put("077777777000071","Unknown");
		myHClade.put("077777777000371","Unknown");
		myHClade.put("077777777413011","EAI5");
		myHClade.put("077777777413031","EAI5");
		myHClade.put("077777777413071","EAI3-IND");
		myHClade.put("077777777413631","EAI5");
		myHClade.put("077777777413700","EAI5");
		myHClade.put("077777777413730","EAI1-SOM");
		myHClade.put("077777777413731","EAI1-SOM");
		myHClade.put("077777777413771","EAI5");
		myHClade.put("077777777420771","Ural-2");
		myHClade.put("077777777560771","T1");
		myHClade.put("077777777720731","H3");
		myHClade.put("077777777760031","T1");
		myHClade.put("077777777760731","T2");
		myHClade.put("077777777760771","T");
		myHClade.put("077777777763731","Manu2");
		myHClade.put("077777777770771","Manu3");
		myHClade.put("077777777777771","Unknown");
		myHClade.put("100000001413031","EAI5");
		myHClade.put("100000004020771","H1");
		myHClade.put("100036677760771","X3");
		myHClade.put("100076777760771","X3");
		myHClade.put("100240204004621","Unknown");
		myHClade.put("100775747413771","EAI6-BGD1");
		myHClade.put("101067740000071","CAS");
		myHClade.put("101777740003771","CAS");
		myHClade.put("102000340003171","CAS1-Delhi");
		myHClade.put("103377400001771","CAS1-Kili");
		myHClade.put("103741000000760","CAS");
		myHClade.put("103767740003171","CAS1-Delhi");
		myHClade.put("103767740003771","CAS1-Delhi");
		myHClade.put("103777600001771","CAS");
		myHClade.put("103777740000371","CAS");
		myHClade.put("103777740000771","CAS");
		myHClade.put("103777740003071","CAS1-Delhi");
		myHClade.put("103777740003171","CAS1-Delhi");
		myHClade.put("103777740003771","CAS1-Delhi");
		myHClade.put("103777777563771","Manu2");
		myHClade.put("104244001020641","H3");
		myHClade.put("114605743440220","Cameroon");
		myHClade.put("114671743740220","Cameroon");
		myHClade.put("115347576360661","T1");
		myHClade.put("117756777760771","X1");
		myHClade.put("117777777560771","T1");
		myHClade.put("130257677420771","Ural-2");
		myHClade.put("137347677560631","T1");
		myHClade.put("137637777760771","T1");
		myHClade.put("137777777413731","EAI1-SOM");
		myHClade.put("153777777760771","T1");
		myHClade.put("155747437740771","T1");
		myHClade.put("157756777760731","X1");
		myHClade.put("157756777760771","X1");
		myHClade.put("157760000060771","LAM-RUS");
		myHClade.put("157777574020571","H1");
		myHClade.put("157777777004340","Unknown");
		myHClade.put("157777777760731","T2");
		myHClade.put("160000000003771","Unknown");
		myHClade.put("160001400000171","Unknown");
		myHClade.put("161777776003771","Unknown");
		myHClade.put("170003607760771","LAM");
		myHClade.put("170023607760771","LAM");
		myHClade.put("170077770007071","AFRI");
		myHClade.put("170777703017671","AFRI_1");
		myHClade.put("170777777777671","AFRI_1");
		myHClade.put("173767607760771","LAM9");
		myHClade.put("173776607760771","LAM9");
		myHClade.put("174000007413731","EAI1-SOM");
		myHClade.put("174067606767071","AFRI_2");
		myHClade.put("174132417537741","Unknown");
		myHClade.put("175477607760771","LAM9");
		myHClade.put("175776677413131","EAI1-SOM");
		myHClade.put("176160000000071","Unknown");
		myHClade.put("176177777760731","T2");
		myHClade.put("176277777760771","S");
		myHClade.put("176377777760771","S");
		myHClade.put("177000377760771","T3-ETH");
		myHClade.put("177177607760771","LAM9");
		myHClade.put("177377777777771","Unknown");
		myHClade.put("177727001760731","T");
		myHClade.put("177727777720771","H3");
		myHClade.put("177737600160771","T3");
		myHClade.put("177737607760771","LAM5");
		myHClade.put("177737777720771","H3");
		myHClade.put("177737777760771","T3");
		myHClade.put("177743777413731","EAI1-SOM");
		myHClade.put("177757606060771","LAM11-ZWE");
		myHClade.put("177757677760771","T1");
		myHClade.put("177760603760771","T1");
		myHClade.put("177767377400671","Unknown");
		myHClade.put("177767377420771","Ural-2");
		myHClade.put("177767623760661","T1");
		myHClade.put("177767737740771","T1");
		myHClade.put("177767737760771","T1");
		myHClade.put("177767777413700","EAI5");
		myHClade.put("177767777720771","H3");
		myHClade.put("177767777760731","T2");
		myHClade.put("177767777777731","Unknown");
		myHClade.put("177770000000731","Unknown");
		myHClade.put("177770200000720","Unknown");
		myHClade.put("177774747413771","EAI6-BGD1");
		myHClade.put("177775757443771","Unknown");
		myHClade.put("177775777760731","T2");
		myHClade.put("177776003760771","T1");
		myHClade.put("177776407760401","X2");
		myHClade.put("177776777760601","X2");
		myHClade.put("177777000000011","Zero-copy");
		myHClade.put("177777177413771","EAI5");
		myHClade.put("177777377000371","Unknown");
		myHClade.put("177777403760771","T1");
		myHClade.put("177777603760771","T1");
		myHClade.put("177777606060471","LAM11-ZWE");
		myHClade.put("177777606400371","Unknown");
		myHClade.put("177777607560740","LAM6");
		myHClade.put("177777607560771","LAM6");
		myHClade.put("177777607760731","LAM4");
		myHClade.put("177777607760771","LAM9");
		myHClade.put("177777617760771","T1");
		myHClade.put("177777637760771","T1");
		myHClade.put("177777657760771","T5");
		myHClade.put("177777677760171","T1");
		myHClade.put("177777677760771","T1");
		myHClade.put("177777677766171","Unknown");
		myHClade.put("177777737760771","T1");
		myHClade.put("177777740000371","Unknown");
		myHClade.put("177777750000000","Unknown");
		myHClade.put("177777764020771","H1");
		myHClade.put("177777770000000","Unknown");
		myHClade.put("177777774003731","EAI1-SOM");
		myHClade.put("177777774020731","H1");
		myHClade.put("177777777013721","EAI1-SOM");
		myHClade.put("177777777413131","EAI1-SOM");
		myHClade.put("177777777413371","EAI5");
		myHClade.put("177777777413731","EAI1-SOM");
		myHClade.put("177777777413771","EAI5");
		myHClade.put("177777777420671","Ural-2");
		myHClade.put("177777777703560","Unknown");
		myHClade.put("177777777720771","H3");
		myHClade.put("177777777760601","T1");
		myHClade.put("177777777760731","T2");
		myHClade.put("177777777760771","T1");
		myHClade.put("200000017377600","BOV_4-CAPRAE");
		myHClade.put("200000077377600","BOV_4-CAPRAE");
		myHClade.put("200001377376600","BOV_4-CAPRAE");
		myHClade.put("200003077377600","BOV_4-CAPRAE");
		myHClade.put("200003317207600","BOV_4-CAPRAE");
		myHClade.put("200003357207600","BOV_4-CAPRAE");
		myHClade.put("200003367207600","BOV_4-CAPRAE");
		myHClade.put("200003370000000","BOV_4-CAPRAE");
		myHClade.put("200003377205200","BOV_4-CAPRAE");
		myHClade.put("200003377207600","BOV_4-CAPRAE");
		myHClade.put("200003377377600","BOV_4-CAPRAE");
		myHClade.put("200003400037600","BOV_4-CAPRAE");
		myHClade.put("200003617207600","BOV_4-CAPRAE");
		myHClade.put("200003637343600","BOV_4-CAPRAE");
		myHClade.put("200003677376600","BOV_4-CAPRAE");
		myHClade.put("200003710000000","BOV_4-CAPRAE");
		myHClade.put("200003717204600","BOV_4-CAPRAE");
		myHClade.put("200003717207400","BOV_4-CAPRAE");
		myHClade.put("200003717207600","BOV_4-CAPRAE");
		myHClade.put("200003757377600","BOV_4-CAPRAE");
		myHClade.put("200003770000000","BOV_4-CAPRAE");
		myHClade.put("200003770003600","BOV_4-CAPRAE");
		myHClade.put("200003773377600","BOV_4-CAPRAE");
		myHClade.put("200003774075600","BOV_4-CAPRAE");
		myHClade.put("200003774207600","BOV_4-CAPRAE");
		myHClade.put("200003776007600","BOV_4-CAPRAE");
		myHClade.put("200003776207600","BOV_4-CAPRAE");
		myHClade.put("200003777203600","BOV_4-CAPRAE");
		myHClade.put("200003777207600","BOV_4-CAPRAE");
		myHClade.put("200003777237600","BOV_4-CAPRAE");
		myHClade.put("200003777357000","BOV_4-CAPRAE");
		myHClade.put("200003777357600","BOV_4-CAPRAE");
		myHClade.put("200003777361600","BOV_4-CAPRAE");
		myHClade.put("200003777363600","BOV_4-CAPRAE");
		myHClade.put("200003777373600","BOV_4-CAPRAE");
		myHClade.put("200003777375600","BOV_4-CAPRAE");
		myHClade.put("200003777377200","BOV_4-CAPRAE");
		myHClade.put("200003777377600","BOV_4-CAPRAE");
		myHClade.put("200003777607600","BOV");
		myHClade.put("200003777777600","BOV");
		myHClade.put("200017777760771","T1");
		myHClade.put("200023717207600","BOV_4-CAPRAE");
		myHClade.put("200023777237600","BOV_4-CAPRAE");
		myHClade.put("200023777377600","BOV_4-CAPRAE");
		myHClade.put("200023777377601","Unknown");
		myHClade.put("200037777760771","T1");
		myHClade.put("200103717207600","BOV_4-CAPRAE");
		myHClade.put("203067777670671","Manu3");
		myHClade.put("203777140000771","Unknown");
		myHClade.put("203777763760771","T1");
		myHClade.put("204773674017600","BOV_4-CAPRAE");
		myHClade.put("237760774360330","T2-Uganda");
		myHClade.put("237775777760730","T2-Uganda");
		myHClade.put("237777777760771","T1");
		myHClade.put("244004000000000","BOV");
		myHClade.put("246773777677600","BOV_1");
		myHClade.put("256423777402600","BOV");
		myHClade.put("256433777402600","BOV_1");
		myHClade.put("262763777776600","BOV");
		myHClade.put("262773777756600","BOV_1");
		myHClade.put("262773777772600","BOV_1");
		myHClade.put("262773777776600","BOV_1");
		myHClade.put("264063777777600","BOV");
		myHClade.put("264071777777600","BOV");
		myHClade.put("264073737777600","BOV_2");
		myHClade.put("264073775777600","BOV_2");
		myHClade.put("264073777777600","BOV_2");
		myHClade.put("266063777777600","BOV");
		myHClade.put("266073777777600","BOV");
		myHClade.put("266773674017600","BOV_4-CAPRAE");
		myHClade.put("267013775777600","Unknown");
		myHClade.put("270000036763600","BOV");
		myHClade.put("272773777777600","BOV_1");
		myHClade.put("276377776060731","S");
		myHClade.put("276573737777600","BOV_1");
		myHClade.put("276573777777600","BOV_1");
		myHClade.put("276763777777600","BOV");
		myHClade.put("276773675777600","BOV_1");
		myHClade.put("276773677777200","BOV_1");
		myHClade.put("276773677777600","BOV_1");
		myHClade.put("276773777777600","BOV_1");
		myHClade.put("277677606000171","Unknown");
		myHClade.put("277737607760771","LAM2");
		myHClade.put("277757577760771","T1");
		myHClade.put("277767607760771","LAM1");
		myHClade.put("277775477413771","EAI2-Manila");
		myHClade.put("277777077413771","EAI2");
		myHClade.put("277777471413771","EAI2-Manila");
		myHClade.put("277777477413771","EAI2-Manila");
		myHClade.put("277777603560771","T1");
		myHClade.put("277777607620771","H3");
		myHClade.put("277777607760771","LAM1");
		myHClade.put("300000007177771","Unknown");
		myHClade.put("300000007760771","LAM3");
		myHClade.put("300036777760771","X3");
		myHClade.put("300036777763771","Manu2");
		myHClade.put("300067777670531","Manu3");
		myHClade.put("300076760000011","Unknown");
		myHClade.put("300076777760771","X3");
		myHClade.put("300077777760771","T1");
		myHClade.put("300767747413771","EAI6-BGD1");
		myHClade.put("300777663660711","T1");
		myHClade.put("300777747413771","EAI6-BGD1");
		myHClade.put("303741000000741","CAS");
		myHClade.put("303741000000760","CAS");
		myHClade.put("303777400003771","CAS");
		myHClade.put("303777740003171","CAS1-Delhi");
		myHClade.put("303777740003731","CAS1-Delhi");
		myHClade.put("303777740003771","CAS1-Delhi");
		myHClade.put("303777775003171","Unknown");
		myHClade.put("306777100003361","Unknown");
		myHClade.put("317777777413731","EAI1-SOM");
		myHClade.put("317777777760771","T1");
		myHClade.put("330777777767671","AFRI_1");
		myHClade.put("331440640000000","Unknown");
		myHClade.put("336177607760771","LAM3");
		myHClade.put("337637777760771","T1");
		myHClade.put("337737777760771","T3");
		myHClade.put("337777607760771","LAM9");
		myHClade.put("337777777412031","EAI5");
		myHClade.put("337777777413731","EAI1-SOM");
		myHClade.put("337777777760731","T2");
		myHClade.put("337777777760771","T1");
		myHClade.put("340000007760731","Unknown");
		myHClade.put("341357000001661","Unknown");
		myHClade.put("341577600001771","Unknown");
		myHClade.put("341777600001771","Unknown");
		myHClade.put("343775777560731","T2");
		myHClade.put("343777740003771","Unknown");
		myHClade.put("357776377760771","T1");
		myHClade.put("357777601560711","T1");
		myHClade.put("357777607720771","H3");
		myHClade.put("357777607760571","LAM9");
		myHClade.put("357777607760731","LAM4");
		myHClade.put("357777607760771","LAM9");
		myHClade.put("357777777760771","T1");
		myHClade.put("363777600560771","T1");
		myHClade.put("363777777403771","Unknown");
		myHClade.put("367777770020731","H3");
		myHClade.put("367777770060731","T2");
		myHClade.put("370000777760771","T1-RUS2");
		myHClade.put("370177607760771","LAM3");
		myHClade.put("370776674672671","AFRI_1");
		myHClade.put("371767643760651","Cameroon");
		myHClade.put("373727777760771","T3");
		myHClade.put("373737606060771","LAM11-ZWE");
		myHClade.put("373767744760771","T1");
		myHClade.put("373767777760771","T1");
		myHClade.put("374176001413771","EAI5");
		myHClade.put("374377602760771","T1");
		myHClade.put("374377607760700","LAM4");
		myHClade.put("374777777420771","Ural-1");
		myHClade.put("375367423560671","T1");
		myHClade.put("376017607760771","LAM12-Madrid1");
		myHClade.put("376037777760771","T1");
		myHClade.put("376117607760771","LAM3");
		myHClade.put("376137607760771","LAM3");
		myHClade.put("376147607760771","LAM3");
		myHClade.put("376157607760661","LAM3");
		myHClade.put("376157767730771","Manu3");
		myHClade.put("376160000000071","Unknown");
		myHClade.put("376177407760771","LAM3");
		myHClade.put("376177600360771","T1");
		myHClade.put("376177607760371","LAM3");
		myHClade.put("376177607760731","LAM3");
		myHClade.put("376177607760771","LAM3");
		myHClade.put("376367777760771","S");
		myHClade.put("376377607760771","Unknown");
		myHClade.put("376377777420770","Ural-1");
		myHClade.put("376377777760731","S");
		myHClade.put("376377777760771","S");
		myHClade.put("376774737760771","T1");
		myHClade.put("376776337760771","T1");
		myHClade.put("376777737760771","T1");
		myHClade.put("377000377760771","T3-ETH");
		myHClade.put("377357606060771","LAM11-ZWE");
		myHClade.put("377367767760771","T1");
		myHClade.put("377377606060771","LAM11-ZWE");
		myHClade.put("377377607760771","LAM9");
		myHClade.put("377377777720771","H3");
		myHClade.put("377377777723771","Manu2");
		myHClade.put("377400000000131","Unknown");
		myHClade.put("377477607760031","LAM4");
		myHClade.put("377577606060771","LAM11-ZWE");
		myHClade.put("377577777003731","Unknown");
		myHClade.put("377600007413371","EAI5");
		myHClade.put("377637607560771","LAM6");
		myHClade.put("377677777760771","T1");
		myHClade.put("377700000001331","Unknown");
		myHClade.put("377700000020731","H3");
		myHClade.put("377703377760731","T");
		myHClade.put("377703777760671","T1");
		myHClade.put("377703777760731","T2");
		myHClade.put("377703777760771","T1");
		myHClade.put("377707777760771","T1");
		myHClade.put("377717607760771","LAM5");
		myHClade.put("377717777760771","T1");
		myHClade.put("377723777760671","T3");
		myHClade.put("377727607760771","LAM5");
		myHClade.put("377734077760771","T3");
		myHClade.put("377736777413430","EAI5");
		myHClade.put("377737377720771","H3");
		myHClade.put("377737606060771","LAM11-ZWE");
		myHClade.put("377737607760771","LAM5");
		myHClade.put("377737670000000","Unknown");
		myHClade.put("377737677760771","T3");
		myHClade.put("377737774020731","H1");
		myHClade.put("377737777420771","Ural-1");
		myHClade.put("377737777660771","T3");
		myHClade.put("377737777760700","T3");
		myHClade.put("377737777760731","T");
		myHClade.put("377737777760771","T3");
		myHClade.put("377740777760771","T1");
		myHClade.put("377743774360771","T1");
		myHClade.put("377757607620771","H3");
		myHClade.put("377757777760771","T1");
		myHClade.put("377760007760771","LAM-RUS");
		myHClade.put("377760017760601","T-tuscany");
		myHClade.put("377767605660771","LAM9");
		myHClade.put("377767607760731","LAM4");
		myHClade.put("377767607760771","LAM9");
		myHClade.put("377767677760771","T1");
		myHClade.put("377767737720771","H3");
		myHClade.put("377767777760731","T2");
		myHClade.put("377767777760761","T1");
		myHClade.put("377767777760771","T1");
		myHClade.put("377773775760771","T1");
		myHClade.put("377774077560771","T1");
		myHClade.put("377774077760771","T1");
		myHClade.put("377774374020771","H1");
		myHClade.put("377776757413371","EAI6-BGD1");
		myHClade.put("377776777760571","X1");
		myHClade.put("377776777760601","X2");
		myHClade.put("377776777760711","X1");
		myHClade.put("377776777760771","X1");
		myHClade.put("377777000413771","EAI5");
		myHClade.put("377777037720771","H3");
		myHClade.put("377777047341771","Unknown");
		myHClade.put("377777364020771","H1");
		myHClade.put("377777374413771","EAI4-VNM");
		myHClade.put("377777377000371","Unknown");
		myHClade.put("377777377413731","EAI1-SOM");
		myHClade.put("377777377760771","T4");
		myHClade.put("377777400000771","Unknown");
		myHClade.put("377777407760771","LAM9");
		myHClade.put("377777423560771","T1");
		myHClade.put("377777477760771","T-H37Rv");
		myHClade.put("377777600000000","Unknown");
		myHClade.put("377777600160731","T2");
		myHClade.put("377777605660771","LAM9");
		myHClade.put("377777606000031","Unknown");
		myHClade.put("377777606060751","LAM11-ZWE");
		myHClade.put("377777606060771","LAM11-ZWE");
		myHClade.put("377777607020771","H3");
		myHClade.put("377777607460731","LAM4");
		myHClade.put("377777607560771","LAM6");
		myHClade.put("377777607620771","H3");
		myHClade.put("377777607660771","LAM9");
		myHClade.put("377777607740371","LAM9");
		myHClade.put("377777607760400","LAM4");
		myHClade.put("377777607760570","LAM9");
		myHClade.put("377777607760571","LAM9");
		myHClade.put("377777607760731","LAM4");
		myHClade.put("377777607760771","LAM9");
		myHClade.put("377777637760771","T1");
		myHClade.put("377777677760671","T1");
		myHClade.put("377777677760771","T1");
		myHClade.put("377777703770351","Unknown");
		myHClade.put("377777707760731","T2");
		myHClade.put("377777707760771","T1");
		myHClade.put("377777737560771","T1");
		myHClade.put("377777737720771","H3");
		myHClade.put("377777737760771","T1");
		myHClade.put("377777743760771","Cameroon");
		myHClade.put("377777747413771","EAI6-BGD1");
		myHClade.put("377777747760771","T1");
		myHClade.put("377777757760771","T5");
		myHClade.put("377777760000000","Unknown");
		myHClade.put("377777760700000","Unknown");
		myHClade.put("377777763660771","T1");
		myHClade.put("377777763760771","T1");
		myHClade.put("377777767760771","T1");
		myHClade.put("377777770000000","Unknown");
		myHClade.put("377777770020731","H3");
		myHClade.put("377777770060771","T1");
		myHClade.put("377777774020731","H1");
		myHClade.put("377777774020771","H1");
		myHClade.put("377777775760771","T1");
		myHClade.put("377777776000771","Unknown");
		myHClade.put("377777776760771","T1");
		myHClade.put("377777777000031","Unknown");
		myHClade.put("377777777413700","EAI5");
		myHClade.put("377777777413731","EAI1-SOM");
		myHClade.put("377777777560731","T2");
		myHClade.put("377777777607771","Unknown");
		myHClade.put("377777777660731","T2");
		myHClade.put("377777777660760","T1");
		myHClade.put("377777777700000","Unknown");
		myHClade.put("377777777720731","H3");
		myHClade.put("377777777720771","H3");
		myHClade.put("377777777740071","T1");
		myHClade.put("377777777740331","T2");
		myHClade.put("377777777740771","T1");
		myHClade.put("377777777741771","Unknown");
		myHClade.put("377777777760571","T1");
		myHClade.put("377777777760601","T1");
		myHClade.put("377777777760700","T1");
		myHClade.put("377777777760731","T2");
		myHClade.put("377777777760760","T1");
		myHClade.put("377777777760771","T1");
		myHClade.put("377777777763771","Manu2");
		myHClade.put("400000000000000","Unknown");
		myHClade.put("400000000000001","Unknown");
		myHClade.put("400000000000011","Unknown");
		myHClade.put("400000000000031","Unknown");
		myHClade.put("400000000002771","Unknown");
		myHClade.put("400000000003771","Unknown");
		myHClade.put("400000000020771","H3");
		myHClade.put("400000000360771","T1");
		myHClade.put("400000000403771","Unknown");
		myHClade.put("400000004160771","LAM3");
		myHClade.put("400000004760771","Turkey");
		myHClade.put("400000007760771","LAM3");
		myHClade.put("400000010020771","H3");
		myHClade.put("400000017777600","BOV");
		myHClade.put("400000020000001","Unknown");
		myHClade.put("400000037760731","T2");
		myHClade.put("400000040003771","Unknown");
		myHClade.put("400000071760571","Unknown");
		myHClade.put("400000074037600","BOV");
		myHClade.put("400000077413671","EAI5");
		myHClade.put("400000077477600","BOV");
		myHClade.put("400000077776600","BOV");
		myHClade.put("400000077777400","BOV");
		myHClade.put("400000077777600","BOV");
		myHClade.put("400000757560771","Unknown");
		myHClade.put("400000757760771","T5");
		myHClade.put("400000777777600","BOV");
		myHClade.put("400001777760771","T1");
		myHClade.put("400003436742200","BOV");
		myHClade.put("400003607760731","LAM3");
		myHClade.put("400003743760771","Cameroon");
		myHClade.put("400003777413771","EAI5");
		myHClade.put("400007036000061","AFRI");
		myHClade.put("400007400413771","EAI5");
		myHClade.put("400007637413671","EAI5");
		myHClade.put("400007777403071","AFRI");
		myHClade.put("400007777413671","EAI5");
		myHClade.put("400017600560731","T2");
		myHClade.put("400017777413071","EAI3-IND");
		myHClade.put("400017777413771","EAI5");
		myHClade.put("400020077413771","EAI5");
		myHClade.put("400020277413071","EAI3-IND");
		myHClade.put("400023743760771","Cameroon");
		myHClade.put("400027777413471","EAI5");
		myHClade.put("400027777413671","EAI5");
		myHClade.put("400027777413771","EAI5");
		myHClade.put("400034037413771","EAI5");
		myHClade.put("400036777413771","EAI5");
		myHClade.put("400037001413771","EAI5");
		myHClade.put("400037377613771","Manu1");
		myHClade.put("400037557413771","EAI6-BGD1");
		myHClade.put("400037607760771","LAM3");
		myHClade.put("400037777403771","Unknown");
		myHClade.put("400037777405771","Unknown");
		myHClade.put("400037777413671","EAI5");
		myHClade.put("400037777413700","EAI5");
		myHClade.put("400037777413711","EAI5");
		myHClade.put("400037777413731","EAI1-SOM");
		myHClade.put("400037777413741","EAI5");
		myHClade.put("400037777413760","EAI5");
		myHClade.put("400037777413761","EAI5");
		myHClade.put("400037777413770","EAI5");
		myHClade.put("400037777413771","EAI5");
		myHClade.put("400037777433771","Manu1");
		myHClade.put("400037777513771","Manu1");
		myHClade.put("400037777613771","Manu1");
		myHClade.put("400040007760771","LAM3");
		myHClade.put("400060000000771","Unknown");
		myHClade.put("400067777760671","T1");
		myHClade.put("400077777413071","EAI3-IND");
		myHClade.put("400177606060771","LAM11-ZWE");
		myHClade.put("400177777413071","EAI3-IND");
		myHClade.put("400177777760771","T1");
		myHClade.put("400343436742200","BOV");
		myHClade.put("400367777740771","T1");
		myHClade.put("400374000000000","BOV");
		myHClade.put("400377607560771","LAM6");
		myHClade.put("400377740003771","Unknown");
		myHClade.put("400773777777600","BOV");
		myHClade.put("400777777620731","H3");
		myHClade.put("401767777413071","EAI3-IND");
		myHClade.put("401776777413071","EAI3-IND");
		myHClade.put("401776777760601","X2");
		myHClade.put("401777777410071","EAI5");
		myHClade.put("401777777413031","EAI5");
		myHClade.put("401777777413071","EAI3-IND");
		myHClade.put("401777777413771","EAI5");
		myHClade.put("402000007760701","LAM3");
		myHClade.put("402767777413771","EAI5");
		myHClade.put("402773777757600","BOV");
		myHClade.put("402777607760731","LAM");
		myHClade.put("403377777403071","Unknown");
		myHClade.put("403567740001320","Unknown");
		myHClade.put("403637757413771","EAI6-BGD1");
		myHClade.put("403737636040401","T3");
		myHClade.put("403767400001771","Unknown");
		myHClade.put("403767777413071","EAI3-IND");
		myHClade.put("403777377413771","EAI5");
		myHClade.put("403777477413771","EAI5");
		myHClade.put("403777702000371","Unknown");
		myHClade.put("403777740000000","Unknown");
		myHClade.put("403777740003471","Unknown");
		myHClade.put("403777740003771","Unknown");
		myHClade.put("403777777720771","H3");
		myHClade.put("403777777760771","T1");
		myHClade.put("406773777773600","BOV");
		myHClade.put("407757607760771","LAM1");
		myHClade.put("407760007760771","LAM-RUS");
		myHClade.put("407760077660771","T1");
		myHClade.put("407777606000031","Unknown");
		myHClade.put("407777607760770","LAM1");
		myHClade.put("407777607760771","LAM1");
		myHClade.put("407777740003171","Unknown");
		myHClade.put("407777777413071","EAI3-IND");
		myHClade.put("407777777760771","T1");
		myHClade.put("410222000000031","Unknown");
		myHClade.put("414007004600001","Unknown");
		myHClade.put("414023417403431","Unknown");
		myHClade.put("414203430400061","AFRI");
		myHClade.put("414203623442000","BOV");
		myHClade.put("414343436642200","BOV");
		myHClade.put("414343436742200","BOV");
		myHClade.put("414343634646200","BOV");
		myHClade.put("414363777777600","BOV");
		myHClade.put("414367777413771","EAI5");
		myHClade.put("415347434640261","T1");
		myHClade.put("416163400000171","Unknown");
		myHClade.put("416177607760751","LAM3");
		myHClade.put("416773677563600","BOV");
		myHClade.put("416773677777200","BOV");
		myHClade.put("416773677777600","BOV");
		myHClade.put("416773777000600","BOV");
		myHClade.put("416773777777000","BOV");
		myHClade.put("416773777777200","BOV");
		myHClade.put("416773777777600","BOV");
		myHClade.put("417136777760731","X1");
		myHClade.put("417176777760731","X1");
		myHClade.put("417177777760731","T2");
		myHClade.put("417177777760771","T1");
		myHClade.put("417336777760731","X1");
		myHClade.put("417763607760771","LAM1");
		myHClade.put("417777377420771","Ural-2");
		myHClade.put("417777377760771","T4");
		myHClade.put("417777437402071","Unknown");
		myHClade.put("417777607760771","LAM1");
		myHClade.put("417777677760771","T1");
		myHClade.put("417777757413371","EAI6-BGD1");
		myHClade.put("417777777403071","Unknown");
		myHClade.put("417777777413011","EAI5");
		myHClade.put("417777777413071","EAI3-IND");
		myHClade.put("417777777413771","EAI5");
		myHClade.put("417777777620771","H3");
		myHClade.put("417777777660771","T1");
		myHClade.put("417777777720731","H3");
		myHClade.put("417777777720771","H3");
		myHClade.put("417777777760700","T1");
		myHClade.put("417777777760771","T1");
		myHClade.put("424043636746600","BOV");
		myHClade.put("424061777777600","BOV");
		myHClade.put("430001077777600","BOV");
		myHClade.put("434161777077600","BOV");
		myHClade.put("434763676777600","BOV");
		myHClade.put("434777747420771","Ural-2");
		myHClade.put("435247676403070","Unknown");
		myHClade.put("435773037604771","Unknown");
		myHClade.put("436375777413731","EAI1-SOM");
		myHClade.put("436761637777600","BOV");
		myHClade.put("436763676777600","BOV");
		myHClade.put("436773437777600","BOV");
		myHClade.put("436773674777600","BOV");
		myHClade.put("436773676777600","BOV");
		myHClade.put("436773677174600","BOV");
		myHClade.put("436773677477600","BOV");
		myHClade.put("436773677774600","BOV");
		myHClade.put("436773677777600","BOV");
		myHClade.put("437677770020771","H3");
		myHClade.put("437723777760771","T3");
		myHClade.put("437727777760771","T3");
		myHClade.put("437737377413371","EAI5");
		myHClade.put("437737607760771","LAM2");
		myHClade.put("437737777420771","Ural-2");
		myHClade.put("437737777760731","T");
		myHClade.put("437737777760771","T3");
		myHClade.put("437760377410771","EAI5");
		myHClade.put("437764777540730","T2-Uganda");
		myHClade.put("437767777720771","H3");
		myHClade.put("437767777760731","T2");
		myHClade.put("437767777760771","T1");
		myHClade.put("437774777760730","T2-Uganda");
		myHClade.put("437776077411171","EAI5");
		myHClade.put("437776777760401","X2");
		myHClade.put("437776777760601","X2");
		myHClade.put("437777007720731","H3");
		myHClade.put("437777037760771","T1");
		myHClade.put("437777277413771","EAI5");
		myHClade.put("437777407760771","LAM1");
		myHClade.put("437777606060600","LAM11-ZWE");
		myHClade.put("437777606060771","LAM11-ZWE");
		myHClade.put("437777607760731","LAM");
		myHClade.put("437777607760771","LAM1");
		myHClade.put("437777647763771","Manu2");
		myHClade.put("437777677760771","T1");
		myHClade.put("437777743760771","Cameroon");
		myHClade.put("437777774020771","H1");
		myHClade.put("437777774320731","H3");
		myHClade.put("437777777400171","Unknown");
		myHClade.put("437777777413771","EAI5");
		myHClade.put("437777777423771","Manu2");
		myHClade.put("437777777720731","H3");
		myHClade.put("437777777720771","H3");
		myHClade.put("437777777721771","Unknown");
		myHClade.put("437777777760011","T1");
		myHClade.put("437777777760601","T1");
		myHClade.put("437777777760771","T1");
		myHClade.put("440003767777600","BOV");
		myHClade.put("440003777677600","BOV");
		myHClade.put("440003777777600","BOV");
		myHClade.put("440013777777600","BOV");
		myHClade.put("440060000000771","Unknown");
		myHClade.put("441737777761771","Unknown");
		myHClade.put("442343677776200","BOV");
		myHClade.put("443777777413071","EAI3-IND");
		myHClade.put("446773777767600","BOV");
		myHClade.put("454000003413661","EAI5");
		myHClade.put("454363637646200","BOV");
		myHClade.put("454377777413771","EAI5");
		myHClade.put("454573777777600","BOV");
		myHClade.put("455307407640071","LAM2");
		myHClade.put("455367776403071","Unknown");
		myHClade.put("456763637743600","BOV");
		myHClade.put("456773677777600","BOV");
		myHClade.put("457001777413771","EAI5");
		myHClade.put("457347607760671","LAM1");
		myHClade.put("457727607740771","LAM2");
		myHClade.put("457727607760761","LAM2");
		myHClade.put("457733577760771","T3");
		myHClade.put("457767621740771","T1");
		myHClade.put("457767637403071","Unknown");
		myHClade.put("457767637413731","EAI1-SOM");
		myHClade.put("457767777413071","EAI3-IND");
		myHClade.put("457767777413771","EAI5");
		myHClade.put("457777407760771","LAM1");
		myHClade.put("457777777413031","EAI5");
		myHClade.put("457777777413771","EAI5");
		myHClade.put("457777777420771","Ural-2");
		myHClade.put("457777777760731","T2");
		myHClade.put("457777777760771","T1");
		myHClade.put("460000000000071","Unknown");
		myHClade.put("460000000000371","Unknown");
		myHClade.put("460023777413771","EAI5");
		myHClade.put("460037037413771","EAI5");
		myHClade.put("460060000000371","Unknown");
		myHClade.put("460060000000771","Unknown");
		myHClade.put("460376775720470","H3");
		myHClade.put("460377777720771","H3");
		myHClade.put("460777777413071","EAI3-IND");
		myHClade.put("461777777413771","EAI5");
		myHClade.put("463777777413071","EAI3-IND");
		myHClade.put("464040777777600","BOV");
		myHClade.put("464043676742200","BOV");
		myHClade.put("464043677777600","BOV");
		myHClade.put("464043777742200","BOV");
		myHClade.put("464063777567600","BOV");
		myHClade.put("464063777577600","BOV");
		myHClade.put("464063777700600","BOV");
		myHClade.put("464063777774600","BOV");
		myHClade.put("464063777777600","BOV");
		myHClade.put("464072177777600","BOV");
		myHClade.put("464073777777600","BOV");
		myHClade.put("464223777774600","BOV");
		myHClade.put("464743677767600","BOV");
		myHClade.put("466177400000171","Unknown");
		myHClade.put("466773677777600","BOV");
		myHClade.put("466773777677600","BOV");
		myHClade.put("467377777413771","EAI5");
		myHClade.put("467400777413071","EAI3-IND");
		myHClade.put("467677777413071","EAI3-IND");
		myHClade.put("467767677403031","Unknown");
		myHClade.put("467777277413771","EAI8-MDG");
		myHClade.put("467777377413771","EAI8-MDG");
		myHClade.put("467777774000071","Unknown");
		myHClade.put("467777777413031","EAI5");
		myHClade.put("467777777413071","EAI3-IND");
		myHClade.put("467777777413771","EAI5");
		myHClade.put("467777777760471","T1");
		myHClade.put("467777777760711","T1");
		myHClade.put("470000377413031","EAI5");
		myHClade.put("470000777760771","T1-RUS2");
		myHClade.put("470377747413771","EAI6-BGD1");
		myHClade.put("470377777413771","EAI5");
		myHClade.put("470777777777671","AFRI_1");
		myHClade.put("471777777413071","EAI3-IND");
		myHClade.put("471777777760631","T1");
		myHClade.put("472733777777600","BOV");
		myHClade.put("472763677773600","BOV");
		myHClade.put("473144404620000","H3");
		myHClade.put("473776077411771","EAI5");
		myHClade.put("473777077011771","EAI5");
		myHClade.put("473777077413771","EAI5");
		myHClade.put("473777377403771","Unknown");
		myHClade.put("473777377413771","EAI8-MDG");
		myHClade.put("473777776013771","EAI5");
		myHClade.put("473777777413070","EAI3-IND");
		myHClade.put("473777777413071","EAI3-IND");
		myHClade.put("474000001413671","EAI5");
		myHClade.put("474000003413771","EAI5");
		myHClade.put("474000077413771","EAI5");
		myHClade.put("474000377413031","EAI3-IND");
		myHClade.put("474001377413771","EAI8-MDG");
		myHClade.put("474003777413071","EAI3-IND");
		myHClade.put("474007777413071","EAI3-IND");
		myHClade.put("474017777413071","EAI3-IND");
		myHClade.put("474023777413071","EAI3-IND");
		myHClade.put("474037777413071","EAI3-IND");
		myHClade.put("474053777777600","BOV");
		myHClade.put("474063777777600","BOV");
		myHClade.put("474077607177031","Unknown");
		myHClade.put("474077607177071","AFRI_2");
		myHClade.put("474077777403031","Unknown");
		myHClade.put("474177777413771","EAI5");
		myHClade.put("474177777613771","Manu1");
		myHClade.put("474360377413771","EAI5");
		myHClade.put("474361777413771","EAI5");
		myHClade.put("474367767413771","EAI5");
		myHClade.put("474367777413771","EAI5");
		myHClade.put("474377377413771","EAI8-MDG");
		myHClade.put("474377707413771","EAI5");
		myHClade.put("474377767413771","EAI5");
		myHClade.put("474377774000771","Unknown");
		myHClade.put("474377774413771","EAI4-VNM");
		myHClade.put("474377777412771","EAI5");
		myHClade.put("474377777413671","EAI5");
		myHClade.put("474377777413731","EAI1-SOM");
		myHClade.put("474377777413771","EAI5");
		myHClade.put("474377777773771","Manu1");
		myHClade.put("474563777777600","BOV");
		myHClade.put("474773674177600","BOV");
		myHClade.put("475347777403621","Unknown");
		myHClade.put("475367677003071","Unknown");
		myHClade.put("475740002000071","Unknown");
		myHClade.put("475747637403071","Unknown");
		myHClade.put("475777677413771","EAI5");
		myHClade.put("475777774000031","Unknown");
		myHClade.put("475777777413071","EAI3-IND");
		myHClade.put("475777777413771","EAI5");
		myHClade.put("476000000000771","Unknown");
		myHClade.put("476000377413071","EAI3-IND");
		myHClade.put("476003477413771","EAI5");
		myHClade.put("476037777413071","EAI3-IND");
		myHClade.put("476077774020771","H1");
		myHClade.put("476173677777200","BOV");
		myHClade.put("476177400000171","Unknown");
		myHClade.put("476177400010171","EAI5");
		myHClade.put("476177607760771","LAM3");
		myHClade.put("476323476777600","BOV");
		myHClade.put("476343777773600","BOV");
		myHClade.put("476363636756200","BOV");
		myHClade.put("476367737760731","S");
		myHClade.put("476377737760771","S");
		myHClade.put("476377774000371","Unknown");
		myHClade.put("476377777720771","H3");
		myHClade.put("476377777760771","S");
		myHClade.put("476573677777200","BOV");
		myHClade.put("476573777777600","BOV");
		myHClade.put("476743677777600","BOV");
		myHClade.put("476743777777600","BOV");
		myHClade.put("476763677742600","BOV");
		myHClade.put("476763677777200","BOV");
		myHClade.put("476763677777600","BOV");
		myHClade.put("476763777777600","BOV");
		myHClade.put("476767777413771","EAI5");
		myHClade.put("476773674177600","BOV");
		myHClade.put("476773676037200","BOV");
		myHClade.put("476773677776600","BOV");
		myHClade.put("476773677777000","BOV");
		myHClade.put("476773677777200","BOV");
		myHClade.put("476773677777400","BOV");
		myHClade.put("476773677777600","BOV");
		myHClade.put("476773757747600","BOV");
		myHClade.put("476773777177600","BOV");
		myHClade.put("476773777377200","BOV");
		myHClade.put("476773777617600","BOV");
		myHClade.put("476773777677600","BOV");
		myHClade.put("476773777743600","BOV");
		myHClade.put("476773777777600","BOV");
		myHClade.put("476773777777771","Unknown");
		myHClade.put("476777777413031","EAI5");
		myHClade.put("476777777413771","EAI5");
		myHClade.put("477001777413771","EAI5");
		myHClade.put("477017777413071","EAI3-IND");
		myHClade.put("477061777413071","EAI3-IND");
		myHClade.put("477067777760771","T1");
		myHClade.put("477077777760771","T1");
		myHClade.put("477347277413671","EAI8-MDG");
		myHClade.put("477347607760661","LAM1");
		myHClade.put("477347777413271","EAI5");
		myHClade.put("477357277413771","EAI8-MDG");
		myHClade.put("477367036003071","Unknown");
		myHClade.put("477377077413671","EAI5");
		myHClade.put("477377707760731","T2");
		myHClade.put("477377777413071","EAI3-IND");
		myHClade.put("477377777730771","Manu3");
		myHClade.put("477377777760731","T2");
		myHClade.put("477407777413071","EAI3-IND");
		myHClade.put("477407777413771","EAI5");
		myHClade.put("477417760000771","Unknown");
		myHClade.put("477417777413071","EAI3-IND");
		myHClade.put("477437777413771","EAI5");
		myHClade.put("477467777413071","EAI3-IND");
		myHClade.put("477477777413071","EAI3-IND");
		myHClade.put("477477777760731","T2");
		myHClade.put("477567777413071","EAI3-IND");
		myHClade.put("477576770000000","Unknown");
		myHClade.put("477577777413071","EAI3-IND");
		myHClade.put("477577777720771","H3");
		myHClade.put("477606317413071","EAI3-IND");
		myHClade.put("477617607760771","LAM2");
		myHClade.put("477617677760771","T1");
		myHClade.put("477623030000071","Unknown");
		myHClade.put("477623760003031","Unknown");
		myHClade.put("477637377417771","Unknown");
		myHClade.put("477673777410771","EAI5");
		myHClade.put("477677376003741","Unknown");
		myHClade.put("477677376203741","Unknown");
		myHClade.put("477677610020771","H3");
		myHClade.put("477677777410771","EAI5");
		myHClade.put("477677777720771","H3");
		myHClade.put("477700204020771","H1");
		myHClade.put("477701777413071","EAI3-IND");
		myHClade.put("477701777413771","EAI5");
		myHClade.put("477703777413751","EAI5");
		myHClade.put("477703777760731","T2");
		myHClade.put("477707077403771","Unknown");
		myHClade.put("477707377413771","EAI8-MDG");
		myHClade.put("477707477413771","EAI5");
		myHClade.put("477717777413071","EAI3-IND");
		myHClade.put("477720017760771","T-tuscany");
		myHClade.put("477723777413071","EAI3-IND");
		myHClade.put("477727607760771","LAM2");
		myHClade.put("477737357403771","Unknown");
		myHClade.put("477737377403771","Unknown");
		myHClade.put("477737377417771","Unknown");
		myHClade.put("477737477413071","EAI3-IND");
		myHClade.put("477737607760731","LAM2");
		myHClade.put("477737607760751","LAM2");
		myHClade.put("477737607760771","LAM2");
		myHClade.put("477737607763771","Manu2");
		myHClade.put("477737637760711","T3");
		myHClade.put("477737637760771","T3");
		myHClade.put("477737737413771","EAI5");
		myHClade.put("477737737760771","T3");
		myHClade.put("477737777413071","EAI3-IND");
		myHClade.put("477737777413771","EAI5");
		myHClade.put("477737777760731","T");
		myHClade.put("477740001760771","T1");
		myHClade.put("477740037413771","EAI5");
		myHClade.put("477747377413771","EAI8-MDG");
		myHClade.put("477747777413071","EAI3-IND");
		myHClade.put("477757277410771","EAI8-MDG");
		myHClade.put("477757377413751","EAI8-MDG");
		myHClade.put("477757377413771","EAI8-MDG");
		myHClade.put("477757777013071","EAI3-IND");
		myHClade.put("477757777413071","EAI3-IND");
		myHClade.put("477760017413071","EAI3-IND");
		myHClade.put("477760777413071","EAI3-IND");
		myHClade.put("477763777413001","EAI5");
		myHClade.put("477763777413071","EAI3-IND");
		myHClade.put("477767037413771","EAI5");
		myHClade.put("477767277413771","EAI8-MDG");
		myHClade.put("477767376000071","Unknown");
		myHClade.put("477767377410771","EAI8-MDG");
		myHClade.put("477767377413671","EAI8-MDG");
		myHClade.put("477767377413771","EAI8-MDG");
		myHClade.put("477767377613771","Manu1");
		myHClade.put("477767477413771","EAI5");
		myHClade.put("477767607760771","LAM1");
		myHClade.put("477767677413771","EAI5");
		myHClade.put("477767740003071","Unknown");
		myHClade.put("477767774000771","Unknown");
		myHClade.put("477767777403001","Unknown");
		myHClade.put("477767777403671","Unknown");
		myHClade.put("477767777413071","EAI3-IND");
		myHClade.put("477767777720731","H3");
		myHClade.put("477770377413071","EAI3-IND");
		myHClade.put("477771377413671","EAI8-MDG");
		myHClade.put("477771777413031","EAI5");
		myHClade.put("477773777413071","EAI3-IND");
		myHClade.put("477773777413760","EAI5");
		myHClade.put("477773777413771","EAI5");
		myHClade.put("477774200013071","EAI3-IND");
		myHClade.put("477774374020771","H1");
		myHClade.put("477774377413031","EAI5");
		myHClade.put("477774377413071","EAI3-IND");
		myHClade.put("477774777413071","EAI3-IND");
		myHClade.put("477774777413571","EAI5");
		myHClade.put("477774777413771","EAI5");
		myHClade.put("477775202013071","EAI3-IND");
		myHClade.put("477775342413071","EAI3-IND");
		myHClade.put("477775360013071","EAI3-IND");
		myHClade.put("477775374020771","H1");
		myHClade.put("477775377413071","EAI3-IND");
		myHClade.put("477775777403771","Unknown");
		myHClade.put("477775777413071","EAI3-IND");
		myHClade.put("477775777720771","H3");
		myHClade.put("477775777760771","T1");
		myHClade.put("477776037413071","EAI3-IND");
		myHClade.put("477776077411171","EAI3-IND");
		myHClade.put("477776077411771","EAI5");
		myHClade.put("477776077413771","EAI5");
		myHClade.put("477776374013771","EAI5");
		myHClade.put("477776377413071","EAI3-IND");
		myHClade.put("477776377413771","EAI5");
		myHClade.put("477776471413771","EAI5");
		myHClade.put("477776630000000","Unknown");
		myHClade.put("477776770000000","Unknown");
		myHClade.put("477776774020771","H1");
		myHClade.put("477776777412771","EAI5");
		myHClade.put("477776777413031","EAI5");
		myHClade.put("477777003413571","EAI5");
		myHClade.put("477777003413771","EAI5");
		myHClade.put("477777037413071","EAI3-IND");
		myHClade.put("477777077403771","Unknown");
		myHClade.put("477777077411171","EAI5");
		myHClade.put("477777077411771","EAI5");
		myHClade.put("477777077413771","EAI5");
		myHClade.put("477777177413071","EAI3-IND");
		myHClade.put("477777277005771","Unknown");
		myHClade.put("477777277400771","Unknown");
		myHClade.put("477777277403771","Unknown");
		myHClade.put("477777277410771","EAI8-MDG");
		myHClade.put("477777277413571","EAI8-MDG");
		myHClade.put("477777277413671","EAI8-MDG");
		myHClade.put("477777277413771","EAI8-MDG");
		myHClade.put("477777277613771","Manu1");
		myHClade.put("477777357413771","EAI8-MDG");
		myHClade.put("477777374003771","Unknown");
		myHClade.put("477777376013771","EAI8-MDG");
		myHClade.put("477777377013771","EAI8-MDG");
		myHClade.put("477777377403771","Unknown");
		myHClade.put("477777377410771","EAI8-MDG");
		myHClade.put("477777377413071","EAI3-IND");
		myHClade.put("477777377413711","EAI8-MDG");
		myHClade.put("477777377413731","EAI8-MDG");
		myHClade.put("477777377413741","EAI8-MDG");
		myHClade.put("477777377413771","EAI8-MDG");
		myHClade.put("477777377417771","Unknown");
		myHClade.put("477777377613071","Manu1");
		myHClade.put("477777402013071","EAI3-IND");
		myHClade.put("477777437413771","EAI5");
		myHClade.put("477777475413470","EAI5");
		myHClade.put("477777477403701","Unknown");
		myHClade.put("477777477410771","EAI5");
		myHClade.put("477777477413071","EAI3-IND");
		myHClade.put("477777477413671","EAI5");
		myHClade.put("477777477413751","EAI5");
		myHClade.put("477777477413771","EAI5");
		myHClade.put("477777577760771","T1");
		myHClade.put("477777604020771","H1");
		myHClade.put("477777606060771","LAM11-ZWE");
		myHClade.put("477777607760771","LAM1");
		myHClade.put("477777637413071","EAI3-IND");
		myHClade.put("477777637760771","T1");
		myHClade.put("477777661760371","T");
		myHClade.put("477777661760771","Unknown");
		myHClade.put("477777677413071","EAI3-IND");
		myHClade.put("477777677413771","EAI5");
		myHClade.put("477777717760731","T2");
		myHClade.put("477777737413071","EAI3-IND");
		myHClade.put("477777737760771","T1");
		myHClade.put("477777740000011","Unknown");
		myHClade.put("477777740000260","Unknown");
		myHClade.put("477777740173071","Manu1");
		myHClade.put("477777743760771","Cameroon");
		myHClade.put("477777757413671","EAI6-BGD1");
		myHClade.put("477777757413771","EAI6-BGD1");
		myHClade.put("477777760177071","Unknown");
		myHClade.put("477777764020771","H1");
		myHClade.put("477777767013071","EAI3-IND");
		myHClade.put("477777767413071","EAI3-IND");
		myHClade.put("477777767760771","T1");
		myHClade.put("477777770000000","Unknown");
		myHClade.put("477777770000001","Unknown");
		myHClade.put("477777770020771","H3");
		myHClade.put("477777770413071","EAI3-IND");
		myHClade.put("477777771413071","EAI3-IND");
		myHClade.put("477777774000061","Unknown");
		myHClade.put("477777774000071","Unknown");
		myHClade.put("477777774013771","EAI5");
		myHClade.put("477777774020771","H1");
		myHClade.put("477777774137771","Unknown");
		myHClade.put("477777774605431","Unknown");
		myHClade.put("477777775413031","EAI5");
		myHClade.put("477777775413051","EAI3-IND");
		myHClade.put("477777775413071","EAI3-IND");
		myHClade.put("477777775413671","EAI5");
		myHClade.put("477777775413760","EAI5");
		myHClade.put("477777776000071","Unknown");
		myHClade.put("477777776013071","EAI3-IND");
		myHClade.put("477777776413071","EAI3-IND");
		myHClade.put("477777776413771","EAI5");
		myHClade.put("477777777000000","Unknown");
		myHClade.put("477777777000071","Unknown");
		myHClade.put("477777777003171","Unknown");
		myHClade.put("477777777013071","EAI3-IND");
		myHClade.put("477777777013771","EAI5");
		myHClade.put("477777777020771","H3");
		myHClade.put("477777777400001","Unknown");
		myHClade.put("477777777400771","Unknown");
		myHClade.put("477777777401071","Unknown");
		myHClade.put("477777777401771","Unknown");
		myHClade.put("477777777403071","EAI3-IND");
		myHClade.put("477777777403771","Unknown");
		myHClade.put("477777777410031","EAI5");
		myHClade.put("477777777410071","EAI5");
		myHClade.put("477777777410171","EAI5");
		myHClade.put("477777777410771","EAI5");
		myHClade.put("477777777411071","EAI3-IND");
		myHClade.put("477777777411770","EAI5");
		myHClade.put("477777777411771","EAI5");
		myHClade.put("477777777412001","EAI5");
		myHClade.put("477777777412071","EAI5");
		myHClade.put("477777777412700","EAI5");
		myHClade.put("477777777412771","EAI5");
		myHClade.put("477777777413001","EAI5");
		myHClade.put("477777777413011","EAI5");
		myHClade.put("477777777413030","EAI5");
		myHClade.put("477777777413031","EAI3-IND");
		myHClade.put("477777777413051","EAI3-IND");
		myHClade.put("477777777413060","EAI3-IND");
		myHClade.put("477777777413061","EAI3-IND");
		myHClade.put("477777777413071","EAI3-IND");
		myHClade.put("477777777413171","EAI5");
		myHClade.put("477777777413371","EAI5");
		myHClade.put("477777777413471","EAI5");
		myHClade.put("477777777413671","EAI5");
		myHClade.put("477777777413700","EAI5");
		myHClade.put("477777777413701","EAI5");
		myHClade.put("477777777413731","EAI1-SOM");
		myHClade.put("477777777413761","EAI5");
		myHClade.put("477777777413771","EAI5");
		myHClade.put("477777777420711","Ural-2");
		myHClade.put("477777777420771","Ural-2");
		myHClade.put("477777777423771","Manu2");
		myHClade.put("477777777426071","Unknown");
		myHClade.put("477777777433071","Manu1");
		myHClade.put("477777777453071","Manu1");
		myHClade.put("477777777513471","Manu1");
		myHClade.put("477777777613071","Manu1");
		myHClade.put("477777777720771","H3");
		myHClade.put("477777777730771","Manu3");
		myHClade.put("477777777760011","T1");
		myHClade.put("477777777760631","T1");
		myHClade.put("477777777760731","T2");
		myHClade.put("477777777760771","T1");
		myHClade.put("477777777763771","Manu2");
		myHClade.put("477777777773771","Manu1");
		myHClade.put("500000000000000","CAS");
		myHClade.put("500000000003771","CAS");
		myHClade.put("500000004000771","Unknown");
		myHClade.put("500000004020771","H1");
		myHClade.put("500000740003671","CAS2");
		myHClade.put("500003740003771","CAS2");
		myHClade.put("500006636540621","X3");
		myHClade.put("500006677560671","X3");
		myHClade.put("500016477740671","X3");
		myHClade.put("500036777760771","X3");
		myHClade.put("500044674740671","T1");
		myHClade.put("500046637740661","X3");
		myHClade.put("500066477760611","X3");
		myHClade.put("500066777740671","X3");
		myHClade.put("500066777760771","X3");
		myHClade.put("500076000000000","CAS");
		myHClade.put("500076777560771","X3");
		myHClade.put("500076777760771","X3");
		myHClade.put("500077777760771","T1");
		myHClade.put("500141016564671","AFRI");
		myHClade.put("500173760003771","Unknown");
		myHClade.put("500207000003671","CAS");
		myHClade.put("500367640003671","CAS2");
		myHClade.put("500367740003771","CAS2");
		myHClade.put("500377600003771","CAS");
		myHClade.put("500377740003771","CAS2");
		myHClade.put("500377777720771","H3");
		myHClade.put("500506000003771","CAS");
		myHClade.put("500740007774671","AFRI");
		myHClade.put("500757677760571","T1");
		myHClade.put("500771744020731","H1");
		myHClade.put("500775747413771","EAI6-BGD1");
		myHClade.put("501347674413671","EAI4-VNM");
		myHClade.put("501347740003661","CAS");
		myHClade.put("501367000003261","CAS");
		myHClade.put("501727740003071","CAS");
		myHClade.put("501767740003071","CAS");
		myHClade.put("501767740003131","CAS");
		myHClade.put("501767740003671","CAS");
		myHClade.put("501777770000001","Unknown");
		myHClade.put("503367400001401","CAS1-Kili");
		myHClade.put("503367400001471","CAS1-Kili");
		myHClade.put("503367400001771","CAS1-Kili");
		myHClade.put("503367740003171","CAS1-Delhi");
		myHClade.put("503377400001771","CAS1-Kili");
		myHClade.put("503377740000371","CAS");
		myHClade.put("503667700003771","CAS");
		myHClade.put("503707700000771","CAS");
		myHClade.put("503767400001771","CAS");
		myHClade.put("503767600003771","CAS");
		myHClade.put("503767640003021","CAS1-Delhi");
		myHClade.put("503767700003771","CAS");
		myHClade.put("503767740000671","CAS");
		myHClade.put("503767740003000","CAS1-Delhi");
		myHClade.put("503767740003171","CAS1-Delhi");
		myHClade.put("503767740003671","CAS1-Delhi");
		myHClade.put("503767740003761","CAS1-Delhi");
		myHClade.put("503767774003771","Unknown");
		myHClade.put("503767777400671","Unknown");
		myHClade.put("503776000003771","CAS");
		myHClade.put("503777000003171","CAS");
		myHClade.put("503777000003771","CAS");
		myHClade.put("503777400003771","CAS");
		myHClade.put("503777700003771","CAS");
		myHClade.put("503777740000171","CAS");
		myHClade.put("503777740000771","CAS");
		myHClade.put("503777740003071","CAS1-Delhi");
		myHClade.put("503777740003171","CAS1-Delhi");
		myHClade.put("503777740003771","CAS1-Delhi");
		myHClade.put("506044034000461","Unknown");
		myHClade.put("507777677413731","EAI1-SOM");
		myHClade.put("507777777420771","Ural-2");
		myHClade.put("510247600003771","Unknown");
		myHClade.put("511367734160600","T1");
		myHClade.put("511760047760371","T1");
		myHClade.put("514132775577741","Unknown");
		myHClade.put("514347776560761","T1");
		myHClade.put("515000236740261","T3-ETH");
		myHClade.put("515047236760661","T4");
		myHClade.put("515047607760661","LAM9");
		myHClade.put("515047636760661","T1");
		myHClade.put("515247004402261","Unknown");
		myHClade.put("515247400640261","T1");
		myHClade.put("515247634160661","T1");
		myHClade.put("515346436640201","X2");
		myHClade.put("515347420040261","T1");
		myHClade.put("515347436640261","T1");
		myHClade.put("515347476763771","Manu2");
		myHClade.put("515347777766271","Unknown");
		myHClade.put("516347777760771","S");
		myHClade.put("517345777756461","Unknown");
		myHClade.put("517346636540201","X2");
		myHClade.put("517347404540621","Turkey");
		myHClade.put("517347606060661","LAM11-ZWE");
		myHClade.put("517347636060661","T1");
		myHClade.put("517367636700771","Unknown");
		myHClade.put("517367637760771","T1");
		myHClade.put("517727403540061","T3");
		myHClade.put("517747637743771","Unknown");
		myHClade.put("517757777760771","T1");
		myHClade.put("517767607760661","LAM9");
		myHClade.put("517767677560771","T1");
		myHClade.put("517767757413771","EAI6-BGD1");
		myHClade.put("517777637760771","T1");
		myHClade.put("517777637763771","Manu2");
		myHClade.put("517777777743771","Unknown");
		myHClade.put("520132417570761","Manu3");
		myHClade.put("530027637560600","T1");
		myHClade.put("530777677763671","AFRI_1");
		myHClade.put("531044224120431","H3");
		myHClade.put("531255676760771","T1");
		myHClade.put("531357657560771","T5");
		myHClade.put("533777347560471","T4-CEU1");
		myHClade.put("534307637760771","T1");
		myHClade.put("534357377750771","Manu3");
		myHClade.put("534357674732671","AFRI");
		myHClade.put("535347777777661","Unknown");
		myHClade.put("535357757403631","Unknown");
		myHClade.put("535743737723771","Manu2");
		myHClade.put("535745777772771","Manu1");
		myHClade.put("535757777776671","Unknown");
		myHClade.put("535767777560671","T1");
		myHClade.put("535767777740731","T2");
		myHClade.put("536177607760771","LAM3");
		myHClade.put("537637777760771","T1");
		myHClade.put("537737777760731","T");
		myHClade.put("537737777760771","T3");
		myHClade.put("537767776760631","T1");
		myHClade.put("537776755413731","EAI");
		myHClade.put("537777757413771","EAI6-BGD1");
		myHClade.put("537777777420771","Ural-2");
		myHClade.put("537777777760711","T1");
		myHClade.put("540177607760771","LAM3");
		myHClade.put("540377777720771","H3");
		myHClade.put("540777757760771","T5");
		myHClade.put("541737777760771","T3-OSA");
		myHClade.put("541777777413071","EAI5");
		myHClade.put("541777777760771","T1");
		myHClade.put("543777607760771","LAM9");
		myHClade.put("547777777600171","Unknown");
		myHClade.put("554347436400261","AFRI");
		myHClade.put("555047607760661","LAM9");
		myHClade.put("555147437740661","T");
		myHClade.put("555147437740771","T1");
		myHClade.put("555307407640071","LAM5");
		myHClade.put("555307437740771","T1");
		myHClade.put("555346437740061","X1");
		myHClade.put("555346477760671","T-H37Rv");
		myHClade.put("555347434000771","Unknown");
		myHClade.put("555347435640271","T1");
		myHClade.put("555347637743771","Unknown");
		myHClade.put("555357637740661","T1");
		myHClade.put("555567607740731","LAM4");
		myHClade.put("555567777740771","T1");
		myHClade.put("555707437000261","Unknown");
		myHClade.put("555727604643071","Unknown");
		myHClade.put("555747420000661","Unknown");
		myHClade.put("555747434000771","Unknown");
		myHClade.put("555767434000061","Unknown");
		myHClade.put("555767637600771","Unknown");
		myHClade.put("555767774000000","Unknown");
		myHClade.put("555767777740771","T1");
		myHClade.put("557163777760671","T1");
		myHClade.put("557346637740261","X1");
		myHClade.put("557347437700771","Unknown");
		myHClade.put("557347607760761","LAM9");
		myHClade.put("557347777760661","T1");
		myHClade.put("557367600740771","T1");
		myHClade.put("557367607740231","LAM4");
		myHClade.put("557367617000761","Unknown");
		myHClade.put("557367677763671","Manu2");
		myHClade.put("557400004020771","H1");
		myHClade.put("557707777000271","Unknown");
		myHClade.put("557727777400771","Unknown");
		myHClade.put("557747606760771","LAM8");
		myHClade.put("557747607740661","LAM9");
		myHClade.put("557747634000771","Unknown");
		myHClade.put("557747637743771","Unknown");
		myHClade.put("557747737000271","Unknown");
		myHClade.put("557757606060771","LAM11-ZWE");
		myHClade.put("557767607540771","LAM9");
		myHClade.put("557767607560771","LAM6");
		myHClade.put("557767607760011","LAM4");
		myHClade.put("557767677760771","T1");
		myHClade.put("557767703760771","T1");
		myHClade.put("557767763660771","T1");
		myHClade.put("557767764000771","Unknown");
		myHClade.put("557767777413700","EAI5");
		myHClade.put("557767777720771","H3");
		myHClade.put("557767777740611","T1");
		myHClade.put("557767777743771","Unknown");
		myHClade.put("557767777763771","Manu2");
		myHClade.put("557777607560771","LAM6");
		myHClade.put("557777607760771","LAM9");
		myHClade.put("557777777420731","Ural-2");
		myHClade.put("557777777740771","T1");
		myHClade.put("557777777760771","T1");
		myHClade.put("560000000700011","Unknown");
		myHClade.put("560000007760771","LAM3");
		myHClade.put("560003777560770","T1");
		myHClade.put("560060000000771","Unknown");
		myHClade.put("560777777560631","T1");
		myHClade.put("566377777760771","S");
		myHClade.put("567737777760771","T3");
		myHClade.put("567777777020171","H3");
		myHClade.put("567777777420771","Ural-2");
		myHClade.put("570000636760601","T1-RUS2");
		myHClade.put("570000675760771","T1-RUS2");
		myHClade.put("570000777760771","T1-RUS2");
		myHClade.put("570040037547071","AFRI");
		myHClade.put("570077777413071","EAI5");
		myHClade.put("570177677740671","T1");
		myHClade.put("570737777777670","AFRI_1");
		myHClade.put("570773777777671","AFRI_1");
		myHClade.put("570777637743671","AFRI_1");
		myHClade.put("570777777600071","AFRI");
		myHClade.put("570777777777271","AFRI_1");
		myHClade.put("570777777777671","AFRI_1");
		myHClade.put("571347677760671","T1");
		myHClade.put("571776770000000","Unknown");
		myHClade.put("571777777760771","T1");
		myHClade.put("573377777420771","Ural-2");
		myHClade.put("573737607760700","LAM");
		myHClade.put("573767777420771","Ural-2");
		myHClade.put("573775757413771","EAI6-BGD1");
		myHClade.put("573777577760771","T1");
		myHClade.put("574003477413771","EAI5");
		myHClade.put("574063670007071","AFRI_3");
		myHClade.put("574066607777071","AFRI_2");
		myHClade.put("574067606767071","AFRI_2");
		myHClade.put("574067607737071","AFRI_2");
		myHClade.put("574067607777071","AFRI_2");
		myHClade.put("574077607177071","AFRI_2");
		myHClade.put("574077607777031","Unknown");
		myHClade.put("574077607777071","AFRI_2");
		myHClade.put("574077607777771","Unknown");
		myHClade.put("574077777777071","AFRI_3");
		myHClade.put("574367375760671","T4");
		myHClade.put("574777777420771","Ural-2");
		myHClade.put("574777777423771","Manu2");
		myHClade.put("574777777760771","T1");
		myHClade.put("575067674000071","Unknown");
		myHClade.put("575167777760671","T1");
		myHClade.put("575207737740771","T1");
		myHClade.put("575347774760661","T1");
		myHClade.put("575367577770671","Manu3");
		myHClade.put("575367777560460","T1");
		myHClade.put("575667635162631","Manu2");
		myHClade.put("575747637700631","Unknown");
		myHClade.put("575761777420771","Ural-2");
		myHClade.put("575766437700771","Unknown");
		myHClade.put("575767737760671","T1");
		myHClade.put("575767777760771","T1");
		myHClade.put("575776347413771","EAI6-BGD1");
		myHClade.put("575777307760771","T4");
		myHClade.put("575777607760771","LAM9");
		myHClade.put("575777707760771","T1");
		myHClade.put("575777777403731","Unknown");
		myHClade.put("575777777413771","EAI5");
		myHClade.put("576167400000171","Unknown");
		myHClade.put("576167605760671","LAM3");
		myHClade.put("576167607740771","LAM3");
		myHClade.put("576177400000171","Unknown");
		myHClade.put("576177417760771","T1");
		myHClade.put("576177607760771","LAM3");
		myHClade.put("576177637000371","Unknown");
		myHClade.put("576177777420771","Ural-2");
		myHClade.put("576337777760771","S");
		myHClade.put("576360777760771","S");
		myHClade.put("576367377660771","S");
		myHClade.put("576377703740731","S");
		myHClade.put("576377767760771","S");
		myHClade.put("576377774300371","Unknown");
		myHClade.put("576377777420771","Ural-2");
		myHClade.put("576377777760771","S");
		myHClade.put("576777777720771","H3");
		myHClade.put("577000007700761","Unknown");
		myHClade.put("577000177760771","T1");
		myHClade.put("577000237760671","T3-ETH");
		myHClade.put("577000377740771","T3-ETH");
		myHClade.put("577000377760771","T3-ETH");
		myHClade.put("577000777760771","T");
		myHClade.put("577002377760771","T3-ETH");
		myHClade.put("577003377720771","H3");
		myHClade.put("577017777420771","Ural-2");
		myHClade.put("577077777420771","Ural-2");
		myHClade.put("577265637740721","T2");
		myHClade.put("577327606560711","LAM6");
		myHClade.put("577347404760471","Turkey");
		myHClade.put("577347477750671","Manu3");
		myHClade.put("577347606060631","LAM11-ZWE");
		myHClade.put("577347774000221","Unknown");
		myHClade.put("577347777000261","Unknown");
		myHClade.put("577347777760671","T1");
		myHClade.put("577347777763661","Manu2");
		myHClade.put("577366737560621","X1");
		myHClade.put("577367056403011","Unknown");
		myHClade.put("577367603740661","T1");
		myHClade.put("577367777403431","Unknown");
		myHClade.put("577377600000171","Unknown");
		myHClade.put("577377774020771","H1");
		myHClade.put("577400000000171","Unknown");
		myHClade.put("577417777760771","T");
		myHClade.put("577437757413771","EAI6-BGD1");
		myHClade.put("577477777420771","Ural-2");
		myHClade.put("577537777760771","T3");
		myHClade.put("577577777413371","EAI5");
		myHClade.put("577577777760711","T1");
		myHClade.put("577600017420771","Ural-2");
		myHClade.put("577601774413771","EAI4-VNM");
		myHClade.put("577603777760700","T1");
		myHClade.put("577637002060771","Unknown");
		myHClade.put("577637417560771","T1");
		myHClade.put("577637677420771","Ural-2");
		myHClade.put("577637777420771","Ural-2");
		myHClade.put("577667777760771","T1");
		myHClade.put("577677077760771","T");
		myHClade.put("577677606060771","LAM11-ZWE");
		myHClade.put("577677777720771","H3");
		myHClade.put("577677777760700","T1");
		myHClade.put("577677777760771","T1");
		myHClade.put("577700777760771","T1");
		myHClade.put("577701777760771","T1");
		myHClade.put("577703607760771","LAM5");
		myHClade.put("577707404760771","Turkey");
		myHClade.put("577707637413661","EAI5");
		myHClade.put("577707677403431","Unknown");
		myHClade.put("577707777760671","T1");
		myHClade.put("577717417560771","T1");
		myHClade.put("577717606060731","LAM11-ZWE");
		myHClade.put("577717777420771","Ural-2");
		myHClade.put("577717777760771","T1");
		myHClade.put("577727437160760","T3");
		myHClade.put("577727674413671","EAI4-VNM");
		myHClade.put("577727777420771","Ural-2");
		myHClade.put("577734777420771","Ural-2");
		myHClade.put("577736777760601","X2");
		myHClade.put("577737377760771","T");
		myHClade.put("577737577760771","T3");
		myHClade.put("577737607760771","LAM5");
		myHClade.put("577737747420771","Ural-2");
		myHClade.put("577737757413771","EAI6-BGD1");
		myHClade.put("577737757760771","T");
		myHClade.put("577737770000771","Unknown");
		myHClade.put("577737774413771","EAI4-VNM");
		myHClade.put("577737775420771","Ural-2");
		myHClade.put("577737777400071","Unknown");
		myHClade.put("577737777420571","Ural-2");
		myHClade.put("577737777420771","Ural-2");
		myHClade.put("577737777520771","H3");
		myHClade.put("577737777720771","H3");
		myHClade.put("577737777760571","T3");
		myHClade.put("577737777760731","T");
		myHClade.put("577737777760761","T3");
		myHClade.put("577737777760771","T3");
		myHClade.put("577737777763761","Manu2");
		myHClade.put("577740517740771","T1");
		myHClade.put("577743607760771","LAM9");
		myHClade.put("577743677660671","T1");
		myHClade.put("577743703720471","H3");
		myHClade.put("577743777760771","T1");
		myHClade.put("577746616540601","X2");
		myHClade.put("577746616560601","X2");
		myHClade.put("577746617760601","X2");
		myHClade.put("577747404560671","Turkey");
		myHClade.put("577747404760671","Turkey");
		myHClade.put("577747437740771","T1");
		myHClade.put("577747607560771","LAM6");
		myHClade.put("577747607760661","LAM9");
		myHClade.put("577747607760771","LAM9");
		myHClade.put("577747620000771","Unknown");
		myHClade.put("577747636700771","Unknown");
		myHClade.put("577747777413471","EAI5");
		myHClade.put("577747777413600","EAI5");
		myHClade.put("577747777420771","Ural-2");
		myHClade.put("577756777760601","X2");
		myHClade.put("577757600413771","EAI5");
		myHClade.put("577757601763771","Manu2");
		myHClade.put("577757604040731","LAM4");
		myHClade.put("577757606060731","LAM11-ZWE");
		myHClade.put("577757606060771","LAM11-ZWE");
		myHClade.put("577757677760671","T1");
		myHClade.put("577757677760771","T1");
		myHClade.put("577757777420771","Ural-2");
		myHClade.put("577757777750771","Manu3");
		myHClade.put("577760057760671","T5");
		myHClade.put("577761757413771","EAI6-BGD1");
		myHClade.put("577761777740371","T1");
		myHClade.put("577763777413071","EAI5");
		myHClade.put("577764000000061","Unknown");
		myHClade.put("577766777760601","X2");
		myHClade.put("577767076640651","T1");
		myHClade.put("577767377740771","T4");
		myHClade.put("577767477400001","Unknown");
		myHClade.put("577767477760671","T-H37Rv");
		myHClade.put("577767477760771","T-H37Rv");
		myHClade.put("577767600760771","T1");
		myHClade.put("577767607720771","H3");
		myHClade.put("577767637000271","Unknown");
		myHClade.put("577767637760771","T1");
		myHClade.put("577767657760771","T5");
		myHClade.put("577767663760771","T1");
		myHClade.put("577767675720671","H3");
		myHClade.put("577767675760771","T1");
		myHClade.put("577767676760631","T1");
		myHClade.put("577767677700771","Unknown");
		myHClade.put("577767677760671","T1");
		myHClade.put("577767737720771","H3");
		myHClade.put("577767737760771","T1");
		myHClade.put("577767745720771","H3");
		myHClade.put("577767750013771","EAI5");
		myHClade.put("577767774020671","H1");
		myHClade.put("577767774413771","EAI4-VNM");
		myHClade.put("577767775400031","Unknown");
		myHClade.put("577767775760571","T1");
		myHClade.put("577767776763771","Manu2");
		myHClade.put("577767777320771","H3");
		myHClade.put("577767777413631","EAI5");
		myHClade.put("577767777413731","EAI1-SOM");
		myHClade.put("577767777420731","Ural-2");
		myHClade.put("577767777420771","Ural-2");
		myHClade.put("577767777520771","H3");
		myHClade.put("577767777700160","Unknown");
		myHClade.put("577767777700771","Unknown");
		myHClade.put("577767777720671","H3");
		myHClade.put("577767777720771","H3");
		myHClade.put("577767777760600","T1");
		myHClade.put("577767777760671","T1");
		myHClade.put("577767777760731","T2");
		myHClade.put("577767777760771","T1");
		myHClade.put("577767777763731","Manu2");
		myHClade.put("577767777763771","Manu2");
		myHClade.put("577770050000000","Unknown");
		myHClade.put("577773774413771","EAI4-VNM");
		myHClade.put("577773776420771","Ural-2");
		myHClade.put("577773777420771","Ural-2");
		myHClade.put("577773777430671","Manu3");
		myHClade.put("577774777420771","Ural-2");
		myHClade.put("577775747413771","EAI6-BGD1");
		myHClade.put("577775750003771","Unknown");
		myHClade.put("577775757411371","EAI6-BGD1");
		myHClade.put("577775757412771","EAI6-BGD1");
		myHClade.put("577775757413771","EAI6-BGD1");
		myHClade.put("577775775760771","T1");
		myHClade.put("577775777413700","EAI5");
		myHClade.put("577775777420771","Ural-2");
		myHClade.put("577775777760771","T1");
		myHClade.put("577776001413771","EAI5");
		myHClade.put("577776377760601","T1");
		myHClade.put("577776744760771","X1");
		myHClade.put("577776755412731","EAI");
		myHClade.put("577776770000000","Unknown");
		myHClade.put("577776774360601","X2");
		myHClade.put("577776777760601","X2");
		myHClade.put("577776777760771","X1");
		myHClade.put("577777000000011","Zero-copy");
		myHClade.put("577777001413771","EAI5");
		myHClade.put("577777002060771","Unknown");
		myHClade.put("577777003413771","EAI5");
		myHClade.put("577777077411171","EAI5");
		myHClade.put("577777102060771","T1");
		myHClade.put("577777177760771","T1");
		myHClade.put("577777207760771","LAM9");
		myHClade.put("577777377413771","EAI5");
		myHClade.put("577777377420771","Ural-2");
		myHClade.put("577777377460771","T4");
		myHClade.put("577777377760700","T4");
		myHClade.put("577777377760731","T");
		myHClade.put("577777403760731","T2");
		myHClade.put("577777404760771","Turkey");
		myHClade.put("577777407560731","LAM6");
		myHClade.put("577777407760771","LAM9");
		myHClade.put("577777437413731","EAI1-SOM");
		myHClade.put("577777437740771","T1");
		myHClade.put("577777437760771","T1");
		myHClade.put("577777477720771","H3");
		myHClade.put("577777577374771","Unknown");
		myHClade.put("577777600000171","Unknown");
		myHClade.put("577777600360771","Unknown");
		myHClade.put("577777603720771","H3");
		myHClade.put("577777603760771","T1");
		myHClade.put("577777604020771","H1");
		myHClade.put("577777606060731","LAM11-ZWE");
		myHClade.put("577777606060771","LAM11-ZWE");
		myHClade.put("577777607560771","LAM9");
		myHClade.put("577777607700771","Unknown");
		myHClade.put("577777607720771","H3");
		myHClade.put("577777607740731","LAM4");
		myHClade.put("577777607741771","Unknown");
		myHClade.put("577777607760571","LAM9");
		myHClade.put("577777607760671","LAM9");
		myHClade.put("577777607760731","LAM4");
		myHClade.put("577777607760751","LAM9");
		myHClade.put("577777607760771","LAM9");
		myHClade.put("577777634020771","H1");
		myHClade.put("577777637760771","T1");
		myHClade.put("577777663560711","T1");
		myHClade.put("577777674020771","H1");
		myHClade.put("577777676763731","Manu2");
		myHClade.put("577777676763771","Manu2");
		myHClade.put("577777677561771","Unknown");
		myHClade.put("577777677720771","H3");
		myHClade.put("577777677760771","T1");
		myHClade.put("577777677763771","Manu2");
		myHClade.put("577777703760771","T1");
		myHClade.put("577777707760771","T1");
		myHClade.put("577777717420771","Ural-2");
		myHClade.put("577777720003771","Unknown");
		myHClade.put("577777737420771","Ural-2");
		myHClade.put("577777740003771","Unknown");
		myHClade.put("577777741413771","EAI6-BGD1");
		myHClade.put("577777743760771","Cameroon");
		myHClade.put("577777744020771","H1");
		myHClade.put("577777750000000","Unknown");
		myHClade.put("577777750003171","Unknown");
		myHClade.put("577777750003771","Unknown");
		myHClade.put("577777755413331","EAI");
		myHClade.put("577777757401371","Unknown");
		myHClade.put("577777757403771","Unknown");
		myHClade.put("577777757410371","EAI6-BGD1");
		myHClade.put("577777757413371","EAI6-BGD1");
		myHClade.put("577777757413771","EAI6-BGD1");
		myHClade.put("577777757473771","Manu1");
		myHClade.put("577777757760771","T5");
		myHClade.put("577777763420771","Ural-2");
		myHClade.put("577777770000000","Unknown");
		myHClade.put("577777770020731","H3");
		myHClade.put("577777770060771","T1");
		myHClade.put("577777772000000","Unknown");
		myHClade.put("577777774000371","Unknown");
		myHClade.put("577777774000771","Unknown");
		myHClade.put("577777774003731","Unknown");
		myHClade.put("577777774020731","H1");
		myHClade.put("577777774020771","H1");
		myHClade.put("577777774160771","T1");
		myHClade.put("577777774220731","H3");
		myHClade.put("577777774413751","EAI4-VNM");
		myHClade.put("577777774413771","EAI4-VNM");
		myHClade.put("577777776000371","Unknown");
		myHClade.put("577777776030371","Unknown");
		myHClade.put("577777776360771","T1");
		myHClade.put("577777776420731","Ural-2");
		myHClade.put("577777776420771","Ural-2");
		myHClade.put("577777776773771","Manu1");
		myHClade.put("577777777060771","T1");
		myHClade.put("577777777220771","H3");
		myHClade.put("577777777360731","T2");
		myHClade.put("577777777400000","Unknown");
		myHClade.put("577777777400371","Unknown");
		myHClade.put("577777777400771","Unknown");
		myHClade.put("577777777403771","Unknown");
		myHClade.put("577777777411731","EAI1-SOM");
		myHClade.put("577777777412071","EAI5");
		myHClade.put("577777777413071","EAI3-IND");
		myHClade.put("577777777413371","EAI5");
		myHClade.put("577777777413631","EAI5");
		myHClade.put("577777777413671","EAI5");
		myHClade.put("577777777413711","EAI5");
		myHClade.put("577777777413731","EAI1-SOM");
		myHClade.put("577777777413770","EAI5");
		myHClade.put("577777777413771","EAI5");
		myHClade.put("577777777420411","Ural-2");
		myHClade.put("577777777420611","Ural-2");
		myHClade.put("577777777420731","Ural-2");
		myHClade.put("577777777420771","Ural-2");
		myHClade.put("577777777433771","Manu1");
		myHClade.put("577777777520771","H3");
		myHClade.put("577777777660471","T1");
		myHClade.put("577777777700000","Unknown");
		myHClade.put("577777777720631","H3");
		myHClade.put("577777777720711","H3");
		myHClade.put("577777777720771","H3");
		myHClade.put("577777777743771","Unknown");
		myHClade.put("577777777760371","T1");
		myHClade.put("577777777760471","T1");
		myHClade.put("577777777760600","T1");
		myHClade.put("577777777760601","T1");
		myHClade.put("577777777760621","T1");
		myHClade.put("577777777760731","T2");
		myHClade.put("577777777760770","T1");
		myHClade.put("577777777760771","T1");
		myHClade.put("577777777763771","Manu2");
		myHClade.put("577777777770771","Manu3");
		myHClade.put("577777777777771","Unknown");
		myHClade.put("600000000000000","Unknown");
		myHClade.put("600000000000011","Unknown");
		myHClade.put("600000000003271","AFRI");
		myHClade.put("600000000060731","T2");
		myHClade.put("600000000060751","T1");
		myHClade.put("600000000360771","T1");
		myHClade.put("600000000560771","Unknown");
		myHClade.put("600000000760731","T2");
		myHClade.put("600000003761771","Unknown");
		myHClade.put("600000004020731","H1");
		myHClade.put("600000004037600","BOV");
		myHClade.put("600000004360601","LAM3");
		myHClade.put("600000004360611","LAM3");
		myHClade.put("600000007177771","Unknown");
		myHClade.put("600000007720771","H3");
		myHClade.put("600000016177600","BOV");
		myHClade.put("600000017703600","BOV");
		myHClade.put("600000017777600","BOV");
		myHClade.put("600000070000000","BOV");
		myHClade.put("600000077413771","EAI5");
		myHClade.put("600000077760771","T1");
		myHClade.put("600000077777600","BOV");
		myHClade.put("600000177760711","T1");
		myHClade.put("600000360000011","Unknown");
		myHClade.put("600000360000371","Unknown");
		myHClade.put("600000377720751","H3");
		myHClade.put("600000377760771","T1");
		myHClade.put("600000606060631","LAM11-ZWE");
		myHClade.put("600000606060731","LAM11-ZWE");
		myHClade.put("600001740003171","Unknown");
		myHClade.put("600001740005471","AFRI");
		myHClade.put("600001740013171","EAI5");
		myHClade.put("600001777760771","T1");
		myHClade.put("600003143140011","Cameroon");
		myHClade.put("600003607760771","LAM3");
		myHClade.put("600003743760771","Cameroon");
		myHClade.put("600003774020771","H1");
		myHClade.put("600003777777600","BOV");
		myHClade.put("600017377760771","T4");
		myHClade.put("600017607760771","LAM3");
		myHClade.put("600017777760731","T2");
		myHClade.put("600017777760771","T1");
		myHClade.put("600017777777600","BOV");
		myHClade.put("600021740003171","Unknown");
		myHClade.put("600023777777600","BOV");
		myHClade.put("600033674177600","BOV");
		myHClade.put("600073774037600","BOV");
		myHClade.put("600073777777600","BOV");
		myHClade.put("600076777760771","X1");
		myHClade.put("600077743760171","Cameroon");
		myHClade.put("600137777760771","T3");
		myHClade.put("600167700060771","T1");
		myHClade.put("600174207760771","LAM3");
		myHClade.put("600177607760771","LAM3");
		myHClade.put("600177777760751","T1");
		myHClade.put("600273740003771","Unknown");
		myHClade.put("600363777377600","BOV");
		myHClade.put("600367747413771","EAI6-BGD1");
		myHClade.put("600373674177600","BOV");
		myHClade.put("600373777777600","BOV");
		myHClade.put("600377477413771","EAI5");
		myHClade.put("600377740003771","Unknown");
		myHClade.put("600377774020771","H1");
		myHClade.put("600377777720771","H3");
		myHClade.put("600377777760771","T1");
		myHClade.put("600776777763771","Manu2");
		myHClade.put("600777477413771","EAI5");
		myHClade.put("600777607760771","LAM1");
		myHClade.put("600777740003671","AFRI");
		myHClade.put("601357606060731","LAM11-ZWE");
		myHClade.put("601357636060731","T2");
		myHClade.put("601737477413771","EAI5");
		myHClade.put("601775607760771","LAM1");
		myHClade.put("601777606060731","LAM11-ZWE");
		myHClade.put("601777606060771","LAM11-ZWE");
		myHClade.put("601777677760771","T1");
		myHClade.put("602363777777600","BOV");
		myHClade.put("602377740003771","Unknown");
		myHClade.put("602421777777400","BOV");
		myHClade.put("602433777777400","BOV_1");
		myHClade.put("602573777777600","BOV_1");
		myHClade.put("602673677777600","BOV_1");
		myHClade.put("602700361000200","BOV");
		myHClade.put("602702037777600","BOV");
		myHClade.put("602733777777600","BOV_1");
		myHClade.put("602743037776600","BOV");
		myHClade.put("602743777777600","BOV");
		myHClade.put("602753377777600","BOV_1");
		myHClade.put("602762037777600","BOV");
		myHClade.put("602763777777600","BOV");
		myHClade.put("602772037777600","BOV_1");
		myHClade.put("602773037777600","BOV_1");
		myHClade.put("602773607777600","BOV_1");
		myHClade.put("602773674177600","BOV_1");
		myHClade.put("602773677777200","BOV_1");
		myHClade.put("602773677777600","BOV_1");
		myHClade.put("602773707777600","BOV_1");
		myHClade.put("602773761000200","BOV_1");
		myHClade.put("602773774777600","BOV_1");
		myHClade.put("602773776000200","BOV_1");
		myHClade.put("602773776277600","BOV_1");
		myHClade.put("602773777677600","BOV_1");
		myHClade.put("602773777753600","BOV_1");
		myHClade.put("602773777773200","BOV_1");
		myHClade.put("602773777774600","BOV_1");
		myHClade.put("602773777775400","BOV_1");
		myHClade.put("602773777777400","BOV_1");
		myHClade.put("602773777777600","BOV_1");
		myHClade.put("603377400001771","Unknown");
		myHClade.put("603757414140731","T2");
		myHClade.put("603777400001771","Unknown");
		myHClade.put("603777477413771","EAI5");
		myHClade.put("603777600003771","Unknown");
		myHClade.put("603777606060771","LAM11-ZWE");
		myHClade.put("603777607560771","LAM6");
		myHClade.put("603777607760771","LAM1");
		myHClade.put("603777677520771","H3");
		myHClade.put("603777677560771","T");
		myHClade.put("603777700000771","Unknown");
		myHClade.put("603777740000071","Unknown");
		myHClade.put("603777740000771","Unknown");
		myHClade.put("603777740003171","CAS1-Delhi");
		myHClade.put("603777740003371","Unknown");
		myHClade.put("603777740003731","Unknown");
		myHClade.put("603777740003771","CAS1-Delhi");
		myHClade.put("603777743760771","Cameroon");
		myHClade.put("603777747770371","Unknown");
		myHClade.put("603777777413771","EAI5");
		myHClade.put("603777777760771","T1");
		myHClade.put("604073777777600","BOV");
		myHClade.put("606573777777600","BOV_1");
		myHClade.put("606673777777600","BOV_1");
		myHClade.put("606763677777600","BOV");
		myHClade.put("606773674177600","BOV_1");
		myHClade.put("606773677747600","BOV_1");
		myHClade.put("606773677777600","BOV_1");
		myHClade.put("606773777775600","BOV_1");
		myHClade.put("606773777777600","BOV_1");
		myHClade.put("607677607560771","LAM6");
		myHClade.put("607677677560771","T1");
		myHClade.put("607737607760731","LAM2");
		myHClade.put("607737777760731","T");
		myHClade.put("607747521100031","Unknown");
		myHClade.put("607774077560771","T1");
		myHClade.put("607777607560771","LAM6");
		myHClade.put("607777607760771","LAM1");
		myHClade.put("607777637760771","T1");
		myHClade.put("607777677560731","T2");
		myHClade.put("607777700003771","Unknown");
		myHClade.put("607777760060731","T2");
		myHClade.put("607777774020771","H1");
		myHClade.put("607777776000371","Unknown");
		myHClade.put("607777777720771","H3");
		myHClade.put("607777777740771","T1");
		myHClade.put("607777777760771","T1");
		myHClade.put("610373777777600","BOV");
		myHClade.put("612773677777600","BOV_1");
		myHClade.put("612773777777600","BOV_1");
		myHClade.put("613776777760601","X2");
		myHClade.put("614001777677600","BOV");
		myHClade.put("614003777777600","BOV");
		myHClade.put("614023777777600","BOV");
		myHClade.put("614073607777600","BOV");
		myHClade.put("614073776037600","BOV");
		myHClade.put("614073777737600","BOV");
		myHClade.put("614073777777600","BOV");
		myHClade.put("614077777777600","BOV");
		myHClade.put("614173776003200","BOV");
		myHClade.put("614173777777600","BOV");
		myHClade.put("614341777777600","BOV");
		myHClade.put("614363677777600","BOV");
		myHClade.put("614363777761600","BOV");
		myHClade.put("614363777767600","BOV");
		myHClade.put("614363777776600","BOV");
		myHClade.put("614363777777200","BOV");
		myHClade.put("614363777777600","BOV");
		myHClade.put("614373777737600","BOV");
		myHClade.put("614373777774600","BOV");
		myHClade.put("614373777777600","BOV");
		myHClade.put("614772777777600","BOV");
		myHClade.put("614773777777600","BOV");
		myHClade.put("616000006777600","BOV");
		myHClade.put("616163777776600","BOV");
		myHClade.put("616173777777600","BOV");
		myHClade.put("616343777777600","BOV");
		myHClade.put("616377707560771","S");
		myHClade.put("616377777760771","S");
		myHClade.put("616473777777600","BOV_1");
		myHClade.put("616600002017600","BOV");
		myHClade.put("616601777777600","BOV");
		myHClade.put("616673777777600","BOV_1");
		myHClade.put("616733777777600","BOV_1");
		myHClade.put("616740004377600","BOV");
		myHClade.put("616743777763600","BOV");
		myHClade.put("616753777777600","BOV_1");
		myHClade.put("616762777560600","X2");
		myHClade.put("616763677777600","BOV");
		myHClade.put("616763757577600","BOV");
		myHClade.put("616763774377600","BOV");
		myHClade.put("616763777077600","BOV");
		myHClade.put("616763777774600","BOV");
		myHClade.put("616763777777200","BOV");
		myHClade.put("616763777777600","BOV");
		myHClade.put("616771777777600","BOV");
		myHClade.put("616772776377600","BOV_1");
		myHClade.put("616772777677600","BOV_1");
		myHClade.put("616772777777600","BOV_1");
		myHClade.put("616773037777600","BOV_1");
		myHClade.put("616773377717600","BOV_1");
		myHClade.put("616773402000000","BOV_1");
		myHClade.put("616773677677600","BOV_1");
		myHClade.put("616773677777200","BOV_1");
		myHClade.put("616773677777600","BOV_1");
		myHClade.put("616773737777600","BOV_1");
		myHClade.put("616773774017600","BOV_1");
		myHClade.put("616773774037600","BOV_1");
		myHClade.put("616773774377600","BOV_1");
		myHClade.put("616773774777600","BOV_1");
		myHClade.put("616773775777600","BOV_1");
		myHClade.put("616773776037600","BOV_1");
		myHClade.put("616773776163600","BOV_1");
		myHClade.put("616773776377600","BOV_1");
		myHClade.put("616773776775600","BOV_1");
		myHClade.put("616773777001600","BOV_1");
		myHClade.put("616773777063600","BOV_1");
		myHClade.put("616773777077600","BOV_1");
		myHClade.put("616773777177200","BOV_1");
		myHClade.put("616773777177600","BOV_1");
		myHClade.put("616773777437200","BOV_1");
		myHClade.put("616773777570600","BOV_1");
		myHClade.put("616773777677600","BOV_1");
		myHClade.put("616773777703600","BOV_1");
		myHClade.put("616773777757600","BOV_1");
		myHClade.put("616773777763600","BOV_1");
		myHClade.put("616773777771600","BOV_1");
		myHClade.put("616773777773600","BOV_1");
		myHClade.put("616773777774600","BOV_1");
		myHClade.put("616773777775600","BOV_1");
		myHClade.put("616773777777000","BOV_1");
		myHClade.put("616773777777200","BOV_1");
		myHClade.put("616773777777400","BOV_1");
		myHClade.put("616773777777600","BOV_1");
		myHClade.put("616777617777600","BOV");
		myHClade.put("616777677777600","BOV");
		myHClade.put("616777777777600","BOV");
		myHClade.put("617400003760771","T1");
		myHClade.put("617623377760771","T4");
		myHClade.put("617736000000000","Unknown");
		myHClade.put("617737377760601","T");
		myHClade.put("617737400020771","H3");
		myHClade.put("617737607760771","LAM2");
		myHClade.put("617737777720771","H3");
		myHClade.put("617772777760771","X1");
		myHClade.put("617774077760601","T1");
		myHClade.put("617774077763771","Manu2");
		myHClade.put("617776777760401","X2");
		myHClade.put("617776777760601","X2");
		myHClade.put("617776777760771","X1");
		myHClade.put("617777377760601","T4");
		myHClade.put("617777377760771","T4");
		myHClade.put("617777477413771","EAI5");
		myHClade.put("617777600020771","H3");
		myHClade.put("617777607760771","LAM1");
		myHClade.put("617777774020760","H1");
		myHClade.put("617777774020771","H1");
		myHClade.put("617777777700771","Unknown");
		myHClade.put("617777777720771","H3");
		myHClade.put("617777777760601","T");
		myHClade.put("617777777760771","T1");
		myHClade.put("623774777760730","T2-Uganda");
		myHClade.put("624033777777600","BOV");
		myHClade.put("624073777700600","BOV");
		myHClade.put("624073777777600","BOV");
		myHClade.put("624775777760730","T2-Uganda");
		myHClade.put("627777607360771","LAM1");
		myHClade.put("630000207760771","LAM3");
		myHClade.put("633774477760730","T");
		myHClade.put("634553777777600","BOV");
		myHClade.put("636167607760771","LAM3");
		myHClade.put("636177400000171","Unknown");
		myHClade.put("636373677777600","BOV");
		myHClade.put("636377607760771","LAM1");
		myHClade.put("636377770000731","Unknown");
		myHClade.put("636377770760771","S");
		myHClade.put("636377777760731","S");
		myHClade.put("636573637777600","BOV_1");
		myHClade.put("636573677777600","BOV_1");
		myHClade.put("636573777777600","BOV_1");
		myHClade.put("636723777777600","BOV");
		myHClade.put("636733777757600","BOV_1");
		myHClade.put("636733777774600","BOV_1");
		myHClade.put("636733777777600","BOV_1");
		myHClade.put("636763677777600","BOV");
		myHClade.put("636773377757600","BOV_1");
		myHClade.put("636773674177600","BOV_1");
		myHClade.put("636773676377600","BOV_1");
		myHClade.put("636773677777200","BOV_1");
		myHClade.put("636773677777600","BOV_1");
		myHClade.put("636773777677600","BOV_1");
		myHClade.put("636773777743600","BOV_1");
		myHClade.put("636773777774600","BOV_1");
		myHClade.put("636773777777600","BOV_1");
		myHClade.put("637344777740130","T2-Uganda");
		myHClade.put("637403607760671","LAM2");
		myHClade.put("637574777760730","T2-Uganda");
		myHClade.put("637603747760771","T1");
		myHClade.put("637637777760771","T1");
		myHClade.put("637676777760131","X1");
		myHClade.put("637677607760771","LAM1");
		myHClade.put("637703777760571","T1");
		myHClade.put("637714377760730","T2-Uganda");
		myHClade.put("637727607760771","LAM2");
		myHClade.put("637735777760730","T2-Uganda");
		myHClade.put("637737607760771","LAM2");
		myHClade.put("637737777420771","Ural-1");
		myHClade.put("637737777760771","T3");
		myHClade.put("637740000060771","T1");
		myHClade.put("637757637760771","T1");
		myHClade.put("637760007760771","LAM-RUS");
		myHClade.put("637764777760730","T2-Uganda");
		myHClade.put("637764777760731","T2");
		myHClade.put("637765777760730","T2-Uganda");
		myHClade.put("637767607763771","Manu2");
		myHClade.put("637767777760730","T2-Uganda");
		myHClade.put("637773777760730","T2-Uganda");
		myHClade.put("637774740000130","Unknown");
		myHClade.put("637774760760730","T2-Uganda");
		myHClade.put("637774777560730","T2-Uganda");
		myHClade.put("637774777740130","T2-Uganda");
		myHClade.put("637774777760730","T2-Uganda");
		myHClade.put("637775777740130","T2-Uganda");
		myHClade.put("637775777760730","T2-Uganda");
		myHClade.put("637775777763770","Manu2");
		myHClade.put("637776477413771","EAI5");
		myHClade.put("637776777760730","T2-Uganda");
		myHClade.put("637776777760771","X1");
		myHClade.put("637777477413750","EAI5");
		myHClade.put("637777477413771","EAI5");
		myHClade.put("637777557760771","T5-Madrid2");
		myHClade.put("637777577360771","T1");
		myHClade.put("637777601560771","T1");
		myHClade.put("637777606060771","LAM11-ZWE");
		myHClade.put("637777607760431","LAM");
		myHClade.put("637777607760731","LAM");
		myHClade.put("637777607760771","LAM1");
		myHClade.put("637777637760031","T1");
		myHClade.put("637777637760771","T1");
		myHClade.put("637777707760771","T1");
		myHClade.put("637777737720771","H3");
		myHClade.put("637777747762771","Manu2");
		myHClade.put("637777774020771","H1");
		myHClade.put("637777777700371","Unknown");
		myHClade.put("637777777720771","H3");
		myHClade.put("637777777760730","T2-Uganda");
		myHClade.put("637777777760731","T2");
		myHClade.put("637777777760771","T1");
		myHClade.put("640000000060731","T2");
		myHClade.put("640000037777600","BOV");
		myHClade.put("640003737777600","BOV");
		myHClade.put("640003777757600","BOV");
		myHClade.put("640003777777600","BOV");
		myHClade.put("640012017777600","BOV_3");
		myHClade.put("640013377777600","BOV_3");
		myHClade.put("640013677777600","BOV_3");
		myHClade.put("640013737777600","BOV_3");
		myHClade.put("640013776377600","BOV_3");
		myHClade.put("640013776777600","BOV_3");
		myHClade.put("640013777177600","BOV_3");
		myHClade.put("640013777763600","BOV_3");
		myHClade.put("640013777776600","BOV_3");
		myHClade.put("640013777777600","BOV_3");
		myHClade.put("640023777775600","BOV");
		myHClade.put("640023777777600","BOV");
		myHClade.put("640033377777600","BOV");
		myHClade.put("640033777776600","BOV");
		myHClade.put("640033777777600","BOV");
		myHClade.put("640033777777710","Unknown");
		myHClade.put("640073774037600","BOV");
		myHClade.put("640077477413771","EAI2-Manila");
		myHClade.put("640203777777600","BOV");
		myHClade.put("640773677777600","BOV");
		myHClade.put("642207777777600","BOV");
		myHClade.put("642217777777600","BOV");
		myHClade.put("642501777177600","BOV");
		myHClade.put("642573777777600","BOV_1");
		myHClade.put("642773677777200","BOV_1");
		myHClade.put("642773677777600","BOV_1");
		myHClade.put("643677607741771","Unknown");
		myHClade.put("643777777413071","EAI5");
		myHClade.put("644043777777600","BOV");
		myHClade.put("644063777700600","BOV");
		myHClade.put("644063777747600","BOV");
		myHClade.put("644063777777600","BOV");
		myHClade.put("644073777677600","BOV");
		myHClade.put("644073777747600","BOV");
		myHClade.put("644073777777600","BOV");
		myHClade.put("644773777767600","BOV");
		myHClade.put("646573777777600","BOV_1");
		myHClade.put("646762077777600","BOV");
		myHClade.put("646763777767600","BOV");
		myHClade.put("646763777777600","BOV");
		myHClade.put("646770077767600","BOV");
		myHClade.put("646772077777600","BOV_1");
		myHClade.put("646773776167600","BOV_1");
		myHClade.put("646773777667600","BOV_1");
		myHClade.put("646773777677600","BOV_1");
		myHClade.put("646773777700600","BOV_1");
		myHClade.put("646773777740200","BOV_1");
		myHClade.put("646773777767200","BOV_1");
		myHClade.put("646773777767600","BOV_1");
		myHClade.put("647677607741771","Unknown");
		myHClade.put("647737607760771","LAM2");
		myHClade.put("647777477013771","EAI2-Manila");
		myHClade.put("647777477413731","EAI2-Manila");
		myHClade.put("647777477413771","EAI2-Manila");
		myHClade.put("647777607760771","LAM1");
		myHClade.put("653700357760771","T5");
		myHClade.put("654343677776600","BOV");
		myHClade.put("654573777767600","BOV");
		myHClade.put("654573777777600","BOV");
		myHClade.put("656343776677600","BOV");
		myHClade.put("656343777677600","BOV");
		myHClade.put("656473637777600","BOV_1");
		myHClade.put("656473737777600","BOV_1");
		myHClade.put("656473777743600","BOV_1");
		myHClade.put("656563377777600","BOV");
		myHClade.put("656573237777200","BOV_1");
		myHClade.put("656573277777600","BOV_1");
		myHClade.put("656573377603600","BOV_1");
		myHClade.put("656573377777200","BOV_1");
		myHClade.put("656573377777600","BOV_1");
		myHClade.put("656573777777600","BOV_1");
		myHClade.put("656772777777600","BOV_1");
		myHClade.put("656773377777600","BOV_1");
		myHClade.put("656773674177600","BOV_1");
		myHClade.put("656773677777200","BOV_1");
		myHClade.put("656773677777600","BOV_1");
		myHClade.put("656773777677600","BOV_1");
		myHClade.put("656773777767600","BOV_1");
		myHClade.put("656773777777600","BOV_1");
		myHClade.put("657636207760771","LAM2");
		myHClade.put("657677607760771","LAM1");
		myHClade.put("657737607760771","LAM2");
		myHClade.put("657740007413771","EAI2");
		myHClade.put("657757607760771","LAM1");
		myHClade.put("657776777760771","X1");
		myHClade.put("657777406060771","LAM11-ZWE");
		myHClade.put("657777406760771","LAM8");
		myHClade.put("657777476413771","EAI2-Manila");
		myHClade.put("657777477413601","EAI2-Manila");
		myHClade.put("657777477413701","EAI2-Manila");
		myHClade.put("657777477413751","EAI2-Manila");
		myHClade.put("657777477413771","EAI2-Manila");
		myHClade.put("657777607160771","LAM1");
		myHClade.put("657777607360761","LAM1");
		myHClade.put("657777607541771","Unknown");
		myHClade.put("657777607760731","LAM");
		myHClade.put("657777607760771","LAM1");
		myHClade.put("657777777413071","EAI5");
		myHClade.put("657777777760671","T1");
		myHClade.put("660000000060771","T1");
		myHClade.put("660000007777600","BOV");
		myHClade.put("660001400000171","Unknown");
		myHClade.put("660003400000171","Unknown");
		myHClade.put("660033777777600","BOV");
		myHClade.put("660073777777600","BOV");
		myHClade.put("660177457413771","EAI2-Manila");
		myHClade.put("660777777660771","T1");
		myHClade.put("661777477413771","EAI2-Manila");
		myHClade.put("661777607760771","LAM1");
		myHClade.put("662043777776200","BOV");
		myHClade.put("662763777417600","BOV");
		myHClade.put("662763777777600","BOV");
		myHClade.put("662773677777600","BOV_1");
		myHClade.put("662773777477600","BOV_1");
		myHClade.put("662773777777600","BOV_1");
		myHClade.put("663600007400071","Unknown");
		myHClade.put("663777477413771","EAI2-Manila");
		myHClade.put("663777607760771","LAM1");
		myHClade.put("664000177777600","BOV");
		myHClade.put("664000377777600","BOV");
		myHClade.put("664000777777600","BOV");
		myHClade.put("664013777777600","BOV");
		myHClade.put("664031777777600","BOV");
		myHClade.put("664033774037600","BOV");
		myHClade.put("664033777747600","BOV");
		myHClade.put("664033777776600","BOV");
		myHClade.put("664033777777600","BOV");
		myHClade.put("664041777777600","BOV");
		myHClade.put("664043774037600","BOV");
		myHClade.put("664043777777600","BOV");
		myHClade.put("664053777775600","BOV");
		myHClade.put("664060767777600","BOV");
		myHClade.put("664061777777600","BOV");
		myHClade.put("664063077777600","BOV");
		myHClade.put("664063377777600","BOV");
		myHClade.put("664063774037600","BOV");
		myHClade.put("664063777700600","BOV");
		myHClade.put("664063777747600","BOV");
		myHClade.put("664063777757600","BOV");
		myHClade.put("664063777767600","BOV");
		myHClade.put("664063777775600","BOV");
		myHClade.put("664063777776600","BOV");
		myHClade.put("664063777777200","BOV");
		myHClade.put("664063777777400","BOV");
		myHClade.put("664063777777600","BOV");
		myHClade.put("664070377777600","BOV");
		myHClade.put("664071777777600","BOV");
		myHClade.put("664072777777600","BOV_2");
		myHClade.put("664073017777600","BOV_2");
		myHClade.put("664073077777600","BOV_2");
		myHClade.put("664073377077600","BOV_2");
		myHClade.put("664073377777600","BOV_2");
		myHClade.put("664073477777600","BOV_2");
		myHClade.put("664073617700600","BOV_2");
		myHClade.put("664073677563600","BOV_2");
		myHClade.put("664073677777200","BOV_2");
		myHClade.put("664073677777600","BOV_2");
		myHClade.put("664073757777600","BOV_2");
		myHClade.put("664073767777600","BOV_2");
		myHClade.put("664073770037600","BOV_2");
		myHClade.put("664073774037600","BOV_2");
		myHClade.put("664073775777600","BOV_2");
		myHClade.put("664073776177600","BOV_2");
		myHClade.put("664073776333600","BOV_2");
		myHClade.put("664073776373600","BOV_2");
		myHClade.put("664073776377600","BOV_2");
		myHClade.put("664073776777600","BOV_2");
		myHClade.put("664073777000600","BOV_2");
		myHClade.put("664073777073600","BOV_2");
		myHClade.put("664073777377600","BOV_2");
		myHClade.put("664073777400600","BOV_2");
		myHClade.put("664073777437600","BOV_2");
		myHClade.put("664073777577600","BOV_2");
		myHClade.put("664073777600200","BOV_2");
		myHClade.put("664073777623600","BOV_2");
		myHClade.put("664073777637600","BOV_2");
		myHClade.put("664073777643600","BOV_2");
		myHClade.put("664073777700600","BOV_2");
		myHClade.put("664073777707600","BOV_2");
		myHClade.put("664073777740600","BOV_2");
		myHClade.put("664073777741600","BOV_2");
		myHClade.put("664073777743600","BOV_2");
		myHClade.put("664073777747600","BOV_2");
		myHClade.put("664073777757600","BOV_2");
		myHClade.put("664073777763200","BOV_2");
		myHClade.put("664073777763600","BOV_2");
		myHClade.put("664073777767600","BOV_2");
		myHClade.put("664073777771600","BOV_2");
		myHClade.put("664073777773600","BOV_2");
		myHClade.put("664073777774600","BOV_2");
		myHClade.put("664073777775600","BOV_2");
		myHClade.put("664073777776200","BOV_2");
		myHClade.put("664073777776600","BOV_2");
		myHClade.put("664073777777200","BOV_2");
		myHClade.put("664073777777400","BOV_2");
		myHClade.put("664073777777600","BOV_2");
		myHClade.put("664077777777200","BOV");
		myHClade.put("664077777777600","BOV");
		myHClade.put("664273777776600","BOV");
		myHClade.put("665777607760771","LAM1");
		myHClade.put("666000003413771","EAI2");
		myHClade.put("666003477413771","EAI2-Manila");
		myHClade.put("666073677777600","BOV");
		myHClade.put("666573677777600","BOV_1");
		myHClade.put("666673677777600","BOV_1");
		myHClade.put("666733677777600","BOV_1");
		myHClade.put("666743677777600","BOV");
		myHClade.put("666743717677600","BOV");
		myHClade.put("666755777413731","EAI1-SOM");
		myHClade.put("666763677777600","BOV");
		myHClade.put("666763777777600","BOV");
		myHClade.put("666770777677600","BOV");
		myHClade.put("666772057777200","BOV_1");
		myHClade.put("666772377677600","BOV_1");
		myHClade.put("666773277777600","BOV_1");
		myHClade.put("666773477777600","BOV_1");
		myHClade.put("666773657777600","BOV_1");
		myHClade.put("666773674177600","BOV_1");
		myHClade.put("666773677737600","BOV_1");
		myHClade.put("666773677776200","BOV_1");
		myHClade.put("666773677776600","BOV_1");
		myHClade.put("666773677777600","BOV_1");
		myHClade.put("666773777643600","BOV_1");
		myHClade.put("666773777677600","BOV_1");
		myHClade.put("666773777777600","BOV_1");
		myHClade.put("667617777777600","Unknown");
		myHClade.put("667737607760771","LAM2");
		myHClade.put("667777470000000","Unknown");
		myHClade.put("667777477413771","EAI2-Manila");
		myHClade.put("667777607760771","LAM1");
		myHClade.put("667777776360771","T1");
		myHClade.put("667777777760471","T1");
		myHClade.put("667777777777771","Unknown");
		myHClade.put("670000000160771","T1");
		myHClade.put("670000775760731","T1-RUS2");
		myHClade.put("670000777760731","T1-RUS2");
		myHClade.put("670000777760771","T1-RUS2");
		myHClade.put("670000777763771","Manu2");
		myHClade.put("670041777767600","BOV");
		myHClade.put("670173677773600","BOV");
		myHClade.put("670177607760731","LAM3");
		myHClade.put("670373677777200","BOV");
		myHClade.put("670773677777600","BOV");
		myHClade.put("670777777477671","AFRI_1");
		myHClade.put("670777777777671","AFRI_1");
		myHClade.put("671527607760671","LAM2");
		myHClade.put("671737607760771","LAM2");
		myHClade.put("671777707413731","EAI1-SOM");
		myHClade.put("672600000000671","mungi");
		myHClade.put("672733777777600","BOV_1");
		myHClade.put("672763777177600","BOV");
		myHClade.put("672763777377600","BOV");
		myHClade.put("672763777775600","BOV");
		myHClade.put("672773674177600","BOV_1");
		myHClade.put("672773677775600","BOV_1");
		myHClade.put("672773677777200","BOV_1");
		myHClade.put("672773677777600","BOV_1");
		myHClade.put("672773776377600","BOV_1");
		myHClade.put("672773777377600","BOV_1");
		myHClade.put("672773777771600","BOV_1");
		myHClade.put("672773777775600","BOV_1");
		myHClade.put("672773777777600","BOV_1");
		myHClade.put("673737607760771","LAM2");
		myHClade.put("673777477413470","EAI2-Manila");
		myHClade.put("673777477413771","EAI2-Manila");
		myHClade.put("673777607760771","LAM1");
		myHClade.put("673777777420731","Ural-1");
		myHClade.put("674000000000071","AFRI");
		myHClade.put("674000000000171","Unknown");
		myHClade.put("674000001413771","EAI2-nonthaburi");
		myHClade.put("674000001613771","Manu1");
		myHClade.put("674000002000071","AFRI");
		myHClade.put("674000002413771","EAI2-nonthaburi");
		myHClade.put("674000003403771","Unknown");
		myHClade.put("674000003413400","EAI2-nonthaburi");
		myHClade.put("674000003413710","EAI2-nonthaburi");
		myHClade.put("674000003413731","EAI2-nonthaburi");
		myHClade.put("674000003413770","EAI2-nonthaburi");
		myHClade.put("674000003413771","EAI2-nonthaburi");
		myHClade.put("674000003417771","Unknown");
		myHClade.put("674000003760771","T1");
		myHClade.put("674000017000771","Unknown");
		myHClade.put("674000674177600","BOV");
		myHClade.put("674001777677600","BOV");
		myHClade.put("674002177677600","BOV");
		myHClade.put("674003674037600","BOV");
		myHClade.put("674013674037600","BOV");
		myHClade.put("674017607760771","LAM3");
		myHClade.put("674020003413771","EAI2");
		myHClade.put("674023777777600","BOV");
		myHClade.put("674033677777200","BOV");
		myHClade.put("674037607760771","LAM3");
		myHClade.put("674063777777600","BOV");
		myHClade.put("674063777777660","AFRI");
		myHClade.put("674073777777600","BOV");
		myHClade.put("674153777777600","BOV");
		myHClade.put("674173677077600","BOV");
		myHClade.put("674173677777600","BOV");
		myHClade.put("674173777773600","BOV");
		myHClade.put("674173777777600","BOV");
		myHClade.put("674177777413751","EAI5");
		myHClade.put("674177777413771","EAI5");
		myHClade.put("674243676763600","BOV");
		myHClade.put("674353677777600","BOV");
		myHClade.put("674363677777600","BOV");
		myHClade.put("674363777776000","BOV");
		myHClade.put("674373677777600","BOV");
		myHClade.put("674373776277600","BOV");
		myHClade.put("674373777677600","BOV");
		myHClade.put("674373777774000","BOV");
		myHClade.put("674373777777600","BOV");
		myHClade.put("674377407760771","LAM1");
		myHClade.put("674377477413771","EAI2-Manila");
		myHClade.put("674377777403771","Unknown");
		myHClade.put("674377777413771","EAI5");
		myHClade.put("674573777743600","BOV");
		myHClade.put("674573777767600","BOV");
		myHClade.put("674573777777600","BOV");
		myHClade.put("674573777777601","Unknown");
		myHClade.put("674753677777600","BOV");
		myHClade.put("674763777077600","BOV");
		myHClade.put("674763777777600","BOV");
		myHClade.put("674773377757600","BOV");
		myHClade.put("674773674377200","BOV");
		myHClade.put("674773675777200","BOV");
		myHClade.put("674773677767600","BOV");
		myHClade.put("674773677775600","BOV");
		myHClade.put("674773677777200","BOV");
		myHClade.put("674773677777600","BOV");
		myHClade.put("674773777637600","BOV");
		myHClade.put("674773777777400","BOV");
		myHClade.put("674773777777600","BOV");
		myHClade.put("674777377420771","Ural-1");
		myHClade.put("674777470001771","EAI2-Manila");
		myHClade.put("674777470003771","Unknown");
		myHClade.put("674777477411771","EAI2-Manila");
		myHClade.put("674777477413771","EAI2-Manila");
		myHClade.put("674777777420771","Ural-1");
		myHClade.put("675567607760671","LAM1");
		myHClade.put("675737607760371","LAM2");
		myHClade.put("675737607760711","LAM2");
		myHClade.put("675737607760771","LAM2");
		myHClade.put("675757477413701","EAI2-Manila");
		myHClade.put("675766607760661","LAM1");
		myHClade.put("675767377413671","EAI5");
		myHClade.put("675767757760671","T5");
		myHClade.put("675777477413771","EAI2-Manila");
		myHClade.put("675777607740331","LAM");
		myHClade.put("675777607760731","LAM");
		myHClade.put("675777607760771","LAM1");
		myHClade.put("675777757760771","T5");
		myHClade.put("676000000000171","Unknown");
		myHClade.put("676000757777600","BOV");
		myHClade.put("676000777777600","BOV");
		myHClade.put("676001677777200","BOV");
		myHClade.put("676003477413771","EAI2-Manila");
		myHClade.put("676013607760771","LAM12-Madrid1");
		myHClade.put("676023477413771","EAI2-Manila");
		myHClade.put("676033677777200","BOV");
		myHClade.put("676033677777600","BOV");
		myHClade.put("676037777740771","T1");
		myHClade.put("676073777767600","BOV");
		myHClade.put("676073777777600","BOV");
		myHClade.put("676077477413771","EAI2-Manila");
		myHClade.put("676077607760371","LAM3");
		myHClade.put("676077607760771","LAM3");
		myHClade.put("676143677477600","BOV");
		myHClade.put("676143777777600","BOV");
		myHClade.put("676173777777600","BOV");
		myHClade.put("676177400000371","Unknown");
		myHClade.put("676177477413771","EAI2-Manila");
		myHClade.put("676177607760771","LAM3");
		myHClade.put("676177770000010","Unknown");
		myHClade.put("676267737760771","S");
		myHClade.put("676277757760771","S");
		myHClade.put("676277777700600","BOV");
		myHClade.put("676303777777600","BOV");
		myHClade.put("676337737760771","S");
		myHClade.put("676341757777600","BOV");
		myHClade.put("676343676037600","BOV");
		myHClade.put("676343677777600","BOV");
		myHClade.put("676343776277600","BOV");
		myHClade.put("676343777677600","BOV");
		myHClade.put("676343777776600","BOV");
		myHClade.put("676353677777600","BOV");
		myHClade.put("676357760000171","Unknown");
		myHClade.put("676363577776600","BOV");
		myHClade.put("676363674177600","BOV");
		myHClade.put("676363677777600","BOV");
		myHClade.put("676363777777600","BOV");
		myHClade.put("676373674177600","BOV");
		myHClade.put("676373677777600","BOV");
		myHClade.put("676373777776600","BOV");
		myHClade.put("676373777777600","BOV");
		myHClade.put("676377607760771","LAM1");
		myHClade.put("676377700140771","S");
		myHClade.put("676377717760471","S");
		myHClade.put("676377717760771","S");
		myHClade.put("676377736000131","Unknown");
		myHClade.put("676377737760711","S");
		myHClade.put("676377737760731","S");
		myHClade.put("676377737760771","S");
		myHClade.put("676377760000171","Unknown");
		myHClade.put("676377776060731","S");
		myHClade.put("676377777720771","H3");
		myHClade.put("676377777760731","S");
		myHClade.put("676377777760771","S");
		myHClade.put("676400377777600","BOV");
		myHClade.put("676400657777600","BOV");
		myHClade.put("676401677777600","BOV");
		myHClade.put("676413677777200","BOV_1");
		myHClade.put("676413777777600","BOV_1");
		myHClade.put("676433677777600","BOV_1");
		myHClade.put("676433777777400","BOV_1");
		myHClade.put("676463777777600","BOV");
		myHClade.put("676473377757600","BOV_1");
		myHClade.put("676473677777600","BOV_1");
		myHClade.put("676473737777600","BOV_1");
		myHClade.put("676473776003600","BOV_1");
		myHClade.put("676473777757600","BOV_1");
		myHClade.put("676473777777600","BOV_1");
		myHClade.put("676501777177600","BOV");
		myHClade.put("676533777777600","BOV_1");
		myHClade.put("676537777777600","BOV");
		myHClade.put("676553777777600","BOV_1");
		myHClade.put("676561777777600","BOV");
		myHClade.put("676562077777600","BOV");
		myHClade.put("676563677477600","BOV");
		myHClade.put("676563677774000","BOV");
		myHClade.put("676563777776600","BOV");
		myHClade.put("676563777777600","BOV");
		myHClade.put("676573077777600","BOV_1");
		myHClade.put("676573377677600","BOV_1");
		myHClade.put("676573377777600","BOV_1");
		myHClade.put("676573400000000","BOV_1");
		myHClade.put("676573417777600","BOV_1");
		myHClade.put("676573477777600","BOV_1");
		myHClade.put("676573604377600","BOV_1");
		myHClade.put("676573677477600","BOV_1");
		myHClade.put("676573677577600","BOV_1");
		myHClade.put("676573677774000","BOV_1");
		myHClade.put("676573677777600","BOV_1");
		myHClade.put("676573774007600","BOV_1");
		myHClade.put("676573777017600","BOV_1");
		myHClade.put("676573777077600","BOV_1");
		myHClade.put("676573777177600","BOV_1");
		myHClade.put("676573777477600","BOV_1");
		myHClade.put("676573777577600","BOV_1");
		myHClade.put("676573777637600","BOV_1");
		myHClade.put("676573777677600","BOV_1");
		myHClade.put("676573777737600","BOV_1");
		myHClade.put("676573777740600","BOV_1");
		myHClade.put("676573777743600","BOV_1");
		myHClade.put("676573777757600","BOV_1");
		myHClade.put("676573777763600","BOV_1");
		myHClade.put("676573777771600","BOV_1");
		myHClade.put("676573777775600","BOV_1");
		myHClade.put("676573777776200","BOV_1");
		myHClade.put("676573777776600","BOV_1");
		myHClade.put("676573777777000","BOV_1");
		myHClade.put("676573777777600","BOV_1");
		myHClade.put("676600000000000","BOV");
		myHClade.put("676600000000371","Unknown");
		myHClade.put("676600000000771","Unknown");
		myHClade.put("676600077777600","BOV");
		myHClade.put("676600377777600","BOV");
		myHClade.put("676603777777600","BOV");
		myHClade.put("676612677777600","BOV_1");
		myHClade.put("676613657777600","BOV_1");
		myHClade.put("676613677777600","BOV_1");
		myHClade.put("676633677777200","BOV_1");
		myHClade.put("676633677777600","BOV_1");
		myHClade.put("676633777777000","BOV_1");
		myHClade.put("676643677777600","BOV");
		myHClade.put("676661677777600","BOV");
		myHClade.put("676663677777600","BOV");
		myHClade.put("676663777777600","BOV");
		myHClade.put("676672677777600","BOV_1");
		myHClade.put("676673474177600","BOV_1");
		myHClade.put("676673674177600","BOV_1");
		myHClade.put("676673676177600","BOV_1");
		myHClade.put("676673677677600","BOV_1");
		myHClade.put("676673677777200","BOV_1");
		myHClade.put("676673677777600","BOV_1");
		myHClade.put("676673757777600","BOV_1");
		myHClade.put("676673777777600","BOV_1");
		myHClade.put("676700017777600","BOV");
		myHClade.put("676700677777600","BOV");
		myHClade.put("676701777677600","BOV");
		myHClade.put("676701777767600","BOV");
		myHClade.put("676703777777600","BOV");
		myHClade.put("676713677767600","BOV_1");
		myHClade.put("676713677777600","BOV_1");
		myHClade.put("676723777777600","BOV");
		myHClade.put("676733617747600","BOV_1");
		myHClade.put("676733674177600","BOV_1");
		myHClade.put("676733677777200","BOV_1");
		myHClade.put("676733677777600","BOV_1");
		myHClade.put("676733776003600","BOV_1");
		myHClade.put("676733777776600","BOV_1");
		myHClade.put("676733777777000","BOV_1");
		myHClade.put("676733777777600","BOV_1");
		myHClade.put("676737607760771","LAM2");
		myHClade.put("676740000037600","BOV");
		myHClade.put("676740677777600","BOV");
		myHClade.put("676741002037600","BOV");
		myHClade.put("676741077757600","BOV");
		myHClade.put("676741377756600","BOV");
		myHClade.put("676741377757600","BOV");
		myHClade.put("676741674177600","BOV");
		myHClade.put("676741777777600","BOV");
		myHClade.put("676743377757600","BOV");
		myHClade.put("676743377777600","BOV");
		myHClade.put("676743676177600","BOV");
		myHClade.put("676743677777600","BOV");
		myHClade.put("676743754777600","BOV");
		myHClade.put("676743776277600","BOV");
		myHClade.put("676743776677600","BOV");
		myHClade.put("676743777677600","BOV");
		myHClade.put("676743777777600","BOV");
		myHClade.put("676753674177600","BOV_1");
		myHClade.put("676753677777600","BOV_1");
		myHClade.put("676753777677600","BOV_1");
		myHClade.put("676760000007600","BOV");
		myHClade.put("676760006067600","BOV");
		myHClade.put("676760077775600","BOV");
		myHClade.put("676760747777200","BOV");
		myHClade.put("676760777777600","BOV");
		myHClade.put("676761037777600","BOV");
		myHClade.put("676761377757600","BOV");
		myHClade.put("676761677777600","BOV");
		myHClade.put("676761757777600","BOV");
		myHClade.put("676762037777600","BOV");
		myHClade.put("676762077777600","BOV");
		myHClade.put("676763277777600","BOV");
		myHClade.put("676763377757600","BOV");
		myHClade.put("676763477777600","BOV");
		myHClade.put("676763637767600","BOV");
		myHClade.put("676763655777600","BOV");
		myHClade.put("676763674177600","BOV");
		myHClade.put("676763675777200","BOV");
		myHClade.put("676763675777600","BOV");
		myHClade.put("676763676377600","BOV");
		myHClade.put("676763677757600","BOV");
		myHClade.put("676763677775200","BOV");
		myHClade.put("676763677775600","BOV");
		myHClade.put("676763677776200","BOV");
		myHClade.put("676763677776600","BOV");
		myHClade.put("676763677777000","BOV");
		myHClade.put("676763677777200","BOV");
		myHClade.put("676763677777400","BOV");
		myHClade.put("676763677777600","BOV");
		myHClade.put("676763776437600","BOV");
		myHClade.put("676763776677600","BOV");
		myHClade.put("676763777674200","BOV");
		myHClade.put("676763777677600","BOV");
		myHClade.put("676763777743600","BOV");
		myHClade.put("676763777767600","BOV");
		myHClade.put("676763777773600","BOV");
		myHClade.put("676763777774600","BOV");
		myHClade.put("676763777777400","BOV");
		myHClade.put("676763777777600","BOV");
		myHClade.put("676767777777600","BOV");
		myHClade.put("676770000000000","BOV");
		myHClade.put("676770077777600","BOV");
		myHClade.put("676770777777600","BOV");
		myHClade.put("676771607177600","BOV");
		myHClade.put("676771677777600","BOV");
		myHClade.put("676771757777600","BOV");
		myHClade.put("676771777774600","BOV");
		myHClade.put("676771777777600","BOV");
		myHClade.put("676772037777600","BOV_1");
		myHClade.put("676772077777600","BOV_1");
		myHClade.put("676772274177600","BOV_1");
		myHClade.put("676772277777600","BOV_1");
		myHClade.put("676772777767600","BOV_1");
		myHClade.put("676773000017600","BOV_1");
		myHClade.put("676773000037600","BOV_1");
		myHClade.put("676773002017600","BOV_1");
		myHClade.put("676773017737600","BOV_1");
		myHClade.put("676773037613600","BOV_1");
		myHClade.put("676773037777600","BOV_1");
		myHClade.put("676773277757600","BOV_1");
		myHClade.put("676773277777200","BOV_1");
		myHClade.put("676773277777600","BOV_1");
		myHClade.put("676773357657600","BOV_1");
		myHClade.put("676773377736600","BOV_1");
		myHClade.put("676773377740600","BOV_1");
		myHClade.put("676773377757600","BOV_1");
		myHClade.put("676773377777600","BOV_1");
		myHClade.put("676773402017600","BOV_1");
		myHClade.put("676773406177600","BOV_1");
		myHClade.put("676773417777600","BOV_1");
		myHClade.put("676773437637600","BOV_1");
		myHClade.put("676773437777600","BOV_1");
		myHClade.put("676773474177600","BOV_1");
		myHClade.put("676773477777600","BOV_1");
		myHClade.put("676773577777600","BOV_1");
		myHClade.put("676773607777600","BOV_1");
		myHClade.put("676773617743400","BOV_1");
		myHClade.put("676773617743600","BOV_1");
		myHClade.put("676773617747600","BOV_1");
		myHClade.put("676773617757600","BOV_1");
		myHClade.put("676773617777600","BOV_1");
		myHClade.put("676773637777200","BOV_1");
		myHClade.put("676773637777600","BOV_1");
		myHClade.put("676773657577600","BOV_1");
		myHClade.put("676773657637600","BOV_1");
		myHClade.put("676773657747200","BOV_1");
		myHClade.put("676773657774600","BOV_1");
		myHClade.put("676773657777200","BOV_1");
		myHClade.put("676773657777600","BOV_1");
		myHClade.put("676773664177600","BOV_1");
		myHClade.put("676773666177600","BOV_1");
		myHClade.put("676773666617600","BOV_1");
		myHClade.put("676773667777600","BOV_1");
		myHClade.put("676773670017200","BOV_1");
		myHClade.put("676773672000000","BOV_1");
		myHClade.put("676773674000200","BOV_1");
		myHClade.put("676773674037200","BOV_1");
		myHClade.put("676773674037400","BOV_1");
		myHClade.put("676773674037600","BOV_1");
		myHClade.put("676773674143600","BOV_1");
		myHClade.put("676773674177400","BOV_1");
		myHClade.put("676773674177600","BOV_1");
		myHClade.put("676773674377600","BOV_1");
		myHClade.put("676773674777600","BOV_1");
		myHClade.put("676773675767400","BOV_1");
		myHClade.put("676773675777600","BOV_1");
		myHClade.put("676773676037600","BOV_1");
		myHClade.put("676773676177600","BOV_1");
		myHClade.put("676773676777200","BOV_1");
		myHClade.put("676773677077600","BOV_1");
		myHClade.put("676773677177600","BOV_1");
		myHClade.put("676773677377200","BOV_1");
		myHClade.put("676773677377600","BOV_1");
		myHClade.put("676773677407200","BOV_1");
		myHClade.put("676773677437600","BOV_1");
		myHClade.put("676773677577200","BOV_1");
		myHClade.put("676773677577600","BOV_1");
		myHClade.put("676773677600600","BOV_1");
		myHClade.put("676773677603600","BOV_1");
		myHClade.put("676773677637600","BOV_1");
		myHClade.put("676773677677200","BOV_1");
		myHClade.put("676773677677600","BOV_1");
		myHClade.put("676773677703600","BOV_1");
		myHClade.put("676773677737600","BOV_1");
		myHClade.put("676773677741600","BOV_1");
		myHClade.put("676773677743600","BOV_1");
		myHClade.put("676773677747600","BOV_1");
		myHClade.put("676773677757200","BOV_1");
		myHClade.put("676773677757600","BOV_1");
		myHClade.put("676773677761600","BOV_1");
		myHClade.put("676773677763600","BOV_1");
		myHClade.put("676773677767600","BOV_1");
		myHClade.put("676773677770600","BOV_1");
		myHClade.put("676773677771200","BOV_1");
		myHClade.put("676773677773600","BOV_1");
		myHClade.put("676773677774200","BOV_1");
		myHClade.put("676773677774600","BOV_1");
		myHClade.put("676773677775200","BOV_1");
		myHClade.put("676773677775600","BOV_1");
		myHClade.put("676773677776000","BOV_1");
		myHClade.put("676773677776200","BOV_1");
		myHClade.put("676773677776600","BOV_1");
		myHClade.put("676773677777000","BOV_1");
		myHClade.put("676773677777200","BOV_1");
		myHClade.put("676773677777400","BOV_1");
		myHClade.put("676773677777600","BOV_1");
		myHClade.put("676773677777700","Unknown");
		myHClade.put("676773707777600","BOV_1");
		myHClade.put("676773737766600","BOV_1");
		myHClade.put("676773737777600","BOV_1");
		myHClade.put("676773747777600","BOV_1");
		myHClade.put("676773757767600","BOV_1");
		myHClade.put("676773757777600","BOV_1");
		myHClade.put("676773764146600","BOV_1");
		myHClade.put("676773770000600","BOV_1");
		myHClade.put("676773770003600","BOV_1");
		myHClade.put("676773772000600","BOV_1");
		myHClade.put("676773774077600","BOV_1");
		myHClade.put("676773774177600","BOV_1");
		myHClade.put("676773774377600","BOV_1");
		myHClade.put("676773775777200","BOV_1");
		myHClade.put("676773776003600","BOV_1");
		myHClade.put("676773776037600","BOV_1");
		myHClade.put("676773776077600","BOV_1");
		myHClade.put("676773776237600","BOV_1");
		myHClade.put("676773776261600","BOV_1");
		myHClade.put("676773776274600","BOV_1");
		myHClade.put("676773776276600","BOV_1");
		myHClade.put("676773776277400","BOV_1");
		myHClade.put("676773776277600","BOV_1");
		myHClade.put("676773776617600","BOV_1");
		myHClade.put("676773776637600","BOV_1");
		myHClade.put("676773776677600","BOV_1");
		myHClade.put("676773776777600","BOV_1");
		myHClade.put("676773777045600","BOV_1");
		myHClade.put("676773777075600","BOV_1");
		myHClade.put("676773777377600","BOV_1");
		myHClade.put("676773777402600","BOV_1");
		myHClade.put("676773777437600","BOV_1");
		myHClade.put("676773777477600","BOV_1");
		myHClade.put("676773777577600","BOV_1");
		myHClade.put("676773777600600","BOV_1");
		myHClade.put("676773777617600","BOV_1");
		myHClade.put("676773777627600","BOV_1");
		myHClade.put("676773777637200","BOV_1");
		myHClade.put("676773777637600","BOV_1");
		myHClade.put("676773777647400","BOV_1");
		myHClade.put("676773777657600","BOV_1");
		myHClade.put("676773777673600","BOV_1");
		myHClade.put("676773777674600","BOV_1");
		myHClade.put("676773777677200","BOV_1");
		myHClade.put("676773777677600","BOV_1");
		myHClade.put("676773777700200","BOV_1");
		myHClade.put("676773777717600","BOV_1");
		myHClade.put("676773777735600","BOV_1");
		myHClade.put("676773777736600","BOV_1");
		myHClade.put("676773777737600","BOV_1");
		myHClade.put("676773777740200","BOV_1");
		myHClade.put("676773777743600","BOV_1");
		myHClade.put("676773777747000","BOV_1");
		myHClade.put("676773777747600","BOV_1");
		myHClade.put("676773777753600","BOV_1");
		myHClade.put("676773777757600","BOV_1");
		myHClade.put("676773777760200","BOV_1");
		myHClade.put("676773777764600","BOV_1");
		myHClade.put("676773777766600","BOV_1");
		myHClade.put("676773777767400","BOV_1");
		myHClade.put("676773777767600","BOV_1");
		myHClade.put("676773777770600","BOV_1");
		myHClade.put("676773777771600","BOV_1");
		myHClade.put("676773777773600","BOV_1");
		myHClade.put("676773777774600","BOV_1");
		myHClade.put("676773777775600","BOV_1");
		myHClade.put("676773777776200","BOV_1");
		myHClade.put("676773777776600","BOV_1");
		myHClade.put("676773777777000","BOV_1");
		myHClade.put("676773777777200","BOV_1");
		myHClade.put("676773777777600","BOV_1");
		myHClade.put("676773777777710","Unknown");
		myHClade.put("676776777760601","X2");
		myHClade.put("676776777777600","BOV");
		myHClade.put("676777477401771","Unknown");
		myHClade.put("676777607760771","LAM1");
		myHClade.put("676777673777600","BOV");
		myHClade.put("676777677777200","BOV");
		myHClade.put("676777677777600","BOV");
		myHClade.put("676777677777601","Unknown");
		myHClade.put("676777757777600","BOV");
		myHClade.put("676777777777600","BOV");
		myHClade.put("677001607700771","Unknown");
		myHClade.put("677003477413771","EAI2-Manila");
		myHClade.put("677177477413771","EAI2-Manila");
		myHClade.put("677300004760731","Turkey");
		myHClade.put("677317607760771","LAM2");
		myHClade.put("677327607760661","LAM2");
		myHClade.put("677336777760771","X1");
		myHClade.put("677337607660771","LAM2");
		myHClade.put("677337607760671","LAM2");
		myHClade.put("677337607760771","LAM2");
		myHClade.put("677347607760771","LAM1");
		myHClade.put("677367607760671","LAM1");
		myHClade.put("677367677413771","EAI5");
		myHClade.put("677367776020661","H3");
		myHClade.put("677367776177661","Unknown");
		myHClade.put("677377477413771","EAI2-Manila");
		myHClade.put("677377607740771","LAM1");
		myHClade.put("677377607763771","Manu2");
		myHClade.put("677377747760771","T1");
		myHClade.put("677377776000271","Unknown");
		myHClade.put("677377777760731","T2");
		myHClade.put("677400000001771","Unknown");
		myHClade.put("677400607760671","LAM2");
		myHClade.put("677417447413771","EAI2-Manila");
		myHClade.put("677467607760771","LAM1");
		myHClade.put("677477776000331","Unknown");
		myHClade.put("677477777760771","T1");
		myHClade.put("677567607760670","LAM1");
		myHClade.put("677576777760571","X1");
		myHClade.put("677577457413771","EAI2-Manila");
		myHClade.put("677577477413771","EAI2-Manila");
		myHClade.put("677577777413071","EAI5");
		myHClade.put("677607477403771","Unknown");
		myHClade.put("677607477413731","EAI2-Manila");
		myHClade.put("677617407413371","EAI2");
		myHClade.put("677636207760771","LAM2");
		myHClade.put("677637477413771","EAI2-Manila");
		myHClade.put("677637607600740","Unknown");
		myHClade.put("677637607760011","LAM2");
		myHClade.put("677637607760711","LAM2");
		myHClade.put("677637607760771","LAM2");
		myHClade.put("677640007760771","LAM1");
		myHClade.put("677677417741671","Unknown");
		myHClade.put("677677417760771","T1");
		myHClade.put("677677475413470","EAI2-Manila");
		myHClade.put("677677477413771","EAI2-Manila");
		myHClade.put("677677606000171","Unknown");
		myHClade.put("677677607741771","Unknown");
		myHClade.put("677677607760771","LAM1");
		myHClade.put("677700000160771","Unknown");
		myHClade.put("677703777413731","EAI1-SOM");
		myHClade.put("677707477412771","EAI2-Manila");
		myHClade.put("677707477413771","EAI2-Manila");
		myHClade.put("677707607700771","Unknown");
		myHClade.put("677707607760671","LAM2");
		myHClade.put("677707607760711","LAM2");
		myHClade.put("677707607760771","LAM2");
		myHClade.put("677717607760771","LAM2");
		myHClade.put("677717777760771","T1");
		myHClade.put("677721607760271","LAM2");
		myHClade.put("677723607400000","Unknown");
		myHClade.put("677727607760671","LAM2");
		myHClade.put("677727607760751","LAM2");
		myHClade.put("677727607760771","LAM2");
		myHClade.put("677727655772671","Manu1");
		myHClade.put("677730207760771","LAM2");
		myHClade.put("677735607560771","LAM2");
		myHClade.put("677735607760771","LAM");
		myHClade.put("677737207760771","LAM2");
		myHClade.put("677737470413771","EAI2-Manila");
		myHClade.put("677737477413771","EAI2-Manila");
		myHClade.put("677737477573771","Manu1");
		myHClade.put("677737600000000","Unknown");
		myHClade.put("677737600060771","T3");
		myHClade.put("677737600760771","T3");
		myHClade.put("677737601760771","LAM");
		myHClade.put("677737603760771","T3");
		myHClade.put("677737605620000","H3");
		myHClade.put("677737606160731","LAM2");
		myHClade.put("677737606360771","LAM2");
		myHClade.put("677737606760771","LAM2");
		myHClade.put("677737607400731","Unknown");
		myHClade.put("677737607560171","LAM2");
		myHClade.put("677737607560771","LAM2");
		myHClade.put("677737607600740","Unknown");
		myHClade.put("677737607600771","Unknown");
		myHClade.put("677737607620760","H3");
		myHClade.put("677737607620771","Unknown");
		myHClade.put("677737607660771","LAM2");
		myHClade.put("677737607700011","Unknown");
		myHClade.put("677737607700371","Unknown");
		myHClade.put("677737607700771","Unknown");
		myHClade.put("677737607720771","LAM");
		myHClade.put("677737607760011","LAM2");
		myHClade.put("677737607760031","LAM2");
		myHClade.put("677737607760351","LAM2");
		myHClade.put("677737607760371","LAM2");
		myHClade.put("677737607760471","LAM2");
		myHClade.put("677737607760571","LAM2");
		myHClade.put("677737607760611","LAM");
		myHClade.put("677737607760631","LAM2");
		myHClade.put("677737607760671","LAM2");
		myHClade.put("677737607760700","LAM2");
		myHClade.put("677737607760711","LAM");
		myHClade.put("677737607760731","LAM2");
		myHClade.put("677737607760751","LAM2");
		myHClade.put("677737607760760","LAM2");
		myHClade.put("677737607760770","LAM2");
		myHClade.put("677737607760771","LAM2");
		myHClade.put("677737607763771","Manu2");
		myHClade.put("677737607770731","Manu3");
		myHClade.put("677737607770771","Manu3");
		myHClade.put("677737707760571","T3");
		myHClade.put("677737707760711","T3");
		myHClade.put("677737707760770","T3");
		myHClade.put("677737707760771","T3");
		myHClade.put("677737777413711","EAI5");
		myHClade.put("677737777420771","Ural-1");
		myHClade.put("677737777460771","T3");
		myHClade.put("677737777760731","T");
		myHClade.put("677737777760771","T3");
		myHClade.put("677740002013671","EAI2");
		myHClade.put("677740017413771","EAI2");
		myHClade.put("677740077413771","EAI2");
		myHClade.put("677740204060771","LAM1");
		myHClade.put("677742000001771","Unknown");
		myHClade.put("677743477413771","EAI2-Manila");
		myHClade.put("677747607560671","LAM6");
		myHClade.put("677747607760771","LAM1");
		myHClade.put("677747757760771","T5");
		myHClade.put("677757477413771","EAI2-Manila");
		myHClade.put("677757607760771","LAM1");
		myHClade.put("677757747413071","EAI6-BGD1");
		myHClade.put("677757777720771","H3");
		myHClade.put("677757777760771","T1");
		myHClade.put("677760007760771","LAM-RUS");
		myHClade.put("677760077413771","EAI2");
		myHClade.put("677760077753271","Manu1");
		myHClade.put("677761607760771","LAM1");
		myHClade.put("677761777760771","T1");
		myHClade.put("677763777760731","T2");
		myHClade.put("677767077413671","EAI2");
		myHClade.put("677767377413771","EAI5");
		myHClade.put("677767477403631","Unknown");
		myHClade.put("677767477403771","Unknown");
		myHClade.put("677767477413771","EAI2-Manila");
		myHClade.put("677767606740371","LAM8");
		myHClade.put("677767606760771","LAM8");
		myHClade.put("677767607740771","LAM1");
		myHClade.put("677767607760671","LAM1");
		myHClade.put("677767607760771","LAM1");
		myHClade.put("677767617760771","T1");
		myHClade.put("677767637563771","Manu2");
		myHClade.put("677767676003771","Unknown");
		myHClade.put("677767677413731","EAI1-SOM");
		myHClade.put("677767677413771","EAI5");
		myHClade.put("677767774000371","Unknown");
		myHClade.put("677767776160771","T1");
		myHClade.put("677767777760071","T1");
		myHClade.put("677767777760771","T1");
		myHClade.put("677770000000771","Unknown");
		myHClade.put("677770003413771","EAI2");
		myHClade.put("677773477413771","EAI2-Manila");
		myHClade.put("677773607760771","LAM1");
		myHClade.put("677773777760730","T2-Uganda");
		myHClade.put("677773777777600","BOV");
		myHClade.put("677774000413771","EAI2");
		myHClade.put("677774207760711","LAM");
		myHClade.put("677774207760771","LAM1");
		myHClade.put("677774477413771","EAI2-Manila");
		myHClade.put("677774764060731","T2");
		myHClade.put("677775000001771","Unknown");
		myHClade.put("677775607763771","Manu2");
		myHClade.put("677775764060731","T2");
		myHClade.put("677775777413771","EAI5");
		myHClade.put("677776017413771","EAI2");
		myHClade.put("677776037413771","EAI2");
		myHClade.put("677776377760571","T1");
		myHClade.put("677776477413471","EAI2-Manila");
		myHClade.put("677776477413771","EAI2-Manila");
		myHClade.put("677776607760771","LAM1");
		myHClade.put("677776777413071","EAI5");
		myHClade.put("677776777560771","X1");
		myHClade.put("677776777760571","X1");
		myHClade.put("677776777760601","X2");
		myHClade.put("677776777760771","X1");
		myHClade.put("677776777773671","Manu1");
		myHClade.put("677777000000331","Unknown");
		myHClade.put("677777037760771","T1");
		myHClade.put("677777077412771","EAI2");
		myHClade.put("677777077413701","EAI2");
		myHClade.put("677777077413771","EAI2");
		myHClade.put("677777177413771","EAI5");
		myHClade.put("677777207760731","LAM");
		myHClade.put("677777207760771","LAM1");
		myHClade.put("677777277473771","Manu1");
		myHClade.put("677777377413771","EAI5");
		myHClade.put("677777377700771","Unknown");
		myHClade.put("677777377760731","T");
		myHClade.put("677777400060571","T1");
		myHClade.put("677777400760771","T1");
		myHClade.put("677777402003771","Unknown");
		myHClade.put("677777404760771","Turkey");
		myHClade.put("677777407413771","EAI2");
		myHClade.put("677777407760771","LAM1");
		myHClade.put("677777423560771","T1");
		myHClade.put("677777437413771","EAI2");
		myHClade.put("677777440003770","Unknown");
		myHClade.put("677777457413771","EAI2-Manila");
		myHClade.put("677777463413701","EAI2-Manila");
		myHClade.put("677777470000000","EAI2-Manila");
		myHClade.put("677777470000071","EAI2-Manila");
		myHClade.put("677777470000771","Unknown");
		myHClade.put("677777470003731","Unknown");
		myHClade.put("677777470003771","Unknown");
		myHClade.put("677777471413771","EAI2-Manila");
		myHClade.put("677777472000000","Unknown");
		myHClade.put("677777472000071","Unknown");
		myHClade.put("677777473413771","EAI2-Manila");
		myHClade.put("677777474003771","Unknown");
		myHClade.put("677777475000051","Unknown");
		myHClade.put("677777475413701","EAI2-Manila");
		myHClade.put("677777475413771","EAI2-Manila");
		myHClade.put("677777476003771","Unknown");
		myHClade.put("677777477003771","Unknown");
		myHClade.put("677777477013771","EAI2-Manila");
		myHClade.put("677777477060771","T-H37Rv");
		myHClade.put("677777477400771","Unknown");
		myHClade.put("677777477403771","EAI2-Manila");
		myHClade.put("677777477410071","EAI2-Manila");
		myHClade.put("677777477412771","EAI2-Manila");
		myHClade.put("677777477413000","EAI2-Manila");
		myHClade.put("677777477413001","EAI2-Manila");
		myHClade.put("677777477413071","EAI2-Manila");
		myHClade.put("677777477413151","EAI2-Manila");
		myHClade.put("677777477413371","EAI2-Manila");
		myHClade.put("677777477413400","EAI2-Manila");
		myHClade.put("677777477413470","EAI2-Manila");
		myHClade.put("677777477413601","EAI2-Manila");
		myHClade.put("677777477413671","EAI2-Manila");
		myHClade.put("677777477413700","EAI2-Manila");
		myHClade.put("677777477413701","EAI2-Manila");
		myHClade.put("677777477413711","EAI2-Manila");
		myHClade.put("677777477413731","EAI2-Manila");
		myHClade.put("677777477413751","EAI2-Manila");
		myHClade.put("677777477413761","EAI2-Manila");
		myHClade.put("677777477413770","EAI2-Manila");
		myHClade.put("677777477413771","EAI2-Manila");
		myHClade.put("677777477417771","Unknown");
		myHClade.put("677777577013771","EAI5");
		myHClade.put("677777577413671","EAI5");
		myHClade.put("677777577413771","EAI5");
		myHClade.put("677777600000000","Unknown");
		myHClade.put("677777600000371","Unknown");
		myHClade.put("677777600060771","T");
		myHClade.put("677777600760771","T1");
		myHClade.put("677777601760771","T1");
		myHClade.put("677777604740771","LAM1");
		myHClade.put("677777604760771","LAM1");
		myHClade.put("677777606060731","LAM11-ZWE");
		myHClade.put("677777606060771","LAM11-ZWE");
		myHClade.put("677777606760771","LAM8");
		myHClade.put("677777607060731","LAM");
		myHClade.put("677777607060771","LAM1");
		myHClade.put("677777607560771","LAM6");
		myHClade.put("677777607660001","LAM");
		myHClade.put("677777607660611","LAM");
		myHClade.put("677777607660711","LAM");
		myHClade.put("677777607660771","LAM1");
		myHClade.put("677777607670771","Manu3");
		myHClade.put("677777607700031","Unknown");
		myHClade.put("677777607700071","Unknown");
		myHClade.put("677777607700371","Unknown");
		myHClade.put("677777607700771","Unknown");
		myHClade.put("677777607740011","LAM");
		myHClade.put("677777607740771","LAM1");
		myHClade.put("677777607741771","Unknown");
		myHClade.put("677777607760071","LAM1");
		myHClade.put("677777607760171","LAM1");
		myHClade.put("677777607760371","LAM");
		myHClade.put("677777607760671","LAM1");
		myHClade.put("677777607760700","LAM");
		myHClade.put("677777607760701","LAM");
		myHClade.put("677777607760711","LAM");
		myHClade.put("677777607760730","LAM");
		myHClade.put("677777607760731","LAM");
		myHClade.put("677777607760740","LAM1");
		myHClade.put("677777607760741","LAM1");
		myHClade.put("677777607760751","LAM1");
		myHClade.put("677777607760771","LAM1");
		myHClade.put("677777607763771","Manu2");
		myHClade.put("677777607770771","Manu3");
		myHClade.put("677777617760371","T1");
		myHClade.put("677777617760771","T1");
		myHClade.put("677777627760771","T1");
		myHClade.put("677777637760671","T1");
		myHClade.put("677777677413751","EAI5");
		myHClade.put("677777677413771","EAI5");
		myHClade.put("677777700000371","Unknown");
		myHClade.put("677777703760771","T1");
		myHClade.put("677777707760371","T1");
		myHClade.put("677777707760771","T1");
		myHClade.put("677777717720771","H3");
		myHClade.put("677777735740771","T1");
		myHClade.put("677777743460771","Cameroon");
		myHClade.put("677777744030771","Manu3");
		myHClade.put("677777747560771","T1");
		myHClade.put("677777747760731","T2");
		myHClade.put("677777750403371","EAI7-BGD2");
		myHClade.put("677777750413371","EAI6-BGD1");
		myHClade.put("677777752403371","Unknown");
		myHClade.put("677777757013371","EAI5");
		myHClade.put("677777757760771","T5");
		myHClade.put("677777757770771","Manu3");
		myHClade.put("677777761760771","T1");
		myHClade.put("677777770000000","Unknown");
		myHClade.put("677777770020771","H3");
		myHClade.put("677777770060731","T2");
		myHClade.put("677777771413731","EAI1-SOM");
		myHClade.put("677777771760771","T1");
		myHClade.put("677777773760700","T1");
		myHClade.put("677777774020771","H1");
		myHClade.put("677777775760771","T1");
		myHClade.put("677777776000371","Unknown");
		myHClade.put("677777776620571","H3");
		myHClade.put("677777777003771","Unknown");
		myHClade.put("677777777360371","T1");
		myHClade.put("677777777400000","Unknown");
		myHClade.put("677777777403701","Unknown");
		myHClade.put("677777777403771","EAI5");
		myHClade.put("677777777412071","EAI5");
		myHClade.put("677777777413071","EAI5");
		myHClade.put("677777777413371","EAI5");
		myHClade.put("677777777413631","EAI5");
		myHClade.put("677777777413671","EAI5");
		myHClade.put("677777777413700","EAI5");
		myHClade.put("677777777413731","EAI1-SOM");
		myHClade.put("677777777413771","EAI5");
		myHClade.put("677777777420731","Ural-1");
		myHClade.put("677777777560771","T1");
		myHClade.put("677777777620771","H3");
		myHClade.put("677777777660771","T1");
		myHClade.put("677777777700771","Unknown");
		myHClade.put("677777777720571","H3");
		myHClade.put("677777777720631","H3");
		myHClade.put("677777777720731","H3");
		myHClade.put("677777777720771","H3");
		myHClade.put("677777777730671","Manu3");
		myHClade.put("677777777730770","Manu3");
		myHClade.put("677777777730771","Manu3");
		myHClade.put("677777777740771","T1");
		myHClade.put("677777777757771","Unknown");
		myHClade.put("677777777760411","T1");
		myHClade.put("677777777760601","T1");
		myHClade.put("677777777760671","T1");
		myHClade.put("677777777760700","T1");
		myHClade.put("677777777760730","T2-Uganda");
		myHClade.put("677777777760731","T2");
		myHClade.put("677777777760771","T1");
		myHClade.put("677777777763771","Manu2");
		myHClade.put("677777777770771","Manu3");
		myHClade.put("677777777773771","Manu1");
		myHClade.put("677777777777600","Unknown");
		myHClade.put("700000000000000","CAS");
		myHClade.put("700000000000011","CAS");
		myHClade.put("700000000000031","CAS");
		myHClade.put("700000000000071","CAS");
		myHClade.put("700000000000371","CAS");
		myHClade.put("700000000000771","Unknown");
		myHClade.put("700000000003771","CAS");
		myHClade.put("700000000020071","Unknown");
		myHClade.put("700000000037641","AFRI");
		myHClade.put("700000000403371","Unknown");
		myHClade.put("700000001413371","EAI5");
		myHClade.put("700000004020711","H1");
		myHClade.put("700000004020771","H1");
		myHClade.put("700000004037771","Ethiopian");
		myHClade.put("700000004147740","Ethiopian");
		myHClade.put("700000004177771","Ethiopian");
		myHClade.put("700000007175771","Ethiopian");
		myHClade.put("700000007177770","Ethiopian");
		myHClade.put("700000007177771","Ethiopian");
		myHClade.put("700000007413371","EAI5");
		myHClade.put("700000007560771","LAM3");
		myHClade.put("700000007720771","H3");
		myHClade.put("700000007760771","LAM3");
		myHClade.put("700000067413371","EAI5");
		myHClade.put("700000074020771","H1");
		myHClade.put("700000077740771","T1");
		myHClade.put("700000207760621","LAM3");
		myHClade.put("700000207760771","LAM3");
		myHClade.put("700000377776671","AFRI_1");
		myHClade.put("700000400000171","CAS");
		myHClade.put("700000740003711","CAS2");
		myHClade.put("700000740003771","CAS2");
		myHClade.put("700000774413771","EAI4-VNM");
		myHClade.put("700001400000061","CAS");
		myHClade.put("700001403760371","T1");
		myHClade.put("700001774020771","H1");
		myHClade.put("700001777413771","EAI5");
		myHClade.put("700002000000000","CAS");
		myHClade.put("700003600010000","EAI5");
		myHClade.put("700003607760771","LAM3");
		myHClade.put("700003760000171","Unknown");
		myHClade.put("700003774020771","H1");
		myHClade.put("700005747413771","EAI6-BGD1");
		myHClade.put("700007400003771","CAS");
		myHClade.put("700007740003170","CAS2");
		myHClade.put("700007777770371","Unknown");
		myHClade.put("700016777760771","X3");
		myHClade.put("700017740000000","CAS");
		myHClade.put("700017770360771","T1");
		myHClade.put("700017777760771","T1");
		myHClade.put("700020047177771","Unknown");
		myHClade.put("700026777760771","X3");
		myHClade.put("700027177760771","T1");
		myHClade.put("700027607760731","LAM3");
		myHClade.put("700036007760771","LAM3");
		myHClade.put("700036377760771","T1");
		myHClade.put("700036477760731","X3");
		myHClade.put("700036774000011","Unknown");
		myHClade.put("700036775560771","X3");
		myHClade.put("700036775760771","X3");
		myHClade.put("700036776060771","X3");
		myHClade.put("700036776360771","X3");
		myHClade.put("700036776760771","X3");
		myHClade.put("700036777400071","AFRI");
		myHClade.put("700036777560471","X3");
		myHClade.put("700036777560771","X3");
		myHClade.put("700036777740031","X3");
		myHClade.put("700036777740171","X3");
		myHClade.put("700036777740771","X3");
		myHClade.put("700036777760031","X3");
		myHClade.put("700036777760471","X3");
		myHClade.put("700036777760631","X3");
		myHClade.put("700036777760671","X3");
		myHClade.put("700036777760711","X3");
		myHClade.put("700036777760730","X3");
		myHClade.put("700036777760731","X3");
		myHClade.put("700036777760771","X3");
		myHClade.put("700036777763771","Manu2");
		myHClade.put("700037000020700","H3");
		myHClade.put("700037177760771","T1");
		myHClade.put("700037377760711","T4");
		myHClade.put("700037607760771","LAM3");
		myHClade.put("700037717560600","T1");
		myHClade.put("700037750060771","T5");
		myHClade.put("700037776360771","T1");
		myHClade.put("700037777560771","T1");
		myHClade.put("700037777720771","H3");
		myHClade.put("700037777760731","T2");
		myHClade.put("700037777760771","Unknown");
		myHClade.put("700040777760771","T1");
		myHClade.put("700046677760671","X3");
		myHClade.put("700047777740600","T1");
		myHClade.put("700056777740371","X3");
		myHClade.put("700056777760771","X3");
		myHClade.put("700060777760771","T1");
		myHClade.put("700066777400771","Unknown");
		myHClade.put("700066777760661","X3");
		myHClade.put("700066777760671","X3");
		myHClade.put("700066777760700","X3");
		myHClade.put("700066777760771","X3");
		myHClade.put("700066777763771","Manu2");
		myHClade.put("700074777760771","T1");
		myHClade.put("700075717760700","T1");
		myHClade.put("700075777760771","T1");
		myHClade.put("700076377760771","T1");
		myHClade.put("700076657760771","X3");
		myHClade.put("700076677760771","X3");
		myHClade.put("700076700000071","CAS");
		myHClade.put("700076717760700","X3");
		myHClade.put("700076717760771","X3");
		myHClade.put("700076737760771","X3");
		myHClade.put("700076747760771","X3");
		myHClade.put("700076750000071","AFRI");
		myHClade.put("700076757760771","X3");
		myHClade.put("700076760000011","X");
		myHClade.put("700076760000071","AFRI");
		myHClade.put("700076770000071","AFRI");
		myHClade.put("700076774160771","X3");
		myHClade.put("700076774360771","X3");
		myHClade.put("700076777700771","Unknown");
		myHClade.put("700076777740000","X3");
		myHClade.put("700076777740271","X3");
		myHClade.put("700076777740371","X3");
		myHClade.put("700076777740771","X3");
		myHClade.put("700076777760271","X3");
		myHClade.put("700076777760600","X3");
		myHClade.put("700076777760661","X3");
		myHClade.put("700076777760671","X3");
		myHClade.put("700076777760700","X3");
		myHClade.put("700076777760711","X3");
		myHClade.put("700076777760731","X3");
		myHClade.put("700076777760760","X3");
		myHClade.put("700076777760770","X3");
		myHClade.put("700076777760771","X3");
		myHClade.put("700076777763771","Manu2");
		myHClade.put("700077377760671","T4");
		myHClade.put("700077740000371","CAS");
		myHClade.put("700077740003771","CAS2");
		myHClade.put("700077774020731","H1");
		myHClade.put("700077777413771","EAI5");
		myHClade.put("700077777760700","T1");
		myHClade.put("700077777760771","T1");
		myHClade.put("700077777767671","AFRI");
		myHClade.put("700137617000000","Unknown");
		myHClade.put("700167760001771","Unknown");
		myHClade.put("700177740003771","CAS2");
		myHClade.put("700177760000171","Unknown");
		myHClade.put("700177774020731","H1");
		myHClade.put("700177777413731","EAI1-SOM");
		myHClade.put("700177777760771","T1");
		myHClade.put("700277740003711","CAS2");
		myHClade.put("700277740003771","CAS2");
		myHClade.put("700277740007001","Unknown");
		myHClade.put("700337400003771","CAS");
		myHClade.put("700337740003631","CAS2");
		myHClade.put("700337740003771","CAS2");
		myHClade.put("700343740003771","CAS2");
		myHClade.put("700367740003771","CAS2");
		myHClade.put("700373740003771","CAS2");
		myHClade.put("700376340003371","CAS2");
		myHClade.put("700377000003771","CAS");
		myHClade.put("700377040003771","CAS2");
		myHClade.put("700377400000771","CAS");
		myHClade.put("700377400003771","CAS");
		myHClade.put("700377600003771","CAS");
		myHClade.put("700377640001771","CAS");
		myHClade.put("700377700000031","CAS");
		myHClade.put("700377700000771","CAS");
		myHClade.put("700377700001771","CAS");
		myHClade.put("700377700003771","CAS");
		myHClade.put("700377707776671","AFRI");
		myHClade.put("700377740000011","CAS");
		myHClade.put("700377740000171","CAS");
		myHClade.put("700377740000771","CAS");
		myHClade.put("700377740001740","CAS2");
		myHClade.put("700377740001771","CAS");
		myHClade.put("700377740002771","CAS2");
		myHClade.put("700377740003371","CAS2");
		myHClade.put("700377740003671","CAS2");
		myHClade.put("700377740003711","CAS2");
		myHClade.put("700377740003731","CAS2");
		myHClade.put("700377740003751","CAS2");
		myHClade.put("700377740003771","CAS2");
		myHClade.put("700377777760731","T2");
		myHClade.put("700377777760771","T1");
		myHClade.put("700623405760471","LAM5");
		myHClade.put("700677747413771","EAI6-BGD1");
		myHClade.put("700700000003771","CAS");
		myHClade.put("700736777760771","X1");
		myHClade.put("700737740003571","CAS");
		myHClade.put("700737740003771","CAS");
		myHClade.put("700737757760771","T");
		myHClade.put("700740007774671","Unknown");
		myHClade.put("700741007764671","AFRI");
		myHClade.put("700741007774671","AFRI");
		myHClade.put("700741407774671","AFRI");
		myHClade.put("700764007413771","EAI5");
		myHClade.put("700765744013671","EAI5");
		myHClade.put("700765747413771","EAI6-BGD1");
		myHClade.put("700767747413771","EAI6-BGD1");
		myHClade.put("700774007403771","Unknown");
		myHClade.put("700774007413771","EAI5");
		myHClade.put("700774747413771","EAI6-BGD1");
		myHClade.put("700775745403771","Unknown");
		myHClade.put("700775746013771","EAI5");
		myHClade.put("700775747413771","EAI6-BGD1");
		myHClade.put("700775747417771","Unknown");
		myHClade.put("700775747473771","Manu1");
		myHClade.put("700775776013771","EAI5");
		myHClade.put("700776747413771","EAI6-BGD1");
		myHClade.put("700777600020771","H3");
		myHClade.put("700777700003771","CAS");
		myHClade.put("700777740003171","CAS");
		myHClade.put("700777740003711","CAS");
		myHClade.put("700777740003771","CAS");
		myHClade.put("700777746013771","EAI5");
		myHClade.put("700777747403771","Unknown");
		myHClade.put("700777747410771","EAI6-BGD1");
		myHClade.put("700777747413671","EAI6-BGD1");
		myHClade.put("700777747413731","EAI");
		myHClade.put("700777747413771","EAI6-BGD1");
		myHClade.put("700777770403731","Unknown");
		myHClade.put("700777777413771","EAI5");
		myHClade.put("700777777760731","T2");
		myHClade.put("700777777760771","T1");
		myHClade.put("701060400003230","CAS");
		myHClade.put("701341000000771","CAS");
		myHClade.put("701347000000771","CAS");
		myHClade.put("701347604001661","Unknown");
		myHClade.put("701357777773771","Manu1");
		myHClade.put("701367740003671","CAS");
		myHClade.put("701377400001771","CAS");
		myHClade.put("701377740003771","CAS");
		myHClade.put("701467740003671","CAS");
		myHClade.put("701573600001771","CAS");
		myHClade.put("701577600023171","Manu2");
		myHClade.put("701721000003771","CAS");
		myHClade.put("701727677760771","T3");
		myHClade.put("701737740003771","CAS");
		myHClade.put("701737777740471","T3");
		myHClade.put("701740007774671","Unknown");
		myHClade.put("701767600003671","CAS");
		myHClade.put("701767607413731","EAI1-SOM");
		myHClade.put("701767700003071","CAS");
		myHClade.put("701767740003070","CAS");
		myHClade.put("701767740003671","CAS");
		myHClade.put("701767774760071","T1");
		myHClade.put("701767777700011","Unknown");
		myHClade.put("701773740003771","CAS");
		myHClade.put("701773760003171","Unknown");
		myHClade.put("701774777720771","H3");
		myHClade.put("701775747413771","EAI6-BGD1");
		myHClade.put("701776000000001","CAS");
		myHClade.put("701776617760601","X2");
		myHClade.put("701776777760601","X2");
		myHClade.put("701777440003771","CAS");
		myHClade.put("701777600001771","CAS");
		myHClade.put("701777607560771","LAM6");
		myHClade.put("701777640003771","CAS");
		myHClade.put("701777700000371","CAS");
		myHClade.put("701777740003171","CAS");
		myHClade.put("701777740003771","CAS1-Delhi");
		myHClade.put("701777743760751","Cameroon");
		myHClade.put("701777747770371","Unknown");
		myHClade.put("701777777403771","Unknown");
		myHClade.put("701777777413731","EAI1-SOM");
		myHClade.put("701777777720771","H3");
		myHClade.put("702000000000771","CAS");
		myHClade.put("702003347770471","Manu3");
		myHClade.put("702037740003771","CAS1-Delhi");
		myHClade.put("702077400001771","CAS");
		myHClade.put("702077740003771","CAS1-Delhi");
		myHClade.put("702177740003171","CAS1-Delhi");
		myHClade.put("702337740003171","CAS1-Delhi");
		myHClade.put("702337740013171","EAI5");
		myHClade.put("702371001760771","S");
		myHClade.put("702377400001771","CAS");
		myHClade.put("702377740003771","CAS1-Delhi");
		myHClade.put("702377777760771","S");
		myHClade.put("702767740003771","CAS1-Delhi");
		myHClade.put("702777000003771","CAS");
		myHClade.put("702777340003771","CAS1-Delhi");
		myHClade.put("702777707763771","Manu2");
		myHClade.put("702777740002771","CAS1-Delhi");
		myHClade.put("702777740003171","CAS1-Delhi");
		myHClade.put("702777740003731","CAS1-Delhi");
		myHClade.put("702777740003771","CAS1-Delhi");
		myHClade.put("703000000000371","CAS");
		myHClade.put("703000000000771","CAS");
		myHClade.put("703000000003171","CAS");
		

		
		if(!myHClade.containsKey(spoligotype)){
			return "ND";
		}
		else{
			return myHClade.get(spoligotype);
		}
		
	}
	
	/**
	 * Method allowing to get Clade_Expert lineage information (contained in SITVIT2)
	 * @param spoligotype
	 * @return
	 * @throws Exception
	 */
	public static String getCladeExpert2(String spoligotype) throws Exception{
		HashMap<String,String> myHClade = new HashMap<String,String>();
		
		
		myHClade.put("703000740003771","CAS1-Delhi");
		myHClade.put("703001740003771","CAS1-Delhi");
		myHClade.put("703177400001771","CAS1-Kili");
		myHClade.put("703177740003031","CAS1-Delhi");
		myHClade.put("703177740003771","CAS1-Delhi");
		myHClade.put("703227660003771","Unknown");
		myHClade.put("703237440003771","CAS1-Delhi");
		myHClade.put("703247640003661","CAS1-Delhi");
		myHClade.put("703307740003771","CAS1-Delhi");
		myHClade.put("703337400001771","CAS1-Kili");
		myHClade.put("703337740000771","CAS");
		myHClade.put("703347740003671","CAS1-Delhi");
		myHClade.put("703347740003771","CAS1-Delhi");
		myHClade.put("703357400001771","CAS1-Kili");
		myHClade.put("703363464001771","Unknown");
		myHClade.put("703367400001771","CAS1-Kili");
		myHClade.put("703367607543771","Unknown");
		myHClade.put("703367740003771","CAS1-Delhi");
		myHClade.put("703370000000000","CAS");
		myHClade.put("703377200001771","CAS");
		myHClade.put("703377400000771","CAS");
		myHClade.put("703377400001471","CAS1-Kili");
		myHClade.put("703377400001731","CAS1-Kili");
		myHClade.put("703377400001771","CAS1-Kili");
		myHClade.put("703377400003771","CAS1-Kili");
		myHClade.put("703377400011771","EAI5");
		myHClade.put("703377400203771","Unknown");
		myHClade.put("703377407761771","Unknown");
		myHClade.put("703377420001771","Unknown");
		myHClade.put("703377440001771","CAS");
		myHClade.put("703377600001771","CAS");
		myHClade.put("703377700001771","CAS");
		myHClade.put("703377700003171","CAS");
		myHClade.put("703377740000771","CAS");
		myHClade.put("703377740003171","CAS1-Delhi");
		myHClade.put("703377740003661","CAS1-Delhi");
		myHClade.put("703377740003771","CAS1-Delhi");
		myHClade.put("703377740005471","Unknown");
		myHClade.put("703377777400371","Unknown");
		myHClade.put("703377777761771","Unknown");
		myHClade.put("703377777763771","Manu2");
		myHClade.put("703400000000000","CAS");
		myHClade.put("703417740003071","CAS1-Delhi");
		myHClade.put("703417740003771","CAS1-Delhi");
		myHClade.put("703457740003771","CAS1-Delhi");
		myHClade.put("703577640003771","CAS1-Delhi");
		myHClade.put("703577740003171","CAS1-Delhi");
		myHClade.put("703600000003771","CAS");
		myHClade.put("703601000003771","CAS");
		myHClade.put("703601740003771","CAS1-Delhi");
		myHClade.put("703607740003171","CAS1-Delhi");
		myHClade.put("703607740003771","CAS1-Delhi");
		myHClade.put("703627400003771","CAS");
		myHClade.put("703637400000000","CAS");
		myHClade.put("703637400003771","CAS");
		myHClade.put("703637700003771","CAS");
		myHClade.put("703637740000771","CAS");
		myHClade.put("703637740002771","CAS1-Delhi");
		myHClade.put("703637740003731","CAS1-Delhi");
		myHClade.put("703637740003771","CAS1-Delhi");
		myHClade.put("703637745770371","Unknown");
		myHClade.put("703660000000331","CAS");
		myHClade.put("703676340003171","CAS1-Delhi");
		myHClade.put("703677400003771","CAS");
		myHClade.put("703677540003771","CAS1-Delhi");
		myHClade.put("703677740000371","CAS");
		myHClade.put("703677740003171","CAS1-Delhi");
		myHClade.put("703677740003771","CAS1-Delhi");
		myHClade.put("703700000000760","CAS");
		myHClade.put("703700040003771","CAS1-Delhi");
		myHClade.put("703700340003771","CAS1-Delhi");
		myHClade.put("703701000000000","CAS");
		myHClade.put("703701000000760","CAS");
		myHClade.put("703701740003171","CAS1-Delhi");
		myHClade.put("703701740003771","CAS1-Delhi");
		myHClade.put("703703700003771","CAS");
		myHClade.put("703707730003771","Unknown");
		myHClade.put("703707740001771","CAS");
		myHClade.put("703707740003471","CAS1-Delhi");
		myHClade.put("703707740003571","CAS1-Delhi");
		myHClade.put("703707740003731","CAS1-Delhi");
		myHClade.put("703707740003770","CAS1-Delhi");
		myHClade.put("703707740003771","CAS1-Delhi");
		myHClade.put("703717700000371","CAS");
		myHClade.put("703717740003771","CAS1-Delhi");
		myHClade.put("703717740013171","EAI5");
		myHClade.put("703721000000371","CAS");
		myHClade.put("703721740003471","CAS1-Delhi");
		myHClade.put("703723740003771","CAS1-Delhi");
		myHClade.put("703727400001771","CAS");
		myHClade.put("703727740003071","CAS1-Delhi");
		myHClade.put("703727740003111","CAS1-Delhi");
		myHClade.put("703727740003171","CAS1-Delhi");
		myHClade.put("703727740003771","CAS1-Delhi");
		myHClade.put("703736140003171","CAS1-Delhi");
		myHClade.put("703737000003371","CAS");
		myHClade.put("703737400001771","CAS");
		myHClade.put("703737740000000","CAS");
		myHClade.put("703737740001171","CAS1-Delhi");
		myHClade.put("703737740002771","CAS1-Delhi");
		myHClade.put("703737740003171","CAS1-Delhi");
		myHClade.put("703737740003371","CAS1-Delhi");
		myHClade.put("703737740003731","CAS1-Delhi");
		myHClade.put("703737740003771","CAS1-Delhi");
		myHClade.put("703737740005471","Unknown");
		myHClade.put("703737740006371","CAS");
		myHClade.put("703737740011571","EAI5");
		myHClade.put("703737740013171","EAI5");
		myHClade.put("703737770003171","Unknown");
		myHClade.put("703737777760771","T3");
		myHClade.put("703740000000700","CAS");
		myHClade.put("703740000000760","CAS");
		myHClade.put("703740000000771","CAS");
		myHClade.put("703740000003171","CAS");
		myHClade.put("703740000003731","CAS");
		myHClade.put("703740000003760","CAS");
		myHClade.put("703740000003771","CAS");
		myHClade.put("703741000000331","CAS");
		myHClade.put("703741000000700","CAS");
		myHClade.put("703741000000760","CAS");
		myHClade.put("703741000000770","CAS");
		myHClade.put("703741000003771","CAS1-Delhi");
		myHClade.put("703741444002371","Unknown");
		myHClade.put("703741600003771","CAS");
		myHClade.put("703743740003471","CAS1-Delhi");
		myHClade.put("703743740003771","CAS1-Delhi");
		myHClade.put("703747640003771","CAS1-Delhi");
		myHClade.put("703747740003471","CAS1-Delhi");
		myHClade.put("703747740003771","CAS1-Delhi");
		myHClade.put("703747740006171","Unknown");
		myHClade.put("703747777777421","Unknown");
		myHClade.put("703755740000171","CAS");
		myHClade.put("703757740000371","CAS");
		myHClade.put("703757740000771","CAS");
		myHClade.put("703757740003771","CAS1-Delhi");
		myHClade.put("703757760001771","Unknown");
		myHClade.put("703760000000000","CAS");
		myHClade.put("703760000000331","CAS");
		myHClade.put("703760000000771","CAS");
		myHClade.put("703760020000331","Unknown");
		myHClade.put("703760040003071","CAS1-Delhi");
		myHClade.put("703760040003471","CAS1-Delhi");
		myHClade.put("703761000000331","CAS");
		myHClade.put("703761000000371","CAS");
		myHClade.put("703761000000471","CAS");
		myHClade.put("703761000003771","CAS");
		myHClade.put("703763500000771","CAS");
		myHClade.put("703763740001171","CAS");
		myHClade.put("703763740003171","CAS1-Delhi");
		myHClade.put("703763740003471","CAS1-Delhi");
		myHClade.put("703763740003771","CAS1-Delhi");
		myHClade.put("703763740013031","EAI5");
		myHClade.put("703767400001771","CAS");
		myHClade.put("703767400003070","CAS");
		myHClade.put("703767400003071","CAS");
		myHClade.put("703767400003771","CAS");
		myHClade.put("703767434001671","Unknown");
		myHClade.put("703767577750751","Manu3");
		myHClade.put("703767640003771","CAS1-Delhi");
		myHClade.put("703767700003771","CAS");
		myHClade.put("703767740000000","CAS");
		myHClade.put("703767740000171","CAS");
		myHClade.put("703767740000371","CAS");
		myHClade.put("703767740000771","CAS");
		myHClade.put("703767740003071","CAS1-Delhi");
		myHClade.put("703767740003171","CAS1-Delhi");
		myHClade.put("703767740003371","CAS1-Delhi");
		myHClade.put("703767740003461","CAS1-Delhi");
		myHClade.put("703767740003470","CAS1-Delhi");
		myHClade.put("703767740003471","CAS1-Delhi");
		myHClade.put("703767740003571","CAS1-Delhi");
		myHClade.put("703767740003661","CAS1-Delhi");
		myHClade.put("703767740003731","CAS1-Delhi");
		myHClade.put("703767740003751","CAS1-Delhi");
		myHClade.put("703767740003761","CAS1-Delhi");
		myHClade.put("703767740003771","CAS1-Delhi");
		myHClade.put("703767740201371","Unknown");
		myHClade.put("703767770003771","Unknown");
		myHClade.put("703767774003571","Unknown");
		myHClade.put("703767777003771","Unknown");
		myHClade.put("703767777760771","T1");
		myHClade.put("703770000000001","CAS");
		myHClade.put("703770000000771","CAS");
		myHClade.put("703770740003771","CAS1-Delhi");
		myHClade.put("703771240003771","CAS1-Delhi");
		myHClade.put("703771740003771","CAS1-Delhi");
		myHClade.put("703771747777771","Unknown");
		myHClade.put("703773700003771","CAS");
		myHClade.put("703773740003171","CAS1-Delhi");
		myHClade.put("703773740003771","CAS1-Delhi");
		myHClade.put("703773747770371","Unknown");
		myHClade.put("703774000000031","CAS");
		myHClade.put("703774000000771","CAS");
		myHClade.put("703774000003171","CAS");
		myHClade.put("703774000003731","CAS");
		myHClade.put("703774000003771","CAS");
		myHClade.put("703774340003771","CAS1-Delhi");
		myHClade.put("703774740003771","CAS1-Delhi");
		myHClade.put("703775000000171","CAS");
		myHClade.put("703775000000371","CAS");
		myHClade.put("703775400000171","CAS");
		myHClade.put("703775640003771","CAS1-Delhi");
		myHClade.put("703775700000171","CAS");
		myHClade.put("703775700003771","CAS");
		myHClade.put("703775740000171","CAS1-Delhi");
		myHClade.put("703775740000771","CAS");
		myHClade.put("703775740003171","CAS1-Delhi");
		myHClade.put("703775740003571","CAS1-Delhi");
		myHClade.put("703775740003771","CAS1-Delhi");
		myHClade.put("703776000000011","CAS");
		myHClade.put("703776000003771","CAS");
		myHClade.put("703776040000771","CAS");
		myHClade.put("703776400001771","CAS");
		myHClade.put("703776600003771","CAS");
		myHClade.put("703776700003771","CAS");
		myHClade.put("703776740003771","CAS1-Delhi");
		myHClade.put("703777000000031","CAS");
		myHClade.put("703777000000071","CAS");
		myHClade.put("703777000000771","CAS1-Delhi");
		myHClade.put("703777000001771","CAS");
		myHClade.put("703777000003171","CAS");
		myHClade.put("703777000003771","CAS1-Delhi");
		myHClade.put("703777020003771","Unknown");
		myHClade.put("703777200000371","CAS");
		myHClade.put("703777300000771","CAS");
		myHClade.put("703777300003771","CAS");
		myHClade.put("703777340003171","CAS1-Delhi");
		myHClade.put("703777340003771","CAS1-Delhi");
		myHClade.put("703777347760471","T4-CEU1");
		myHClade.put("703777400000371","CAS");
		myHClade.put("703777400000771","CAS");
		myHClade.put("703777400001771","CAS1-Delhi");
		myHClade.put("703777400003031","CAS");
		myHClade.put("703777400003111","CAS");
		myHClade.put("703777400003171","CAS1-Delhi");
		myHClade.put("703777400003471","CAS");
		myHClade.put("703777400003771","CAS1-Delhi");
		myHClade.put("703777440003771","CAS1-Delhi");
		myHClade.put("703777500003771","CAS");
		myHClade.put("703777540003171","CAS1-Delhi");
		myHClade.put("703777540003771","CAS1-Delhi");
		myHClade.put("703777600000031","CAS1-Delhi");
		myHClade.put("703777600000171","CAS");
		myHClade.put("703777600000371","CAS");
		myHClade.put("703777600000771","CAS1-Delhi");
		myHClade.put("703777600001771","CAS1-Delhi");
		myHClade.put("703777600002771","CAS");
		myHClade.put("703777600003071","CAS");
		myHClade.put("703777600003151","CAS");
		myHClade.put("703777600003171","CAS1-Delhi");
		myHClade.put("703777600003271","CAS");
		myHClade.put("703777600003760","CAS");
		myHClade.put("703777600003771","CAS1-Delhi");
		myHClade.put("703777607760771","LAM9");
		myHClade.put("703777640003171","CAS1-Delhi");
		myHClade.put("703777640003770","CAS1-Delhi");
		myHClade.put("703777640003771","CAS1-Delhi");
		myHClade.put("703777640007760","Unknown");
		myHClade.put("703777700000000","CAS1-Delhi");
		myHClade.put("703777700000071","CAS1-Delhi");
		myHClade.put("703777700000171","CAS");
		myHClade.put("703777700000340","CAS");
		myHClade.put("703777700000371","CAS");
		myHClade.put("703777700000771","CAS");
		myHClade.put("703777700001771","CAS");
		myHClade.put("703777700002771","CAS");
		myHClade.put("703777700003171","CAS");
		myHClade.put("703777700003471","CAS");
		myHClade.put("703777700003771","CAS1-Delhi");
		myHClade.put("703777700004071","Unknown");
		myHClade.put("703777700005471","Unknown");
		myHClade.put("703777700010771","EAI5");
		myHClade.put("703777700013771","EAI5");
		myHClade.put("703777707770371","Unknown");
		myHClade.put("703777707770771","Manu3");
		myHClade.put("703777707773771","Manu1");
		myHClade.put("703777740000000","CAS1-Delhi");
		myHClade.put("703777740000001","CAS");
		myHClade.put("703777740000011","CAS");
		myHClade.put("703777740000031","CAS");
		myHClade.put("703777740000071","CAS");
		myHClade.put("703777740000171","CAS");
		myHClade.put("703777740000271","CAS");
		myHClade.put("703777740000370","CAS");
		myHClade.put("703777740000371","CAS");
		myHClade.put("703777740000471","CAS");
		myHClade.put("703777740000600","CAS");
		myHClade.put("703777740000671","CAS");
		myHClade.put("703777740000730","CAS");
		myHClade.put("703777740000731","CAS");
		myHClade.put("703777740000760","CAS");
		myHClade.put("703777740000771","CAS1-Delhi");
		myHClade.put("703777740001071","CAS");
		myHClade.put("703777740001171","CAS1-Delhi");
		myHClade.put("703777740001471","CAS1-Delhi");
		myHClade.put("703777740001571","CAS");
		myHClade.put("703777740001700","CAS");
		myHClade.put("703777740001751","CAS");
		myHClade.put("703777740001771","CAS");
		myHClade.put("703777740002000","CAS1-Delhi");
		myHClade.put("703777740002001","CAS1-Delhi");
		myHClade.put("703777740002031","CAS1-Delhi");
		myHClade.put("703777740002171","CAS1-Delhi");
		myHClade.put("703777740002331","CAS1-Delhi");
		myHClade.put("703777740002371","CAS1-Delhi");
		myHClade.put("703777740002771","CAS1-Delhi");
		myHClade.put("703777740003000","CAS1-Delhi");
		myHClade.put("703777740003010","CAS1-Delhi");
		myHClade.put("703777740003011","CAS1-Delhi");
		myHClade.put("703777740003031","CAS1-Delhi");
		myHClade.put("703777740003051","CAS1-Delhi");
		myHClade.put("703777740003071","CAS1-Delhi");
		myHClade.put("703777740003111","CAS1-Delhi");
		myHClade.put("703777740003131","CAS1-Delhi");
		myHClade.put("703777740003161","CAS1-Delhi");
		myHClade.put("703777740003170","CAS1-Delhi");
		myHClade.put("703777740003171","CAS1-Delhi");
		myHClade.put("703777740003371","CAS1-Delhi");
		myHClade.put("703777740003411","CAS1-Delhi");
		myHClade.put("703777740003451","CAS1-Delhi");
		myHClade.put("703777740003471","CAS1-Delhi");
		myHClade.put("703777740003571","CAS1-Delhi");
		myHClade.put("703777740003600","CAS1-Delhi");
		myHClade.put("703777740003611","CAS1-Delhi");
		myHClade.put("703777740003631","CAS1-Delhi");
		myHClade.put("703777740003661","CAS1-Delhi");
		myHClade.put("703777740003671","CAS1-Delhi");
		myHClade.put("703777740003700","CAS1-Delhi");
		myHClade.put("703777740003711","CAS1-Delhi");
		myHClade.put("703777740003721","CAS1-Delhi");
		myHClade.put("703777740003731","CAS1-Delhi");
		myHClade.put("703777740003741","CAS1-Delhi");
		myHClade.put("703777740003751","CAS1-Delhi");
		myHClade.put("703777740003760","CAS1-Delhi");
		myHClade.put("703777740003761","CAS1-Delhi");
		myHClade.put("703777740003770","CAS1-Delhi");
		myHClade.put("703777740003771","CAS1-Delhi");
		myHClade.put("703777740005071","Unknown");
		myHClade.put("703777740005471","Unknown");
		myHClade.put("703777740006371","Unknown");
		myHClade.put("703777740007770","Unknown");
		myHClade.put("703777740010000","EAI5");
		myHClade.put("703777740013031","EAI5");
		myHClade.put("703777740013071","EAI5");
		myHClade.put("703777740013171","EAI5");
		myHClade.put("703777740013771","EAI5");
		myHClade.put("703777740023771","Manu2");
		myHClade.put("703777740203171","Unknown");
		myHClade.put("703777740423171","Manu2");
		myHClade.put("703777741413771","EAI6-BGD1");
		myHClade.put("703777742003571","Unknown");
		myHClade.put("703777742003771","Unknown");
		myHClade.put("703777744003771","Unknown");
		myHClade.put("703777744034771","Unknown");
		myHClade.put("703777744037771","Unknown");
		myHClade.put("703777747570371","Unknown");
		myHClade.put("703777747670371","Unknown");
		myHClade.put("703777747760371","T1");
		myHClade.put("703777747763371","Manu2");
		myHClade.put("703777747763771","Manu2");
		myHClade.put("703777747770171","Unknown");
		myHClade.put("703777747770271","Unknown");
		myHClade.put("703777747770371","EAI");
		myHClade.put("703777747770771","Manu3");
		myHClade.put("703777747773771","Manu1");
		myHClade.put("703777747777771","Unknown");
		myHClade.put("703777750003771","Unknown");
		myHClade.put("703777756002771","Unknown");
		myHClade.put("703777757403771","Unknown");
		myHClade.put("703777757760371","T5");
		myHClade.put("703777760000171","Unknown");
		myHClade.put("703777760000371","Unknown");
		myHClade.put("703777760003010","Unknown");
		myHClade.put("703777760003131","Unknown");
		myHClade.put("703777760003171","Unknown");
		myHClade.put("703777760003771","CAS1-Delhi");
		myHClade.put("703777770001771","Unknown");
		myHClade.put("703777774000171","Unknown");
		myHClade.put("703777774000771","Unknown");
		myHClade.put("703777774003771","Unknown");
		myHClade.put("703777776003471","Unknown");
		myHClade.put("703777777013741","EAI5");
		myHClade.put("703777777403771","Unknown");
		myHClade.put("703777777413731","EAI1-SOM");
		myHClade.put("703777777747771","Unknown");
		myHClade.put("703777777760731","T2");
		myHClade.put("703777777760771","T1");
		myHClade.put("703777777763771","Manu2");
		myHClade.put("703777777770371","Unknown");
		myHClade.put("704003347560471","T4-CEU1");
		myHClade.put("704003347760400","T4");
		myHClade.put("704003347760471","T4-CEU1");
		myHClade.put("704023345760471","T4-CEU1");
		myHClade.put("704023347760471","T4-CEU1");
		myHClade.put("704036776360771","X3");
		myHClade.put("704175417672360","Manu1");
		myHClade.put("704237746413271","EAI6-BGD1");
		myHClade.put("704377747413371","EAI6-BGD1");
		myHClade.put("706003357760671","T");
		myHClade.put("706137607760771","LAM3");
		myHClade.put("706377710000671","Unknown");
		myHClade.put("706377777760731","S");
		myHClade.put("706377777760771","S");
		myHClade.put("706637777760771","T1");
		myHClade.put("706777777760771","T1");
		myHClade.put("707377776003771","Unknown");
		myHClade.put("707706000000000","Unknown");
		myHClade.put("707727607760771","LAM5");
		myHClade.put("707737307760431","T");
		myHClade.put("707737607760771","LAM5");
		myHClade.put("707757741603131","Unknown");
		myHClade.put("707761740003771","Unknown");
		myHClade.put("707776777760771","X1");
		myHClade.put("707777344560471","T4-CEU1");
		myHClade.put("707777347740471","T4-CEU1");
		myHClade.put("707777347760451","T4-CEU1");
		myHClade.put("707777347760471","T4-CEU1");
		myHClade.put("707777377360771","T4");
		myHClade.put("707777607760771","LAM9");
		myHClade.put("707777740003000","Unknown");
		myHClade.put("707777740003771","Unknown");
		myHClade.put("707777742037171","Unknown");
		myHClade.put("707777757413031","EAI6-BGD1");
		myHClade.put("707777760000740","Unknown");
		myHClade.put("707777770060731","T2");
		myHClade.put("707777774413731","EAI4-VNM");
		myHClade.put("707777777413700","EAI5");
		myHClade.put("707777777413731","EAI1-SOM");
		myHClade.put("707777777720771","H3");
		myHClade.put("707777777760571","T1");
		myHClade.put("707777777760700","T1");
		myHClade.put("707777777760711","T1");
		myHClade.put("707777777760731","T2");
		myHClade.put("707777777760771","T1");
		myHClade.put("707777777763771","Manu2");
		myHClade.put("710003600077761","Unknown");
		myHClade.put("710157777763771","Manu2");
		myHClade.put("710777777777631","Unknown");
		myHClade.put("711007676420621","Ural-1");
		myHClade.put("711022103370371","Unknown");
		myHClade.put("711041003760661","T1");
		myHClade.put("711041007740661","LAM9");
		myHClade.put("711054206360771","LAM9");
		myHClade.put("711054776763771","Manu2");
		myHClade.put("711067777706771","Unknown");
		myHClade.put("711762007760771","LAM9");
		myHClade.put("711767435002430","Unknown");
		myHClade.put("711777640003771","Unknown");
		myHClade.put("712366674303000","Unknown");
		myHClade.put("712517477760631","T-H37Rv");
		myHClade.put("713757777743771","Unknown");
		myHClade.put("713777740003771","Unknown");
		myHClade.put("713777777563771","Manu2");
		myHClade.put("713777777763171","Manu2");
		myHClade.put("714115777672360","Manu1");
		myHClade.put("714140377776741","Unknown");
		myHClade.put("714175754072360","Manu1");
		myHClade.put("714566703670340","Unknown");
		myHClade.put("715357777760771","T1");
		myHClade.put("715757503413131","EAI1-SOM");
		myHClade.put("715767777760771","T1");
		myHClade.put("715777757760771","T5");
		myHClade.put("715777777413771","EAI5");
		myHClade.put("716000003760771","T1");
		myHClade.put("716077670000000","Unknown");
		myHClade.put("716137607760731","LAM3");
		myHClade.put("716177607760771","LAM3");
		myHClade.put("716376777760771","S");
		myHClade.put("716377777760771","S");
		myHClade.put("716763677756200","Unknown");
		myHClade.put("717001777413771","EAI5");
		myHClade.put("717337607760771","LAM5");
		myHClade.put("717337706001371","Unknown");
		myHClade.put("717357776760771","T1");
		myHClade.put("717367777513621","Manu1");
		myHClade.put("717377777760771","T1");
		myHClade.put("717477777413731","EAI1-SOM");
		myHClade.put("717603752000000","Unknown");
		myHClade.put("717603757410371","EAI6-BGD1");
		myHClade.put("717707606740771","LAM8");
		myHClade.put("717717774020771","H1");
		myHClade.put("717727607760771","LAM5");
		myHClade.put("717727777764671","Unknown");
		myHClade.put("717737607760771","LAM5");
		myHClade.put("717737774020731","H1");
		myHClade.put("717737777760771","T3");
		myHClade.put("717747606760771","LAM8");
		myHClade.put("717747761700771","Unknown");
		myHClade.put("717747764000771","Unknown");
		myHClade.put("717757777760371","T1");
		myHClade.put("717766777760771","X1");
		myHClade.put("717767730000471","Unknown");
		myHClade.put("717767740003771","Unknown");
		myHClade.put("717767777503771","Unknown");
		myHClade.put("717776777760771","X1");
		myHClade.put("717777177760731","T2");
		myHClade.put("717777404760771","Turkey");
		myHClade.put("717777477413731","EAI1-SOM");
		myHClade.put("717777606060771","LAM11-ZWE");
		myHClade.put("717777607760771","LAM9");
		myHClade.put("717777610020771","H3");
		myHClade.put("717777663560731","T2");
		myHClade.put("717777700003371","Unknown");
		myHClade.put("717777742360771","Cameroon");
		myHClade.put("717777757413671","EAI6-BGD1");
		myHClade.put("717777757413771","EAI6-BGD1");
		myHClade.put("717777757760771","T5");
		myHClade.put("717777757777771","Unknown");
		myHClade.put("717777774020771","H1");
		myHClade.put("717777774413771","EAI4-VNM");
		myHClade.put("717777775412731","EAI1-SOM");
		myHClade.put("717777776010371","EAI5");
		myHClade.put("717777777400771","Unknown");
		myHClade.put("717777777412371","EAI5");
		myHClade.put("717777777413631","EAI5");
		myHClade.put("717777777413700","EAI5");
		myHClade.put("717777777413731","EAI1-SOM");
		myHClade.put("717777777413771","EAI5");
		myHClade.put("717777777620611","H3");
		myHClade.put("717777777660771","T1");
		myHClade.put("717777777703771","Unknown");
		myHClade.put("717777777720771","H3");
		myHClade.put("717777777760601","T1");
		myHClade.put("717777777760611","T1");
		myHClade.put("717777777760731","T2");
		myHClade.put("717777777760771","T1");
		myHClade.put("720000007760731","LAM3");
		myHClade.put("720001400000171","Unknown");
		myHClade.put("720001400000771","Unknown");
		myHClade.put("720021400000171","Unknown");
		myHClade.put("720036777760731","X1");
		myHClade.put("720147606560771","LAM3");
		myHClade.put("723777607760771","LAM9");
		myHClade.put("723777777413731","EAI1-SOM");
		myHClade.put("727777777000171","Unknown");
		myHClade.put("730000000000171","Unknown");
		myHClade.put("730000777760771","T1-RUS2");
		myHClade.put("730001607560771","LAM3");
		myHClade.put("730017777411631","EAI5");
		myHClade.put("730036777760731","X1");
		myHClade.put("730036777760771","X1");
		myHClade.put("730076777763771","Manu2");
		myHClade.put("730177577760771","T1");
		myHClade.put("730177777413631","EAI5");
		myHClade.put("730377600003771","Unknown");
		myHClade.put("730777470001671","AFRI_1");
		myHClade.put("730777777723600","Manu2");
		myHClade.put("730777777723670","AFRI_1");
		myHClade.put("731044606763771","Manu2");
		myHClade.put("731047436540661","T1");
		myHClade.put("731737777420771","Ural-1");
		myHClade.put("733357777760770","T1");
		myHClade.put("733737477753771","Manu1");
		myHClade.put("733737673420771","Ural-1");
		myHClade.put("733757606060771","LAM11-ZWE");
		myHClade.put("733777623760771","T1");
		myHClade.put("733777777060770","T1");
		myHClade.put("733777777060771","T1");
		myHClade.put("733777777760731","T2");
		myHClade.put("734000050013331","EAI1-SOM");
		myHClade.put("734077607777071","AFRI_2");
		myHClade.put("734177777413671","EAI5");
		myHClade.put("734177777413771","EAI5");
		myHClade.put("735777774001740","Unknown");
		myHClade.put("735777777413731","EAI1-SOM");
		myHClade.put("735777777760731","T2");
		myHClade.put("736000000007771","Unknown");
		myHClade.put("736177607700171","AFRI_2");
		myHClade.put("736177607760711","LAM3");
		myHClade.put("736177607760731","LAM3");
		myHClade.put("736177607760741","LAM3");
		myHClade.put("736177607760771","LAM3");
		myHClade.put("736177777760771","T1");
		myHClade.put("736300000020771","H3");
		myHClade.put("736377737560731","S");
		myHClade.put("736377737760731","S");
		myHClade.put("736377777760771","S");
		myHClade.put("737000377413731","EAI1-SOM");
		myHClade.put("737000377760771","T3-ETH");
		myHClade.put("737001777411731","EAI1-SOM");
		myHClade.put("737001777413731","EAI1-SOM");
		myHClade.put("737003400700771","Unknown");
		myHClade.put("737021777413731","EAI1-SOM");
		myHClade.put("737337777760730","T2-Uganda");
		myHClade.put("737376777760771","X1");
		myHClade.put("737377606061771","Unknown");
		myHClade.put("737377607560171","LAM6");
		myHClade.put("737377607760671","LAM9");
		myHClade.put("737377607760731","LAM4");
		myHClade.put("737377607760761","LAM9");
		myHClade.put("737377677760771","T1");
		myHClade.put("737377771760731","T2");
		myHClade.put("737377776760771","T1");
		myHClade.put("737377777760731","T2");
		myHClade.put("737377777760771","T1");
		myHClade.put("737377777761771","Unknown");
		myHClade.put("737400004020771","H1");
		myHClade.put("737417606060731","LAM11-ZWE");
		myHClade.put("737603407760771","LAM5");
		myHClade.put("737637400020771","H3");
		myHClade.put("737637417760771","T1");
		myHClade.put("737637770000731","Unknown");
		myHClade.put("737675757413371","EAI6-BGD1");
		myHClade.put("737677777560611","T1");
		myHClade.put("737700357412071","EAI6-BGD1");
		myHClade.put("737700777413700","EAI5");
		myHClade.put("737723777760771","T3");
		myHClade.put("737727777760731","T");
		myHClade.put("737736770000000","Unknown");
		myHClade.put("737737607160771","LAM5");
		myHClade.put("737737607760400","LAM");
		myHClade.put("737737607760771","LAM5");
		myHClade.put("737737737560771","T3");
		myHClade.put("737737737760771","T3");
		myHClade.put("737737770020771","H3");
		myHClade.put("737737777413771","EAI5");
		myHClade.put("737737777720731","H3");
		myHClade.put("737737777740071","T3");
		myHClade.put("737737777740371","T3");
		myHClade.put("737737777760730","T2-Uganda");
		myHClade.put("737737777760731","T");
		myHClade.put("737737777760771","T3");
		myHClade.put("737737777770731","Manu3");
		myHClade.put("737741007760771","LAM9");
		myHClade.put("737747606100000","Unknown");
		myHClade.put("737753777760771","T1");
		myHClade.put("737765637760771","T1");
		myHClade.put("737766777760601","X2");
		myHClade.put("737767401760630","T1");
		myHClade.put("737767607760771","LAM9");
		myHClade.put("737767770000000","Unknown");
		myHClade.put("737767777740771","T1");
		myHClade.put("737767777760731","T2");
		myHClade.put("737767777760771","T1");
		myHClade.put("737773637413771","EAI5");
		myHClade.put("737774077560771","T");
		myHClade.put("737775677413671","EAI5");
		myHClade.put("737775757413771","EAI6-BGD1");
		myHClade.put("737775777760771","T1");
		myHClade.put("737776607760771","LAM9");
		myHClade.put("737776677760771","X1");
		myHClade.put("737776777560771","X1");
		myHClade.put("737776777760571","X1");
		myHClade.put("737776777760601","X2");
		myHClade.put("737776777760731","X1");
		myHClade.put("737776777760771","X1");
		myHClade.put("737777207760771","LAM9");
		myHClade.put("737777272772271","Manu1");
		myHClade.put("737777377760731","T");
		myHClade.put("737777404020771","H1");
		myHClade.put("737777423560771","T1");
		myHClade.put("737777474020771","H1");
		myHClade.put("737777603760730","T2-Uganda");
		myHClade.put("737777606060731","LAM11-ZWE");
		myHClade.put("737777606060771","LAM11-ZWE");
		myHClade.put("737777606160771","LAM9");
		myHClade.put("737777607030771","Manu3");
		myHClade.put("737777607560731","LAM6");
		myHClade.put("737777607560771","LAM6");
		myHClade.put("737777607720771","H3");
		myHClade.put("737777607760600","LAM4");
		myHClade.put("737777607760601","LAM4");
		myHClade.put("737777607760700","LAM4");
		myHClade.put("737777607760731","LAM4");
		myHClade.put("737777607760771","LAM9");
		myHClade.put("737777637760771","T1");
		myHClade.put("737777677460731","T2");
		myHClade.put("737777677670371","Unknown");
		myHClade.put("737777700000031","Unknown");
		myHClade.put("737777700000611","Unknown");
		myHClade.put("737777704020771","H1");
		myHClade.put("737777707413171","EAI5");
		myHClade.put("737777710000071","Unknown");
		myHClade.put("737777737720771","H3");
		myHClade.put("737777740003771","Unknown");
		myHClade.put("737777740023771","Manu2");
		myHClade.put("737777742020771","H3");
		myHClade.put("737777747413771","EAI6-BGD1");
		myHClade.put("737777750060771","T5");
		myHClade.put("737777756763771","Manu2");
		myHClade.put("737777757413151","EAI6-BGD1");
		myHClade.put("737777757413371","EAI6-BGD1");
		myHClade.put("737777757413771","EAI6-BGD1");
		myHClade.put("737777757760771","T5");
		myHClade.put("737777760013731","EAI1-SOM");
		myHClade.put("737777764020771","H1");
		myHClade.put("737777772020771","H3");
		myHClade.put("737777773760771","T1");
		myHClade.put("737777774020731","H1");
		myHClade.put("737777774020751","H1");
		myHClade.put("737777774020771","H1");
		myHClade.put("737777774220771","H3");
		myHClade.put("737777774413771","EAI4-VNM");
		myHClade.put("737777775720771","H3");
		myHClade.put("737777776413771","EAI5");
		myHClade.put("737777777000731","Unknown");
		myHClade.put("737777777401761","Unknown");
		myHClade.put("737777777403771","Unknown");
		myHClade.put("737777777413371","EAI5");
		myHClade.put("737777777413700","EAI5");
		myHClade.put("737777777413731","EAI1-SOM");
		myHClade.put("737777777413771","EAI5");
		myHClade.put("737777777420731","Ural-1");
		myHClade.put("737777777520771","H3");
		myHClade.put("737777777660771","T1");
		myHClade.put("737777777700000","Unknown");
		myHClade.put("737777777700171","Unknown");
		myHClade.put("737777777700771","Unknown");
		myHClade.put("737777777720031","H3");
		myHClade.put("737777777720671","H3");
		myHClade.put("737777777720731","H3");
		myHClade.put("737777777720771","H3");
		myHClade.put("737777777723771","Manu2");
		myHClade.put("737777777740771","T1");
		myHClade.put("737777777760031","T1");
		myHClade.put("737777777760371","T1");
		myHClade.put("737777777760531","T2");
		myHClade.put("737777777760571","T1");
		myHClade.put("737777777760601","T1");
		myHClade.put("737777777760610","T1");
		myHClade.put("737777777760631","T1");
		myHClade.put("737777777760700","T1");
		myHClade.put("737777777760720","T2");
		myHClade.put("737777777760731","T2");
		myHClade.put("737777777760771","T1");
		myHClade.put("737777777761771","Unknown");
		myHClade.put("737777777763771","Manu2");
		myHClade.put("737777777767771","Unknown");
		myHClade.put("737777777777771","Unknown");
		myHClade.put("740000000000001","Unknown");
		myHClade.put("740000000000031","Unknown");
		myHClade.put("740000000020771","H3");
		myHClade.put("740000004020771","H1");
		myHClade.put("740000004024771","Unknown");
		myHClade.put("740000007320771","H3");
		myHClade.put("740000007560771","LAM3");
		myHClade.put("740000007720771","H3");
		myHClade.put("740000007760731","Unknown");
		myHClade.put("740000007760771","LAM3");
		myHClade.put("740000021760771","T1");
		myHClade.put("740000370360771","T1");
		myHClade.put("740000377760771","T1");
		myHClade.put("740000377777671","AFRI");
		myHClade.put("740000607760731","LAM3");
		myHClade.put("740000607760771","LAM3");
		myHClade.put("740003606760771","LAM3");
		myHClade.put("740003777420771","Ural-1");
		myHClade.put("740007606760771","LAM3");
		myHClade.put("740007777420771","Ural-1");
		myHClade.put("740017607760171","LAM3");
		myHClade.put("740027606760771","LAM3");
		myHClade.put("740037777760700","T1");
		myHClade.put("740037777760771","T1");
		myHClade.put("740037777777671","AFRI");
		myHClade.put("740076777760601","X2");
		myHClade.put("740077777760731","T2");
		myHClade.put("740077777777671","AFRI");
		myHClade.put("740177400000171","Unknown");
		myHClade.put("740177777013731","EAI1-SOM");
		myHClade.put("740376770000000","Unknown");
		myHClade.put("740376777760771","X1");
		myHClade.put("740377607760771","LAM9");
		myHClade.put("740377740003771","Unknown");
		myHClade.put("740377773740731","T2");
		myHClade.put("740377777740000","T1");
		myHClade.put("740377777760731","T2");
		myHClade.put("740377777760771","T1");
		myHClade.put("740377777777671","AFRI");
		myHClade.put("740617777760731","T2");
		myHClade.put("740617777760771","T3-OSA");
		myHClade.put("740637777760771","T1");
		myHClade.put("740657600000071","AFRI");
		myHClade.put("740657600003571","Unknown");
		myHClade.put("740737607760731","LAM");
		myHClade.put("740737677760731","T");
		myHClade.put("740737777777471","AFRI");
		myHClade.put("740737777777771","Unknown");
		myHClade.put("740757600000061","AFRI");
		myHClade.put("740757600000171","Unknown");
		myHClade.put("740767777413331","EAI1-SOM");
		myHClade.put("740767777760731","T2");
		myHClade.put("740777606000011","Unknown");
		myHClade.put("740777607760771","LAM9");
		myHClade.put("740777674000011","Unknown");
		myHClade.put("740777676000011","Unknown");
		myHClade.put("740777757777771","Unknown");
		myHClade.put("740777776360771","T1");
		myHClade.put("740777777760700","T1");
		myHClade.put("740777777760731","T2");
		myHClade.put("740777777760771","T1");
		myHClade.put("741407677560771","T1");
		myHClade.put("741407777760771","T1");
		myHClade.put("741427777760771","T1");
		myHClade.put("741703577760771","T1");
		myHClade.put("741703775760771","T1");
		myHClade.put("741703777760771","T1");
		myHClade.put("741727757560771","T3-OSA");
		myHClade.put("741736777760771","T3-OSA");
		myHClade.put("741737177760771","T3-OSA");
		myHClade.put("741737437760771","T3-OSA");
		myHClade.put("741737577760731","T3-OSA");
		myHClade.put("741737577760771","T3-OSA");
		myHClade.put("741737607560771","Unknown");
		myHClade.put("741737717760771","T3-OSA");
		myHClade.put("741737777400771","Unknown");
		myHClade.put("741737777760371","T3-OSA");
		myHClade.put("741737777760671","T3-OSA");
		myHClade.put("741737777760731","T3-OSA");
		myHClade.put("741737777760771","T3-OSA");
		myHClade.put("741737777763771","Manu2");
		myHClade.put("741757706060771","T1");
		myHClade.put("741761007760771","LAM9");
		myHClade.put("741765047560771","T1");
		myHClade.put("741774047560771","T1");
		myHClade.put("741774077560771","T1");
		myHClade.put("741775047560771","Unknown");
		myHClade.put("741777000000011","Zero-copy");
		myHClade.put("741777604760771","LAM9");
		myHClade.put("741777606060771","LAM11-ZWE");
		myHClade.put("741777740003771","Unknown");
		myHClade.put("741777774020771","H1");
		myHClade.put("741777777413700","EAI5");
		myHClade.put("741777777613771","Manu1");
		myHClade.put("741777777720771","H3");
		myHClade.put("741777777760771","T1");
		myHClade.put("742161047700771","Unknown");
		myHClade.put("742177777760731","T2");
		myHClade.put("742301040003771","Unknown");
		myHClade.put("742377777760771","S");
		myHClade.put("743077740003171","Unknown");
		myHClade.put("743417606060671","LAM11-ZWE");
		myHClade.put("743567777401731","Unknown");
		myHClade.put("743577777760731","T2");
		myHClade.put("743717670000731","Unknown");
		myHClade.put("743737607760731","LAM");
		myHClade.put("743737607760771","LAM5");
		myHClade.put("743737740003171","Unknown");
		myHClade.put("743747740003771","Unknown");
		myHClade.put("743760047700771","Unknown");
		myHClade.put("743767607760671","LAM9");
		myHClade.put("743767740003171","Unknown");
		myHClade.put("743774077560731","Unknown");
		myHClade.put("743776777760601","X2");
		myHClade.put("743777040003771","Unknown");
		myHClade.put("743777417740771","T1");
		myHClade.put("743777600003771","Unknown");
		myHClade.put("743777606060731","LAM11-ZWE");
		myHClade.put("743777607760731","LAM4");
		myHClade.put("743777607760771","LAM9");
		myHClade.put("743777700003371","EAI7-BGD2");
		myHClade.put("743777700060771","T1");
		myHClade.put("743777740023671","Manu2");
		myHClade.put("743777740023771","Manu2");
		myHClade.put("743777760003771","Unknown");
		myHClade.put("743777774020771","H1");
		myHClade.put("743777777001731","Unknown");
		myHClade.put("743777777413030","EAI5");
		myHClade.put("743777777413771","EAI5");
		myHClade.put("743777777520731","H3");
		myHClade.put("743777777760700","T1");
		myHClade.put("743777777760771","T1");
		myHClade.put("745737777760771","T3");
		myHClade.put("746000076360731","T2");
		myHClade.put("746177607760771","LAM3");
		myHClade.put("746377775760771","S");
		myHClade.put("747377777760771","T1");
		myHClade.put("747737607760771","LAM5");
		myHClade.put("747737757760771","T");
		myHClade.put("747737777760731","T");
		myHClade.put("747740007770771","Manu3");
		myHClade.put("747740017760771","T1");
		myHClade.put("747757606760771","LAM8");
		myHClade.put("747760017760771","T-tuscany");
		myHClade.put("747763600000000","Unknown");
		myHClade.put("747773777460771","T1");
		myHClade.put("747774077020771","H3");
		myHClade.put("747776777760601","X2");
		myHClade.put("747777577743771","Unknown");
		myHClade.put("747777600000000","Unknown");
		myHClade.put("747777600000771","Unknown");
		myHClade.put("747777606060731","LAM11-ZWE");
		myHClade.put("747777607760771","LAM9");
		myHClade.put("747777634000000","Unknown");
		myHClade.put("747777647560771","T1");
		myHClade.put("747777667560771","T1");
		myHClade.put("747777717760771","T1");
		myHClade.put("747777737760731","T2");
		myHClade.put("747777737760771","T1");
		myHClade.put("747777747413771","EAI6-BGD1");
		myHClade.put("747777757413771","EAI6-BGD1");
		myHClade.put("747777757660771","T5");
		myHClade.put("747777770020771","H3");
		myHClade.put("747777770060771","T1");
		myHClade.put("747777774020731","H1");
		myHClade.put("747777774160731","T2");
		myHClade.put("747777776001731","Unknown");
		myHClade.put("747777777413071","EAI5");
		myHClade.put("747777777413700","EAI5");
		myHClade.put("747777777553771","Manu1");
		myHClade.put("747777777720771","H3");
		myHClade.put("747777777760731","T2");
		myHClade.put("747777777760771","T1");
		myHClade.put("750037747760471","T1");
		myHClade.put("750327405000061","AFRI");
		myHClade.put("751767777760771","T1");
		myHClade.put("753347777760771","T1");
		myHClade.put("753765247703771","Unknown");
		myHClade.put("753767776760771","T1");
		myHClade.put("753777776760771","T1");
		myHClade.put("753777777413731","EAI1-SOM");
		myHClade.put("753777777760531","T2");
		myHClade.put("754100777676701","Unknown");
		myHClade.put("754100777677761","Unknown");
		myHClade.put("754135757672360","Manu1");
		myHClade.put("754175417672361","Manu1");
		myHClade.put("754175617672360","Manu1");
		myHClade.put("754177777413731","EAI1-SOM");
		myHClade.put("754177777413771","EAI5");
		myHClade.put("754337607760700","LAM");
		myHClade.put("754337777760771","T3");
		myHClade.put("754377607760700","LAM");
		myHClade.put("754777774020621","H1");
		myHClade.put("754777777420771","Ural-1");
		myHClade.put("755567677402631","Unknown");
		myHClade.put("756171777760771","T1");
		myHClade.put("756177607760771","LAM3");
		myHClade.put("756177607763771","Manu2");
		myHClade.put("756177607770771","Manu3");
		myHClade.put("756177777760731","T2");
		myHClade.put("756367777760771","S");
		myHClade.put("756377770000731","Unknown");
		myHClade.put("756377774000731","Unknown");
		myHClade.put("756377777720771","H3");
		myHClade.put("756377777760771","S");
		myHClade.put("756737407760771","LAM5");
		myHClade.put("756777777760771","T1");
		myHClade.put("757000004020731","H1");
		myHClade.put("757037777413700","EAI5");
		myHClade.put("757037777760731","T2");
		myHClade.put("757177777760771","T1");
		myHClade.put("757357777777371","Unknown");
		myHClade.put("757367607320631","H3");
		myHClade.put("757367777760731","T2");
		myHClade.put("757367777760771","T1");
		myHClade.put("757377377763771","Manu2");
		myHClade.put("757377704320771","H3");
		myHClade.put("757377774020771","H1");
		myHClade.put("757377777700000","Unknown");
		myHClade.put("757377777720771","H3");
		myHClade.put("757377777760731","T2");
		myHClade.put("757400004020731","H1");
		myHClade.put("757400004020771","H1");
		myHClade.put("757400037760771","T1");
		myHClade.put("757427607413731","EAI1-SOM");
		myHClade.put("757430000000371","Unknown");
		myHClade.put("757547477770771","Manu3");
		myHClade.put("757600377760771","T1");
		myHClade.put("757677677413731","EAI1-SOM");
		myHClade.put("757677777413731","EAI1-SOM");
		myHClade.put("757677777760771","T1");
		myHClade.put("757700000003371","Unknown");
		myHClade.put("757700777760771","T1");
		myHClade.put("757703607760771","LAM5");
		myHClade.put("757703777760731","T2");
		myHClade.put("757703777760771","T1");
		myHClade.put("757717607760771","LAM5");
		myHClade.put("757717637760771","T1");
		myHClade.put("757723777760771","T3");
		myHClade.put("757734777720730","H3");
		myHClade.put("757737604020771","H1");
		myHClade.put("757737607760760","LAM5");
		myHClade.put("757737607760771","LAM5");
		myHClade.put("757737774020771","H1");
		myHClade.put("757737777403731","Unknown");
		myHClade.put("757737777413731","EAI1-SOM");
		myHClade.put("757737777420771","Ural-1");
		myHClade.put("757737777760731","T");
		myHClade.put("757737777760771","T3");
		myHClade.put("757740005720771","H3");
		myHClade.put("757740007420771","Ural-1");
		myHClade.put("757740007760771","LAM9");
		myHClade.put("757740777760771","T1");
		myHClade.put("757741477776761","Unknown");
		myHClade.put("757743607760771","LAM9");
		myHClade.put("757743747760731","T2");
		myHClade.put("757747524000771","Unknown");
		myHClade.put("757747667760771","T1");
		myHClade.put("757757606060771","LAM11-ZWE");
		myHClade.put("757757764000771","Unknown");
		myHClade.put("757760007760771","LAM-RUS");
		myHClade.put("757760017760771","T-tuscany");
		myHClade.put("757761774760731","T2");
		myHClade.put("757761777720771","H3");
		myHClade.put("757767400741771","Unknown");
		myHClade.put("757767607760671","LAM9");
		myHClade.put("757767607760771","LAM9");
		myHClade.put("757767637763771","Manu2");
		myHClade.put("757767677413731","EAI1-SOM");
		myHClade.put("757767740003171","Unknown");
		myHClade.put("757767774000671","Unknown");
		myHClade.put("757767774020771","H1");
		myHClade.put("757767777300771","Unknown");
		myHClade.put("757767777413631","EAI5");
		myHClade.put("757767777413731","EAI1-SOM");
		myHClade.put("757767777720771","H3");
		myHClade.put("757767777723671","Manu2");
		myHClade.put("757767777740771","T1");
		myHClade.put("757767777743771","Unknown");
		myHClade.put("757767777760671","T1");
		myHClade.put("757767777760700","T1");
		myHClade.put("757767777760771","T1");
		myHClade.put("757773777760731","T2");
		myHClade.put("757775747413771","EAI6-BGD1");
		myHClade.put("757775777413731","EAI1-SOM");
		myHClade.put("757775777720771","H3");
		myHClade.put("757775777760771","T1");
		myHClade.put("757776377760731","T2");
		myHClade.put("757776740020771","H3");
		myHClade.put("757776777760571","X1");
		myHClade.put("757776777760601","X2");
		myHClade.put("757776777760771","X1");
		myHClade.put("757777000000700","Unknown");
		myHClade.put("757777001760771","T1");
		myHClade.put("757777037760771","T1");
		myHClade.put("757777077760771","T1");
		myHClade.put("757777277760771","T4");
		myHClade.put("757777347760471","T4-CEU1");
		myHClade.put("757777377760771","T4");
		myHClade.put("757777400020771","H3");
		myHClade.put("757777403760730","T2-Uganda");
		myHClade.put("757777403760731","T2");
		myHClade.put("757777404760771","Turkey");
		myHClade.put("757777407560731","LAM");
		myHClade.put("757777474020771","H1");
		myHClade.put("757777477413771","EAI5");
		myHClade.put("757777577413731","EAI1-SOM");
		myHClade.put("757777600000000","Unknown");
		myHClade.put("757777600000331","Unknown");
		myHClade.put("757777600003371","Unknown");
		myHClade.put("757777601760731","T2");
		myHClade.put("757777605560771","LAM6");
		myHClade.put("757777606060771","LAM11-ZWE");
		myHClade.put("757777606760631","LAM8");
		myHClade.put("757777606760771","LAM8");
		myHClade.put("757777607020771","H3");
		myHClade.put("757777607360340","LAM9");
		myHClade.put("757777607412401","EAI5");
		myHClade.put("757777607413731","EAI1-SOM");
		myHClade.put("757777607460731","LAM4");
		myHClade.put("757777607540771","LAM9");
		myHClade.put("757777607560000","LAM6");
		myHClade.put("757777607560771","LAM6");
		myHClade.put("757777607720771","H3");
		myHClade.put("757777607760611","LAM4");
		myHClade.put("757777607760760","LAM9");
		myHClade.put("757777607760771","LAM9");
		myHClade.put("757777616760631","T1");
		myHClade.put("757777630760771","T1");
		myHClade.put("757777637760771","T1");
		myHClade.put("757777677320771","H3");
		myHClade.put("757777677413731","EAI1-SOM");
		myHClade.put("757777700000771","Unknown");
		myHClade.put("757777700003371","Unknown");
		myHClade.put("757777707720671","H3");
		myHClade.put("757777717760671","T1");
		myHClade.put("757777737410031","EAI5");
		myHClade.put("757777737410071","EAI5");
		myHClade.put("757777737413731","EAI1-SOM");
		myHClade.put("757777740763771","Manu2");
		myHClade.put("757777741720771","H3");
		myHClade.put("757777743760731","Cameroon");
		myHClade.put("757777744020771","H1");
		myHClade.put("757777744743771","Unknown");
		myHClade.put("757777746000771","Unknown");
		myHClade.put("757777753413771","EAI6-BGD1");
		myHClade.put("757777757370371","Unknown");
		myHClade.put("757777757413371","EAI6-BGD1");
		myHClade.put("757777757413771","EAI6-BGD1");
		myHClade.put("757777757760711","T5");
		myHClade.put("757777757760771","T5");
		myHClade.put("757777767760631","T1");
		myHClade.put("757777767760731","T2");
		myHClade.put("757777770000000","Unknown");
		myHClade.put("757777770020731","H3");
		myHClade.put("757777770020771","H3");
		myHClade.put("757777770760731","T2");
		myHClade.put("757777774000000","Unknown");
		myHClade.put("757777774000771","Unknown");
		myHClade.put("757777774020471","H1");
		myHClade.put("757777774020700","H1");
		myHClade.put("757777774020731","H1");
		myHClade.put("757777774020770","H1");
		myHClade.put("757777774020771","H1");
		myHClade.put("757777774030771","Manu3");
		myHClade.put("757777776000761","Unknown");
		myHClade.put("757777776000771","Unknown");
		myHClade.put("757777776010771","EAI5");
		myHClade.put("757777776520250","H3");
		myHClade.put("757777776720771","H3");
		myHClade.put("757777776760771","T1");
		myHClade.put("757777777020771","H3");
		myHClade.put("757777777030771","Manu3");
		myHClade.put("757777777320771","H3");
		myHClade.put("757777777360771","T1");
		myHClade.put("757777777413631","EAI5");
		myHClade.put("757777777413671","EAI5");
		myHClade.put("757777777413731","EAI1-SOM");
		myHClade.put("757777777413771","EAI5");
		myHClade.put("757777777420771","Ural-1");
		myHClade.put("757777777620771","H3");
		myHClade.put("757777777660771","T1");
		myHClade.put("757777777700331","Unknown");
		myHClade.put("757777777700371","Unknown");
		myHClade.put("757777777700771","Unknown");
		myHClade.put("757777777713771","Manu1");
		myHClade.put("757777777720700","H3");
		myHClade.put("757777777720731","H3");
		myHClade.put("757777777720751","H3");
		myHClade.put("757777777720771","H3");
		myHClade.put("757777777740771","T1");
		myHClade.put("757777777760571","T1");
		myHClade.put("757777777760631","T1");
		myHClade.put("757777777760700","T1");
		myHClade.put("757777777760711","T");
		myHClade.put("757777777760731","T2");
		myHClade.put("757777777760751","T1");
		myHClade.put("757777777760760","T1");
		myHClade.put("757777777760771","T1");
		myHClade.put("757777777763771","Manu2");
		myHClade.put("760000000000000","Unknown");
		myHClade.put("760000000000031","Unknown");
		myHClade.put("760000000000771","Unknown");
		myHClade.put("760000000003771","Unknown");
		myHClade.put("760000000060751","T1");
		myHClade.put("760000000340371","T1");
		myHClade.put("760000000760771","T1");
		myHClade.put("760000001760771","Unknown");
		myHClade.put("760000003760771","T1");
		myHClade.put("760000004020771","H1");
		myHClade.put("760000004160471","LAM3");
		myHClade.put("760000005720771","H3");
		myHClade.put("760000007720771","H3");
		myHClade.put("760000007760731","LAM3");
		myHClade.put("760000007760771","LAM3");
		myHClade.put("760000060160771","T1");
		myHClade.put("760000074020771","H1");
		myHClade.put("760000077760771","T1");
		myHClade.put("760000077777671","AFRI");
		myHClade.put("760000171027771","Unknown");
		myHClade.put("760000207560771","LAM3");
		myHClade.put("760000377777671","AFRI");
		myHClade.put("760000777760771","T1");
		myHClade.put("760001400000071","AFRI");
		myHClade.put("760001400000171","LAM");
		myHClade.put("760001400004071","AFRI");
		myHClade.put("760001500000171","Unknown");
		myHClade.put("760001600000371","Unknown");
		myHClade.put("760001607720171","H3");
		myHClade.put("760001743760771","Cameroon");
		myHClade.put("760002001760771","T1");
		myHClade.put("760002377760731","T2");
		myHClade.put("760002777760771","X1");
		myHClade.put("760003400000171","Unknown");
		myHClade.put("760003743760771","Cameroon");
		myHClade.put("760003777760771","T1");
		myHClade.put("760017607760771","LAM3");
		myHClade.put("760017777720771","H3");
		myHClade.put("760121400200771","Unknown");
		myHClade.put("760147777720771","H3");
		myHClade.put("760177607760771","LAM3");
		myHClade.put("760177777413771","EAI5");
		myHClade.put("760177777720771","H3");
		myHClade.put("760177777760771","T1");
		myHClade.put("760337777420771","Ural-1");
		myHClade.put("760356777760701","X1");
		myHClade.put("760377377743671","AFRI");
		myHClade.put("760377403720771","H3");
		myHClade.put("760377637760701","T1");
		myHClade.put("760377757773771","Manu1");
		myHClade.put("760377757777671","AFRI");
		myHClade.put("760377777743671","AFRI_1");
		myHClade.put("760377777760701","T1");
		myHClade.put("760377777760731","T2");
		myHClade.put("760377777777671","AFRI_1");
		myHClade.put("760717777760771","T1");
		myHClade.put("760737004177671","AFRI");
		myHClade.put("760760007700771","Unknown");
		myHClade.put("760760047700771","Unknown");
		myHClade.put("760767777760731","T2");
		myHClade.put("760770047700771","Unknown");
		myHClade.put("760777607760711","LAM4");
		myHClade.put("760777607760771","LAM9");
		myHClade.put("760777774020771","H1");
		myHClade.put("760777777720771","H3");
		myHClade.put("760777777740371","T1");
		myHClade.put("760777777760731","T2");
		myHClade.put("760777777760771","T1");
		myHClade.put("761737747413771","EAI6-BGD1");
		myHClade.put("761737777760731","T");
		myHClade.put("761760000000000","Unknown");
		myHClade.put("761767743760731","Cameroon");
		myHClade.put("761767776000371","Unknown");
		myHClade.put("761773607760731","LAM4");
		myHClade.put("761777404760771","Turkey");
		myHClade.put("761777607760371","LAM9");
		myHClade.put("761777607760771","LAM9");
		myHClade.put("761777607761771","Unknown");
		myHClade.put("761777743760731","Cameroon");
		myHClade.put("761777743760771","Cameroon");
		myHClade.put("761777777413700","EAI5");
		myHClade.put("761777777600171","Unknown");
		myHClade.put("761777777720731","H3");
		myHClade.put("761777777720771","H3");
		myHClade.put("761777777760731","T2");
		myHClade.put("761777777760771","T1");
		myHClade.put("762174607760771","LAM3");
		myHClade.put("762177607760771","LAM3");
		myHClade.put("762737777760771","T3");
		myHClade.put("763000077757000","Unknown");
		myHClade.put("763000077757031","Unknown");
		myHClade.put("763000077777031","Unknown");
		myHClade.put("763737777420771","Ural-1");
		myHClade.put("763740777760771","T1");
		myHClade.put("763743777413771","EAI5");
		myHClade.put("763761017760771","T1");
		myHClade.put("763763607560771","LAM6");
		myHClade.put("763777607560760","LAM6");
		myHClade.put("763777607560771","LAM6");
		myHClade.put("763777607760771","LAM9");
		myHClade.put("763777677560771","T1");
		myHClade.put("763777774413771","EAI4-VNM");
		myHClade.put("763777777413371","EAI5");
		myHClade.put("763777777413700","EAI5");
		myHClade.put("763777777413771","EAI5");
		myHClade.put("763777777420771","Ural-1");
		myHClade.put("763777777720771","H3");
		myHClade.put("763777777760371","T1");
		myHClade.put("763777777760700","T1");
		myHClade.put("763777777760711","T1");
		myHClade.put("763777777760731","T2");
		myHClade.put("763777777760771","T1");
		myHClade.put("763777777763771","Manu2");
		myHClade.put("764077607777071","AFRI_2");
		myHClade.put("764137607720771","H3");
		myHClade.put("764777777420771","Ural-1");
		myHClade.put("765677777760771","T1");
		myHClade.put("765777757760771","T5");
		myHClade.put("765777777413771","EAI5");
		myHClade.put("766073777760771","T1");
		myHClade.put("766137607760771","LAM3");
		myHClade.put("766167607760771","LAM3");
		myHClade.put("766177007760771","LAM3");
		myHClade.put("766177407760771","LAM3");
		myHClade.put("766177607760771","LAM3");
		myHClade.put("766377777760731","S");
		myHClade.put("766377777760771","S");
		myHClade.put("767000007720731","H3");
		myHClade.put("767367777760661","T1");
		myHClade.put("767677777760771","T1");
		myHClade.put("767700777760731","T2");
		myHClade.put("767701003740771","T1");
		myHClade.put("767717601720771","H3");
		myHClade.put("767717607760771","LAM5");
		myHClade.put("767717737757000","Unknown");
		myHClade.put("767737607760771","LAM5");
		myHClade.put("767737675560771","T3");
		myHClade.put("767737777720771","H3");
		myHClade.put("767737777760771","T3");
		myHClade.put("767740003760771","T1");
		myHClade.put("767757740000000","Unknown");
		myHClade.put("767757740000771","Unknown");
		myHClade.put("767757777760771","T1");
		myHClade.put("767763774020771","H1");
		myHClade.put("767767607760731","LAM4");
		myHClade.put("767767740000000","Unknown");
		myHClade.put("767774077020771","H3");
		myHClade.put("767774077560771","T1");
		myHClade.put("767776037760771","T1");
		myHClade.put("767776377003731","Unknown");
		myHClade.put("767776777760601","X2");
		myHClade.put("767776777760771","X1");
		myHClade.put("767776777770371","Unknown");
		myHClade.put("767776777770771","Manu3");
		myHClade.put("767777377760771","T4");
		myHClade.put("767777570020731","H3");
		myHClade.put("767777577740771","T1");
		myHClade.put("767777605760671","LAM9");
		myHClade.put("767777605760771","LAM9");
		myHClade.put("767777606060731","LAM11-ZWE");
		myHClade.put("767777606760771","LAM8");
		myHClade.put("767777607560771","LAM6");
		myHClade.put("767777607760631","LAM4");
		myHClade.put("767777607760771","LAM9");
		myHClade.put("767777663760371","T1");
		myHClade.put("767777702003371","Unknown");
		myHClade.put("767777707720771","H3");
		myHClade.put("767777743413771","EAI6-BGD1");
		myHClade.put("767777744020750","H1");
		myHClade.put("767777744020771","H1");
		myHClade.put("767777747413771","EAI6-BGD1");
		myHClade.put("767777755413371","EAI6-BGD1");
		myHClade.put("767777757413771","EAI6-BGD1");
		myHClade.put("767777767720771","H3");
		myHClade.put("767777767760771","T1");
		myHClade.put("767777770000771","Unknown");
		myHClade.put("767777770020771","H3");
		myHClade.put("767777774020571","H1");
		myHClade.put("767777774020731","H1");
		myHClade.put("767777774020771","H1");
		myHClade.put("767777774413771","EAI4-VNM");
		myHClade.put("767777777413731","EAI1-SOM");
		myHClade.put("767777777420731","Ural-1");
		myHClade.put("767777777420771","Ural-1");
		myHClade.put("767777777620771","H3");
		myHClade.put("767777777720131","H3");
		myHClade.put("767777777720571","H3");
		myHClade.put("767777777720631","H3");
		myHClade.put("767777777720771","H3");
		myHClade.put("767777777722071","Manu2");
		myHClade.put("767777777740771","T1");
		myHClade.put("767777777760471","T1");
		myHClade.put("767777777760530","T2-Uganda");
		myHClade.put("767777777760571","T1");
		myHClade.put("767777777760611","T1");
		myHClade.put("767777777760631","T1");
		myHClade.put("767777777760730","T2-Uganda");
		myHClade.put("767777777760731","T2");
		myHClade.put("767777777760761","T1");
		myHClade.put("767777777760771","T1");
		myHClade.put("767777777763771","Manu2");
		myHClade.put("770000000000071","AFRI");
		myHClade.put("770000000003671","AFRI");
		myHClade.put("770000000003771","Unknown");
		myHClade.put("770000000020771","H3");
		myHClade.put("770000000400071","AFRI");
		myHClade.put("770000001760771","T1");
		myHClade.put("770000003760771","Unknown");
		myHClade.put("770000004760771","Turkey");
		myHClade.put("770000007720771","H3");
		myHClade.put("770000007760771","LAM3");
		myHClade.put("770000077757031","Unknown");
		myHClade.put("770000103760771","T1");
		myHClade.put("770000207760771","Unknown");
		myHClade.put("770000377760771","T1-RUS2");
		myHClade.put("770000603760771","T1-RUS2");
		myHClade.put("770000607760731","T1-RUS2");
		myHClade.put("770000607760771","Unknown");
		myHClade.put("770000643760771","T1-RUS2");
		myHClade.put("770000675760771","T1-RUS2");
		myHClade.put("770000677760700","T1-RUS2");
		myHClade.put("770000677760731","T1-RUS2");
		myHClade.put("770000677760771","T1-RUS2");
		myHClade.put("770000700000000","Unknown");
		myHClade.put("770000704020771","H1");
		myHClade.put("770000707760771","T1-RUS2");
		myHClade.put("770000737760771","T1-RUS2");
		myHClade.put("770000740000331","Unknown");
		myHClade.put("770000740020771","H3");
		myHClade.put("770000757760711","T1-RUS2");
		myHClade.put("770000757760771","T1-RUS2");
		myHClade.put("770000770000000","Unknown");
		myHClade.put("770000771760771","T1-RUS2");
		myHClade.put("770000777360771","T1-RUS2");
		myHClade.put("770000777400071","AFRI");
		myHClade.put("770000777600031","Unknown");
		myHClade.put("770000777620731","H3");
		myHClade.put("770000777620771","H3");
		myHClade.put("770000777700000","Unknown");
		myHClade.put("770000777740171","T1-RUS2");
		myHClade.put("770000777760371","T1-RUS2");
		myHClade.put("770000777760571","T1-RUS2");
		myHClade.put("770000777760701","T1-RUS2");
		myHClade.put("770000777760731","T1-RUS2");
		myHClade.put("770000777760771","T1-RUS2");
		myHClade.put("770001777760771","T1");
		myHClade.put("770003400003071","AFRI_2");
		myHClade.put("770003600007071","AFRI");
		myHClade.put("770003602000031","AFRI_2");
		myHClade.put("770003604377671","AFRI");
		myHClade.put("770003605777071","AFRI");
		myHClade.put("770003606377031","Unknown");
		myHClade.put("770003606377071","AFRI_2");
		myHClade.put("770003607777001","AFRI_2");
		myHClade.put("770003607777071","AFRI_2");
		myHClade.put("770003627777071","AFRI");
		myHClade.put("770003777720771","H3");
		myHClade.put("770003777760731","T2");
		myHClade.put("770003777760771","T1");
		myHClade.put("770003777767671","AFRI");
		myHClade.put("770007000003071","AFRI");
		myHClade.put("770007607777071","AFRI");
		myHClade.put("770011777760771","T1");
		myHClade.put("770014777760771","T1");
		myHClade.put("770016777760771","X1");
		myHClade.put("770017037760711","T1");
		myHClade.put("770017177420771","Ural-1");
		myHClade.put("770020607760771","LAM3");
		myHClade.put("770023206371071","AFRI");
		myHClade.put("770037000020700","H3");
		myHClade.put("770037607760771","LAM3");
		myHClade.put("770037757760771","T5");
		myHClade.put("770037777740751","T1");
		myHClade.put("770037777760700","T1");
		myHClade.put("770047606377071","AFRI");
		myHClade.put("770056676160000","X2");
		myHClade.put("770064004007071","AFRI");
		myHClade.put("770077032007071","AFRI");
		myHClade.put("770077407760671","LAM3");
		myHClade.put("770077407760771","LAM3");
		myHClade.put("770077606060731","LAM11-ZWE");
		myHClade.put("770077607560771","LAM3");
		myHClade.put("770077637760731","T2");
		myHClade.put("770077757400771","Unknown");
		myHClade.put("770077777760771","T1");
		myHClade.put("770141000000000","Unknown");
		myHClade.put("770160007740771","LAM-RUS");
		myHClade.put("770163774020771","H1");
		myHClade.put("770167607760771","LAM3");
		myHClade.put("770171777740771","T1");
		myHClade.put("770177607760771","LAM3");
		myHClade.put("770177743760751","Cameroon");
		myHClade.put("770177774020771","H1");
		myHClade.put("770177777413771","EAI5");
		myHClade.put("770177777740071","T1");
		myHClade.put("770337740000177","Unknown");
		myHClade.put("770337777420771","Ural-1");
		myHClade.put("770346627740371","X1");
		myHClade.put("770357777777660","AFRI");
		myHClade.put("770363777777600","Unknown");
		myHClade.put("770373777767671","AFRI");
		myHClade.put("770376777760601","X2");
		myHClade.put("770376777760701","X1");
		myHClade.put("770377037760771","T1");
		myHClade.put("770377756003771","Unknown");
		myHClade.put("770377774020771","H1");
		myHClade.put("770377777720071","H3");
		myHClade.put("770377777760711","T1");
		myHClade.put("770377777762771","Manu2");
		myHClade.put("770377777763771","Manu2");
		myHClade.put("770377777767671","AFRI");
		myHClade.put("770377777777671","AFRI");
		myHClade.put("770477777777671","AFRI_1");
		myHClade.put("770537777777671","AFRI_1");
		myHClade.put("770637777767671","AFRI_1");
		myHClade.put("770637777773771","Manu1");
		myHClade.put("770677177777671","AFRI_1");
		myHClade.put("770703107767671","AFRI_1");
		myHClade.put("770717777777671","AFRI_1");
		myHClade.put("770737004177671","AFRI_1");
		myHClade.put("770737404177671","AFRI_1");
		myHClade.put("770737777617471","AFRI");
		myHClade.put("770737777760731","T");
		myHClade.put("770737777774671","AFRI_1");
		myHClade.put("770737777777671","AFRI_1");
		myHClade.put("770740777777671","AFRI_1");
		myHClade.put("770757777767671","AFRI_1");
		myHClade.put("770760007767671","AFRI_1");
		myHClade.put("770761777777671","AFRI_1");
		myHClade.put("770767607767651","AFRI_1");
		myHClade.put("770767677777671","AFRI_1");
		myHClade.put("770767707777671","AFRI_1");
		myHClade.put("770767747413771","EAI6-BGD1");
		myHClade.put("770767777403671","AFRI_1");
		myHClade.put("770767777737671","AFRI_1");
		myHClade.put("770767777767671","AFRI_1");
		myHClade.put("770767777777671","AFRI_1");
		myHClade.put("770770777777671","AFRI_1");
		myHClade.put("770773777777671","AFRI_1");
		myHClade.put("770777004177271","AFRI_1");
		myHClade.put("770777077777671","AFRI_1");
		myHClade.put("770777404177671","AFRI_1");
		myHClade.put("770777407560731","LAM6");
		myHClade.put("770777407777671","AFRI_1");
		myHClade.put("770777477775671","AFRI_1");
		myHClade.put("770777477777671","AFRI_1");
		myHClade.put("770777507777671","AFRI_1");
		myHClade.put("770777577377671","AFRI_1");
		myHClade.put("770777577777671","AFRI_1");
		myHClade.put("770777607560771","LAM6");
		myHClade.put("770777607760731","LAM4");
		myHClade.put("770777607760771","LAM9");
		myHClade.put("770777607777651","AFRI_1");
		myHClade.put("770777607777671","AFRI_1");
		myHClade.put("770777637743671","AFRI_1");
		myHClade.put("770777677777671","AFRI_1");
		myHClade.put("770777703017671","AFRI_1");
		myHClade.put("770777704377671","AFRI_1");
		myHClade.put("770777707774671","AFRI_1");
		myHClade.put("770777707776671","AFRI_1");
		myHClade.put("770777707777271","AFRI_1");
		myHClade.put("770777707777631","Unknown");
		myHClade.put("770777707777651","AFRI_1");
		myHClade.put("770777707777660","AFRI_1");
		myHClade.put("770777707777671","AFRI_1");
		myHClade.put("770777737777671","AFRI_1");
		myHClade.put("770777747413771","EAI6-BGD1");
		myHClade.put("770777747777771","Unknown");
		myHClade.put("770777757777471","AFRI");
		myHClade.put("770777767777671","AFRI_1");
		myHClade.put("770777770000000","Unknown");
		myHClade.put("770777770360771","T1");
		myHClade.put("770777774377671","AFRI_1");
		myHClade.put("770777774777671","AFRI_1");
		myHClade.put("770777775720771","H3");
		myHClade.put("770777775767671","AFRI_1");
		myHClade.put("770777776000071","AFRI");
		myHClade.put("770777777000000","Unknown");
		myHClade.put("770777777177671","AFRI_1");
		myHClade.put("770777777377440","AFRI");
		myHClade.put("770777777377671","AFRI_1");
		myHClade.put("770777777403671","AFRI_1");
		myHClade.put("770777777420711","Ural-1");
		myHClade.put("770777777420771","Ural-1");
		myHClade.put("770777777477671","AFRI_1");
		myHClade.put("770777777567671","AFRI_1");
		myHClade.put("770777777600671","AFRI_1");
		myHClade.put("770777777700670","AFRI_1");
		myHClade.put("770777777720771","H3");
		myHClade.put("770777777737671","AFRI_1");
		myHClade.put("770777777740671","AFRI_1");
		myHClade.put("770777777743671","AFRI_1");
		myHClade.put("770777777747071","AFRI");
		myHClade.put("770777777747671","AFRI_1");
		myHClade.put("770777777760731","T2");
		myHClade.put("770777777760771","T1");
		myHClade.put("770777777767671","AFRI_1");
		myHClade.put("770777777770371","Unknown");
		myHClade.put("770777777776271","AFRI_1");
		myHClade.put("770777777777071","AFRI");
		myHClade.put("770777777777271","AFRI_1");
		myHClade.put("770777777777471","AFRI");
		myHClade.put("770777777777600","Unknown");
		myHClade.put("770777777777601","AFRI_1");
		myHClade.put("770777777777611","Unknown");
		myHClade.put("770777777777631","Unknown");
		myHClade.put("770777777777651","AFRI_1");
		myHClade.put("770777777777661","AFRI_1");
		myHClade.put("770777777777671","AFRI_1");
		myHClade.put("771000006760771","LAM8");
		myHClade.put("771077607760771","LAM9");
		myHClade.put("771077777363771","Manu2");
		myHClade.put("771347777740771","T1");
		myHClade.put("771375600000771","Unknown");
		myHClade.put("771567607760671","LAM9");
		myHClade.put("771567757760671","T5");
		myHClade.put("771600000000011","Unknown");
		myHClade.put("771603405760471","LAM5");
		myHClade.put("771737777760771","T3");
		myHClade.put("771767607760771","LAM9");
		myHClade.put("771767677560671","T1");
		myHClade.put("771773777403600","Unknown");
		myHClade.put("771776777660771","X1");
		myHClade.put("771777377760771","T4");
		myHClade.put("771777600000000","Unknown");
		myHClade.put("771777607760731","LAM4");
		myHClade.put("771777607760771","LAM9");
		myHClade.put("771777717760771","T1");
		myHClade.put("771777737760771","T1");
		myHClade.put("771777740013700","EAI5");
		myHClade.put("771777757760771","T5");
		myHClade.put("771777767613770","Manu1");
		myHClade.put("771777777420771","Ural-1");
		myHClade.put("771777777720771","H3");
		myHClade.put("771777777760731","T2");
		myHClade.put("771777777760771","T1");
		myHClade.put("772000007775671","Unknown");
		myHClade.put("772017607760771","LAM12-Madrid1");
		myHClade.put("772174607760771","LAM3");
		myHClade.put("772177606560771","LAM3");
		myHClade.put("772177606760771","LAM3");
		myHClade.put("772177607760771","LAM3");
		myHClade.put("772202777760771","S");
		myHClade.put("772375747413771","EAI6-BGD1");
		myHClade.put("772377777760771","S");
		myHClade.put("772600000000671","Unknown");
		myHClade.put("772607777777671","Unknown");
		myHClade.put("772700000003631","Unknown");
		myHClade.put("772700000003671","Unknown");
		myHClade.put("772700200003671","Unknown");
		myHClade.put("772701000003631","Unknown");
		myHClade.put("772763000001671","Unknown");
		myHClade.put("772763400001671","Unknown");
		myHClade.put("772777400001671","Unknown");
		myHClade.put("772777700000000","Unknown");
		myHClade.put("772777777777671","Unknown");
		myHClade.put("773177777411631","EAI5");
		myHClade.put("773367777720661","H3");
		myHClade.put("773367777760771","T1");
		myHClade.put("773377557413371","EAI6-BGD1");
		myHClade.put("773377740003771","Unknown");
		myHClade.put("773377777760771","T1");
		myHClade.put("773377777761771","Unknown");
		myHClade.put("773377777763671","Manu2");
		myHClade.put("773417760060771","Unknown");
		myHClade.put("773567675560671","T1");
		myHClade.put("773601757013371","EAI5");
		myHClade.put("773603774360771","T1");
		myHClade.put("773676775720470","H3");
		myHClade.put("773700777413700","EAI5");
		myHClade.put("773736777720771","H3");
		myHClade.put("773737003760771","T3");
		myHClade.put("773737677420771","Ural-1");
		myHClade.put("773737774020771","H1");
		myHClade.put("773737777400771","Unknown");
		myHClade.put("773737777760731","T");
		myHClade.put("773737777760771","T3");
		myHClade.put("773756607760771","LAM9");
		myHClade.put("773757777413471","EAI5");
		myHClade.put("773760007760771","LAM-RUS");
		myHClade.put("773763777760771","T1");
		myHClade.put("773767477770371","Unknown");
		myHClade.put("773767666560671","T1");
		myHClade.put("773767777413671","EAI5");
		myHClade.put("773767777720771","H3");
		myHClade.put("773767777730771","Manu3");
		myHClade.put("773773745760771","T1");
		myHClade.put("773773757413371","EAI6-BGD1");
		myHClade.put("773774745413470","EAI6-BGD1");
		myHClade.put("773776407760601","X2");
		myHClade.put("773776607760771","LAM9");
		myHClade.put("773776777760601","X2");
		myHClade.put("773776777760771","X1");
		myHClade.put("773777037760771","T1");
		myHClade.put("773777137760771","T1");
		myHClade.put("773777207760771","LAM9");
		myHClade.put("773777377720771","H3");
		myHClade.put("773777404760771","Turkey");
		myHClade.put("773777577503771","Unknown");
		myHClade.put("773777600000771","Unknown");
		myHClade.put("773777606060731","LAM11-ZWE");
		myHClade.put("773777606060771","LAM11-ZWE");
		myHClade.put("773777607560171","LAM6");
		myHClade.put("773777607560771","LAM6");
		myHClade.put("773777607760700","LAM4");
		myHClade.put("773777607760771","LAM9");
		myHClade.put("773777607763771","Manu2");
		myHClade.put("773777623760771","T1");
		myHClade.put("773777663760771","T1");
		myHClade.put("773777674020771","H1");
		myHClade.put("773777676000771","Unknown");
		myHClade.put("773777676760771","T1");
		myHClade.put("773777677413031","EAI5");
		myHClade.put("773777677760611","T1");
		myHClade.put("773777707760771","T1");
		myHClade.put("773777740000171","Unknown");
		myHClade.put("773777740003171","Unknown");
		myHClade.put("773777740003471","Unknown");
		myHClade.put("773777743760731","Cameroon");
		myHClade.put("773777747403771","Unknown");
		myHClade.put("773777751413771","EAI6-BGD1");
		myHClade.put("773777757413371","EAI6-BGD1");
		myHClade.put("773777757413771","EAI6-BGD1");
		myHClade.put("773777757563771","Manu2");
		myHClade.put("773777757760771","T5");
		myHClade.put("773777760000771","Unknown");
		myHClade.put("773777770000000","Unknown");
		myHClade.put("773777770001771","Unknown");
		myHClade.put("773777773640400","T1");
		myHClade.put("773777774020771","H1");
		myHClade.put("773777774412771","EAI4-VNM");
		myHClade.put("773777774413731","EAI4-VNM");
		myHClade.put("773777774413771","EAI4-VNM");
		myHClade.put("773777776760731","T2");
		myHClade.put("773777776760771","T1");
		myHClade.put("773777777360771","T1");
		myHClade.put("773777777720631","H3");
		myHClade.put("773777777720771","H3");
		myHClade.put("773777777740171","T1");
		myHClade.put("773777777760031","T1");
		myHClade.put("773777777760731","T2");
		myHClade.put("773777777760771","T1");
		myHClade.put("773777777763771","Manu2");
		myHClade.put("773777777770771","Manu3");
		myHClade.put("774000000000371","Unknown");
		myHClade.put("774000000760771","T1");
		myHClade.put("774000003770331","Unknown");
		myHClade.put("774000007720771","H3");
		myHClade.put("774000007760731","LAM3");
		myHClade.put("774000007760771","LAM3");
		myHClade.put("774000007763771","Manu2");
		myHClade.put("774000017413771","EAI5");
		myHClade.put("774000017760771","Unknown");
		myHClade.put("774000067777071","AFRI");
		myHClade.put("774000077777031","Unknown");
		myHClade.put("774000174020771","H1");
		myHClade.put("774000370020611","H3");
		myHClade.put("774000370020771","H3");
		myHClade.put("774000377760771","T1");
		myHClade.put("774000677760371","T1");
		myHClade.put("774000730000000","Unknown");
		myHClade.put("774000777740771","T1");
		myHClade.put("774002000760771","T1");
		myHClade.put("774003607740071","LAM3");
		myHClade.put("774003607760771","LAM3");
		myHClade.put("774003607777071","AFRI");
		myHClade.put("774003774020771","H1");
		myHClade.put("774003777775071","AFRI_3");
		myHClade.put("774006777760771","X1");
		myHClade.put("774007774020771","H1");
		myHClade.put("774017477740771","T1");
		myHClade.put("774017606060731","LAM11-ZWE");
		myHClade.put("774017607777071","AFRI");
		myHClade.put("774017770020771","H3");
		myHClade.put("774023607777071","AFRI");
		myHClade.put("774027777413731","EAI1-SOM");
		myHClade.put("774034607777071","AFRI");
		myHClade.put("774036777760700","X1");
		myHClade.put("774037000020700","Unknown");
		myHClade.put("774037377760700","T4");
		myHClade.put("774037600000000","Unknown");
		myHClade.put("774037600760700","Unknown");
		myHClade.put("774037600760710","T1");
		myHClade.put("774037603760700","T1");
		myHClade.put("774037607760771","LAM3");
		myHClade.put("774037607777071","AFRI_2");
		myHClade.put("774037703760700","T1");
		myHClade.put("774037707560700","T1");
		myHClade.put("774037707760700","Unknown");
		myHClade.put("774037707760771","T1");
		myHClade.put("774037750000371","Unknown");
		myHClade.put("774037777360700","T1");
		myHClade.put("774037777720771","H3");
		myHClade.put("774037777740700","T1");
		myHClade.put("774037777760300","T1");
		myHClade.put("774037777760700","T");
		myHClade.put("774037777760701","T1");
		myHClade.put("774037777760771","T1");
		myHClade.put("774037777777071","AFRI_3");
		myHClade.put("774040070000001","Unknown");
		myHClade.put("774040077777031","Unknown");
		myHClade.put("774042077777071","AFRI_3");
		myHClade.put("774047777760661","T1");
		myHClade.put("774057607777071","AFRI_2");
		myHClade.put("774063777777600","Unknown");
		myHClade.put("774066006037071","AFRI_3");
		myHClade.put("774067400603031","Unknown");
		myHClade.put("774067402603031","Unknown");
		myHClade.put("774067407777031","Unknown");
		myHClade.put("774067407777071","AFRI_3");
		myHClade.put("774067600000071","AFRI");
		myHClade.put("774067600001071","AFRI_3");
		myHClade.put("774067606727071","AFRI_2");
		myHClade.put("774067606767031","Unknown");
		myHClade.put("774067606767071","AFRI_2");
		myHClade.put("774067607761031","Unknown");
		myHClade.put("774067607777031","Unknown");
		myHClade.put("774067607777071","AFRI_2");
		myHClade.put("774067700001071","AFRI_3");
		myHClade.put("774067760000071","AFRI");
		myHClade.put("774067770005071","AFRI_3");
		myHClade.put("774067770007071","AFRI_3");
		myHClade.put("774067772000071","AFRI");
		myHClade.put("774067772001071","AFRI_3");
		myHClade.put("774067776001671","AFRI");
		myHClade.put("774067777777071","AFRI_3");
		myHClade.put("774074006035071","AFRI_3");
		myHClade.put("774074006037071","AFRI_3");
		myHClade.put("774076000003031","Unknown");
		myHClade.put("774076002003071","AFRI_3");
		myHClade.put("774076005777071","AFRI_3");
		myHClade.put("774076006037071","AFRI_3");
		myHClade.put("774076007777071","AFRI_2");
		myHClade.put("774077007777071","AFRI_3");
		myHClade.put("774077077777071","AFRI_3");
		myHClade.put("774077407777071","AFRI_3");
		myHClade.put("774077600000031","Unknown");
		myHClade.put("774077600000071","AFRI");
		myHClade.put("774077602000000","Unknown");
		myHClade.put("774077602000031","AFRI_2");
		myHClade.put("774077602001071","AFRI_3");
		myHClade.put("774077604000031","AFRI_2");
		myHClade.put("774077606000031","Unknown");
		myHClade.put("774077606756071","AFRI");
		myHClade.put("774077606767071","AFRI_2");
		myHClade.put("774077607757071","AFRI_2");
		myHClade.put("774077607761031","AFRI_2");
		myHClade.put("774077607761071","AFRI_2");
		myHClade.put("774077607777000","Unknown");
		myHClade.put("774077607777031","AFRI_2");
		myHClade.put("774077607777041","AFRI_2");
		myHClade.put("774077607777061","AFRI_2");
		myHClade.put("774077607777071","AFRI_2");
		myHClade.put("774077670007071","AFRI_3");
		myHClade.put("774077700000071","AFRI");
		myHClade.put("774077743760771","Cameroon");
		myHClade.put("774077752001071","AFRI_3");
		myHClade.put("774077770001071","AFRI_3");
		myHClade.put("774077770005071","AFRI_3");
		myHClade.put("774077770007071","AFRI_3");
		myHClade.put("774077772001031","Unknown");
		myHClade.put("774077772001071","AFRI_3");
		myHClade.put("774077772005071","AFRI_3");
		myHClade.put("774077774413771","EAI4-VNM");
		myHClade.put("774077776001071","AFRI_3");
		myHClade.put("774077777037071","AFRI_3");
		myHClade.put("774077777760771","T");
		myHClade.put("774077777767071","AFRI_3");
		myHClade.put("774077777773071","AFRI_3");
		myHClade.put("774077777775061","AFRI_3");
		myHClade.put("774077777775071","AFRI_3");
		myHClade.put("774077777777071","AFRI_3");
		myHClade.put("774137600020771","Unknown");
		myHClade.put("774137600060771","Unknown");
		myHClade.put("774137607560771","LAM3");
		myHClade.put("774137607720771","H3");
		myHClade.put("774137777413731","EAI1-SOM");
		myHClade.put("774137777720771","H3");
		myHClade.put("774137777757771","Unknown");
		myHClade.put("774147607760271","LAM3");
		myHClade.put("774147775413470","EAI5");
		myHClade.put("774167607760771","LAM3");
		myHClade.put("774167777717771","Unknown");
		myHClade.put("774177007760771","LAM3");
		myHClade.put("774177077413771","EAI5");
		myHClade.put("774177374760771","T4");
		myHClade.put("774177406760771","LAM3");
		myHClade.put("774177407760771","LAM3");
		myHClade.put("774177607760771","LAM3");
		myHClade.put("774177677413771","EAI5");
		myHClade.put("774177747757771","Unknown");
		myHClade.put("774177757413771","EAI6-BGD1");
		myHClade.put("774177773717700","Unknown");
		myHClade.put("774177774020731","H1");
		myHClade.put("774177774020771","H1");
		myHClade.put("774177777357771","Unknown");
		myHClade.put("774177777413671","EAI5");
		myHClade.put("774177777413731","EAI1-SOM");
		myHClade.put("774177777413771","EAI5");
		myHClade.put("774177777707740","Unknown");
		myHClade.put("774177777717771","Unknown");
		myHClade.put("774177777720771","H3");
		myHClade.put("774177777757771","Unknown");
		myHClade.put("774177777760731","T2");
		myHClade.put("774177777760771","T1");
		myHClade.put("774337606560771","LAM6");
		myHClade.put("774337607760700","LAM");
		myHClade.put("774337757660771","T");
		myHClade.put("774375776563771","Manu2");
		myHClade.put("774377003740371","T1");
		myHClade.put("774377003740771","T1");
		myHClade.put("774377607760700","LAM4");
		myHClade.put("774377607760731","LAM4");
		myHClade.put("774377607760771","LAM9");
		myHClade.put("774377677420771","Ural-1");
		myHClade.put("774377700003071","AFRI");
		myHClade.put("774377774020771","H1");
		myHClade.put("774377776360751","T1");
		myHClade.put("774377776413771","EAI5");
		myHClade.put("774377777413771","EAI5");
		myHClade.put("774377777420771","Ural-1");
		myHClade.put("774377777660731","T2");
		myHClade.put("774377777760731","T2");
		myHClade.put("774377777777671","AFRI");
		myHClade.put("774677607777071","AFRI");
		myHClade.put("774707777413731","EAI1-SOM");
		myHClade.put("774737600160671","T3");
		myHClade.put("774737777420771","Ural-1");
		myHClade.put("774737777737771","Unknown");
		myHClade.put("774737777760731","T");
		myHClade.put("774747377420771","Ural-1");
		myHClade.put("774767607760771","LAM9");
		myHClade.put("774767777420771","Ural-1");
		myHClade.put("774774003420771","Ural-1");
		myHClade.put("774775003420771","Ural-1");
		myHClade.put("774776000420771","Ural-1");
		myHClade.put("774776777420771","Ural-1");
		myHClade.put("774777000420771","Ural-1");
		myHClade.put("774777207420771","Ural-1");
		myHClade.put("774777374020771","H1");
		myHClade.put("774777377420771","Ural-1");
		myHClade.put("774777400020771","H3");
		myHClade.put("774777477411771","EAI5");
		myHClade.put("774777600420731","Ural-1");
		myHClade.put("774777607420771","Ural-1");
		myHClade.put("774777607760771","LAM9");
		myHClade.put("774777616000371","Unknown");
		myHClade.put("774777677420771","Ural-1");
		myHClade.put("774777716000371","Unknown");
		myHClade.put("774777737420731","Ural-1");
		myHClade.put("774777737420771","Ural-1");
		myHClade.put("774777737423731","Manu2");
		myHClade.put("774777747420731","Ural-1");
		myHClade.put("774777750003331","EAI7-BGD2");
		myHClade.put("774777757420771","Ural-1");
		myHClade.put("774777770020771","H3");
		myHClade.put("774777774000771","Unknown");
		myHClade.put("774777774413771","EAI4-VNM");
		myHClade.put("774777775410771","EAI5");
		myHClade.put("774777775420771","Ural-1");
		myHClade.put("774777776420771","Ural-1");
		myHClade.put("774777777020771","H3");
		myHClade.put("774777777410771","EAI5");
		myHClade.put("774777777413731","EAI1-SOM");
		myHClade.put("774777777413771","EAI5");
		myHClade.put("774777777420371","Ural-1");
		myHClade.put("774777777420700","Ural-1");
		myHClade.put("774777777420731","Ural-1");
		myHClade.put("774777777420771","Ural-1");
		myHClade.put("774777777423771","Manu2");
		myHClade.put("774777777430771","Manu3");
		myHClade.put("774777777460771","T1");
		myHClade.put("774777777520771","H3");
		myHClade.put("774777777760731","T2");
		myHClade.put("774777777760771","T1");
		myHClade.put("774777777777671","AFRI");
		myHClade.put("775147677700000","Unknown");
		myHClade.put("775167607760671","LAM9");
		myHClade.put("775337607760661","LAM5");
		myHClade.put("775345437774271","Unknown");
		myHClade.put("775347437740261","T1");
		myHClade.put("775357777760771","T1");
		myHClade.put("775357777772671","Manu1");
		myHClade.put("775367637700771","Unknown");
		myHClade.put("775375777777601","Unknown");
		myHClade.put("775377757763771","Manu2");
		myHClade.put("775377777413771","EAI5");
		myHClade.put("775377777776771","Unknown");
		myHClade.put("775547743760671","Cameroon");
		myHClade.put("775567404760771","Turkey");
		myHClade.put("775567607760771","LAM9");
		myHClade.put("775567677720671","H3");
		myHClade.put("775567677760771","T1");
		myHClade.put("775567774020631","H1");
		myHClade.put("775577747777671","Unknown");
		myHClade.put("775677757760771","T5");
		myHClade.put("775700007760771","LAM5");
		myHClade.put("775706767703671","Unknown");
		myHClade.put("775707757760771","T5");
		myHClade.put("775736777411731","EAI1-SOM");
		myHClade.put("775737607760771","LAM5");
		myHClade.put("775737777720771","H3");
		myHClade.put("775737777760771","T3");
		myHClade.put("775740003760711","T1");
		myHClade.put("775740003760771","LAM-RUS");
		myHClade.put("775740003763771","Manu2");
		myHClade.put("775741000760571","T1");
		myHClade.put("775741003760771","T1");
		myHClade.put("775743003763771","Manu2");
		myHClade.put("775743757760771","T5");
		myHClade.put("775757777776771","Unknown");
		myHClade.put("775760007760771","LAM-RUS");
		myHClade.put("775760017760771","T-tuscany");
		myHClade.put("775767034020671","H1");
		myHClade.put("775767600160771","T1");
		myHClade.put("775767600760771","T1");
		myHClade.put("775767604020651","H1");
		myHClade.put("775767607560671","LAM6");
		myHClade.put("775767607760671","LAM9");
		myHClade.put("775767756760771","T5");
		myHClade.put("775767757760771","T5");
		myHClade.put("775767760060771","T1");
		myHClade.put("775767774020771","H1");
		myHClade.put("775767777760471","T1");
		myHClade.put("775767777760671","T1");
		myHClade.put("775767777760771","T1");
		myHClade.put("775775757413771","EAI6-BGD1");
		myHClade.put("775775757760771","T5");
		myHClade.put("775777377760771","T4");
		myHClade.put("775777401760771","T1");
		myHClade.put("775777557760771","T5-Madrid2");
		myHClade.put("775777606060731","LAM11-ZWE");
		myHClade.put("775777607760771","LAM9");
		myHClade.put("775777653760771","T5");
		myHClade.put("775777676740600","T1");
		myHClade.put("775777717760771","T1");
		myHClade.put("775777757413331","EAI");
		myHClade.put("775777757413771","EAI6-BGD1");
		myHClade.put("775777757760760","T5");
		myHClade.put("775777757760771","T5");
		myHClade.put("775777760060771","T1");
		myHClade.put("775777770000000","Unknown");
		myHClade.put("775777774020771","H1");
		myHClade.put("775777774413701","EAI4-VNM");
		myHClade.put("775777777401771","Unknown");
		myHClade.put("775777777403771","Unknown");
		myHClade.put("775777777412731","EAI1-SOM");
		myHClade.put("775777777413371","EAI5");
		myHClade.put("775777777413700","EAI5");
		myHClade.put("775777777413771","EAI5");
		myHClade.put("775777777420771","Ural-1");
		myHClade.put("775777777660771","T1");
		myHClade.put("775777777720751","H3");
		myHClade.put("775777777720771","H3");
		myHClade.put("775777777760221","T1");
		myHClade.put("775777777760601","T1");
		myHClade.put("775777777760700","T1");
		myHClade.put("775777777760731","T2");
		myHClade.put("775777777760771","T1");
		myHClade.put("775777777763771","Manu2");
		myHClade.put("775777777770771","Manu3");
		myHClade.put("775777777773771","Manu1");
		myHClade.put("775777777777471","Unknown");
		myHClade.put("776000000000000","Unknown");
		myHClade.put("776000000000111","Unknown");
		myHClade.put("776000000000171","Unknown");
		myHClade.put("776000000000771","Unknown");
		myHClade.put("776000000160771","Unknown");
		myHClade.put("776000000760771","T1");
		myHClade.put("776000003660771","T1");
		myHClade.put("776000003760771","T1");
		myHClade.put("776000004020771","H1");
		myHClade.put("776000007760771","LAM3");
		myHClade.put("776000017760471","T1");
		myHClade.put("776000017760771","T1");
		myHClade.put("776000100000171","Unknown");
		myHClade.put("776000377760771","T1");
		myHClade.put("776000606760771","LAM3");
		myHClade.put("776000607760771","LAM3");
		myHClade.put("776001743760771","Cameroon");
		myHClade.put("776003606060731","LAM11-ZWE");
		myHClade.put("776007404760771","Turkey");
		myHClade.put("776007606060731","LAM11-ZWE");
		myHClade.put("776007607760771","LAM3");
		myHClade.put("776007777760771","T1");
		myHClade.put("776017400000171","Unknown");
		myHClade.put("776017404760771","LAM12-Madrid1");
		myHClade.put("776017407760771","LAM12-Madrid1");
		myHClade.put("776017607160771","LAM12-Madrid1");
		myHClade.put("776017607760771","LAM12-Madrid1");
		myHClade.put("776017607762771","Manu2");
		myHClade.put("776017607770771","Manu3");
		myHClade.put("776017774020771","H1");
		myHClade.put("776017777760400","T1");
		myHClade.put("776017777760571","T1");
		myHClade.put("776017777760771","T1");
		myHClade.put("776020777760771","Unknown");
		myHClade.put("776021607760771","LAM3");
		myHClade.put("776023777760771","T1");
		myHClade.put("776036777760771","X1");
		myHClade.put("776037400000771","Unknown");
		myHClade.put("776037404760771","Turkey");
		myHClade.put("776037606760771","LAM3");
		myHClade.put("776037607760771","LAM3");
		myHClade.put("776037777760571","T1");
		myHClade.put("776037777760771","T1");
		myHClade.put("776057606560771","LAM3");
		myHClade.put("776066663020771","H3");
		myHClade.put("776067600000771","Unknown");
		myHClade.put("776071777760771","T1");
		myHClade.put("776077207760771","LAM3");
		myHClade.put("776077606760771","LAM3");
		myHClade.put("776077607760771","LAM3");
		myHClade.put("776077607770771","Manu3");
		myHClade.put("776077703770371","Unknown");
		myHClade.put("776077737760771","T1");
		myHClade.put("776077747720671","H3");
		myHClade.put("776077774020671","H1");
		myHClade.put("776077774020771","H1");
		myHClade.put("776077777413731","EAI1-SOM");
		myHClade.put("776077777760671","T1");
		myHClade.put("776077777760771","T1");
		myHClade.put("776100207740771","LAM3");
		myHClade.put("776103007760771","LAM3");
		myHClade.put("776107400000171","Unknown");
		myHClade.put("776107607760771","LAM3");
		myHClade.put("776117400000171","Unknown");
		myHClade.put("776117607760771","LAM3");
		myHClade.put("776123407760771","LAM3");
		myHClade.put("776127400000171","Unknown");
		myHClade.put("776137400000171","Unknown");
		myHClade.put("776137600020771","H3");
		myHClade.put("776137600760771","T3");
		myHClade.put("776137607760371","LAM3");
		myHClade.put("776137607760531","LAM3");
		myHClade.put("776137607760631","LAM3");
		myHClade.put("776137607760711","LAM3");
		myHClade.put("776137607760731","LAM");
		myHClade.put("776137607760771","LAM3");
		myHClade.put("776137777760771","T3");
		myHClade.put("776141607760731","LAM3");
		myHClade.put("776141607761771","Unknown");
		myHClade.put("776147607360671","LAM3");
		myHClade.put("776147607760661","LAM3");
		myHClade.put("776147607760671","LAM3");
		myHClade.put("776147607760771","LAM3");
		myHClade.put("776157007760771","LAM3");
		myHClade.put("776157400000171","Unknown");
		myHClade.put("776157607760731","LAM3");
		myHClade.put("776157607760771","LAM3");
		myHClade.put("776157777760771","T1");
		myHClade.put("776160000000071","Unknown");
		myHClade.put("776161000000071","Unknown");
		myHClade.put("776161004000071","Unknown");
		myHClade.put("776162000000071","Unknown");
		myHClade.put("776163000000071","Unknown");
		myHClade.put("776163400000171","Unknown");
		myHClade.put("776163607010171","EAI5");
		myHClade.put("776163607760771","LAM3");
		myHClade.put("776167400000171","Unknown");
		myHClade.put("776167407760771","LAM3");
		myHClade.put("776167607760071","LAM3");
		myHClade.put("776167607760371","LAM3");
		myHClade.put("776167607760401","LAM3");
		myHClade.put("776167607760611","LAM3");
		myHClade.put("776167607760671","LAM3");
		myHClade.put("776167607760711","LAM3");
		myHClade.put("776167607760731","LAM3");
		myHClade.put("776167607760771","LAM3");
		myHClade.put("776167777760171","T1");
		myHClade.put("776167777760621","T1");
		myHClade.put("776167777760671","T1");
		myHClade.put("776167777760731","T2");
		myHClade.put("776167777760751","T1");
		myHClade.put("776167777760771","T1");
		myHClade.put("776171000000071","Unknown");
		myHClade.put("776171607760771","LAM3");
		myHClade.put("776173400000071","Unknown");
		myHClade.put("776173400000171","Unknown");
		myHClade.put("776173607760771","LAM3");
		myHClade.put("776173777760771","T1");
		myHClade.put("776174000000071","Unknown");
		myHClade.put("776174000060771","T1");
		myHClade.put("776174007760771","LAM3");
		myHClade.put("776174207760771","LAM3");
		myHClade.put("776174407760771","LAM3");
		myHClade.put("776175407760771","LAM3");
		myHClade.put("776175770760771","T1");
		myHClade.put("776176207760740","LAM3");
		myHClade.put("776176600020771","H3");
		myHClade.put("776176607760771","LAM3");
		myHClade.put("776177007760771","LAM3");
		myHClade.put("776177207760771","LAM3");
		myHClade.put("776177400000071","Unknown");
		myHClade.put("776177400000131","Unknown");
		myHClade.put("776177400000171","Unknown");
		myHClade.put("776177400010171","EAI5");
		myHClade.put("776177400010371","EAI5");
		myHClade.put("776177400010771","EAI5");
		myHClade.put("776177400020171","H3");
		myHClade.put("776177400020771","H3");
		myHClade.put("776177403760771","T1");
		myHClade.put("776177404000171","Unknown");
		myHClade.put("776177407760731","LAM3");
		myHClade.put("776177407760771","LAM3");
		myHClade.put("776177577760771","T1");
		myHClade.put("776177600000000","Unknown");
		myHClade.put("776177600000171","Unknown");
		myHClade.put("776177600000771","Unknown");
		myHClade.put("776177600160771","T1");
		myHClade.put("776177600560771","T1");
		myHClade.put("776177604360771","LAM3");
		myHClade.put("776177605760771","LAM3");
		myHClade.put("776177606063771","Manu2");
		myHClade.put("776177606360771","LAM3");
		myHClade.put("776177606760771","LAM");
		myHClade.put("776177607000371","Unknown");
		myHClade.put("776177607000771","Unknown");
		myHClade.put("776177607020771","H3");
		myHClade.put("776177607360731","LAM3");
		myHClade.put("776177607360771","LAM3");
		myHClade.put("776177607400771","LAM2");
		myHClade.put("776177607460771","LAM3");
		myHClade.put("776177607560771","LAM");
		myHClade.put("776177607660771","LAM3");
		myHClade.put("776177607700171","LAM3");
		myHClade.put("776177607700771","Unknown");
		myHClade.put("776177607720771","H3");
		myHClade.put("776177607740711","LAM3");
		myHClade.put("776177607740731","LAM3");
		myHClade.put("776177607740771","LAM3");
		myHClade.put("776177607760371","LAM3");
		myHClade.put("776177607760400","LAM3");
		myHClade.put("776177607760471","LAM3");
		myHClade.put("776177607760600","LAM3");
		myHClade.put("776177607760671","LAM3");
		myHClade.put("776177607760700","LAM3");
		myHClade.put("776177607760711","LAM3");
		myHClade.put("776177607760731","LAM3");
		myHClade.put("776177607760740","LAM3");
		myHClade.put("776177607760770","LAM3");
		myHClade.put("776177607760771","LAM3");
		myHClade.put("776177607763731","Manu2");
		myHClade.put("776177607763771","Manu2");
		myHClade.put("776177607770371","Unknown");
		myHClade.put("776177607770670","Manu3");
		myHClade.put("776177607770731","Manu3");
		myHClade.put("776177607770771","Manu3");
		myHClade.put("776177607773771","Manu1");
		myHClade.put("776177637570771","Manu3");
		myHClade.put("776177637763771","Manu2");
		myHClade.put("776177677500071","Unknown");
		myHClade.put("776177707760771","T1");
		myHClade.put("776177730000000","Unknown");
		myHClade.put("776177747760771","T1");
		myHClade.put("776177760000000","Unknown");
		myHClade.put("776177770000000","Unknown");
		myHClade.put("776177770000731","Unknown");
		myHClade.put("776177770000771","Unknown");
		myHClade.put("776177774030171","Unknown");
		myHClade.put("776177775413731","EAI1-SOM");
		myHClade.put("776177775760771","T1");
		myHClade.put("776177777013700","EAI5");
		myHClade.put("776177777413731","EAI1-SOM");
		myHClade.put("776177777413771","EAI5");
		myHClade.put("776177777720771","H3");
		myHClade.put("776177777760731","T2");
		myHClade.put("776177777760751","T1");
		myHClade.put("776177777760771","S");
		myHClade.put("776177777770771","Manu3");
		myHClade.put("776200000060771","S");
		myHClade.put("776217777777771","Unknown");
		myHClade.put("776237777413400","EAI5");
		myHClade.put("776247777760571","S");
		myHClade.put("776267777760771","S");
		myHClade.put("776277777760771","S");
		myHClade.put("776303177760771","S");
		myHClade.put("776303777760771","S");
		myHClade.put("776307777760770","S");
		myHClade.put("776307777760771","S");
		myHClade.put("776317777760771","S");
		myHClade.put("776321377760771","S");
		myHClade.put("776321777760771","S");
		myHClade.put("776327777760771","S");
		myHClade.put("776333760360771","S");
		myHClade.put("776333777760771","S");
		myHClade.put("776335777760771","S");
		myHClade.put("776337377760771","S");
		myHClade.put("776337403760771","S");
		myHClade.put("776337407760771","LAM5");
		myHClade.put("776337677760771","S");
		myHClade.put("776337700760771","S");
		myHClade.put("776337770760771","S");
		myHClade.put("776337777000071","Unknown");
		myHClade.put("776337777360571","S");
		myHClade.put("776337777660731","S");
		myHClade.put("776337777760731","S");
		myHClade.put("776337777760771","S");
		myHClade.put("776337777763771","Manu2");
		myHClade.put("776340000000071","Unknown");
		myHClade.put("776340000000171","Unknown");
		myHClade.put("776340007760731","LAM4");
		myHClade.put("776340007760771","LAM9");
		myHClade.put("776347757760771","S");
		myHClade.put("776347777760661","S");
		myHClade.put("776357467760671","T-H37Rv");
		myHClade.put("776357777700371","Unknown");
		myHClade.put("776357777760571","S");
		myHClade.put("776357777760731","S");
		myHClade.put("776357777760771","S");
		myHClade.put("776360004720771","H3");
		myHClade.put("776361607760771","LAM9");
		myHClade.put("776361741760771","S");
		myHClade.put("776363400000171","Unknown");
		myHClade.put("776363677760771","S");
		myHClade.put("776363770040731","S");
		myHClade.put("776363777413731","EAI1-SOM");
		myHClade.put("776367377760771","S");
		myHClade.put("776367407760771","LAM9");
		myHClade.put("776367677760771","S");
		myHClade.put("776367770000731","Unknown");
		myHClade.put("776367774020771","H1");
		myHClade.put("776367777660771","S");
		myHClade.put("776367777760731","S");
		myHClade.put("776367777760771","S");
		myHClade.put("776370001760331","S");
		myHClade.put("776370001760731","S");
		myHClade.put("776370001760771","S");
		myHClade.put("776371001760771","S");
		myHClade.put("776371777760771","S");
		myHClade.put("776373001760771","S");
		myHClade.put("776373777760731","S");
		myHClade.put("776374007413771","EAI5");
		myHClade.put("776375767720771","H3");
		myHClade.put("776376003760771","S");
		myHClade.put("776376037760751","S");
		myHClade.put("776376037760771","S");
		myHClade.put("776376137760771","S");
		myHClade.put("776376177760771","S");
		myHClade.put("776376737560731","S");
		myHClade.put("776376777760601","X2");
		myHClade.put("776376777760731","S");
		myHClade.put("776377077760771","S");
		myHClade.put("776377177760771","S");
		myHClade.put("776377377760631","S");
		myHClade.put("776377377760731","S");
		myHClade.put("776377377760771","S");
		myHClade.put("776377400000171","Unknown");
		myHClade.put("776377403760771","S");
		myHClade.put("776377560000731","Unknown");
		myHClade.put("776377577760731","S");
		myHClade.put("776377577760771","S");
		myHClade.put("776377600000371","Unknown");
		myHClade.put("776377603760771","S");
		myHClade.put("776377606760711","LAM8");
		myHClade.put("776377607560771","LAM6");
		myHClade.put("776377607600371","Unknown");
		myHClade.put("776377607760100","LAM4");
		myHClade.put("776377607760570","LAM9");
		myHClade.put("776377607760571","LAM9");
		myHClade.put("776377607760771","LAM9");
		myHClade.put("776377637760771","S");
		myHClade.put("776377677700771","Unknown");
		myHClade.put("776377677760771","S");
		myHClade.put("776377700000001","Unknown");
		myHClade.put("776377700000611","Unknown");
		myHClade.put("776377700160771","S");
		myHClade.put("776377700360771","S");
		myHClade.put("776377701740731","S");
		myHClade.put("776377701760771","S");
		myHClade.put("776377703740731","S");
		myHClade.put("776377703760771","S");
		myHClade.put("776377707760771","S");
		myHClade.put("776377730360771","S");
		myHClade.put("776377733760771","S");
		myHClade.put("776377737560731","S");
		myHClade.put("776377737760711","S");
		myHClade.put("776377737760771","S");
		myHClade.put("776377740000031","Unknown");
		myHClade.put("776377740020731","H3");
		myHClade.put("776377740760771","S");
		myHClade.put("776377741760771","S");
		myHClade.put("776377743760771","Cameroon");
		myHClade.put("776377745720771","H3");
		myHClade.put("776377747413771","EAI6-BGD1");
		myHClade.put("776377747760771","S");
		myHClade.put("776377760000171","Unknown");
		myHClade.put("776377760000371","Unknown");
		myHClade.put("776377760000731","Unknown");
		myHClade.put("776377760020771","H3");
		myHClade.put("776377760360771","S");
		myHClade.put("776377761760771","S");
		myHClade.put("776377763760771","S");
		myHClade.put("776377767760771","S");
		myHClade.put("776377770000000","Unknown");
		myHClade.put("776377770000731","Unknown");
		myHClade.put("776377770000771","Unknown");
		myHClade.put("776377770020771","H3");
		myHClade.put("776377770200771","Unknown");
		myHClade.put("776377770360771","S");
		myHClade.put("776377770760731","S");
		myHClade.put("776377770760771","S");
		myHClade.put("776377771760771","S");
		myHClade.put("776377773740731","S");
		myHClade.put("776377773760771","S");
		myHClade.put("776377774000371","Unknown");
		myHClade.put("776377774020071","H1");
		myHClade.put("776377774020771","H1");
		myHClade.put("776377774030771","Manu3");
		myHClade.put("776377775540771","S");
		myHClade.put("776377775720071","H3");
		myHClade.put("776377775760771","S");
		myHClade.put("776377776360771","S");
		myHClade.put("776377776760771","S");
		myHClade.put("776377777000000","Unknown");
		myHClade.put("776377777000771","Unknown");
		myHClade.put("776377777360771","S");
		myHClade.put("776377777400071","Unknown");
		myHClade.put("776377777413731","EAI1-SOM");
		myHClade.put("776377777420771","Ural-1");
		myHClade.put("776377777520771","H3");
		myHClade.put("776377777560771","S");
		myHClade.put("776377777660771","S");
		myHClade.put("776377777700771","Unknown");
		myHClade.put("776377777720071","H3");
		myHClade.put("776377777720271","H3");
		myHClade.put("776377777720731","H3");
		myHClade.put("776377777720771","Unknown");
		myHClade.put("776377777730371","Unknown");
		myHClade.put("776377777740000","S");
		myHClade.put("776377777740731","S");
		myHClade.put("776377777740771","S");
		myHClade.put("776377777760071","S");
		myHClade.put("776377777760371","S");
		myHClade.put("776377777760431","S");
		myHClade.put("776377777760611","S");
		myHClade.put("776377777760700","S");
		myHClade.put("776377777760711","S");
		myHClade.put("776377777760731","S");
		myHClade.put("776377777760741","S");
		myHClade.put("776377777760751","S");
		myHClade.put("776377777760760","S");
		myHClade.put("776377777760770","S");
		myHClade.put("776377777760771","S");
		myHClade.put("776377777763771","Manu2");
		myHClade.put("776377777770771","Manu3");
		myHClade.put("776460647015740","Unknown");
		myHClade.put("776541402711340","Unknown");
		myHClade.put("776573777777600","Unknown");
		myHClade.put("776637777760771","T1");
		myHClade.put("776707777737071","Unknown");
		myHClade.put("776717777760771","T1");
		myHClade.put("776737377720771","H3");
		myHClade.put("776737377760700","T");
		myHClade.put("776737377760771","T");
		myHClade.put("776737407760771","LAM5");
		myHClade.put("776737437760771","T3");
		myHClade.put("776737607770371","Unknown");
		myHClade.put("776737617760771","T3");
		myHClade.put("776737677700011","Unknown");
		myHClade.put("776737777420771","Ural-1");
		myHClade.put("776737777760731","T");
		myHClade.put("776737777760751","T3");
		myHClade.put("776737777760771","T3");
		myHClade.put("776760003760771","LAM-RUS");
		myHClade.put("776763777773660","Manu1");
		myHClade.put("776767776020771","H3");
		myHClade.put("776767777777771","Unknown");
		myHClade.put("776773777777600","Unknown");
		myHClade.put("776775747413771","EAI6-BGD1");
		myHClade.put("776776707760601","X2");
		myHClade.put("776776777360771","X1");
		myHClade.put("776776777760601","X2");
		myHClade.put("776777347760471","T4-CEU1");
		myHClade.put("776777374020771","H1");
		myHClade.put("776777400000001","Unknown");
		myHClade.put("776777404760771","Turkey");
		myHClade.put("776777577720771","H3");
		myHClade.put("776777600000720","Unknown");
		myHClade.put("776777604000771","Unknown");
		myHClade.put("776777606020771","H3");
		myHClade.put("776777607560771","LAM6");
		myHClade.put("776777607760731","LAM4");
		myHClade.put("776777607760771","LAM9");
		myHClade.put("776777647763771","Manu2");
		myHClade.put("776777706060731","T2");
		myHClade.put("776777737760771","T1");
		myHClade.put("776777743760771","Cameroon");
		myHClade.put("776777757760771","T5");
		myHClade.put("776777770000731","Unknown");
		myHClade.put("776777770060771","T1");
		myHClade.put("776777774020771","H1");
		myHClade.put("776777776000271","Unknown");
		myHClade.put("776777777412731","EAI1-SOM");
		myHClade.put("776777777413731","EAI1-SOM");
		myHClade.put("776777777420771","Ural-1");
		myHClade.put("776777777720731","H3");
		myHClade.put("776777777720771","H3");
		myHClade.put("776777777760601","T1");
		myHClade.put("776777777760731","T2");
		myHClade.put("776777777760771","T1");
		myHClade.put("776777777770730","Manu3");
		myHClade.put("776777777777671","Unknown");
		myHClade.put("777000000000371","Unknown");
		myHClade.put("777000000000771","Unknown");
		myHClade.put("777000000020571","H3");
		myHClade.put("777000000060771","T1");
		myHClade.put("777000001760771","T1");
		myHClade.put("777000002000000","Unknown");
		myHClade.put("777000003760731","T2");
		myHClade.put("777000003760771","T1");
		myHClade.put("777000007720771","H3");
		myHClade.put("777000010060731","T2");
		myHClade.put("777000020020771","H3");
		myHClade.put("777000057760771","T5");
		myHClade.put("777000177760771","T1");
		myHClade.put("777000177770771","Manu3");
		myHClade.put("777000200000000","Unknown");
		myHClade.put("777000206060771","T3-ETH");
		myHClade.put("777000317760771","T3-ETH");
		myHClade.put("777000323506471","Unknown");
		myHClade.put("777000337760771","T3-ETH");
		myHClade.put("777000357760771","T3-ETH");
		myHClade.put("777000376760671","T3-ETH");
		myHClade.put("777000377560771","T3-ETH");
		myHClade.put("777000377720771","H3");
		myHClade.put("777000377740071","T3-ETH");
		myHClade.put("777000377760671","T3-ETH");
		myHClade.put("777000377760731","T3-ETH");
		myHClade.put("777000377760751","T3-ETH");
		myHClade.put("777000377760761","T3-ETH");
		myHClade.put("777000377760771","T3-ETH");
		myHClade.put("777000777760731","T2");
		myHClade.put("777000777760771","T1");
		myHClade.put("777001745720771","H3");
		myHClade.put("777001745760771","T1");
		myHClade.put("777002377760771","T1");
		myHClade.put("777002777760731","X1");
		myHClade.put("777003377760771","T4");
		myHClade.put("777003704020771","H1");
		myHClade.put("777003777413771","EAI5");
		myHClade.put("777003777760771","T1");
		myHClade.put("777007773757700","Unknown");
		myHClade.put("777007774020771","H1");
		myHClade.put("777007777760771","T1");
		myHClade.put("777016777760671","X1");
		myHClade.put("777017577420771","Ural-1");
		myHClade.put("777017607760771","LAM5");
		myHClade.put("777017777420771","Ural-1");
		myHClade.put("777017777700000","Unknown");
		myHClade.put("777017777760771","T1");
		myHClade.put("777020777760731","T2");
		myHClade.put("777033777760771","T1");
		myHClade.put("777036777760771","X1");
		myHClade.put("777037377413731","EAI1-SOM");
		myHClade.put("777037607760771","LAM5");
		myHClade.put("777037750060771","T5");
		myHClade.put("777037757760771","T5");
		myHClade.put("777037777413631","EAI5");
		myHClade.put("777037777413700","EAI5");
		myHClade.put("777037777413731","EAI1-SOM");
		myHClade.put("777037777760031","T1");
		myHClade.put("777037777760611","T1");
		myHClade.put("777037777760730","T2-Uganda");
		myHClade.put("777037777760771","T1");
		myHClade.put("777067777760771","T1");
		myHClade.put("777071377760771","T4");
		myHClade.put("777076777760731","X1");
		myHClade.put("777076777760771","X1");
		myHClade.put("777077037720771","H3");
		myHClade.put("777077607560771","LAM6");
		myHClade.put("777077607760771","LAM9");
		myHClade.put("777077700003371","Unknown");
		myHClade.put("777077777720771","H3");
		myHClade.put("777077777760771","T1");
		myHClade.put("777123777760771","T3");
		myHClade.put("777137757760771","T");
		myHClade.put("777137777760731","T");
		myHClade.put("777140007760771","LAM9");
		myHClade.put("777155774760761","T1");
		myHClade.put("777157607760771","LAM9");
		myHClade.put("777157777760671","T1");
		myHClade.put("777167607760671","LAM9");
		myHClade.put("777170777520771","H3");
		myHClade.put("777176003760771","T1");
		myHClade.put("777176707013771","EAI5");
		myHClade.put("777177000020771","H3");
		myHClade.put("777177377760731","T");
		myHClade.put("777177474760631","T-H37Rv");
		myHClade.put("777177600000000","Unknown");
		myHClade.put("777177606060771","LAM11-ZWE");
		myHClade.put("777177607760771","LAM9");
		myHClade.put("777177616040771","T1");
		myHClade.put("777177677760771","T1");
		myHClade.put("777177717760571","T1");
		myHClade.put("777177774000771","Unknown");
		myHClade.put("777177776160771","T1");
		myHClade.put("777177777413731","EAI1-SOM");
		myHClade.put("777177777413771","EAI5");
		myHClade.put("777177777460771","T1");
		myHClade.put("777177777510171","Unknown");
		myHClade.put("777177777720771","H3");
		myHClade.put("777177777760731","T2");
		myHClade.put("777177777760771","T1");
		myHClade.put("777207747760731","T2");
		myHClade.put("777207773757741","Unknown");
		myHClade.put("777236777760601","X2");
		myHClade.put("777237777420771","Ural-1");
		myHClade.put("777300003760771","T1");
		myHClade.put("777317607760671","LAM5");
		myHClade.put("777317607760771","LAM5");
		myHClade.put("777317677760771","T1");
		myHClade.put("777317777760731","T2");
		myHClade.put("777323777760771","T3");
		myHClade.put("777327607760671","LAM5");
		myHClade.put("777327607760770","LAM5");
		myHClade.put("777327607760771","LAM5");
		myHClade.put("777327677774771","Unknown");
		myHClade.put("777327777720771","H3");
		myHClade.put("777327777760731","T");
		myHClade.put("777336377760771","T3");
		myHClade.put("777337557560771","T5-Madrid2");
		myHClade.put("777337577360771","T3");
		myHClade.put("777337606060731","LAM11-ZWE");
		myHClade.put("777337606060771","LAM11-ZWE");
		myHClade.put("777337607560771","LAM6");
		myHClade.put("777337607660771","LAM5");
		myHClade.put("777337607760771","LAM5");
		myHClade.put("777337777760731","T");
		myHClade.put("777340004020771","H1");
		myHClade.put("777340007760771","LAM9");
		myHClade.put("777341017760771","T1");
		myHClade.put("777345650760661","T5");
		myHClade.put("777346777760601","X2");
		myHClade.put("777347557760661","T5-Madrid2");
		myHClade.put("777347606560671","LAM6");
		myHClade.put("777347607760661","LAM");
		myHClade.put("777347607760671","LAM9");
		myHClade.put("777347677403671","Unknown");
		myHClade.put("777347707720771","H3");
		myHClade.put("777347742140771","Cameroon");
		myHClade.put("777347757413771","EAI6-BGD1");
		myHClade.put("777347777413600","EAI5");
		myHClade.put("777347777413671","EAI5");
		myHClade.put("777347777433671","Manu1");
		myHClade.put("777347777720671","H3");
		myHClade.put("777347777760371","T1");
		myHClade.put("777347777760661","T1");
		myHClade.put("777347777760671","T1");
		myHClade.put("777347777760771","T1");
		myHClade.put("777347777763671","Manu2");
		myHClade.put("777347777763771","Manu2");
		myHClade.put("777353777420331","Ural-1");
		myHClade.put("777356774020661","H1");
		myHClade.put("777356777760601","X2");
		myHClade.put("777357606060571","LAM11-ZWE");
		myHClade.put("777357606060671","LAM11-ZWE");
		myHClade.put("777357606060731","LAM11-ZWE");
		myHClade.put("777357606060771","LAM11-ZWE");
		myHClade.put("777357607560731","LAM6");
		myHClade.put("777357607760631","LAM4");
		myHClade.put("777357607760671","LAM9");
		myHClade.put("777357607760771","LAM9");
		myHClade.put("777357677770761","Manu3");
		myHClade.put("777357677774671","Unknown");
		myHClade.put("777357777020771","H3");
		myHClade.put("777357777542771","Unknown");
		myHClade.put("777357777720671","H3");
		myHClade.put("777357777720771","H3");
		myHClade.put("777357777760611","T1");
		myHClade.put("777357777760631","T1");
		myHClade.put("777357777760671","T1");
		myHClade.put("777357777760771","T1");
		myHClade.put("777357777777251","Unknown");
		myHClade.put("777357777777771","Unknown");
		myHClade.put("777361377760671","T4");
		myHClade.put("777365637760771","T1");
		myHClade.put("777367400003771","Unknown");
		myHClade.put("777367477400771","Unknown");
		myHClade.put("777367577760771","T1");
		myHClade.put("777367606020661","H3");
		myHClade.put("777367606060361","LAM11-ZWE");
		myHClade.put("777367606060661","LAM11-ZWE");
		myHClade.put("777367606060771","LAM11-ZWE");
		myHClade.put("777367607560771","LAM6");
		myHClade.put("777367607760661","LAM9");
		myHClade.put("777367607760671","LAM9");
		myHClade.put("777367607760771","LAM9");
		myHClade.put("777367623540771","T1");
		myHClade.put("777367626760771","T1");
		myHClade.put("777367677763771","Manu2");
		myHClade.put("777367743760661","Cameroon");
		myHClade.put("777367756403671","Unknown");
		myHClade.put("777367757760771","T5");
		myHClade.put("777367757763771","Manu2");
		myHClade.put("777367761760771","T1");
		myHClade.put("777367774020661","H1");
		myHClade.put("777367774020771","H1");
		myHClade.put("777367776020771","H3");
		myHClade.put("777367776760771","T1");
		myHClade.put("777367777560661","T1");
		myHClade.put("777367777720661","H3");
		myHClade.put("777367777720671","H3");
		myHClade.put("777367777720771","H3");
		myHClade.put("777367777740771","T1");
		myHClade.put("777367777760661","T1");
		myHClade.put("777367777760671","T1");
		myHClade.put("777367777760761","T1");
		myHClade.put("777367777760771","T1");
		myHClade.put("777375777760771","T1");
		myHClade.put("777376077560771","T1");
		myHClade.put("777376607560771","LAM6");
		myHClade.put("777376677763771","Manu2");
		myHClade.put("777376750003271","EAI7-BGD2");
		myHClade.put("777376776360601","X2");
		myHClade.put("777376777560601","X2");
		myHClade.put("777376777760601","X2");
		myHClade.put("777376777760771","X1");
		myHClade.put("777377346763771","Manu2");
		myHClade.put("777377347760400","T4");
		myHClade.put("777377347760471","T4-CEU1");
		myHClade.put("777377577360771","T1");
		myHClade.put("777377577771771","Unknown");
		myHClade.put("777377600000000","Unknown");
		myHClade.put("777377606060671","LAM11-ZWE");
		myHClade.put("777377606060731","LAM11-ZWE");
		myHClade.put("777377606060771","LAM11-ZWE");
		myHClade.put("777377606760771","LAM8");
		myHClade.put("777377607560761","LAM6");
		myHClade.put("777377607700771","Unknown");
		myHClade.put("777377607760671","LAM9");
		myHClade.put("777377607760731","LAM4");
		myHClade.put("777377607760761","LAM9");
		myHClade.put("777377607760771","LAM9");
		myHClade.put("777377637760771","T1");
		myHClade.put("777377647760771","T1");
		myHClade.put("777377663760771","T1");
		myHClade.put("777377677760671","T");
		myHClade.put("777377677763771","Manu2");
		myHClade.put("777377677777771","Unknown");
		myHClade.put("777377700003371","Unknown");
		myHClade.put("777377700003531","EAI1-SOM");
		myHClade.put("777377702003371","Unknown");
		myHClade.put("777377702003531","EAI1-SOM");
		myHClade.put("777377706001351","Unknown");
		myHClade.put("777377706001371","Unknown");
		myHClade.put("777377706060731","T2");
		myHClade.put("777377707760771","T1");
		myHClade.put("777377717760671","T1");
		myHClade.put("777377740003371","Unknown");
		myHClade.put("777377757413271","EAI6-BGD1");
		myHClade.put("777377757413331","EAI");
		myHClade.put("777377757413371","EAI6-BGD1");
		myHClade.put("777377757413771","EAI6-BGD1");
		myHClade.put("777377757760771","T5");
		myHClade.put("777377762013731","EAI1-SOM");
		myHClade.put("777377766320361","H3");
		myHClade.put("777377767560761","T1");
		myHClade.put("777377770000000","Unknown");
		myHClade.put("777377770000131","Unknown");
		myHClade.put("777377770000671","Unknown");
		myHClade.put("777377770000771","Unknown");
		myHClade.put("777377774020621","H1");
		myHClade.put("777377774020731","H1");
		myHClade.put("777377774020771","H1");
		myHClade.put("777377777413731","EAI1-SOM");
		myHClade.put("777377777413771","EAI5");
		myHClade.put("777377777520771","H3");
		myHClade.put("777377777573771","Manu1");
		myHClade.put("777377777620731","H3");
		myHClade.put("777377777700000","Unknown");
		myHClade.put("777377777720771","H3");
		myHClade.put("777377777740771","T1");
		myHClade.put("777377777757771","Unknown");
		myHClade.put("777377777760601","T1");
		myHClade.put("777377777760661","T1");
		myHClade.put("777377777760671","T1");
		myHClade.put("777377777760700","T1");
		myHClade.put("777377777760731","T2");
		myHClade.put("777377777760751","T1");
		myHClade.put("777377777760771","T1");
		myHClade.put("777377777761771","Unknown");
		myHClade.put("777377777763671","Manu2");
		myHClade.put("777377777763771","Manu2");
		myHClade.put("777377777770371","Unknown");
		myHClade.put("777377777770771","Manu3");
		myHClade.put("777377777773771","Manu1");
		myHClade.put("777377777777771","Unknown");
		myHClade.put("777400000060771","T1");
		myHClade.put("777400000760731","T2");
		myHClade.put("777400002760771","T1");
		myHClade.put("777400004020771","H1");
		myHClade.put("777400004120771","H3");
		myHClade.put("777400007720771","H3");
		myHClade.put("777400007760771","LAM5");
		myHClade.put("777400014020771","H1");
		myHClade.put("777400017660731","T2");
		myHClade.put("777400017720771","H3");
		myHClade.put("777400037760771","T1");
		myHClade.put("777400037770371","Unknown");
		myHClade.put("777400167760771","T1");
		myHClade.put("777400377760771","T1");
		myHClade.put("777400777720771","H3");
		myHClade.put("777401607760771","LAM5");
		myHClade.put("777402000000171","Unknown");
		myHClade.put("777403607760771","LAM5");
		myHClade.put("777403777720771","H3");
		myHClade.put("777406777760771","X1");
		myHClade.put("777407607760771","LAM5");
		myHClade.put("777407773757741","Unknown");
		myHClade.put("777417600000000","Unknown");
		myHClade.put("777417606060731","LAM11-ZWE");
		myHClade.put("777417607760700","LAM");
		myHClade.put("777417700000000","Unknown");
		myHClade.put("777417707700000","Unknown");
		myHClade.put("777417730000000","Unknown");
		myHClade.put("777417737700000","Unknown");
		myHClade.put("777417740773771","Manu1");
		myHClade.put("777417760020611","H3");
		myHClade.put("777417760060771","T1");
		myHClade.put("777417774413711","EAI4-VNM");
		myHClade.put("777417777400000","Unknown");
		myHClade.put("777417777420771","Ural-1");
		myHClade.put("777417777700000","Unknown");
		myHClade.put("777417777720771","H3");
		myHClade.put("777417777760731","T2");
		myHClade.put("777436777760731","X1");
		myHClade.put("777437707700000","Unknown");
		myHClade.put("777437737700000","Unknown");
		myHClade.put("777437757413371","EAI6-BGD1");
		myHClade.put("777437774020771","H1");
		myHClade.put("777437777720771","H3");
		myHClade.put("777437777740371","T1");
		myHClade.put("777437777760671","T1");
		myHClade.put("777437777760700","T");
		myHClade.put("777460003760771","LAM-RUS");
		myHClade.put("777460007760771","LAM-RUS");
		myHClade.put("777467607500671","Unknown");
		myHClade.put("777467607760771","LAM9");
		myHClade.put("777476777760771","X1");
		myHClade.put("777477607700771","Unknown");
		myHClade.put("777477607760771","LAM9");
		myHClade.put("777477607763771","Manu2");
		myHClade.put("777477677760731","T2");
		myHClade.put("777477757413371","EAI6-BGD1");
		myHClade.put("777477757413771","EAI6-BGD1");
		myHClade.put("777477764020771","H1");
		myHClade.put("777477770020771","H3");
		myHClade.put("777477774020731","H1");
		myHClade.put("777477776760770","T1");
		myHClade.put("777477777407371","Unknown");
		myHClade.put("777477777460731","T2");
		myHClade.put("777477777720771","H3");
		myHClade.put("777477777740371","T1");
		myHClade.put("777477777760711","T1");
		myHClade.put("777477777760731","T2");
		myHClade.put("777477777760771","T1");
		myHClade.put("777500003760771","T1");
		myHClade.put("777535777760730","T2-Uganda");
		myHClade.put("777537606760771","LAM8");
		myHClade.put("777537607760771","LAM5");
		myHClade.put("777537777413371","EAI5");
		myHClade.put("777537777420771","Ural-1");
		myHClade.put("777537777760771","T3");
		myHClade.put("777557733760771","T1");
		myHClade.put("777557757003331","Unknown");
		myHClade.put("777557773760771","T1");
		myHClade.put("777557777003731","Unknown");
		myHClade.put("777557777760431","T1");
		myHClade.put("777566777760771","X1");
		myHClade.put("777567607760771","LAM9");
		myHClade.put("777567777003731","Unknown");
		myHClade.put("777567777760671","T1");
		myHClade.put("777567777760771","T1");
		myHClade.put("777576757413771","EAI6-BGD1");
		myHClade.put("777577600000000","Unknown");
		myHClade.put("777577604000771","Unknown");
		myHClade.put("777577606060731","LAM11-ZWE");
		myHClade.put("777577606060751","LAM11-ZWE");
		myHClade.put("777577606060771","LAM11-ZWE");
		myHClade.put("777577607760771","LAM9");
		myHClade.put("777577737760771","T1");
		myHClade.put("777577747760471","T1");
		myHClade.put("777577747760771","T1");
		myHClade.put("777577757003300","Unknown");
		myHClade.put("777577757413771","EAI6-BGD1");
		myHClade.put("777577767777771","Unknown");
		myHClade.put("777577774020771","H1");
		myHClade.put("777577777001731","Unknown");
		myHClade.put("777577777003731","EAI1-SOM");
		myHClade.put("777577777003771","Unknown");
		myHClade.put("777577777013371","EAI5");
		myHClade.put("777577777403171","Unknown");
		myHClade.put("777577777413071","EAI5");
		myHClade.put("777577777413231","EAI5");
		myHClade.put("777577777413371","EAI5");
		myHClade.put("777577777413771","EAI5");
		myHClade.put("777577777417071","Unknown");
		myHClade.put("777577777460771","T1");
		myHClade.put("777577777620771","H3");
		myHClade.put("777577777720771","H3");
		myHClade.put("777577777740771","T1");
		myHClade.put("777577777760331","T2");
		myHClade.put("777577777760471","T1");
		myHClade.put("777577777760730","T2-Uganda");
		myHClade.put("777577777760731","T2");
		myHClade.put("777577777760771","T1");
		myHClade.put("777577777777771","Unknown");
		myHClade.put("777600000000000","Unknown");
		myHClade.put("777600000000031","Unknown");
		myHClade.put("777600000000171","Unknown");
		myHClade.put("777600000020771","H3");
		myHClade.put("777600000060771","T1");
		myHClade.put("777600000760711","T1");
		myHClade.put("777600000760771","T1");
		myHClade.put("777600001760711","T1");
		myHClade.put("777600001760771","T1");
		myHClade.put("777600003760771","Unknown");
		myHClade.put("777600005413371","EAI5");
		myHClade.put("777600007400261","Unknown");
		myHClade.put("777600007413371","EAI5");
		myHClade.put("777600007413771","EAI5");
		myHClade.put("777600007760771","LAM5");
		myHClade.put("777600017413371","EAI5");
		myHClade.put("777600037760771","Unknown");
		myHClade.put("777600073760771","T1");
		myHClade.put("777600077760771","T1");
		myHClade.put("777600167760771","T1");
		myHClade.put("777600207760771","LAM5");
		myHClade.put("777600377413731","EAI1-SOM");
		myHClade.put("777600377760771","T1");
		myHClade.put("777600674020771","H1");
		myHClade.put("777600774020771","H1");
		myHClade.put("777600774413771","EAI4-VNM");
		myHClade.put("777600777720771","H3");
		myHClade.put("777600777760700","T1");
		myHClade.put("777600777760771","T1");
		myHClade.put("777601757413371","EAI6-BGD1");
		myHClade.put("777601760000171","Unknown");
		myHClade.put("777601774413771","EAI4-VNM");
		myHClade.put("777601776413771","EAI5");
		myHClade.put("777601777760731","T2");
		myHClade.put("777602004360471","LAM5");
		myHClade.put("777603000000011","Zero-copy");
		myHClade.put("777603017760631","T1");
		myHClade.put("777603347760471","T4-CEU1");
		myHClade.put("777603404060471","LAM5");
		myHClade.put("777603405760430","LAM");
		myHClade.put("777603405760471","LAM5");
		myHClade.put("777603405760771","LAM5");
		myHClade.put("777603415760771","T1");
		myHClade.put("777603427760771","T1");
		myHClade.put("777603606060771","LAM11-ZWE");
		myHClade.put("777603607400011","Unknown");
		myHClade.put("777603677760701","T1");
		myHClade.put("777603717760771","T1");
		myHClade.put("777603777713771","Manu1");
		myHClade.put("777603777720731","H3");
		myHClade.put("777606777760771","X1");
		myHClade.put("777607607760771","LAM5");
		myHClade.put("777607770000731","Unknown");
		myHClade.put("777607777720771","H3");
		myHClade.put("777607777760671","T1");
		myHClade.put("777607777760771","T1");
		myHClade.put("777613405760471","LAM");
		myHClade.put("777614007760731","LAM");
		myHClade.put("777616777760771","X1");
		myHClade.put("777617017777771","Unknown");
		myHClade.put("777617347760471","T4-CEU1");
		myHClade.put("777617404760771","Turkey");
		myHClade.put("777617600000000","Unknown");
		myHClade.put("777617607760771","LAM5");
		myHClade.put("777617705413371","EAI5");
		myHClade.put("777617707760771","T1");
		myHClade.put("777617757413371","EAI6-BGD1");
		myHClade.put("777617770020771","H3");
		myHClade.put("777617777413771","EAI5");
		myHClade.put("777617777700000","Unknown");
		myHClade.put("777617777720771","H3");
		myHClade.put("777617777760731","T2");
		myHClade.put("777617777760771","T3");
		myHClade.put("777617777777771","Unknown");
		myHClade.put("777621777760771","T1");
		myHClade.put("777623377760771","T4");
		myHClade.put("777623405760471","LAM5");
		myHClade.put("777623405760771","LAM5");
		myHClade.put("777623777760731","T2");
		myHClade.put("777623777760771","T1");
		myHClade.put("777627777720771","H3");
		myHClade.put("777633777420771","Ural-1");
		myHClade.put("777635757413371","EAI6-BGD1");
		myHClade.put("777636777413731","EAI1-SOM");
		myHClade.put("777637002060771","T1");
		myHClade.put("777637077760771","T1");
		myHClade.put("777637347760471","T4-CEU1");
		myHClade.put("777637417560771","T");
		myHClade.put("777637547013771","EAI5");
		myHClade.put("777637600020771","H3");
		myHClade.put("777637600500771","Unknown");
		myHClade.put("777637603760771","T1");
		myHClade.put("777637607400031","LAM");
		myHClade.put("777637607560771","LAM6");
		myHClade.put("777637607760771","LAM5");
		myHClade.put("777637677420771","Ural-1");
		myHClade.put("777637763660771","T1");
		myHClade.put("777637770000000","Unknown");
		myHClade.put("777637770020771","H3");
		myHClade.put("777637774020731","H1");
		myHClade.put("777637774020771","H1");
		myHClade.put("777637777410071","EAI5");
		myHClade.put("777637777410171","EAI5");
		myHClade.put("777637777413731","EAI1-SOM");
		myHClade.put("777637777413771","EAI5");
		myHClade.put("777637777420731","Ural-1");
		myHClade.put("777637777420771","Ural-1");
		myHClade.put("777637777720760","H3");
		myHClade.put("777637777720771","H3");
		myHClade.put("777637777723771","Manu2");
		myHClade.put("777637777740771","T1");
		myHClade.put("777637777760071","T1");
		myHClade.put("777637777760471","T1");
		myHClade.put("777637777760711","T1");
		myHClade.put("777637777760731","T2");
		myHClade.put("777637777760771","T1");
		myHClade.put("777637777770711","Manu3");
		myHClade.put("777637777777771","Unknown");
		myHClade.put("777640007720731","H3");
		myHClade.put("777640007760671","LAM9");
		myHClade.put("777640007760731","Unknown");
		myHClade.put("777640007760771","LAM");
		myHClade.put("777640037760771","T1");
		myHClade.put("777640107760771","T1");
		myHClade.put("777647000000011","Zero-copy");
		myHClade.put("777657477760771","T-H37Rv");
		myHClade.put("777657777720771","H3");
		myHClade.put("777657777760771","T1");
		myHClade.put("777660007760771","LAM-RUS");
		myHClade.put("777660017760771","T-tuscany");
		myHClade.put("777660040000071","Unknown");
		myHClade.put("777661043333771","Manu1");
		myHClade.put("777663405760471","LAM9");
		myHClade.put("777667477760771","T-H37Rv");
		myHClade.put("777667607760771","LAM9");
		myHClade.put("777667777700000","Unknown");
		myHClade.put("777667777740771","T1");
		myHClade.put("777667777760771","T1");
		myHClade.put("777672757413771","EAI6-BGD1");
		myHClade.put("777674077413671","EAI5");
		myHClade.put("777675757413370","EAI6-BGD1");
		myHClade.put("777675757413371","EAI6-BGD1");
		myHClade.put("777675777760600","T1");
		myHClade.put("777676776360601","X2");
		myHClade.put("777676777760601","X2");
		myHClade.put("777676777760771","X1");
		myHClade.put("777677003423771","Manu2");
		myHClade.put("777677207600771","Unknown");
		myHClade.put("777677207760771","LAM9");
		myHClade.put("777677374020731","H1");
		myHClade.put("777677377760771","T4");
		myHClade.put("777677577413771","EAI5");
		myHClade.put("777677606060771","LAM11-ZWE");
		myHClade.put("777677607560771","LAM6");
		myHClade.put("777677607760771","LAM9");
		myHClade.put("777677637763771","Manu2");
		myHClade.put("777677647403371","Unknown");
		myHClade.put("777677677760771","T1");
		myHClade.put("777677677770771","Manu3");
		myHClade.put("777677700020771","H3");
		myHClade.put("777677700041771","Unknown");
		myHClade.put("777677704020751","H1");
		myHClade.put("777677723760731","T2");
		myHClade.put("777677737760771","T1");
		myHClade.put("777677740013371","EAI5");
		myHClade.put("777677741760771","T1");
		myHClade.put("777677747760471","T1");
		myHClade.put("777677757412371","EAI6-BGD1");
		myHClade.put("777677757413171","EAI6-BGD1");
		myHClade.put("777677757720771","H3");
		myHClade.put("777677770000000","Unknown");
		myHClade.put("777677770060771","T1");
		myHClade.put("777677774014351","Unknown");
		myHClade.put("777677774020771","H1");
		myHClade.put("777677777413431","EAI5");
		myHClade.put("777677777413671","EAI5");
		myHClade.put("777677777413731","EAI1-SOM");
		myHClade.put("777677777413771","EAI5");
		myHClade.put("777677777720731","H3");
		myHClade.put("777677777720771","H3");
		myHClade.put("777677777760471","T1");
		myHClade.put("777677777760731","T2");
		myHClade.put("777677777760750","T1");
		myHClade.put("777677777760771","T1");
		myHClade.put("777677777770771","Manu3");
		myHClade.put("777700000000000","Unknown");
		myHClade.put("777700000000011","Unknown");
		myHClade.put("777700000000071","Unknown");
		myHClade.put("777700000000671","Unknown");
		myHClade.put("777700000001031","Unknown");
		myHClade.put("777700000001131","Unknown");
		myHClade.put("777700000001331","Unknown");
		myHClade.put("777700000001431","Unknown");
		myHClade.put("777700000001631","Unknown");
		myHClade.put("777700000001731","Unknown");
		myHClade.put("777700000060771","Unknown");
		myHClade.put("777700000360771","T1");
		myHClade.put("777700000760751","T1");
		myHClade.put("777700000760771","T1");
		myHClade.put("777700001760771","Unknown");
		myHClade.put("777700002001731","Unknown");
		myHClade.put("777700003760000","T1");
		myHClade.put("777700003760631","T1");
		myHClade.put("777700003760661","T1");
		myHClade.put("777700003760771","Unknown");
		myHClade.put("777700004020771","H1");
		myHClade.put("777700007413731","EAI1-SOM");
		myHClade.put("777700007560771","LAM6");
		myHClade.put("777700007760771","LAM");
		myHClade.put("777700017560771","T1");
		myHClade.put("777700017760771","T1");
		myHClade.put("777700037760771","T1");
		myHClade.put("777700043700171","Unknown");
		myHClade.put("777700074413771","EAI4-VNM");
		myHClade.put("777700076413771","EAI5");
		myHClade.put("777700077760600","T1");
		myHClade.put("777700077760771","Unknown");
		myHClade.put("777700207760771","LAM5");
		myHClade.put("777700277760771","T1");
		myHClade.put("777700377760771","T1");
		myHClade.put("777700607740771","LAM5");
		myHClade.put("777700607760771","LAM5");
		myHClade.put("777700637760771","T1");
		myHClade.put("777700654000000","Unknown");
		myHClade.put("777700700000611","Unknown");
		myHClade.put("777700770000071","Unknown");
		myHClade.put("777700776413771","EAI5");
		myHClade.put("777700777413700","EAI5");
		myHClade.put("777700777760601","T1");
		myHClade.put("777700777760771","T1");
		myHClade.put("777701000000000","Unknown");
		myHClade.put("777701000000431","Unknown");
		myHClade.put("777701000020771","H3");
		myHClade.put("777701000160771","T1");
		myHClade.put("777701001760771","T1");
		myHClade.put("777701002001431","Unknown");
		myHClade.put("777701002001731","Unknown");
		myHClade.put("777701003560771","LAM6");
		myHClade.put("777701003760771","T1");
		myHClade.put("777701007760771","LAM5");
		myHClade.put("777701017760771","T1");
		myHClade.put("777701037760771","T1");
		myHClade.put("777701377720771","H3");
		myHClade.put("777701400000011","Unknown");
		myHClade.put("777701400000300","Unknown");
		myHClade.put("777701607560731","LAM6");
		myHClade.put("777701607760770","LAM5");
		myHClade.put("777701607760771","LAM5");
		myHClade.put("777701757760771","T5");
		myHClade.put("777701777413700","EAI5");
		myHClade.put("777701777760771","T1");
		myHClade.put("777702004001731","Unknown");
		myHClade.put("777702017400771","Unknown");
		myHClade.put("777702777760771","X1");
		myHClade.put("777703000760771","T1");
		myHClade.put("777703077760771","T1");
		myHClade.put("777703377760731","T");
		myHClade.put("777703400000360","Unknown");
		myHClade.put("777703600020770","H3");
		myHClade.put("777703600020771","H3");
		myHClade.put("777703607760771","LAM5");
		myHClade.put("777703677720771","H3");
		myHClade.put("777703700000000","Unknown");
		myHClade.put("777703700003371","Unknown");
		myHClade.put("777703737760771","T1");
		myHClade.put("777703747760771","T1");
		myHClade.put("777703757760771","T5");
		myHClade.put("777703757761771","Unknown");
		myHClade.put("777703774000011","Unknown");
		myHClade.put("777703776760771","T1");
		myHClade.put("777703777000071","Unknown");
		myHClade.put("777703777360771","T");
		myHClade.put("777703777400371","Unknown");
		myHClade.put("777703777460771","T1");
		myHClade.put("777703777560731","T2");
		myHClade.put("777703777560771","T1");
		myHClade.put("777703777720771","H3");
		myHClade.put("777703777760471","T1");
		myHClade.put("777703777760770","T1");
		myHClade.put("777703777760771","T");
		myHClade.put("777705077560571","T1");
		myHClade.put("777707377760771","T4");
		myHClade.put("777707437000771","Unknown");
		myHClade.put("777707606060771","LAM11-ZWE");
		myHClade.put("777707607560771","LAM6");
		myHClade.put("777707607760771","LAM5");
		myHClade.put("777707607770771","Manu3");
		myHClade.put("777707637740771","T1");
		myHClade.put("777707677763771","Manu2");
		myHClade.put("777707737720771","H3");
		myHClade.put("777707737777771","Unknown");
		myHClade.put("777707743760771","Cameroon");
		myHClade.put("777707760000171","Unknown");
		myHClade.put("777707760020771","H3");
		myHClade.put("777707767020771","H3");
		myHClade.put("777707776360751","T1");
		myHClade.put("777707777413771","EAI5");
		myHClade.put("777707777620671","H3");
		myHClade.put("777707777720771","H3");
		myHClade.put("777707777760730","T2-Uganda");
		myHClade.put("777707777760731","T2");
		myHClade.put("777707777760771","T1");
		myHClade.put("777716777760601","X2");
		myHClade.put("777716777760771","X1");
		myHClade.put("777717377760771","T4");
		myHClade.put("777717605760771","LAM5");
		myHClade.put("777717606060771","LAM11-ZWE");
		myHClade.put("777717606760770","LAM8");
		myHClade.put("777717607020771","H3");
		myHClade.put("777717607560771","LAM6");
		myHClade.put("777717607760771","LAM5");
		myHClade.put("777717630000000","Unknown");
		myHClade.put("777717743760771","Cameroon");
		myHClade.put("777717761720771","H3");
		myHClade.put("777717774020771","H1");
		myHClade.put("777717776760731","T2");
		myHClade.put("777717777413671","EAI5");
		myHClade.put("777717777413731","EAI1-SOM");
		myHClade.put("777717777660771","T1");
		myHClade.put("777717777720771","H3");
		myHClade.put("777717777760371","T1");
		myHClade.put("777717777760571","T1");
		myHClade.put("777717777760671","T1");
		myHClade.put("777717777760730","T2-Uganda");
		myHClade.put("777717777760731","T2");
		myHClade.put("777717777760770","T1");
		myHClade.put("777717777760771","T1");
		myHClade.put("777720000020771","H3");
		myHClade.put("777720007760771","LAM-RUS");
		myHClade.put("777720774760771","T3");
		myHClade.put("777720777760671","T3");
		myHClade.put("777720777760771","T3");
		myHClade.put("777721000001731","Unknown");
		myHClade.put("777721002001431","Unknown");
		myHClade.put("777721007760771","LAM5");
		myHClade.put("777721607560731","LAM6");
		myHClade.put("777721607570731","Manu3");
		myHClade.put("777721607570771","Manu3");
		myHClade.put("777721777720731","H3");
		myHClade.put("777721777760771","T3");
		myHClade.put("777723002001731","Unknown");
		myHClade.put("777723400000360","Unknown");
		myHClade.put("777723607760571","LAM5");
		myHClade.put("777723607760771","LAM5");
		myHClade.put("777723717760771","T3");
		myHClade.put("777723737760771","T3");
		myHClade.put("777723775413731","EAI1-SOM");
		myHClade.put("777723777360771","T3");
		myHClade.put("777723777420771","Ural-1");
		myHClade.put("777723777760731","T");
		myHClade.put("777723777760771","T3");
		myHClade.put("777724234320771","H3");
		myHClade.put("777727000000471","Unknown");
		myHClade.put("777727400000771","Unknown");
		myHClade.put("777727404760771","Turkey");
		myHClade.put("777727606060771","LAM11-ZWE");
		myHClade.put("777727607160771","LAM5");
		myHClade.put("777727607700031","Unknown");
		myHClade.put("777727607760761","LAM5");
		myHClade.put("777727607760770","LAM5");
		myHClade.put("777727607760771","LAM5");
		myHClade.put("777727637760771","T3");
		myHClade.put("777727677413731","EAI1-SOM");
		myHClade.put("777727737760760","T3");
		myHClade.put("777727737760771","T3");
		myHClade.put("777727757760771","T");
		myHClade.put("777727761760771","T3");
		myHClade.put("777727770000000","Unknown");
		myHClade.put("777727770000071","Unknown");
		myHClade.put("777727770000771","Unknown");
		myHClade.put("777727770020720","H3");
		myHClade.put("777727775540731","T");
		myHClade.put("777727777000771","Unknown");
		myHClade.put("777727777413731","EAI1-SOM");
		myHClade.put("777727777420771","Ural-1");
		myHClade.put("777727777760071","T3");
		myHClade.put("777727777760731","T");
		myHClade.put("777727777760771","T3");
		myHClade.put("777727777770351","Unknown");
		myHClade.put("777730000000771","Unknown");
		myHClade.put("777730004160771","LAM5");
		myHClade.put("777730607760771","LAM5");
		myHClade.put("777730777760771","T3");
		myHClade.put("777731374020771","H1");
		myHClade.put("777731603770371","Unknown");
		myHClade.put("777731607760771","LAM5");
		myHClade.put("777731757413671","EAI6-BGD1");
		myHClade.put("777731777760731","T");
		myHClade.put("777733607760771","LAM5");
		myHClade.put("777733777420771","Ural-1");
		myHClade.put("777733777760731","T");
		myHClade.put("777734047560771","T3");
		myHClade.put("777734077560771","T3");
		myHClade.put("777734277700771","Unknown");
		myHClade.put("777734377760771","T3");
		myHClade.put("777735207760770","LAM5");
		myHClade.put("777735207760771","LAM5");
		myHClade.put("777735747413771","EAI6-BGD1");
		myHClade.put("777735777420771","Ural-1");
		myHClade.put("777735777423771","Manu2");
		myHClade.put("777735777450771","Manu3");
		myHClade.put("777735777760730","T2-Uganda");
		myHClade.put("777735777760771","T3");
		myHClade.put("777736000000000","Unknown");
		myHClade.put("777736077570771","Manu3");
		myHClade.put("777736377760571","T3");
		myHClade.put("777736407760601","X2");
		myHClade.put("777736570000000","Unknown");
		myHClade.put("777736600000000","Unknown");
		myHClade.put("777736607760771","LAM5");
		myHClade.put("777736707760771","X1");
		myHClade.put("777736747413771","EAI6-BGD1");
		myHClade.put("777736770000000","Unknown");
		myHClade.put("777736776000071","Unknown");
		myHClade.put("777736777360771","X1");
		myHClade.put("777736777420771","Ural-1");
		myHClade.put("777736777560731","X1");
		myHClade.put("777736777760571","X1");
		myHClade.put("777736777760601","X2");
		myHClade.put("777736777760671","X1");
		myHClade.put("777736777760731","X1");
		myHClade.put("777736777760751","X1");
		myHClade.put("777736777760771","X1");
		myHClade.put("777737007760771","LAM5");
		myHClade.put("777737017420771","Ural-1");
		myHClade.put("777737037760771","T3");
		myHClade.put("777737177720771","H3");
		myHClade.put("777737177760771","T3");
		myHClade.put("777737206060731","LAM11-ZWE");
		myHClade.put("777737206060771","LAM11-ZWE");
		myHClade.put("777737207760771","LAM5");
		myHClade.put("777737240000031","Unknown");
		myHClade.put("777737340000371","Unknown");
		myHClade.put("777737347760411","T");
		myHClade.put("777737347760471","T4-CEU1");
		myHClade.put("777737347760771","T");
		myHClade.put("777737357773771","Manu1");
		myHClade.put("777737374020771","H1");
		myHClade.put("777737376000171","Unknown");
		myHClade.put("777737377420771","Ural-1");
		myHClade.put("777737377460771","T");
		myHClade.put("777737377720700","H3");
		myHClade.put("777737377720771","H3");
		myHClade.put("777737377760731","T");
		myHClade.put("777737377760771","T");
		myHClade.put("777737400000011","Unknown");
		myHClade.put("777737400003771","Unknown");
		myHClade.put("777737400413731","EAI1-SOM");
		myHClade.put("777737400600171","Unknown");
		myHClade.put("777737404020771","H1");
		myHClade.put("777737404760771","Turkey");
		myHClade.put("777737405777731","Unknown");
		myHClade.put("777737407777731","Unknown");
		myHClade.put("777737467760771","T-H37Rv");
		myHClade.put("777737477400001","Unknown");
		myHClade.put("777737477760751","T-H37Rv");
		myHClade.put("777737477760771","T-H37Rv");
		myHClade.put("777737557560771","T5-Madrid2");
		myHClade.put("777737557760771","T5-Madrid2");
		myHClade.put("777737557770771","Manu3");
		myHClade.put("777737570000000","Unknown");
		myHClade.put("777737577420771","Ural-1");
		myHClade.put("777737600000000","Unknown");
		myHClade.put("777737600000171","Unknown");
		myHClade.put("777737600020771","H3");
		myHClade.put("777737603774351","Unknown");
		myHClade.put("777737604760771","LAM5");
		myHClade.put("777737605700771","Unknown");
		myHClade.put("777737606000171","Unknown");
		myHClade.put("777737606060731","LAM11-ZWE");
		myHClade.put("777737606060771","LAM11-ZWE");
		myHClade.put("777737606063771","Manu2");
		myHClade.put("777737606360771","LAM5");
		myHClade.put("777737606560771","LAM6");
		myHClade.put("777737606760771","LAM8");
		myHClade.put("777737607000771","Unknown");
		myHClade.put("777737607020771","H3");
		myHClade.put("777737607400031","Unknown");
		myHClade.put("777737607400071","Unknown");
		myHClade.put("777737607420771","Ural-1");
		myHClade.put("777737607460771","LAM5");
		myHClade.put("777737607560570","LAM6");
		myHClade.put("777737607560731","LAM");
		myHClade.put("777737607560760","LAM6");
		myHClade.put("777737607560771","LAM6");
		myHClade.put("777737607660471","LAM5");
		myHClade.put("777737607660771","LAM5");
		myHClade.put("777737607700031","Unknown");
		myHClade.put("777737607700171","Unknown");
		myHClade.put("777737607700351","Unknown");
		myHClade.put("777737607700371","Unknown");
		myHClade.put("777737607700771","Unknown");
		myHClade.put("777737607703771","Unknown");
		myHClade.put("777737607740771","LAM5");
		myHClade.put("777737607760000","LAM");
		myHClade.put("777737607760371","LAM5");
		myHClade.put("777737607760400","LAM");
		myHClade.put("777737607760671","LAM5");
		myHClade.put("777737607760700","LAM");
		myHClade.put("777737607760711","LAM");
		myHClade.put("777737607760731","LAM");
		myHClade.put("777737607760760","LAM5");
		myHClade.put("777737607760770","LAM5");
		myHClade.put("777737607760771","LAM5");
		myHClade.put("777737607770771","Manu3");
		myHClade.put("777737637400071","Unknown");
		myHClade.put("777737637760071","T3");
		myHClade.put("777737637760771","T3");
		myHClade.put("777737663560731","T");
		myHClade.put("777737667760771","T3");
		myHClade.put("777737674020771","H1");
		myHClade.put("777737676740131","T");
		myHClade.put("777737676760771","T3");
		myHClade.put("777737677420731","Ural-1");
		myHClade.put("777737677420771","Ural-1");
		myHClade.put("777737677560771","T3");
		myHClade.put("777737677720731","H3");
		myHClade.put("777737677760671","T3");
		myHClade.put("777737677760771","T3");
		myHClade.put("777737700000001","Unknown");
		myHClade.put("777737700000171","Unknown");
		myHClade.put("777737700020731","H3");
		myHClade.put("777737703420771","Ural-1");
		myHClade.put("777737703605751","Unknown");
		myHClade.put("777737707413771","EAI5");
		myHClade.put("777737707760711","T3");
		myHClade.put("777737707760771","T3");
		myHClade.put("777737710000131","Unknown");
		myHClade.put("777737717760731","T");
		myHClade.put("777737717760771","T3");
		myHClade.put("777737730000000","Unknown");
		myHClade.put("777737737420771","Ural-1");
		myHClade.put("777737737741771","Unknown");
		myHClade.put("777737737760371","T3");
		myHClade.put("777737737760571","T3");
		myHClade.put("777737737760600","T3");
		myHClade.put("777737737760771","T3");
		myHClade.put("777737740000071","Unknown");
		myHClade.put("777737740020771","H3");
		myHClade.put("777737740160771","T3");
		myHClade.put("777737742412771","EAI6-BGD1");
		myHClade.put("777737743760771","Cameroon");
		myHClade.put("777737747001771","Unknown");
		myHClade.put("777737747413701","EAI6-BGD1");
		myHClade.put("777737747413771","EAI6-BGD1");
		myHClade.put("777737747720771","H3");
		myHClade.put("777737747753771","Manu1");
		myHClade.put("777737747760411","T3");
		myHClade.put("777737747760760","T3");
		myHClade.put("777737747760771","T3");
		myHClade.put("777737754020771","H1");
		myHClade.put("777737754413771","EAI4-VNM");
		myHClade.put("777737755413271","EAI6-BGD1");
		myHClade.put("777737757410071","EAI6-BGD1");
		myHClade.put("777737757412371","EAI6-BGD1");
		myHClade.put("777737757413271","EAI6-BGD1");
		myHClade.put("777737757413371","EAI6-BGD1");
		myHClade.put("777737757413771","EAI6-BGD1");
		myHClade.put("777737757420771","Ural-1");
		myHClade.put("777737757700011","Unknown");
		myHClade.put("777737757720771","H3");
		myHClade.put("777737757760771","T");
		myHClade.put("777737760000151","Unknown");
		myHClade.put("777737760060771","T3");
		myHClade.put("777737760160731","T");
		myHClade.put("777737760177731","Unknown");
		myHClade.put("777737761760771","T");
		myHClade.put("777737764020771","H1");
		myHClade.put("777737767413631","EAI5");
		myHClade.put("777737767760711","T3");
		myHClade.put("777737767760771","T3");
		myHClade.put("777737770000000","Unknown");
		myHClade.put("777737770000071","Unknown");
		myHClade.put("777737770000371","Unknown");
		myHClade.put("777737770000771","Unknown");
		myHClade.put("777737770020731","H3");
		myHClade.put("777737770020771","H3");
		myHClade.put("777737770060771","T3");
		myHClade.put("777737770360771","T3");
		myHClade.put("777737770760771","T3");
		myHClade.put("777737770770041","Unknown");
		myHClade.put("777737774020731","H1");
		myHClade.put("777737774020771","H1");
		myHClade.put("777737774060671","T3");
		myHClade.put("777737774403431","Unknown");
		myHClade.put("777737774413731","EAI4-VNM");
		myHClade.put("777737774413771","EAI4-VNM");
		myHClade.put("777737774760731","T");
		myHClade.put("777737774760771","T3");
		myHClade.put("777737775413731","EAI1-SOM");
		myHClade.put("777737775420771","Ural-1");
		myHClade.put("777737776000131","Unknown");
		myHClade.put("777737776360771","T3");
		myHClade.put("777737776413731","EAI1-SOM");
		myHClade.put("777737776413771","EAI5");
		myHClade.put("777737776613771","Manu1");
		myHClade.put("777737776720771","H3");
		myHClade.put("777737776760731","T");
		myHClade.put("777737776760771","T3");
		myHClade.put("777737777000371","Unknown");
		myHClade.put("777737777000771","Unknown");
		myHClade.put("777737777020771","H3");
		myHClade.put("777737777320771","H3");
		myHClade.put("777737777410171","EAI5");
		myHClade.put("777737777410731","EAI1-SOM");
		myHClade.put("777737777413671","EAI5");
		myHClade.put("777737777413700","EAI5");
		myHClade.put("777737777413701","EAI5");
		myHClade.put("777737777413731","EAI1-SOM");
		myHClade.put("777737777413771","EAI5");
		myHClade.put("777737777420371","Ural-1");
		myHClade.put("777737777420600","Ural-1");
		myHClade.put("777737777420711","Ural-1");
		myHClade.put("777737777420731","Ural-1");
		myHClade.put("777737777420770","Ural-1");
		myHClade.put("777737777420771","Ural-1");
		myHClade.put("777737777423771","Manu2");
		myHClade.put("777737777460731","T");
		myHClade.put("777737777460771","T3");
		myHClade.put("777737777560721","T");
		myHClade.put("777737777560771","T3");
		myHClade.put("777737777600011","Unknown");
		myHClade.put("777737777600171","Unknown");
		myHClade.put("777737777620000","H3");
		myHClade.put("777737777620731","H3");
		myHClade.put("777737777620771","H3");
		myHClade.put("777737777660700","T3");
		myHClade.put("777737777660731","T");
		myHClade.put("777737777700000","Unknown");
		myHClade.put("777737777700011","Unknown");
		myHClade.put("777737777700171","Unknown");
		myHClade.put("777737777700771","Unknown");
		myHClade.put("777737777720171","H3");
		myHClade.put("777737777720671","H3");
		myHClade.put("777737777720731","H3");
		myHClade.put("777737777720771","H3");
		myHClade.put("777737777721771","Unknown");
		myHClade.put("777737777740131","T");
		myHClade.put("777737777740171","T3");
		myHClade.put("777737777740331","T");
		myHClade.put("777737777740371","T3");
		myHClade.put("777737777740731","T");
		myHClade.put("777737777740770","T3");
		myHClade.put("777737777740771","T3");
		myHClade.put("777737777754771","Unknown");
		myHClade.put("777737777760000","T3");
		myHClade.put("777737777760031","T3");
		myHClade.put("777737777760071","T3");
		myHClade.put("777737777760131","T");
		myHClade.put("777737777760371","T3");
		myHClade.put("777737777760431","T3");
		myHClade.put("777737777760471","T3");
		myHClade.put("777737777760500","T3");
		myHClade.put("777737777760571","T2-Uganda");
		myHClade.put("777737777760601","T3");
		myHClade.put("777737777760671","T3");
		myHClade.put("777737777760700","T3");
		myHClade.put("777737777760711","T3");
		myHClade.put("777737777760730","T2-Uganda");
		myHClade.put("777737777760731","T");
		myHClade.put("777737777760740","T3");
		myHClade.put("777737777760760","T3");
		myHClade.put("777737777760761","T3");
		myHClade.put("777737777760770","T3");
		myHClade.put("777737777760771","T3");
		myHClade.put("777737777761711","Unknown");
		myHClade.put("777737777761771","Unknown");
		myHClade.put("777737777763771","Manu2");
		myHClade.put("777737777770351","Unknown");
		myHClade.put("777737777777731","Unknown");
		myHClade.put("777740000000000","Unknown");
		myHClade.put("777740000000011","Unknown");
		myHClade.put("777740000000071","Unknown");
		myHClade.put("777740000020771","H3");
		myHClade.put("777740000160771","T1");
		myHClade.put("777740000660771","T1");
		myHClade.put("777740000760531","T2");
		myHClade.put("777740000760771","T1");
		myHClade.put("777740001560771","T1");
		myHClade.put("777740001760771","T1");
		myHClade.put("777740003760731","T2");
		myHClade.put("777740003760771","LAM-RUS");
		myHClade.put("777740003763771","Manu2");
		myHClade.put("777740004020771","H1");
		myHClade.put("777740004720771","H3");
		myHClade.put("777740007760371","LAM9");
		myHClade.put("777740007760531","LAM4");
		myHClade.put("777740007760571","LAM9");
		myHClade.put("777740007760730","Unknown");
		myHClade.put("777740007760731","LAM4");
		myHClade.put("777740007760771","LAM9");
		myHClade.put("777740007763771","Manu2");
		myHClade.put("777740015760771","T1");
		myHClade.put("777740016360771","T1");
		myHClade.put("777740017000331","Unknown");
		myHClade.put("777740017160771","T1");
		myHClade.put("777740017413771","EAI5");
		myHClade.put("777740017760720","T2");
		myHClade.put("777740017760731","T2");
		myHClade.put("777740017760771","T-tuscany");
		myHClade.put("777740037760771","T1");
		myHClade.put("777740070020771","H3");
		myHClade.put("777740077560771","T1");
		myHClade.put("777740077760771","T1");
		myHClade.put("777740167760771","T1");
		myHClade.put("777740177620771","H3");
		myHClade.put("777740177760471","T1");
		myHClade.put("777740177760771","T1");
		myHClade.put("777740207760771","LAM9");
		myHClade.put("777740360060771","T1");
		myHClade.put("777740370000731","Unknown");
		myHClade.put("777740377760771","T1");
		myHClade.put("777740607760771","LAM9");
		myHClade.put("777740617760771","T1");
		myHClade.put("777740677760771","T1");
		myHClade.put("777740777760531","T2");
		myHClade.put("777740777760731","T2");
		myHClade.put("777740777760771","T1");
		myHClade.put("777741000000000","Unknown");
		myHClade.put("777741000020601","H3");
		myHClade.put("777741000060771","T1");
		myHClade.put("777741000160771","T1");
		myHClade.put("777741000360771","T1");
		myHClade.put("777741000760771","T1");
		myHClade.put("777741001760771","T1");
		myHClade.put("777741003760731","T2");
		myHClade.put("777741003760771","T1");
		myHClade.put("777741007720731","H3");
		myHClade.put("777741007760511","LAM4");
		myHClade.put("777741007760601","LAM4");
		myHClade.put("777741007760771","LAM9");
		myHClade.put("777741007763771","Manu2");
		myHClade.put("777741014020771","H1");
		myHClade.put("777741017760731","T2");
		myHClade.put("777741017760771","Unknown");
		myHClade.put("777741077760771","T1");
		myHClade.put("777741177720771","H3");
		myHClade.put("777741207760771","LAM9");
		myHClade.put("777741343700000","Unknown");
		myHClade.put("777741370000731","Unknown");
		myHClade.put("777741377760771","T4");
		myHClade.put("777741607730771","Manu3");
		myHClade.put("777741637760771","T1");
		myHClade.put("777741757413371","EAI6-BGD1");
		myHClade.put("777741764020771","H1");
		myHClade.put("777741767720771","H3");
		myHClade.put("777741777413731","EAI1-SOM");
		myHClade.put("777741777720671","H3");
		myHClade.put("777741777720771","H3");
		myHClade.put("777741777760771","T1");
		myHClade.put("777742011003671","Unknown");
		myHClade.put("777742777760601","X2");
		myHClade.put("777742777760671","X1");
		myHClade.put("777743300000000","Unknown");
		myHClade.put("777743377760771","T4");
		myHClade.put("777743557760771","T5-Madrid2");
		myHClade.put("777743677760771","T1");
		myHClade.put("777743757413371","EAI6-BGD1");
		myHClade.put("777743767760700","T1");
		myHClade.put("777743777320771","H3");
		myHClade.put("777743777420771","Ural-1");
		myHClade.put("777743777700771","Unknown");
		myHClade.put("777743777720431","H3");
		myHClade.put("777743777720771","H3");
		myHClade.put("777743777760731","T2");
		myHClade.put("777743777760771","T1");
		myHClade.put("777746777760601","X2");
		myHClade.put("777746777760771","X1");
		myHClade.put("777747237760671","T4");
		myHClade.put("777747337760771","T4");
		myHClade.put("777747376776771","Unknown");
		myHClade.put("777747404740771","Turkey");
		myHClade.put("777747434000371","Unknown");
		myHClade.put("777747606040671","LAM9");
		myHClade.put("777747606060771","LAM11-ZWE");
		myHClade.put("777747606560771","LAM6");
		myHClade.put("777747607560671","LAM6");
		myHClade.put("777747607740031","LAM");
		myHClade.put("777747607740331","LAM4");
		myHClade.put("777747607760771","LAM9");
		myHClade.put("777747637740771","T1");
		myHClade.put("777747637760771","T1");
		myHClade.put("777747637763771","Manu2");
		myHClade.put("777747663740771","T1");
		myHClade.put("777747666760771","T1");
		myHClade.put("777747717760771","T1");
		myHClade.put("777747743760771","Cameroon");
		myHClade.put("777747757413271","EAI6-BGD1");
		myHClade.put("777747757413771","EAI6-BGD1");
		myHClade.put("777747761700771","Unknown");
		myHClade.put("777747763762771","Manu2");
		myHClade.put("777747770000771","Unknown");
		myHClade.put("777747777413671","EAI5");
		myHClade.put("777747777413731","EAI1-SOM");
		myHClade.put("777747777720671","H3");
		myHClade.put("777747777720771","H3");
		myHClade.put("777747777740771","T1");
		myHClade.put("777747777760671","T1");
		myHClade.put("777747777760731","T2");
		myHClade.put("777747777760771","T1");
		myHClade.put("777747777777771","Unknown");
		myHClade.put("777753777742070","Unknown");
		myHClade.put("777754007763771","Manu2");
		myHClade.put("777754377777730","Unknown");
		myHClade.put("777755047413371","EAI6-BGD1");
		myHClade.put("777755747413671","EAI6-BGD1");
		myHClade.put("777756777760301","X1");
		myHClade.put("777756777760601","X2");
		myHClade.put("777756777760671","X1");
		myHClade.put("777756777760731","X1");
		myHClade.put("777756777760771","X1");
		myHClade.put("777757206760771","LAM8");
		myHClade.put("777757360000171","Unknown");
		myHClade.put("777757377000371","Unknown");
		myHClade.put("777757377607771","Unknown");
		myHClade.put("777757377760601","T4");
		myHClade.put("777757404760771","Turkey");
		myHClade.put("777757474413771","EAI4-VNM");
		myHClade.put("777757557760771","T5-Madrid2");
		myHClade.put("777757577772771","Manu1");
		myHClade.put("777757602760771","T1");
		myHClade.put("777757603760771","T1");
		myHClade.put("777757606060731","LAM11-ZWE");
		myHClade.put("777757606060771","LAM11-ZWE");
		myHClade.put("777757606760611","LAM8");
		myHClade.put("777757606760771","LAM8");
		myHClade.put("777757607060671","LAM9");
		myHClade.put("777757607560771","LAM6");
		myHClade.put("777757607760400","LAM4");
		myHClade.put("777757607760671","LAM9");
		myHClade.put("777757607760731","LAM4");
		myHClade.put("777757607760770","LAM9");
		myHClade.put("777757607760771","LAM9");
		myHClade.put("777757617760771","T1");
		myHClade.put("777757633740371","T1");
		myHClade.put("777757637760431","T");
		myHClade.put("777757662760771","T1");
		myHClade.put("777757663760731","T2");
		myHClade.put("777757677420771","Ural-1");
		myHClade.put("777757677760770","T1");
		myHClade.put("777757677763771","Manu2");
		myHClade.put("777757700003371","Unknown");
		myHClade.put("777757747413771","EAI6-BGD1");
		myHClade.put("777757757413271","EAI6-BGD1");
		myHClade.put("777757757413371","EAI6-BGD1");
		myHClade.put("777757757413771","EAI6-BGD1");
		myHClade.put("777757760160771","T1");
		myHClade.put("777757767720771","H3");
		myHClade.put("777757774020771","H1");
		myHClade.put("777757776764661","Unknown");
		myHClade.put("777757777020771","H3");
		myHClade.put("777757777403771","Unknown");
		myHClade.put("777757777413731","EAI1-SOM");
		myHClade.put("777757777413771","EAI5");
		myHClade.put("777757777420611","Ural-1");
		myHClade.put("777757777420771","Ural-1");
		myHClade.put("777757777520771","H3");
		myHClade.put("777757777560731","T2");
		myHClade.put("777757777620771","H3");
		myHClade.put("777757777700000","Unknown");
		myHClade.put("777757777720671","H3");
		myHClade.put("777757777720771","H3");
		myHClade.put("777757777760471","T1");
		myHClade.put("777757777760531","T2");
		myHClade.put("777757777760671","T1");
		myHClade.put("777757777760700","T1");
		myHClade.put("777757777760731","T2");
		myHClade.put("777757777760771","T1");
		myHClade.put("777760000000000","Unknown");
		myHClade.put("777760000000731","Unknown");
		myHClade.put("777760000000771","Unknown");
		myHClade.put("777760000020611","H3");
		myHClade.put("777760001560771","LAM-RUS");
		myHClade.put("777760001760771","LAM-RUS");
		myHClade.put("777760003760771","LAM-RUS");
		myHClade.put("777760003763771","Manu2");
		myHClade.put("777760004020731","H1");
		myHClade.put("777760005760771","LAM-RUS");
		myHClade.put("777760007413731","EAI1-SOM");
		myHClade.put("777760007560571","LAM-RUS");
		myHClade.put("777760007560771","LAM-RUS");
		myHClade.put("777760007600771","Unknown");
		myHClade.put("777760007760051","LAM-RUS");
		myHClade.put("777760007760571","LAM-RUS");
		myHClade.put("777760007760600","LAM-RUS");
		myHClade.put("777760007760671","LAM-RUS");
		myHClade.put("777760007760731","LAM-RUS");
		myHClade.put("777760007760751","LAM-RUS");
		myHClade.put("777760007760771","LAM-RUS");
		myHClade.put("777760007763771","Manu2");
		myHClade.put("777760014020771","H1");
		myHClade.put("777760016760601","T-tuscany");
		myHClade.put("777760017760601","T-tuscany");
		myHClade.put("777760017760731","T-tuscany");
		myHClade.put("777760017760771","T-tuscany");
		myHClade.put("777760037560771","T1");
		myHClade.put("777760037720771","H3");
		myHClade.put("777760037760771","T1");
		myHClade.put("777760040020771","H3");
		myHClade.put("777760044000771","Unknown");
		myHClade.put("777760047500771","Unknown");
		myHClade.put("777760047760071","T1");
		myHClade.put("777760047760371","T1");
		myHClade.put("777760047760771","T1");
		myHClade.put("777760053760771","T5");
		myHClade.put("777760057740771","T5");
		myHClade.put("777760057760771","T5");
		myHClade.put("777760063560771","T1");
		myHClade.put("777760063760771","T1");
		myHClade.put("777760070000771","Unknown");
		myHClade.put("777760077620771","H3");
		myHClade.put("777760077760740","T1");
		myHClade.put("777760077760771","T1");
		myHClade.put("777760147760771","T1");
		myHClade.put("777760177760771","T1");
		myHClade.put("777760343740771","Cameroon");
		myHClade.put("777760370000000","Unknown");
		myHClade.put("777760377720771","H3");
		myHClade.put("777760377760771","T1");
		myHClade.put("777760577760771","T1");
		myHClade.put("777760607760700","LAM4");
		myHClade.put("777760770020771","H3");
		myHClade.put("777761000000000","Unknown");
		myHClade.put("777761000000031","Unknown");
		myHClade.put("777761000000171","Unknown");
		myHClade.put("777761000001771","Unknown");
		myHClade.put("777761003760771","T1");
		myHClade.put("777761004020731","H1");
		myHClade.put("777761004020771","H1");
		myHClade.put("777761007560771","LAM6");
		myHClade.put("777761007600771","Unknown");
		myHClade.put("777761007720771","H3");
		myHClade.put("777761007760651","LAM9");
		myHClade.put("777761007760731","LAM4");
		myHClade.put("777761007760771","LAM9");
		myHClade.put("777761007763771","Manu2");
		myHClade.put("777761017760771","T1");
		myHClade.put("777761047760371","T1");
		myHClade.put("777761047760771","T1");
		myHClade.put("777761050000771","Unknown");
		myHClade.put("777761057760771","T5");
		myHClade.put("777761057770771","Manu3");
		myHClade.put("777761077620771","H3");
		myHClade.put("777761077760770","T1");
		myHClade.put("777761374020771","H1");
		myHClade.put("777761377760731","T");
		myHClade.put("777761377760771","T4");
		myHClade.put("777761607473771","Manu1");
		myHClade.put("777761607760771","LAM9");
		myHClade.put("777761675720771","H3");
		myHClade.put("777761707760771","T1");
		myHClade.put("777761757760771","T5");
		myHClade.put("777761777413700","EAI5");
		myHClade.put("777761777413731","EAI1-SOM");
		myHClade.put("777761777720771","H3");
		myHClade.put("777761777760601","T1");
		myHClade.put("777761777760771","T1");
		myHClade.put("777761777763771","Manu2");
		myHClade.put("777762047700771","Unknown");
		myHClade.put("777762047760771","T1");
		myHClade.put("777762777760601","X2");
		myHClade.put("777763004020771","H1");
		myHClade.put("777763004120771","H3");
		myHClade.put("777763007760771","LAM9");
		myHClade.put("777763007763771","Manu2");
		myHClade.put("777763017760771","T1");
		myHClade.put("777763047700771","Unknown");
		myHClade.put("777763057760771","T5");
		myHClade.put("777763362413731","EAI1-SOM");
		myHClade.put("777763377760771","T4");
		myHClade.put("777763417760771","T1");
		myHClade.put("777763477400001","Unknown");
		myHClade.put("777763557760771","T5-Madrid2");
		myHClade.put("777763607560771","LAM6");
		myHClade.put("777763607760771","LAM9");
		myHClade.put("777763670020771","H3");
		myHClade.put("777763740020771","H3");
		myHClade.put("777763770000000","Unknown");
		myHClade.put("777763770020731","H3");
		myHClade.put("777763770020771","H3");
		myHClade.put("777763774020771","H1");
		myHClade.put("777763777413500","EAI5");
		myHClade.put("777763777413731","EAI1-SOM");
		myHClade.put("777763777413771","EAI5");
		myHClade.put("777763777700000","Unknown");
		myHClade.put("777763777720671","H3");
		myHClade.put("777763777720771","H3");
		myHClade.put("777763777760771","T1");
		myHClade.put("777763777777600","Unknown");
		myHClade.put("777763777777771","Unknown");
		myHClade.put("777764037560771","T1");
		myHClade.put("777764040020771","H3");
		myHClade.put("777764077760771","T1");
		myHClade.put("777764606060731","LAM11-ZWE");
		myHClade.put("777764607760771","LAM9");
		myHClade.put("777764777760771","T1");
		myHClade.put("777765606060631","LAM11-ZWE");
		myHClade.put("777765606060731","LAM11-ZWE");
		myHClade.put("777765757413371","EAI6-BGD1");
		myHClade.put("777765777760771","T1");
		myHClade.put("777766377760601","T1");
		myHClade.put("777766407760601","X2");
		myHClade.put("777766607760771","LAM9");
		myHClade.put("777766637760771","X1");
		myHClade.put("777766737760571","X1");
		myHClade.put("777766770000000","Unknown");
		myHClade.put("777766777060771","X1");
		myHClade.put("777766777760071","X1");
		myHClade.put("777766777760600","X2");
		myHClade.put("777766777760601","X2");
		myHClade.put("777766777760631","X1");
		myHClade.put("777766777760731","X1");
		myHClade.put("777766777760771","X1");
		myHClade.put("777767004020771","H1");
		myHClade.put("777767037720771","H3");
		myHClade.put("777767207760771","LAM9");
		myHClade.put("777767347760471","T4-CEU1");
		myHClade.put("777767374020771","H1");
		myHClade.put("777767377760771","T4");
		myHClade.put("777767404760771","Turkey");
		myHClade.put("777767407560731","LAM6");
		myHClade.put("777767407760770","LAM9");
		myHClade.put("777767417760771","T1");
		myHClade.put("777767437740761","T1");
		myHClade.put("777767477342671","Unknown");
		myHClade.put("777767477403671","Unknown");
		myHClade.put("777767477413671","EAI5");
		myHClade.put("777767477720771","H3");
		myHClade.put("777767477740670","T1");
		myHClade.put("777767477760771","T-H37Rv");
		myHClade.put("777767477763771","Manu2");
		myHClade.put("777767477770771","Manu3");
		myHClade.put("777767504760771","T1");
		myHClade.put("777767557760671","T5-Madrid2");
		myHClade.put("777767570020731","H3");
		myHClade.put("777767577660671","T1");
		myHClade.put("777767600000000","Unknown");
		myHClade.put("777767600000011","Unknown");
		myHClade.put("777767600000071","Unknown");
		myHClade.put("777767600000731","Unknown");
		myHClade.put("777767600360771","T1");
		myHClade.put("777767600760771","T1");
		myHClade.put("777767601760771","T1");
		myHClade.put("777767604020771","H1");
		myHClade.put("777767604320771","H3");
		myHClade.put("777767606060631","LAM11-ZWE");
		myHClade.put("777767606060661","LAM11-ZWE");
		myHClade.put("777767606060671","LAM11-ZWE");
		myHClade.put("777767606060731","LAM11-ZWE");
		myHClade.put("777767606060771","LAM11-ZWE");
		myHClade.put("777767607540131","LAM4");
		myHClade.put("777767607560671","LAM6");
		myHClade.put("777767607560731","LAM6");
		myHClade.put("777767607560771","LAM6");
		myHClade.put("777767607570771","Manu3");
		myHClade.put("777767607700771","Unknown");
		myHClade.put("777767607760671","LAM9");
		myHClade.put("777767607760700","LAM4");
		myHClade.put("777767607760730","LAM4");
		myHClade.put("777767607760731","LAM4");
		myHClade.put("777767607760760","LAM9");
		myHClade.put("777767607760771","LAM9");
		myHClade.put("777767607763771","Manu2");
		myHClade.put("777767614760771","T1");
		myHClade.put("777767617760771","T1");
		myHClade.put("777767636760771","T1");
		myHClade.put("777767637741771","Unknown");
		myHClade.put("777767637760321","T2");
		myHClade.put("777767637760371","T1");
		myHClade.put("777767637760771","T1");
		myHClade.put("777767643760771","Cameroon");
		myHClade.put("777767674020731","H1");
		myHClade.put("777767675760771","T1");
		myHClade.put("777767677403631","Unknown");
		myHClade.put("777767677720771","H3");
		myHClade.put("777767677760631","T1");
		myHClade.put("777767677760671","T1");
		myHClade.put("777767677760731","T2");
		myHClade.put("777767677760771","T1");
		myHClade.put("777767677763771","Manu2");
		myHClade.put("777767677773671","Manu1");
		myHClade.put("777767702003371","Unknown");
		myHClade.put("777767706760711","T1");
		myHClade.put("777767707760771","T1");
		myHClade.put("777767714020771","H1");
		myHClade.put("777767717413731","EAI1-SOM");
		myHClade.put("777767717720771","H3");
		myHClade.put("777767717760711","T1");
		myHClade.put("777767717760771","T1");
		myHClade.put("777767737760621","T1");
		myHClade.put("777767737760771","T1");
		myHClade.put("777767740000371","Unknown");
		myHClade.put("777767741760771","T");
		myHClade.put("777767743413331","EAI");
		myHClade.put("777767743460771","Cameroon");
		myHClade.put("777767743760771","Cameroon");
		myHClade.put("777767743763771","Manu2");
		myHClade.put("777767747720771","H3");
		myHClade.put("777767747743540","Unknown");
		myHClade.put("777767747760471","T1");
		myHClade.put("777767750003271","EAI7-BGD2");
		myHClade.put("777767750060771","T5");
		myHClade.put("777767754020731","H1");
		myHClade.put("777767757760671","T5");
		myHClade.put("777767757760771","T5");
		myHClade.put("777767760000000","Unknown");
		myHClade.put("777767760000371","Unknown");
		myHClade.put("777767760060771","T1");
		myHClade.put("777767760360371","T1");
		myHClade.put("777767764020771","H1");
		myHClade.put("777767766020761","H3");
		myHClade.put("777767767760661","T1");
		myHClade.put("777767770000000","Unknown");
		myHClade.put("777767770000771","Unknown");
		myHClade.put("777767770020720","H3");
		myHClade.put("777767770020731","H3");
		myHClade.put("777767770720671","H3");
		myHClade.put("777767774000671","Unknown");
		myHClade.put("777767774020571","H1");
		myHClade.put("777767774020631","H1");
		myHClade.put("777767774020671","H1");
		myHClade.put("777767774020731","H1");
		myHClade.put("777767774020771","H1");
		myHClade.put("777767774060771","T1");
		myHClade.put("777767774413771","EAI4-VNM");
		myHClade.put("777767774720731","H3");
		myHClade.put("777767774760771","T1");
		myHClade.put("777767775720771","H3");
		myHClade.put("777767775760771","T1");
		myHClade.put("777767776000271","Unknown");
		myHClade.put("777767776020771","H3");
		myHClade.put("777767776160601","T1");
		myHClade.put("777767776413771","EAI5");
		myHClade.put("777767776760731","T2");
		myHClade.put("777767776760771","T1");
		myHClade.put("777767777000271","Unknown");
		myHClade.put("777767777320771","H3");
		myHClade.put("777767777360731","T2");
		myHClade.put("777767777400771","Unknown");
		myHClade.put("777767777403671","Unknown");
		myHClade.put("777767777413331","EAI1-SOM");
		myHClade.put("777767777413600","EAI5");
		myHClade.put("777767777413631","EAI5");
		myHClade.put("777767777413731","EAI1-SOM");
		myHClade.put("777767777413771","EAI5");
		myHClade.put("777767777420731","Ural-1");
		myHClade.put("777767777460611","T1");
		myHClade.put("777767777460731","T2");
		myHClade.put("777767777460771","T1");
		myHClade.put("777767777520771","H3");
		myHClade.put("777767777553771","Manu1");
		myHClade.put("777767777560731","T2");
		myHClade.put("777767777560771","T1");
		myHClade.put("777767777600771","Unknown");
		myHClade.put("777767777620731","H3");
		myHClade.put("777767777620771","H3");
		myHClade.put("777767777720631","H3");
		myHClade.put("777767777720671","H3");
		myHClade.put("777767777720731","H3");
		myHClade.put("777767777720751","H3");
		myHClade.put("777767777720771","H3");
		myHClade.put("777767777722771","Manu2");
		myHClade.put("777767777740051","T1");
		myHClade.put("777767777740671","T1");
		myHClade.put("777767777740761","T1");
		myHClade.put("777767777740771","T1");
		myHClade.put("777767777760171","T1");
		myHClade.put("777767777760410","T1");
		myHClade.put("777767777760431","T1");
		myHClade.put("777767777760471","T1");
		myHClade.put("777767777760600","T");
		myHClade.put("777767777760601","T2");
		myHClade.put("777767777760611","T1");
		myHClade.put("777767777760671","T1");
		myHClade.put("777767777760700","T1");
		myHClade.put("777767777760701","T1");
		myHClade.put("777767777760711","T1");
		myHClade.put("777767777760721","T2");
		myHClade.put("777767777760731","T2");
		myHClade.put("777767777760771","T1");
		myHClade.put("777767777762771","Manu2");
		myHClade.put("777767777763771","Manu2");
		myHClade.put("777767777773771","Manu1");
		myHClade.put("777767777777771","Unknown");
		myHClade.put("777770000000000","Unknown");
		myHClade.put("777770000000371","Unknown");
		myHClade.put("777770000000731","Unknown");
		myHClade.put("777770000020071","H3");
		myHClade.put("777770004000731","Unknown");
		myHClade.put("777770007600771","Unknown");
		myHClade.put("777770007760771","LAM9");
		myHClade.put("777770014020771","H1");
		myHClade.put("777770017760731","T2");
		myHClade.put("777770017760771","T1");
		myHClade.put("777770037413731","EAI1-SOM");
		myHClade.put("777770077700771","Unknown");
		myHClade.put("777770077760601","T1");
		myHClade.put("777770177413700","EAI5");
		myHClade.put("777770343760771","Cameroon");
		myHClade.put("777770367760771","T1");
		myHClade.put("777770374020771","H1");
		myHClade.put("777770377760771","T1");
		myHClade.put("777770606360771","LAM9");
		myHClade.put("777770607760771","LAM9");
		myHClade.put("777770677760771","T1");
		myHClade.put("777770770020771","H3");
		myHClade.put("777770777760771","T1");
		myHClade.put("777771000000000","Unknown");
		myHClade.put("777771000000071","Unknown");
		myHClade.put("777771000000371","Manu");
		myHClade.put("777771000000731","Unknown");
		myHClade.put("777771000000771","Unknown");
		myHClade.put("777771000020771","H3");
		myHClade.put("777771004720771","H3");
		myHClade.put("777771006020731","H3");
		myHClade.put("777771007760731","LAM4");
		myHClade.put("777771007760771","LAM9");
		myHClade.put("777771070000771","Unknown");
		myHClade.put("777771340000071","Unknown");
		myHClade.put("777771343760771","Cameroon");
		myHClade.put("777771350060771","T");
		myHClade.put("777771374020771","H1");
		myHClade.put("777771377760771","T4");
		myHClade.put("777771477413671","EAI5");
		myHClade.put("777771607760771","LAM9");
		myHClade.put("777771677413671","EAI5");
		myHClade.put("777771751413470","EAI6-BGD1");
		myHClade.put("777771775413671","EAI5");
		myHClade.put("777771777413671","EAI5");
		myHClade.put("777771777720631","H3");
		myHClade.put("777771777720771","H3");
		myHClade.put("777771777760771","T1");
		myHClade.put("777772777760601","X2");
		myHClade.put("777773000000071","Unknown");
		myHClade.put("777773000000331","Unknown");
		myHClade.put("777773000000731","Unknown");
		myHClade.put("777773002000011","Unknown");
		myHClade.put("777773037760771","T1");
		myHClade.put("777773374000000","Unknown");
		myHClade.put("777773377720771","H3");
		myHClade.put("777773377760771","T4");
		myHClade.put("777773604060731","LAM11-ZWE");
		myHClade.put("777773606060731","LAM11-ZWE");
		myHClade.put("777773607760731","LAM4");
		myHClade.put("777773607760771","LAM9");
		myHClade.put("777773637560731","T2");
		myHClade.put("777773676760730","T2-Uganda");
		myHClade.put("777773677777600","Unknown");
		myHClade.put("777773700020731","H3");
		myHClade.put("777773740000031","Unknown");
		myHClade.put("777773740000071","Unknown");
		myHClade.put("777773743760771","Cameroon");
		myHClade.put("777773750000000","Unknown");
		myHClade.put("777773760000171","Unknown");
		myHClade.put("777773770000000","Unknown");
		myHClade.put("777773770020771","H3");
		myHClade.put("777773773742070","Unknown");
		myHClade.put("777773773760771","T1");
		myHClade.put("777773774020771","H1");
		myHClade.put("777773774060371","T1");
		myHClade.put("777773776000371","Unknown");
		myHClade.put("777773776760771","T1");
		myHClade.put("777773777410331","EAI1-SOM");
		myHClade.put("777773777413700","EAI");
		myHClade.put("777773777513761","Manu1");
		myHClade.put("777773777720731","H3");
		myHClade.put("777773777720771","H3");
		myHClade.put("777773777760171","T1");
		myHClade.put("777773777760371","T1");
		myHClade.put("777773777760571","T1");
		myHClade.put("777773777760731","T2");
		myHClade.put("777773777760771","T1");
		myHClade.put("777774000000171","Unknown");
		myHClade.put("777774000000371","Unknown");
		myHClade.put("777774000000771","Unknown");
		myHClade.put("777774000060771","T1");
		myHClade.put("777774003660771","T1");
		myHClade.put("777774003760731","T2");
		myHClade.put("777774004020771","H1");
		myHClade.put("777774005777731","Unknown");
		myHClade.put("777774007413771","EAI5");
		myHClade.put("777774007720771","H3");
		myHClade.put("777774007760731","LAM4");
		myHClade.put("777774037760771","T1");
		myHClade.put("777774047560771","T1");
		myHClade.put("777774057760771","T5");
		myHClade.put("777774063560771","T1");
		myHClade.put("777774070000171","Unknown");
		myHClade.put("777774077020771","H3");
		myHClade.put("777774077560711","T1");
		myHClade.put("777774077560731","T2");
		myHClade.put("777774077560751","T1");
		myHClade.put("777774077560771","Unknown");
		myHClade.put("777774077760731","T2");
		myHClade.put("777774077760771","T1");
		myHClade.put("777774177720771","H3");
		myHClade.put("777774177760771","T1");
		myHClade.put("777774206060771","LAM11-ZWE");
		myHClade.put("777774344120771","H3");
		myHClade.put("777774376000711","Unknown");
		myHClade.put("777774377403400","Unknown");
		myHClade.put("777774377413671","EAI5");
		myHClade.put("777774377413731","EAI1-SOM");
		myHClade.put("777774377720771","H3");
		myHClade.put("777774377760131","T2");
		myHClade.put("777774602760731","T2");
		myHClade.put("777774606060731","LAM11-ZWE");
		myHClade.put("777774607760731","LAM4");
		myHClade.put("777774607760771","LAM9");
		myHClade.put("777774674020771","H1");
		myHClade.put("777774777000171","Unknown");
		myHClade.put("777774777720771","H3");
		myHClade.put("777774777760601","T1");
		myHClade.put("777774777760751","T1");
		myHClade.put("777774777760771","T1");
		myHClade.put("777775000000000","Unknown");
		myHClade.put("777775000000171","Unknown");
		myHClade.put("777775000000371","Unknown");
		myHClade.put("777775003770351","Unknown");
		myHClade.put("777775007413671","EAI5");
		myHClade.put("777775007413771","EAI5");
		myHClade.put("777775007760731","LAM4");
		myHClade.put("777775037560771","T1");
		myHClade.put("777775070000171","Unknown");
		myHClade.put("777775074060771","T1");
		myHClade.put("777775077020771","H3");
		myHClade.put("777775077560700","T1");
		myHClade.put("777775077560771","T1");
		myHClade.put("777775177760571","T1");
		myHClade.put("777775177760731","T2");
		myHClade.put("777775237560771","T4");
		myHClade.put("777775343760771","Cameroon");
		myHClade.put("777775376000711","Unknown");
		myHClade.put("777775377660771","T4");
		myHClade.put("777775377720771","H3");
		myHClade.put("777775377760571","T4");
		myHClade.put("777775557760771","T5-Madrid2");
		myHClade.put("777775606000731","Unknown");
		myHClade.put("777775606060731","LAM11-ZWE");
		myHClade.put("777775606060771","LAM11-ZWE");
		myHClade.put("777775607560771","LAM6");
		myHClade.put("777775607700771","Unknown");
		myHClade.put("777775607760771","LAM9");
		myHClade.put("777775677560771","T1");
		myHClade.put("777775743760771","Cameroon");
		myHClade.put("777775745413771","EAI6-BGD1");
		myHClade.put("777775747413771","EAI6-BGD1");
		myHClade.put("777775747760471","T1");
		myHClade.put("777775750003771","Unknown");
		myHClade.put("777775757413371","EAI6-BGD1");
		myHClade.put("777775770020771","H3");
		myHClade.put("777775774020771","H1");
		myHClade.put("777775774413771","EAI4-VNM");
		myHClade.put("777775777577571","Unknown");
		myHClade.put("777775777720731","H3");
		myHClade.put("777775777720771","H3");
		myHClade.put("777775777760431","T1");
		myHClade.put("777775777760700","T1");
		myHClade.put("777775777760731","T2");
		myHClade.put("777775777760771","T1");
		myHClade.put("777776000000071","Unknown");
		myHClade.put("777776000000771","Unknown");
		myHClade.put("777776000120771","H3");
		myHClade.put("777776000160771","T1");
		myHClade.put("777776001413771","EAI5");
		myHClade.put("777776001720771","H3");
		myHClade.put("777776001760771","T1");
		myHClade.put("777776003760771","T1");
		myHClade.put("777776004760771","Turkey");
		myHClade.put("777776007560771","LAM6");
		myHClade.put("777776007760471","LAM9");
		myHClade.put("777776017760771","T1");
		myHClade.put("777776037760771","T1");
		myHClade.put("777776047560771","T1");
		myHClade.put("777776077020771","H3");
		myHClade.put("777776077413771","EAI5");
		myHClade.put("777776077570771","Manu3");
		myHClade.put("777776077760711","T1");
		myHClade.put("777776077760771","T1");
		myHClade.put("777776177760731","T2");
		myHClade.put("777776201560771","T1");
		myHClade.put("777776206060771","LAM11-ZWE");
		myHClade.put("777776207760771","LAM9");
		myHClade.put("777776374360711","T1");
		myHClade.put("777776376360731","T2");
		myHClade.put("777776377060771","T1");
		myHClade.put("777776377412731","EAI1-SOM");
		myHClade.put("777776377760601","T1");
		myHClade.put("777776377760760","T1");
		myHClade.put("777776377760771","T1");
		myHClade.put("777776404160601","X2");
		myHClade.put("777776404360601","X2");
		myHClade.put("777776407360601","X2");
		myHClade.put("777776407560771","LAM6");
		myHClade.put("777776407760601","X2");
		myHClade.put("777776407760611","LAM4");
		myHClade.put("777776417760771","X1");
		myHClade.put("777776437760771","X1");
		myHClade.put("777776477760601","X2");
		myHClade.put("777776477760771","T-H37Rv");
		myHClade.put("777776577760601","X2");
		myHClade.put("777776607560771","LAM6");
		myHClade.put("777776607760601","X2");
		myHClade.put("777776607760700","LAM4");
		myHClade.put("777776607760771","LAM9");
		myHClade.put("777776617560771","X1");
		myHClade.put("777776617760601","X2");
		myHClade.put("777776657413771","EAI6-BGD1");
		myHClade.put("777776675760601","X2");
		myHClade.put("777776677760601","X2");
		myHClade.put("777776677760711","X1");
		myHClade.put("777776677760771","X1");
		myHClade.put("777776700000000","Unknown");
		myHClade.put("777776707760771","X1");
		myHClade.put("777776727760771","X1");
		myHClade.put("777776737560771","X1");
		myHClade.put("777776737760571","X1");
		myHClade.put("777776747760601","X2");
		myHClade.put("777776756360711","X1");
		myHClade.put("777776757403771","Unknown");
		myHClade.put("777776757412771","EAI6-BGD1");
		myHClade.put("777776757413371","EAI6-BGD1");
		myHClade.put("777776757413671","EAI6-BGD1");
		myHClade.put("777776757413771","EAI6-BGD1");
		myHClade.put("777776757760571","X1");
		myHClade.put("777776757760601","X2");
		myHClade.put("777776757760771","X1");
		myHClade.put("777776761763771","Manu2");
		myHClade.put("777776767760571","X1");
		myHClade.put("777776770000000","Unknown");
		myHClade.put("777776771763771","Manu2");
		myHClade.put("777776774000000","Unknown");
		myHClade.put("777776774020601","H1");
		myHClade.put("777776774020771","H1");
		myHClade.put("777776774720171","H3");
		myHClade.put("777776774760731","X1");
		myHClade.put("777776775413470","EAI5");
		myHClade.put("777776775413700","EAI5");
		myHClade.put("777776775720771","H3");
		myHClade.put("777776775760571","X1");
		myHClade.put("777776775760600","X2");
		myHClade.put("777776775760700","X1");
		myHClade.put("777776775760731","X1");
		myHClade.put("777776775760771","X1");
		myHClade.put("777776776000601","Unknown");
		myHClade.put("777776776160601","X2");
		myHClade.put("777776776360711","X1");
		myHClade.put("777776776360771","X1");
		myHClade.put("777776776760601","X2");
		myHClade.put("777776777060771","X1");
		myHClade.put("777776777160771","X1");
		myHClade.put("777776777360771","X1");
		myHClade.put("777776777400731","Unknown");
		myHClade.put("777776777411731","EAI1-SOM");
		myHClade.put("777776777412131","EAI1-SOM");
		myHClade.put("777776777412731","EAI1-SOM");
		myHClade.put("777776777413101","EAI5");
		myHClade.put("777776777413131","EAI1-SOM");
		myHClade.put("777776777413731","EAI1-SOM");
		myHClade.put("777776777413771","EAI5");
		myHClade.put("777776777420771","Ural-1");
		myHClade.put("777776777460771","X1");
		myHClade.put("777776777560601","X2");
		myHClade.put("777776777560731","X1");
		myHClade.put("777776777560771","X1");
		myHClade.put("777776777600000","Unknown");
		myHClade.put("777776777600001","Unknown");
		myHClade.put("777776777600171","Unknown");
		myHClade.put("777776777620601","H3");
		myHClade.put("777776777650771","Manu3");
		myHClade.put("777776777700731","Unknown");
		myHClade.put("777776777700771","Unknown");
		myHClade.put("777776777720631","H3");
		myHClade.put("777776777720671","H3");
		myHClade.put("777776777720771","H3");
		myHClade.put("777776777740000","X2");
		myHClade.put("777776777740001","X2");
		myHClade.put("777776777740601","X2");
		myHClade.put("777776777740771","X1");
		myHClade.put("777776777760071","X1");
		myHClade.put("777776777760171","X1");
		myHClade.put("777776777760351","X1");
		myHClade.put("777776777760401","X2");
		myHClade.put("777776777760571","X1");
		myHClade.put("777776777760600","X2");
		myHClade.put("777776777760601","X2");
		myHClade.put("777776777760611","X1");
		myHClade.put("777776777760631","X1");
		myHClade.put("777776777760641","X1");
		myHClade.put("777776777760661","X1");
		myHClade.put("777776777760671","X1");
		myHClade.put("777776777760700","X1");
		myHClade.put("777776777760701","X1");
		myHClade.put("777776777760711","X1");
		myHClade.put("777776777760731","X1");
		myHClade.put("777776777760740","X1");
		myHClade.put("777776777760770","X1");
		myHClade.put("777776777760771","X1");
		myHClade.put("777776777761771","Unknown");
		myHClade.put("777776777763771","Manu2");
		myHClade.put("777776777767711","Unknown");
		myHClade.put("777776777770351","Unknown");
		myHClade.put("777776777770771","Manu3");
		myHClade.put("777776777777771","Unknown");
		myHClade.put("777777000000001","Unknown");
		myHClade.put("777777000000011","Zero-copy");
		myHClade.put("777777000000071","Unknown");
		myHClade.put("777777000000371","Unknown");
		myHClade.put("777777000000771","Unknown");
		myHClade.put("777777000020731","H3");
		myHClade.put("777777000020771","H3");
		myHClade.put("777777000160771","T1");
		myHClade.put("777777000320771","H3");
		myHClade.put("777777000360771","T1");
		myHClade.put("777777000760411","T1");
		myHClade.put("777777000760731","T2");
		myHClade.put("777777000760771","T1");
		myHClade.put("777777001413771","EAI5");
		myHClade.put("777777001660771","T1");
		myHClade.put("777777001760771","T1");
		myHClade.put("777777002000011","Unknown");
		myHClade.put("777777003013311","EAI5");
		myHClade.put("777777003413771","EAI5");
		myHClade.put("777777003473771","Manu1");
		myHClade.put("777777003760731","T2");
		myHClade.put("777777003760771","T1");
		myHClade.put("777777004020771","H1");
		myHClade.put("777777004760771","Turkey");
		myHClade.put("777777005720771","H3");
		myHClade.put("777777005760471","LAM9");
		myHClade.put("777777006720771","H3");
		myHClade.put("777777007113771","Manu1");
		myHClade.put("777777007560771","LAM6");
		myHClade.put("777777007600331","Unknown");
		myHClade.put("777777007660751","LAM9");
		myHClade.put("777777007660771","LAM9");
		myHClade.put("777777007720731","H3");
		myHClade.put("777777007720771","H3");
		myHClade.put("777777007760631","LAM4");
		myHClade.put("777777007760671","LAM9");
		myHClade.put("777777007760731","LAM4");
		myHClade.put("777777010003371","EAI7-BGD2");
		myHClade.put("777777014020771","H1");
		myHClade.put("777777017720771","H3");
		myHClade.put("777777017760731","T2");
		myHClade.put("777777017760771","T");
		myHClade.put("777777020020671","H3");
		myHClade.put("777777034020771","H1");
		myHClade.put("777777037700771","Unknown");
		myHClade.put("777777037720471","H3");
		myHClade.put("777777037720671","H3");
		myHClade.put("777777037720771","H3");
		myHClade.put("777777037723771","Manu2");
		myHClade.put("777777037740071","T1");
		myHClade.put("777777037760031","T1");
		myHClade.put("777777037760731","T2");
		myHClade.put("777777037760771","Unknown");
		myHClade.put("777777047413371","EAI6-BGD1");
		myHClade.put("777777071760771","T1");
		myHClade.put("777777076760771","T1");
		myHClade.put("777777077400001","Unknown");
		myHClade.put("777777077560771","T1");
		myHClade.put("777777077660771","T1");
		myHClade.put("777777077720771","H3");
		myHClade.put("777777077760771","T1");
		myHClade.put("777777137760771","T1");
		myHClade.put("777777143760771","Cameroon");
		myHClade.put("777777147720771","H3");
		myHClade.put("777777174020731","H1");
		myHClade.put("777777174020771","H1");
		myHClade.put("777777177413371","EAI5");
		myHClade.put("777777177720731","H3");
		myHClade.put("777777177720771","H3");
		myHClade.put("777777177760571","T1");
		myHClade.put("777777177760731","T2");
		myHClade.put("777777177760771","T1");
		myHClade.put("777777204020771","H1");
		myHClade.put("777777204060731","LAM4");
		myHClade.put("777777204760771","LAM9");
		myHClade.put("777777205760771","LAM9");
		myHClade.put("777777206060731","LAM11-ZWE");
		myHClade.put("777777206060771","LAM11-ZWE");
		myHClade.put("777777207400171","Unknown");
		myHClade.put("777777207560771","LAM6");
		myHClade.put("777777207600331","Unknown");
		myHClade.put("777777207600371","Unknown");
		myHClade.put("777777207760631","LAM4");
		myHClade.put("777777207760711","LAM4");
		myHClade.put("777777207760751","LAM9");
		myHClade.put("777777207760771","LAM9");
		myHClade.put("777777263720771","H3");
		myHClade.put("777777263760771","T4");
		myHClade.put("777777274020771","H1");
		myHClade.put("777777277413771","EAI5");
		myHClade.put("777777300000171","Unknown");
		myHClade.put("777777307760771","T4");
		myHClade.put("777777317760571","T4");
		myHClade.put("777777337760771","T4");
		myHClade.put("777777340020771","H3");
		myHClade.put("777777341413331","EAI");
		myHClade.put("777777343760771","Cameroon");
		myHClade.put("777777343770771","Manu3");
		myHClade.put("777777345760471","T4-CEU1");
		myHClade.put("777777347160471","T4-CEU1");
		myHClade.put("777777347400071","Unknown");
		myHClade.put("777777347560471","T4-CEU1");
		myHClade.put("777777347760400","T4");
		myHClade.put("777777347760431","T4");
		myHClade.put("777777347760471","T4-CEU1");
		myHClade.put("777777347760771","T4");
		myHClade.put("777777347763771","Manu2");
		myHClade.put("777777350000171","Unknown");
		myHClade.put("777777350000371","Unknown");
		myHClade.put("777777350010371","EAI5");
		myHClade.put("777777350060771","T");
		myHClade.put("777777354020771","H1");
		myHClade.put("777777357720771","H3");
		myHClade.put("777777357760700","T");
		myHClade.put("777777357760771","T");
		myHClade.put("777777360060771","T4");
		myHClade.put("777777363760771","T4");
		myHClade.put("777777364020771","H1");
		myHClade.put("777777370020751","H3");
		myHClade.put("777777370020771","H3");
		myHClade.put("777777373761631","Unknown");
		myHClade.put("777777374020731","H1");
		myHClade.put("777777374020771","H1");
		myHClade.put("777777376413771","EAI5");
		myHClade.put("777777376760771","T4");
		myHClade.put("777777377000371","Unknown");
		myHClade.put("777777377000771","Unknown");
		myHClade.put("777777377013771","EAI5");
		myHClade.put("777777377120771","H3");
		myHClade.put("777777377360771","T4");
		myHClade.put("777777377400171","Unknown");
		myHClade.put("777777377410071","EAI5");
		myHClade.put("777777377413331","EAI1-SOM");
		myHClade.put("777777377413511","EAI5");
		myHClade.put("777777377413711","EAI5");
		myHClade.put("777777377413771","EAI5");
		myHClade.put("777777377420771","Ural-1");
		myHClade.put("777777377520771","H3");
		myHClade.put("777777377660771","T4");
		myHClade.put("777777377700771","Unknown");
		myHClade.put("777777377720731","H3");
		myHClade.put("777777377720771","H3");
		myHClade.put("777777377740371","T4");
		myHClade.put("777777377760371","T4");
		myHClade.put("777777377760700","T4");
		myHClade.put("777777377760731","T");
		myHClade.put("777777377760760","T4");
		myHClade.put("777777377760770","T4");
		myHClade.put("777777377760771","T4");
		myHClade.put("777777377763771","Manu2");
		myHClade.put("777777377766771","Unknown");
		myHClade.put("777777400000000","Unknown");
		myHClade.put("777777400000171","Unknown");
		myHClade.put("777777400000371","Unknown");
		myHClade.put("777777400000731","Unknown");
		myHClade.put("777777400000771","Unknown");
		myHClade.put("777777400020471","H3");
		myHClade.put("777777400020731","H3");
		myHClade.put("777777400020771","H3");
		myHClade.put("777777400060771","Unknown");
		myHClade.put("777777400400071","Unknown");
		myHClade.put("777777400600000","Unknown");
		myHClade.put("777777400740371","T1");
		myHClade.put("777777400740731","T2");
		myHClade.put("777777400760731","T2");
		myHClade.put("777777400760771","T1");
		myHClade.put("777777401013371","EAI5");
		myHClade.put("777777401413371","EAI5");
		myHClade.put("777777402000371","Unknown");
		myHClade.put("777777402374371","Unknown");
		myHClade.put("777777403760411","T1");
		myHClade.put("777777403760730","T2-Uganda");
		myHClade.put("777777403760731","T2");
		myHClade.put("777777403760771","T1");
		myHClade.put("777777404020731","H1");
		myHClade.put("777777404020771","H1");
		myHClade.put("777777404120771","H3");
		myHClade.put("777777404320731","H3");
		myHClade.put("777777404620771","H3");
		myHClade.put("777777404720571","H3");
		myHClade.put("777777404760411","Turkey");
		myHClade.put("777777404760571","Turkey");
		myHClade.put("777777404760711","Turkey");
		myHClade.put("777777404760731","Turkey");
		myHClade.put("777777404760760","Turkey");
		myHClade.put("777777404760771","Turkey");
		myHClade.put("777777404770771","Manu3");
		myHClade.put("777777405720771","H3");
		myHClade.put("777777405760571","LAM9");
		myHClade.put("777777407060771","LAM9");
		myHClade.put("777777407420771","Ural-1");
		myHClade.put("777777407560730","LAM6");
		myHClade.put("777777407560731","LAM");
		myHClade.put("777777407560771","LAM6");
		myHClade.put("777777407700071","Unknown");
		myHClade.put("777777407720771","H3");
		myHClade.put("777777407760720","LAM4");
		myHClade.put("777777407760771","LAM9");
		myHClade.put("777777407773771","Manu1");
		myHClade.put("777777407777771","Unknown");
		myHClade.put("777777414020731","H1");
		myHClade.put("777777414141571","Unknown");
		myHClade.put("777777414147771","Unknown");
		myHClade.put("777777414220771","H3");
		myHClade.put("777777414760771","T1");
		myHClade.put("777777417300771","Unknown");
		myHClade.put("777777417720771","H3");
		myHClade.put("777777417740371","T1");
		myHClade.put("777777417760771","T1");
		myHClade.put("777777423560771","T1");
		myHClade.put("777777434020771","H1");
		myHClade.put("777777434760771","T1");
		myHClade.put("777777437500000","Unknown");
		myHClade.put("777777437720771","H3");
		myHClade.put("777777437760541","T1");
		myHClade.put("777777437760731","T2");
		myHClade.put("777777437760771","T1");
		myHClade.put("777777440003771","Unknown");
		myHClade.put("777777441413331","EAI");
		myHClade.put("777777457413771","EAI6-BGD1");
		myHClade.put("777777457760771","T-H37Rv");
		myHClade.put("777777467400000","Unknown");
		myHClade.put("777777467760771","T-H37Rv");
		myHClade.put("777777470000000","Unknown");
		myHClade.put("777777470000771","Unknown");
		myHClade.put("777777470060771","T-H37Rv");
		myHClade.put("777777474020731","H1");
		myHClade.put("777777474020771","H1");
		myHClade.put("777777474760771","T-H37Rv");
		myHClade.put("777777476000001","Unknown");
		myHClade.put("777777477400001","Unknown");
		myHClade.put("777777477410001","EAI5");
		myHClade.put("777777477410071","EAI5");
		myHClade.put("777777477413671","EAI5");
		myHClade.put("777777477413700","EAI5");
		myHClade.put("777777477413701","EAI5");
		myHClade.put("777777477413751","EAI5");
		myHClade.put("777777477413771","EAI5");
		myHClade.put("777777477513771","EAI2-Manila");
		myHClade.put("777777477670400","Manu3");
		myHClade.put("777777477700731","Unknown");
		myHClade.put("777777477720771","H3");
		myHClade.put("777777477753771","Manu1");
		myHClade.put("777777477760411","T-H37Rv");
		myHClade.put("777777477760631","T-H37Rv");
		myHClade.put("777777477760731","T-H37Rv");
		myHClade.put("777777477760771","T-H37Rv");
		myHClade.put("777777477763771","Manu2");
		myHClade.put("777777477777771","Unknown");
		myHClade.put("777777500760771","T1");
		myHClade.put("777777504760771","T1");
		myHClade.put("777777507760771","T1");
		myHClade.put("777777544320771","H3");
		myHClade.put("777777547760471","T1");
		myHClade.put("777777547760771","T1");
		myHClade.put("777777554760771","T5-Madrid2");
		myHClade.put("777777556360771","T5-Madrid2");
		myHClade.put("777777557010771","EAI5");
		myHClade.put("777777557410371","EAI6-BGD1");
		myHClade.put("777777557413771","EAI6-BGD1");
		myHClade.put("777777557700771","Unknown");
		myHClade.put("777777557760631","T5-Madrid2");
		myHClade.put("777777557760700","T5-Madrid2");
		myHClade.put("777777557760771","T5-Madrid2");
		myHClade.put("777777557770671","Manu3");
		myHClade.put("777777557770771","Manu3");
		myHClade.put("777777557777771","Unknown");
		myHClade.put("777777560020611","H3");
		myHClade.put("777777570000000","Unknown");
		myHClade.put("777777570020731","H3");
		myHClade.put("777777574020771","H1");
		myHClade.put("777777576000000","Unknown");
		myHClade.put("777777577363771","Manu2");
		myHClade.put("777777577413731","EAI1-SOM");
		myHClade.put("777777577560771","T1");
		myHClade.put("777777577620771","H3");
		myHClade.put("777777577660071","T1");
		myHClade.put("777777577660771","T1");
		myHClade.put("777777577720671","H3");
		myHClade.put("777777577720731","H3");
		myHClade.put("777777577720771","H3");
		myHClade.put("777777577733751","Manu1");
		myHClade.put("777777577760731","T2");
		myHClade.put("777777577760770","T1");
		myHClade.put("777777577760771","T1");
		myHClade.put("777777577763771","Manu2");
		myHClade.put("777777577770771","Manu3");
		myHClade.put("777777577773771","Manu1");
		myHClade.put("777777600000000","Unknown");
		myHClade.put("777777600000171","Unknown");
		myHClade.put("777777600000200","Unknown");
		myHClade.put("777777600000371","Unknown");
		myHClade.put("777777600000611","Unknown");
		myHClade.put("777777600000711","Unknown");
		myHClade.put("777777600000731","Unknown");
		myHClade.put("777777600000771","LAM");
		myHClade.put("777777600003271","Unknown");
		myHClade.put("777777600003371","Unknown");
		myHClade.put("777777600003771","Unknown");
		myHClade.put("777777600020371","H3");
		myHClade.put("777777600020731","H3");
		myHClade.put("777777600020771","H3");
		myHClade.put("777777600040771","T1");
		myHClade.put("777777600060731","T2");
		myHClade.put("777777600060771","Unknown");
		myHClade.put("777777600160731","T2");
		myHClade.put("777777600160771","Unknown");
		myHClade.put("777777600360771","T1");
		myHClade.put("777777600500771","Unknown");
		myHClade.put("777777600560731","T2");
		myHClade.put("777777600560771","T1");
		myHClade.put("777777600760771","T1");
		myHClade.put("777777601560711","T1");
		myHClade.put("777777601560731","T2");
		myHClade.put("777777601560751","T1");
		myHClade.put("777777601560771","T1");
		myHClade.put("777777601740771","T1");
		myHClade.put("777777601760771","T1");
		myHClade.put("777777602003771","Unknown");
		myHClade.put("777777602060371","T1");
		myHClade.put("777777602060771","T1");
		myHClade.put("777777602760771","T1");
		myHClade.put("777777603401700","Unknown");
		myHClade.put("777777603413700","EAI5");
		myHClade.put("777777603413731","EAI1-SOM");
		myHClade.put("777777603560771","T1");
		myHClade.put("777777603760371","T1");
		myHClade.put("777777603760631","T1");
		myHClade.put("777777603760730","T2-Uganda");
		myHClade.put("777777603760771","LAM9");
		myHClade.put("777777604000000","Unknown");
		myHClade.put("777777604000171","Unknown");
		myHClade.put("777777604020731","H1");
		myHClade.put("777777604020771","Unknown");
		myHClade.put("777777604030771","Manu3");
		myHClade.put("777777604060631","LAM4");
		myHClade.put("777777604060731","LAM11-ZWE");
		myHClade.put("777777604060771","LAM9");
		myHClade.put("777777604063771","Manu2");
		myHClade.put("777777604360771","LAM9");
		myHClade.put("777777604760771","LAM9");
		myHClade.put("777777605560771","LAM6");
		myHClade.put("777777605660771","LAM9");
		myHClade.put("777777605760471","LAM9");
		myHClade.put("777777605760771","LAM9");
		myHClade.put("777777605761771","Unknown");
		myHClade.put("777777605770471","Manu3");
		myHClade.put("777777606000000","Unknown");
		myHClade.put("777777606000031","Unknown");
		myHClade.put("777777606000171","Unknown");
		myHClade.put("777777606000371","Unknown");
		myHClade.put("777777606000771","Unknown");
		myHClade.put("777777606020731","H3");
		myHClade.put("777777606020771","H3");
		myHClade.put("777777606041771","Unknown");
		myHClade.put("777777606060331","LAM11-ZWE");
		myHClade.put("777777606060470","LAM11-ZWE");
		myHClade.put("777777606060471","LAM11-ZWE");
		myHClade.put("777777606060571","LAM11-ZWE");
		myHClade.put("777777606060621","LAM11-ZWE");
		myHClade.put("777777606060631","LAM11-ZWE");
		myHClade.put("777777606060671","LAM11-ZWE");
		myHClade.put("777777606060701","LAM11-ZWE");
		myHClade.put("777777606060730","LAM11-ZWE");
		myHClade.put("777777606060731","LAM11-ZWE");
		myHClade.put("777777606060751","LAM11-ZWE");
		myHClade.put("777777606060770","LAM11-ZWE");
		myHClade.put("777777606060771","LAM11-ZWE");
		myHClade.put("777777606061771","Unknown");
		myHClade.put("777777606063771","Manu2");
		myHClade.put("777777606160771","LAM9");
		myHClade.put("777777606340771","LAM9");
		myHClade.put("777777606360771","LAM9");
		myHClade.put("777777606560731","LAM6");
		myHClade.put("777777606560771","LAM6");
		myHClade.put("777777606760571","LAM8");
		myHClade.put("777777606760631","LAM8");
		myHClade.put("777777606760771","LAM8");
		myHClade.put("777777607000000","Unknown");
		myHClade.put("777777607000031","Unknown");
		myHClade.put("777777607000771","Unknown");
		myHClade.put("777777607020731","H3");
		myHClade.put("777777607020771","H3");
		myHClade.put("777777607060771","LAM9");
		myHClade.put("777777607160731","LAM4");
		myHClade.put("777777607160771","LAM9");
		myHClade.put("777777607200771","Unknown");
		myHClade.put("777777607360421","LAM4");
		myHClade.put("777777607360731","LAM4");
		myHClade.put("777777607360771","LAM9");
		myHClade.put("777777607400000","LAM");
		myHClade.put("777777607400071","Unknown");
		myHClade.put("777777607400171","Unknown");
		myHClade.put("777777607400371","Unknown");
		myHClade.put("777777607403171","Unknown");
		myHClade.put("777777607420771","Ural-1");
		myHClade.put("777777607440771","LAM9");
		myHClade.put("777777607460471","LAM9");
		myHClade.put("777777607460730","LAM4");
		myHClade.put("777777607460731","LAM4");
		myHClade.put("777777607460771","LAM9");
		myHClade.put("777777607500771","Unknown");
		myHClade.put("777777607520771","H3");
		myHClade.put("777777607540410","LAM4");
		myHClade.put("777777607540570","LAM9");
		myHClade.put("777777607540731","LAM4");
		myHClade.put("777777607540761","LAM9");
		myHClade.put("777777607540771","LAM9");
		myHClade.put("777777607560371","LAM6");
		myHClade.put("777777607560471","LAM6");
		myHClade.put("777777607560531","LAM6");
		myHClade.put("777777607560570","LAM6");
		myHClade.put("777777607560661","LAM6");
		myHClade.put("777777607560721","LAM6");
		myHClade.put("777777607560731","LAM6");
		myHClade.put("777777607560751","LAM6");
		myHClade.put("777777607560760","LAM6");
		myHClade.put("777777607560771","LAM6");
		myHClade.put("777777607563771","Manu2");
		myHClade.put("777777607570771","Manu3");
		myHClade.put("777777607600031","Unknown");
		myHClade.put("777777607600171","Unknown");
		myHClade.put("777777607600331","Unknown");
		myHClade.put("777777607600371","Unknown");
		myHClade.put("777777607610031","Unknown");
		myHClade.put("777777607660671","LAM9");
		myHClade.put("777777607660731","LAM4");
		myHClade.put("777777607660771","LAM9");
		myHClade.put("777777607670771","Manu3");
		myHClade.put("777777607700001","Unknown");
		myHClade.put("777777607700071","Unknown");
		myHClade.put("777777607700171","EAI5");
		myHClade.put("777777607700731","LAM4");
		myHClade.put("777777607700771","LAM9");
		myHClade.put("777777607703771","Unknown");
		myHClade.put("777777607720731","H3");
		myHClade.put("777777607720771","LAM9");
		myHClade.put("777777607723771","Manu2");
		myHClade.put("777777607730771","Manu3");
		myHClade.put("777777607740000","LAM4");
		myHClade.put("777777607740011","LAM4");
		myHClade.put("777777607740031","LAM");
		myHClade.put("777777607740071","LAM9");
		myHClade.put("777777607740171","LAM9");
		myHClade.put("777777607740271","LAM9");
		myHClade.put("777777607740371","LAM9");
		myHClade.put("777777607740771","LAM9");
		myHClade.put("777777607741771","Unknown");
		myHClade.put("777777607760071","LAM9");
		myHClade.put("777777607760171","LAM9");
		myHClade.put("777777607760271","LAM9");
		myHClade.put("777777607760371","LAM9");
		myHClade.put("777777607760400","LAM4");
		myHClade.put("777777607760411","LAM4");
		myHClade.put("777777607760431","LAM4");
		myHClade.put("777777607760471","LAM9");
		myHClade.put("777777607760531","LAM4");
		myHClade.put("777777607760571","LAM9");
		myHClade.put("777777607760600","LAM");
		myHClade.put("777777607760601","LAM4");
		myHClade.put("777777607760611","LAM4");
		myHClade.put("777777607760621","LAM4");
		myHClade.put("777777607760631","LAM4");
		myHClade.put("777777607760671","LAM9");
		myHClade.put("777777607760700","LAM4");
		myHClade.put("777777607760701","LAM4");
		myHClade.put("777777607760711","LAM4");
		myHClade.put("777777607760721","LAM4");
		myHClade.put("777777607760731","LAM4");
		myHClade.put("777777607760751","LAM9");
		myHClade.put("777777607760760","LAM9");
		myHClade.put("777777607760761","LAM9");
		myHClade.put("777777607760770","LAM9");
		myHClade.put("777777607760771","LAM9");
		myHClade.put("777777607761771","Unknown");
		myHClade.put("777777607762771","Manu2");
		myHClade.put("777777607763771","Manu2");
		myHClade.put("777777607770770","Manu3");
		myHClade.put("777777607770771","Manu3");
		myHClade.put("777777607777771","Unknown");
		myHClade.put("777777617560571","T1");
		myHClade.put("777777617760731","T2");
		myHClade.put("777777617760771","T1");
		myHClade.put("777777617762771","Manu2");
		myHClade.put("777777620000000","Unknown");
		myHClade.put("777777626060771","T1");
		myHClade.put("777777627760731","T2");
		myHClade.put("777777634020771","H1");
		myHClade.put("777777636760771","T1");
		myHClade.put("777777637400071","Unknown");
		myHClade.put("777777637413071","EAI5");
		myHClade.put("777777637413771","EAI5");
		myHClade.put("777777637560771","T1");
		myHClade.put("777777637720751","H3");
		myHClade.put("777777637720771","H3");
		myHClade.put("777777637760731","T2");
		myHClade.put("777777637760751","T1");
		myHClade.put("777777637760771","T1");
		myHClade.put("777777637763771","Manu2");
		myHClade.put("777777640000000","Unknown");
		myHClade.put("777777640000271","Unknown");
		myHClade.put("777777640000771","Unknown");
		myHClade.put("777777640020711","H3");
		myHClade.put("777777640760601","T1");
		myHClade.put("777777641560771","T1");
		myHClade.put("777777643560771","Cameroon");
		myHClade.put("777777644020771","H1");
		myHClade.put("777777644720771","H3");
		myHClade.put("777777647660761","T1");
		myHClade.put("777777647760771","T1");
		myHClade.put("777777656413371","EAI6-BGD1");
		myHClade.put("777777657413771","EAI6-BGD1");
		myHClade.put("777777657760071","T5");
		myHClade.put("777777660000131","Unknown");
		myHClade.put("777777660000171","Unknown");
		myHClade.put("777777660000371","Unknown");
		myHClade.put("777777660020771","H3");
		myHClade.put("777777660060771","T1");
		myHClade.put("777777661760771","T1");
		myHClade.put("777777662020771","H3");
		myHClade.put("777777663360771","T1");
		myHClade.put("777777663520771","H3");
		myHClade.put("777777663560771","T");
		myHClade.put("777777663600171","Unknown");
		myHClade.put("777777663720771","H3");
		myHClade.put("777777663740771","T1");
		myHClade.put("777777663760000","T1");
		myHClade.put("777777663760331","T2");
		myHClade.put("777777663760731","T2");
		myHClade.put("777777663760771","T1");
		myHClade.put("777777666000771","Unknown");
		myHClade.put("777777666003771","Unknown");
		myHClade.put("777777666020771","H3");
		myHClade.put("777777667560771","T1");
		myHClade.put("777777667760771","T1");
		myHClade.put("777777667763771","Manu2");
		myHClade.put("777777670000771","Unknown");
		myHClade.put("777777670020771","H3");
		myHClade.put("777777670160771","T1");
		myHClade.put("777777673760771","T1");
		myHClade.put("777777674020371","H1");
		myHClade.put("777777674020731","H1");
		myHClade.put("777777674020771","H1");
		myHClade.put("777777674063771","Manu2");
		myHClade.put("777777674720771","H3");
		myHClade.put("777777674760771","T1");
		myHClade.put("777777675760771","T1");
		myHClade.put("777777676060771","T1");
		myHClade.put("777777676561771","Unknown");
		myHClade.put("777777676740031","T1");
		myHClade.put("777777676760771","T1");
		myHClade.put("777777676761771","Unknown");
		myHClade.put("777777677413731","EAI1-SOM");
		myHClade.put("777777677421771","Unknown");
		myHClade.put("777777677560371","T1");
		myHClade.put("777777677560671","T1");
		myHClade.put("777777677560731","T2");
		myHClade.put("777777677560771","T1");
		myHClade.put("777777677577771","Unknown");
		myHClade.put("777777677660771","T1");
		myHClade.put("777777677720771","H3");
		myHClade.put("777777677723600","Manu2");
		myHClade.put("777777677760000","T1");
		myHClade.put("777777677760601","T1");
		myHClade.put("777777677760671","T1");
		myHClade.put("777777677760700","T1");
		myHClade.put("777777677760711","T1");
		myHClade.put("777777677760731","T2");
		myHClade.put("777777677760770","T1");
		myHClade.put("777777677760771","T1");
		myHClade.put("777777677763771","Manu2");
		myHClade.put("777777677770771","Manu3");
		myHClade.put("777777677773771","Manu1");
		myHClade.put("777777677777671","Unknown");
		myHClade.put("777777677777771","Unknown");
		myHClade.put("777777700000000","Unknown");
		myHClade.put("777777700000001","Unknown");
		myHClade.put("777777700000171","Unknown");
		myHClade.put("777777700000371","Unknown");
		myHClade.put("777777700000611","Unknown");
		myHClade.put("777777700000731","Unknown");
		myHClade.put("777777700000771","Unknown");
		myHClade.put("777777700003070","Unknown");
		myHClade.put("777777700003341","Unknown");
		myHClade.put("777777700003371","EAI7-BGD2");
		myHClade.put("777777700003531","Unknown");
		myHClade.put("777777700003771","Unknown");
		myHClade.put("777777700020571","H3");
		myHClade.put("777777700020611","H3");
		myHClade.put("777777700020731","H3");
		myHClade.put("777777700020771","H3");
		myHClade.put("777777700023771","Manu2");
		myHClade.put("777777700060771","T1");
		myHClade.put("777777700160771","T1");
		myHClade.put("777777701760731","T2");
		myHClade.put("777777701760771","T1");
		myHClade.put("777777702003371","Unknown");
		myHClade.put("777777702003531","Unknown");
		myHClade.put("777777702003771","Unknown");
		myHClade.put("777777702010331","EAI1-SOM");
		myHClade.put("777777702020771","H3");
		myHClade.put("777777702760731","T2");
		myHClade.put("777777703760771","T1");
		myHClade.put("777777704020751","H1");
		myHClade.put("777777704020771","H1");
		myHClade.put("777777704022731","Manu2");
		myHClade.put("777777704303771","Unknown");
		myHClade.put("777777704720771","H3");
		myHClade.put("777777706360771","T1");
		myHClade.put("777777706660731","T2");
		myHClade.put("777777707003371","Unknown");
		myHClade.put("777777707413371","EAI5");
		myHClade.put("777777707413731","EAI1-SOM");
		myHClade.put("777777707413771","EAI5");
		myHClade.put("777777707560731","T2");
		myHClade.put("777777707560771","T1");
		myHClade.put("777777707700371","Unknown");
		myHClade.put("777777707720771","H3");
		myHClade.put("777777707740371","T1");
		myHClade.put("777777707740771","T1");
		myHClade.put("777777707741770","Unknown");
		myHClade.put("777777707741771","Unknown");
		myHClade.put("777777707760351","T1");
		myHClade.put("777777707760371","T1");
		myHClade.put("777777707760711","T1");
		myHClade.put("777777707760731","T2");
		myHClade.put("777777707760771","T1");
		myHClade.put("777777707770771","Manu3");
		myHClade.put("777777710000171","Unknown");
		myHClade.put("777777710020771","H3");
		myHClade.put("777777714020771","H1");
		myHClade.put("777777714160371","T1");
		myHClade.put("777777714413771","EAI4-VNM");
		myHClade.put("777777717413371","EAI5");
		myHClade.put("777777717413731","EAI1-SOM");
		myHClade.put("777777717560771","T1");
		myHClade.put("777777717620731","H3");
		myHClade.put("777777717720731","H3");
		myHClade.put("777777717720771","H3");
		myHClade.put("777777717760571","T1");
		myHClade.put("777777717760731","T2");
		myHClade.put("777777717760770","T1");
		myHClade.put("777777717760771","T1");
		myHClade.put("777777717763771","Manu2");
		myHClade.put("777777720000171","Unknown");
		myHClade.put("777777724020771","H1");
		myHClade.put("777777727760771","T1");
		myHClade.put("777777730000771","Unknown");
		myHClade.put("777777730020771","H3");
		myHClade.put("777777734020571","H1");
		myHClade.put("777777734413771","EAI4-VNM");
		myHClade.put("777777734720771","H3");
		myHClade.put("777777736760731","T2");
		myHClade.put("777777736760771","T1");
		myHClade.put("777777737413731","EAI1-SOM");
		myHClade.put("777777737413771","EAI5");
		myHClade.put("777777737660771","T1");
		myHClade.put("777777737720771","H3");
		myHClade.put("777777737740071","T1");
		myHClade.put("777777737740771","T1");
		myHClade.put("777777737760471","T1");
		myHClade.put("777777737760571","T1");
		myHClade.put("777777737760700","T1");
		myHClade.put("777777737760731","T2");
		myHClade.put("777777737760740","T1");
		myHClade.put("777777737760751","T1");
		myHClade.put("777777737760771","T1");
		myHClade.put("777777740000000","Unknown");
		myHClade.put("777777740000011","Unknown");
		myHClade.put("777777740000031","Unknown");
		myHClade.put("777777740000071","Unknown");
		myHClade.put("777777740000171","Unknown");
		myHClade.put("777777740000371","Unknown");
		myHClade.put("777777740003171","Unknown");
		myHClade.put("777777740003371","Unknown");
		myHClade.put("777777740003771","Unknown");
		myHClade.put("777777740020731","H3");
		myHClade.put("777777740020771","H");
		myHClade.put("777777740023771","Manu2");
		myHClade.put("777777740060771","T1");
		myHClade.put("777777740070771","Manu3");
		myHClade.put("777777740160771","T1");
		myHClade.put("777777740360771","T1");
		myHClade.put("777777740403371","Unknown");
		myHClade.put("777777740413371","EAI6-BGD1");
		myHClade.put("777777740660771","T1");
		myHClade.put("777777740760771","T1");
		myHClade.put("777777740763771","Manu2");
		myHClade.put("777777741413371","EAI6-BGD1");
		myHClade.put("777777741760771","T1");
		myHClade.put("777777742000071","Unknown");
		myHClade.put("777777742003371","Unknown");
		myHClade.put("777777742003771","Unknown");
		myHClade.put("777777742120771","H3");
		myHClade.put("777777742160731","Cameroon");
		myHClade.put("777777742160771","Cameroon");
		myHClade.put("777777742413371","EAI6-BGD1");
		myHClade.put("777777742760771","Cameroon");
		myHClade.put("777777743360771","Cameroon");
		myHClade.put("777777743413311","EAI6-BGD1");
		myHClade.put("777777743413331","EAI");
		myHClade.put("777777743413371","EAI6-BGD1");
		myHClade.put("777777743440771","Cameroon");
		myHClade.put("777777743460731","Cameroon");
		myHClade.put("777777743460771","Cameroon");
		myHClade.put("777777743560771","Cameroon");
		myHClade.put("777777743700171","Unknown");
		myHClade.put("777777743720751","H3");
		myHClade.put("777777743720771","H3");
		myHClade.put("777777743760171","Cameroon");
		myHClade.put("777777743760371","Cameroon");
		myHClade.put("777777743760571","Cameroon");
		myHClade.put("777777743760711","Cameroon");
		myHClade.put("777777743760731","Cameroon");
		myHClade.put("777777743760751","Cameroon");
		myHClade.put("777777743760771","Cameroon");
		myHClade.put("777777743763771","Manu2");
		myHClade.put("777777743773771","Manu1");
		myHClade.put("777777744003671","Unknown");
		myHClade.put("777777744020731","H1");
		myHClade.put("777777744020771","H1");
		myHClade.put("777777744030771","Manu3");
		myHClade.put("777777744533771","Manu1");
		myHClade.put("777777744720771","H3");
		myHClade.put("777777744773771","Manu1");
		myHClade.put("777777745720771","H3");
		myHClade.put("777777746020771","H3");
		myHClade.put("777777746060771","T1");
		myHClade.put("777777747013371","EAI5");
		myHClade.put("777777747411371","EAI6-BGD1");
		myHClade.put("777777747413371","EAI6-BGD1");
		myHClade.put("777777747413671","EAI6-BGD1");
		myHClade.put("777777747413731","EAI");
		myHClade.put("777777747413771","EAI6-BGD1");
		myHClade.put("777777747420771","Ural-1");
		myHClade.put("777777747473771","Manu1");
		myHClade.put("777777747613371","Manu1");
		myHClade.put("777777747613771","Unknown");
		myHClade.put("777777747700771","Unknown");
		myHClade.put("777777747720730","H3");
		myHClade.put("777777747720731","H3");
		myHClade.put("777777747720771","H3");
		myHClade.put("777777747743771","Unknown");
		myHClade.put("777777747760411","T1");
		myHClade.put("777777747760431","T");
		myHClade.put("777777747760471","T1");
		myHClade.put("777777747760671","T1");
		myHClade.put("777777747760731","T2");
		myHClade.put("777777747760771","T");
		myHClade.put("777777747762771","Manu2");
		myHClade.put("777777747763771","Manu2");
		myHClade.put("777777747764771","Unknown");
		myHClade.put("777777747770471","Manu3");
		myHClade.put("777777747773771","Manu1");
		myHClade.put("777777747777771","Unknown");
		myHClade.put("777777750000000","Unknown");
		myHClade.put("777777750000371","Unknown");
		myHClade.put("777777750000471","Unknown");
		myHClade.put("777777750002371","Unknown");
		myHClade.put("777777750003331","EAI7-BGD2");
		myHClade.put("777777750003371","EAI7-BGD2");
		myHClade.put("777777750003771","Unknown");
		myHClade.put("777777750013371","EAI5");
		myHClade.put("777777750020771","H3");
		myHClade.put("777777750060671","T5");
		myHClade.put("777777750060731","T");
		myHClade.put("777777750060771","T");
		myHClade.put("777777750400371","Unknown");
		myHClade.put("777777750403371","Unknown");
		myHClade.put("777777750413371","EAI6-BGD1");
		myHClade.put("777777750413400","EAI6-BGD1");
		myHClade.put("777777750413771","EAI6-BGD1");
		myHClade.put("777777751413331","EAI");
		myHClade.put("777777751413671","EAI6-BGD1");
		myHClade.put("777777751413771","EAI6-BGD1");
		myHClade.put("777777752000000","Unknown");
		myHClade.put("777777752000071","Unknown");
		myHClade.put("777777752003371","Unknown");
		myHClade.put("777777752403371","Unknown");
		myHClade.put("777777752413371","EAI6-BGD1");
		myHClade.put("777777753400671","Unknown");
		myHClade.put("777777753413771","EAI6-BGD1");
		myHClade.put("777777754020731","H1");
		myHClade.put("777777754020771","H1");
		myHClade.put("777777754030670","Manu3");
		myHClade.put("777777754030771","Manu3");
		myHClade.put("777777754360771","T5");
		myHClade.put("777777754413771","EAI4-VNM");
		myHClade.put("777777754613771","Manu1");
		myHClade.put("777777755020771","H3");
		myHClade.put("777777755413331","EAI");
		myHClade.put("777777755413371","EAI6-BGD1");
		myHClade.put("777777755413731","EAI");
		myHClade.put("777777755413771","EAI6-BGD1");
		myHClade.put("777777755460771","T5");
		myHClade.put("777777756013371","EAI5");
		myHClade.put("777777756413771","EAI6-BGD1");
		myHClade.put("777777756760771","T5");
		myHClade.put("777777757003371","Unknown");
		myHClade.put("777777757003771","Unknown");
		myHClade.put("777777757013171","EAI5");
		myHClade.put("777777757013371","EAI5");
		myHClade.put("777777757400000","Unknown");
		myHClade.put("777777757400011","Unknown");
		myHClade.put("777777757400071","Unknown");
		myHClade.put("777777757403371","Unknown");
		myHClade.put("777777757403711","Unknown");
		myHClade.put("777777757403771","Unknown");
		myHClade.put("777777757410000","EAI6-BGD1");
		myHClade.put("777777757410071","EAI6-BGD1");
		myHClade.put("777777757410371","EAI6-BGD1");
		myHClade.put("777777757411371","EAI6-BGD1");
		myHClade.put("777777757413071","EAI6-BGD1");
		myHClade.put("777777757413171","EAI6-BGD1");
		myHClade.put("777777757413271","EAI6-BGD1");
		myHClade.put("777777757413331","EAI");
		myHClade.put("777777757413350","EAI6-BGD1");
		myHClade.put("777777757413371","EAI6-BGD1");
		myHClade.put("777777757413671","EAI6-BGD1");
		myHClade.put("777777757413731","EAI");
		myHClade.put("777777757413771","EAI6-BGD1");
		myHClade.put("777777757460771","T5");
		myHClade.put("777777757560771","T5");
		myHClade.put("777777757600000","Unknown");
		myHClade.put("777777757620370","H3");
		myHClade.put("777777757660771","T5");
		myHClade.put("777777757720771","H3");
		myHClade.put("777777757733771","Manu1");
		myHClade.put("777777757760571","T5");
		myHClade.put("777777757760611","T5");
		myHClade.put("777777757760671","T5");
		myHClade.put("777777757760700","T5");
		myHClade.put("777777757760711","T5");
		myHClade.put("777777757760731","T");
		myHClade.put("777777757760751","T5");
		myHClade.put("777777757760760","T5");
		myHClade.put("777777757760770","T5");
		myHClade.put("777777757760771","T5");
		myHClade.put("777777757761771","Unknown");
		myHClade.put("777777757762771","Manu2");
		myHClade.put("777777757763771","Manu2");
		myHClade.put("777777757767771","Unknown");
		myHClade.put("777777757770771","Manu3");
		myHClade.put("777777757777771","Unknown");
		myHClade.put("777777760000000","Unknown");
		myHClade.put("777777760000001","Unknown");
		myHClade.put("777777760000011","Unknown");
		myHClade.put("777777760000031","Unknown");
		myHClade.put("777777760000071","Unknown");
		myHClade.put("777777760000131","Unknown");
		myHClade.put("777777760000140","Unknown");
		myHClade.put("777777760000171","Unknown");
		myHClade.put("777777760000371","Unknown");
		myHClade.put("777777760000611","Unknown");
		myHClade.put("777777760000731","Unknown");
		myHClade.put("777777760000771","Unknown");
		myHClade.put("777777760020611","H3");
		myHClade.put("777777760020751","H3");
		myHClade.put("777777760020771","H3");
		myHClade.put("777777760060771","T1");
		myHClade.put("777777760070771","Manu3");
		myHClade.put("777777760120731","H3");
		myHClade.put("777777760120771","H3");
		myHClade.put("777777760160731","T2");
		myHClade.put("777777760160771","T1");
		myHClade.put("777777760340000","T1");
		myHClade.put("777777760360731","T2");
		myHClade.put("777777760360771","T1");
		myHClade.put("777777760413731","EAI1-SOM");
		myHClade.put("777777760760731","T2");
		myHClade.put("777777760760771","T1");
		myHClade.put("777777761413700","EAI5");
		myHClade.put("777777761520771","H3");
		myHClade.put("777777761700000","Unknown");
		myHClade.put("777777761720740","H3");
		myHClade.put("777777761720771","H3");
		myHClade.put("777777761740771","T1");
		myHClade.put("777777761760771","T1");
		myHClade.put("777777763000371","Unknown");
		myHClade.put("777777763420771","Ural-1");
		myHClade.put("777777763560731","T2");
		myHClade.put("777777763660771","T1");
		myHClade.put("777777763760771","T1");
		myHClade.put("777777764020631","H1");
		myHClade.put("777777764020731","H1");
		myHClade.put("777777764020740","H1");
		myHClade.put("777777764020771","H1");
		myHClade.put("777777764120771","H3");
		myHClade.put("777777764130771","Manu3");
		myHClade.put("777777764760771","T");
		myHClade.put("777777765720771","H3");
		myHClade.put("777777766000771","Unknown");
		myHClade.put("777777767400071","Unknown");
		myHClade.put("777777767413731","EAI1-SOM");
		myHClade.put("777777767413771","EAI5");
		myHClade.put("777777767600000","Unknown");
		myHClade.put("777777767605571","Unknown");
		myHClade.put("777777767700000","Unknown");
		myHClade.put("777777767720731","H3");
		myHClade.put("777777767720771","H3");
		myHClade.put("777777767760371","T1");
		myHClade.put("777777767760731","T2");
		myHClade.put("777777767760760","T1");
		myHClade.put("777777767760761","T1");
		myHClade.put("777777767760771","T1");
		myHClade.put("777777767763771","Manu2");
		myHClade.put("777777767770731","Manu3");
		myHClade.put("777777767777771","Unknown");
		myHClade.put("777777770000000","Unknown");
		myHClade.put("777777770000011","Unknown");
		myHClade.put("777777770000031","Unknown");
		myHClade.put("777777770000171","Unknown");
		myHClade.put("777777770000311","Unknown");
		myHClade.put("777777770000371","Unknown");
		myHClade.put("777777770000631","Unknown");
		myHClade.put("777777770000711","Unknown");
		myHClade.put("777777770000731","Unknown");
		myHClade.put("777777770000751","Unknown");
		myHClade.put("777777770000771","Unknown");
		myHClade.put("777777770002771","Unknown");
		myHClade.put("777777770003171","Unknown");
		myHClade.put("777777770003371","EAI7-BGD2");
		myHClade.put("777777770003731","Unknown");
		myHClade.put("777777770003771","Unknown");
		myHClade.put("777777770014351","Unknown");
		myHClade.put("777777770020731","H3");
		myHClade.put("777777770020751","H3");
		myHClade.put("777777770020771","H");
		myHClade.put("777777770023771","Manu2");
		myHClade.put("777777770030731","Manu3");
		myHClade.put("777777770033771","Manu1");
		myHClade.put("777777770060571","T1");
		myHClade.put("777777770060731","T2");
		myHClade.put("777777770060771","T1");
		myHClade.put("777777770200771","Unknown");
		myHClade.put("777777770203371","Unknown");
		myHClade.put("777777770360771","T1");
		myHClade.put("777777770413771","EAI5");
		myHClade.put("777777770720700","H3");
		myHClade.put("777777770760671","T1");
		myHClade.put("777777770760731","T2");
		myHClade.put("777777770760760","T1");
		myHClade.put("777777770760771","T1");
		myHClade.put("777777771413470","EAI5");
		myHClade.put("777777771700000","Unknown");
		myHClade.put("777777771720771","H3");
		myHClade.put("777777772013731","EAI1-SOM");
		myHClade.put("777777772413731","EAI1-SOM");
		myHClade.put("777777773410771","EAI5");
		myHClade.put("777777773413731","EAI1-SOM");
		myHClade.put("777777773720771","H3");
		myHClade.put("777777773760771","T1");
		myHClade.put("777777773763771","Manu2");
		myHClade.put("777777773774731","Unknown");
		myHClade.put("777777774000000","Unknown");
		myHClade.put("777777774000031","Unknown");
		myHClade.put("777777774000071","Unknown");
		myHClade.put("777777774000131","Unknown");
		myHClade.put("777777774000331","Unknown");
		myHClade.put("777777774000371","Unknown");
		myHClade.put("777777774000731","Unknown");
		myHClade.put("777777774000771","Unknown");
		myHClade.put("777777774003771","Unknown");
		myHClade.put("777777774004771","Unknown");
		myHClade.put("777777774013031","EAI5");
		myHClade.put("777777774020131","H1");
		myHClade.put("777777774020371","H1");
		myHClade.put("777777774020531","H1");
		myHClade.put("777777774020571","H1");
		myHClade.put("777777774020611","H1");
		myHClade.put("777777774020630","H1");
		myHClade.put("777777774020631","H1");
		myHClade.put("777777774020660","H1");
		myHClade.put("777777774020671","H1");
		myHClade.put("777777774020700","H1");
		myHClade.put("777777774020701","H1");
		myHClade.put("777777774020711","H1");
		myHClade.put("777777774020720","H1");
		myHClade.put("777777774020730","H1");
		myHClade.put("777777774020731","H1");
		myHClade.put("777777774020751","H1");
		myHClade.put("777777774020760","H1");
		myHClade.put("777777774020770","H1");
		myHClade.put("777777774020771","H1");
		myHClade.put("777777774021771","Unknown");
		myHClade.put("777777774023771","Manu2");
		myHClade.put("777777774025471","Unknown");
		myHClade.put("777777774030771","Manu3");
		myHClade.put("777777774050771","Manu3");
		myHClade.put("777777774060671","T1");
		myHClade.put("777777774060771","T1");
		myHClade.put("777777774100000","Unknown");
		myHClade.put("777777774120771","H3");
		myHClade.put("777777774160731","T2");
		myHClade.put("777777774160771","T1");
		myHClade.put("777777774220771","H3");
		myHClade.put("777777774320771","H3");
		myHClade.put("777777774360771","T1");
		myHClade.put("777777774400771","Unknown");
		myHClade.put("777777774403771","Unknown");
		myHClade.put("777777774410171","EAI4-VNM");
		myHClade.put("777777774411600","EAI4-VNM");
		myHClade.put("777777774412071","EAI4-VNM");
		myHClade.put("777777774412771","EAI4-VNM");
		myHClade.put("777777774413301","EAI4-VNM");
		myHClade.put("777777774413471","EAI4-VNM");
		myHClade.put("777777774413700","EAI4-VNM");
		myHClade.put("777777774413711","EAI4-VNM");
		myHClade.put("777777774413731","EAI4-VNM");
		myHClade.put("777777774413771","EAI4-VNM");
		myHClade.put("777777774417771","Unknown");
		myHClade.put("777777774520771","H3");
		myHClade.put("777777774560771","T1");
		myHClade.put("777777774600771","Unknown");
		myHClade.put("777777774620731","H3");
		myHClade.put("777777774620771","H3");
		myHClade.put("777777774720731","H3");
		myHClade.put("777777774720771","H3");
		myHClade.put("777777774760771","T1");
		myHClade.put("777777775000011","Unknown");
		myHClade.put("777777775020371","H3");
		myHClade.put("777777775020711","H3");
		myHClade.put("777777775403731","Unknown");
		myHClade.put("777777775411031","EAI5");
		myHClade.put("777777775413371","EAI5");
		myHClade.put("777777775413671","EAI5");
		myHClade.put("777777775413700","EAI5");
		myHClade.put("777777775413731","EAI1-SOM");
		myHClade.put("777777775413771","EAI5");
		myHClade.put("777777775600000","Unknown");
		myHClade.put("777777775660771","T1");
		myHClade.put("777777775720471","H3");
		myHClade.put("777777775720731","H3");
		myHClade.put("777777775720771","H3");
		myHClade.put("777777775723771","Manu2");
		myHClade.put("777777775733771","Manu1");
		myHClade.put("777777775760371","T1");
		myHClade.put("777777775760571","T1");
		myHClade.put("777777775760700","T1");
		myHClade.put("777777775760731","T2");
		myHClade.put("777777775760770","T1");
		myHClade.put("777777775760771","T1");
		myHClade.put("777777776000071","Unknown");
		myHClade.put("777777776000171","Unknown");
		myHClade.put("777777776000371","Unknown");
		myHClade.put("777777776000731","Unknown");
		myHClade.put("777777776000740","Unknown");
		myHClade.put("777777776000771","Unknown");
		myHClade.put("777777776001700","Unknown");
		myHClade.put("777777776003731","Unknown");
		myHClade.put("777777776003771","Unknown");
		myHClade.put("777777776013731","EAI1-SOM");
		myHClade.put("777777776013771","EAI5");
		myHClade.put("777777776020371","H3");
		myHClade.put("777777776020731","H3");
		myHClade.put("777777776020771","H3");
		myHClade.put("777777776023771","Manu2");
		myHClade.put("777777776060771","T");
		myHClade.put("777777776120771","H3");
		myHClade.put("777777776160771","T1");
		myHClade.put("777777776220771","H3");
		myHClade.put("777777776360711","T1");
		myHClade.put("777777776360731","T2");
		myHClade.put("777777776360771","T1");
		myHClade.put("777777776403731","Unknown");
		myHClade.put("777777776413700","EAI5");
		myHClade.put("777777776413711","EAI5");
		myHClade.put("777777776413731","EAI1-SOM");
		myHClade.put("777777776413771","EAI5");
		myHClade.put("777777776613771","Manu1");
		myHClade.put("777777776700000","Unknown");
		myHClade.put("777777776700331","Unknown");
		myHClade.put("777777776720671","H3");
		myHClade.put("777777776720771","H3");
		myHClade.put("777777776740171","T1");
		myHClade.put("777777776760571","T1");
		myHClade.put("777777776760600","T1");
		myHClade.put("777777776760731","T2");
		myHClade.put("777777776760771","T1");
		myHClade.put("777777776761771","Unknown");
		myHClade.put("777777776773771","Manu1");
		myHClade.put("777777777000000","Unknown");
		myHClade.put("777777777000011","Unknown");
		myHClade.put("777777777000031","Unknown");
		myHClade.put("777777777000071","Unknown");
		myHClade.put("777777777000171","Unknown");
		myHClade.put("777777777000271","Unknown");
		myHClade.put("777777777000371","Unknown");
		myHClade.put("777777777000700","Unknown");
		myHClade.put("777777777000731","Unknown");
		myHClade.put("777777777000771","Unknown");
		myHClade.put("777777777003700","Unknown");
		myHClade.put("777777777003731","Unknown");
		myHClade.put("777777777003771","Unknown");
		myHClade.put("777777777013071","EAI5");
		myHClade.put("777777777013700","EAI5");
		myHClade.put("777777777013731","EAI1-SOM");
		myHClade.put("777777777013771","EAI5");
		myHClade.put("777777777020700","H3");
		myHClade.put("777777777020731","H");
		myHClade.put("777777777020771","H3");
		myHClade.put("777777777060771","T1");
		myHClade.put("777777777120771","H3");
		myHClade.put("777777777160031","T1");
		myHClade.put("777777777160771","T1");
		myHClade.put("777777777300471","Unknown");
		myHClade.put("777777777320571","H3");
		myHClade.put("777777777320731","H3");
		myHClade.put("777777777320771","H3");
		myHClade.put("777777777360200","T1");
		myHClade.put("777777777360371","T1");
		myHClade.put("777777777360411","T1");
		myHClade.put("777777777360700","T2");
		myHClade.put("777777777360731","T2");
		myHClade.put("777777777360771","T1");
		myHClade.put("777777777400000","Unknown");
		myHClade.put("777777777400001","Unknown");
		myHClade.put("777777777400011","Unknown");
		myHClade.put("777777777400031","Unknown");
		myHClade.put("777777777400131","Unknown");
		myHClade.put("777777777400171","Unknown");
		myHClade.put("777777777400371","Unknown");
		myHClade.put("777777777400771","Unknown");
		myHClade.put("777777777401731","Unknown");
		myHClade.put("777777777402077","Unknown");
		myHClade.put("777777777403671","Unknown");
		myHClade.put("777777777403700","Unknown");
		myHClade.put("777777777403701","Unknown");
		myHClade.put("777777777403731","Unknown");
		myHClade.put("777777777403771","EAI5");
		myHClade.put("777777777405700","Unknown");
		myHClade.put("777777777410000","EAI5");
		myHClade.put("777777777410030","EAI5");
		myHClade.put("777777777410031","EAI5");
		myHClade.put("777777777410331","EAI1-SOM");
		myHClade.put("777777777410700","EAI5");
		myHClade.put("777777777410770","EAI5");
		myHClade.put("777777777410771","EAI5");
		myHClade.put("777777777411731","EAI1-SOM");
		myHClade.put("777777777412031","EAI5");
		myHClade.put("777777777412071","EAI5");
		myHClade.put("777777777412671","EAI5");
		myHClade.put("777777777412700","EAI5");
		myHClade.put("777777777412771","EAI5");
		myHClade.put("777777777413020","EAI5");
		myHClade.put("777777777413031","EAI5");
		myHClade.put("777777777413071","EAI5");
		myHClade.put("777777777413131","EAI1-SOM");
		myHClade.put("777777777413271","EAI5");
		myHClade.put("777777777413301","EAI5");
		myHClade.put("777777777413311","EAI5");
		myHClade.put("777777777413331","EAI1-SOM");
		myHClade.put("777777777413371","EAI5");
		myHClade.put("777777777413400","EAI5");
		myHClade.put("777777777413431","EAI5");
		myHClade.put("777777777413531","EAI1-SOM");
		myHClade.put("777777777413571","EAI5");
		myHClade.put("777777777413600","EAI5");
		myHClade.put("777777777413611","EAI5");
		myHClade.put("777777777413631","EAI5");
		myHClade.put("777777777413641","EAI5");
		myHClade.put("777777777413671","EAI5");
		myHClade.put("777777777413700","EAI5");
		myHClade.put("777777777413701","EAI5");
		myHClade.put("777777777413711","EAI5");
		myHClade.put("777777777413720","EAI1-SOM");
		myHClade.put("777777777413730","EAI1-SOM");
		myHClade.put("777777777413731","EAI1-SOM");
		myHClade.put("777777777413741","EAI5");
		myHClade.put("777777777413751","EAI5");
		myHClade.put("777777777413761","EAI5");
		myHClade.put("777777777413770","EAI5");
		myHClade.put("777777777413771","EAI5");
		myHClade.put("777777777416761","Unknown");
		myHClade.put("777777777420311","Ural-1");
		myHClade.put("777777777420371","Ural-1");
		myHClade.put("777777777420410","Ural-1");
		myHClade.put("777777777420601","Ural-1");
		myHClade.put("777777777420611","Ural-1");
		myHClade.put("777777777420631","Ural-1");
		myHClade.put("777777777420731","Ural-1");
		myHClade.put("777777777420760","Ural-1");
		myHClade.put("777777777420770","Ural-1");
		myHClade.put("777777777420771","Ural-1");
		myHClade.put("777777777423771","Manu2");
		myHClade.put("777777777433771","Manu1");
		myHClade.put("777777777437771","Unknown");
		myHClade.put("777777777460631","T1");
		myHClade.put("777777777460731","T2");
		myHClade.put("777777777460760","T1");
		myHClade.put("777777777460771","T1");
		myHClade.put("777777777463731","Manu2");
		myHClade.put("777777777473771","Manu1");
		myHClade.put("777777777503771","Unknown");
		myHClade.put("777777777513731","Manu1");
		myHClade.put("777777777520731","H3");
		myHClade.put("777777777520771","H3");
		myHClade.put("777777777530771","Manu3");
		myHClade.put("777777777540771","T1");
		myHClade.put("777777777543771","Unknown");
		myHClade.put("777777777560031","T2");
		myHClade.put("777777777560611","T1");
		myHClade.put("777777777560700","T1");
		myHClade.put("777777777560731","T2");
		myHClade.put("777777777560740","T1");
		myHClade.put("777777777560771","T1");
		myHClade.put("777777777563771","Manu2");
		myHClade.put("777777777567771","Unknown");
		myHClade.put("777777777570771","Manu3");
		myHClade.put("777777777573731","Manu1");
		myHClade.put("777777777573771","Manu1");
		myHClade.put("777777777600000","Unknown");
		myHClade.put("777777777600011","Unknown");
		myHClade.put("777777777600031","Unknown");
		myHClade.put("777777777600071","Unknown");
		myHClade.put("777777777600131","Unknown");
		myHClade.put("777777777600171","Unknown");
		myHClade.put("777777777600371","Unknown");
		myHClade.put("777777777600471","Unknown");
		myHClade.put("777777777600700","Unknown");
		myHClade.put("777777777600731","Unknown");
		myHClade.put("777777777600760","Unknown");
		myHClade.put("777777777600771","Unknown");
		myHClade.put("777777777601731","Unknown");
		myHClade.put("777777777613071","Manu1");
		myHClade.put("777777777613771","Manu1");
		myHClade.put("777777777620031","H3");
		myHClade.put("777777777620731","H3");
		myHClade.put("777777777620760","H3");
		myHClade.put("777777777620771","H3");
		myHClade.put("777777777623771","Manu2");
		myHClade.put("777777777633771","Manu1");
		myHClade.put("777777777640171","T1");
		myHClade.put("777777777641770","Unknown");
		myHClade.put("777777777641771","Unknown");
		myHClade.put("777777777660031","T1");
		myHClade.put("777777777660071","T1");
		myHClade.put("777777777660731","T2");
		myHClade.put("777777777660771","T1");
		myHClade.put("777777777677731","Unknown");
		myHClade.put("777777777677771","Unknown");
		myHClade.put("777777777700000","Unknown");
		myHClade.put("777777777700001","Unknown");
		myHClade.put("777777777700011","Unknown");
		myHClade.put("777777777700031","T");
		myHClade.put("777777777700071","Unknown");
		myHClade.put("777777777700131","Unknown");
		myHClade.put("777777777700160","Unknown");
		myHClade.put("777777777700171","Unknown");
		myHClade.put("777777777700231","Unknown");
		myHClade.put("777777777700331","Unknown");
		myHClade.put("777777777700371","T");
		myHClade.put("777777777700571","Unknown");
		myHClade.put("777777777700731","Unknown");
		myHClade.put("777777777700760","Unknown");
		myHClade.put("777777777700771","Unknown");
		myHClade.put("777777777703761","Unknown");
		myHClade.put("777777777703771","Unknown");
		myHClade.put("777777777713771","Manu1");
		myHClade.put("777777777720031","H3");
		myHClade.put("777777777720071","H3");
		myHClade.put("777777777720171","H3");
		myHClade.put("777777777720371","H3");
		myHClade.put("777777777720471","H3");
		myHClade.put("777777777720531","H3");
		myHClade.put("777777777720571","H3");
		myHClade.put("777777777720600","H3");
		myHClade.put("777777777720601","H3");
		myHClade.put("777777777720631","H3");
		myHClade.put("777777777720671","H3");
		myHClade.put("777777777720700","H3");
		myHClade.put("777777777720711","H3");
		myHClade.put("777777777720731","H3");
		myHClade.put("777777777720740","H3");
		myHClade.put("777777777720741","H3");
		myHClade.put("777777777720751","H3");
		myHClade.put("777777777720760","H3");
		myHClade.put("777777777720761","H3");
		myHClade.put("777777777720770","H3");
		myHClade.put("777777777720771","H3");
		myHClade.put("777777777722771","Manu2");
		myHClade.put("777777777723771","Manu2");
		myHClade.put("777777777730351","Unknown");
		myHClade.put("777777777730731","Manu3");
		myHClade.put("777777777730771","Manu3");
		myHClade.put("777777777731771","Unknown");
		myHClade.put("777777777733671","Manu1");
		myHClade.put("777777777740000","T1");
		myHClade.put("777777777740001","Unknown");
		myHClade.put("777777777740011","T1");
		myHClade.put("777777777740031","T");
		myHClade.put("777777777740071","T1");
		myHClade.put("777777777740171","T1");
		myHClade.put("777777777740300","T1");
		myHClade.put("777777777740331","T2");
		myHClade.put("777777777740371","T1");
		myHClade.put("777777777740411","T1");
		myHClade.put("777777777740671","T1");
		myHClade.put("777777777740700","T1");
		myHClade.put("777777777740711","T1");
		myHClade.put("777777777740731","T2");
		myHClade.put("777777777740771","T1");
		myHClade.put("777777777741671","Unknown");
		myHClade.put("777777777741771","Unknown");
		myHClade.put("777777777743771","Unknown");
		myHClade.put("777777777747771","Unknown");
		myHClade.put("777777777750371","Unknown");
		myHClade.put("777777777750771","Manu3");
		myHClade.put("777777777753731","Manu1");
		myHClade.put("777777777753771","Manu1");
		myHClade.put("777777777760000","T1");
		myHClade.put("777777777760011","T1");
		myHClade.put("777777777760031","T2");
		myHClade.put("777777777760071","T1");
		myHClade.put("777777777760131","T2");
		myHClade.put("777777777760151","T1");
		myHClade.put("777777777760171","T1");
		myHClade.put("777777777760231","T1");
		myHClade.put("777777777760300","T1");
		myHClade.put("777777777760331","T2");
		myHClade.put("777777777760370","T1");
		myHClade.put("777777777760371","T");
		myHClade.put("777777777760400","T1");
		myHClade.put("777777777760410","T1");
		myHClade.put("777777777760411","T1");
		myHClade.put("777777777760431","T1");
		myHClade.put("777777777760471","T1");
		myHClade.put("777777777760500","T1");
		myHClade.put("777777777760531","T2");
		myHClade.put("777777777760561","T1");
		myHClade.put("777777777760570","T1");
		myHClade.put("777777777760571","T1");
		myHClade.put("777777777760600","T1");
		myHClade.put("777777777760601","T1");
		myHClade.put("777777777760611","T1");
		myHClade.put("777777777760631","T2");
		myHClade.put("777777777760661","T1");
		myHClade.put("777777777760671","T1");
		myHClade.put("777777777760700","T1");
		myHClade.put("777777777760701","T1");
		myHClade.put("777777777760710","T2-Uganda");
		myHClade.put("777777777760711","T");
		myHClade.put("777777777760720","T2");
		myHClade.put("777777777760730","T2-Uganda");
		myHClade.put("777777777760731","T2");
		myHClade.put("777777777760740","T1");
		myHClade.put("777777777760741","T1");
		myHClade.put("777777777760751","T1");
		myHClade.put("777777777760760","T1");
		myHClade.put("777777777760761","T1");
		myHClade.put("777777777760770","T1");
		myHClade.put("777777777760771","T1");
		myHClade.put("777777777761771","Unknown");
		myHClade.put("777777777762771","Manu2");
		myHClade.put("777777777763571","Manu2");
		myHClade.put("777777777763731","Manu2");
		myHClade.put("777777777763751","Manu2");
		myHClade.put("777777777763761","Manu2");
		myHClade.put("777777777763771","Manu2");
		myHClade.put("777777777765771","Unknown");
		myHClade.put("777777777766771","Unknown");
		myHClade.put("777777777767771","Unknown");
		myHClade.put("777777777770000","Unknown");
		myHClade.put("777777777770031","Unknown");
		myHClade.put("777777777770231","Unknown");
		myHClade.put("777777777770331","Unknown");
		myHClade.put("777777777770370","Unknown");
		myHClade.put("777777777770371","Unknown");
		myHClade.put("777777777770471","Manu3");
		myHClade.put("777777777770711","Manu3");
		myHClade.put("777777777770771","Manu3");
		myHClade.put("777777777771771","Unknown");
		myHClade.put("777777777772771","Manu1");
		myHClade.put("777777777773071","Manu1");
		myHClade.put("777777777773771","Manu1");
		myHClade.put("777777777774731","Unknown");
		myHClade.put("777777777774771","Unknown");
		myHClade.put("777777777775771","Unknown");
		myHClade.put("777777777776071","Unknown");
		myHClade.put("777777777777331","Unknown");
		myHClade.put("777777777777600","Unknown");
		myHClade.put("777777777777601","Unknown");
		myHClade.put("777777777777671","Unknown");
		myHClade.put("777777777777731","Unknown");
		myHClade.put("777777777777751","Unknown");
		myHClade.put("777777777777771","Manu_ancestor");
		
		if(!myHClade.containsKey(spoligotype)){
			return "ND";
		}
		else{
			return myHClade.get(spoligotype);
		}
		
	}
	
	/**
	 * Method allowing to get Country distribution from SIT (contained in SITVIT2)
	 * @param sit
	 * @return
	 * @throws Exception
	 */
	public static String getCountry(String sit) throws Exception{
		HashMap<String,String> myHCountry = new HashMap<String,String>();
		
		myHCountry.put("1","[US=1990, CN=1885, JP=1144, ZA=823, RU=691, VN=387, PE=310, IN=262, MY=238, BD=174, TW=172, ID=146, LV=123, TH=115, IT=111, FR=109, KG=103, SE=99, MM=98, SA=76, NL=66, KZ=65, KR=61, GE=58, AZ=52, EE=50, AM=49, AU=41, BE=36, ES=33, MZ=32, OM=31, MG=28, TN=27, CU=26, TZ=25, AT=25, PK=23, IR=23, DE=19, DK=18, BR=17, CZ=16, LK=14, GB=14, PL=13, TR=12, PA=11, GM=11, PH=11, GF=10, IL=10, MU=10, SN=8, NP=8, MX=8, GW=7, SG=7, SR=6, KE=6, TT=6, MA=6, RE=6, BG=5, MQ=5, FI=4, CA=4, JM=4, NZ=4, AL=4, PT=4, ZW=4, GR=3, SD=3, ET=3, AR=3, LY=3, GP=2, PS=2, VE=2, CO=1, CC=1, GY=1, BS=1, PY=1, MW=1, ZM=1, HT=1, DZ=1, HN=1]");
		myHCountry.put("2","[US=89, HT=37, AR=35, GF=24, FR=22, CU=21, BR=21, GP=16, AT=15, MX=11, CZ=11, ES=7, TR=7, PL=6, IT=6, MA=6, FI=6, BE=6, MQ=5, GM=4, NL=4, DK=3, TN=3, HN=3, ZA=3, SA=2, PA=2, CO=2, GR=2, RU=2, SE=2, DE=2, DZ=2, PY=1, IN=1, GB=1, LY=1]");
		myHCountry.put("3","[US=42, MX=13, FR=10, BR=7, AT=6, GP=4, IT=4, CO=3, NL=3, CU=3, ET=2, AR=2, MA=1, PL=1, VE=1, HT=1, SR=1, ZA=1]");
		myHCountry.put("4","[US=36, IT=33, ZA=30, TR=29, BR=29, BG=20, ET=16, FR=15, AL=15, SE=11, CA=10, BE=9, NL=8, TZ=7, ES=7, AT=7, AR=6, JP=5, VE=5, RU=4, SA=3, GP=3, GR=3, OM=3, GF=3, PY=3, PS=3, PT=3, HT=3, PE=2, MQ=2, MY=2, MZ=2, ZM=2, LY=2, CN=1, CO=1, PL=1, GW=1, FI=1, KE=1, BD=1, NO=1, TN=1, GB=1, IQ=1, IN=1, MW=1, MA=1, DE=1, MG=1]");
		myHCountry.put("5","[US=13, HT=7, GF=4, GP=3, MQ=2, FR=1]");
		myHCountry.put("6","[DK=13, NL=7, SE=7, US=7, GB=5, IN=5, OM=4, AT=3, MM=2, NZ=2, MY=1, ET=1]");
		myHCountry.put("7","[TR=14, US=9, FR=6, GP=4, ES=4, CN=3, NL=3, GB=3, GF=3, ZA=3, JP=3, PY=2, AT=2, MA=2, BE=2, BG=2, HT=2, SA=1, CM=1, PE=1, MY=1, MX=1, SD=1, DE=1, BR=1, BS=1]");
		myHCountry.put("8","[MX=34, OM=29, US=29, DK=16, MZ=12, TZ=11, GB=10, ZM=8, SA=6, ZA=6, IN=4, CO=3, NL=3, TN=3, BE=2, NO=2, TR=2, IT=1, MY=1, SE=1]");
		myHCountry.put("9","[NL=2, GB=1, US=1]");
		myHCountry.put("10","[NL=19, DK=18, GB=11, SA=10, BE=5, SE=5, FR=3, OM=3, IT=2, PS=1, ZM=1, ET=1, MG=1, YE=1, MW=1, US=1]");
		myHCountry.put("11","[IN=210, US=119, GB=45, OM=33, NL=32, BD=23, MY=17, DK=17, SA=14, LK=12, IR=10, SG=9, MM=7, BE=6, NZ=6, IT=5, FR=5, SE=4, AT=4, TN=4, GF=4, PK=3, GY=3, DE=2, BB=2, BR=2, PA=1, PE=1, GP=1, SR=1, AU=1, CU=1, MZ=1, ID=1, NP=1, JP=1]");
		myHCountry.put("12","[US=3, GP=2]");
		myHCountry.put("13","[GP=2, US=1]");
		myHCountry.put("14","[GP=27, US=6, MQ=2, GF=2, VE=1]");
		myHCountry.put("15","[GP=2]");
		myHCountry.put("16","[DK=2, GB=2, SA=1, BD=1]");
		myHCountry.put("17","[BR=184, VE=179, US=119, ES=29, HT=22, GP=20, ZA=14, CO=13, GF=12, DO=9, MX=8, PT=6, CZ=5, FR=4, NL=3, PH=3, MQ=3, BE=3, SA=2, CM=2, IT=2, MM=2, AR=2, PA=1, PY=1, TT=1, NA=1, PE=1, MY=1, CU=1, OM=1]");
		myHCountry.put("18","[BR=20, ZA=11, GB=5, NL=2, US=2]");
		myHCountry.put("19","[US=415, PH=92, MX=78, MY=43, IT=39, TW=36, SA=29, JP=24, SE=20, SG=17, AT=13, AU=11, OM=10, TH=8, FR=6, TN=6, PA=5, BE=5, NL=4, VN=3, GP=2, CN=1, PE=1, SR=1, MQ=1, FI=1, TC=1, NP=1, GB=1, JM=1, GF=1]");
		myHCountry.put("20","[US=159, BR=147, NA=62, PT=49, FR=49, VE=42, ZA=27, HT=26, ES=25, BE=19, MX=18, GF=17, MZ=15, CO=14, GM=13, NL=13, GW=11, MW=11, SE=11, DE=10, GP=8, CM=7, IT=5, CU=5, EE=4, RU=4, ZM=4, TZ=3, PE=3, JM=3, PH=2, AU=2, AR=2, ZW=2, SA=1, PY=1, MQ=1, YE=1, IQ=1, AT=1, FI=1, OM=1, TR=1, TN=1, GB=1, LY=1, VN=1, GE=1]");
		myHCountry.put("21","[TZ=89, ET=47, MG=46, US=25, ZA=23, ZM=22, SE=19, KE=17, SA=14, GB=14, NL=13, FR=13, OM=9, MZ=7, TN=6, KM=3, MW=3, ZW=3, BE=3, YE=2, DK=2, FI=2, PK=1, MQ=1, IT=1, MU=1, SD=1, AT=1]");
		myHCountry.put("22","[SA=30, IQ=20, OM=14, IR=6, TZ=3, FI=2, NL=2, BD=2, ZM=2, GB=2, IN=2, US=2, EG=1, FR=1, UG=1, YE=1, BR=1]");
		myHCountry.put("23","[GB=3]");
		myHCountry.put("24","[SD=7, TR=4, IR=3, NL=2, IN=2, ET=2, GB=1, SE=1, OM=1]");
		myHCountry.put("25","[ET=144, SA=94, SD=69, US=52, IN=49, OM=42, IQ=29, IR=17, SE=15, NL=14, GB=12, FR=9, LY=9, IT=8, EG=6, PK=6, TZ=4, BD=4, CZ=3, BE=3, ZA=3, AU=2, ID=2, TR=2, CN=1, EE=1, CF=1, SN=1, MZ=1, MU=1, AT=1, ZW=1, NP=1, GE=1]");
		myHCountry.put("26","[IN=457, US=219, PK=144, OM=109, SA=78, BD=76, IR=62, IT=48, NL=43, ET=43, GB=35, SE=21, IQ=20, FR=15, TZ=13, BE=12, NP=11, AU=9, AT=9, KE=9, TN=7, SD=6, DE=6, ZA=6, MM=5, CN=4, YE=4, MY=4, EG=2, PT=2, MU=2, ZM=2, NZ=2, TW=1, PS=1, GP=1, KM=1, MA=1, DK=1, FI=1, BB=1, MG=1, GE=1]");
		myHCountry.put("27","[GB=4, IN=4, US=4, IR=3, OM=3, DE=2, CN=1, KE=1, NP=1, AU=1, PK=1, SE=1]");
		myHCountry.put("28","[NL=4, SA=1, MY=1]");
		myHCountry.put("29","[US=18, FR=9, GP=5, IQ=2, BR=2, ES=2, YE=1, SE=1, PT=1]");
		myHCountry.put("30","[GP=2, ZA=2, US=1]");
		myHCountry.put("31","[FR=20, CZ=20, ES=10, MG=4, IT=4, MA=3, GF=3, US=3, GP=2, FI=1, BE=1, NL=1, MQ=1, AU=1, IN=1, CU=1, AT=1]");
		myHCountry.put("32","[BE=49, ID=10, NL=7, SA=6, IT=4, NZ=4, SE=4, BR=3, BD=2, ZM=2, FR=2, MY=2, GF=2, US=2, MA=1, CN=1, DE=1, IN=1]");
		myHCountry.put("33","[ZA=325, US=159, BR=132, ES=89, AR=57, PE=56, FR=54, IT=46, HN=43, MA=23, CU=15, BE=13, NL=10, MZ=10, CO=9, VE=9, PT=7, TN=7, PA=6, MX=5, AT=5, GF=5, GM=4, SA=3, MG=3, GB=3, PY=2, GP=2, SE=2, DO=2, OM=2, LY=2, FI=1, BD=1, HT=1]");
		myHCountry.put("34","[ZA=154, US=134, IT=106, CA=74, BR=55, FR=36, BG=31, BE=21, SA=16, AT=16, MA=16, SE=14, PY=14, MX=9, PT=8, VE=8, GF=7, EG=7, HT=7, ES=6, MZ=6, AR=6, DZ=6, TR=5, CO=4, GR=4, OM=4, TN=4, GB=4, NL=4, KE=3, GE=3, AL=3, ID=3, DE=3, MG=3, GP=2, FI=2, NZ=2, ET=2, IQ=2, AU=2, LY=2, VN=2, CN=1, PE=1, CU=1, SD=1, TH=1, JM=1, BS=1, MQ=1, MM=1, IR=1, IN=1, MW=1, ZM=1, DO=1, HN=1, RE=1]");
		myHCountry.put("35","[RU=30, AT=13, US=10, IR=9, TR=9, CZ=8, LV=6, SA=5, KZ=5, ET=4, IT=4, FI=4, FR=4, AM=3, NL=3, AZ=3, CN=2, EE=2, PL=2, KG=2, DE=2, GB=2, JP=2, GP=1, ES=1, SE=1, MX=1, BE=1, ZA=1]");
		myHCountry.put("36","[ZA=19, FR=17, US=9, IT=7, MA=6, NL=5, DE=5, TR=5, AT=4, LY=4, CN=3, AL=3, PE=3, ES=3, AU=3, SE=3, CZ=3, BE=3, BR=3, LV=3, PY=2, FI=2, GM=1, GR=1, ET=1, RU=1, PL=1, MQ=1, IQ=1, MZ=1, AR=1, BG=1, VE=1, TN=1, GF=1, JP=1]");
		myHCountry.put("37","[ET=77, US=40, SE=27, SA=26, CN=26, FR=22, IT=21, DK=17, ZA=17, NL=14, IN=13, CZ=12, AT=12, BR=10, OM=9, YE=9, BE=8, TR=8, PL=7, JP=6, CU=5, FI=5, CA=5, AL=5, BG=4, MZ=4, DE=4, ZW=4, GY=3, IR=3, MA=3, MG=3, CO=2, GR=2, NZ=2, ES=2, IQ=2, AU=2, VE=2, GM=1, PK=1, GP=1, RU=1, CI=1, NP=1, TN=1, GB=1, TH=1, GF=1, GE=1, EE=1, TZ=1, EG=1, NG=1, MQ=1, MM=1, ID=1, ZM=1, DO=1, HT=1, DZ=1, LY=1, LV=1]");
		myHCountry.put("38","[US=153, GB=3, IN=3, TN=2]");
		myHCountry.put("39","[ZA=51, AT=21, US=18, IT=8, DE=6, FR=6, PL=6, BR=4, SA=3, AM=2, PE=2, BE=2, NL=2, IN=2, SE=2, LV=2, CZ=2, KG=1, TN=1, GB=1, VN=1]");
		myHCountry.put("40","[US=28, IT=14, AT=13, FR=13, CZ=11, LV=9, ET=7, RU=6, DE=5, GP=4, NL=4, IN=4, DK=4, FI=4, EG=3, PL=3, GR=2, YE=2, MA=2, BG=2, MG=2, TR=2, GB=2, GF=2, CN=1, ES=1, KZ=1, SE=1, BE=1, OM=1, LY=1, ZA=1, GE=1]");
		myHCountry.put("41","[TR=212, BG=97, NL=28, ET=26, FR=17, IT=8, US=8, SA=7, AL=7, DK=7, EG=6, DE=6, GB=6, IQ=5, SE=5, AT=5, NO=4, GR=3, CZ=3, BE=3, FI=2, NZ=2, ES=1, PL=1, YE=1, SD=1, OM=1, RO=1, TN=1, GF=1, ZA=1]");
		myHCountry.put("42","[BR=401, US=397, CO=255, MA=236, IT=219, FR=170, ES=112, VE=111, ZA=106, BE=74, PE=73, RU=68, AR=63, MX=60, LV=45, PT=44, BD=43, SE=42, NL=42, PY=40, PA=38, GW=36, TN=35, MZ=34, CZ=27, HT=27, NZ=26, ZW=26, FI=25, AT=25, GP=24, GF=24, SA=21, HN=21, TR=20, CU=19, KZ=17, OM=16, ZM=16, ET=13, IN=13, TZ=12, DZ=11, GM=9, PL=9, KE=8, GB=8, MQ=8, GE=7, MM=6, DE=6, PH=5, JP=5, EE=5, NG=5, IR=5, MW=5, MG=5, BG=4, AM=4, AZ=4, DO=4, CF=3, KG=3, AU=3, CN=2, GR=2, SR=2, SN=2, TT=2, AL=2, EG=2, LY=2, CM=1, GY=1, SD=1, TH=1, JM=1, BS=1, PS=1, IQ=1, MY=1, DM=1, VN=1]");
		myHCountry.put("43","[US=8, IT=5, SE=4, DE=3, NL=3, GB=3, SA=2, LK=1, ET=1, OM=1, VN=1, MY=1, IN=1]");
		myHCountry.put("44","[AT=28, IT=26, US=22, ZA=20, FR=13, GB=11, CZ=9, GM=6, GF=6, CN=5, GP=4, PL=4, PT=4, IN=4, MZ=4, MA=4, BE=4, VE=4, NL=3, ES=3, RU=3, AR=3, BR=3, FI=2, DE=2, CM=1, EE=1, EG=1, ET=1, NG=1, GW=1, SR=1, AU=1, SE=1, DK=1, ID=1, ZW=1, BG=1]");
		myHCountry.put("45","[AT=18, MQ=12, US=10, IT=9, BR=9, CO=8, GP=8, DE=4, NL=3, LC=3, SE=3, DK=2, ES=2, RU=2, CZ=2, PA=1, FR=1, MG=1, PT=1, HT=1, TR=1, GB=1, MY=1, ZA=1]");
		myHCountry.put("46","[US=23, BR=21, FR=16, CZ=14, IT=11, ID=10, ZA=10, IN=9, FI=9, MQ=8, PA=7, PL=7, MX=6, AT=6, AR=5, SA=4, PE=4, ES=4, RU=4, SE=4, GP=3, TR=3, HN=3, DE=2, MG=2, CM=1, TZ=1, AL=1, EG=1, NL=1, ET=1, IQ=1, AU=1, MA=1, DK=1, BD=1, BG=1, GF=1]");
		myHCountry.put("47","[US=238, IT=129, AT=126, BR=110, FR=104, CZ=59, ES=56, SE=53, MA=44, PE=40, NL=39, CF=37, TR=35, FI=29, ZA=29, BG=28, BE=25, MX=25, PL=24, DE=24, PT=19, GW=18, SA=13, TN=13, CU=12, VE=11, RU=10, MG=10, AR=8, GP=7, SN=6, GB=6, GF=6, LV=6, GM=5, CA=5, AL=5, ET=5, IQ=5, CO=4, BD=4, OM=4, JM=4, AM=4, EE=4, HT=4, CN=3, PA=3, CM=3, SR=3, AU=3, GR=2, PY=2, IN=2, KG=1, JP=1, TZ=1, NG=1, MQ=1, AZ=1, YE=1, MZ=1, DK=1, ZW=1, HU=1, RO=1, HN=1]");
		myHCountry.put("48","[DK=59, BD=54, ZA=51, NL=47, IN=36, MZ=34, GB=20, SE=19, US=16, SA=15, FR=12, MY=9, OM=9, NO=9, BR=5, IT=4, MM=4, ID=4, ET=3, TH=3, VN=3, LK=2, AU=2, AT=2, SD=2, KE=2, BE=2, ZM=2, TN=2, GF=2, TW=1, PK=1, SR=1, NP=1, JP=1]");
		myHCountry.put("49","[US=26, FI=23, FR=21, SE=19, PE=12, IT=12, PT=7, AT=7, CM=5, MQ=5, AR=5, MX=4, HU=4, CF=3, PY=2, ES=2, NG=2, BE=2, SA=1, CN=1, CO=1, NL=1, RU=1, IQ=1, AU=1, MY=1, MA=1, DE=1, BG=1, OM=1, VE=1, DZ=1, TR=1, BR=1, ZA=1]");
		myHCountry.put("50","[US=595, BR=239, FR=233, AT=206, ES=184, IT=184, PE=146, ZA=137, CZ=124, SE=96, GF=81, TR=65, TN=64, MA=63, BE=60, CO=54, DE=51, CN=49, CM=48, NL=45, HT=37, AR=29, PL=28, FI=28, MX=26, SA=25, GP=23, CU=22, TW=21, MG=19, ET=18, BG=17, HU=17, PA=16, MQ=16, DO=16, GB=15, JP=15, GM=14, RU=14, VE=13, GW=12, IQ=12, GR=10, PT=10, PY=9, DZ=9, IN=8, MY=7, ID=7, SR=6, AU=6, VN=5, SD=4, NZ=4, NG=4, SN=3, OM=3, AL=3, LY=3, LK=2, CF=2, BO=2, BS=2, GE=2, AM=2, TT=2, EE=2, DK=2, ZM=2, LV=2, PK=1, KM=1, KE=1, TH=1, TZ=1, EG=1, MZ=1, RO=1, RE=1]");
		myHCountry.put("51","[US=42, BR=40, IT=34, AT=24, MY=19, HT=17, GP=12, FR=12, GF=12, VE=10, CN=7, BE=7, PT=5, AR=4, IR=3, SE=3, MX=3, TN=3, JM=3, ES=2, AU=2, MZ=2, GB=2, JP=2, TW=1, PE=1, NL=1, OM=1, MG=1, TR=1, VN=1, ZA=1]");
		myHCountry.put("52","[CN=109, US=107, SE=90, FR=83, BE=45, IT=32, JP=30, NL=27, CZ=25, CM=24, ET=24, AT=23, ZA=21, SA=20, MX=17, TR=16, PA=15, IN=12, AM=10, CF=9, RU=9, BR=9, AL=9, PL=8, GB=8, ES=8, MY=7, MG=7, DK=6, HU=6, PE=5, FI=4, KE=4, CA=4, TH=4, PY=4, BD=3, NO=3, NG=3, UG=3, MQ=3, DE=3, GM=2, PK=2, GP=2, SD=2, NZ=2, TW=2, AU=2, ID=2, ZM=2, VE=2, HT=2, DZ=2, LK=1, CO=1, CC=1, CI=1, KZ=1, CU=1, KG=1, OM=1, NP=1, TN=1, GF=1, PS=1, YE=1, IR=1, MA=1, DO=1, RO=1, LV=1]");
		myHCountry.put("53","[US=831, FR=496, IT=335, BR=322, ZA=305, TR=218, AT=215, CN=194, MX=179, PE=171, SA=157, ET=154, ES=145, SE=127, BE=119, BG=98, NL=97, MA=96, IN=92, RU=90, GF=82, GB=75, JP=70, PL=66, AR=64, EG=64, DE=56, CO=53, FI=49, VE=49, TN=46, GY=45, AL=43, GR=41, DK=41, ZM=38, GP=37, BD=34, CZ=32, GM=31, LV=30, CI=27, PT=27, ZW=27, HT=27, MG=27, MQ=24, CU=23, YE=22, OM=22, CM=21, SN=21, IR=21, JM=17, AU=17, CA=16, ID=16, HN=16, NG=16, SR=15, NZ=14, MY=14, VN=14, KZ=12, GE=12, MW=12, TH=11, PS=11, PK=11, MZ=11, GW=10, TZ=10, IQ=9, EE=9, BS=8, HU=8, AM=7, NA=7, SD=6, BB=6, PY=6, PA=6, LC=5, KG=5, LY=5, NO=4, DO=4, DZ=4, CF=3, MM=3, LK=2, KE=2, BZ=2, NP=2, TW=2, KM=1, TC=1, AZ=1, RO=1, RE=1, TT=1]");
		myHCountry.put("54","[CN=37, IN=35, EG=34, SA=20, US=20, ZA=16, IQ=14, RU=10, BR=6, KZ=4, MX=4, PL=3, MG=3, GB=3, NL=2, GW=2, SE=2, SD=2, KG=2, FR=2, OM=2, TR=2, GE=2, AM=1, PA=1, PE=1, PS=1, GR=1, GY=1, IT=1, SN=1, IR=1, KN=1, AU=1, CU=1, MZ=1, BD=1, TH=1, GF=1, LV=1]");
		myHCountry.put("55","[FR=2]");
		myHCountry.put("56","[FR=7, FI=3, CN=3, IT=3, RU=3, GP=2, PY=1, CO=1, BE=1, BR=1, US=1, LV=1]");
		myHCountry.put("57","[FR=3, US=2]");
		myHCountry.put("58","[US=47, ES=42, BR=17, FR=13, PE=8, AR=8, MA=6, CU=5, NL=4, VE=4, TN=3, BE=2, PT=2, IT=2, ZA=2, AT=2, PY=1, DZ=1, MQ=1, GF=1, SE=1, MX=1]");
		myHCountry.put("59","[ZM=106, ZW=77, ZA=74, BE=65, TZ=32, MW=16, US=16, MZ=13, OM=11, MG=9, GR=5, SE=5, BR=4, DK=3, KE=3, FR=3, GB=3, NL=2, GP=2, MU=2, NZ=2, ET=1, RU=1, PT=1, IT=1, KM=1, AT=1, RE=1]");
		myHCountry.put("60","[ZA=163, BR=58, FR=22, US=19, MA=19, IT=16, VE=15, GM=13, TN=9, SA=8, GW=8, MZ=8, CO=6, ES=6, CU=4, MX=4, RU=3, AT=3, PA=2, PY=2, PE=2, GP=2, NL=2, PT=2, SN=2, AR=2, DE=2, BE=2, DZ=2, GF=2, JP=2, TT=1, IN=1, SE=1, ZW=1, BG=1, MG=1, GB=1, HN=1]");
		myHCountry.put("61","[NG=261, CM=169, FR=82, US=80, SA=44, IT=26, CZ=22, BE=19, ES=12, GM=10, MG=9, JM=8, TT=7, CF=7, SN=7, MA=7, ZA=6, NL=5, DE=5, PA=4, MZ=4, LY=4, TZ=3, SE=3, GW=2, TN=2, GB=2, CN=1, CO=1, AL=1, GP=1, RU=1, CI=1, PT=1, MQ=1, GY=1, MY=1, AT=1, BB=1, BG=1, OM=1, VN=1, BR=1, GF=1]");
		myHCountry.put("62","[CO=186, US=68, FR=66, IT=41, ES=21, GM=17, CF=13, AT=11, SE=10, ZA=10, MG=9, BE=7, TR=7, AL=6, MQ=6, CZ=6, MA=6, VE=5, MZ=4, FI=4, GP=3, NL=3, ET=3, HU=3, DZ=3, PA=2, TZ=2, AU=2, AR=2, ZW=2, BG=2, GF=2, PY=1, LK=1, PE=1, PS=1, RU=1, GW=1, SR=1, IQ=1, CU=1, DE=1, HT=1, TN=1, LY=1, BR=1, BS=1]");
		myHCountry.put("63","[MQ=2, GP=2, ES=1, GF=1]");
		myHCountry.put("64","[BR=138, US=94, GF=22, PT=14, FR=12, PE=7, MX=6, OM=6, AT=5, BE=5, SA=4, CO=4, IN=4, SE=4, AR=4, VE=4, GM=3, CZ=3, MA=3, NL=2, ES=2, IT=2, MM=2, IQ=2, MY=2, MW=2, TT=1, PK=1, MQ=1, SN=1, AU=1, KM=1, MG=1, DZ=1, GB=1, HN=1]");
		myHCountry.put("65","[BR=41, US=13, HT=12, FR=9, ES=5, AT=4, CM=3, DE=2, BE=2, GP=2, SE=2, CZ=2, MA=1, RU=1, DZ=1, MQ=1, IT=1, KZ=1, GF=1, ZA=1, JP=1]");
		myHCountry.put("66","[IR=7, GF=2]");
		myHCountry.put("67","[US=24, HN=18, IR=14, GF=7, GP=3, HT=1, PA=1, IT=1, MX=1, LV=1]");
		myHCountry.put("68","[AR=26, ES=9, CZ=8, FR=6, IT=5, AT=4, PA=3, DE=3, NL=3, SE=2, US=2, LK=1, BE=1, BB=1, MQ=1, GB=1]");
		myHCountry.put("69","[PH=2]");
		myHCountry.put("70","[US=70, MX=9, HT=8, BS=8, GF=6, GP=3, ZA=3, PH=2, TN=2, GB=2, TT=1, TR=1, MZ=1]");
		myHCountry.put("71","[ZA=31, IT=18, FR=12, CU=5, US=4, VE=3, BE=2, CA=2, MZ=2, MA=1, NL=1, ES=1, IN=1, HN=1, AR=1]");
		myHCountry.put("72","[US=12, GF=11, GY=4, MM=4, BB=2, OM=2, NP=1, IT=1, GP=1, IN=1]");
		myHCountry.put("73","[IT=37, US=36, ZA=28, FR=20, CN=12, BR=12, MZ=10, MX=8, GF=7, NL=6, GW=6, ET=5, AT=5, BE=5, BG=5, SA=4, AL=4, SE=4, VN=4, EG=3, PT=3, MG=3, GE=3, RU=2, SR=2, HN=2, TT=1, GM=1, PE=1, GP=1, MQ=1, MM=1, AU=1, IN=1, MA=1, FI=1, KG=1, ID=1, ZM=1, VE=1, HT=1, NP=1]");
		myHCountry.put("74","[MG=6, BR=5, CU=3, MQ=1, FR=1]");
		myHCountry.put("75","[IR=7, FR=7, BR=7, IQ=6, CM=5, US=5, AR=3, MG=3, TR=3, GF=3, SA=2, GP=2, ES=2, IT=2, RO=2, GM=1, NG=1, PT=1, GW=1, MZ=1, MX=1, DE=1, HU=1, HT=1, JP=1]");
		myHCountry.put("76","[GF=2, TN=1]");
		myHCountry.put("77","[GF=7, US=5, HT=4, MA=2, GP=2, BS=1]");
		myHCountry.put("78","[MG=13, IN=7, TR=6, US=5, SA=3, GB=3, CN=2, ES=2, IT=2, SE=2, AT=2, CZ=2, MA=2, FR=2, BR=2, GF=2, PE=1, GM=1, NL=1, MM=1, MZ=1, MX=1, FI=1, BG=1, OM=1, ZA=1]");
		myHCountry.put("79","[ZW=2]");
		myHCountry.put("80","[CU=8]");
		myHCountry.put("81","[CU=16, ES=6, BE=3, BR=3, IT=2, MX=2, ZW=1, ZA=1, AR=1, FR=1]");
		myHCountry.put("82","[ZW=4]");
		myHCountry.put("83","[ZW=2, CN=1, IT=1]");
		myHCountry.put("84","[ZW=2]");
		myHCountry.put("85","[ZW=2]");
		myHCountry.put("86","[MG=34, TR=14, FR=11, CN=5, IT=5, US=5, BR=4, ES=3, PE=2, SE=2, AT=2, CO=1, EG=1, NL=1, VE=1, IQ=1, GF=1]");
		myHCountry.put("87","[ZW=2]");
		myHCountry.put("88","[FR=8, GF=2, DZ=1, IT=1, GP=1, CU=1, SE=1]");
		myHCountry.put("89","[TH=36, US=21, MM=10, NL=5, SE=5, SA=4, MY=4, SR=3, ID=2, FR=2, TN=2, VN=2, TW=1, GP=1, OM=1, IT=1, AU=1, SG=1, GF=1, JP=1]");
		myHCountry.put("90","[IT=10, ES=8, PL=4, BG=3, AT=2, TR=1, AL=1, NL=1]");
		myHCountry.put("91","[US=118, HT=29, PE=25, ES=16, CO=12, CA=8, GF=8, GB=6, DK=5, BR=4, AR=3, BE=2, GP=2, NL=2, DO=2, VE=2, TN=2, MX=2, FR=1, MQ=1]");
		myHCountry.put("92","[ZA=205, US=84, BR=25, MX=18, GB=14, MZ=9, NL=8, GF=8, PE=5, IT=5, HN=5, IN=4, DK=4, FR=3, TR=3, JP=3, GW=2, AU=2, CU=2, SE=2, ZM=2, NZ=2, TW=1, PY=1, ES=1, RU=1, MW=1, AR=1, AT=1, ZW=1, BD=1, NO=1, TN=1, BS=1]");
		myHCountry.put("93","[VE=94, US=75, BR=43, IT=21, GP=18, HT=15, PE=13, ES=11, FR=11, GF=9, MA=7, DO=7, PY=6, TW=3, DE=3, BE=3, CO=2, AR=2, TN=2, TT=1, NL=1, ET=1, GW=1, AU=1, SE=1, MX=1, OM=1, ZA=1]");
		myHCountry.put("94","[GF=6]");
		myHCountry.put("95","[BR=21, US=8, GF=5, PY=2, PE=2, MX=2, IT=1, GB=1, MZ=1, SE=1, ZA=1, FR=1]");
		myHCountry.put("96","[BD=39, GF=4, US=2]");
		myHCountry.put("97","[IR=2, NL=2, SE=2, BE=1]");
		myHCountry.put("98","[NL=3, DE=1]");
		myHCountry.put("99","[IR=11, US=9, IT=8, FR=7, MG=7, ES=4, NL=3, BR=3, CZ=3, AT=3, PE=2, GP=2, JP=2, FI=1, DE=1, DZ=1, RO=1, TN=1, IN=1, SE=1, ZA=1, AR=1]");
		myHCountry.put("100","[IN=31, SA=18, US=8, GB=6, ZA=3, MX=3, OM=2, MM=2, MZ=2, FR=1, MG=1, TH=1, VN=1, MY=1, SE=1]");
		myHCountry.put("101","[CM=8, US=6, FR=6, GP=2, NG=1]");
		myHCountry.put("102","[US=13, BR=7, IT=6, ID=5, BE=5, ZA=5, FR=3, AU=3, MY=3, AR=3, MA=2, DE=2, GP=2, RU=2, MX=2, PY=1, CO=1, ZM=1, OM=1, IQ=1, IN=1, GF=1, KR=1, MU=1]");
		myHCountry.put("103","[GP=3]");
		myHCountry.put("104","[ES=2]");
		myHCountry.put("105","[US=17, ES=10, BR=5, IT=3, MA=2, GM=2, FR=2, DZ=1, BE=1, NL=1]");
		myHCountry.put("106","[ES=48, US=23, PA=19, BR=9, CO=5, FR=5, AR=5, MA=4, DZ=3, PE=2, BE=2, VE=2, IT=2, AU=1, RE=1]");
		myHCountry.put("107","[FR=5, AU=1, ZA=1]");
		myHCountry.put("108","[FR=2]");
		myHCountry.put("109","[MG=59, SA=31, FR=8, OM=7, DK=1, YE=1, NL=1, IN=1, GR=1, SE=1, US=1, SD=1]");
		myHCountry.put("110","[FR=2]");
		myHCountry.put("111","[AR=6, ES=4, MZ=4, FR=4, BR=3, US=3, HN=2, ZA=2, PE=1, MX=1, PT=1]");
		myHCountry.put("112","[FR=6]");
		myHCountry.put("113","[IR=10, FR=3, MM=1, NL=1]");
		myHCountry.put("114","[FR=2]");
		myHCountry.put("115","[CM=11, FR=8, NG=3, SA=1, IT=1, BB=1]");
		myHCountry.put("116","[FR=2, BR=1]");
		myHCountry.put("117","[FR=6, CM=3, SA=2, CF=2, IQ=2, DK=1, CN=1, ES=1, RO=1, TN=1, BR=1, MZ=1, ZA=1, AR=1, JP=1]");
		myHCountry.put("118","[FR=24, TR=19, IT=10, CN=9, VE=9, US=8, MZ=7, IQ=6, CZ=6, BR=6, JP=6, AR=5, MX=5, SA=2, EG=2, AL=2, CF=2, PT=2, IN=2, DK=2, HN=2, ZA=2, CM=1, PS=1, ES=1, PL=1, MQ=1, KZ=1, AZ=1, SE=1, AT=1, FI=1, KG=1, ID=1, DE=1, BE=1, MG=1, GF=1]");
		myHCountry.put("119","[US=630, ZA=143, MX=104, GB=28, BR=24, IN=17, NL=16, IT=12, CO=11, SA=7, CA=7, BE=6, PE=5, ES=5, FR=5, BG=4, TN=4, LV=4, TT=3, GR=3, MQ=3, DO=3, CN=2, PA=2, AL=2, GP=2, GW=2, SR=2, AT=2, NO=2, TR=2, JM=2, ET=1, RU=1, KZ=1, MZ=1, MW=1, FI=1, DK=1, ZW=1, VC=1, HT=1, LY=1, HN=1, GF=1]");
		myHCountry.put("120","[FR=5, AT=2, MA=1, MX=1]");
		myHCountry.put("121","[TW=19, ET=7, FR=6, CZ=4, SA=3, US=3, PE=2, FI=1, CN=1, ES=1, IT=1, GB=1, BR=1, SE=1]");
		myHCountry.put("122","[FR=6, GR=3, GM=2, GP=2, ES=2, US=2, KE=1, PA=1, KG=1, CO=1, NL=1, ZM=1, IT=1, VN=1, MY=1, AT=1]");
		myHCountry.put("123","[AT=8, TR=4, BD=4, OM=3, BG=2, GP=1, NL=1, GR=1, SE=1, US=1, FR=1]");
		myHCountry.put("124","[IN=11, US=9, LK=6, CN=5, CO=5, SA=4, NL=4, FR=4, BR=3, DE=2, SE=2, FI=1, TW=1, EG=1, PE=1, BE=1, GP=1, ES=1, RU=1, OM=1, MQ=1, IQ=1, AT=1]");
		myHCountry.put("125","[BG=74, NL=6, BR=4, ZA=4, FR=3, IT=3, PY=2, BE=2, US=2, JP=2, SA=1, CA=1, IR=1, IN=1, MZ=1, GF=1]");
		myHCountry.put("126","[IN=34, OM=17, GB=15, SA=8, LK=7, NL=7, US=6, TZ=4, BD=4, MM=4, DK=3, TT=1, BE=1, MG=1, IT=1, SN=1, MY=1, GF=1, SE=1, ZA=1]");
		myHCountry.put("127","[IR=96, NL=15, SA=12, CN=10, IQ=10, SE=9, AT=9, DE=7, IN=5, US=5, KG=4, PK=4, OM=3, IT=2, GB=2, RU=1, FR=1, NP=1, TR=1, AU=1]");
		myHCountry.put("128","[ZA=9, FR=5, DE=4, TZ=3, NL=3, ZM=3, BE=2, TN=1, ES=1, OM=1]");
		myHCountry.put("129","[MZ=15, GF=8, US=5, MW=4, ZA=3, TN=2, TZ=1, DE=1, ZW=1, NL=1, ZM=1, ES=1, FR=1, MG=1, GW=1, BR=1]");
		myHCountry.put("130","[US=32, ZA=25, CO=10, PE=8, ES=6, VE=5, BR=5, IT=4, HN=3, FR=2, MX=2, BE=1, NL=1, MG=1, CU=1, AT=1]");
		myHCountry.put("131","[GF=33, US=15, GY=13, SR=11, CN=9, IT=3, FI=2, MQ=2, AM=1, BE=1, GP=1, BD=1, NL=1, ET=1, BB=1, GR=1, FR=1, PL=1, TR=1, AR=1, AT=1]");
		myHCountry.put("132","[IR=5, US=3, VE=2, IT=1, PE=1, NL=1, BR=1, CU=1, MX=1]");
		myHCountry.put("133","[MQ=2, GP=2, NL=1]");
		myHCountry.put("134","[FR=21, ET=9, US=9, IT=5, SE=2, MQ=1, YE=1, NL=1, BR=1, GF=1, SD=1]");
		myHCountry.put("135","[ZA=6, US=6, DE=4, NL=3, AT=3, SA=2, FR=2, TW=1, UG=1, KE=1, GB=1, MY=1]");
		myHCountry.put("136","[DZ=10, FR=8, ZA=7, NL=3, GF=3, ID=2, ES=2, CF=2, GB=2, BE=1, GP=1, MG=1, MZ=1, US=1]");
		myHCountry.put("137","[US=732, ZA=65, GB=55, BR=46, BE=16, DK=15, CA=11, ES=10, NL=9, CM=7, GR=6, MX=6, SA=5, AU=5, IN=5, BD=4, GF=4, IT=3, SE=3, ZM=3, FR=3, MM=2, MQ=1, MY=1, BB=1, HT=1, NP=1, BZ=1, TN=1, LY=1]");
		myHCountry.put("138","[BD=35, IN=22, US=17, DK=9, OM=8, GB=8, NL=7, SR=3, IT=3, NP=2, FI=1, SA=1, TT=1, LK=1, GP=1, MM=1, TN=1, MY=1, SE=1, GE=1]");
		myHCountry.put("139","[VN=181, US=90, DK=32, SE=9, NL=4, AU=3, NZ=2, SA=1, FI=1, TN=1, GB=1, SG=1, IN=1, JP=1]");
		myHCountry.put("140","[FR=2, ET=1]");
		myHCountry.put("141","[IN=11, IQ=9, US=4, FR=3, SA=1, MM=1, PK=1, NL=1, MZ=1]");
		myHCountry.put("142","[US=7, SA=6, ET=5, IN=5, PK=2, BD=2, FR=2, SE=2, DE=1, NL=1, OM=1, IT=1, GB=1, ZA=1, SD=1]");
		myHCountry.put("143","[FR=6, MA=5, ZA=4, IT=2, NL=2, DZ=1, TR=1, ES=1]");
		myHCountry.put("144","[BE=16, GM=15, FR=8, BG=6, US=4, IT=2, TN=1, DE=1, GB=1, NL=1, PT=1]");
		myHCountry.put("145","[FR=3]");
		myHCountry.put("146","[FR=2]");
		myHCountry.put("147","[IR=6, FR=3]");
		myHCountry.put("148","[FR=2, IR=1]");
		myHCountry.put("149","[ET=271, DK=57, SA=32, US=23, SE=22, NL=17, GB=14, FR=10, IT=5, ES=3, BE=2, NO=2, YE=2, FI=1, DE=1, LY=1, ZA=1, AT=1]");
		myHCountry.put("150","[BE=22, BR=11, MZ=11, PT=9, FR=8, IT=6, AR=6, VE=5, RU=4, MA=2, HN=2, GF=2, FI=1, TZ=1, EG=1, ES=1, DZ=1, TR=1, ZA=1, MX=1, US=1]");
		myHCountry.put("151","[TR=4, FR=3, CF=2, CU=2, MA=1, SA=1, EG=1, ES=1, ET=1, PL=1, PT=1, HT=1, IT=1, BR=1, SE=1, ZA=1, AR=1, AT=1]");
		myHCountry.put("152","[US=22, VN=8, FR=6, NL=2, CN=1, IT=1, TN=1, GF=1, SE=1]");
		myHCountry.put("153","[IT=21, SE=14, BR=10, US=7, CN=6, CU=5, FI=4, FR=3, KG=1, NL=1, ET=1, AT=1]");
		myHCountry.put("154","[IT=29, BG=27, AT=12, US=9, FR=8, ES=4, GR=4, DE=3, TR=3, AR=3, ZA=2, FI=1, CN=1, NL=1, ET=1, RU=1, HU=1, AU=1]");
		myHCountry.put("155","[IT=2]");
		myHCountry.put("156","[MG=24, IT=7, BR=7, US=5, RU=4, PE=3, FR=3, ES=2, BE=1, AT=1]");
		myHCountry.put("157","[BR=22, PT=4, IT=3, SA=1, DE=1]");
		myHCountry.put("158","[ZA=5, IT=2, BR=1]");
		myHCountry.put("159","[IT=32, AR=9, AT=8, PY=4, US=3, EE=2, ET=2, CN=1, DE=1, BE=1, NL=1, RU=1, FR=1, KZ=1, YE=1, AU=1, BR=1, MX=1, LV=1]");
		myHCountry.put("160","[US=5, IT=3, ES=3, BE=2, PA=1, NL=1, CU=1]");
		myHCountry.put("161","[BR=4, IT=3, MA=1, BE=1, NL=1, RU=1, US=1]");
		myHCountry.put("162","[CO=8, VE=5, IT=4, US=3, FR=3, SA=2, FI=1, MA=1, PY=1, GY=1, ES=1, AR=1]");
		myHCountry.put("163","[US=15, IT=7, HT=3, MA=2, MX=2, BR=1, ES=1, FR=1]");
		myHCountry.put("164","[IT=28, MA=3, US=3, FR=3, AT=2, HU=1, BE=1, NL=1, GF=1, BG=1, MG=1]");
		myHCountry.put("165","[IT=3]");
		myHCountry.put("166","[IT=8, US=1]");
		myHCountry.put("167","[ES=17, FR=9, US=9, SA=5, NL=5, SE=4, BE=3, VE=3, BR=3, AT=3, FI=2, CN=1, CO=1, EG=1, ID=1, OM=1, TR=1, IT=1, GB=1, CU=1, SD=1]");
		myHCountry.put("168","[NL=4, HT=4, US=4, MX=2, CN=1, CO=1, GP=1, NG=1, IT=1, MY=1, GF=1, ZA=1, GE=1, AT=1]");
		myHCountry.put("169","[IT=8, GP=1]");
		myHCountry.put("170","[FR=2]");
		myHCountry.put("171","[FR=2, IT=1, BR=1]");
		myHCountry.put("172","[US=8, AT=7, CN=6, FR=5, IT=5, SA=2, DE=2, BG=2, HT=2, TR=2, BR=2, JP=2, FI=1, BE=1, PK=1, NL=1, GR=1, ET=1, RU=1, IN=1, MY=1, SE=1]");
		myHCountry.put("173","[TR=8, FR=4, US=3, BR=1]");
		myHCountry.put("174","[FR=12, BR=3, US=3, OM=2, HT=1, VN=1, HN=1, RU=1]");
		myHCountry.put("175","[AR=8, IT=4, HT=2, FR=2, SR=1, PE=1, BR=1, ZA=1, MX=1, US=1]");
		myHCountry.put("176","[US=14, BR=10, GF=5, MX=4, SR=1, CF=1, ES=1, HN=1]");
		myHCountry.put("177","[BR=52, FR=10, ES=9, IT=5, MA=3, SA=3, PY=3, US=3, CO=2, EG=2, ZA=2, TW=1, PE=1, VE=1, MM=1, MY=1, GF=1, AR=1]");
		myHCountry.put("178","[VE=2, SN=1, CU=1]");
		myHCountry.put("179","[BR=15, GF=1, US=1, PT=1]");
		myHCountry.put("180","[ES=18, CF=6, FR=4, IT=4, CZ=4, SA=3, PL=3, US=3, AT=3, ZA=2, MA=1, AL=1, NL=1, RU=1, CI=1, VN=1, MY=1, BR=1, SE=1]");
		myHCountry.put("181","[GW=98, GM=84, US=21, FR=16, IT=13, NL=11, GB=8, SN=7, NG=3, BE=2, ES=2, GF=2, DK=1, GN=1, DE=1, TN=1, SL=1, SE=1]");
		myHCountry.put("182","[GM=2, SN=1, NL=1]");
		myHCountry.put("183","[ES=17, IT=8, GM=6, US=4, CZ=3, MA=2, FR=2, SN=2, SE=2, ZA=2, KG=1, DE=1, HU=1]");
		myHCountry.put("184","[ZW=9, ZM=3, FR=2, BE=1]");
		myHCountry.put("185","[US=4, DK=3, FR=2, IT=1, GP=1, BR=1]");
		myHCountry.put("186","[TR=8, FR=2, EG=1, IQ=1]");
		myHCountry.put("187","[GW=47, GM=16, US=11, FR=11, TN=5, BE=2, IT=1, SN=1, GN=1, GB=1, MG=1]");
		myHCountry.put("188","[GW=3, GB=1]");
		myHCountry.put("189","[US=3, PE=2, CU=2, FI=1, CN=1, ES=1, SE=1]");
		myHCountry.put("190","[US=59, CN=44, CO=29, JP=12, KR=6, RU=5, TH=5, IN=3, LV=3, PE=2, FR=2, VN=2, MY=2, GE=2, TW=1, ID=1, NL=1, OM=1, TN=1, CU=1, MZ=1]");
		myHCountry.put("191","[IT=4, US=4, RU=2, TN=1, GP=1, NL=1, BR=1, SE=1, AR=1, GE=1, BG=1, AT=1]");
		myHCountry.put("192","[US=2]");
		myHCountry.put("193","[US=9, HT=8, GF=6, ES=1, DO=1]");
		myHCountry.put("194","[US=9, VE=7, IN=5, BR=4, MX=2, OM=1]");
		myHCountry.put("195","[US=4, BR=3, BE=2, ES=2, GF=2, GP=1, FR=1]");
		myHCountry.put("196","[TR=15, BR=10, US=5, CN=4, MG=4, IT=4, SE=4, AT=4, NL=3, DZ=3, ZA=3, ES=2, FR=2, VE=2, GW=2, TN=2, TW=1, ET=1, RU=1, BG=1, HT=1, MQ=1, GE=1, SD=1]");
		myHCountry.put("197","[US=338, IN=3, TN=2, GB=1, ES=1, VC=1]");
		myHCountry.put("198","[US=29]");
		myHCountry.put("199","[US=13, MX=2, BR=1, BB=1]");
		myHCountry.put("200","[US=18, GB=5, GM=2, MW=2, NO=1, AU=1, IN=1, ZA=1, MX=1, NG=1, CI=1, OM=1]");
		myHCountry.put("201","[US=2]");
		myHCountry.put("202","[US=4]");
		myHCountry.put("203","[US=7, SA=2, PK=2, IN=2, SE=2, FR=2, GM=1, OM=1]");
		myHCountry.put("204","[US=7, TH=1, VN=1, ET=1, FR=1]");
		myHCountry.put("205","[BR=9, US=9, FR=7, SA=2, CN=2, TZ=2, BG=2, TR=2, IT=2, KE=1, DE=1, ZM=1, RU=1, VE=1, HT=1, GY=1, YE=1, TN=1, GB=1, AU=1, VN=1, MY=1, SE=1, MX=1]");
		myHCountry.put("206","[US=7, HN=6, BE=1]");
		myHCountry.put("207","[CO=9, US=9, MX=5, BR=2, AT=2, SA=1, MQ=1, PE=1, FR=1]");
		myHCountry.put("208","[US=2]");
		myHCountry.put("209","[ES=20, FR=6, IT=3, CU=3, BE=2, US=2, MA=1, TN=1, BR=1, SE=1, MX=1]");
		myHCountry.put("210","[US=123, FR=3, CN=1, CO=1]");
		myHCountry.put("211","[US=32, MX=22, FR=20, BR=10, ES=5, PT=3, PY=1, PE=1, AU=1, NL=1, ZA=1, AR=1, MG=1]");
		myHCountry.put("212","[CA=7, US=5, AT=1]");
		myHCountry.put("213","[US=12, IT=3, MY=1]");
		myHCountry.put("214","[US=46]");
		myHCountry.put("215","[ES=4, US=2, PE=1]");
		myHCountry.put("216","[BR=8, US=5, IT=2, AR=1, VE=1]");
		myHCountry.put("217","[US=18, MX=7, CO=3, PA=2, BR=2, IT=1, AU=1, GB=1]");
		myHCountry.put("218","[ZA=10, US=7, FR=3, SE=3, AT=3, MA=2, ES=2, ID=1, ZW=1, BE=1, NL=1, RU=1, HN=1]");
		myHCountry.put("219","[US=31, PE=28]");
		myHCountry.put("220","[US=7]");
		myHCountry.put("221","[US=9, LY=1, ZA=1]");
		myHCountry.put("222","[PE=25, US=17, ES=3, IT=2, SE=2, CO=1, MX=1]");
		myHCountry.put("223","[US=6]");
		myHCountry.put("224","[US=9]");
		myHCountry.put("225","[US=3, BE=2, FR=1]");
		myHCountry.put("226","[ZA=4, US=2, BE=1, KR=1]");
		myHCountry.put("227","[US=4]");
		myHCountry.put("228","[US=5]");
		myHCountry.put("229","[US=2]");
		myHCountry.put("230","[US=15, GW=1, HT=1, CO=1]");
		myHCountry.put("231","[US=2, CN=1, IT=1]");
		myHCountry.put("232","[MX=4, US=4, CN=1, NL=1]");
		myHCountry.put("233","[US=11]");
		myHCountry.put("234","[US=9, VN=4, BD=2, DK=1, MM=1, AU=1, NL=1, SE=1, FR=1]");
		myHCountry.put("235","[US=2]");
		myHCountry.put("236","[US=46, IN=18, BD=15, TH=12, SA=9, MY=7, MM=6, FR=6, AU=5, VN=5, DK=4, OM=4, NL=3, SE=3, GB=3, MX=2, NZ=2, GE=2, PK=1, GW=1, IT=1, SN=1, MZ=1, ID=1, ZW=1, BE=1, MG=1, NO=1]");
		myHCountry.put("237","[US=13, RU=10, AT=10, IT=9, DE=8, PL=7, CM=5, PE=5, ID=5, FR=5, BE=4, SE=3, BG=3, MG=3, BR=3, GB=2, ZA=2, LV=2, LK=1, CO=1, AL=1, PT=1, SR=1, KZ=1, FI=1, TR=1, GF=1]");
		myHCountry.put("238","[US=9, FR=5, BR=2, TW=1]");
		myHCountry.put("239","[US=32, MX=12, VE=3, PE=2, BD=2, BR=2]");
		myHCountry.put("240","[BR=11, US=10, IT=2, DE=2, TW=1, CN=1, CU=1, GF=1, SE=1, AT=1]");
		myHCountry.put("241","[US=8, SE=6, BE=3, FR=3, FI=2, IT=1]");
		myHCountry.put("242","[HT=2, CN=1, AL=1, ES=1, GF=1, AR=1, US=1]");
		myHCountry.put("243","[US=6, MY=5, BE=3, ES=2, DK=1, IT=1, ZM=1, BR=1, KR=1, FR=1]");
		myHCountry.put("244","[ZA=19, PT=16, BD=14, FR=13, BR=13, ZM=8, GW=7, US=5, MZ=3, ET=2, HT=2, IN=2, SE=2, MA=1, TZ=1, BE=1, NL=1, ES=1, GR=1, NG=1, MG=1, AR=1]");
		myHCountry.put("245","[BR=3, ZM=2, FR=2, IT=2, US=2, EG=1, ZW=1, NL=1, ES=1, VE=1, TN=1, GF=1, AR=1, AT=1]");
		myHCountry.put("246","[US=10, CN=5, NL=2, MY=2, SA=1, TN=1, TH=1]");
		myHCountry.put("247","[ET=11, SA=10, IQ=8, IN=2, US=2, TZ=1, PK=1, FR=1, SD=1]");
		myHCountry.put("248","[VN=2, US=2]");
		myHCountry.put("249","[US=9, SA=3, VN=3, NZ=2, SE=2, CN=1, AU=1, NL=1, MY=1]");
		myHCountry.put("250","[US=11, CN=8, TW=5, IN=3, ID=2, VN=2, AU=1, BD=1, MY=1, FR=1, JP=1]");
		myHCountry.put("251","[RU=6, MA=3, US=2, FI=1, KG=1, NL=1, ZA=1]");
		myHCountry.put("252","[RU=40, LV=5, SA=4, US=4]");
		myHCountry.put("253","[ID=4, PL=4, AR=4, RU=3, NL=2, FR=2, BR=2, US=2, SA=1, FI=1, DK=1, TW=1, ES=1, VE=1, TR=1, KZ=1]");
		myHCountry.put("254","[RU=62, LV=25, KZ=12, US=11, AT=10, ZA=9, GE=9, TR=6, IT=3, KG=3, BR=3, SA=2, AZ=2, SE=2, DE=2, TW=1, PY=1, TZ=1, CO=1, NL=1, ES=1, PL=1, DK=1, FI=1, BE=1, FR=1, GF=1, JP=1]");
		myHCountry.put("255","[RU=6, US=6, CN=4, JP=4, IT=3, TH=3, MY=3, CO=1, BE=1, GP=1, FR=1, MG=1, MM=1, VN=1, RE=1, SE=1]");
		myHCountry.put("256","[MY=18, US=12, TH=10, SA=3, MM=2, ID=2, DK=1, AU=1, NL=1, IN=1, SG=1, JP=1]");
		myHCountry.put("257","[DK=9]");
		myHCountry.put("258","[US=6, MX=2]");
		myHCountry.put("259","[MQ=3, GP=2]");
		myHCountry.put("260","[US=4, RU=2, CN=1, KG=1, JP=1]");
		myHCountry.put("261","[RU=2]");
		myHCountry.put("262","[RU=38, IT=34, LV=12, AT=12, TR=11, FI=8, US=8, BE=6, BG=6, KZ=6, KG=5, IR=5, GE=5, ES=4, ET=4, FR=4, PL=4, SE=4, EE=3, AM=2, NL=2, CZ=2, SA=1, AZ=1]");
		myHCountry.put("263","[RU=4, FI=1, CN=1]");
		myHCountry.put("264","[RU=19, PL=10, GE=8, US=5, LV=5, CN=1, TR=1, KG=1, DE=1]");
		myHCountry.put("265","[RU=19, US=18, CN=12, ES=5, IL=5, FR=4, JP=4, NL=2, BG=2, TH=2, KR=2, SE=2, IT=1, AZ=1, VN=1, MY=1]");
		myHCountry.put("266","[RU=11, US=2, DZ=1, LV=1]");
		myHCountry.put("267","[GE=4, US=3, IT=2, DE=2, LY=2, RU=2, BE=1]");
		myHCountry.put("268","[PE=2, MQ=1, BR=1, ES=1, RU=1, US=1]");
		myHCountry.put("269","[US=33, RU=10, JP=10, CN=7, PE=6, KR=6, TR=4, SA=2, DE=2, FR=2, HT=2, IT=2, TN=2, ID=1, KZ=1, IR=1, VN=1, IN=1, GE=1, MX=1]");
		myHCountry.put("270","[IN=3, NL=2, MY=1, US=1, OM=1]");
		myHCountry.put("271","[NL=2]");
		myHCountry.put("272","[US=5, IT=3, TR=2, ES=2, EG=1, BE=1]");
		myHCountry.put("273","[IT=4, FR=1, VE=1]");
		myHCountry.put("274","[AT=2, SA=1, IT=1, BR=1, GF=1, RU=1, US=1]");
		myHCountry.put("275","[IT=4, CN=3, BR=2, BE=1, CU=1, US=1]");
		myHCountry.put("276","[US=17, AT=5, GP=2, IN=2, SE=2, FR=2, CZ=2, IT=1, NL=1, ES=1, PL=1]");
		myHCountry.put("277","[IT=7, FR=2]");
		myHCountry.put("278","[CN=2, IT=2, IN=2, US=2, FI=1, SN=1, BR=1, ES=1]");
		myHCountry.put("279","[MY=3, AU=2]");
		myHCountry.put("280","[US=15, PL=9, FI=7, RU=7, SE=6, BG=5, AU=4, DE=3, AT=3, ES=2, TR=2, KG=1, EE=1, KZ=1, IT=1, ZA=1, GE=1, LV=1]");
		myHCountry.put("281","[SA=5, AU=4, GR=4, NL=3, FR=3, EG=2, BE=2, ES=2, CN=1, TZ=1, PK=1, MX=1]");
		myHCountry.put("282","[AU=2]");
		myHCountry.put("283","[LV=14, IN=7, AT=7, US=6, FR=5, SE=4, NL=3, AU=3, CZ=3, IT=2, AR=2, AL=1, PE=1, BD=1, ES=1, RU=1, OM=1, BR=1]");
		myHCountry.put("284","[BG=40, TR=35, SA=23, IQ=16, SE=11, AT=10, FR=6, BE=5, DE=4, GR=4, IR=4, US=4, NL=3, OM=3, ZA=2, FI=1, DK=1, PK=1, ES=1, RU=1, IT=1, YE=1, AU=1, MX=1]");
		myHCountry.put("285","[CN=1, IT=1, AL=1, AU=1, TH=1, FR=1]");
		myHCountry.put("286","[PH=2, US=2, SA=1, IT=1, ID=1, AU=1, GP=1, OM=1]");
		myHCountry.put("287","[US=7, MY=3, PH=2, MX=2, IR=1, MM=1, AU=1, NL=1, FR=1]");
		myHCountry.put("288","[IN=50, US=24, OM=11, IR=11, NL=8, BD=4, GB=3, SA=2, TZ=2, DE=2, PK=2, TT=1, BE=1, NP=1, AU=1, ZA=1]");
		myHCountry.put("289","[PK=2, NL=2, SA=1, IT=1, AU=1, BD=1, IN=1, ET=1, US=1]");
		myHCountry.put("290","[US=7, NL=5, ID=4, SA=3, ES=3, GF=2, AU=1, GP=1, MY=1, MX=1, FR=1, OM=1]");
		myHCountry.put("291","[BR=17, IT=6, ZW=6, GE=6, EG=5, ES=4, MX=3, AR=3, CZ=3, BD=3, JP=3, CN=2, PE=2, PL=2, MA=2, TR=2, ZA=2, SA=1, NA=1, CO=1, NL=1, RU=1, AU=1, IN=1, US=1, ID=1, BE=1, FR=1, MG=1, VE=1, HT=1, VN=1, HN=1]");
		myHCountry.put("292","[BD=82, MM=28, US=15, SA=4, MY=3, OM=3, FI=1, TW=1, IT=1, AU=1, IN=1, GE=1]");
		myHCountry.put("293","[US=4, CN=3, FR=2, AU=1, BE=1, BR=1, PT=1]");
		myHCountry.put("294","[US=5, DE=3, HN=3, CN=2, RU=2, TR=2, IT=2, BR=2, ID=1, BE=1, NL=1, FR=1, HT=1, GB=1, MY=1, AT=1]");
		myHCountry.put("295","[US=9, IT=6, HT=5, DO=2, GP=1]");
		myHCountry.put("296","[IR=9, DK=2]");
		myHCountry.put("297","[DK=2, MG=2, SA=1]");
		myHCountry.put("298","[DK=23, IR=14, IN=2, US=1, OM=1]");
		myHCountry.put("299","[DK=12, NL=7, IN=5, US=3, NZ=2, MY=1, SG=1]");
		myHCountry.put("300","[DK=2]");
		myHCountry.put("301","[DK=3]");
		myHCountry.put("302","[DK=18, SA=4, KE=4, IT=2, GB=2, NO=1, NL=1, MG=1]");
		myHCountry.put("303","[DK=2]");
		myHCountry.put("304","[US=3]");
		myHCountry.put("305","[US=2]");
		myHCountry.put("306","[BE=7, US=7, FR=3, GB=2, CF=2, BR=1, GF=1]");
		myHCountry.put("307","[CF=4, SA=2, IT=1, NG=1]");
		myHCountry.put("308","[CM=4, CF=2, US=1, FR=1]");
		myHCountry.put("309","[IQ=17, CF=2, SA=1, ET=1]");
		myHCountry.put("310","[CF=3]");
		myHCountry.put("311","[CF=2]");
		myHCountry.put("312","[CF=6, CM=2, FR=1]");
		myHCountry.put("313","[CF=2]");
		myHCountry.put("314","[CF=4, FR=2]");
		myHCountry.put("315","[CF=6, US=3, FR=2, TR=1, IT=1, CO=1, ES=1, MG=1]");
		myHCountry.put("316","[CF=13, NG=9, CM=5, FR=5, SA=3, SD=3, US=2, TW=1, CO=1, MG=1, CI=1, TH=1, LY=1, SE=1]");
		myHCountry.put("317","[CM=10, FR=4, BE=3, CF=2, ZM=1]");
		myHCountry.put("318","[GM=3, GW=2, CI=1]");
		myHCountry.put("319","[US=7, CM=3, FR=1, NG=1, CI=1, PL=1]");
		myHCountry.put("320","[NG=5, CM=2, US=2, FR=2, ES=1]");
		myHCountry.put("321","[CI=2, IT=1]");
		myHCountry.put("322","[MR=2, SN=1]");
		myHCountry.put("323","[SN=2, FR=1]");
		myHCountry.put("324","[FR=3, SN=2, GW=1]");
		myHCountry.put("325","[US=7, GM=4, FR=2, IT=1]");
		myHCountry.put("326","[US=15, NL=7, FR=7, GM=5, IT=4, TN=3, BE=3, GW=2, MQ=1, DE=1, GN=1, GP=1]");
		myHCountry.put("327","[CM=3]");
		myHCountry.put("328","[CM=8, IT=1, SN=1]");
		myHCountry.put("329","[CM=3, NL=1, CI=1]");
		myHCountry.put("330","[NG=2, CM=1, TN=1, BE=1, CF=1]");
		myHCountry.put("331","[NG=37, US=10, IT=5, CM=3, CI=3, NL=1, BF=1]");
		myHCountry.put("332","[CM=5]");
		myHCountry.put("333","[VE=8, GP=1, ES=1, GF=1]");
		myHCountry.put("334","[CN=22, IT=13, US=9, ES=6, FR=5, ZA=4, GW=3, MX=3, BE=3, GF=3, GE=3, IN=2, SE=2, FI=2, GB=2, VN=2, PE=1, NL=1, GP=1, MQ=1, KZ=1, SN=1, MA=1, HT=1, TN=1, BR=1, JP=1]");
		myHCountry.put("335","[FR=9, CZ=8, IT=7, RU=3, AT=3, BE=2, MQ=2, US=2, SA=1, KG=1, BD=1, ES=1, RO=1, VN=1, BR=1, SE=1]");
		myHCountry.put("336","[SE=48, ZA=22, US=19, MX=6, GB=5, SA=4, ET=2, RU=2, MG=2, IT=2, DK=1, BG=1, MZ=1]");
		myHCountry.put("337","[KE=2, IN=1, FR=1]");
		myHCountry.put("338","[GB=5, BD=1, MY=1]");
		myHCountry.put("339","[GB=16]");
		myHCountry.put("340","[US=14, IN=11, MM=4, NL=3, FR=3, DK=2, SA=2, GB=2, LK=1, AU=1, MY=1, OM=1]");
		myHCountry.put("341","[GB=2]");
		myHCountry.put("342","[MX=8, NG=5, GM=2, BE=2, GB=2, US=2, DK=1, MM=1, DE=1, NL=1]");
		myHCountry.put("343","[ET=3, GB=2, US=1]");
		myHCountry.put("344","[PT=4, TR=3, ES=3, IN=3, ZA=3, GB=2, GF=1, MX=1, OM=1]");
		myHCountry.put("345","[GB=4, ET=3, KE=2, US=2, FI=1, NO=1]");
		myHCountry.put("346","[GB=2, IN=2]");
		myHCountry.put("347","[ZA=25, US=16, BR=4, GB=2, NZ=2, IT=1, GF=1]");
		myHCountry.put("348","[ZA=22, GB=2, US=1]");
		myHCountry.put("349","[US=11, GF=3, SR=2, MM=2, GB=2, BD=2, MZ=2, GY=1, TN=1, IN=1, SE=1]");
		myHCountry.put("350","[GB=4, IT=1, AT=1]");
		myHCountry.put("351","[GB=14, NL=4, US=1]");
		myHCountry.put("352","[US=9, ZA=4, GB=2, NZ=2, ID=1]");
		myHCountry.put("353","[GR=7, TR=5, US=4, AT=3, GB=2, NL=2, SE=2, CN=1, EG=1, RU=1]");
		myHCountry.put("354","[US=3, TZ=2, GB=2, AU=1, TH=1, NL=1]");
		myHCountry.put("355","[IN=20, US=11, LK=10, GB=5, SA=3, NL=2, OM=2, MY=2, DK=1, NG=1, FR=1, MM=1, SG=1, MU=1]");
		myHCountry.put("356","[ZA=5, GB=2, IN=2, SD=2, SA=1, SE=1, US=1]");
		myHCountry.put("357","[IN=16, BD=13, SA=10, PK=7, OM=5, YE=5, IR=5, NL=4, SE=4, US=4, GB=3, CN=2, MM=2, DK=1, BE=1, GR=1]");
		myHCountry.put("358","[BD=8, CN=4, TR=4, ET=3, GB=2, FI=1, MQ=1, IT=1, IN=1, US=1, SD=1]");
		myHCountry.put("359","[AT=2]");
		myHCountry.put("360","[AT=4]");
		myHCountry.put("361","[AT=2, TR=1, IR=1, PK=1, NL=1, SE=1, US=1, OM=1]");
		myHCountry.put("362","[AT=18]");
		myHCountry.put("363","[TR=3, AT=3, IT=2, EG=2, SA=1, FR=1]");
		myHCountry.put("364","[AT=3]");
		myHCountry.put("365","[AT=6, BE=1, RU=1]");
		myHCountry.put("366","[AT=10]");
		myHCountry.put("367","[TR=7, AT=2, DK=1, PS=1, NL=1, FR=1]");
		myHCountry.put("368","[AT=2]");
		myHCountry.put("369","[AT=2, DE=1]");
		myHCountry.put("370","[BR=7, AT=5, SE=2, US=2, EE=1, TN=1, BE=1, RU=1, FR=1]");
		myHCountry.put("371","[AT=7, CZ=2, US=1]");
		myHCountry.put("372","[AT=2]");
		myHCountry.put("373","[NG=16, ZA=8, FR=7, CZ=5, BE=4, BR=4, US=4, PE=3, AT=3, ES=2, IT=2, MA=1, FI=1, SA=1, NL=1, GR=1, ET=1, SE=1, MX=1]");
		myHCountry.put("374","[AT=5, IT=1, RU=1, FR=1]");
		myHCountry.put("375","[VE=5, BE=1]");
		myHCountry.put("376","[US=21, HN=12, VE=8, BR=2, BE=1, ES=1, JM=1, ZA=1]");
		myHCountry.put("377","[US=3, VE=2, CO=1, BR=1]");
		myHCountry.put("378","[VE=2, IT=1, GE=1, US=1]");
		myHCountry.put("379","[AR=12, FR=4, CO=2, BR=2, VE=1]");
		myHCountry.put("380","[IN=2, SA=1, AU=1, US=1, AT=1]");
		myHCountry.put("381","[IN=15, SA=5, PK=2, NL=2, US=2, OM=2, IT=1, GB=1, BD=1, ET=1, SE=1, AT=1]");
		myHCountry.put("382","[AT=3, IT=2, US=2, PL=2, EE=1, AL=1, ES=1, FR=1]");
		myHCountry.put("383","[BR=4, FI=3, IT=3, AT=2, DE=1, SE=1]");
		myHCountry.put("384","[CZ=3, TR=1, CO=1, PE=1, US=1, FR=1, AT=1]");
		myHCountry.put("385","[GF=2, MQ=1]");
		myHCountry.put("386","[BR=3]");
		myHCountry.put("387","[BR=2]");
		myHCountry.put("388","[JP=11, DZ=8, BR=5, VE=2, CN=1, RU=1, US=1, PT=1]");
		myHCountry.put("389","[PT=7, BR=5, MZ=2, VE=2, FI=1, RU=1, ZA=1]");
		myHCountry.put("390","[IT=6, US=4, PE=3, MQ=3, BR=3, BE=2, FR=2, AT=2, EG=1, DE=1, PS=1, GP=1, SE=1, ZA=1]");
		myHCountry.put("391","[PY=21, BR=2]");
		myHCountry.put("392","[BR=4, AR=1]");
		myHCountry.put("393","[CN=8, FR=4, US=4, SE=3, ZM=2, BR=2, GF=2, DK=1, PY=1, EE=1, EG=1, ES=1, ET=1, IT=1, VN=1, ZA=1, GE=1, AR=1]");
		myHCountry.put("394","[VN=1, CU=1, ET=1]");
		myHCountry.put("395","[US=4, ES=1, FR=1]");
		myHCountry.put("396","[BR=9, IT=3, SE=2, PE=1, PT=1]");
		myHCountry.put("397","[BR=10, US=4, VE=4, SA=1, PY=1, IT=1, ES=1, AR=1]");
		myHCountry.put("398","[MX=7, US=4, BR=2, BE=1, CU=1, FR=1, AT=1]");
		myHCountry.put("399","[BR=2]");
		myHCountry.put("400","[AT=1, VE=1]");
		myHCountry.put("401","[BR=4, AU=1, GR=1]");
		myHCountry.put("402","[BR=15, US=7, GE=3, RU=2, FR=2, ZA=2, TZ=1, LK=1, PT=1, DZ=1, IT=1, AU=1, IN=1, GF=1, AT=1]");
		myHCountry.put("403","[NG=18, CM=15, US=3, IT=2, IN=2, FR=2, SA=1, CN=1, BE=1, AT=1]");
		myHCountry.put("404","[US=2]");
		myHCountry.put("405","[VN=8, US=5, DK=1]");
		myHCountry.put("406","[US=10, JP=2, CN=1, IT=1, TN=1, BE=1, IN=1, KR=1]");
		myHCountry.put("407","[US=2]");
		myHCountry.put("408","[US=3]");
		myHCountry.put("409","[US=42, AR=5, GB=4, MX=3, IE=1]");
		myHCountry.put("410","[US=3]");
		myHCountry.put("411","[US=6, FR=2, MW=1]");
		myHCountry.put("412","[US=4]");
		myHCountry.put("413","[US=6, PH=3, SA=1, IT=1, MM=1, VN=1]");
		myHCountry.put("414","[US=7, NL=3, IN=1, MY=1, SG=1, BB=1, GF=1]");
		myHCountry.put("415","[US=14, IN=6, ZM=3, NZ=3, TZ=2]");
		myHCountry.put("416","[CA=7]");
		myHCountry.put("417","[US=2]");
		myHCountry.put("418","[PE=8, US=4, AR=1]");
		myHCountry.put("419","[US=4]");
		myHCountry.put("420","[US=8, BE=5, TZ=2, UG=1, DE=1, ZM=1, SE=1]");
		myHCountry.put("421","[US=13]");
		myHCountry.put("422","[US=8, ES=1]");
		myHCountry.put("423","[US=8, BE=4]");
		myHCountry.put("424","[US=9, OM=3, IR=2]");
		myHCountry.put("425","[US=6]");
		myHCountry.put("426","[US=2]");
		myHCountry.put("427","[IN=9, US=4, IT=1, TN=1, DE=1, GB=1, OM=1]");
		myHCountry.put("428","[IN=5, US=5, ZA=4, IR=3, SA=1, MM=1, PK=1, BD=1, FR=1, OM=1]");
		myHCountry.put("429","[US=11, IN=9, GB=2, SA=1, YE=1, SE=1, OM=1, SD=1]");
		myHCountry.put("430","[US=10, PE=3, ES=2, VE=2, PA=1]");
		myHCountry.put("431","[IT=1, BR=1, US=1]");
		myHCountry.put("432","[US=3]");
		myHCountry.put("433","[US=22, MA=10, ES=5, DO=5, IT=1, BE=1, FR=1]");
		myHCountry.put("434","[US=5]");
		myHCountry.put("435","[US=4, BR=1]");
		myHCountry.put("436","[US=11]");
		myHCountry.put("437","[CM=1, SL=1, US=1, NG=1]");
		myHCountry.put("438","[NG=5, US=2, SL=1]");
		myHCountry.put("439","[BR=4, TR=3, US=2, PY=1]");
		myHCountry.put("440","[US=12]");
		myHCountry.put("441","[US=2]");
		myHCountry.put("442","[US=4, IT=2, IN=1, FR=1, PL=1]");
		myHCountry.put("443","[US=47, BE=3, BR=3, GF=1, VE=1]");
		myHCountry.put("444","[US=26, KZ=4, RU=4, AT=3, YE=1, DE=1, GE=1]");
		myHCountry.put("445","[US=11]");
		myHCountry.put("446","[US=26]");
		myHCountry.put("447","[US=6, CO=1, AT=1]");
		myHCountry.put("448","[US=5, CM=2, PT=2, FR=1, NG=1, OM=1]");
		myHCountry.put("449","[US=10, MX=1]");
		myHCountry.put("450","[US=37, CM=19, PE=8, MX=7, CZ=5, FR=2, IT=2, BR=2, SE=2, DK=1, TT=1, TC=1, NL=1]");
		myHCountry.put("451","[US=87, SA=7, GB=6, BG=5, NL=4, VE=4, BR=4, MG=3, TN=3, NA=2, EG=2, FR=2, PT=2, MM=2, AR=2, MA=1, TW=1, LK=1, GP=1, SN=1, VN=1, IN=1, SE=1, AT=1]");
		myHCountry.put("452","[US=15, SE=1]");
		myHCountry.put("453","[US=4, BG=4, BR=3, SE=2, ES=1, GF=1, VC=1, PT=1]");
		myHCountry.put("454","[US=4]");
		myHCountry.put("455","[US=15, HT=6, GP=1, DO=1, BG=1]");
		myHCountry.put("456","[US=4]");
		myHCountry.put("457","[US=25, DO=6, IT=2, FI=1, HT=1, ES=1, SE=1]");
		myHCountry.put("458","[TH=8, IN=8, US=6, GW=2, VN=2, DK=1, IT=1]");
		myHCountry.put("459","[US=6, FR=3]");
		myHCountry.put("460","[US=2]");
		myHCountry.put("461","[US=61, TZ=2, NZ=2, IN=1]");
		myHCountry.put("462","[CN=9, US=8, ET=7, BR=6, PL=5, IT=5, FR=4, FI=2, PE=2, ZA=2, SA=1, CM=1, ZM=1, RU=1, NG=1, PT=1, AT=1]");
		myHCountry.put("463","[BR=5, PL=4, US=3, SE=2, AR=2, AT=1]");
		myHCountry.put("464","[US=3]");
		myHCountry.put("465","[US=4]");
		myHCountry.put("466","[DZ=9, BE=6, MA=4, FR=4, ZA=3, TR=2, GR=2, SA=1, DE=1, NL=1, ES=1, MG=1]");
		myHCountry.put("467","[CN=5, US=4, TW=3, IT=2, LY=1, BR=1]");
		myHCountry.put("468","[US=2, AU=1]");
		myHCountry.put("469","[US=13, PE=9, IT=2, EG=2, HT=1, PA=1, CO=1, MM=1, TH=1, ZM=1, GF=1, FR=1]");
		myHCountry.put("470","[US=2, SA=1, FR=1]");
		myHCountry.put("471","[IN=5, US=4, BD=2, GB=1]");
		myHCountry.put("472","[BR=2, US=2, CZ=2, BE=1, NL=1, ES=1, FR=1]");
		myHCountry.put("473","[US=3, IN=2, OM=2, MM=1, DE=1, AU=1, MY=1, SG=1]");
		myHCountry.put("474","[IN=4, GB=1, US=1]");
		myHCountry.put("475","[US=4, IN=2, GB=1]");
		myHCountry.put("476","[US=2, GB=1]");
		myHCountry.put("477","[US=4, DK=3, AU=1, NL=1, IN=1, MY=1]");
		myHCountry.put("478","[US=19, MX=7, TT=2, NO=1, IT=1, GB=1, JM=1]");
		myHCountry.put("479","[AR=42, IE=30, US=22, FR=12, GB=9, BR=4, MX=3, IT=1, AU=1, KR=1, ZA=1]");
		myHCountry.put("480","[US=3]");
		myHCountry.put("481","[ES=4784, FR=165, BR=99, ZA=95, NL=60, DE=41, AR=38, BE=21, MX=8, US=8, IT=5, SE=3, PT=2, TN=2, GB=2, MA=1, UY=1, DZ=1]");
		myHCountry.put("482","[ES=569, FR=471, RU=134, IT=84, SE=55, NL=53, DE=52, BR=34, DK=29, US=28, AR=22, IR=21, SA=12, AT=12, DZ=10, TN=10, BE=8, GB=7, OM=6, ZA=4, GP=3, PT=3, MX=3, ZM=3, PS=2, ET=2, CZ=2, BG=2, JM=2, GF=2, NZ=2, TW=1, NA=1, SR=1, MZ=1, KR=1, VE=1, NO=1]");
		myHCountry.put("483","[US=11, SA=1, IT=1, MM=1, TN=1, MY=1, PH=1, GF=1, FR=1]");
		myHCountry.put("484","[US=2]");
		myHCountry.put("485","[IR=7, US=6, IN=3, SA=2, PK=2, IT=1, IQ=1, GB=1, SE=1, AT=1, OM=1]");
		myHCountry.put("486","[SA=23, OM=7, IN=6, PK=4, US=4, IR=2, ZA=2, TZ=1, NL=1, YE=1, MM=1, SE=1, AT=1]");
		myHCountry.put("487","[CZ=8, US=3, DE=1, AT=1]");
		myHCountry.put("488","[FR=5, US=3, IT=2]");
		myHCountry.put("489","[ZA=5, US=4, PE=1, AU=1]");
		myHCountry.put("490","[GB=12, US=3]");
		myHCountry.put("491","[AU=1, US=1]");
		myHCountry.put("492","[US=5, HT=2, MA=2, TR=2, IT=2, NL=2, ES=2, GF=1]");
		myHCountry.put("493","[NL=3, MM=1, MY=1, US=1]");
		myHCountry.put("494","[US=2, IT=1, VE=1]");
		myHCountry.put("495","[AZ=2, RU=2, US=1]");
		myHCountry.put("496","[RU=7, GE=1, US=1]");
		myHCountry.put("497","[VN=3, US=1]");
		myHCountry.put("498","[BR=11, IN=3, FR=2, CZ=2, SA=1, CN=1, EG=1, DE=1, GB=1, ZA=1, US=1, AT=1, OM=1, PL=1]");
		myHCountry.put("499","[RU=2, US=1]");
		myHCountry.put("500","[RU=4, LV=2, FI=1, TR=1, US=1, PL=1]");
		myHCountry.put("501","[US=3, CA=2, GB=1, JP=1]");
		myHCountry.put("502","[US=1, FR=1, OM=1]");
		myHCountry.put("503","[CZ=2, GP=1, US=1]");
		myHCountry.put("504","[US=7, ET=3, OM=3, CN=2, FR=2, NG=2, IT=1, TZ=1, IN=1]");
		myHCountry.put("505","[HT=4, US=4, BR=2, IT=1, FR=1]");
		myHCountry.put("506","[RU=2, MX=1, US=1]");
		myHCountry.put("507","[US=2]");
		myHCountry.put("508","[US=5, PE=1, GB=1]");
		myHCountry.put("509","[US=4, BR=2, FR=2, SA=1, SR=1, GP=1, BS=1]");
		myHCountry.put("510","[US=2, OM=1]");
		myHCountry.put("511","[US=11, FR=7, CZ=7, SE=4, PL=3, CN=2, DE=2, FI=1, TW=1, ES=1, RU=1, KZ=1, IT=1, IR=1, LV=1]");
		myHCountry.put("512","[US=8, FR=4, BR=3, CN=2, ES=2, HU=1, IT=1, PE=1, DE=1, BE=1, BD=1, IN=1, AT=1]");
		myHCountry.put("513","[CZ=3, IT=2, GB=1, ES=1, US=1]");
		myHCountry.put("514","[US=6, SG=1]");
		myHCountry.put("515","[IT=4, US=3, CN=2, SE=2, TR=1, NL=1, GR=1]");
		myHCountry.put("516","[IT=3, IQ=2, US=2, SA=1, EG=1, ID=1, RU=1, FR=1, SR=1, DZ=1, TN=1, GB=1, SE=1, GF=1]");
		myHCountry.put("517","[US=5, TN=1, VN=1, OM=1]");
		myHCountry.put("518","[US=2]");
		myHCountry.put("519","[ZA=4, US=3, SR=2, NA=2, BR=2, GB=1, ZM=1, MX=1, BG=1, PT=1]");
		myHCountry.put("520","[CZ=5, SA=2, US=2, CN=1, DE=1, PK=1, RU=1, FR=1, MQ=1, TR=1, IT=1, IQ=1, SE=1, AT=1]");
		myHCountry.put("521","[PY=6, ZA=5, US=3, FR=2, BR=2, AR=2, ES=1, GW=1, BZ=1, IT=1, SE=1, JP=1, AT=1]");
		myHCountry.put("522","[US=4, GW=2, HT=1, KE=1, IT=1, YE=1, ID=1, NL=1, MX=1, FR=1, AT=1]");
		myHCountry.put("523","[US=5, SA=4, CN=4, JP=4, NG=3, MY=3, KR=3, TW=2, BE=2, TH=2, VN=2, ZA=2, EG=1, ID=1, GP=1, RU=1, FR=1, GW=1, BR=1, SE=1, AT=1]");
		myHCountry.put("524","[AT=3, ZA=2, IT=1, GM=1, NL=1, GR=1, MZ=1, SE=1, US=1]");
		myHCountry.put("525","[GW=2]");
		myHCountry.put("527","[GW=10]");
		myHCountry.put("528","[GW=4, GM=3, US=2]");
		myHCountry.put("529","[GW=3, IN=1, SE=1, FR=1]");
		myHCountry.put("530","[GW=2]");
		myHCountry.put("531","[CZ=3, RU=2, US=2, GW=1, FR=1]");
		myHCountry.put("532","[GW=2]");
		myHCountry.put("533","[GW=2, SE=1]");
		myHCountry.put("534","[BR=7, GW=3, PE=1, ES=1]");
		myHCountry.put("535","[BR=37, US=4, SA=2, CN=2, GW=2, ZA=2, FI=1, MA=1, CM=1, BE=1, GP=1, RU=1, FR=1, PT=1, VE=1, TR=1, IT=1, IR=1, MY=1, GF=1, GE=1]");
		myHCountry.put("536","[GW=6, US=2]");
		myHCountry.put("537","[GW=2]");
		myHCountry.put("538","[GW=1, US=1]");
		myHCountry.put("539","[NL=11, TN=10, FR=4, US=3, GB=1]");
		myHCountry.put("540","[US=3]");
		myHCountry.put("541","[US=7, CN=4, TW=2, JP=2, NL=1]");
		myHCountry.put("542","[US=4]");
		myHCountry.put("543","[US=2]");
		myHCountry.put("544","[US=2]");
		myHCountry.put("545","[CO=4, US=2, FR=2]");
		myHCountry.put("546","[US=8, HN=5, PE=1]");
		myHCountry.put("547","[US=2]");
		myHCountry.put("548","[US=6, RU=1]");
		myHCountry.put("549","[US=5, TN=1]");
		myHCountry.put("550","[US=3]");
		myHCountry.put("551","[US=2]");
		myHCountry.put("552","[US=4]");
		myHCountry.put("553","[US=3, JP=1]");
		myHCountry.put("554","[US=7]");
		myHCountry.put("555","[US=2]");
		myHCountry.put("556","[US=4, TN=1]");
		myHCountry.put("557","[US=24, TN=1]");
		myHCountry.put("558","[US=2]");
		myHCountry.put("559","[US=5]");
		myHCountry.put("560","[RU=11, IT=2, BR=2, IN=1, GE=1, US=1, FR=1]");
		myHCountry.put("561","[RU=2, US=2]");
		myHCountry.put("562","[US=4, MY=1, MX=1]");
		myHCountry.put("563","[US=7, MZ=2, BD=1, BR=1, ZA=1, FR=1]");
		myHCountry.put("564","[US=7, IR=5, VN=4, NO=2, TH=1]");
		myHCountry.put("565","[ZA=4, US=3, ET=1, MX=1]");
		myHCountry.put("566","[TT=84, US=11]");
		myHCountry.put("567","[RU=5, US=2]");
		myHCountry.put("568","[TR=2, IQ=1, TH=1, US=1]");
		myHCountry.put("569","[CZ=4, US=2]");
		myHCountry.put("570","[US=3]");
		myHCountry.put("571","[US=3]");
		myHCountry.put("572","[US=4, TH=1]");
		myHCountry.put("573","[MX=3, US=3, BR=1, VE=1]");
		myHCountry.put("574","[CN=3, US=3, TW=1, ES=1]");
		myHCountry.put("575","[US=8, IN=1]");
		myHCountry.put("576","[US=22]");
		myHCountry.put("577","[DK=5]");
		myHCountry.put("578","[US=19, HT=3, VE=3, MQ=2, PY=1, GF=1]");
		myHCountry.put("579","[US=3, SE=1]");
		myHCountry.put("580","[NA=2]");
		myHCountry.put("581","[NA=1, US=1]");
		myHCountry.put("582","[FR=2]");
		myHCountry.put("583","[FR=2, CN=1]");
		myHCountry.put("584","[ET=7, SE=4, SA=3, FR=3, NL=2, OM=2, CN=1, TZ=1, TN=1, US=1]");
		myHCountry.put("585","[BR=4, US=2, FR=2]");
		myHCountry.put("586","[FR=2]");
		myHCountry.put("587","[BD=3, US=3, IT=2, GB=2, IN=2, DK=1, IR=1, NL=1, FR=1]");
		myHCountry.put("588","[FR=2, PK=1]");
		myHCountry.put("589","[FR=10, MA=2, ES=2, IT=1, NL=1]");
		myHCountry.put("590","[BE=2, DE=1, US=1, FR=1]");
		myHCountry.put("591","[SE=15, BD=10, MY=10, IN=9, MM=5, NL=4, OM=4, SA=2, FR=2, IT=2, GB=2, TH=2, AT=2, ET=1, SG=1, US=1]");
		myHCountry.put("592","[TN=6, NL=5, GP=1]");
		myHCountry.put("593","[NL=4, FR=3, GB=2, TN=1]");
		myHCountry.put("594","[ZA=16, AR=13, IE=10, BR=10, KZ=7, GB=7, NL=3, US=3, PY=1, SE=1]");
		myHCountry.put("595","[IR=2, DK=1, NL=1, US=1]");
		myHCountry.put("596","[IR=2]");
		myHCountry.put("597","[IR=2, NL=1, FR=1]");
		myHCountry.put("598","[IR=3, IN=2]");
		myHCountry.put("599","[US=5, NP=3, IR=2, MM=2, BE=2, IN=2, AT=2, BD=1, OM=1]");
		myHCountry.put("600","[IR=5, ET=1]");
		myHCountry.put("601","[IN=2, IR=1, AT=1]");
		myHCountry.put("602","[GE=21, TR=17, BR=10, US=8, MA=7, IT=4, SE=4, AT=4, ZA=4, FR=3, SA=2, NL=2, BE=2, CN=1, AL=1, ES=1, ET=1, RU=1, PL=1, IR=1, FI=1, ID=1, DE=1, BG=1, VN=1, NZ=1, JP=1]");
		myHCountry.put("603","[VE=4, US=3, DO=1]");
		myHCountry.put("604","[GM=8, GW=2, BE=1, NL=1, VE=1]");
		myHCountry.put("605","[VE=28, US=2]");
		myHCountry.put("606","[VE=3, FR=1]");
		myHCountry.put("607","[FR=2]");
		myHCountry.put("608","[TR=5, FR=3, DE=2, BE=2, US=1]");
		myHCountry.put("609","[FR=3, MA=2]");
		myHCountry.put("610","[FR=2, CO=1, IQ=1, BE=1, US=1, BG=1]");
		myHCountry.put("611","[GW=1, FR=1]");
		myHCountry.put("612","[US=7, ES=5, ET=4, FR=3, SE=2, BE=1, GB=1, AR=1, BG=1, LV=1, PL=1]");
		myHCountry.put("613","[AL=6, US=5, AT=3, IT=1, SE=1, FR=1]");
		myHCountry.put("614","[US=1, FR=1]");
		myHCountry.put("615","[ES=2, US=2, SA=1, FR=1, AT=1]");
		myHCountry.put("616","[CN=4, VN=2, IN=1, US=1]");
		myHCountry.put("617","[US=2, FR=2, VN=1]");
		myHCountry.put("618","[US=6, FR=3, VN=2, TH=1, NL=1]");
		myHCountry.put("619","[US=2, VN=1, FR=1]");
		myHCountry.put("620","[AT=3, PE=2, IN=2, US=2, NP=1, BE=1, VN=1, SE=1, FR=1]");
		myHCountry.put("621","[JP=15, CN=9, AM=2, VN=2, IN=2, IT=1, US=1, FR=1]");
		myHCountry.put("622","[IR=5, VN=5]");
		myHCountry.put("623","[VN=5, CN=3, MY=2, DE=1, GF=1, US=1]");
		myHCountry.put("624","[US=2, IT=1, IN=1]");
		myHCountry.put("625","[IN=3, US=2, GB=1, NL=1, OM=1]");
		myHCountry.put("626","[US=2]");
		myHCountry.put("627","[JP=35, US=2, FI=1, IN=1]");
		myHCountry.put("628","[ZA=4, SA=3, IT=1, EG=1, PE=1, NL=1, IN=1, SE=1, US=1, FR=1, AT=1]");
		myHCountry.put("629","[GB=1, IN=1, US=1]");
		myHCountry.put("630","[US=3, HT=2, GP=1, FR=1]");
		myHCountry.put("631","[US=5, BE=2, CZ=2, CN=1, CM=1, IT=1, BR=1, BG=1, FR=1, NG=1, AT=1]");
		myHCountry.put("632","[CN=15, IN=4, JP=4, TW=1, BE=1, CC=1, GE=1, FR=1]");
		myHCountry.put("633","[HT=3, GP=1]");
		myHCountry.put("634","[NL=5, ES=1]");
		myHCountry.put("635","[BR=5, BE=2, ET=1, HN=1, MZ=1, US=1]");
		myHCountry.put("636","[BE=7]");
		myHCountry.put("637","[BE=56, FR=28, ES=8, DZ=1]");
		myHCountry.put("638","[NL=2, US=1]");
		myHCountry.put("639","[GB=3]");
		myHCountry.put("640","[GB=2]");
		myHCountry.put("641","[GB=7, TN=3, US=1]");
		myHCountry.put("642","[GB=3, FR=3, DE=1, NL=1]");
		myHCountry.put("643","[NL=2]");
		myHCountry.put("644","[ES=79, DE=3, MA=1, PE=1, FR=1]");
		myHCountry.put("645","[ES=743, NL=1]");
		myHCountry.put("646","[ES=273, IT=11, DE=4, FR=2]");
		myHCountry.put("647","[DE=80, IT=17, HU=16, FR=12, HR=11, AT=8, ES=6, CZ=4, NL=2, UA=2, HN=2, AL=1, BE=1, GB=1]");
		myHCountry.put("648","[NL=2]");
		myHCountry.put("649","[SE=8, SA=3, ZA=3, KE=2, NL=2, US=1]");
		myHCountry.put("650","[ES=18, FR=6, BE=2, NL=2, DE=1]");
		myHCountry.put("651","[GB=1, NZ=1]");
		myHCountry.put("652","[DK=2, NL=2, GB=1]");
		myHCountry.put("653","[NL=2]");
		myHCountry.put("654","[NL=2, OM=2, MM=1, IN=1, US=1]");
		myHCountry.put("655","[US=10, SA=7, NL=6, IN=4, ID=3, BE=3, SD=2, NP=1, IT=1, EG=1, PK=1, FR=1, OM=1]");
		myHCountry.put("656","[IR=8, SA=4, SE=2, NL=1]");
		myHCountry.put("657","[AR=2]");
		myHCountry.put("658","[AR=34, NL=4, BR=4, MX=4, US=4]");
		myHCountry.put("659","[NL=6, BI=5, ET=4, BE=2]");
		myHCountry.put("660","[NL=4, BI=1]");
		myHCountry.put("661","[ET=22, IT=12, TZ=9, ES=5, NL=4, ZA=3, UG=1, DE=1, FR=1]");
		myHCountry.put("662","[FR=21, BE=10]");
		myHCountry.put("663","[ES=46, CA=1, IT=1, NL=1]");
		myHCountry.put("664","[NL=2, DZ=1]");
		myHCountry.put("665","[ES=1895, FR=83, BE=21, IT=13, BR=9, ML=6, ET=6, ZA=4, DE=3, NL=3, GB=3, TN=2, RE=2, DZ=1, AR=1]");
		myHCountry.put("666","[GB=5, NL=2, TW=1]");
		myHCountry.put("667","[IE=9, GB=8, NL=2, AU=1, ES=1, AR=1]");
		myHCountry.put("668","[GB=7, AR=3, BR=2, UY=1, ZA=1]");
		myHCountry.put("669","[AR=14]");
		myHCountry.put("670","[ES=8, ZA=5, GB=4, US=3, MX=1]");
		myHCountry.put("671","[IE=3, GB=1]");
		myHCountry.put("672","[GB=5, IE=4, AR=4, MX=4, BR=3, ES=2, IT=1, NL=1, NZ=1]");
		myHCountry.put("673","[IE=2, AU=2, GB=1]");
		myHCountry.put("674","[AR=3]");
		myHCountry.put("675","[GB=6, CL=2, AU=1, NL=1, ES=1]");
		myHCountry.put("676","[GB=105, IE=71, NL=6, ES=1]");
		myHCountry.put("677","[GB=31, IE=2]");
		myHCountry.put("678","[GB=15, AR=2, AU=1]");
		myHCountry.put("679","[IE=316, GB=111, NL=2, AR=2]");
		myHCountry.put("680","[NL=4, IE=1]");
		myHCountry.put("681","[GB=7, BR=4, CL=1, AR=1, FR=1]");
		myHCountry.put("682","[IE=54, GB=11, AR=3, AU=2]");
		myHCountry.put("683","[IE=994, GB=461, AR=331, ES=157, BR=43, KR=43, NL=37, PT=24, NZ=24, AU=18, ZA=17, CL=16, FR=16, US=9, MX=8, IT=5, UY=3, MZ=3, PY=2, BE=2, SZ=1, MQ=1]");
		myHCountry.put("684","[ES=924, FR=10, US=7, ZA=6, BE=4, NL=3, TW=2, IT=1, TN=1, DE=1, PT=1]");
		myHCountry.put("685","[DE=5, GB=3, AR=2, FR=2, IE=1, TR=1, IT=1, BR=1, NZ=1]");
		myHCountry.put("686","[NL=2]");
		myHCountry.put("687","[NL=2]");
		myHCountry.put("688","[IE=23, GB=5, IT=1, NL=1]");
		myHCountry.put("689","[IE=11, NL=2, IT=1]");
		myHCountry.put("690","[ZA=6, SE=1]");
		myHCountry.put("691","[ES=524, AR=85, ZA=42, NZ=31, BR=23, IE=21, FR=13, US=13, GB=10, SE=10, MX=4, PY=3, PT=3, TN=3, NL=1, TR=1, IT=1]");
		myHCountry.put("692","[ES=99, NL=4]");
		myHCountry.put("693","[NL=2]");
		myHCountry.put("694","[NL=2, DK=1, DE=1]");
		myHCountry.put("695","[ES=284, NL=3, DE=2, MX=2, BE=1, BR=1, US=1]");
		myHCountry.put("696","[ES=1350, FR=16, NL=2, DE=1, BE=1, AT=1]");
		myHCountry.put("697","[ES=21, NL=6, DE=2, FR=1]");
		myHCountry.put("698","[ES=817, BR=97, AR=14, FR=5, PT=5, MX=4, NL=3, BE=2, CR=1, US=1]");
		myHCountry.put("699","[ET=4, NL=2, US=2, IT=1, SE=1]");
		myHCountry.put("700","[NL=4]");
		myHCountry.put("701","[NL=3]");
		myHCountry.put("702","[MZ=11, MW=9, BR=4, ZM=3, CU=2, TN=1, GB=1, GF=1, FR=1]");
		myHCountry.put("703","[SE=7, NL=2]");
		myHCountry.put("704","[IR=2, NL=2, IQ=1, US=1]");
		myHCountry.put("705","[NL=2]");
		myHCountry.put("706","[NL=2]");
		myHCountry.put("707","[ES=4, AR=2, ID=1, NL=1]");
		myHCountry.put("708","[RU=5, NL=4, DZ=1, US=1, FR=1, AT=1, PL=1]");
		myHCountry.put("709","[NL=2]");
		myHCountry.put("710","[NL=4]");
		myHCountry.put("711","[IN=6, DK=2, NL=2, FI=1, GB=1, BD=1, US=1]");
		myHCountry.put("712","[CN=4, NL=2]");
		myHCountry.put("713","[FR=6, NL=2, DK=1, CN=1, KG=1, NZ=1, US=1]");
		myHCountry.put("714","[HT=4, US=1]");
		myHCountry.put("715","[NL=2, CM=1]");
		myHCountry.put("716","[GB=2]");
		myHCountry.put("717","[NL=1, CU=1, ZA=1, FR=1]");
		myHCountry.put("718","[NL=1, CU=1]");
		myHCountry.put("719","[ZA=78, US=4, ZM=3, MZ=2, BE=1, NL=1, AR=1, FR=1]");
		myHCountry.put("720","[AR=4]");
		myHCountry.put("721","[ZA=6, IT=4, NL=2, DK=1, US=1]");
		myHCountry.put("722","[NL=2]");
		myHCountry.put("723","[NL=2]");
		myHCountry.put("724","[MY=2, NL=1, IN=1, OM=1]");
		myHCountry.put("725","[AR=8]");
		myHCountry.put("726","[SE=5, NL=3, GB=2, US=2, IT=1, AU=1, ZA=1]");
		myHCountry.put("727","[CO=25, US=7, NG=2, NL=1, ES=1, PL=1, VE=1]");
		myHCountry.put("728","[NL=2]");
		myHCountry.put("729","[HT=3, GP=1, BR=1, ZA=1]");
		myHCountry.put("730","[NL=2]");
		myHCountry.put("731","[MA=6, FR=4, NL=2, ES=2, ZA=2, IT=1, BE=1]");
		myHCountry.put("732","[US=5, AR=4, ZA=3, ES=2, RU=2, FR=2, IT=2, SA=1, CN=1, GM=1, TR=1, SE=1, AT=1]");
		myHCountry.put("733","[NL=3, IN=2, TZ=1, US=1]");
		myHCountry.put("734","[NL=2, AU=1]");
		myHCountry.put("735","[NL=2, DK=1]");
		myHCountry.put("736","[NL=12, IT=6, SR=1, CN=1]");
		myHCountry.put("737","[FR=6, IT=4, MA=2, PY=2, VE=2, GF=1, US=1]");
		myHCountry.put("738","[AR=3, BR=1, LV=1]");
		myHCountry.put("739","[NL=3, IN=1]");
		myHCountry.put("740","[ZA=3, PE=2, NL=2, BR=2, IT=1, DE=1, GP=1, AT=1]");
		myHCountry.put("741","[MA=4, NL=2, BR=2, AT=2, IT=1, GF=1, RU=1, US=1, FR=1, PL=1]");
		myHCountry.put("742","[CN=13, US=10, TW=9, SE=6, MY=3, SA=2, IT=2, ID=2, VN=2, AT=2, JP=2, MQ=1, TH=1, BR=1]");
		myHCountry.put("743","[MY=2, CZ=2, US=1]");
		myHCountry.put("744","[NL=2]");
		myHCountry.put("745","[MY=26, IN=2]");
		myHCountry.put("746","[US=7, BR=4, FR=3, MA=2, BE=2, NL=2, ZA=2, PY=1, CN=1, ES=1, DO=1, PL=1, IT=1, TN=1, SE=1]");
		myHCountry.put("747","[NL=4, BR=3]");
		myHCountry.put("748","[NL=2, PE=1, SE=1, LV=1]");
		myHCountry.put("749","[CN=1, ID=1, JP=1]");
		myHCountry.put("750","[AT=8, US=7, AR=3, DE=1, PS=1, BR=1]");
		myHCountry.put("751","[FR=6, BE=5, MA=2, TR=2, US=2, CN=1, NL=1, ES=1, OM=1]");
		myHCountry.put("752","[US=2, GB=1]");
		myHCountry.put("753","[ZW=4, US=3, PY=2, ES=2, IN=2, MW=2, CO=1, NL=1, BR=1, GF=1, ZA=1]");
		myHCountry.put("754","[IR=5, IN=2, OM=2, GB=1, PK=1, BD=1, NL=1]");
		myHCountry.put("755","[NL=1, US=1]");
		myHCountry.put("756","[VN=1, MX=1, US=1]");
		myHCountry.put("757","[NG=6, GB=1, NL=1, TD=1]");
		myHCountry.put("758","[US=3]");
		myHCountry.put("759","[TR=3]");
		myHCountry.put("760","[DZ=3, FR=3, MA=2, IT=1, NL=1, AT=1]");
		myHCountry.put("761","[US=2, NL=1]");
		myHCountry.put("762","[RU=2, NL=1]");
		myHCountry.put("763","[US=4, SR=3, DK=1, MM=1, BD=1, NL=1, IN=1, MY=1, OM=1]");
		myHCountry.put("764","[BE=2, TW=1, TR=1, SN=1, NL=1, VN=1, IN=1, ET=1, US=1]");
		myHCountry.put("765","[US=4, PY=1, GM=1, DE=1, NL=1, BR=1, FR=1]");
		myHCountry.put("766","[ZA=13, EE=5, TR=5, IT=4, FR=3, SE=3, LV=3, FI=2, ES=2, RU=2, KZ=2, NL=1, BD=1, PT=1, GW=1]");
		myHCountry.put("767","[GB=2, IT=1, NL=1]");
		myHCountry.put("768","[DE=2, US=2, NL=1, SE=1, AT=1]");
		myHCountry.put("769","[FR=2, CU=1, AT=1]");
		myHCountry.put("770","[BR=8, ES=5, SE=2, MA=1, PA=1, IT=1, AR=1, FR=1, SD=1]");
		myHCountry.put("771","[BE=2, BG=2, NL=1, AR=1, US=1, AT=1]");
		myHCountry.put("772","[US=4, SE=3, IT=2, FR=2, NL=1]");
		myHCountry.put("773","[US=4, NL=1]");
		myHCountry.put("774","[US=3, CM=2, BE=1, NL=1, ES=1, RU=1, NG=1]");
		myHCountry.put("775","[CZ=13, US=5, IT=4, AT=4, FR=3, AL=2, SE=2, PL=2, HU=1, BE=1, NL=1, IN=1]");
		myHCountry.put("776","[MY=2, SA=1, NL=1, US=1]");
		myHCountry.put("777","[ET=14, SA=6, RU=5, US=5, TR=3, KZ=2, SE=2, FI=1, AM=1, EG=1, PE=1, PK=1, NL=1, ES=1, BG=1, IT=1, IQ=1, BR=1, GE=1]");
		myHCountry.put("778","[MX=5, AR=3, NL=2]");
		myHCountry.put("779","[FR=2, PE=1]");
		myHCountry.put("780","[FR=5, SA=2, IN=2, CN=1, ZW=1, ET=1, MZ=1]");
		myHCountry.put("781","[US=4, IT=2, FR=2, PY=1, TR=1, NL=1, SE=1, AT=1]");
		myHCountry.put("782","[FR=2, IT=1, US=1]");
		myHCountry.put("783","[MY=3, US=3, TW=1, CN=1, FR=1]");
		myHCountry.put("784","[CA=14, SA=13, BR=5, US=5, FR=5, MA=3, TR=3, IT=3, ZA=3, PY=1, PE=1, CU=1, ES=1, VE=1]");
		myHCountry.put("785","[NL=1, FR=1]");
		myHCountry.put("786","[IT=3, PE=3, SE=2, CM=1, NL=1, FR=1, NG=1, AT=1]");
		myHCountry.put("787","[MX=2, US=2, FR=1, AT=1]");
		myHCountry.put("788","[SE=5, US=2, NL=1]");
		myHCountry.put("789","[ZA=5, US=3, AT=2, IT=1, SE=1, AR=1]");
		myHCountry.put("790","[ZM=3, ZA=3, US=2, AU=1, BE=1, SE=1, OM=1]");
		myHCountry.put("791","[CN=2, FR=2, CZ=2, IT=1, CU=1, SE=1, US=1]");
		myHCountry.put("792","[US=2, SE=1, FR=1]");
		myHCountry.put("793","[US=2]");
		myHCountry.put("794","[BD=7, PK=4, GB=2, NP=1, IN=1, US=1]");
		myHCountry.put("795","[EG=3, MY=1, GR=1]");
		myHCountry.put("796","[GE=3, SD=3, BD=2, VN=1, US=1]");
		myHCountry.put("797","[FR=3, US=1]");
		myHCountry.put("798","[NL=1, ZA=1, FR=1]");
		myHCountry.put("799","[SN=2]");
		myHCountry.put("800","[GY=3, US=2, VN=1]");
		myHCountry.put("801","[US=2, GW=1, SA=1, KZ=1, DE=1, ZW=1, SE=1, FR=1]");
		myHCountry.put("802","[VN=2]");
		myHCountry.put("803","[RU=8, CN=5, GE=5, AM=2, EE=2, VN=2, ZA=2, SA=1, BE=1, FR=1, PL=1, BR=1, SE=1, US=1, LV=1]");
		myHCountry.put("804","[GW=3, IT=3, ZW=3, US=3, SA=2, CN=2, SE=2, FR=2, DK=1, TW=1, IN=1, MX=1, AT=1]");
		myHCountry.put("805","[ZW=2]");
		myHCountry.put("806","[ZA=24, MZ=15, US=8, NO=2, ZW=2, FR=1]");
		myHCountry.put("807","[ZW=3]");
		myHCountry.put("808","[ZW=2]");
		myHCountry.put("809","[MZ=4, ZW=3, TZ=1]");
		myHCountry.put("810","[ZW=2]");
		myHCountry.put("811","[ZA=23, MZ=15, ZW=8, US=5, TZ=2, BE=2, ZM=2]");
		myHCountry.put("812","[ZW=3, US=1]");
		myHCountry.put("813","[ZW=6, ZM=3, ZA=2, BE=1, GB=1, US=1]");
		myHCountry.put("814","[ZW=4, ZM=3, BE=1, ZA=1]");
		myHCountry.put("815","[ZM=63, ZA=29, BE=27, ZW=9, MZ=9, TZ=2, MW=2, US=2, OM=2, TN=1]");
		myHCountry.put("816","[MW=5, ZW=1, BE=1, US=1, MG=1]");
		myHCountry.put("817","[ET=14, US=3, SA=1, IT=1, ZW=1, GR=1, NG=1]");
		myHCountry.put("818","[ES=22, DE=5, IT=4, FR=4, AT=1]");
		myHCountry.put("819","[DE=3, ES=2]");
		myHCountry.put("820","[DK=29, ES=16, DE=6, BR=5, FR=4, MW=2, US=2, NO=1, IT=1, MX=1]");
		myHCountry.put("821","[US=10, BR=1, VE=1]");
		myHCountry.put("822","[BR=4, US=4, FR=4, ES=2, MA=1, TR=1, CO=1, TN=1, BD=1, GP=1, BG=1]");
		myHCountry.put("823","[BR=4, US=4, IT=2, FI=1, DE=1, FR=1]");
		myHCountry.put("824","[ES=3, PE=1, BR=1]");
		myHCountry.put("825","[AT=2, GB=1, BR=1, US=1]");
		myHCountry.put("826","[BR=4, PE=3, IT=1, VE=1]");
		myHCountry.put("827","[IT=3, BR=3, AR=1]");
		myHCountry.put("828","[BR=24, PY=2, GW=1, IT=1, ZA=1]");
		myHCountry.put("829","[MX=4, AR=2, GB=1, GP=1]");
		myHCountry.put("830","[CM=2, US=2, GP=1, FR=1]");
		myHCountry.put("831","[IT=2, FR=2, AL=1, IQ=1, GP=1, ES=1, MY=1, MZ=1, ZA=1, US=1, PT=1]");
		myHCountry.put("832","[FR=3, EG=1, TN=1, BR=1, MZ=1]");
		myHCountry.put("833","[FR=2]");
		myHCountry.put("834","[BR=3, FR=2, CN=1, ID=1, PT=1]");
		myHCountry.put("835","[FR=2, ES=1]");
		myHCountry.put("836","[CM=1, IT=1, US=1]");
		myHCountry.put("837","[CM=1, SE=1, FR=1]");
		myHCountry.put("838","[NG=19, CM=4, SA=1, BE=1, FR=1, SD=1]");
		myHCountry.put("839","[CM=2, FI=1, FR=1]");
		myHCountry.put("840","[IT=3, CM=2, SE=1]");
		myHCountry.put("841","[CM=3]");
		myHCountry.put("842","[CM=5, IT=1]");
		myHCountry.put("843","[CM=1, MX=1, FR=1, AT=1, PL=1]");
		myHCountry.put("844","[CM=2, FR=1]");
		myHCountry.put("845","[CM=3]");
		myHCountry.put("846","[CM=2, SA=1, IT=1, US=1, NG=1]");
		myHCountry.put("847","[CM=6]");
		myHCountry.put("848","[SA=7, SE=5, US=5, CM=4, CN=2, NG=2, FI=1, DE=1, GB=1, LY=1]");
		myHCountry.put("849","[US=3, PE=2, CM=1, IT=1, SN=1, GM=1, DE=1]");
		myHCountry.put("850","[CM=2]");
		myHCountry.put("851","[CM=2]");
		myHCountry.put("852","[CM=18, SA=3, NG=3, BE=1, US=1, FR=1]");
		myHCountry.put("853","[CM=13, AT=3, SA=2, CN=2, BE=2, TR=1, IT=1, DE=1, BR=1, US=1, FR=1, NG=1]");
		myHCountry.put("854","[CM=1, CI=1]");
		myHCountry.put("855","[CM=2]");
		myHCountry.put("856","[CM=4, TN=1]");
		myHCountry.put("857","[CM=2]");
		myHCountry.put("858","[CM=3, TN=1]");
		myHCountry.put("859","[CM=2, US=1]");
		myHCountry.put("860","[CM=1, CI=1]");
		myHCountry.put("861","[CM=2]");
		myHCountry.put("862","[CM=2]");
		myHCountry.put("863","[BR=4, US=1]");
		myHCountry.put("864","[US=8, IN=2, SA=1, SR=1, IR=1, NL=1, OM=1]");
		myHCountry.put("865","[US=3]");
		myHCountry.put("866","[BR=14, IT=10, US=7, MA=6, FR=6, ES=2, GW=1, RU=1, MX=1, VE=1]");
		myHCountry.put("867","[US=10, ES=2, PE=1]");
		myHCountry.put("868","[US=3]");
		myHCountry.put("869","[US=4]");
		myHCountry.put("870","[US=2]");
		myHCountry.put("871","[US=10, ES=3, IT=2, AR=2, FR=2, AM=1, FI=1]");
		myHCountry.put("872","[US=5, IT=2, GM=2, FR=2, NL=1]");
		myHCountry.put("873","[US=2, NL=1]");
		myHCountry.put("874","[SA=3, US=2, AL=1]");
		myHCountry.put("875","[US=3, FI=1]");
		myHCountry.put("876","[IN=3, NL=2, US=1]");
		myHCountry.put("877","[TR=2, KZ=2, ES=2, ZA=2, US=2, FI=1, NL=1, FR=1]");
		myHCountry.put("878","[BG=9, TR=4, AL=2, IQ=2, AT=2, IT=1, EG=1, BE=1, GB=1, LY=1, US=1]");
		myHCountry.put("879","[ZA=2, SA=1, TZ=1, ZM=1, RU=1, AR=1, US=1, BG=1, FR=1]");
		myHCountry.put("880","[MX=4, US=3, ZA=1, VE=1]");
		myHCountry.put("881","[CO=18, US=4, IT=1, ES=1, VE=1]");
		myHCountry.put("882","[BD=19, SA=6, US=2, OM=2, IT=1, IN=1, MY=1, GE=1]");
		myHCountry.put("883","[ES=7, US=3, IT=2, TN=2, IN=2, BG=1]");
		myHCountry.put("884","[ES=3, US=3, ZA=2, SA=1, TR=1, IT=1]");
		myHCountry.put("885","[US=2]");
		myHCountry.put("886","[US=1]");
		myHCountry.put("887","[US=1]");
		myHCountry.put("888","[ZA=4, CN=3, US=2, GW=1, IT=1, PE=1, PK=1, ES=1, SE=1, MX=1, PL=1]");
		myHCountry.put("889","[US=2, NL=1]");
		myHCountry.put("890","[US=9]");
		myHCountry.put("891","[PL=17, IT=3, US=3, BR=2, NL=1, ES=1, FR=1]");
		myHCountry.put("892","[MX=5, US=3, TN=1]");
		myHCountry.put("893","[IT=3, PE=3, SN=1, GM=1, US=1]");
		myHCountry.put("894","[US=7, PH=4, MY=1, SG=1, MX=1]");
		myHCountry.put("895","[US=2, PH=1]");
		myHCountry.put("896","[US=2, CU=1]");
		myHCountry.put("897","[US=6, AU=1, MY=1, SG=1]");
		myHCountry.put("898","[US=4, ES=2, NL=1, FR=1]");
		myHCountry.put("899","[US=2]");
		myHCountry.put("900","[US=2]");
		myHCountry.put("901","[US=6]");
		myHCountry.put("902","[US=3]");
		myHCountry.put("903","[US=3]");
		myHCountry.put("904","[US=8]");
		myHCountry.put("905","[VE=10, US=5, AR=1]");
		myHCountry.put("906","[US=4]");
		myHCountry.put("907","[US=2, OM=1]");
		myHCountry.put("908","[US=7, DO=2, BR=1]");
		myHCountry.put("909","[CA=9, US=7, GF=1]");
		myHCountry.put("910","[ET=18, US=3, FR=2]");
		myHCountry.put("911","[US=26, CO=3, BR=2, GP=1, AR=1, FR=1]");
		myHCountry.put("912","[US=4, FR=4, NO=1, SE=1]");
		myHCountry.put("913","[CN=7, US=3, SE=2, AT=2, FI=1, TW=1, ES=1, MW=1, MX=1, VE=1]");
		myHCountry.put("914","[CA=22, FI=7, IT=2, US=2, HT=1, BR=1, VE=1]");
		myHCountry.put("915","[FI=2]");
		myHCountry.put("916","[US=4, FI=2]");
		myHCountry.put("917","[US=3, FI=2, DK=1, CN=1, GB=1, SE=1, LV=1]");
		myHCountry.put("918","[SE=10, EE=4, FR=2, FI=1, IT=1]");
		myHCountry.put("919","[CZ=2, PL=2, FI=1, US=1]");
		myHCountry.put("920","[FI=2]");
		myHCountry.put("921","[FI=5, RU=1, SE=1, FR=1]");
		myHCountry.put("922","[FI=2]");
		myHCountry.put("923","[FI=3]");
		myHCountry.put("924","[GF=4, SE=4, TN=3, KM=2, FI=1, GB=1, NL=1, BR=1, ET=1, MZ=1, US=1, MU=1]");
		myHCountry.put("925","[FI=3, AT=2, FR=1]");
		myHCountry.put("926","[ZA=16, US=4, AM=3, TR=2, FI=1, MA=1, IT=1, GF=1, SE=1, FR=1, VE=1]");
		myHCountry.put("927","[FI=2]");
		myHCountry.put("928","[FI=8, SE=8, US=2, SA=1, FR=1]");
		myHCountry.put("929","[FI=1, CN=1, GB=1]");
		myHCountry.put("930","[TR=9, BE=5, FI=2, SA=2, NL=1]");
		myHCountry.put("931","[FI=1, NL=1]");
		myHCountry.put("932","[ZM=6, US=2, FI=1, BE=1, RU=1]");
		myHCountry.put("933","[US=5, ES=2, RU=2, FI=1]");
		myHCountry.put("934","[US=3, IN=2, SE=2, FI=1, TH=1, GF=1]");
		myHCountry.put("935","[US=6, FI=2, TW=1, ZA=1]");
		myHCountry.put("936","[MQ=2]");
		myHCountry.put("937","[TH=3]");
		myHCountry.put("938","[TH=2]");
		myHCountry.put("939","[TH=2, IN=1, US=1]");
		myHCountry.put("940","[TH=2, TW=1, IT=1, VN=1, US=1]");
		myHCountry.put("941","[CN=9, US=4, FR=3, TW=2, TH=2, JP=2, GB=1, MX=1]");
		myHCountry.put("942","[TH=3, CN=1, US=1]");
		myHCountry.put("943","[TH=3, NZ=2, ZA=2, FR=2, TN=1, US=1, AT=1, JP=1]");
		myHCountry.put("944","[MY=2, MQ=1, TH=1]");
		myHCountry.put("945","[TH=2]");
		myHCountry.put("946","[TW=6, CN=4, SA=3, ZA=2, ID=1, TH=1, NL=1, SE=1, JP=1]");
		myHCountry.put("947","[MM=3, SE=3, IN=2, US=2, DK=1, TH=1, MY=1]");
		myHCountry.put("948","[US=5, IT=2, PE=2, ES=1, MX=1]");
		myHCountry.put("949","[PE=2]");
		myHCountry.put("950","[US=6, BZ=2, JM=2, HN=2, AU=1]");
		myHCountry.put("951","[MY=9, MM=4, TH=4, NZ=2, US=2, DK=1, AU=1, SG=1]");
		myHCountry.put("952","[SE=3, IN=2, US=2, BD=1, ET=1]");
		myHCountry.put("953","[US=1, AT=1]");
		myHCountry.put("954","[IN=7, US=3, IQ=2, IR=1, GB=1, PK=1, OM=1]");
		myHCountry.put("955","[US=4, VN=1, JP=1]");
		myHCountry.put("956","[US=6, CN=3, JP=2, TN=1, BR=1]");
		myHCountry.put("957","[US=2]");
		myHCountry.put("958","[US=8, GY=3, TT=2, TN=2, BB=2, GF=2, SR=1, GB=1]");
		myHCountry.put("959","[US=2, GB=1]");
		myHCountry.put("960","[VE=7, BR=2, US=2]");
		myHCountry.put("961","[SN=1, GE=1, US=1]");
		myHCountry.put("962","[IN=3, MY=3, US=1, OM=1]");
		myHCountry.put("963","[IT=3, US=2, CZ=2, TW=1, NL=1, SE=1]");
		myHCountry.put("964","[MA=8, SE=5, FR=4, TZ=1, ES=1, ZA=1, US=1]");
		myHCountry.put("965","[FR=2]");
		myHCountry.put("966","[FR=3, ZA=2, SA=1, TR=1, EE=1, NL=1, GR=1, AR=1, MU=1, NG=1]");
		myHCountry.put("967","[ES=3, FR=1]");
		myHCountry.put("968","[FR=2]");
		myHCountry.put("969","[FR=10]");
		myHCountry.put("970","[FR=28, ES=2]");
		myHCountry.put("971","[FR=3]");
		myHCountry.put("972","[ES=7, FR=3]");
		myHCountry.put("973","[FR=2]");
		myHCountry.put("974","[FR=17]");
		myHCountry.put("975","[FR=18]");
		myHCountry.put("976","[FR=5]");
		myHCountry.put("977","[ES=123, IT=2, DE=2, SE=1, AR=1, FR=1]");
		myHCountry.put("978","[FR=7, AT=2, HU=1]");
		myHCountry.put("979","[IT=1, BE=1, FR=1]");
		myHCountry.put("980","[ES=339, FR=16, NL=1]");
		myHCountry.put("981","[FR=5]");
		myHCountry.put("982","[ES=82, FR=1]");
		myHCountry.put("983","[FR=3]");
		myHCountry.put("984","[FR=2]");
		myHCountry.put("985","[FR=3, ES=1]");
		myHCountry.put("986","[DE=2, ES=2, FR=2, BE=1]");
		myHCountry.put("987","[FR=2, MG=2]");
		myHCountry.put("988","[ES=2, FR=2]");
		myHCountry.put("989","[ES=3, US=3, FR=2]");
		myHCountry.put("990","[FR=17, BE=4]");
		myHCountry.put("991","[FR=28, BE=1]");
		myHCountry.put("992","[FR=2]");
		myHCountry.put("993","[ES=3, FR=3]");
		myHCountry.put("994","[FR=2, ES=1]");
		myHCountry.put("995","[ES=11, FR=4]");
		myHCountry.put("996","[FR=2]");
		myHCountry.put("997","[FR=20, ES=10, MX=5, BR=3, IT=2, AR=2, DZ=1, IR=1]");
		myHCountry.put("998","[FR=2]");
		myHCountry.put("999","[FR=22]");
		myHCountry.put("1000","[FR=19]");
		myHCountry.put("1001","[FR=4]");
		myHCountry.put("1002","[ES=52, FR=9]");
		myHCountry.put("1003","[ES=15, FR=4, MW=1]");
		myHCountry.put("1004","[IT=48, FR=4]");
		myHCountry.put("1005","[ES=92, FR=1]");
		myHCountry.put("1006","[FR=2]");
		myHCountry.put("1007","[FR=2]");
		myHCountry.put("1008","[ES=7, AR=3, GB=1, ZA=1, MX=1, FR=1]");
		myHCountry.put("1009","[FR=2]");
		myHCountry.put("1010","[ES=8, FR=3, PT=2, BR=1, US=1]");
		myHCountry.put("1011","[FR=2, NL=1]");
		myHCountry.put("1012","[FR=2]");
		myHCountry.put("1013","[FR=8]");
		myHCountry.put("1014","[FR=4]");
		myHCountry.put("1015","[FR=5]");
		myHCountry.put("1016","[FR=13]");
		myHCountry.put("1017","[FR=3]");
		myHCountry.put("1018","[ES=106, PT=3]");
		myHCountry.put("1019","[DE=3, FR=3, DK=1]");
		myHCountry.put("1020","[FR=7]");
		myHCountry.put("1021","[BR=22, ES=18, FR=4, MX=2]");
		myHCountry.put("1022","[DE=8, FR=8, BR=2, ES=1]");
		myHCountry.put("1023","[ES=38, FR=4, BR=1, ZA=1]");
		myHCountry.put("1024","[FR=2]");
		myHCountry.put("1025","[ES=5, FR=3]");
		myHCountry.put("1026","[FR=3, ES=2, MA=1]");
		myHCountry.put("1027","[NL=1, FR=1]");
		myHCountry.put("1028","[ES=91, NL=1, FR=1]");
		myHCountry.put("1029","[FR=3]");
		myHCountry.put("1030","[FR=26, DE=1, ES=1]");
		myHCountry.put("1031","[FR=2]");
		myHCountry.put("1032","[FR=4]");
		myHCountry.put("1033","[FR=5, DE=1]");
		myHCountry.put("1034","[FR=3]");
		myHCountry.put("1035","[FR=2]");
		myHCountry.put("1036","[FR=4, DZ=2, TD=1]");
		myHCountry.put("1037","[NG=94, TD=35, CM=6, FR=3, IT=1, ML=1, US=1]");
		myHCountry.put("1038","[FR=8]");
		myHCountry.put("1039","[FR=3, ES=1]");
		myHCountry.put("1040","[ES=21, FR=15, DE=1, AT=1]");
		myHCountry.put("1041","[FR=9, ES=3]");
		myHCountry.put("1042","[FR=7, ES=1]");
		myHCountry.put("1043","[FR=6]");
		myHCountry.put("1044","[FR=56, ES=47]");
		myHCountry.put("1045","[FR=5, BE=2]");
		myHCountry.put("1046","[FR=5]");
		myHCountry.put("1047","[ES=60, FR=12, IT=2, DZ=1, BR=1]");
		myHCountry.put("1048","[FR=10]");
		myHCountry.put("1049","[GB=2]");
		myHCountry.put("1050","[RU=2, AZ=1, US=1]");
		myHCountry.put("1051","[PL=21, TR=1, AZ=1, NL=1, ES=1]");
		myHCountry.put("1052","[EG=1, FR=1]");
		myHCountry.put("1053","[BR=3, EG=1, MX=1, FR=1]");
		myHCountry.put("1054","[CF=5, US=2, FR=2]");
		myHCountry.put("1055","[CF=3, GB=1]");
		myHCountry.put("1056","[SA=4, FR=3, CM=1, CF=1, US=1]");
		myHCountry.put("1057","[CM=3, NG=2]");
		myHCountry.put("1058","[MA=1, CM=1, IT=1, ZA=1, AT=1]");
		myHCountry.put("1059","[VN=2, GF=1]");
		myHCountry.put("1060","[VN=2, MX=1]");
		myHCountry.put("1061","[US=2, MG=1]");
		myHCountry.put("1062","[MZ=5, TN=1, GF=1, MG=1]");
		myHCountry.put("1063","[DZ=22, PY=1, DE=1]");
		myHCountry.put("1064","[MA=11, ZA=5, DZ=4, BE=4, FR=4, IT=3, TN=3, CO=1]");
		myHCountry.put("1065","[DZ=2]");
		myHCountry.put("1066","[GF=3, BR=2, NG=2, DZ=1, SE=1, US=1, VE=1]");
		myHCountry.put("1067","[ZA=26, FR=4, US=2, DZ=1, MA=1]");
		myHCountry.put("1068","[MA=4, TR=1, IT=1, BE=1]");
		myHCountry.put("1069","[MA=8, AT=7, ES=3, ZA=2, US=1, FR=1, PT=1]");
		myHCountry.put("1070","[MA=7, FR=4, IT=3, HT=1, CO=1, BE=1, BD=1, ES=1, US=1, AT=1]");
		myHCountry.put("1071","[MA=4, ES=1, FR=1]");
		myHCountry.put("1072","[MA=5, ES=2]");
		myHCountry.put("1073","[EG=2, IN=2, FR=2, MA=1]");
		myHCountry.put("1074","[MA=11, FR=6, ZA=5, DE=2, ES=2, US=2, IT=1, BR=1, ET=1]");
		myHCountry.put("1075","[MA=2, BE=1, BR=1, US=1, FR=1]");
		myHCountry.put("1076","[BR=5, RU=2, ID=1, MZ=1, US=1]");
		myHCountry.put("1077","[ID=2, CN=1, VN=1, ET=1, AT=1]");
		myHCountry.put("1078","[ID=2]");
		myHCountry.put("1079","[ID=1, NL=1, MY=1]");
		myHCountry.put("1080","[US=3, ID=2, IT=1, PE=1]");
		myHCountry.put("1081","[SR=1, ID=1, US=1]");
		myHCountry.put("1082","[ID=1, NL=1]");
		myHCountry.put("1083","[IN=4, US=4, SA=3, MY=2, ID=1]");
		myHCountry.put("1084","[GF=5, SR=1]");
		myHCountry.put("1085","[SA=1, MQ=1, FR=1]");
		myHCountry.put("1086","[MQ=3]");
		myHCountry.put("1087","[GP=2, HT=1, US=1]");
		myHCountry.put("1088","[IN=5, IQ=4, EG=1, ZA=1, OM=1, PT=1]");
		myHCountry.put("1089","[IN=3, US=1, OM=1]");
		myHCountry.put("1090","[IN=3, US=2, TZ=1, NL=1]");
		myHCountry.put("1091","[US=6, IN=3, SA=2, PK=2, IR=1, SE=1]");
		myHCountry.put("1092","[ZA=15, IN=3, PK=2, IT=1, IR=1, BE=1, US=1]");
		myHCountry.put("1093","[IN=6, BE=3, US=2, SA=1, GB=1, FR=1, OM=1]");
		myHCountry.put("1094","[IN=2]");
		myHCountry.put("1095","[IN=2, SA=1]");
		myHCountry.put("1096","[CN=8, IN=2]");
		myHCountry.put("1097","[IN=2, MM=1]");
		myHCountry.put("1098","[US=2, CN=1, AU=1]");
		myHCountry.put("1099","[US=3]");
		myHCountry.put("1100","[US=30]");
		myHCountry.put("1101","[US=2]");
		myHCountry.put("1102","[US=2]");
		myHCountry.put("1103","[US=3, RU=1]");
		myHCountry.put("1104","[VE=2]");
		myHCountry.put("1105","[PE=6, US=5, SA=3, CN=3, MA=2, TR=2, IT=1, EG=1, ES=1]");
		myHCountry.put("1106","[PT=9, BR=6, ES=4, IT=1, PE=1]");
		myHCountry.put("1107","[IT=1, EG=1, PE=1, SE=1, FR=1]");
		myHCountry.put("1108","[DE=3, AT=1]");
		myHCountry.put("1109","[AT=6]");
		myHCountry.put("1110","[AT=20]");
		myHCountry.put("1111","[AT=20, DE=1]");
		myHCountry.put("1112","[US=3, AT=2]");
		myHCountry.put("1113","[AT=8, BE=3, IT=1, ES=1, ZA=1]");
		myHCountry.put("1114","[AT=4, LV=1, PL=1]");
		myHCountry.put("1115","[AT=3]");
		myHCountry.put("1116","[AT=2]");
		myHCountry.put("1117","[RU=2, LV=2, AT=1]");
		myHCountry.put("1118","[AT=2, US=1]");
		myHCountry.put("1119","[US=3, AT=1]");
		myHCountry.put("1120","[BD=2, SA=1, DE=1, PK=1, IN=1, AT=1]");
		myHCountry.put("1121","[AT=34]");
		myHCountry.put("1122","[BG=15, AT=8, GB=7, ES=4, TR=4, IT=4, US=4, CN=2, PE=2, BE=2, FR=2, CZ=2, AL=1, PK=1, AR=1]");
		myHCountry.put("1123","[BE=2, US=2, AU=1, AT=1]");
		myHCountry.put("1124","[AT=5]");
		myHCountry.put("1125","[AT=4, FR=1]");
		myHCountry.put("1126","[GB=1, AT=1]");
		myHCountry.put("1127","[CA=3, AT=3, IT=1, US=1, BG=1]");
		myHCountry.put("1128","[AT=4, IT=1, FR=1]");
		myHCountry.put("1129","[US=3, FR=3, AT=3, MA=1, CN=1, BR=1, IN=1, ZA=1, BG=1]");
		myHCountry.put("1130","[AT=8, BE=1]");
		myHCountry.put("1131","[AT=3, DE=1, GP=1]");
		myHCountry.put("1132","[GB=1, AT=1]");
		myHCountry.put("1133","[US=1, AT=1]");
		myHCountry.put("1134","[RU=18, GE=5, US=4, AM=2, KZ=2, TR=1, AT=1]");
		myHCountry.put("1135","[MA=3, FR=2, HU=1, IT=1, AT=1]");
		myHCountry.put("1136","[AT=6, US=2]");
		myHCountry.put("1137","[AT=4]");
		myHCountry.put("1138","[AT=3, IT=1]");
		myHCountry.put("1139","[PE=2, SE=2, US=2, MA=1, AT=1]");
		myHCountry.put("1140","[AT=7]");
		myHCountry.put("1141","[IR=1, US=1, AT=1]");
		myHCountry.put("1142","[AT=6, IT=1]");
		myHCountry.put("1143","[SE=1, AT=1, PL=1]");
		myHCountry.put("1144","[IQ=37, US=2, GR=1, VE=1]");
		myHCountry.put("1145","[HT=1, US=1]");
		myHCountry.put("1146","[CU=2, US=1]");
		myHCountry.put("1147","[US=1, VE=1]");
		myHCountry.put("1148","[US=2]");
		myHCountry.put("1149","[US=4, TN=1, TH=1]");
		myHCountry.put("1150","[PE=1, US=1]");
		myHCountry.put("1151","[IN=2, US=2, GB=1]");
		myHCountry.put("1152","[US=3]");
		myHCountry.put("1153","[NL=1, CH=1, US=1]");
		myHCountry.put("1154","[US=6, BR=3, CZ=2, HT=1, MA=1, AU=1, ES=1]");
		myHCountry.put("1155","[US=6, MA=1, IT=1, BR=1]");
		myHCountry.put("1156","[HN=2, US=2]");
		myHCountry.put("1157","[US=2]");
		myHCountry.put("1158","[KR=12, US=4, AR=2, TW=1]");
		myHCountry.put("1159","[US=7, SE=1]");
		myHCountry.put("1160","[US=2, PE=1, VE=1]");
		myHCountry.put("1161","[US=3]");
		myHCountry.put("1162","[CN=8, US=7, MY=1]");
		myHCountry.put("1163","[BR=10, US=5, IN=1]");
		myHCountry.put("1164","[US=5]");
		myHCountry.put("1165","[US=2, RU=1]");
		myHCountry.put("1166","[US=9, BR=8, ZA=4, SA=3, BE=3, GR=3, IT=3, ES=2, VE=2, CN=1, TZ=1, PE=1, EG=1, PK=1, TN=1, GB=1, MW=1, MX=1]");
		myHCountry.put("1167","[US=3, FR=1]");
		myHCountry.put("1168","[US=3, CN=2, IT=1, IN=1]");
		myHCountry.put("1169","[PH=4, IT=2, US=2, SG=1, SE=1]");
		myHCountry.put("1170","[PH=2, US=1]");
		myHCountry.put("1171","[TH=1, PH=1, US=1]");
		myHCountry.put("1172","[EE=3]");
		myHCountry.put("1173","[SE=3, FI=1, EE=1, DE=1, RU=1]");
		myHCountry.put("1174","[EE=2]");
		myHCountry.put("1175","[EE=4, LV=1]");
		myHCountry.put("1176","[ID=4, BD=4, GE=3, EE=1, EG=1, BE=1, NL=1, BR=1, AR=1, FR=1, LV=1, MG=1]");
		myHCountry.put("1177","[EE=3, PE=1]");
		myHCountry.put("1178","[SA=6, NL=1]");
		myHCountry.put("1179","[US=2, HN=1]");
		myHCountry.put("1180","[US=2, SE=1]");
		myHCountry.put("1181","[ES=26, IR=18, BE=2, SE=2]");
		myHCountry.put("1182","[SE=2]");
		myHCountry.put("1183","[DK=1, NL=1, SE=1]");
		myHCountry.put("1184","[RU=5, VN=1, SE=1, FR=1]");
		myHCountry.put("1185","[SE=3, FR=1]");
		myHCountry.put("1186","[TH=3, NL=1]");
		myHCountry.put("1187","[TH=4]");
		myHCountry.put("1188","[TH=2]");
		myHCountry.put("1189","[MY=2, US=2, JP=2, SA=1, BE=1, TH=1]");
		myHCountry.put("1190","[VN=8]");
		myHCountry.put("1191","[VN=2]");
		myHCountry.put("1192","[CN=4, IN=1, GE=1, US=1, MG=1]");
		myHCountry.put("1193","[MX=5, IN=2]");
		myHCountry.put("1194","[IN=2]");
		myHCountry.put("1195","[IN=2]");
		myHCountry.put("1196","[IN=2, SA=1, TZ=1, EG=1, US=1]");
		myHCountry.put("1197","[NO=21]");
		myHCountry.put("1198","[IQ=10, SD=8, IR=6, SA=3, GB=2, ET=2, ZA=2, US=2, DE=1, SE=1, FR=1]");
		myHCountry.put("1199","[SA=7, US=2, ET=1, SD=1]");
		myHCountry.put("1200","[US=3, GM=2, ET=2, FR=2, GW=1, SD=1]");
		myHCountry.put("1201","[TR=2, CM=1, SD=1]");
		myHCountry.put("1202","[BR=3, LY=1, MG=1]");
		myHCountry.put("1203","[LY=1, AT=1]");
		myHCountry.put("1204","[US=2, NG=2, GW=1, SA=1, LY=1]");
		myHCountry.put("1205","[IT=3, US=1]");
		myHCountry.put("1206","[IT=1, US=1]");
		myHCountry.put("1207","[IT=2]");
		myHCountry.put("1208","[US=1, FR=1]");
		myHCountry.put("1209","[MX=2, US=1]");
		myHCountry.put("1210","[BR=2, MX=2, US=2, IT=1]");
		myHCountry.put("1211","[MX=12, CN=2, SE=2, ES=1, US=1]");
		myHCountry.put("1212","[MX=2]");
		myHCountry.put("1213","[US=2, DO=1]");
		myHCountry.put("1214","[MG=10, CN=3, PL=2, IT=1, PE=1, DE=1, BR=1, GF=1]");
		myHCountry.put("1215","[MX=6, US=2, VE=1]");
		myHCountry.put("1216","[US=3]");
		myHCountry.put("1217","[US=2]");
		myHCountry.put("1218","[GB=1, US=1]");
		myHCountry.put("1219","[MX=4]");
		myHCountry.put("1220","[US=2, PE=1]");
		myHCountry.put("1221","[MZ=1, MX=1, FR=1]");
		myHCountry.put("1222","[ID=1, US=1]");
		myHCountry.put("1223","[US=3, SA=1, TN=1, ES=1, MG=1]");
		myHCountry.put("1224","[MX=3, US=3]");
		myHCountry.put("1225","[MX=2]");
		myHCountry.put("1226","[TW=3, US=3, CN=1]");
		myHCountry.put("1227","[ES=6, BR=2, US=2, MA=1, DE=1, ZA=1]");
		myHCountry.put("1228","[AU=1, MX=1]");
		myHCountry.put("1229","[US=1, AT=1, PL=1]");
		myHCountry.put("1230","[IT=1, PE=1, NL=1, AR=1, US=1]");
		myHCountry.put("1231","[US=2]");
		myHCountry.put("1232","[US=3, IT=1]");
		myHCountry.put("1233","[TR=3, MX=2, IT=1, GP=1, US=1]");
		myHCountry.put("1234","[US=6]");
		myHCountry.put("1235","[PE=1, MX=1, US=1]");
		myHCountry.put("1236","[US=12, MX=7]");
		myHCountry.put("1237","[US=4, MX=1]");
		myHCountry.put("1238","[GP=1, BR=1, MX=1, US=1, VE=1]");
		myHCountry.put("1239","[MX=1, US=1]");
		myHCountry.put("1240","[LV=2, ET=1, RU=1, US=1]");
		myHCountry.put("1241","[ZA=43, BR=21, AR=13, US=2, TN=1, GF=1, SE=1]");
		myHCountry.put("1242","[IT=6, US=2]");
		myHCountry.put("1243","[ES=4, TR=2, MX=2, SA=1, TW=1, IT=1, CO=1, NL=1, BR=1, MY=1, FR=1]");
		myHCountry.put("1244","[CA=2, PE=1]");
		myHCountry.put("1245","[CA=2, US=1]");
		myHCountry.put("1246","[CA=7, EE=1]");
		myHCountry.put("1247","[RU=9, KZ=2, IR=1, IN=1, ZA=1, LV=1]");
		myHCountry.put("1248","[RU=3]");
		myHCountry.put("1249","[RU=3, US=1]");
		myHCountry.put("1250","[RU=1, FR=1]");
		myHCountry.put("1251","[MZ=7, DK=1, NO=1]");
		myHCountry.put("1252","[FR=2, TR=1, EG=1, AU=1, RU=1, US=1, BG=1]");
		myHCountry.put("1253","[ES=4, RU=4, TR=3, FR=2, PL=2, DE=1, ZA=1, AR=1]");
		myHCountry.put("1254","[NL=1, RU=1, US=1]");
		myHCountry.put("1255","[RU=2, NL=1]");
		myHCountry.put("1256","[US=2, KZ=1, RU=1]");
		myHCountry.put("1257","[RU=1, AT=1]");
		myHCountry.put("1258","[CA=41, TR=1, DE=1]");
		myHCountry.put("1259","[RU=2, US=1]");
		myHCountry.put("1260","[MX=1, US=1]");
		myHCountry.put("1261","[TR=25, BE=2, DK=1, US=1, BG=1, FR=1, AT=1]");
		myHCountry.put("1262","[MX=2]");
		myHCountry.put("1263","[BE=2, IN=1]");
		myHCountry.put("1264","[IN=9, SA=3, PK=3, ET=2, US=2, GY=1, FR=1]");
		myHCountry.put("1265","[US=2, IT=1]");
		myHCountry.put("1266","[IN=4, US=2, BE=1, OM=1]");
		myHCountry.put("1267","[IN=1, US=1, FR=1]");
		myHCountry.put("1268","[IN=3, US=2, PK=1, MY=1, OM=1]");
		myHCountry.put("1269","[RU=1, US=1]");
		myHCountry.put("1270","[US=1]");
		myHCountry.put("1271","[US=1, VE=1]");
		myHCountry.put("1272","[HT=1, US=1]");
		myHCountry.put("1273","[NZ=2, ZA=1, US=1]");
		myHCountry.put("1274","[FI=1, US=1]");
		myHCountry.put("1275","[AT=8, US=2]");
		myHCountry.put("1276","[RU=2, US=2, LV=1]");
		myHCountry.put("1277","[DO=8, BR=3, MX=3, US=3, ES=2, AR=2, MA=1, CO=1, GF=1, RU=1, LV=1]");
		myHCountry.put("1278","[FR=4, PL=4, CZ=3, IT=2, ES=1, US=1, AT=1]");
		myHCountry.put("1279","[CZ=6, US=2]");
		myHCountry.put("1280","[BG=6, CZ=3, US=2, IT=1, AL=1, EG=1, AR=1]");
		myHCountry.put("1281","[CZ=3, NL=1, US=1]");
		myHCountry.put("1282","[IT=6, FR=2, CZ=2, US=1]");
		myHCountry.put("1283","[US=5, HT=1, TR=1, IT=1, CO=1]");
		myHCountry.put("1284","[BR=5, BE=3, US=2, GF=1, FR=1]");
		myHCountry.put("1285","[NO=2]");
		myHCountry.put("1286","[NO=2]");
		myHCountry.put("1287","[NO=4]");
		myHCountry.put("1288","[RU=4]");
		myHCountry.put("1289","[RU=1, SD=1]");
		myHCountry.put("1290","[RU=3]");
		myHCountry.put("1291","[RU=2, US=1]");
		myHCountry.put("1292","[RU=1, LV=1, AT=1]");
		myHCountry.put("1293","[ZA=6, US=1]");
		myHCountry.put("1294","[ZA=10, US=1]");
		myHCountry.put("1295","[ZA=8, AR=2, TR=1]");
		myHCountry.put("1296","[ZA=3, NL=1]");
		myHCountry.put("1297","[GB=2]");
		myHCountry.put("1298","[MX=2]");
		myHCountry.put("1299","[MX=4]");
		myHCountry.put("1300","[US=4]");
		myHCountry.put("1301","[ZM=8, IN=4, ZA=3]");
		myHCountry.put("1302","[CN=12, IT=6, TW=1, CM=1, ZA=1, US=1]");
		myHCountry.put("1303","[IE=2, CL=1, BE=1]");
		myHCountry.put("1304","[GB=2]");
		myHCountry.put("1305","[MX=4, BR=3]");
		myHCountry.put("1306","[MX=2, GB=1, US=1]");
		myHCountry.put("1307","[MX=3]");
		myHCountry.put("1308","[MX=2]");
		myHCountry.put("1309","[MX=3]");
		myHCountry.put("1310","[MX=2]");
		myHCountry.put("1311","[IN=3, CN=2, IT=1, AU=1, US=1]");
		myHCountry.put("1312","[ET=2, US=2, IN=1, SE=1, SD=1]");
		myHCountry.put("1313","[NL=1, IN=1]");
		myHCountry.put("1314","[IN=3, SA=1]");
		myHCountry.put("1315","[IN=1, FR=1]");
		myHCountry.put("1316","[IN=4, NL=1]");
		myHCountry.put("1317","[RO=1, IN=1]");
		myHCountry.put("1318","[IN=4, IQ=2, EG=1, GF=1]");
		myHCountry.put("1319","[IR=8, IN=2]");
		myHCountry.put("1320","[IN=2]");
		myHCountry.put("1321","[BR=5, RU=3, PT=3, IN=2, US=2, GF=1, AR=1, FR=1]");
		myHCountry.put("1322","[IN=1, US=1]");
		myHCountry.put("1323","[IN=2]");
		myHCountry.put("1324","[ZA=3, IN=2, US=1]");
		myHCountry.put("1325","[IN=3, SA=1]");
		myHCountry.put("1326","[PK=1, IN=1]");
		myHCountry.put("1327","[US=4, IN=3, PK=1]");
		myHCountry.put("1328","[HN=5, HT=2, US=2]");
		myHCountry.put("1329","[ZA=75, MX=2, US=2]");
		myHCountry.put("1330","[ZA=3, TR=1, RU=1]");
		myHCountry.put("1331","[ZA=2]");
		myHCountry.put("1332","[CN=5, ZA=5, FR=4, BR=2, MG=2, FI=1, DE=1]");
		myHCountry.put("1333","[ZA=4]");
		myHCountry.put("1334","[ZA=8, US=1]");
		myHCountry.put("1335","[ZA=4]");
		myHCountry.put("1336","[ZA=2]");
		myHCountry.put("1337","[BR=8, GF=6, RU=3, IT=1]");
		myHCountry.put("1338","[DK=1, MQ=1, AL=1, SL=1, JP=1]");
		myHCountry.put("1339","[GF=1, US=1]");
		myHCountry.put("1340","[SR=18, GF=14, GY=6, TN=4, US=2, GP=1]");
		myHCountry.put("1341","[US=10]");
		myHCountry.put("1342","[US=4, IN=2, DK=1, NL=1]");
		myHCountry.put("1343","[IN=5, US=3, SA=2, SE=2, NL=1, OM=1]");
		myHCountry.put("1344","[IN=3, NL=1]");
		myHCountry.put("1345","[US=3, IN=2, BD=1, NL=1, SE=1, OM=1]");
		myHCountry.put("1346","[IT=15, IN=1]");
		myHCountry.put("1347","[IT=8]");
		myHCountry.put("1348","[IT=2]");
		myHCountry.put("1349","[IT=9]");
		myHCountry.put("1350","[IT=2]");
		myHCountry.put("1351","[IT=3]");
		myHCountry.put("1352","[IT=3]");
		myHCountry.put("1353","[IT=5, NG=2, TR=1, TN=1]");
		myHCountry.put("1354","[PE=3, IT=2, HN=1]");
		myHCountry.put("1355","[PE=33, IT=10, US=7, ES=3, ZA=1, AR=1]");
		myHCountry.put("1356","[IT=10, PY=2, PE=2, BR=1, MX=1, US=1]");
		myHCountry.put("1357","[IT=2]");
		myHCountry.put("1358","[IT=3]");
		myHCountry.put("1359","[IT=16, GB=1, FR=1]");
		myHCountry.put("1360","[IT=3]");
		myHCountry.put("1361","[IT=5, US=1]");
		myHCountry.put("1362","[IT=3]");
		myHCountry.put("1363","[IT=2, AT=1, JP=1, OM=1]");
		myHCountry.put("1364","[JP=9, MY=2, ID=1, NL=1, US=1]");
		myHCountry.put("1365","[MY=2, PH=2, TZ=1, SG=1]");
		myHCountry.put("1366","[PA=4, SA=1, ID=1, MY=1, US=1]");
		myHCountry.put("1367","[OM=2, PY=1, NL=1, MY=1, FR=1]");
		myHCountry.put("1368","[MY=2]");
		myHCountry.put("1369","[PA=1, GB=1, MY=1]");
		myHCountry.put("1370","[MY=2]");
		myHCountry.put("1371","[MY=2]");
		myHCountry.put("1372","[US=2, VN=1, MY=1]");
		myHCountry.put("1373","[SA=1, IN=1, MY=1, US=1]");
		myHCountry.put("1374","[MY=3, IT=2]");
		myHCountry.put("1375","[SA=2, MY=2, AU=1]");
		myHCountry.put("1376","[MY=2]");
		myHCountry.put("1377","[NL=1, MY=1]");
		myHCountry.put("1378","[ES=21, SA=6, IN=5, KR=4, AM=1, EG=1, SD=1]");
		myHCountry.put("1379","[AM=3]");
		myHCountry.put("1380","[AM=2]");
		myHCountry.put("1381","[AM=6]");
		myHCountry.put("1382","[AM=2]");
		myHCountry.put("1383","[AM=1, HT=1]");
		myHCountry.put("1384","[AM=2]");
		myHCountry.put("1385","[AM=2]");
		myHCountry.put("1386","[BD=2, US=1]");
		myHCountry.put("1387","[AM=2]");
		myHCountry.put("1388","[BD=4, US=1, OM=1]");
		myHCountry.put("1389","[BD=9, SA=4, GB=1, GE=1, US=1]");
		myHCountry.put("1390","[SA=2, BD=2, US=2, MM=1, DE=1, IN=1]");
		myHCountry.put("1391","[BD=12, MM=3, IN=2, MY=2, US=2, SA=1, IT=1, OM=1]");
		myHCountry.put("1392","[BD=4, MY=1, US=1, FR=1]");
		myHCountry.put("1393","[BD=2]");
		myHCountry.put("1394","[BD=3, ZA=2, BR=1, MX=1, FR=1]");
		myHCountry.put("1395","[BD=2, US=2, SA=1]");
		myHCountry.put("1396","[BD=5]");
		myHCountry.put("1397","[BD=8]");
		myHCountry.put("1398","[BD=2]");
		myHCountry.put("1399","[BD=4]");
		myHCountry.put("1400","[BD=10]");
		myHCountry.put("1401","[PK=5, BD=3, GB=1, IN=1]");
		myHCountry.put("1402","[BD=2]");
		myHCountry.put("1403","[BD=3]");
		myHCountry.put("1404","[BD=3, OM=1]");
		myHCountry.put("1405","[BD=2, GB=1]");
		myHCountry.put("1406","[BD=4]");
		myHCountry.put("1407","[BD=2, SA=1]");
		myHCountry.put("1408","[SA=1, BD=1, NL=1, OM=1]");
		myHCountry.put("1409","[US=4, MM=2, TN=1, BD=1, JM=1]");
		myHCountry.put("1410","[BR=7, SA=3, GM=3, IT=2, PL=2, BD=1, ES=1, ET=1, BG=1, AT=1, PT=1]");
		myHCountry.put("1411","[BD=2]");
		myHCountry.put("1412","[BD=3]");
		myHCountry.put("1413","[BD=2]");
		myHCountry.put("1414","[BD=1, US=1]");
		myHCountry.put("1415","[BD=2]");
		myHCountry.put("1416","[BD=1, VN=1]");
		myHCountry.put("1417","[BD=4]");
		myHCountry.put("1418","[BD=3, MM=1]");
		myHCountry.put("1419","[BD=3]");
		myHCountry.put("1420","[BD=5, GB=1]");
		myHCountry.put("1421","[GB=1, GE=1]");
		myHCountry.put("1422","[BD=2, IN=2, GB=1]");
		myHCountry.put("1423","[BD=2]");
		myHCountry.put("1424","[BD=2]");
		myHCountry.put("1425","[BD=5, MM=1]");
		myHCountry.put("1426","[IN=2, GB=1, BD=1]");
		myHCountry.put("1427","[BD=3, GE=2]");
		myHCountry.put("1428","[BD=3]");
		myHCountry.put("1429","[BD=3]");
		myHCountry.put("1430","[BD=2, US=1]");
		myHCountry.put("1431","[BD=2]");
		myHCountry.put("1432","[BD=2]");
		myHCountry.put("1433","[BD=2]");
		myHCountry.put("1434","[BD=3]");
		myHCountry.put("1435","[BD=4]");
		myHCountry.put("1436","[BD=3]");
		myHCountry.put("1437","[BD=2]");
		myHCountry.put("1438","[BD=2, NP=1, US=1]");
		myHCountry.put("1439","[ES=4, BE=1]");
		myHCountry.put("1440","[BE=8]");
		myHCountry.put("1441","[BE=4]");
		myHCountry.put("1442","[BE=6]");
		myHCountry.put("1443","[BE=2]");
		myHCountry.put("1444","[BE=3]");
		myHCountry.put("1445","[GE=4]");
		myHCountry.put("1446","[GE=5]");
		myHCountry.put("1447","[GE=8, IR=2, RU=1, US=1]");
		myHCountry.put("1448","[GE=3]");
		myHCountry.put("1449","[GE=2]");
		myHCountry.put("1450","[GE=2]");
		myHCountry.put("1451","[GE=7, SA=4, CN=1, RU=1, US=1]");
		myHCountry.put("1452","[GE=5, TR=1]");
		myHCountry.put("1453","[GE=3]");
		myHCountry.put("1454","[GE=2]");
		myHCountry.put("1455","[GE=3]");
		myHCountry.put("1456","[GE=5]");
		myHCountry.put("1457","[GE=15, IR=11, ET=1, RU=1]");
		myHCountry.put("1458","[GE=2]");
		myHCountry.put("1459","[GE=1, US=1]");
		myHCountry.put("1460","[GE=2, KG=1, RU=1]");
		myHCountry.put("1461","[GE=2]");
		myHCountry.put("1462","[VN=1, IN=1, GE=1, US=1]");
		myHCountry.put("1463","[GE=3]");
		myHCountry.put("1464","[GE=2]");
		myHCountry.put("1465","[BE=3, CD=2, SE=1]");
		myHCountry.put("1466","[MW=5, ZW=1, BE=1]");
		myHCountry.put("1467","[MW=2, BE=1, ZM=1, MZ=1, ZA=1]");
		myHCountry.put("1468","[ZM=12, BE=7, MW=7, ZA=2, TZ=1, GB=1]");
		myHCountry.put("1469","[MW=2]");
		myHCountry.put("1470","[TZ=6, US=4, IQ=2, ZA=2, IT=1, BE=1, BR=1, MW=1, SE=1, MG=1, OM=1]");
		myHCountry.put("1471","[MW=5, TZ=3, BE=2, ZM=2, FR=1, OM=1]");
		myHCountry.put("1472","[MW=3]");
		myHCountry.put("1473","[MW=3, FR=1]");
		myHCountry.put("1474","[MW=2]");
		myHCountry.put("1475","[MW=2, CN=1, BE=1, BR=1, ET=1, AR=1, VE=1]");
		myHCountry.put("1476","[PE=7, US=3, AR=2, FR=1]");
		myHCountry.put("1477","[PH=2]");
		myHCountry.put("1478","[PH=2, TW=1]");
		myHCountry.put("1479","[US=4, MY=2, PH=2, DE=1, BE=1, SG=1]");
		myHCountry.put("1480","[RU=4]");
		myHCountry.put("1481","[RU=3]");
		myHCountry.put("1482","[RU=2, LV=1]");
		myHCountry.put("1483","[RU=2, US=1]");
		myHCountry.put("1484","[CU=1, RU=1]");
		myHCountry.put("1485","[RU=3, CN=1]");
		myHCountry.put("1486","[CO=1, GF=1, US=1]");
		myHCountry.put("1487","[TW=1, MY=1, US=1]");
		myHCountry.put("1488","[LK=1, MY=1, US=1]");
		myHCountry.put("1489","[MY=2]");
		myHCountry.put("1490","[US=3, MY=2]");
		myHCountry.put("1491","[BR=12, MY=3, TW=1, GB=1, US=1, JP=1]");
		myHCountry.put("1492","[MY=2]");
		myHCountry.put("1493","[GB=1, MY=1, SG=1, OM=1]");
		myHCountry.put("1494","[ID=3, BE=1, MY=1]");
		myHCountry.put("1495","[TH=1, MY=1]");
		myHCountry.put("1496","[MY=2]");
		myHCountry.put("1497","[TH=1, MY=1]");
		myHCountry.put("1498","[IT=7, US=4, MY=2, RU=1, NG=1, PL=1]");
		myHCountry.put("1499","[MY=2, US=1]");
		myHCountry.put("1500","[MY=2]");
		myHCountry.put("1501","[MY=3, PH=1, SE=1]");
		myHCountry.put("1502","[MY=2]");
		myHCountry.put("1503","[MY=2]");
		myHCountry.put("1504","[MY=2]");
		myHCountry.put("1505","[MY=2]");
		myHCountry.put("1506","[MY=3]");
		myHCountry.put("1507","[MY=2]");
		myHCountry.put("1508","[MY=2]");
		myHCountry.put("1509","[US=2, DE=1, PH=1]");
		myHCountry.put("1510","[IT=1, PH=1, US=1]");
		myHCountry.put("1511","[US=2, SG=1, PH=1]");
		myHCountry.put("1512","[MY=3, MM=1]");
		myHCountry.put("1513","[MY=2]");
		myHCountry.put("1514","[MG=4, US=1, FR=1]");
		myHCountry.put("1515","[MG=3, PK=1]");
		myHCountry.put("1516","[MG=3, ZA=1]");
		myHCountry.put("1517","[MG=6]");
		myHCountry.put("1518","[MG=16]");
		myHCountry.put("1519","[MG=2]");
		myHCountry.put("1520","[MG=3]");
		myHCountry.put("1521","[MG=2]");
		myHCountry.put("1522","[ZA=2, MG=2]");
		myHCountry.put("1523","[MG=2]");
		myHCountry.put("1524","[MG=3]");
		myHCountry.put("1525","[MG=3, FR=2, PE=1]");
		myHCountry.put("1526","[GF=3, SR=1, GY=1]");
		myHCountry.put("1527","[DE=1, SE=1, US=1]");
		myHCountry.put("1528","[MA=1, BR=1, ZA=1]");
		myHCountry.put("1529","[ES=1, AT=1]");
		myHCountry.put("1530","[US=4, ES=3, TZ=1, BR=1, ET=1, FR=1]");
		myHCountry.put("1531","[ES=2]");
		myHCountry.put("1532","[ES=1, US=1]");
		myHCountry.put("1533","[CO=2, IT=1, ES=1, US=1, FR=1, AT=1]");
		myHCountry.put("1534","[ES=7, US=6, CH=2, BE=1]");
		myHCountry.put("1535","[BR=4, FR=3, US=2, BE=1, ES=1, AR=1]");
		myHCountry.put("1536","[MA=7, BR=3, ES=3, ZA=3, US=2, CO=1, BE=1, HN=1]");
		myHCountry.put("1537","[MA=4, ES=3, ZA=2, IT=1, MX=1]");
		myHCountry.put("1538","[MA=6, BE=2, ES=2, FR=2, NL=1, ZA=1]");
		myHCountry.put("1539","[ES=12, MA=1, MY=1, MZ=1]");
		myHCountry.put("1540","[ID=1, ES=1]");
		myHCountry.put("1541","[ES=3, MA=1]");
		myHCountry.put("1542","[US=2, ES=1]");
		myHCountry.put("1543","[ES=2]");
		myHCountry.put("1544","[BE=4, FR=2, TR=1, IT=1, IN=1, US=1]");
		myHCountry.put("1545","[BE=5, BR=1]");
		myHCountry.put("1546","[US=3, NZ=2]");
		myHCountry.put("1547","[IQ=3, CN=2, DK=1, EG=1, ID=1, BR=1, MZ=1, PL=1]");
		myHCountry.put("1548","[BE=4, FR=3, OM=1]");
		myHCountry.put("1549","[BE=4, TZ=2, NL=1, ZA=1, FR=1]");
		myHCountry.put("1550","[AT=2]");
		myHCountry.put("1551","[US=4, IN=3, IT=1, GB=1, PK=1, ET=1, SE=1, AT=1, OM=1]");
		myHCountry.put("1552","[AT=4, ES=1]");
		myHCountry.put("1553","[AT=2]");
		myHCountry.put("1554","[AT=6, DE=1]");
		myHCountry.put("1555","[PL=4]");
		myHCountry.put("1556","[AT=1, PL=1]");
		myHCountry.put("1557","[PL=20, SA=3, PE=1]");
		myHCountry.put("1558","[PL=17, DE=1, SE=1, AT=1]");
		myHCountry.put("1559","[PL=5, FR=2, MQ=1, US=1]");
		myHCountry.put("1560","[BD=4, SE=1, FR=1, PL=1]");
		myHCountry.put("1561","[PL=2, CO=1, RU=1, SE=1]");
		myHCountry.put("1562","[LV=2, FI=1, BE=1, PL=1]");
		myHCountry.put("1563","[PE=1, NL=1, PL=1]");
		myHCountry.put("1564","[PL=3, RU=2, KG=1, ES=1, US=1, BG=1]");
		myHCountry.put("1565","[AT=3, CO=1, SE=1, PL=1]");
		myHCountry.put("1566","[IT=1, CU=1, ES=1, PL=1]");
		myHCountry.put("1567","[BR=5, FR=1, AT=1]");
		myHCountry.put("1568","[IT=1, RU=1, US=1]");
		myHCountry.put("1569","[US=2]");
		myHCountry.put("1570","[TR=1, ID=1, MY=1, US=1]");
		myHCountry.put("1571","[US=2, NL=1]");
		myHCountry.put("1572","[US=4]");
		myHCountry.put("1573","[US=2, TN=1, VN=1]");
		myHCountry.put("1574","[US=5, SA=1, MX=1, FR=1]");
		myHCountry.put("1575","[US=1]");
		myHCountry.put("1576","[US=3]");
		myHCountry.put("1577","[ES=2, DE=1, AR=1, FR=1]");
		myHCountry.put("1578","[FR=2, CN=1, IT=1, AL=1, DE=1, AT=1]");
		myHCountry.put("1579","[DE=1, PL=1]");
		myHCountry.put("1580","[NG=4, SA=3, MA=2, BR=2, IT=1, DE=1, ES=1, RU=1, ZA=1, GE=1, US=1, FR=1]");
		myHCountry.put("1581","[US=2, DE=1]");
		myHCountry.put("1582","[DE=1, GP=1, BR=1, RU=1]");
		myHCountry.put("1583","[BR=2, SA=1, CN=1, IR=1, DE=1, US=1, FR=1]");
		myHCountry.put("1584","[DE=1, US=1]");
		myHCountry.put("1585","[IT=8, DE=1, US=1]");
		myHCountry.put("1586","[DE=9, CZ=2, CO=1, US=1, AT=1]");
		myHCountry.put("1587","[DE=1, AT=1]");
		myHCountry.put("1588","[DE=2, BG=2]");
		myHCountry.put("1589","[DE=1, NL=1]");
		myHCountry.put("1590","[IN=4, OM=3, SA=2, DE=1, PK=1, US=1]");
		myHCountry.put("1591","[DE=1, IN=1]");
		myHCountry.put("1592","[NG=4, DE=1, FR=1]");
		myHCountry.put("1593","[SA=1, DE=1, PL=1]");
		myHCountry.put("1594","[PL=2]");
		myHCountry.put("1595","[DE=3, DZ=1]");
		myHCountry.put("1596","[DE=2]");
		myHCountry.put("1597","[RU=5, US=2, TZ=1, ID=1, MZ=1, ZA=1, LV=1]");
		myHCountry.put("1598","[DE=3]");
		myHCountry.put("1599","[US=3, DE=1]");
		myHCountry.put("1600","[DE=3]");
		myHCountry.put("1601","[ES=53, DE=4, ZA=1]");
		myHCountry.put("1602","[DE=2]");
		myHCountry.put("1603","[ML=8, DE=2, NL=1]");
		myHCountry.put("1604","[DE=1, AT=1]");
		myHCountry.put("1605","[DE=3]");
		myHCountry.put("1606","[SA=4, IN=3, AU=1, US=1]");
		myHCountry.put("1607","[ZM=3, ZA=2, ZW=1, SE=1, US=1]");
		myHCountry.put("1608","[US=1]");
		myHCountry.put("1609","[US=2, VN=1, FR=1]");
		myHCountry.put("1610","[US=4, PY=1]");
		myHCountry.put("1611","[US=5, GP=1, SE=1, FR=1]");
		myHCountry.put("1612","[US=2, BD=1]");
		myHCountry.put("1613","[IT=3, US=1, AT=1]");
		myHCountry.put("1614","[US=2]");
		myHCountry.put("1615","[US=9]");
		myHCountry.put("1616","[IN=3, NL=1, US=1]");
		myHCountry.put("1617","[PE=1, ES=1, US=1]");
		myHCountry.put("1618","[VN=1, US=1]");
		myHCountry.put("1619","[US=2]");
		myHCountry.put("1620","[ES=7, BE=1, BR=1, US=1]");
		myHCountry.put("1621","[SE=3, ES=1, US=1]");
		myHCountry.put("1622","[TR=1, US=1, AT=1]");
		myHCountry.put("1623","[CN=1, US=1]");
		myHCountry.put("1624","[US=2, ES=1, ET=1]");
		myHCountry.put("1625","[US=2, MX=1]");
		myHCountry.put("1626","[US=3, SA=2, SE=2, MA=1, CN=1, IT=1, GB=1, MX=1]");
		myHCountry.put("1627","[ZA=2, US=1]");
		myHCountry.put("1628","[US=6, OM=2]");
		myHCountry.put("1629","[US=3, IT=1, MX=1, FR=1, MU=1]");
		myHCountry.put("1630","[CO=3, IQ=1, NL=1, US=1]");
		myHCountry.put("1631","[US=2]");
		myHCountry.put("1632","[US=1, AT=1]");
		myHCountry.put("1633","[US=2]");
		myHCountry.put("1634","[SA=3, CN=2, US=2, TW=1, ID=1, ES=1, IN=1, RU=1, ZA=1]");
		myHCountry.put("1635","[DE=2]");
		myHCountry.put("1636","[DE=2, UA=1]");
		myHCountry.put("1637","[DE=1, NL=1]");
		myHCountry.put("1638","[CN=2, IQ=1]");
		myHCountry.put("1639","[ZA=1, FR=1, CI=1]");
		myHCountry.put("1640","[FR=2, NL=1, PL=1]");
		myHCountry.put("1641","[IT=1, PK=1, FR=1]");
		myHCountry.put("1642","[FR=2]");
		myHCountry.put("1643","[US=4, IT=1, FR=1]");
		myHCountry.put("1644","[BR=2, CZ=2, MQ=1, US=1, FR=1]");
		myHCountry.put("1645","[TR=1, AL=1, GP=1, FR=1]");
		myHCountry.put("1646","[FR=4]");
		myHCountry.put("1647","[CN=1, DE=1, FR=1, AT=1]");
		myHCountry.put("1648","[KM=2, NL=1]");
		myHCountry.put("1649","[NL=1, MZ=1]");
		myHCountry.put("1650","[MZ=2]");
		myHCountry.put("1651","[RU=10, TR=1, GM=1, ID=1, IN=1]");
		myHCountry.put("1652","[SA=2, ID=1, VN=1, ES=1, US=1]");
		myHCountry.put("1653","[ID=2]");
		myHCountry.put("1654","[ID=2]");
		myHCountry.put("1655","[ID=14, SA=11, MA=4, IT=2, ZA=2, LK=1, IQ=1, BR=1, IN=1]");
		myHCountry.put("1656","[ID=2, EG=1, ET=1, MG=1]");
		myHCountry.put("1657","[SA=1, ID=1, US=1, FR=1, PL=1]");
		myHCountry.put("1658","[ID=1, AT=1]");
		myHCountry.put("1659","[BR=4, CO=1]");
		myHCountry.put("1660","[BR=2, GF=1, AR=1]");
		myHCountry.put("1661","[BR=3, VE=2, MM=1]");
		myHCountry.put("1662","[BR=2]");
		myHCountry.put("1663","[BR=2]");
		myHCountry.put("1664","[BR=3]");
		myHCountry.put("1665","[BR=2]");
		myHCountry.put("1666","[BR=3]");
		myHCountry.put("1667","[BR=4, AR=4, MX=1]");
		myHCountry.put("1668","[BR=6]");
		myHCountry.put("1669","[BR=4]");
		myHCountry.put("1670","[AR=13, BR=9, ES=1, MX=1]");
		myHCountry.put("1671","[BR=1, FR=1]");
		myHCountry.put("1672","[MG=2]");
		myHCountry.put("1673","[MG=39, ZA=2, NL=1]");
		myHCountry.put("1674","[CN=3, US=2, RE=1, MG=1]");
		myHCountry.put("1675","[TZ=7, MG=7, ET=2, MZ=1, FR=1]");
		myHCountry.put("1676","[MG=2]");
		myHCountry.put("1677","[MG=3]");
		myHCountry.put("1678","[MG=2]");
		myHCountry.put("1679","[MG=5, IQ=2, AL=1, DE=1, MX=1]");
		myHCountry.put("1680","[GB=1, MG=1]");
		myHCountry.put("1681","[AL=1, PE=1, BE=1, BR=1, AT=1]");
		myHCountry.put("1682","[ES=8, NL=2, BR=1]");
		myHCountry.put("1683","[BR=1, RU=1, US=1]");
		myHCountry.put("1684","[FR=2, BR=1, NG=1]");
		myHCountry.put("1685","[ES=3, BR=1, ZA=1]");
		myHCountry.put("1686","[BR=2, AT=1]");
		myHCountry.put("1687","[BR=1, US=1]");
		myHCountry.put("1688","[IT=5, ET=5, CN=3, CZ=2, BR=1, US=1]");
		myHCountry.put("1689","[MQ=1, AL=1, BR=1, ET=1]");
		myHCountry.put("1690","[CN=1, EG=1, BR=1, US=1]");
		myHCountry.put("1691","[VE=14, PT=2, ZA=1, MX=1]");
		myHCountry.put("1692","[VE=4, ES=2]");
		myHCountry.put("1693","[VE=2, FR=1]");
		myHCountry.put("1694","[US=2, BR=1, ES=1, VE=1]");
		myHCountry.put("1695","[BR=1, VE=1]");
		myHCountry.put("1696","[VE=5]");
		myHCountry.put("1697","[FR=1, VE=1]");
		myHCountry.put("1698","[VE=3]");
		myHCountry.put("1699","[VE=2, CO=1, BR=1]");
		myHCountry.put("1700","[VE=6]");
		myHCountry.put("1701","[VE=3]");
		myHCountry.put("1702","[VE=3]");
		myHCountry.put("1703","[VE=2]");
		myHCountry.put("1704","[VE=2]");
		myHCountry.put("1705","[VE=3]");
		myHCountry.put("1706","[IT=2, CZ=2, TR=1, BR=1, ES=1, US=1, VE=1]");
		myHCountry.put("1707","[VE=2]");
		myHCountry.put("1708","[LV=1, VE=1]");
		myHCountry.put("1709","[US=1, OM=1, VE=1]");
		myHCountry.put("1710","[BR=2, TT=1, LK=1, NL=1, US=1, VE=1]");
		myHCountry.put("1711","[VE=2, CO=1, ES=1, MX=1]");
		myHCountry.put("1712","[US=1, VE=1]");
		myHCountry.put("1713","[VE=3]");
		myHCountry.put("1714","[US=2, VE=2]");
		myHCountry.put("1715","[VE=2, IT=1]");
		myHCountry.put("1716","[VE=2]");
		myHCountry.put("1717","[AU=1, VE=1]");
		myHCountry.put("1718","[VE=3, BR=1]");
		myHCountry.put("1719","[VE=2]");
		myHCountry.put("1720","[DK=2]");
		myHCountry.put("1721","[DK=3]");
		myHCountry.put("1722","[GB=1, US=1]");
		myHCountry.put("1723","[GB=2]");
		myHCountry.put("1724","[GB=2]");
		myHCountry.put("1725","[MY=2]");
		myHCountry.put("1726","[DK=1, US=1]");
		myHCountry.put("1727","[DK=1, NL=1, SE=1, US=1]");
		myHCountry.put("1728","[DK=2]");
		myHCountry.put("1729","[ET=18, DK=1]");
		myHCountry.put("1730","[DK=2, ES=2, IT=1, DE=1]");
		myHCountry.put("1731","[DK=2]");
		myHCountry.put("1732","[DK=4]");
		myHCountry.put("1733","[DK=1, NL=1]");
		myHCountry.put("1734","[DK=2, NP=1]");
		myHCountry.put("1735","[IT=2, MA=1, ZA=1]");
		myHCountry.put("1736","[BG=4, IT=1, US=1, FR=1]");
		myHCountry.put("1737","[ZA=15, IT=12, US=1]");
		myHCountry.put("1738","[IT=9]");
		myHCountry.put("1739","[IT=3, GF=1]");
		myHCountry.put("1740","[IT=2, US=1]");
		myHCountry.put("1741","[US=2, IT=1]");
		myHCountry.put("1742","[IT=1, FR=1]");
		myHCountry.put("1743","[BE=3, IT=2, NL=1, FR=1]");
		myHCountry.put("1744","[IT=2]");
		myHCountry.put("1745","[CZ=3, IT=1, TZ=1, US=1]");
		myHCountry.put("1746","[PL=3, IT=1]");
		myHCountry.put("1747","[DE=1, US=1]");
		myHCountry.put("1748","[DE=5, CN=1]");
		myHCountry.put("1749","[DE=2]");
		myHCountry.put("1750","[MZ=4, PT=2, IT=1, BR=1, ZA=1]");
		myHCountry.put("1751","[PT=2, AU=1, MX=1]");
		myHCountry.put("1752","[PT=2, US=1]");
		myHCountry.put("1753","[ES=5, PE=2, FR=2, PT=1]");
		myHCountry.put("1754","[PT=5, MZ=2]");
		myHCountry.put("1755","[US=3, BR=2, PT=2, MZ=1]");
		myHCountry.put("1756","[US=1, PT=1]");
		myHCountry.put("1757","[PT=2, BE=1]");
		myHCountry.put("1758","[BR=8, PT=1]");
		myHCountry.put("1759","[PT=3]");
		myHCountry.put("1760","[DE=1, PT=1]");
		myHCountry.put("1761","[BE=1, BR=1, GF=1]");
		myHCountry.put("1762","[TN=2, MQ=1, GF=1]");
		myHCountry.put("1763","[ES=1, GF=1, US=1]");
		myHCountry.put("1764","[KE=2, OM=1]");
		myHCountry.put("1765","[US=2, UG=1, KE=1, BE=1, ZM=1]");
		myHCountry.put("1766","[KE=1, US=1]");
		myHCountry.put("1767","[KE=1, IT=1, RU=1]");
		myHCountry.put("1768","[BR=2, CU=1, MY=1]");
		myHCountry.put("1769","[CU=1, VE=1]");
		myHCountry.put("1770","[FR=2]");
		myHCountry.put("1771","[HU=3]");
		myHCountry.put("1772","[HU=1, GR=1, FR=1]");
		myHCountry.put("1773","[HU=10]");
		myHCountry.put("1774","[BE=2]");
		myHCountry.put("1775","[BE=1, GB=1]");
		myHCountry.put("1776","[BE=3, FR=2]");
		myHCountry.put("1777","[BE=2]");
		myHCountry.put("1778","[MA=1, KE=1, TZ=1, BE=1]");
		myHCountry.put("1780","[BE=7]");
		myHCountry.put("1781","[US=3, BE=1]");
		myHCountry.put("1782","[BE=1, GB=1, GF=1]");
		myHCountry.put("1783","[NG=3, BE=1, NL=1, ZA=1]");
		myHCountry.put("1784","[DK=1, BE=1]");
		myHCountry.put("1785","[BE=1]");
		myHCountry.put("1787","[SD=4, SA=1, ET=1]");
		myHCountry.put("1789","[OM=5, GB=4, IN=4, US=3, FR=3, PK=2, SA=1, IT=1, IQ=1, BE=1, BD=1]");
		myHCountry.put("1790","[EG=1, BE=1, AT=1]");
		myHCountry.put("1792","[BE=3]");
		myHCountry.put("1793","[BE=1, AT=1]");
		myHCountry.put("1795","[AT=1]");
		myHCountry.put("1796","[BE=3, ES=1]");
		myHCountry.put("1797","[TW=1, IT=1, BE=1, FR=1]");
		myHCountry.put("1798","[BE=2, FR=1]");
		myHCountry.put("1799","[HU=1, BE=1, SE=1, AT=1]");
		myHCountry.put("1800","[BR=7, CN=2, LV=2, TZ=1, AL=1, MW=1, US=1, BG=1]");
		myHCountry.put("1801","[US=1, FR=1]");
		myHCountry.put("1802","[FR=2, GP=1]");
		myHCountry.put("1803","[PY=2, GM=2, FR=2, MA=1, CO=1, SN=1, ZA=1, AR=1]");
		myHCountry.put("1804","[CO=2, US=2, IT=1, BR=1, AT=1]");
		myHCountry.put("1805","[AT=2, BR=1]");
		myHCountry.put("1806","[AT=2]");
		myHCountry.put("1807","[US=3, SE=2, AT=2, ES=1, GF=1]");
		myHCountry.put("1808","[BE=1, AT=1]");
		myHCountry.put("1809","[AT=2]");
		myHCountry.put("1810","[AT=4]");
		myHCountry.put("1811","[AT=2]");
		myHCountry.put("1812","[AT=2, BR=1, VE=1]");
		myHCountry.put("1813","[FR=1, AT=1]");
		myHCountry.put("1814","[AT=2]");
		myHCountry.put("1815","[BR=5, CO=2, IT=1, GF=1]");
		myHCountry.put("1816","[BR=2, MX=1]");
		myHCountry.put("1817","[BR=2]");
		myHCountry.put("1818","[BR=1, MX=1]");
		myHCountry.put("1819","[MX=4]");
		myHCountry.put("1820","[MX=3, BR=1, ES=1]");
		myHCountry.put("1821","[EG=2, NO=1, BR=1, ES=1, AR=1, PL=1]");
		myHCountry.put("1822","[AR=8, US=7]");
		myHCountry.put("1823","[TT=2, DE=1, AR=1]");
		myHCountry.put("1824","[BR=2, AR=2, NL=1, ZA=1]");
		myHCountry.put("1825","[AR=6]");
		myHCountry.put("1826","[GB=1, AR=1]");
		myHCountry.put("1827","[PY=3, AR=3]");
		myHCountry.put("1828","[NZ=2, AR=1]");
		myHCountry.put("1829","[AR=1, FR=1]");
		myHCountry.put("1830","[IT=1, CU=1, ES=1, AR=1]");
		myHCountry.put("1831","[AR=2]");
		myHCountry.put("1832","[AR=5, PY=3, MA=2, NZ=2, US=2, FR=2, CO=1, GP=1, RU=1, ZA=1]");
		myHCountry.put("1833","[AR=2]");
		myHCountry.put("1834","[BG=3, TR=2, BE=1, AR=1, US=1]");
		myHCountry.put("1835","[AR=4]");
		myHCountry.put("1836","[AR=2, FR=1]");
		myHCountry.put("1837","[AR=2, BR=1]");
		myHCountry.put("1838","[ES=4, AR=2, BR=1]");
		myHCountry.put("1839","[AR=2]");
		myHCountry.put("1840","[AR=2]");
		myHCountry.put("1841","[ZA=2, AR=1]");
		myHCountry.put("1842","[IT=1, BR=1, AR=1]");
		myHCountry.put("1843","[AR=2]");
		myHCountry.put("1844","[AR=3, MA=2, ZA=2, FR=1, JP=1]");
		myHCountry.put("1845","[BR=5, AR=3]");
		myHCountry.put("1846","[AR=2, CL=1]");
		myHCountry.put("1847","[ES=18, AR=2]");
		myHCountry.put("1848","[BR=3, AR=2, ES=1, MX=1]");
		myHCountry.put("1849","[AR=5, AU=2, ES=2, BR=1, NZ=1]");
		myHCountry.put("1850","[AR=5]");
		myHCountry.put("1851","[GB=1, AR=1]");
		myHCountry.put("1852","[BR=3, AR=2]");
		myHCountry.put("1853","[BR=5, AR=2]");
		myHCountry.put("1854","[FR=2, AR=1]");
		myHCountry.put("1855","[BR=9, AR=3, AU=1]");
		myHCountry.put("1856","[AR=7, AU=1]");
		myHCountry.put("1857","[AR=4, ES=1]");
		myHCountry.put("1858","[AR=4]");
		myHCountry.put("1859","[AR=2, BR=1]");
		myHCountry.put("1860","[IE=3, GB=2]");
		myHCountry.put("1861","[IE=5, GB=1]");
		myHCountry.put("1862","[IE=4, GB=1]");
		myHCountry.put("1863","[CL=4, IE=4, MX=1]");
		myHCountry.put("1864","[TZ=3, US=2, ZA=1]");
		myHCountry.put("1865","[TZ=3, US=3]");
		myHCountry.put("1866","[US=2, BD=1, NL=1]");
		myHCountry.put("1867","[NL=1, US=1]");
		myHCountry.put("1868","[TW=1, PH=1, US=1]");
		myHCountry.put("1869","[US=3, NO=1, NG=1]");
		myHCountry.put("1870","[SE=1, US=1]");
		myHCountry.put("1871","[US=7]");
		myHCountry.put("1872","[TR=2, ZW=1, BR=1, US=1]");
		myHCountry.put("1873","[ZA=10, TZ=6, US=3, MZ=2, ZW=1]");
		myHCountry.put("1874","[ZA=2, US=1]");
		myHCountry.put("1875","[DE=1, US=1]");
		myHCountry.put("1876","[US=4]");
		myHCountry.put("1877","[SA=5, IN=3, US=2, BR=1, ET=1, FR=1]");
		myHCountry.put("1878","[US=3]");
		myHCountry.put("1879","[IN=2]");
		myHCountry.put("1880","[US=6]");
		myHCountry.put("1881","[FR=1]");
		myHCountry.put("1882","[US=2]");
		myHCountry.put("1883","[SA=2, US=2]");
		myHCountry.put("1884","[AT=1]");
		myHCountry.put("1885","[SA=5]");
		myHCountry.put("1886","[OM=4, SA=3, SG=1, PH=1]");
		myHCountry.put("1887","[SA=16, IN=1, OM=1]");
		myHCountry.put("1888","[BR=5, VE=1]");
		myHCountry.put("1889","[BR=4]");
		myHCountry.put("1890","[BR=4, CN=3]");
		myHCountry.put("1891","[BR=6]");
		myHCountry.put("1892","[BR=7, PY=1]");
		myHCountry.put("1893","[BR=2]");
		myHCountry.put("1894","[BR=2]");
		myHCountry.put("1895","[BR=12, US=1]");
		myHCountry.put("1896","[BD=1]");
		myHCountry.put("1897","[BD=10]");
		myHCountry.put("1898","[BD=24]");
		myHCountry.put("1899","[VN=2]");
		myHCountry.put("1900","[VN=2]");
		myHCountry.put("1901","[AU=1, VN=1]");
		myHCountry.put("1902","[VN=4]");
		myHCountry.put("1903","[VN=2]");
		myHCountry.put("1904","[IT=2, VN=1, BR=1, AT=1]");
		myHCountry.put("1905","[BR=17, PE=3, IT=1, AR=1]");
		myHCountry.put("1906","[BR=8]");
		myHCountry.put("1907","[BR=4, NG=1]");
		myHCountry.put("1908","[BR=2, US=2, TR=1, IN=1]");
		myHCountry.put("1909","[BR=4, IT=1, VE=1]");
		myHCountry.put("1910","[BR=4]");
		myHCountry.put("1911","[BR=4]");
		myHCountry.put("1912","[MQ=1, AR=1]");
		myHCountry.put("1913","[IQ=1, GP=1, MZ=1, FR=1]");
		myHCountry.put("1914","[BR=3, IT=1, VE=1]");
		myHCountry.put("1915","[MQ=1, IT=1, SE=1, ZA=1, BG=1]");
		myHCountry.put("1916","[TR=4, IT=1, SN=1, MX=1, FR=1]");
		myHCountry.put("1917","[IT=2]");
		myHCountry.put("1918","[IT=3]");
		myHCountry.put("1919","[IT=3]");
		myHCountry.put("1920","[IT=15]");
		myHCountry.put("1921","[IT=2]");
		myHCountry.put("1922","[IT=1, US=1]");
		myHCountry.put("1923","[IT=1, US=1]");
		myHCountry.put("1924","[IT=2, EG=1]");
		myHCountry.put("1925","[IT=2]");
		myHCountry.put("1926","[IT=2, SA=1, TW=1, IN=1, AR=1, FR=1]");
		myHCountry.put("1927","[IT=1, AT=1]");
		myHCountry.put("1928","[IT=2, PL=1]");
		myHCountry.put("1929","[IT=1, US=1, SD=1]");
		myHCountry.put("1930","[IT=2]");
		myHCountry.put("1931","[IT=2]");
		myHCountry.put("1932","[ES=4, IT=2, FR=1]");
		myHCountry.put("1933","[IT=1, BR=1, US=1, FR=1]");
		myHCountry.put("1934","[IT=1, NL=1]");
		myHCountry.put("1935","[GF=1, PT=1]");
		myHCountry.put("1936","[SE=2, TR=1, BE=1]");
		myHCountry.put("1937","[TR=2]");
		myHCountry.put("1938","[TR=2, NL=1, US=1]");
		myHCountry.put("1939","[US=2]");
		myHCountry.put("1940","[IN=5, US=1]");
		myHCountry.put("1941","[IN=4, US=2]");
		myHCountry.put("1942","[IN=5, GR=2, SA=1, DE=1, US=1]");
		myHCountry.put("1943","[IN=1, US=1]");
		myHCountry.put("1944","[IN=4]");
		myHCountry.put("1945","[IN=3]");
		myHCountry.put("1946","[IN=2]");
		myHCountry.put("1947","[IN=3, BD=1]");
		myHCountry.put("1948","[IN=7, PK=3, US=2, SA=1]");
		myHCountry.put("1949","[IN=7, OM=4, SA=1, MM=1, US=1, SD=1]");
		myHCountry.put("1950","[IN=4, SA=1]");
		myHCountry.put("1951","[IN=5, SE=2, PK=1, FR=1]");
		myHCountry.put("1952","[SA=6, LK=4, AL=4, IN=3, US=2, MA=1, TR=1, IT=1, ET=1, MX=1]");
		myHCountry.put("1953","[GB=1, US=1]");
		myHCountry.put("1954","[SN=1, GB=1]");
		myHCountry.put("1955","[GB=1, FR=1]");
		myHCountry.put("1956","[IN=3, SA=2, SG=2, OM=1]");
		myHCountry.put("1957","[LK=9, IN=2, US=2, SA=1]");
		myHCountry.put("1958","[IN=2]");
		myHCountry.put("1959","[IN=2, SA=1]");
		myHCountry.put("1960","[ES=2, TN=1]");
		myHCountry.put("1961","[IN=2, MM=1]");
		myHCountry.put("1962","[IN=3, SA=1, US=1]");
		myHCountry.put("1963","[IN=2, FR=1]");
		myHCountry.put("1964","[IN=2, OM=1]");
		myHCountry.put("1965","[IN=2, NL=1]");
		myHCountry.put("1966","[IN=2, FR=1]");
		myHCountry.put("1967","[SA=3, IN=1, FR=1]");
		myHCountry.put("1968","[IN=3, BE=1]");
		myHCountry.put("1969","[IN=3]");
		myHCountry.put("1970","[IN=6, US=3, TN=1, DE=1, VC=1, OM=1]");
		myHCountry.put("1971","[IN=2]");
		myHCountry.put("1972","[IN=1, AT=1]");
		myHCountry.put("1973","[IN=2]");
		myHCountry.put("1974","[IN=2]");
		myHCountry.put("1975","[IN=2, IR=1]");
		myHCountry.put("1976","[IN=1, SD=1]");
		myHCountry.put("1977","[IN=2]");
		myHCountry.put("1978","[NL=1, IN=1]");
		myHCountry.put("1979","[AU=1, IN=1, MX=1]");
		myHCountry.put("1980","[PK=3, IR=2, IN=1, OM=1]");
		myHCountry.put("1981","[SA=4, SD=3, FR=2, IN=1]");
		myHCountry.put("1982","[IN=2, MY=1]");
		myHCountry.put("1983","[BR=4, IT=1, IN=1]");
		myHCountry.put("1984","[IT=2]");
		myHCountry.put("1985","[IT=2]");
		myHCountry.put("1986","[IT=2]");
		myHCountry.put("1987","[IT=1, ID=1]");
		myHCountry.put("1988","[IT=1, AU=1]");
		myHCountry.put("1989","[IT=1, BR=1]");
		myHCountry.put("1990","[IT=3, SE=1]");
		myHCountry.put("1991","[IT=2]");
		myHCountry.put("1992","[IT=2, MY=1]");
		myHCountry.put("1993","[IT=2]");
		myHCountry.put("1994","[IT=2]");
		myHCountry.put("1995","[IT=1, AT=1]");
		myHCountry.put("1996","[IT=3]");
		myHCountry.put("1997","[IT=5]");
		myHCountry.put("1998","[IT=1, AT=1]");
		myHCountry.put("1999","[IT=2, GF=2, MA=1]");
		myHCountry.put("2000","[IT=2]");
		myHCountry.put("2001","[IT=2]");
		myHCountry.put("2002","[IT=2]");
		myHCountry.put("2003","[GP=1, US=1]");
		myHCountry.put("2004","[US=2]");
		myHCountry.put("2005","[US=2]");
		myHCountry.put("2006","[US=2]");
		myHCountry.put("2007","[DO=2, US=2, FR=1]");
		myHCountry.put("2008","[GR=2]");
		myHCountry.put("2009","[SG=1, US=1]");
		myHCountry.put("2010","[IN=2, SG=1]");
		myHCountry.put("2011","[SG=2, SA=1]");
		myHCountry.put("2012","[GR=2]");
		myHCountry.put("2013","[US=2, ZA=1]");
		myHCountry.put("2014","[ZA=3, MX=1]");
		myHCountry.put("2015","[ES=2, ZA=2]");
		myHCountry.put("2016","[JM=13, ZA=12, US=4]");
		myHCountry.put("2017","[ZA=9, ZM=3, BE=2, TZ=1]");
		myHCountry.put("2018","[ZA=6, US=1]");
		myHCountry.put("2019","[ZA=9, US=1]");
		myHCountry.put("2020","[ZA=8, GB=1]");
		myHCountry.put("2021","[ZA=2, RU=1, US=1]");
		myHCountry.put("2022","[ZA=3, US=1]");
		myHCountry.put("2023","[IT=10, BE=1]");
		myHCountry.put("2024","[IT=2]");
		myHCountry.put("2025","[BR=3, RU=2, JP=2, MA=1, IT=1, CO=1, US=1]");
		myHCountry.put("2026","[TR=5, IT=1, US=1]");
		myHCountry.put("2027","[IT=1, NL=1]");
		myHCountry.put("2028","[US=2, IT=1, PE=1, SE=1, BG=1]");
		myHCountry.put("2029","[IT=2]");
		myHCountry.put("2030","[GW=1, IT=1]");
		myHCountry.put("2031","[FI=1, IT=1]");
		myHCountry.put("2032","[TR=2, DK=1, IT=1, IQ=1]");
		myHCountry.put("2033","[IT=2]");
		myHCountry.put("2034","[IT=1, US=1]");
		myHCountry.put("2035","[SA=2]");
		myHCountry.put("2036","[SA=1, VE=1]");
		myHCountry.put("2037","[TN=3, TR=1, FR=1, AT=1]");
		myHCountry.put("2038","[US=4]");
		myHCountry.put("2039","[US=3, VE=1]");
		myHCountry.put("2040","[US=13]");
		myHCountry.put("2041","[JP=7, MA=2, RU=2, ZA=2, BG=2, AL=1, GR=1, PL=1]");
		myHCountry.put("2042","[BG=2]");
		myHCountry.put("2043","[BG=2]");
		myHCountry.put("2044","[ES=1, RU=1]");
		myHCountry.put("2045","[ES=9, FR=5, BE=1]");
		myHCountry.put("2046","[CZ=3, ES=2]");
		myHCountry.put("2047","[ES=2, FR=2, MA=1, GM=1, ZA=1]");
		myHCountry.put("2048","[US=5, DO=2, ES=1]");
		myHCountry.put("2049","[US=1, BG=1]");
		myHCountry.put("2050","[BG=3]");
		myHCountry.put("2051","[BG=3]");
		myHCountry.put("2052","[BG=2, FR=2]");
		myHCountry.put("2053","[GP=1, BR=1, ZA=1]");
		myHCountry.put("2054","[HT=1, PE=1, GP=1, BR=1, US=1]");
		myHCountry.put("2055","[CZ=3, IT=2, US=1]");
		myHCountry.put("2056","[US=2, GN=1, FR=1]");
		myHCountry.put("2057","[RU=1, AR=1]");
		myHCountry.put("2058","[US=1, NG=1]");
		myHCountry.put("2059","[NZ=2]");
		myHCountry.put("2060","[NZ=2, IN=1]");
		myHCountry.put("2061","[NZ=2]");
		myHCountry.put("2062","[NZ=2]");
		myHCountry.put("2063","[SL=1, US=1]");
		myHCountry.put("2064","[SL=1, US=1]");
		myHCountry.put("2065","[TR=1, SE=1]");
		myHCountry.put("2066","[TR=1, CO=1, AU=1, FR=1]");
		myHCountry.put("2067","[TR=11, FR=1]");
		myHCountry.put("2068","[TR=4]");
		myHCountry.put("2069","[TR=2, FR=1]");
		myHCountry.put("2070","[BR=5, ES=3, PE=2, HT=1, TR=1, AL=1]");
		myHCountry.put("2071","[TR=2]");
		myHCountry.put("2072","[TR=2]");
		myHCountry.put("2073","[US=4]");
		myHCountry.put("2074","[US=2]");
		myHCountry.put("2075","[TR=3, RU=2, US=1]");
		myHCountry.put("2076","[US=2]");
		myHCountry.put("2077","[TR=6, US=1]");
		myHCountry.put("2078","[GR=2]");
		myHCountry.put("2079","[PL=2]");
		myHCountry.put("2080","[PL=2]");
		myHCountry.put("2081","[TW=1, IN=1]");
		myHCountry.put("2082","[TW=1, RU=1]");
		myHCountry.put("2083","[US=2, TW=1]");
		myHCountry.put("2084","[TW=1, MY=1]");
		myHCountry.put("2085","[TW=1, SG=1]");
		myHCountry.put("2086","[TW=1, TH=1]");
		myHCountry.put("2087","[TW=1, US=1]");
		myHCountry.put("2088","[US=2, FR=2, NG=1]");
		myHCountry.put("2089","[TW=16, FR=3]");
		myHCountry.put("2090","[SA=2, TW=2]");
		myHCountry.put("2091","[TW=3, SE=1]");
		myHCountry.put("2092","[CN=4, TW=2]");
		myHCountry.put("2093","[TW=6]");
		myHCountry.put("2094","[TW=2, ID=1]");
		myHCountry.put("2095","[TW=2]");
		myHCountry.put("2096","[TW=2, SA=1]");
		myHCountry.put("2097","[TW=3, US=1]");
		myHCountry.put("2098","[TW=2]");
		myHCountry.put("2099","[TW=2, MY=1, US=1]");
		myHCountry.put("2100","[US=6]");
		myHCountry.put("2101","[TW=2, US=2, CN=1, JP=1]");
		myHCountry.put("2102","[US=2]");
		myHCountry.put("2103","[US=11]");
		myHCountry.put("2104","[TN=1, BB=1, US=1]");
		myHCountry.put("2105","[US=2]");
		myHCountry.put("2106","[US=4]");
		myHCountry.put("2107","[US=2, FR=1]");
		myHCountry.put("2108","[US=2, ZA=1]");
		myHCountry.put("2109","[US=2]");
		myHCountry.put("2110","[BR=2, US=2]");
		myHCountry.put("2111","[US=4]");
		myHCountry.put("2112","[DE=1, US=1]");
		myHCountry.put("2113","[US=2, IT=1, ZA=1, OM=1]");
		myHCountry.put("2114","[TR=2, CN=1, CO=1, ES=1, MX=1, US=1]");
		myHCountry.put("2115","[TR=2]");
		myHCountry.put("2116","[TR=2, AR=1]");
		myHCountry.put("2117","[ZA=5]");
		myHCountry.put("2118","[US=3]");
		myHCountry.put("2119","[US=2]");
		myHCountry.put("2120","[TR=2]");
		myHCountry.put("2121","[JP=3]");
		myHCountry.put("2122","[JP=2]");
		myHCountry.put("2123","[JP=2]");
		myHCountry.put("2124","[JP=15]");
		myHCountry.put("2125","[TR=5, NL=1]");
		myHCountry.put("2126","[US=2]");
		myHCountry.put("2127","[TR=2]");
		myHCountry.put("2128","[TR=1, GF=1]");
		myHCountry.put("2129","[BR=3, AR=1]");
		myHCountry.put("2130","[BR=10]");
		myHCountry.put("2131","[IE=1, US=1]");
		myHCountry.put("2132","[US=2]");
		myHCountry.put("2133","[BR=3]");
		myHCountry.put("2134","[US=3]");
		myHCountry.put("2135","[TR=1, ID=1]");
		myHCountry.put("2136","[TR=2]");
		myHCountry.put("2137","[US=2]");
		myHCountry.put("2138","[US=2]");
		myHCountry.put("2139","[TR=2, BG=2]");
		myHCountry.put("2140","[TR=3, ES=2, CN=1, GR=1]");
		myHCountry.put("2141","[ES=7, BR=5, FR=1]");
		myHCountry.put("2142","[TR=2, BR=1]");
		myHCountry.put("2143","[TR=1, IN=1]");
		myHCountry.put("2144","[TR=3]");
		myHCountry.put("2145","[IN=3, PK=2, BD=2, NP=1, YE=1, IQ=1]");
		myHCountry.put("2146","[BD=2, MM=1]");
		myHCountry.put("2147","[US=3]");
		myHCountry.put("2148","[IN=2, NP=1, IT=1, BD=1, PT=1]");
		myHCountry.put("2149","[BD=2]");
		myHCountry.put("2150","[US=4]");
		myHCountry.put("2151","[BD=2, HN=2, BR=1, RU=1]");
		myHCountry.put("2152","[OM=4, NP=1, IR=1, BD=1, JM=1]");
		myHCountry.put("2153","[EC=10, IE=1, GB=1, AR=1]");
		myHCountry.put("2154","[US=4, TR=1]");
		myHCountry.put("2155","[BD=9]");
		myHCountry.put("2156","[BD=2]");
		myHCountry.put("2157","[US=2]");
		myHCountry.put("2158","[SA=2, BD=2, US=1]");
		myHCountry.put("2159","[BD=2]");
		myHCountry.put("2160","[TR=2]");
		myHCountry.put("2161","[MA=1, TR=1, AL=1]");
		myHCountry.put("2162","[TR=5]");
		myHCountry.put("2163","[TR=2]");
		myHCountry.put("2164","[GM=1, SE=1]");
		myHCountry.put("2165","[BR=2, ID=1, ZA=1]");
		myHCountry.put("2166","[US=3]");
		myHCountry.put("2167","[US=2]");
		myHCountry.put("2168","[BR=2]");
		myHCountry.put("2169","[BR=1, ES=1]");
		myHCountry.put("2170","[US=2]");
		myHCountry.put("2171","[US=2]");
		myHCountry.put("2172","[US=2]");
		myHCountry.put("2173","[ZM=13, BE=8, ZA=3, US=2, NL=1, BR=1]");
		myHCountry.put("2174","[US=2, NG=1]");
		myHCountry.put("2175","[US=2]");
		myHCountry.put("2176","[US=2]");
		myHCountry.put("2177","[ZA=3]");
		myHCountry.put("2178","[US=2]");
		myHCountry.put("2179","[ES=2, US=2, PE=1]");
		myHCountry.put("2180","[US=2, ZA=1]");
		myHCountry.put("2181","[ZA=4]");
		myHCountry.put("2182","[ZA=4]");
		myHCountry.put("2183","[US=2]");
		myHCountry.put("2184","[ZA=2]");
		myHCountry.put("2185","[ZA=2]");
		myHCountry.put("2186","[ZA=3]");
		myHCountry.put("2187","[DE=1, ZA=1]");
		myHCountry.put("2188","[SA=1, IT=1, MX=1, US=1]");
		myHCountry.put("2189","[US=4]");
		myHCountry.put("2190","[US=2, CN=1, KZ=1]");
		myHCountry.put("2191","[ZA=2, FR=1]");
		myHCountry.put("2192","[CU=1, ZA=1]");
		myHCountry.put("2193","[US=2]");
		myHCountry.put("2194","[ZA=3, GB=1]");
		myHCountry.put("2195","[US=2]");
		myHCountry.put("2196","[MZ=5, ZA=4, ZM=2, FI=1, TZ=1, BE=1]");
		myHCountry.put("2197","[US=1, AT=1, OM=1]");
		myHCountry.put("2198","[US=3]");
		myHCountry.put("2199","[JM=3, US=3]");
		myHCountry.put("2200","[US=2]");
		myHCountry.put("2201","[OM=3, GW=2, IT=2, BE=1, GP=1, BR=1, SE=1, MX=1, US=1]");
		myHCountry.put("2202","[US=2]");
		myHCountry.put("2203","[NL=1, US=1, JP=1]");
		myHCountry.put("2204","[ZA=2]");
		myHCountry.put("2205","[IT=1, US=1]");
		myHCountry.put("2206","[US=2]");
		myHCountry.put("2207","[US=2, BD=1]");
		myHCountry.put("2208","[CN=1, TR=1, US=1, CI=1]");
		myHCountry.put("2209","[US=5, CZ=2]");
		myHCountry.put("2210","[CZ=2, US=1]");
		myHCountry.put("2211","[CZ=2, US=1]");
		myHCountry.put("2212","[US=2]");
		myHCountry.put("2213","[US=2, CZ=2]");
		myHCountry.put("2214","[CZ=2, US=1]");
		myHCountry.put("2215","[CZ=2, US=1]");
		myHCountry.put("2216","[CZ=2, US=1]");
		myHCountry.put("2217","[CZ=2, US=1]");
		myHCountry.put("2218","[CZ=2, US=1]");
		myHCountry.put("2219","[CZ=2, US=1]");
		myHCountry.put("2220","[CZ=2, US=1]");
		myHCountry.put("2221","[CZ=2, US=1]");
		myHCountry.put("2222","[ZA=2]");
		myHCountry.put("2223","[ZA=3]");
		myHCountry.put("2224","[ZA=3]");
		myHCountry.put("2225","[ZA=2]");
		myHCountry.put("2226","[ZA=2]");
		myHCountry.put("2227","[US=3]");
		myHCountry.put("2228","[ZA=2]");
		myHCountry.put("2229","[ZA=4, US=1]");
		myHCountry.put("2230","[CZ=3, IQ=2, PE=1]");
		myHCountry.put("2231","[CZ=3]");
		myHCountry.put("2232","[CZ=3]");
		myHCountry.put("2233","[CZ=3]");
		myHCountry.put("2234","[CZ=3, TR=1, FR=1]");
		myHCountry.put("2235","[CZ=2, US=1]");
		myHCountry.put("2236","[US=2]");
		myHCountry.put("2237","[IN=1, US=1]");
		myHCountry.put("2238","[IN=1, US=1]");
		myHCountry.put("2239","[IN=1, US=1]");
		myHCountry.put("2240","[IN=1, US=1]");
		myHCountry.put("2241","[IN=1, US=1]");
		myHCountry.put("2242","[US=2, FR=1]");
		myHCountry.put("2243","[NG=3, US=1]");
		myHCountry.put("2244","[NG=3]");
		myHCountry.put("2245","[FR=1, NG=1]");
		myHCountry.put("2246","[US=2, TR=1, RU=1, ZA=1]");
		myHCountry.put("2247","[RU=1, US=1]");
		myHCountry.put("2248","[US=2]");
		myHCountry.put("2249","[KZ=1, RU=1, US=1]");
		myHCountry.put("2250","[US=2, RU=1]");
		myHCountry.put("2251","[RU=1, US=1]");
		myHCountry.put("2252","[RU=1, US=1]");
		myHCountry.put("2253","[RU=1, US=1]");
		myHCountry.put("2254","[RU=1, US=1]");
		myHCountry.put("2255","[RU=5, US=1]");
		myHCountry.put("2256","[RU=1, US=1]");
		myHCountry.put("2257","[RU=1, US=1]");
		myHCountry.put("2258","[US=7, IT=1, FR=1]");
		myHCountry.put("2259","[RU=1, US=1]");
		myHCountry.put("2260","[RU=1, US=1]");
		myHCountry.put("2261","[RU=1, US=1]");
		myHCountry.put("2262","[RU=1, US=1]");
		myHCountry.put("2263","[US=2, IT=1, BR=1, RU=1]");
		myHCountry.put("2264","[TZ=2, ZM=1]");
		myHCountry.put("2265","[ZM=6, ZA=4, TZ=2, BE=2, MW=1]");
		myHCountry.put("2266","[TZ=2]");
		myHCountry.put("2267","[TZ=2, MW=1]");
		myHCountry.put("2268","[MZ=3, DK=1, TZ=1, ZM=1, MG=1]");
		myHCountry.put("2269","[ZA=3, OM=1]");
		myHCountry.put("2270","[BE=1, ES=1, ZA=1]");
		myHCountry.put("2271","[ZA=9]");
		myHCountry.put("2272","[ZA=1, FR=1, AT=1]");
		myHCountry.put("2273","[ZA=2, IT=1]");
		myHCountry.put("2274","[PE=6, US=4]");
		myHCountry.put("2275","[ZA=2, FR=2]");
		myHCountry.put("2276","[CN=2, US=2]");
		myHCountry.put("2277","[ZA=3, ET=1, OM=1]");
		myHCountry.put("2278","[ZA=5]");
		myHCountry.put("2279","[ZA=2, ZW=1]");
		myHCountry.put("2280","[TZ=1, ZA=1]");
		myHCountry.put("2281","[DZ=1, ZA=1]");
		myHCountry.put("2282","[ZA=3, IT=1, AT=1]");
		myHCountry.put("2283","[ZA=2]");
		myHCountry.put("2284","[ZA=3]");
		myHCountry.put("2285","[TR=1, BE=1, ZA=1]");
		myHCountry.put("2286","[ZA=3, BR=1]");
		myHCountry.put("2287","[ZA=2]");
		myHCountry.put("2288","[US=2]");
		myHCountry.put("2289","[US=2]");
		myHCountry.put("2290","[US=2]");
		myHCountry.put("2291","[US=2]");
		myHCountry.put("2292","[US=3]");
		myHCountry.put("2293","[US=2]");
		myHCountry.put("2294","[ZA=5]");
		myHCountry.put("2295","[ZA=3]");
		myHCountry.put("2296","[ZA=5, JM=1]");
		myHCountry.put("2297","[ZA=4, ES=3, TR=1]");
		myHCountry.put("2298","[ZA=1, GE=1]");
		myHCountry.put("2299","[ZA=2]");
		myHCountry.put("2300","[ZA=4, TN=1]");
		myHCountry.put("2301","[ZA=1, AR=1]");
		myHCountry.put("2302","[ZA=4, US=1]");
		myHCountry.put("2303","[ZA=6]");
		myHCountry.put("2304","[ZA=5, RU=1]");
		myHCountry.put("2305","[ZA=2, US=1]");
		myHCountry.put("2306","[ZA=2, TN=1, PL=1]");
		myHCountry.put("2307","[ZA=3, TN=2]");
		myHCountry.put("2308","[ZA=2]");
		myHCountry.put("2309","[ZA=6]");
		myHCountry.put("2310","[BR=1, GF=1, ZA=1]");
		myHCountry.put("2311","[US=2, BR=1]");
		myHCountry.put("2312","[US=2, GM=1]");
		myHCountry.put("2313","[US=2]");
		myHCountry.put("2314","[US=2]");
		myHCountry.put("2315","[US=2]");
		myHCountry.put("2316","[US=3, JP=1]");
		myHCountry.put("2317","[US=2]");
		myHCountry.put("2318","[US=2]");
		myHCountry.put("2319","[US=2]");
		myHCountry.put("2320","[ES=2, US=2]");
		myHCountry.put("2321","[US=2]");
		myHCountry.put("2322","[TW=4, US=2, SR=1, CN=1]");
		myHCountry.put("2323","[US=2]");
		myHCountry.put("2324","[US=3]");
		myHCountry.put("2325","[US=2]");
		myHCountry.put("2326","[US=4, BR=1]");
		myHCountry.put("2327","[BR=1, ZA=1]");
		myHCountry.put("2328","[ZA=3]");
		myHCountry.put("2329","[MA=6, ZA=6, ES=1]");
		myHCountry.put("2330","[MA=2, ZA=2]");
		myHCountry.put("2331","[MA=1, BE=1, BR=1, ZA=1]");
		myHCountry.put("2332","[MA=15, ZA=7, IT=1]");
		myHCountry.put("2333","[MA=5, ZA=2, BR=1, CU=1, FR=1, PL=1]");
		myHCountry.put("2334","[ZA=2]");
		myHCountry.put("2335","[ZA=2]");
		myHCountry.put("2336","[BR=3, MA=1, ES=1, ZA=1]");
		myHCountry.put("2337","[US=2, MA=1, IT=1, AL=1, ZA=1, MX=1, FR=1]");
		myHCountry.put("2338","[MA=3, FR=3, ES=1, ZA=1, US=1]");
		myHCountry.put("2339","[ZA=2]");
		myHCountry.put("2340","[ZA=2, TN=1]");
		myHCountry.put("2341","[MA=2, CU=1, ZA=1]");
		myHCountry.put("2342","[BR=2, IT=1, ZA=1]");
		myHCountry.put("2343","[ZA=2, BR=1]");
		myHCountry.put("2344","[ZA=2]");
		myHCountry.put("2345","[RU=6, ZA=1, LV=1]");
		myHCountry.put("2346","[US=2]");
		myHCountry.put("2347","[US=2]");
		myHCountry.put("2348","[US=4]");
		myHCountry.put("2349","[US=2, NL=1]");
		myHCountry.put("2350","[US=2]");
		myHCountry.put("2351","[US=3]");
		myHCountry.put("2352","[US=2]");
		myHCountry.put("2353","[US=2]");
		myHCountry.put("2354","[LV=2, CZ=2]");
		myHCountry.put("2355","[US=2]");
		myHCountry.put("2356","[US=2, MX=1]");
		myHCountry.put("2357","[US=2]");
		myHCountry.put("2358","[US=2]");
		myHCountry.put("2359","[ET=16, US=3, CN=1, IQ=1, OM=1, SD=1]");
		myHCountry.put("2360","[US=2]");
		myHCountry.put("2361","[US=2]");
		myHCountry.put("2362","[US=2]");
		myHCountry.put("2363","[US=2]");
		myHCountry.put("2364","[US=2, PK=1]");
		myHCountry.put("2365","[US=2]");
		myHCountry.put("2366","[US=2, CN=1, AR=1, LV=1]");
		myHCountry.put("2367","[ET=2, ZA=1, US=1, FR=1, JP=1]");
		myHCountry.put("2368","[ZM=1, US=1]");
		myHCountry.put("2369","[US=2]");
		myHCountry.put("2370","[ES=10, IT=1, FR=1]");
		myHCountry.put("2371","[BE=1, US=1]");
		myHCountry.put("2372","[BE=2, MA=1]");
		myHCountry.put("2373","[BE=1, US=1]");
		myHCountry.put("2374","[BE=1, SE=1, AT=1]");
		myHCountry.put("2375","[ZA=7, PE=2, BE=2, DE=1, MZ=1]");
		myHCountry.put("2376","[BE=2, FR=1]");
		myHCountry.put("2377","[BE=2]");
		myHCountry.put("2378","[BE=5]");
		myHCountry.put("2379","[BE=2, IR=1, US=1]");
		myHCountry.put("2380","[ZA=16, BE=1, ES=1]");
		myHCountry.put("2381","[BR=4, IT=1, PE=1, BE=1, RU=1]");
		myHCountry.put("2382","[BE=2, FR=2]");
		myHCountry.put("2383","[CO=2, PE=1, GP=1, ES=1]");
		myHCountry.put("2384","[BE=3]");
		myHCountry.put("2385","[BE=4, FR=2, ZM=1]");
		myHCountry.put("2386","[BE=1, BR=1]");
		myHCountry.put("2387","[FR=4, BE=2, TN=1]");
		myHCountry.put("2388","[ZA=2, MQ=1]");
		myHCountry.put("2389","[IR=3]");
		myHCountry.put("2390","[IR=3]");
		myHCountry.put("2391","[IR=2, ET=2, SA=1, ZA=1, US=1, OM=1]");
		myHCountry.put("2392","[IR=4, IN=2, SA=1, US=1, OM=1, SD=1]");
		myHCountry.put("2393","[TZ=7, IR=2, TW=1, IT=1]");
		myHCountry.put("2394","[US=3]");
		myHCountry.put("2395","[US=4, FR=1]");
		myHCountry.put("2396","[US=3]");
		myHCountry.put("2397","[US=5, GM=2]");
		myHCountry.put("2398","[GM=3, US=2, SA=1, ET=1]");
		myHCountry.put("2399","[US=3, ES=1]");
		myHCountry.put("2400","[SA=3, US=2]");
		myHCountry.put("2401","[US=2, FR=1]");
		myHCountry.put("2402","[ZA=1, US=1]");
		myHCountry.put("2403","[US=3, MX=2]");
		myHCountry.put("2404","[BD=1, US=1]");
		myHCountry.put("2405","[US=2]");
		myHCountry.put("2406","[JM=9, US=2, ES=1]");
		myHCountry.put("2407","[IT=1, US=1, NG=1]");
		myHCountry.put("2408","[GR=1, US=1]");
		myHCountry.put("2409","[ET=3, US=2, BR=1]");
		myHCountry.put("2410","[US=3, CO=1]");
		myHCountry.put("2411","[ZA=1, US=1]");
		myHCountry.put("2412","[US=2]");
		myHCountry.put("2413","[CN=5, AU=1, US=1]");
		myHCountry.put("2414","[IT=1, US=1]");
		myHCountry.put("2415","[BR=1, US=1]");
		myHCountry.put("2416","[US=2]");
		myHCountry.put("2417","[US=2]");
		myHCountry.put("2418","[HT=1, US=1]");
		myHCountry.put("2419","[IN=4, US=3, PK=1]");
		myHCountry.put("2420","[CN=1, IT=1, ES=1, GF=1, US=1]");
		myHCountry.put("2421","[GM=1, BE=1, US=1]");
		myHCountry.put("2422","[CZ=2]");
		myHCountry.put("2423","[CZ=2, FR=1]");
		myHCountry.put("2424","[CZ=2, FR=1]");
		myHCountry.put("2425","[CZ=2]");
		myHCountry.put("2426","[CZ=2]");
		myHCountry.put("2427","[CZ=2]");
		myHCountry.put("2428","[NG=2, BR=1, GF=1, US=1]");
		myHCountry.put("2429","[NG=2]");
		myHCountry.put("2430","[IE=1, AR=1]");
		myHCountry.put("2431","[US=2]");
		myHCountry.put("2432","[ES=2]");
		myHCountry.put("2433","[ES=2]");
		myHCountry.put("2434","[ES=2]");
		myHCountry.put("2435","[US=2]");
		myHCountry.put("2436","[US=2]");
		myHCountry.put("2437","[MQ=1, GM=1, GP=1, ES=1, US=1]");
		myHCountry.put("2438","[GM=1, US=1]");
		myHCountry.put("2439","[GM=3]");
		myHCountry.put("2440","[GM=2, GW=1]");
		myHCountry.put("2441","[GM=2]");
		myHCountry.put("2442","[GM=2]");
		myHCountry.put("2443","[GM=2]");
		myHCountry.put("2444","[GM=1, FR=1]");
		myHCountry.put("2445","[GW=2, GM=2, FR=1]");
		myHCountry.put("2446","[GM=3]");
		myHCountry.put("2447","[BR=2]");
		myHCountry.put("2448","[BR=1, US=1]");
		myHCountry.put("2449","[BR=2, IT=1]");
		myHCountry.put("2450","[US=2]");
		myHCountry.put("2451","[US=2]");
		myHCountry.put("2452","[US=2]");
		myHCountry.put("2453","[US=2]");
		myHCountry.put("2454","[US=2]");
		myHCountry.put("2455","[US=2]");
		myHCountry.put("2456","[US=2]");
		myHCountry.put("2457","[US=2, IN=1]");
		myHCountry.put("2458","[US=2]");
		myHCountry.put("2459","[US=2, IN=1]");
		myHCountry.put("2460","[US=2]");
		myHCountry.put("2461","[US=2]");
		myHCountry.put("2462","[US=2]");
		myHCountry.put("2463","[US=2]");
		myHCountry.put("2464","[US=2]");
		myHCountry.put("2465","[US=2]");
		myHCountry.put("2466","[US=2, CO=1]");
		myHCountry.put("2467","[US=2]");
		myHCountry.put("2468","[US=2]");
		myHCountry.put("2469","[US=2]");
		myHCountry.put("2470","[US=2, UG=1]");
		myHCountry.put("2471","[US=2]");
		myHCountry.put("2472","[US=2]");
		myHCountry.put("2473","[US=2]");
		myHCountry.put("2474","[US=2]");
		myHCountry.put("2475","[US=2]");
		myHCountry.put("2476","[US=2]");
		myHCountry.put("2477","[US=2]");
		myHCountry.put("2478","[BE=3]");
		myHCountry.put("2479","[US=2]");
		myHCountry.put("2480","[US=2]");
		myHCountry.put("2481","[US=2]");
		myHCountry.put("2482","[OM=2, TZ=1, ZM=1]");
		myHCountry.put("2483","[TZ=2]");
		myHCountry.put("2484","[TZ=2, US=1]");
		myHCountry.put("2485","[SE=2, TZ=1, IN=1, SD=1]");
		myHCountry.put("2486","[ZM=5]");
		myHCountry.put("2487","[ZM=5, ZA=1]");
		myHCountry.put("2488","[ZM=4, ZA=1]");
		myHCountry.put("2489","[ZM=5]");
		myHCountry.put("2490","[ZM=2, US=1]");
		myHCountry.put("2491","[ZM=2, ZA=1]");
		myHCountry.put("2492","[ZM=3, BE=1]");
		myHCountry.put("2493","[ZM=2]");
		myHCountry.put("2494","[BG=4, ZM=1, ZA=1]");
		myHCountry.put("2495","[US=2, ZM=1, BR=1]");
		myHCountry.put("2496","[ES=11, IR=1, ZM=1, MX=1]");
		myHCountry.put("2497","[BR=4, CO=1, PT=1]");
		myHCountry.put("2498","[BR=8]");
		myHCountry.put("2499","[BR=5, PE=1, ZA=1]");
		myHCountry.put("2500","[BR=1, BG=1]");
		myHCountry.put("2501","[BR=3, IT=1, ES=1]");
		myHCountry.put("2502","[BR=3, PE=2]");
		myHCountry.put("2503","[BR=10, AT=1]");
		myHCountry.put("2504","[PL=3, AT=1]");
		myHCountry.put("2505","[BR=2]");
		myHCountry.put("2506","[TN=2, GY=1, BR=1, AR=1]");
		myHCountry.put("2507","[BR=2]");
		myHCountry.put("2508","[BR=10, GF=5]");
		myHCountry.put("2509","[BR=2, AL=1]");
		myHCountry.put("2510","[BR=2, IT=1]");
		myHCountry.put("2511","[BR=3, IT=1, BE=1]");
		myHCountry.put("2512","[BR=5]");
		myHCountry.put("2513","[BR=3]");
		myHCountry.put("2514","[BR=2]");
		myHCountry.put("2515","[BR=3]");
		myHCountry.put("2516","[MA=2, BR=2, IT=1]");
		myHCountry.put("2517","[BR=8]");
		myHCountry.put("2518","[BR=2]");
		myHCountry.put("2519","[BR=3]");
		myHCountry.put("2520","[BR=2]");
		myHCountry.put("2521","[BR=3, GF=1, VE=1]");
		myHCountry.put("2522","[BR=2, GF=1, US=1]");
		myHCountry.put("2523","[BR=3]");
		myHCountry.put("2524","[BR=3]");
		myHCountry.put("2525","[BR=7, US=1]");
		myHCountry.put("2526","[BR=2, NL=1]");
		myHCountry.put("2527","[BR=1, ZA=1]");
		myHCountry.put("2528","[BR=2]");
		myHCountry.put("2529","[BR=3]");
		myHCountry.put("2530","[BR=1, ZA=1]");
		myHCountry.put("2531","[ZA=21, BR=2]");
		myHCountry.put("2532","[BR=6]");
		myHCountry.put("2533","[BR=2]");
		myHCountry.put("2534","[BR=2]");
		myHCountry.put("2535","[BR=3, US=1]");
		myHCountry.put("2536","[BR=2]");
		myHCountry.put("2537","[BR=3, ES=3]");
		myHCountry.put("2538","[PL=4, CN=2, RU=1]");
		myHCountry.put("2539","[BR=2]");
		myHCountry.put("2540","[BR=1, RU=1, FR=1]");
		myHCountry.put("2541","[BR=2]");
		myHCountry.put("2542","[BR=3]");
		myHCountry.put("2543","[BR=2]");
		myHCountry.put("2544","[BR=2]");
		myHCountry.put("2545","[BR=2]");
		myHCountry.put("2546","[BR=2]");
		myHCountry.put("2547","[BR=2]");
		myHCountry.put("2548","[BR=2]");
		myHCountry.put("2549","[BR=2]");
		myHCountry.put("2550","[TT=7, NG=4, TR=1, BR=1, US=1, MG=1]");
		myHCountry.put("2551","[BR=2, ET=2, CO=1, FR=1]");
		myHCountry.put("2552","[BR=3]");
		myHCountry.put("2553","[BR=4, PK=1]");
		myHCountry.put("2554","[BR=2]");
		myHCountry.put("2555","[PK=1, BR=1]");
		myHCountry.put("2556","[BR=2]");
		myHCountry.put("2557","[BR=3]");
		myHCountry.put("2558","[JP=2, TR=1]");
		myHCountry.put("2559","[TR=2]");
		myHCountry.put("2560","[TR=2, NL=1]");
		myHCountry.put("2561","[TR=1, GP=1]");
		myHCountry.put("2562","[IT=1, BR=1]");
		myHCountry.put("2563","[BR=2, ZA=1]");
		myHCountry.put("2564","[BR=2]");
		myHCountry.put("2565","[GB=1, BR=1]");
		myHCountry.put("2566","[BR=3, CO=1, MG=1]");
		myHCountry.put("2567","[MA=2, PL=2]");
		myHCountry.put("2568","[BR=2]");
		myHCountry.put("2569","[BR=3]");
		myHCountry.put("2570","[BR=2, SE=1]");
		myHCountry.put("2571","[BR=3]");
		myHCountry.put("2572","[BR=2, FR=1]");
		myHCountry.put("2573","[BR=3]");
		myHCountry.put("2574","[BR=1, US=1]");
		myHCountry.put("2575","[BE=2]");
		myHCountry.put("2576","[MA=1, BE=1, ZA=1]");
		myHCountry.put("2577","[BG=5, BE=2, FR=1, NG=1, JP=1]");
		myHCountry.put("2578","[BE=3]");
		myHCountry.put("2579","[BE=3]");
		myHCountry.put("2580","[BE=1, BR=1]");
		myHCountry.put("2581","[BE=2]");
		myHCountry.put("2582","[BE=1, GB=1]");
		myHCountry.put("2583","[SA=2, NL=1]");
		myHCountry.put("2584","[SA=2]");
		myHCountry.put("2585","[SA=6, YE=2]");
		myHCountry.put("2586","[SA=2]");
		myHCountry.put("2587","[SA=1, AU=1, US=1]");
		myHCountry.put("2588","[SA=2]");
		myHCountry.put("2589","[OM=2, SA=1, KE=1, YE=1]");
		myHCountry.put("2590","[SA=1, US=1]");
		myHCountry.put("2591","[SA=1, PK=1]");
		myHCountry.put("2592","[SA=6, IT=1]");
		myHCountry.put("2593","[SA=2, OM=1]");
		myHCountry.put("2594","[SA=3]");
		myHCountry.put("2595","[SA=3]");
		myHCountry.put("2596","[SA=6, ET=1]");
		myHCountry.put("2597","[ES=10, IT=1, SD=1]");
		myHCountry.put("2598","[ES=8]");
		myHCountry.put("2599","[ES=2]");
		myHCountry.put("2600","[ES=7]");
		myHCountry.put("2601","[ES=1, IN=1]");
		myHCountry.put("2602","[ES=15, IN=1]");
		myHCountry.put("2603","[ES=3, AR=1]");
		myHCountry.put("2604","[ES=6]");
		myHCountry.put("2605","[ES=2]");
		myHCountry.put("2606","[ES=2]");
		myHCountry.put("2607","[ES=3]");
		myHCountry.put("2608","[ES=2]");
		myHCountry.put("2609","[ES=3]");
		myHCountry.put("2610","[SA=1, TW=1, CN=1, ES=1, JP=1]");
		myHCountry.put("2611","[ES=2]");
		myHCountry.put("2612","[ES=3]");
		myHCountry.put("2613","[FI=1, ES=1]");
		myHCountry.put("2614","[ES=2]");
		myHCountry.put("2615","[ES=3]");
		myHCountry.put("2616","[ES=1, MG=1, PT=1]");
		myHCountry.put("2617","[ES=2]");
		myHCountry.put("2618","[IT=1, ES=1]");
		myHCountry.put("2619","[ES=3]");
		myHCountry.put("2620","[ES=2]");
		myHCountry.put("2621","[IN=2, PK=1]");
		myHCountry.put("2622","[IT=1, ES=1]");
		myHCountry.put("2623","[BR=1, ES=1]");
		myHCountry.put("2624","[ES=1, US=1, SD=1]");
		myHCountry.put("2625","[ES=6]");
		myHCountry.put("2626","[ES=2, PE=1, US=1]");
		myHCountry.put("2627","[ES=2]");
		myHCountry.put("2628","[ES=2]");
		myHCountry.put("2629","[ES=2]");
		myHCountry.put("2630","[ES=12, BR=2]");
		myHCountry.put("2631","[ES=5, BR=2, NL=1]");
		myHCountry.put("2632","[SA=3, FR=2, MX=1]");
		myHCountry.put("2633","[IT=2]");
		myHCountry.put("2634","[IT=4]");
		myHCountry.put("2635","[IT=2]");
		myHCountry.put("2636","[IT=1, AR=1]");
		myHCountry.put("2637","[IT=2, IR=1, ES=1]");
		myHCountry.put("2638","[IT=1, BE=1]");
		myHCountry.put("2639","[IT=3]");
		myHCountry.put("2640","[IT=1, AT=1]");
		myHCountry.put("2641","[IT=2]");
		myHCountry.put("2642","[PY=1, MX=1]");
		myHCountry.put("2643","[PY=13, TR=1]");
		myHCountry.put("2644","[FR=4, PY=1, TN=1, SE=1]");
		myHCountry.put("2645","[PY=2]");
		myHCountry.put("2646","[PY=1, ZA=1]");
		myHCountry.put("2647","[PY=4, MA=2]");
		myHCountry.put("2648","[PY=1, MY=1]");
		myHCountry.put("2649","[PY=1, US=1]");
		myHCountry.put("2650","[PY=2]");
		myHCountry.put("2651","[PY=1, VE=1]");
		myHCountry.put("2652","[PA=2, PY=1, ES=1, PT=1]");
		myHCountry.put("2653","[PY=1, IT=1]");
		myHCountry.put("2654","[PY=3, BR=1]");
		myHCountry.put("2655","[PY=1, AT=1]");
		myHCountry.put("2656","[SA=2]");
		myHCountry.put("2657","[SA=3]");
		myHCountry.put("2658","[SA=1, MY=1]");
		myHCountry.put("2659","[SA=2]");
		myHCountry.put("2660","[FR=3, SA=2, NG=1]");
		myHCountry.put("2661","[SA=2]");
		myHCountry.put("2662","[SA=2]");
		myHCountry.put("2663","[SA=2]");
		myHCountry.put("2664","[SA=1, GR=1]");
		myHCountry.put("2665","[SA=1, US=1]");
		myHCountry.put("2666","[SA=3]");
		myHCountry.put("2667","[SA=1, BD=1]");
		myHCountry.put("2668","[IT=2, SA=1]");
		myHCountry.put("2669","[SA=58, TN=41, US=6, GB=2, JP=2, DO=1]");
		myHCountry.put("2670","[SA=2]");
		myHCountry.put("2671","[SA=25]");
		myHCountry.put("2672","[SA=1, US=1]");
		myHCountry.put("2673","[SA=1, US=1]");
		myHCountry.put("2674","[SA=1, NL=1]");
		myHCountry.put("2675","[SA=1, US=1]");
		myHCountry.put("2676","[SA=2]");
		myHCountry.put("2677","[SA=3]");
		myHCountry.put("2678","[SA=2, CN=1]");
		myHCountry.put("2679","[SA=4]");
		myHCountry.put("2680","[SA=3]");
		myHCountry.put("2681","[SA=2, GB=1]");
		myHCountry.put("2682","[SA=2, PK=1]");
		myHCountry.put("2683","[SA=6, NL=1, ET=1, SE=1, OM=1]");
		myHCountry.put("2684","[SA=3]");
		myHCountry.put("2685","[SA=3, GB=1]");
		myHCountry.put("2686","[IQ=4, SA=2]");
		myHCountry.put("2687","[SA=2]");
		myHCountry.put("2688","[SA=2]");
		myHCountry.put("2689","[SA=1, GP=1]");
		myHCountry.put("2690","[SA=3]");
		myHCountry.put("2691","[SA=3, TR=1, IQ=1]");
		myHCountry.put("2692","[SA=3, ET=1]");
		myHCountry.put("2693","[SA=4, IT=2, GB=1, PK=1]");
		myHCountry.put("2694","[SA=4, BE=1]");
		myHCountry.put("2695","[SA=3]");
		myHCountry.put("2696","[SA=3, GB=3, ET=1, ZA=1]");
		myHCountry.put("2697","[SA=4]");
		myHCountry.put("2698","[SA=3]");
		myHCountry.put("2699","[SA=1, VN=1]");
		myHCountry.put("2700","[SA=4]");
		myHCountry.put("2701","[SA=1, GB=1]");
		myHCountry.put("2702","[AR=4]");
		myHCountry.put("2703","[NL=1, US=1]");
		myHCountry.put("2704","[US=4]");
		myHCountry.put("2705","[IT=1, DO=1]");
		myHCountry.put("2706","[PK=2]");
		myHCountry.put("2707","[IQ=1, PK=1, IN=1]");
		myHCountry.put("2708","[DK=1, PK=1]");
		myHCountry.put("2709","[PK=2]");
		myHCountry.put("2710","[PK=2, NL=1]");
		myHCountry.put("2711","[PK=5]");
		myHCountry.put("2712","[PK=2]");
		myHCountry.put("2713","[PK=2]");
		myHCountry.put("2714","[PK=2]");
		myHCountry.put("2715","[PK=2]");
		myHCountry.put("2716","[PK=1, BR=1, SE=1]");
		myHCountry.put("2717","[SA=3]");
		myHCountry.put("2718","[SA=2]");
		myHCountry.put("2719","[SA=2]");
		myHCountry.put("2720","[TW=1, US=1]");
		myHCountry.put("2721","[FR=2, SE=1, US=1]");
		myHCountry.put("2722","[US=2]");
		myHCountry.put("2723","[US=2]");
		myHCountry.put("2724","[IT=3, DE=1, US=1]");
		myHCountry.put("2725","[US=2]");
		myHCountry.put("2726","[GB=1, IN=1, US=1]");
		myHCountry.put("2727","[MG=2]");
		myHCountry.put("2728","[IQ=3, SA=2, IR=1]");
		myHCountry.put("2729","[MG=41]");
		myHCountry.put("2730","[BE=2]");
		myHCountry.put("2731","[BE=1, PK=1]");
		myHCountry.put("2732","[MA=1, BE=1]");
		myHCountry.put("2733","[IT=1, BE=1, FR=1]");
		myHCountry.put("2734","[BE=2]");
		myHCountry.put("2735","[DE=1, ES=1, FR=1]");
		myHCountry.put("2736","[IT=1, EG=1, ES=1]");
		myHCountry.put("2737","[ES=3]");
		myHCountry.put("2738","[ES=2]");
		myHCountry.put("2739","[IT=1, ES=1]");
		myHCountry.put("2740","[SA=2]");
		myHCountry.put("2741","[CN=1, GP=1, ES=1, FR=1]");
		myHCountry.put("2742","[IT=1, ES=1]");
		myHCountry.put("2743","[ES=3, JP=1]");
		myHCountry.put("2744","[PE=1, BR=1, ES=1]");
		myHCountry.put("2745","[ES=1, US=1]");
		myHCountry.put("2746","[ES=3, BE=1]");
		myHCountry.put("2747","[ES=2]");
		myHCountry.put("2748","[ES=1, US=1]");
		myHCountry.put("2749","[ES=2]");
		myHCountry.put("2750","[ES=2]");
		myHCountry.put("2751","[ES=3]");
		myHCountry.put("2752","[ES=2]");
		myHCountry.put("2753","[BR=2]");
		myHCountry.put("2754","[IT=10, ES=2]");
		myHCountry.put("2755","[ES=1, GE=1]");
		myHCountry.put("2756","[IN=2, PK=1]");
		myHCountry.put("2757","[IN=3]");
		myHCountry.put("2758","[IN=4, US=1]");
		myHCountry.put("2759","[IT=1, IN=1]");
		myHCountry.put("2760","[IN=2]");
		myHCountry.put("2761","[IN=3]");
		myHCountry.put("2762","[GM=2]");
		myHCountry.put("2763","[SA=3]");
		myHCountry.put("2764","[ES=2, MZ=1]");
		myHCountry.put("2765","[GM=2, FR=1]");
		myHCountry.put("2766","[SA=2]");
		myHCountry.put("2767","[SA=2]");
		myHCountry.put("2768","[SA=1, GF=1]");
		myHCountry.put("2769","[BD=3]");
		myHCountry.put("2770","[BD=4]");
		myHCountry.put("2771","[BD=2]");
		myHCountry.put("2772","[BD=2]");
		myHCountry.put("2773","[BD=2]");
		myHCountry.put("2774","[MM=4, BD=2, GB=1]");
		myHCountry.put("2775","[SA=2, US=1]");
		myHCountry.put("2776","[BD=2, TR=1]");
		myHCountry.put("2777","[BD=1, MY=1]");
		myHCountry.put("2778","[BD=1, US=1]");
		myHCountry.put("2779","[DK=1, BD=1]");
		myHCountry.put("2780","[GB=1, BD=1, SE=1]");
		myHCountry.put("2781","[SA=2, CN=2, EG=1, IN=1, AR=1, NG=1]");
		myHCountry.put("2782","[SA=2]");
		myHCountry.put("2783","[OM=3, SA=1, NL=1, SE=1]");
		myHCountry.put("2784","[SA=2]");
		myHCountry.put("2785","[SA=2]");
		myHCountry.put("2786","[SA=1, AU=1]");
		myHCountry.put("2787","[VE=3]");
		myHCountry.put("2788","[VE=7, BR=1]");
		myHCountry.put("2789","[VE=3]");
		myHCountry.put("2790","[BR=1, VE=1]");
		myHCountry.put("2791","[GP=3, FR=1]");
		myHCountry.put("2792","[BE=1, IN=1, US=1]");
		myHCountry.put("2793","[ET=2, KE=1, US=1]");
		myHCountry.put("2794","[US=2]");
		myHCountry.put("2795","[TR=1, US=1]");
		myHCountry.put("2796","[US=2]");
		myHCountry.put("2797","[US=2, FR=1]");
		myHCountry.put("2798","[US=3]");
		myHCountry.put("2799","[ET=2]");
		myHCountry.put("2800","[ES=1, ET=1]");
		myHCountry.put("2801","[ET=2]");
		myHCountry.put("2802","[SA=2]");
		myHCountry.put("2803","[SA=2]");
		myHCountry.put("2804","[SA=2]");
		myHCountry.put("2805","[SA=2]");
		myHCountry.put("2806","[SA=3]");
		myHCountry.put("2807","[SA=1, ES=1, MZ=1, US=1]");
		myHCountry.put("2808","[SA=2]");
		myHCountry.put("2809","[SA=2]");
		myHCountry.put("2810","[SA=1, CN=1, US=1]");
		myHCountry.put("2811","[NG=4]");
		myHCountry.put("2812","[NG=3]");
		myHCountry.put("2813","[NG=29, TD=4, GB=1]");
		myHCountry.put("2814","[IT=1, GM=1]");
		myHCountry.put("2815","[GM=1, US=1]");
		myHCountry.put("2816","[GM=2]");
		myHCountry.put("2817","[GM=1, NL=1]");
		myHCountry.put("2819","[GM=1, US=1, FR=1]");
		myHCountry.put("2820","[ES=2, NG=2, CN=1, FR=1]");
		myHCountry.put("2822","[NG=8]");
		myHCountry.put("2823","[NG=3]");
		myHCountry.put("2824","[FR=1, NG=1]");
		myHCountry.put("2825","[NG=2]");
		myHCountry.put("2826","[NG=3]");
		myHCountry.put("2827","[NG=2]");
		myHCountry.put("2828","[NG=2]");
		myHCountry.put("2829","[NG=2]");
		myHCountry.put("2830","[US=1, NG=1]");
		myHCountry.put("2831","[NG=3]");
		myHCountry.put("2832","[NG=2, US=1]");
		myHCountry.put("2833","[NG=2]");
		myHCountry.put("2834","[US=1, NG=1]");
		myHCountry.put("2835","[NG=4]");
		myHCountry.put("2836","[MA=1, GR=1]");
		myHCountry.put("2837","[NG=2, FR=1]");
		myHCountry.put("2838","[NG=2]");
		myHCountry.put("2839","[NG=7, CM=3, TD=3]");
		myHCountry.put("2840","[NG=3]");
		myHCountry.put("2841","[NG=5]");
		myHCountry.put("2842","[TD=10, NG=5, CM=3]");
		myHCountry.put("2843","[NG=4]");
		myHCountry.put("2844","[TN=1, NG=1]");
		myHCountry.put("2845","[PK=1, BR=1, OM=1]");
		myHCountry.put("2846","[FI=1, CO=1, BR=1]");
		myHCountry.put("2847","[BR=2]");
		myHCountry.put("2848","[GF=2]");
		myHCountry.put("2849","[IT=3]");
		myHCountry.put("2850","[IT=2]");
		myHCountry.put("2851","[IT=2]");
		myHCountry.put("2852","[IT=6]");
		myHCountry.put("2853","[IT=2, US=1]");
		myHCountry.put("2854","[IT=1, BR=1]");
		myHCountry.put("2855","[IT=2]");
		myHCountry.put("2856","[IT=1, AR=1]");
		myHCountry.put("2857","[IT=1, BR=1, SE=1]");
		myHCountry.put("2858","[PE=4]");
		myHCountry.put("2859","[BR=2]");
		myHCountry.put("2860","[BR=5, US=1]");
		myHCountry.put("2861","[TN=1, GF=1]");
		myHCountry.put("2862","[TN=1, GF=1]");
		myHCountry.put("2863","[MQ=1, TN=1]");
		myHCountry.put("2864","[TN=1, GF=1]");
		myHCountry.put("2865","[TN=2, CO=1, US=1]");
		myHCountry.put("2866","[RU=2, TN=1, BR=1]");
		myHCountry.put("2867","[SA=1, TN=1, PK=1, ZA=1]");
		myHCountry.put("2868","[TN=2]");
		myHCountry.put("2869","[TN=2]");
		myHCountry.put("2870","[TN=2, BR=1]");
		myHCountry.put("2871","[TN=2, US=1]");
		myHCountry.put("2872","[TN=2]");
		myHCountry.put("2873","[TN=1, GB=1]");
		myHCountry.put("2874","[TN=3]");
		myHCountry.put("2875","[SN=1, TN=1]");
		myHCountry.put("2876","[CM=1, TN=1]");
		myHCountry.put("2877","[CM=1, TN=1]");
		myHCountry.put("2878","[TN=3]");
		myHCountry.put("2879","[TN=4]");
		myHCountry.put("2880","[TN=2]");
		myHCountry.put("2881","[TN=1, NL=1]");
		myHCountry.put("2882","[IT=1, BR=1]");
		myHCountry.put("2883","[BR=4, FR=2, US=1]");
		myHCountry.put("2884","[MQ=2, BR=1]");
		myHCountry.put("2885","[BR=9, PE=1, CU=1]");
		myHCountry.put("2886","[ID=1, MG=1]");
		myHCountry.put("2887","[MG=11, DK=1]");
		myHCountry.put("2888","[MG=2]");
		myHCountry.put("2889","[MG=2, FR=1]");
		myHCountry.put("2890","[RU=4, IT=1]");
		myHCountry.put("2891","[MA=3, ZA=1]");
		myHCountry.put("2892","[MA=2]");
		myHCountry.put("2893","[MA=1, ZA=1]");
		myHCountry.put("2894","[MA=1, ZA=1]");
		myHCountry.put("2895","[MA=1, ZA=1]");
		myHCountry.put("2896","[MA=1, ZA=1]");
		myHCountry.put("2897","[MA=1, JP=1]");
		myHCountry.put("2898","[MA=1, ZA=1]");
		myHCountry.put("2899","[MA=1, ZA=1, FR=1]");
		myHCountry.put("2900","[TR=2, IT=2, RU=2, AL=1, NG=1]");
		myHCountry.put("2901","[TR=1, US=1]");
		myHCountry.put("2902","[TR=3]");
		myHCountry.put("2903","[TR=2]");
		myHCountry.put("2904","[PA=2]");
		myHCountry.put("2905","[BG=6, FR=2]");
		myHCountry.put("2906","[BG=2]");
		myHCountry.put("2907","[LK=2]");
		myHCountry.put("2908","[LK=1, MY=1, SE=1, OM=1]");
		myHCountry.put("2909","[IN=3, LK=2]");
		myHCountry.put("2910","[LK=2, ID=1]");
		myHCountry.put("2911","[LK=1, ES=1]");
		myHCountry.put("2912","[SE=3]");
		myHCountry.put("2913","[SE=1, US=1]");
		myHCountry.put("2914","[MX=4, BE=1, SE=1]");
		myHCountry.put("2915","[SE=2, MZ=1]");
		myHCountry.put("2916","[SE=2, PE=1, US=1]");
		myHCountry.put("2917","[SE=1, US=1]");
		myHCountry.put("2918","[TZ=1, PK=1]");
		myHCountry.put("2919","[SA=1, TZ=1, MZ=1]");
		myHCountry.put("2920","[TZ=1, FR=1]");
		myHCountry.put("2921","[TZ=2, NO=1]");
		myHCountry.put("2922","[TZ=2]");
		myHCountry.put("2923","[TZ=2]");
		myHCountry.put("2924","[TZ=2]");
		myHCountry.put("2925","[TZ=2]");
		myHCountry.put("2926","[TZ=2]");
		myHCountry.put("2927","[TZ=3]");
		myHCountry.put("2928","[TZ=3]");
		myHCountry.put("2929","[BR=1, SE=1]");
		myHCountry.put("2930","[ES=1, SE=1]");
		myHCountry.put("2931","[PK=1, SE=1]");
		myHCountry.put("2932","[GB=1, SE=1]");
		myHCountry.put("2933","[GF=2]");
		myHCountry.put("2934","[TT=9]");
		myHCountry.put("2935","[TT=5, US=1]");
		myHCountry.put("2936","[AL=8, IT=2]");
		myHCountry.put("2937","[AL=9, GR=1]");
		myHCountry.put("2938","[AL=5, IT=1]");
		myHCountry.put("2939","[CO=5, US=1]");
		myHCountry.put("2940","[CO=5, TZ=1, PE=1]");
		myHCountry.put("2941","[CO=2, VE=1]");
		myHCountry.put("2942","[CO=2]");
		myHCountry.put("2943","[CO=2]");
		myHCountry.put("2944","[CO=2]");
		myHCountry.put("2945","[CO=1, US=1]");
		myHCountry.put("2946","[ES=3, FR=2, BE=1]");
		myHCountry.put("2947","[ES=2]");
		myHCountry.put("2948","[ES=2]");
		myHCountry.put("2949","[HU=1, ES=1]");
		myHCountry.put("2950","[GB=1, ES=1]");
		myHCountry.put("2951","[ES=2]");
		myHCountry.put("2952","[CN=3, ES=2]");
		myHCountry.put("2953","[TN=1, ES=1, FR=1]");
		myHCountry.put("2954","[ES=1, SE=1, VE=1]");
		myHCountry.put("2955","[SA=1, ES=1, RU=1]");
		myHCountry.put("2956","[KG=3]");
		myHCountry.put("2957","[PA=1, FR=1]");
		myHCountry.put("2958","[PA=5]");
		myHCountry.put("2959","[PA=2, ES=1]");
		myHCountry.put("2960","[PA=2]");
		myHCountry.put("2961","[PE=2]");
		myHCountry.put("2962","[PA=1, ES=1]");
		myHCountry.put("2963","[SD=30]");
		myHCountry.put("2964","[SD=2, ES=1]");
		myHCountry.put("2965","[SD=2]");
		myHCountry.put("2966","[SD=3]");
		myHCountry.put("2967","[SD=3]");
		myHCountry.put("2968","[SD=2]");
		myHCountry.put("2969","[US=1, SD=1]");
		myHCountry.put("2970","[SD=5, US=1]");
		myHCountry.put("2971","[SA=1, SD=1]");
		myHCountry.put("2972","[SD=2]");
		myHCountry.put("2973","[ET=4, BE=1, SD=1]");
		myHCountry.put("2974","[ZA=1, SD=1]");
		myHCountry.put("2977","[EG=2, ET=1]");
		myHCountry.put("2978","[EG=1, AT=1]");
		myHCountry.put("2979","[CN=5, MM=2, TW=1, EG=1]");
		myHCountry.put("2980","[GP=2, GF=1]");
		myHCountry.put("2981","[HT=1, BS=1]");
		myHCountry.put("2982","[JM=3]");
		myHCountry.put("2983","[JM=2]");
		myHCountry.put("2984","[NG=3, JM=1, US=1]");
		myHCountry.put("2985","[JM=2]");
		myHCountry.put("2986","[SR=1, SG=1]");
		myHCountry.put("2987","[SR=2, IN=1]");
		myHCountry.put("2988","[SR=2]");
		myHCountry.put("2989","[SR=2]");
		myHCountry.put("2990","[SR=2]");
		myHCountry.put("2991","[VC=2, BD=1]");
		myHCountry.put("2992","[ZA=2, GB=1]");
		myHCountry.put("2993","[GB=2, IN=2]");
		myHCountry.put("2994","[GB=1, US=1]");
		myHCountry.put("2995","[OM=6, TZ=1, GB=1, IN=1]");
		myHCountry.put("2996","[ZA=2]");
		myHCountry.put("2997","[ZA=9, AU=1, MZ=1]");
		myHCountry.put("2998","[ZA=2]");
		myHCountry.put("2999","[ZA=1, LV=1, PL=1]");
		myHCountry.put("3000","[PE=2]");
		myHCountry.put("3001","[PE=19, US=1]");
		myHCountry.put("3002","[BR=2]");
		myHCountry.put("3003","[BR=2]");
		myHCountry.put("3004","[PE=2]");
		myHCountry.put("3005","[PE=2]");
		myHCountry.put("3006","[PE=2, CN=1, ZA=1]");
		myHCountry.put("3007","[PE=1, US=1]");
		myHCountry.put("3008","[PE=1, HN=1, US=1]");
		myHCountry.put("3009","[PE=2]");
		myHCountry.put("3010","[CO=6, IT=1, PE=1, US=1]");
		myHCountry.put("3011","[PE=4, BR=1]");
		myHCountry.put("3012","[PE=2]");
		myHCountry.put("3013","[PE=2, US=1]");
		myHCountry.put("3014","[PE=1, AR=1]");
		myHCountry.put("3015","[PE=2]");
		myHCountry.put("3016","[PA=1, PE=1]");
		myHCountry.put("3017","[PE=3]");
		myHCountry.put("3018","[CO=1, US=1]");
		myHCountry.put("3019","[BR=2, CO=1, SN=1]");
		myHCountry.put("3020","[GW=2, CO=1, FR=1]");
		myHCountry.put("3021","[BR=2]");
		myHCountry.put("3022","[CO=3]");
		myHCountry.put("3023","[CO=1, CU=1]");
		myHCountry.put("3024","[BR=2]");
		myHCountry.put("3025","[TD=17, NG=12]");
		myHCountry.put("3026","[SD=2, ET=1]");
		myHCountry.put("3027","[IN=1, SD=1]");
		myHCountry.put("3028","[SD=6]");
		myHCountry.put("3029","[SD=2]");
		myHCountry.put("3030","[SD=2]");
		myHCountry.put("3031","[SD=3]");
		myHCountry.put("3032","[SD=2]");
		myHCountry.put("3033","[SD=4]");
		myHCountry.put("3034","[MX=1, PL=1]");
		myHCountry.put("3035","[BR=1, MX=1]");
		myHCountry.put("3036","[NL=1, MX=1]");
		myHCountry.put("3037","[MX=2]");
		myHCountry.put("3038","[SE=1, MX=1, US=1]");
		myHCountry.put("3039","[CO=4]");
		myHCountry.put("3040","[CO=1, ES=1]");
		myHCountry.put("3041","[CO=2, BR=1]");
		myHCountry.put("3042","[CO=1, VE=1]");
		myHCountry.put("3043","[CO=1, DE=1]");
		myHCountry.put("3044","[CO=2]");
		myHCountry.put("3045","[CO=1, VE=1]");
		myHCountry.put("3046","[BR=2]");
		myHCountry.put("3047","[CO=2]");
		myHCountry.put("3048","[MZ=2]");
		myHCountry.put("3049","[TH=1, MZ=1]");
		myHCountry.put("3050","[MZ=2]");
		myHCountry.put("3051","[MA=4, TZ=1]");
		myHCountry.put("3052","[IT=1, MZ=1]");
		myHCountry.put("3053","[MZ=2]");
		myHCountry.put("3054","[MZ=3, BR=1]");
		myHCountry.put("3055","[MZ=1, US=1]");
		myHCountry.put("3056","[MZ=2, CN=1, ZA=1]");
		myHCountry.put("3057","[MZ=2]");
		myHCountry.put("3058","[MZ=2]");
		myHCountry.put("3059","[BR=1, MZ=1]");
		myHCountry.put("3060","[MZ=1, ZA=1]");
		myHCountry.put("3061","[GB=1, MZ=1]");
		myHCountry.put("3062","[MZ=1, ZA=1]");
		myHCountry.put("3063","[ZM=1, MZ=1]");
		myHCountry.put("3064","[MZ=2]");
		myHCountry.put("3065","[GB=1, MZ=1]");
		myHCountry.put("3066","[MZ=2]");
		myHCountry.put("3067","[MZ=1, AR=1]");
		myHCountry.put("3068","[NG=2]");
		myHCountry.put("3069","[NG=3]");
		myHCountry.put("3070","[NG=2]");
		myHCountry.put("3071","[NG=4, IT=1]");
		myHCountry.put("3072","[MA=3]");
		myHCountry.put("3073","[MA=3, IT=1, US=1]");
		myHCountry.put("3074","[MA=2, NL=1]");
		myHCountry.put("3075","[MA=3, ES=1]");
		myHCountry.put("3076","[MA=7, IT=1, FR=1]");
		myHCountry.put("3077","[MA=1, BE=1]");
		myHCountry.put("3078","[MA=1, BE=1]");
		myHCountry.put("3079","[MX=2]");
		myHCountry.put("3080","[MX=2, SA=1, ET=1]");
		myHCountry.put("3081","[HN=3]");
		myHCountry.put("3082","[HN=2]");
		myHCountry.put("3083","[HN=2, BR=1]");
		myHCountry.put("3084","[HN=2]");
		myHCountry.put("3085","[HN=2, NL=1]");
		myHCountry.put("3086","[HN=3]");
		myHCountry.put("3087","[HN=2]");
		myHCountry.put("3088","[IT=1, HN=1]");
		myHCountry.put("3089","[PE=1, MX=1, FR=1]");
		myHCountry.put("3090","[MX=1, US=1]");
		myHCountry.put("3091","[CN=3, TW=1, SE=1, FR=1, PL=1]");
		myHCountry.put("3092","[PL=2]");
		myHCountry.put("3093","[IT=1, PL=1]");
		myHCountry.put("3094","[PL=2]");
		myHCountry.put("3095","[CN=7, US=1]");
		myHCountry.put("3096","[JP=2]");
		myHCountry.put("3097","[CN=5, VN=1]");
		myHCountry.put("3098","[FR=1, JP=1]");
		myHCountry.put("3099","[BR=4, CN=1]");
		myHCountry.put("3100","[GW=3]");
		myHCountry.put("3101","[GW=1, BR=1, ES=1]");
		myHCountry.put("3102","[BR=2, FR=1]");
		myHCountry.put("3103","[BR=1, VE=1]");
		myHCountry.put("3104","[GW=2]");
		myHCountry.put("3105","[BR=5, CU=1]");
		myHCountry.put("3106","[BR=1, AT=1]");
		myHCountry.put("3107","[BR=1, US=1]");
		myHCountry.put("3108","[RU=2]");
		myHCountry.put("3109","[KG=1, KZ=1, BE=1, IN=1, SE=1]");
		myHCountry.put("3110","[LV=2, KZ=1, SE=1]");
		myHCountry.put("3111","[GW=2]");
		myHCountry.put("3112","[BR=3]");
		myHCountry.put("3113","[BR=2]");
		myHCountry.put("3114","[BR=3]");
		myHCountry.put("3115","[BR=4]");
		myHCountry.put("3116","[BR=2]");
		myHCountry.put("3117","[BR=5]");
		myHCountry.put("3118","[GW=2]");
		myHCountry.put("3119","[BR=2]");
		myHCountry.put("3120","[BR=3]");
		myHCountry.put("3121","[BR=2]");
		myHCountry.put("3122","[BR=4]");
		myHCountry.put("3123","[BR=2]");
		myHCountry.put("3124","[BR=2, AR=1]");
		myHCountry.put("3125","[BR=2, IT=1]");
		myHCountry.put("3126","[BR=1, PT=1]");
		myHCountry.put("3127","[FR=2, BR=1]");
		myHCountry.put("3128","[BR=1, PT=1]");
		myHCountry.put("3129","[GW=2]");
		myHCountry.put("3130","[GW=1, NL=1]");
		myHCountry.put("3131","[GW=2]");
		myHCountry.put("3132","[GW=1, ID=1]");
		myHCountry.put("3133","[ET=2]");
		myHCountry.put("3134","[ET=44]");
		myHCountry.put("3135","[CN=1, ET=1, MX=1]");
		myHCountry.put("3136","[ET=2, ES=1]");
		myHCountry.put("3137","[ET=16, IT=1, AT=1]");
		myHCountry.put("3138","[ET=1, US=1]");
		myHCountry.put("3139","[CN=2, ET=2, SE=1]");
		myHCountry.put("3140","[ET=1, US=1]");
		myHCountry.put("3141","[ET=3, NL=1]");
		myHCountry.put("3142","[BR=1, AR=1]");
		myHCountry.put("3143","[BR=1, MG=1]");
		myHCountry.put("3144","[BR=2]");
		myHCountry.put("3145","[BR=2, CO=1]");
		myHCountry.put("3146","[TZ=1, BR=1]");
		myHCountry.put("3147","[BR=2]");
		myHCountry.put("3148","[BR=1, GF=1]");
		myHCountry.put("3149","[BR=1, FR=1]");
		myHCountry.put("3150","[BR=2]");
		myHCountry.put("3151","[BW=5]");
		myHCountry.put("3152","[RU=4, FI=1]");
		myHCountry.put("3153","[RU=1, US=1]");
		myHCountry.put("3154","[MX=2, US=1]");
		myHCountry.put("3155","[MX=1, FR=1]");
		myHCountry.put("3156","[SE=2]");
		myHCountry.put("3157","[SE=2]");
		myHCountry.put("3158","[SE=2]");
		myHCountry.put("3159","[SE=2]");
		myHCountry.put("3160","[SE=2]");
		myHCountry.put("3161","[OM=2, IT=1, PK=1, SE=1]");
		myHCountry.put("3162","[SE=1, US=1]");
		myHCountry.put("3163","[SE=3]");
		myHCountry.put("3164","[BE=1, SE=1, FR=1]");
		myHCountry.put("3165","[SE=2]");
		myHCountry.put("3166","[SE=2]");
		myHCountry.put("3167","[SE=1, US=1]");
		myHCountry.put("3168","[SE=3, PE=1]");
		myHCountry.put("3169","[SE=2, RU=1, US=1]");
		myHCountry.put("3170","[SE=6]");
		myHCountry.put("3171","[BR=1, SE=1]");
		myHCountry.put("3172","[SE=1, VE=1]");
		myHCountry.put("3173","[SE=2, PL=1]");
		myHCountry.put("3174","[SE=2]");
		myHCountry.put("3175","[SE=1, US=1]");
		myHCountry.put("3176","[SE=1, ZA=1]");
		myHCountry.put("3177","[GP=1, SE=1]");
		myHCountry.put("3178","[AL=2, SE=1]");
		myHCountry.put("3179","[SE=2, FR=2]");
		myHCountry.put("3180","[SE=2, US=1]");
		myHCountry.put("3181","[SE=1, US=1]");
		myHCountry.put("3182","[AU=1, SE=1]");
		myHCountry.put("3183","[AL=1, SE=1, FR=1]");
		myHCountry.put("3184","[SE=2]");
		myHCountry.put("3185","[SE=2]");
		myHCountry.put("3186","[SE=2]");
		myHCountry.put("3187","[SE=2]");
		myHCountry.put("3188","[BE=1, IN=1, SE=1, OM=1]");
		myHCountry.put("3189","[DE=1, SE=1]");
		myHCountry.put("3190","[DE=1, SE=1]");
		myHCountry.put("3191","[SE=2]");
		myHCountry.put("3192","[JP=2]");
		myHCountry.put("3193","[BR=1, JP=1]");
		myHCountry.put("3194","[JP=2]");
		myHCountry.put("3195","[SA=1, JP=1]");
		myHCountry.put("3196","[US=1, JP=1]");
		myHCountry.put("3197","[JP=2]");
		myHCountry.put("3198","[BR=1, JP=1]");
		myHCountry.put("3199","[CN=1, SE=1, JP=1]");
		myHCountry.put("3200","[CN=3]");
		myHCountry.put("3201","[CN=3]");
		myHCountry.put("3202","[CN=3, JP=1]");
		myHCountry.put("3203","[CN=2, ET=1]");
		myHCountry.put("3204","[CN=2, BR=1]");
		myHCountry.put("3205","[CN=2]");
		myHCountry.put("3206","[CN=2, US=1]");
		myHCountry.put("3207","[CN=2]");
		myHCountry.put("3208","[CN=2, IN=1]");
		myHCountry.put("3209","[CN=2]");
		myHCountry.put("3210","[CN=2]");
		myHCountry.put("3211","[CN=2]");
		myHCountry.put("3212","[CN=2, ZA=1]");
		myHCountry.put("3213","[CN=2]");
		myHCountry.put("3214","[CN=2, FR=1]");
		myHCountry.put("3215","[CN=2]");
		myHCountry.put("3216","[CN=2]");
		myHCountry.put("3217","[CN=2]");
		myHCountry.put("3218","[CN=2]");
		myHCountry.put("3219","[CN=1, US=1]");
		myHCountry.put("3220","[CN=1, VE=1]");
		myHCountry.put("3221","[CN=1, FR=1]");
		myHCountry.put("3222","[CN=1, FR=1]");
		myHCountry.put("3223","[CN=1, ZW=1, IN=1]");
		myHCountry.put("3224","[SA=1, CN=1]");
		myHCountry.put("3225","[CN=1, IT=1]");
		myHCountry.put("3226","[CN=1, IR=1]");
		myHCountry.put("3227","[SA=1, CN=1]");
		myHCountry.put("3228","[CN=2]");
		myHCountry.put("3229","[CN=1, MY=1]");
		myHCountry.put("3230","[TW=1, CN=1]");
		myHCountry.put("3231","[CN=1, NL=1]");
		myHCountry.put("3232","[CN=1, BR=1]");
		myHCountry.put("3233","[CN=3]");
		myHCountry.put("3234","[CN=2, LK=1, FR=1]");
		myHCountry.put("3235","[CN=2]");
		myHCountry.put("3236","[CN=2]");
		myHCountry.put("3237","[CN=1, US=1]");
		myHCountry.put("3238","[CN=2]");
		myHCountry.put("3239","[CN=1, FR=1]");
		myHCountry.put("3240","[CN=2]");
		myHCountry.put("3241","[CN=2, US=1, OM=1]");
		myHCountry.put("3242","[IT=1, US=1]");
		myHCountry.put("3243","[IT=1, NG=1]");
		myHCountry.put("3244","[IT=1, US=1]");
		myHCountry.put("3245","[IT=2]");
		myHCountry.put("3246","[IT=1, AR=1]");
		myHCountry.put("3247","[IT=1, ET=1, AT=1]");
		myHCountry.put("3248","[IT=3, IN=3, PK=1]");
		myHCountry.put("3249","[IT=1, GF=1]");
		myHCountry.put("3250","[IT=1, BE=1]");
		myHCountry.put("3251","[IT=1, BR=1]");
		myHCountry.put("3252","[IT=2]");
		myHCountry.put("3253","[IT=2]");
		myHCountry.put("3254","[IT=1, US=1]");
		myHCountry.put("3255","[IT=1, BE=1]");
		myHCountry.put("3256","[IT=3, FR=1]");
		myHCountry.put("3257","[IT=2]");
		myHCountry.put("3258","[PY=1, IT=1]");
		myHCountry.put("3259","[IT=1, SE=1]");
		myHCountry.put("3260","[IT=1, BG=1, AT=1]");
		myHCountry.put("3261","[IT=2]");
		myHCountry.put("3262","[IT=2]");
		myHCountry.put("3263","[IT=2]");
		myHCountry.put("3264","[IT=1, PK=1]");
		myHCountry.put("3265","[IT=2]");
		myHCountry.put("3266","[IT=1, AT=1]");
		myHCountry.put("3267","[IT=1, US=1]");
		myHCountry.put("3268","[IT=1, FR=1]");
		myHCountry.put("3269","[IT=1, PE=1]");
		myHCountry.put("3270","[IT=1, RU=1]");
		myHCountry.put("3271","[IT=1, GB=1]");
		myHCountry.put("3272","[IT=2, FR=2]");
		myHCountry.put("3273","[IT=1, VE=1]");
		myHCountry.put("3274","[IT=1, BD=1]");
		myHCountry.put("3275","[IQ=1, FR=1]");
		myHCountry.put("3276","[IT=1, CO=1, SE=1]");
		myHCountry.put("3277","[MX=2]");
		myHCountry.put("3278","[MX=2, ES=1]");
		myHCountry.put("3279","[NL=1, MX=1]");
		myHCountry.put("3280","[NL=1, MX=1]");
		myHCountry.put("3281","[ES=1, KR=1]");
		myHCountry.put("3282","[KR=3]");
		myHCountry.put("3283","[KR=1, US=1]");
		myHCountry.put("3284","[BE=2, MW=1]");
		myHCountry.put("3285","[BE=1, VE=1]");
		myHCountry.put("3286","[BE=2]");
		myHCountry.put("3287","[BE=2]");
		myHCountry.put("3288","[BE=2]");
		myHCountry.put("3289","[BE=2, ZM=1]");
		myHCountry.put("3290","[BE=1, BR=1]");
		myHCountry.put("3291","[BE=2, ZM=1]");
		myHCountry.put("3292","[BE=5]");
		myHCountry.put("3293","[BE=2]");
		myHCountry.put("3294","[BE=2, US=1]");
		myHCountry.put("3295","[BE=2, ZM=1]");
		myHCountry.put("3296","[BE=1, GR=1]");
		myHCountry.put("3297","[BE=1, US=1]");
		myHCountry.put("3298","[BE=3]");
		myHCountry.put("3299","[BE=3]");
		myHCountry.put("3300","[BE=1, BR=1]");
		myHCountry.put("3301","[BE=2]");
		myHCountry.put("3302","[BE=1, MZ=1]");
		myHCountry.put("3303","[BE=2, US=1]");
		myHCountry.put("3304","[BE=1, ZA=1]");
		myHCountry.put("3305","[BE=1, GB=1]");
		myHCountry.put("3306","[CO=12]");
		myHCountry.put("3307","[CO=4]");
		myHCountry.put("3308","[CO=2]");
		myHCountry.put("3309","[CO=3, ET=2, CN=1]");
		myHCountry.put("3310","[CO=1, US=1]");
		myHCountry.put("3311","[CO=3]");
		myHCountry.put("3312","[CO=2]");
		myHCountry.put("3313","[FR=4, BE=1, ET=1]");
		myHCountry.put("3314","[ET=7]");
		myHCountry.put("3315","[ET=7]");
		myHCountry.put("3316","[SA=1, ET=1]");
		myHCountry.put("3317","[ET=2]");
		myHCountry.put("3318","[ET=1, FR=1]");
		myHCountry.put("3319","[GB=1, ET=1]");
		myHCountry.put("3320","[ET=2]");
		myHCountry.put("3321","[ET=4, CN=2, YE=1]");
		myHCountry.put("3322","[ET=2]");
		myHCountry.put("3323","[ET=2]");
		myHCountry.put("3324","[ET=3]");
		myHCountry.put("3325","[SA=1, ET=1, OM=1]");
		myHCountry.put("3326","[ET=2]");
		myHCountry.put("3327","[ET=3]");
		myHCountry.put("3328","[DK=1, ET=1]");
		myHCountry.put("3329","[ET=1, US=1]");
		myHCountry.put("3330","[ET=2]");
		myHCountry.put("3331","[NL=1, ET=1]");
		myHCountry.put("3332","[ET=2]");
		myHCountry.put("3333","[ET=2]");
		myHCountry.put("3334","[ET=2]");
		myHCountry.put("3335","[ET=1, SE=1]");
		myHCountry.put("3336","[ET=2]");
		myHCountry.put("3337","[ET=2]");
		myHCountry.put("3338","[LV=3, FR=1]");
		myHCountry.put("3339","[RU=1, LV=1]");
		myHCountry.put("3340","[LV=3]");
		myHCountry.put("3341","[LV=1, AT=1]");
		myHCountry.put("3342","[LV=2]");
		myHCountry.put("3343","[LV=2]");
		myHCountry.put("3344","[LV=2]");
		myHCountry.put("3345","[LV=2]");
		myHCountry.put("3346","[IQ=11]");
		myHCountry.put("3347","[FR=6, GP=1]");
		myHCountry.put("3348","[PK=1, PS=1, OM=1]");
		myHCountry.put("3349","[CN=1, RU=1]");
		myHCountry.put("3350","[RU=4]");
		myHCountry.put("3351","[RU=2]");
		myHCountry.put("3352","[RU=2]");
		myHCountry.put("3353","[IN=3]");
		myHCountry.put("3354","[IN=2]");
		myHCountry.put("3355","[IT=1, IN=1]");
		myHCountry.put("3356","[IN=1, US=1]");
		myHCountry.put("3357","[IN=2]");
		myHCountry.put("3358","[SA=1, IN=1]");
		myHCountry.put("3359","[IN=1, SG=1]");
		myHCountry.put("3360","[IN=4]");
		myHCountry.put("3361","[IN=1, ZA=1]");
		myHCountry.put("3362","[ET=2, US=1]");
		myHCountry.put("3363","[ET=2]");
		myHCountry.put("3364","[ET=2]");
		myHCountry.put("3365","[ET=1, US=1]");
		myHCountry.put("3366","[IN=3]");
		myHCountry.put("3367","[IN=2, ID=1]");
		myHCountry.put("3368","[NL=1, IN=1]");
		myHCountry.put("3369","[SA=1, IN=1]");
		myHCountry.put("3370","[IN=2, OM=1]");
		myHCountry.put("3371","[IQ=2, DE=1, IN=1]");
		myHCountry.put("3372","[IN=2]");
		myHCountry.put("3373","[IN=2]");
		myHCountry.put("3374","[IN=2]");
		myHCountry.put("3375","[IN=1, FR=1]");
		myHCountry.put("3376","[IN=4]");
		myHCountry.put("3377","[IN=2, TZ=1]");
		myHCountry.put("3378","[IN=1, US=1]");
		myHCountry.put("3379","[IN=2]");
		myHCountry.put("3380","[IN=2]");
		myHCountry.put("3381","[IN=2]");
		myHCountry.put("3382","[IN=2]");
		myHCountry.put("3383","[IN=2]");
		myHCountry.put("3384","[IN=3]");
		myHCountry.put("3385","[IN=2]");
		myHCountry.put("3386","[IN=2]");
		myHCountry.put("3387","[IN=2]");
		myHCountry.put("3388","[IN=4]");
		myHCountry.put("3389","[LK=1, IN=1]");
		myHCountry.put("3390","[IN=4]");
		myHCountry.put("3391","[IN=2]");
		myHCountry.put("3392","[IN=2]");
		myHCountry.put("3393","[IN=5]");
		myHCountry.put("3394","[IN=2]");
		myHCountry.put("3395","[IN=6]");
		myHCountry.put("3396","[NG=2, CM=1]");
		myHCountry.put("3397","[FR=1, NG=1]");
		myHCountry.put("3398","[DE=1, NG=1]");
		myHCountry.put("3399","[NG=1, AT=1]");
		myHCountry.put("3400","[US=1, NG=1]");
		myHCountry.put("3401","[CN=2]");
		myHCountry.put("3402","[CN=2]");
		myHCountry.put("3403","[CN=2]");
		myHCountry.put("3404","[CN=3]");
		myHCountry.put("3405","[TZ=1, ET=1]");
		myHCountry.put("3406","[ET=5]");
		myHCountry.put("3407","[ET=1, US=1]");
		myHCountry.put("3408","[ET=2]");
		myHCountry.put("3409","[ET=2]");
		myHCountry.put("3410","[FI=1, ET=1]");
		myHCountry.put("3411","[SA=1, ET=1]");
		myHCountry.put("3412","[ET=2, SA=1]");
		myHCountry.put("3413","[AL=2]");
		myHCountry.put("3414","[AL=2]");
		myHCountry.put("3415","[AL=2]");
		myHCountry.put("3416","[AL=1, NL=1]");
		myHCountry.put("3417","[AL=1, US=1]");
		myHCountry.put("3418","[AL=4]");
		myHCountry.put("3419","[BR=2]");
		myHCountry.put("3420","[BR=1, ES=1]");
		myHCountry.put("3421","[BR=1, AT=1]");
		myHCountry.put("3422","[BR=3]");
		myHCountry.put("3423","[BR=1, UA=1]");
		myHCountry.put("3424","[IR=2]");
		myHCountry.put("3425","[IR=7]");
		myHCountry.put("3426","[IR=6]");
		myHCountry.put("3427","[IR=3, IN=1]");
		myHCountry.put("3428","[IR=4]");
		myHCountry.put("3429","[IR=1, IN=1]");
		myHCountry.put("3430","[IR=2]");
		myHCountry.put("3432","[PE=1]");
		myHCountry.put("3433","[BR=1]");
		myHCountry.put("3434","[KG=3]");
		myHCountry.put("3435","[KG=1, FR=1]");
		myHCountry.put("3436","[MA=1, FR=1]");
		myHCountry.put("3437","[FR=2]");
		myHCountry.put("3438","[FR=2]");
		myHCountry.put("3439","[ET=1, FR=1]");
		myHCountry.put("3440","[GF=1, US=1, FR=1]");
		myHCountry.put("3441","[FR=6]");
		myHCountry.put("3442","[FR=3, FI=1]");
		myHCountry.put("3443","[FR=2]");
		myHCountry.put("3444","[MQ=1, FR=1]");
		myHCountry.put("3445","[ES=1, FR=1]");
		myHCountry.put("3446","[NL=1, FR=1]");
		myHCountry.put("3447","[FR=4]");
		myHCountry.put("3448","[FR=2, IT=1]");
		myHCountry.put("3449","[FR=3]");
		myHCountry.put("3450","[ES=1, FR=1]");
		myHCountry.put("3451","[FR=4]");
		myHCountry.put("3452","[FR=2, IE=1]");
		myHCountry.put("3453","[ET=59, SE=2, FR=1]");
		myHCountry.put("3454","[FR=2]");
		myHCountry.put("3455","[CM=1, FR=1]");
		myHCountry.put("3456","[SA=1, FR=1]");
		myHCountry.put("3457","[FR=1, VE=1]");
		myHCountry.put("3458","[BD=1, FR=1]");
		myHCountry.put("3459","[FR=1, AT=1]");
		myHCountry.put("3460","[TH=1, FR=1]");
		myHCountry.put("3461","[FR=2]");
		myHCountry.put("3462","[CO=1, FR=1]");
		myHCountry.put("3463","[NL=1, FR=1]");
		myHCountry.put("3464","[BD=1, FR=1]");
		myHCountry.put("3465","[FR=2]");
		myHCountry.put("3466","[SE=1, FR=1]");
		myHCountry.put("3467","[CM=1, FR=1]");
		myHCountry.put("3468","[FR=2]");
		myHCountry.put("3469","[ES=1, FR=1]");
		myHCountry.put("3470","[FR=4]");
		myHCountry.put("3471","[FR=4]");
		myHCountry.put("3472","[BR=1, FR=1]");
		myHCountry.put("3473","[FR=2, HT=1]");
		myHCountry.put("3474","[HT=1, FR=1]");
		myHCountry.put("3475","[FR=2, GM=1]");
		myHCountry.put("3476","[US=1, FR=1]");
		myHCountry.put("3477","[FR=2]");
		myHCountry.put("3478","[ES=5, NL=1, FR=1]");
		myHCountry.put("3479","[FR=3]");
		myHCountry.put("3480","[FR=2]");
		myHCountry.put("3481","[FR=1, AT=1]");
		myHCountry.put("3482","[IT=1, FR=1]");
		myHCountry.put("3483","[FR=2, MY=1]");
		myHCountry.put("3484","[MY=1, FR=1]");
		myHCountry.put("3485","[ES=1, FR=1]");
		myHCountry.put("3486","[SE=1, FR=1]");
		myHCountry.put("3487","[FR=1, JP=1]");
		myHCountry.put("3488","[TR=1, FR=1]");
		myHCountry.put("3489","[SE=1, FR=1]");
		myHCountry.put("3490","[BR=1, FR=1]");
		myHCountry.put("3491","[ZA=1, FR=1]");
		myHCountry.put("3492","[US=1, FR=1]");
		myHCountry.put("3493","[AR=1, FR=1]");
		myHCountry.put("3494","[ZA=1, FR=1]");
		myHCountry.put("3495","[BR=1, FR=1]");
		myHCountry.put("3496","[FR=1]");
		myHCountry.put("3497","[IQ=8]");
		myHCountry.put("3498","[GW=1, FR=1]");
		myHCountry.put("3499","[IT=1, FR=1]");
		myHCountry.put("3500","[UG=2]");
		myHCountry.put("3501","[UG=3]");
		myHCountry.put("3502","[UG=1, US=1]");
		myHCountry.put("3503","[UG=1, NL=1]");
		myHCountry.put("3504","[GF=3]");
		myHCountry.put("3505","[GP=2]");
		myHCountry.put("3506","[GF=1]");
		myHCountry.put("3507","[ES=154]");
		myHCountry.put("3508","[ES=137]");
		myHCountry.put("3509","[ES=135]");
		myHCountry.put("3510","[ES=119]");
		myHCountry.put("3511","[ES=116, AR=1]");
		myHCountry.put("3512","[ES=105, NL=2]");
		myHCountry.put("3513","[ES=84]");
		myHCountry.put("3514","[ES=76, US=1]");
		myHCountry.put("3515","[ES=73]");
		myHCountry.put("3516","[ES=70]");
		myHCountry.put("3517","[ES=63]");
		myHCountry.put("3518","[ES=55]");
		myHCountry.put("3519","[ES=49]");
		myHCountry.put("3520","[ES=47, PT=2]");
		myHCountry.put("3521","[ES=45, AR=1]");
		myHCountry.put("3522","[ES=45]");
		myHCountry.put("3523","[ES=44]");
		myHCountry.put("3524","[ES=44]");
		myHCountry.put("3525","[ES=42]");
		myHCountry.put("3526","[ES=42]");
		myHCountry.put("3527","[ES=38]");
		myHCountry.put("3528","[ES=39]");
		myHCountry.put("3529","[ES=37, IT=4, CL=1, FR=1]");
		myHCountry.put("3530","[ES=36]");
		myHCountry.put("3531","[ES=36]");
		myHCountry.put("3532","[ES=36, NL=1, PT=1]");
		myHCountry.put("3533","[ES=35]");
		myHCountry.put("3534","[ES=33]");
		myHCountry.put("3535","[ES=29]");
		myHCountry.put("3536","[ES=27, PT=2]");
		myHCountry.put("3537","[ES=27]");
		myHCountry.put("3538","[ES=26]");
		myHCountry.put("3539","[ES=25]");
		myHCountry.put("3540","[ES=25]");
		myHCountry.put("3541","[ES=26]");
		myHCountry.put("3542","[ES=25]");
		myHCountry.put("3543","[ES=23]");
		myHCountry.put("3544","[ES=23]");
		myHCountry.put("3545","[ES=22]");
		myHCountry.put("3546","[ES=20]");
		myHCountry.put("3547","[ES=20, PT=2, NA=1]");
		myHCountry.put("3548","[ES=19]");
		myHCountry.put("3549","[ES=18, NL=1]");
		myHCountry.put("3550","[ES=18]");
		myHCountry.put("3551","[ES=19, NL=1]");
		myHCountry.put("3552","[ES=22]");
		myHCountry.put("3553","[ES=17]");
		myHCountry.put("3554","[ES=18]");
		myHCountry.put("3555","[ES=17]");
		myHCountry.put("3556","[ES=16, PT=3, NL=2]");
		myHCountry.put("3557","[ES=16, NL=1]");
		myHCountry.put("3558","[ES=16]");
		myHCountry.put("3559","[ES=17]");
		myHCountry.put("3560","[ES=15]");
		myHCountry.put("3561","[ES=15]");
		myHCountry.put("3562","[ES=19, FR=1]");
		myHCountry.put("3563","[ES=15]");
		myHCountry.put("3564","[ES=15]");
		myHCountry.put("3565","[ES=14]");
		myHCountry.put("3566","[ES=13]");
		myHCountry.put("3567","[ES=15]");
		myHCountry.put("3568","[ES=14]");
		myHCountry.put("3569","[ES=13]");
		myHCountry.put("3570","[ES=12]");
		myHCountry.put("3571","[ES=12]");
		myHCountry.put("3572","[ES=13]");
		myHCountry.put("3573","[ES=15, BR=2]");
		myHCountry.put("3574","[ES=11]");
		myHCountry.put("3575","[ES=12]");
		myHCountry.put("3576","[ES=12]");
		myHCountry.put("3577","[ES=10, FR=1]");
		myHCountry.put("3578","[ES=10]");
		myHCountry.put("3579","[ES=10]");
		myHCountry.put("3580","[ES=10]");
		myHCountry.put("3581","[ES=10]");
		myHCountry.put("3582","[ES=9]");
		myHCountry.put("3583","[ES=9]");
		myHCountry.put("3584","[ES=9, BR=1]");
		myHCountry.put("3585","[ES=8]");
		myHCountry.put("3586","[ES=8]");
		myHCountry.put("3587","[ES=8]");
		myHCountry.put("3588","[ES=7]");
		myHCountry.put("3589","[ES=7, AR=1]");
		myHCountry.put("3590","[ES=7]");
		myHCountry.put("3591","[ES=10, BI=4]");
		myHCountry.put("3592","[ES=7]");
		myHCountry.put("3593","[ES=7]");
		myHCountry.put("3594","[ES=7]");
		myHCountry.put("3595","[ES=6]");
		myHCountry.put("3596","[ES=6]");
		myHCountry.put("3597","[ES=7]");
		myHCountry.put("3598","[ES=7]");
		myHCountry.put("3599","[ES=6]");
		myHCountry.put("3600","[ES=6]");
		myHCountry.put("3601","[ES=6]");
		myHCountry.put("3602","[ES=5]");
		myHCountry.put("3603","[ES=5]");
		myHCountry.put("3604","[ES=5]");
		myHCountry.put("3605","[ES=5]");
		myHCountry.put("3606","[ES=5]");
		myHCountry.put("3607","[ES=5]");
		myHCountry.put("3608","[ES=5]");
		myHCountry.put("3609","[ES=5]");
		myHCountry.put("3610","[ES=5]");
		myHCountry.put("3611","[ES=5]");
		myHCountry.put("3612","[ES=5]");
		myHCountry.put("3613","[ES=5]");
		myHCountry.put("3614","[ES=5]");
		myHCountry.put("3615","[ES=5]");
		myHCountry.put("3616","[ES=5]");
		myHCountry.put("3617","[ES=5, GB=1]");
		myHCountry.put("3618","[ES=5]");
		myHCountry.put("3619","[ES=5]");
		myHCountry.put("3620","[ES=5, BE=1]");
		myHCountry.put("3621","[ES=4, FR=1]");
		myHCountry.put("3622","[ES=4]");
		myHCountry.put("3623","[ES=5]");
		myHCountry.put("3624","[ES=4]");
		myHCountry.put("3625","[ES=4]");
		myHCountry.put("3626","[ES=4]");
		myHCountry.put("3627","[ES=5]");
		myHCountry.put("3628","[ES=5]");
		myHCountry.put("3629","[ES=4]");
		myHCountry.put("3630","[ES=7]");
		myHCountry.put("3631","[ES=4]");
		myHCountry.put("3632","[ES=4]");
		myHCountry.put("3633","[ES=4]");
		myHCountry.put("3634","[ES=4]");
		myHCountry.put("3635","[ES=4]");
		myHCountry.put("3636","[ES=3, FR=1]");
		myHCountry.put("3637","[ES=3, PT=2]");
		myHCountry.put("3638","[ES=4]");
		myHCountry.put("3639","[ES=3]");
		myHCountry.put("3640","[ES=3]");
		myHCountry.put("3641","[ES=3]");
		myHCountry.put("3642","[ES=4]");
		myHCountry.put("3643","[ES=3]");
		myHCountry.put("3644","[ES=4]");
		myHCountry.put("3645","[ES=4]");
		myHCountry.put("3646","[ES=3]");
		myHCountry.put("3647","[ES=3]");
		myHCountry.put("3648","[ES=3, AR=1]");
		myHCountry.put("3649","[ES=3]");
		myHCountry.put("3650","[ES=3]");
		myHCountry.put("3651","[ES=3]");
		myHCountry.put("3652","[ES=3]");
		myHCountry.put("3653","[ES=3]");
		myHCountry.put("3654","[ES=3]");
		myHCountry.put("3655","[ES=3]");
		myHCountry.put("3656","[ES=3]");
		myHCountry.put("3657","[ES=3]");
		myHCountry.put("3658","[ES=3]");
		myHCountry.put("3659","[ES=3]");
		myHCountry.put("3660","[ES=3]");
		myHCountry.put("3661","[ES=3]");
		myHCountry.put("3662","[ES=2]");
		myHCountry.put("3663","[ES=2, MX=1]");
		myHCountry.put("3664","[ES=2]");
		myHCountry.put("3665","[ES=3]");
		myHCountry.put("3666","[ES=3, ZA=2]");
		myHCountry.put("3667","[PT=3, ES=2]");
		myHCountry.put("3668","[ES=2]");
		myHCountry.put("3669","[ES=2]");
		myHCountry.put("3670","[ES=2]");
		myHCountry.put("3671","[ES=2]");
		myHCountry.put("3672","[ES=2]");
		myHCountry.put("3673","[ES=2]");
		myHCountry.put("3674","[ES=2]");
		myHCountry.put("3675","[ES=2]");
		myHCountry.put("3676","[ES=2]");
		myHCountry.put("3677","[ES=2]");
		myHCountry.put("3678","[ES=3]");
		myHCountry.put("3679","[ES=2]");
		myHCountry.put("3680","[ES=2]");
		myHCountry.put("3681","[ES=2]");
		myHCountry.put("3682","[ES=2]");
		myHCountry.put("3683","[ES=2]");
		myHCountry.put("3684","[ES=2]");
		myHCountry.put("3685","[ES=2]");
		myHCountry.put("3686","[ES=2]");
		myHCountry.put("3687","[ES=2]");
		myHCountry.put("3688","[ES=3]");
		myHCountry.put("3689","[ES=2]");
		myHCountry.put("3690","[ES=2]");
		myHCountry.put("3691","[ES=2]");
		myHCountry.put("3692","[ES=2]");
		myHCountry.put("3693","[ES=2]");
		myHCountry.put("3694","[ES=2]");
		myHCountry.put("3695","[ES=2]");
		myHCountry.put("3696","[ES=2]");
		myHCountry.put("3697","[ES=2]");
		myHCountry.put("3698","[ES=2]");
		myHCountry.put("3699","[ES=2]");
		myHCountry.put("3700","[ES=2]");
		myHCountry.put("3701","[ES=2]");
		myHCountry.put("3702","[ES=2]");
		myHCountry.put("3703","[ES=2]");
		myHCountry.put("3704","[ES=2]");
		myHCountry.put("3705","[ES=2]");
		myHCountry.put("3706","[ES=2]");
		myHCountry.put("3707","[ES=1, AR=1]");
		myHCountry.put("3708","[IQ=3]");
		myHCountry.put("3709","[ES=1, FR=1]");
		myHCountry.put("3710","[BR=1, ES=1]");
		myHCountry.put("3711","[DK=1, BR=1, ES=1]");
		myHCountry.put("3712","[DE=1, ES=1]");
		myHCountry.put("3713","[DE=2, ES=1]");
		myHCountry.put("3714","[ES=251]");
		myHCountry.put("3715","[ES=17]");
		myHCountry.put("3716","[ES=13]");
		myHCountry.put("3717","[ES=3]");
		myHCountry.put("3718","[ES=2]");
		myHCountry.put("3719","[TD=3]");
		myHCountry.put("3720","[TD=3, NG=3]");
		myHCountry.put("3721","[TD=2]");
		myHCountry.put("3722","[TD=2]");
		myHCountry.put("3723","[TD=2]");
		myHCountry.put("3724","[TD=2]");
		myHCountry.put("3725","[NG=6]");
		myHCountry.put("3726","[NG=4]");
		myHCountry.put("3727","[NG=3]");
		myHCountry.put("3728","[NG=2]");
		myHCountry.put("3729","[NG=2]");
		myHCountry.put("3730","[NG=3]");
		myHCountry.put("3731","[NG=2]");
		myHCountry.put("3732","[NG=2]");
		myHCountry.put("3733","[NG=3]");
		myHCountry.put("3734","[NG=3]");
		myHCountry.put("3735","[NG=2]");
		myHCountry.put("3736","[NG=5]");
		myHCountry.put("3737","[NG=2]");
		myHCountry.put("3738","[NG=2]");
		myHCountry.put("3739","[NG=2]");
		myHCountry.put("3740","[NG=3]");
		myHCountry.put("3741","[NG=3]");
		myHCountry.put("3742","[NG=3, BJ=1]");
		myHCountry.put("3743","[NG=3]");
		myHCountry.put("3744","[UG=2]");
		myHCountry.put("3745","[UG=6]");
		myHCountry.put("3746","[UG=2]");
		myHCountry.put("3747","[ET=32]");
		myHCountry.put("3748","[MZ=28, DE=1, RU=1]");
		myHCountry.put("3749","[ET=9]");
		myHCountry.put("3750","[ET=2]");
		myHCountry.put("3751","[UG=3]");
		myHCountry.put("3752","[UG=3]");
		myHCountry.put("3753","[TZ=5, ZA=1]");
		myHCountry.put("3754","[IT=2, IR=1, FR=1]");
		myHCountry.put("3755","[IT=1, BE=1]");
		myHCountry.put("3756","[ES=2]");
		myHCountry.put("3757","[NG=2]");
		myHCountry.put("3758","[NG=2]");
		myHCountry.put("3759","[BE=1, ML=1]");
		myHCountry.put("3760","[ML=2]");
		myHCountry.put("3761","[BG=2]");
		myHCountry.put("3762","[BG=2]");
		myHCountry.put("3763","[BG=2]");
		myHCountry.put("3764","[BG=4]");
		myHCountry.put("3765","[MM=2]");
		myHCountry.put("3766","[MM=5]");
		myHCountry.put("3767","[MM=1, NL=1]");
		myHCountry.put("3768","[MM=1, US=1]");
		myHCountry.put("3769","[MM=3]");
		myHCountry.put("3770","[MM=1, NL=1]");
		myHCountry.put("3771","[MM=1, IN=1]");
		myHCountry.put("3772","[MM=1, BD=1]");
		myHCountry.put("3773","[MM=1, SD=1]");
		myHCountry.put("3774","[MM=1, MY=1]");
		myHCountry.put("3775","[IT=1, MM=1]");
		myHCountry.put("3776","[IR=1, IQ=1]");
		myHCountry.put("3777","[PE=1, ES=1]");
		myHCountry.put("3778","[PE=5]");
		myHCountry.put("3779","[PE=2]");
		myHCountry.put("3780","[PE=2]");
		myHCountry.put("3781","[RU=2, BE=1]");
		myHCountry.put("3782","[RU=1, US=1]");
		myHCountry.put("3783","[VN=1, RU=1]");
		myHCountry.put("3784","[BR=1, RU=1]");
		myHCountry.put("3785","[DE=1, RU=1]");
		myHCountry.put("3786","[RU=2]");
		myHCountry.put("3787","[RU=2]");
		myHCountry.put("3788","[KZ=1, RU=1]");
		myHCountry.put("3789","[IQ=3]");
		myHCountry.put("3790","[IQ=4]");
		myHCountry.put("3791","[IQ=4]");
		myHCountry.put("3792","[IQ=2, IN=1]");
		myHCountry.put("3793","[IQ=1, IN=1]");
		myHCountry.put("3794","[IQ=1, IN=1]");
		myHCountry.put("3795","[IQ=1, BR=1]");
		myHCountry.put("3796","[IQ=1, PK=1]");
		myHCountry.put("3797","[IQ=1, MX=1]");
		myHCountry.put("3798","[IQ=2]");
		myHCountry.put("3799","[IQ=1, BR=1]");
		myHCountry.put("3800","[IQ=2]");
		myHCountry.put("3801","[IN=2]");
		myHCountry.put("3802","[IN=1, US=1]");
		myHCountry.put("3803","[OM=3, TZ=1]");
		myHCountry.put("3804","[OM=26]");
		myHCountry.put("3805","[OM=2]");
		myHCountry.put("3806","[OM=2]");
		myHCountry.put("3807","[OM=2, PK=1]");
		myHCountry.put("3808","[OM=2]");
		myHCountry.put("3809","[BD=1, OM=1]");
		myHCountry.put("3810","[OM=2]");
		myHCountry.put("3811","[IN=1, OM=1]");
		myHCountry.put("3812","[OM=3]");
		myHCountry.put("3813","[BB=1, OM=1]");
		myHCountry.put("3814","[OM=2]");
		myHCountry.put("3815","[US=1, OM=1]");
		myHCountry.put("3816","[OM=2]");
		myHCountry.put("3817","[OM=2, GB=1]");
		myHCountry.put("3818","[OM=4]");
		myHCountry.put("3819","[OM=3, MG=1]");
		myHCountry.put("3820","[OM=3, SA=1]");
		myHCountry.put("3821","[PK=1, OM=1]");
		myHCountry.put("3822","[OM=2]");
		myHCountry.put("3823","[NL=1, OM=1]");
		myHCountry.put("3824","[IN=1, OM=1]");
		myHCountry.put("3825","[MY=1, OM=1]");
		myHCountry.put("3826","[IN=1, OM=1]");
		myHCountry.put("3827","[OM=2]");
		myHCountry.put("3828","[BE=1, OM=1]");
		myHCountry.put("3829","[OM=2, IT=1]");
		myHCountry.put("3830","[OM=2]");
		myHCountry.put("3831","[OM=2]");
		myHCountry.put("3832","[OM=2]");
		myHCountry.put("3833","[OM=2]");
		myHCountry.put("3834","[MM=1, OM=1]");
		myHCountry.put("3835","[OM=2]");
		myHCountry.put("3836","[BR=1, OM=1]");
		myHCountry.put("3837","[PE=1, OM=1]");
		myHCountry.put("3838","[IT=1, OM=1]");
		myHCountry.put("3839","[OM=2]");
		myHCountry.put("3840","[ZA=1, OM=1]");
		myHCountry.put("3841","[OM=2]");
		myHCountry.put("3842","[TW=1, OM=1]");
		myHCountry.put("3843","[IT=1, OM=1]");
		myHCountry.put("3844","[IT=3]");
		myHCountry.put("3845","[IT=14]");
		myHCountry.put("3846","[IT=2, TR=1]");
		myHCountry.put("3847","[DZ=1, NL=1]");
		myHCountry.put("3848","[DE=2]");
		myHCountry.put("3849","[DE=2]");
		myHCountry.put("3850","[DE=2]");
		myHCountry.put("3851","[DE=2]");
		myHCountry.put("3852","[DE=2]");
		myHCountry.put("3853","[AR=2]");
		myHCountry.put("3854","[BR=2]");
		myHCountry.put("3855","[MX=2]");
		myHCountry.put("3856","[GB=1, MX=1]");
		myHCountry.put("3857","[MX=1, US=1]");
		myHCountry.put("3858","[CA=10]");
		myHCountry.put("3859","[AU=2]");
		myHCountry.put("3860","[AU=2]");
		myHCountry.put("3861","[IR=8]");
		myHCountry.put("3862","[IT=1, PT=1]");

		
		if(!myHCountry.containsKey(sit)){
			return "ND";
		}
		else{
			return myHCountry.get(sit);
		}
	}
	
	/**
	 * Method allowing to get Country distribution from OrphanSpol (contained in SITVIT2)
	 * @param spoligotype
	 * @return
	 * @throws Exception
	 */
	public static String getCountryOrphan(String spoligotype) throws Exception{
		HashMap<String,String> myHCountry = new HashMap<String,String>();
		
		myHCountry.put("000000007737071","[CM=1]");
		myHCountry.put("000000377737671","[SN=1]");
		myHCountry.put("000001740005471","[SD=1]");
		myHCountry.put("000020034003671","[ID=1]");
		myHCountry.put("000037777763661","[ES=1]");
		myHCountry.put("000373767501471","[MY=1]");
		myHCountry.put("030771462603471","[SA=1]");
		myHCountry.put("070377777767671","[GW=1]");
		myHCountry.put("170077770007071","[CM=1]");
		myHCountry.put("400007036000061","[TZ=1]");
		myHCountry.put("400007777403071","[BB=1]");
		myHCountry.put("414203430400061","[IT=1]");
		myHCountry.put("500141016564671","[NL=1]");
		myHCountry.put("500740007774671","[US=1]");
		myHCountry.put("534357674732671","[ET=1]");
		myHCountry.put("570040037547071","[FR=1]");
		myHCountry.put("570777777600071","[GW=1]");
		myHCountry.put("600000000003271","[IT=1]");
		myHCountry.put("600001740005471","[SD=1]");
		myHCountry.put("600777740003671","[LY=1]");
		myHCountry.put("674000002000071","[TH=1]");
		myHCountry.put("674063777777660","[GB=1]");
		myHCountry.put("700000000037641","[IT=1]");
		myHCountry.put("700036777400071","[US=1]");
		myHCountry.put("700076750000071","[US=1]");
		myHCountry.put("700076760000071","[LK=1]");
		myHCountry.put("700076770000071","[US=1]");
		myHCountry.put("700077777767671","[FR=1]");
		myHCountry.put("700741407774671","[GB=1]");
		myHCountry.put("740000377777671","[GM=1]");
		myHCountry.put("740037777777671","[GW=1]");
		myHCountry.put("740657600000071","[ID=1]");
		myHCountry.put("740737777777471","[SN=1]");
		myHCountry.put("740757600000061","[ID=1]");
		myHCountry.put("750327405000061","[BE=1]");
		myHCountry.put("760000077777671","[GW=1]");
		myHCountry.put("760001400000071","[ES=1]");
		myHCountry.put("760377377743671","[GW=1]");
		myHCountry.put("760377757777671","[GW=1]");
		myHCountry.put("760737004177671","[CI=1]");
		myHCountry.put("770000000003671","[US=1]");
		myHCountry.put("770000000400071","[MM=1]");
		myHCountry.put("770000777400071","[TR=1]");
		myHCountry.put("770003600007071","[JM=1]");
		myHCountry.put("770003604377671","[TN=1]");
		myHCountry.put("770003605777071","[FR=1]");
		myHCountry.put("770003627777071","[NG=1]");
		myHCountry.put("770003777767671","[GW=1]");
		myHCountry.put("770007607777071","[FR=1]");
		myHCountry.put("770047606377071","[BE=1]");
		myHCountry.put("770064004007071","[FR=1]");
		myHCountry.put("770077032007071","[IT=1]");
		myHCountry.put("770357777777660","[GB=1]");
		myHCountry.put("770373777767671","[GW=1]");
		myHCountry.put("770377777767671","[ES=1]");
		myHCountry.put("770737777617471","[NL=1]");
		myHCountry.put("770777757777471","[CO=1]");
		myHCountry.put("770777776000071","[GM=1]");
		myHCountry.put("770777777777071","[TN=1]");
		myHCountry.put("770777777777471","[GB=1]");
		myHCountry.put("774000067777071","[FR=1]");
		myHCountry.put("774003607777071","[IT=1]");
		myHCountry.put("774017607777071","[TT=1]");
		myHCountry.put("774023607777071","[BJ=1]");
		myHCountry.put("774034607777071","[IT=1]");
		myHCountry.put("774067760000071","[CM=1]");
		myHCountry.put("774067772000071","[FR=1]");
		myHCountry.put("774067776001671","[TN=1]");
		myHCountry.put("774077600000071","[IT=1]");
		myHCountry.put("774077606756071","[BE=1]");
		myHCountry.put("774077700000071","[IT=1]");
		myHCountry.put("774377700003071","[SD=1]");
		myHCountry.put("774377777777671","[FR=1]");
		myHCountry.put("774777777777671","[GM=1]");
		myHCountry.put("170777703017671","[IT=1]");
		myHCountry.put("170777777777671","[GM=1]");
		myHCountry.put("330777777767671","[US=1]");
		myHCountry.put("370776674672671","[EG=1]");
		myHCountry.put("470777777777671","[GW=1]");
		myHCountry.put("530777677763671","[NL=1]");
		myHCountry.put("570737777777670","[US=1]");
		myHCountry.put("570773777777671","[TN=1]");
		myHCountry.put("570777777777271","[GW=1]");
		myHCountry.put("670777777777671","[GW=1]");
		myHCountry.put("730777470001671","[GW=1]");
		myHCountry.put("730777777723670","[NL=1]");
		myHCountry.put("770537777777671","[US=1]");
		myHCountry.put("770677177777671","[IT=1]");
		myHCountry.put("770703107767671","[GW=1]");
		myHCountry.put("770717777777671","[NL=1]");
		myHCountry.put("770737404177671","[GM=1]");
		myHCountry.put("770737777774671","[GM=1]");
		myHCountry.put("770740777777671","[US=1]");
		myHCountry.put("770760007767671","[TN=1]");
		myHCountry.put("770761777777671","[GW=1]");
		myHCountry.put("770767607767651","[FR=1]");
		myHCountry.put("770767707777671","[FR=1]");
		myHCountry.put("770767777403671","[GW=1]");
		myHCountry.put("770770777777671","[US=1]");
		myHCountry.put("770777004177271","[US=1]");
		myHCountry.put("770777077777671","[US=1]");
		myHCountry.put("770777407777671","[TN=1]");
		myHCountry.put("770777477777671","[GW=1]");
		myHCountry.put("770777577377671","[US=1]");
		myHCountry.put("770777607777651","[NL=1]");
		myHCountry.put("770777637743671","[GW=1]");
		myHCountry.put("770777704377671","[GM=1]");
		myHCountry.put("770777707774671","[CO=1]");
		myHCountry.put("770777707777271","[FR=1]");
		myHCountry.put("770777707777660","[NL=1]");
		myHCountry.put("770777737777671","[GM=1]");
		myHCountry.put("770777774777671","[GW=1]");
		myHCountry.put("770777775767671","[GW=1]");
		myHCountry.put("770777777177671","[DK=1]");
		myHCountry.put("770777777403671","[GW=1]");
		myHCountry.put("770777777477671","[FR=1]");
		myHCountry.put("770777777567671","[DE=1]");
		myHCountry.put("770777777700670","[US=1]");
		myHCountry.put("770777777737671","[TN=1]");
		myHCountry.put("770777777743671","[GW=1]");
		myHCountry.put("770777777747671","[US=1]");
		myHCountry.put("770777777776271","[GM=1]");
		myHCountry.put("770777777777271","[GM=1]");
		myHCountry.put("770777777777651","[GW=1]");
		myHCountry.put("770777777777661","[US=1]");
		myHCountry.put("174067606767071","[CM=1]");
		myHCountry.put("474077607177071","[CI=1]");
		myHCountry.put("574066607777071","[CM=1]");
		myHCountry.put("574067607737071","[CM=1]");
		myHCountry.put("574077607177071","[US=1]");
		myHCountry.put("734077607777071","[NG=1]");
		myHCountry.put("764077607777071","[IT=1]");
		myHCountry.put("774057607777071","[US=1]");
		myHCountry.put("774077607761071","[NG=1]");
		myHCountry.put("574077777777071","[DE=1]");
		myHCountry.put("774042077777071","[BJ=1]");
		myHCountry.put("774066006037071","[FR=1]");
		myHCountry.put("774067407777071","[FR=1]");
		myHCountry.put("774067600001071","[CM=1]");
		myHCountry.put("774067700001071","[US=1]");
		myHCountry.put("774067772001071","[CM=1]");
		myHCountry.put("774067777777071","[NG=1]");
		myHCountry.put("774074006035071","[GW=1]");
		myHCountry.put("774074006037071","[FR=1]");
		myHCountry.put("774076002003071","[NG=1]");
		myHCountry.put("774076005777071","[IT=1]");
		myHCountry.put("774076006037071","[FR=1]");
		myHCountry.put("774077077777071","[GB=1]");
		myHCountry.put("774077602001071","[FR=1]");
		myHCountry.put("774077670007071","[CM=1]");
		myHCountry.put("774077752001071","[NL=1]");
		myHCountry.put("774077770007071","[NG=1]");
		myHCountry.put("774077772001071","[FR=1]");
		myHCountry.put("774077772005071","[CM=1]");
		myHCountry.put("774077776001071","[ES=1]");
		myHCountry.put("774077777037071","[NG=1]");
		myHCountry.put("774077777767071","[NG=1]");
		myHCountry.put("774077777775061","[SL=1]");
		myHCountry.put("000000000000001","[IT=1]");
		myHCountry.put("000000000000020","[MY=1]");
		myHCountry.put("000000000000061","[PK=1]");
		myHCountry.put("000000000000071","[CM=1]");
		myHCountry.put("000000000001071","[FR=1]");
		myHCountry.put("000000000001200","[MY=1]");
		myHCountry.put("000000000002371","[CN=1]");
		myHCountry.put("000000000003001","[US=1]");
		myHCountry.put("000000000003011","[CN=1]");
		myHCountry.put("000000000003020","[AT=1]");
		myHCountry.put("000000000003151","[US=1]");
		myHCountry.put("000000000003331","[CN=1]");
		myHCountry.put("000000000003531","[JP=1]");
		myHCountry.put("000000000003601","[RU=1]");
		myHCountry.put("000000000003630","[CN=1]");
		myHCountry.put("000001745777600","[FR=1]");
		myHCountry.put("000004000000000","[MY=1]");
		myHCountry.put("000007374000000","[ES=1]");
		myHCountry.put("000037740000000","[PK=1]");
		myHCountry.put("000046037160000","[ES=1]");
		myHCountry.put("000177600000000","[BR=1]");
		myHCountry.put("000377600000000","[BR=1]");
		myHCountry.put("000773777773600","[ET=1]");
		myHCountry.put("000773777777600","[FR=1]");
		myHCountry.put("004061427777600","[MX=1]");
		myHCountry.put("004267637746600","[MX=1]");
		myHCountry.put("006773777777600","[ES=1]");
		myHCountry.put("016703677746600","[MX=1]");
		myHCountry.put("016761777777600","[IT=1]");
		myHCountry.put("024041767542600","[MX=1]");
		myHCountry.put("024061637777600","[MX=1]");
		myHCountry.put("024061777777600","[MX=1]");
		myHCountry.put("042773777777600","[AR=1]");
		myHCountry.put("044263777746600","[MX=1]");
		myHCountry.put("064061767776600","[MX=1]");
		myHCountry.put("064063407777600","[MX=1]");
		myHCountry.put("064063767777600","[MX=1]");
		myHCountry.put("064063777767600","[MX=1]");
		myHCountry.put("064263777746600","[MX=1]");
		myHCountry.put("066263777777600","[MX=1]");
		myHCountry.put("076753777777600","[ES=1]");
		myHCountry.put("076763677777600","[MX=1]");
		myHCountry.put("200003777607600","[ES=1]");
		myHCountry.put("244004000000000","[GR=1]");
		myHCountry.put("256423777402600","[TZ=1]");
		myHCountry.put("266063777777600","[MX=1]");
		myHCountry.put("266073777777600","[US=1]");
		myHCountry.put("270000036763600","[NL=1]");
		myHCountry.put("276763777777600","[TN=1]");
		myHCountry.put("400000017777600","[FR=1]");
		myHCountry.put("400000074037600","[ES=1]");
		myHCountry.put("400000777777600","[MG=1]");
		myHCountry.put("400003436742200","[IT=1]");
		myHCountry.put("400374000000000","[TH=1]");
		myHCountry.put("406773777773600","[FR=1]");
		myHCountry.put("414203623442000","[BR=1]");
		myHCountry.put("414343436642200","[IT=1]");
		myHCountry.put("414343634646200","[BR=1]");
		myHCountry.put("416773677563600","[US=1]");
		myHCountry.put("416773677777200","[ES=1]");
		myHCountry.put("416773777000600","[US=1]");
		myHCountry.put("416773777777200","[BR=1]");
		myHCountry.put("424043636746600","[NZ=1]");
		myHCountry.put("424061777777600","[MX=1]");
		myHCountry.put("430001077777600","[NL=1]");
		myHCountry.put("434161777077600","[MX=1]");
		myHCountry.put("434763676777600","[ES=1]");
		myHCountry.put("436761637777600","[MX=1]");
		myHCountry.put("436773674777600","[ES=1]");
		myHCountry.put("436773677174600","[ES=1]");
		myHCountry.put("436773677477600","[ES=1]");
		myHCountry.put("440003767777600","[MX=1]");
		myHCountry.put("440003777677600","[MX=1]");
		myHCountry.put("442343677776200","[BR=1]");
		myHCountry.put("454363637646200","[BR=1]");
		myHCountry.put("454573777777600","[GB=1]");
		myHCountry.put("456763637743600","[DE=1]");
		myHCountry.put("464040777777600","[FR=1]");
		myHCountry.put("464043676742200","[IE=1]");
		myHCountry.put("464043677777600","[NZ=1]");
		myHCountry.put("464043777742200","[GB=1]");
		myHCountry.put("464063777567600","[MX=1]");
		myHCountry.put("464063777577600","[GB=1]");
		myHCountry.put("464063777700600","[MX=1]");
		myHCountry.put("464063777774600","[IE=1]");
		myHCountry.put("464072177777600","[US=1]");
		myHCountry.put("464743677767600","[NL=1]");
		myHCountry.put("472733777777600","[DE=1]");
		myHCountry.put("472763677773600","[ES=1]");
		myHCountry.put("474053777777600","[GB=1]");
		myHCountry.put("474063777777600","[GB=1]");
		myHCountry.put("474773674177600","[ES=1]");
		myHCountry.put("476173677777200","[ES=1]");
		myHCountry.put("476323476777600","[ET=1]");
		myHCountry.put("476343777773600","[DK=1]");
		myHCountry.put("476573777777600","[ES=1]");
		myHCountry.put("476743677777600","[ES=1]");
		myHCountry.put("476743777777600","[IT=1]");
		myHCountry.put("476763677742600","[DE=1]");
		myHCountry.put("476773674177600","[ES=1]");
		myHCountry.put("476773676037200","[ES=1]");
		myHCountry.put("476773677777400","[ZA=1]");
		myHCountry.put("476773757747600","[DE=1]");
		myHCountry.put("476773777177600","[DE=1]");
		myHCountry.put("476773777377200","[IR=1]");
		myHCountry.put("476773777743600","[FR=1]");
		myHCountry.put("600000004037600","[IT=1]");
		myHCountry.put("600000017703600","[AR=1]");
		myHCountry.put("600000070000000","[NL=1]");
		myHCountry.put("600017777777600","[DK=1]");
		myHCountry.put("600073774037600","[IE=1]");
		myHCountry.put("600073777777600","[FR=1]");
		myHCountry.put("602363777777600","[UG=1]");
		myHCountry.put("602421777777400","[ES=1]");
		myHCountry.put("602700361000200","[ET=1]");
		myHCountry.put("602702037777600","[UG=1]");
		myHCountry.put("602743037776600","[UG=1]");
		myHCountry.put("602743777777600","[UG=1]");
		myHCountry.put("604073777777600","[BE=1]");
		myHCountry.put("606763677777600","[ES=1]");
		myHCountry.put("610373777777600","[MG=1]");
		myHCountry.put("614001777677600","[CM=1]");
		myHCountry.put("614003777777600","[IT=1]");
		myHCountry.put("614023777777600","[IT=1]");
		myHCountry.put("614073607777600","[FR=1]");
		myHCountry.put("614073776037600","[FR=1]");
		myHCountry.put("614073777737600","[FR=1]");
		myHCountry.put("614173776003200","[FR=1]");
		myHCountry.put("614341777777600","[MG=1]");
		myHCountry.put("614363677777600","[MG=1]");
		myHCountry.put("614363777761600","[MG=1]");
		myHCountry.put("614363777767600","[MG=1]");
		myHCountry.put("614363777776600","[BR=1]");
		myHCountry.put("614363777777200","[MG=1]");
		myHCountry.put("614373777737600","[MG=1]");
		myHCountry.put("614373777774600","[MG=1]");
		myHCountry.put("614772777777600","[FR=1]");
		myHCountry.put("616000006777600","[PT=1]");
		myHCountry.put("616163777776600","[TN=1]");
		myHCountry.put("616173777777600","[IT=1]");
		myHCountry.put("616600002017600","[DZ=1]");
		myHCountry.put("616743777763600","[ES=1]");
		myHCountry.put("616763677777600","[ES=1]");
		myHCountry.put("616763774377600","[ES=1]");
		myHCountry.put("616763777777200","[TN=1]");
		myHCountry.put("616771777777600","[FR=1]");
		myHCountry.put("616777617777600","[DK=1]");
		myHCountry.put("624033777777600","[GB=1]");
		myHCountry.put("634553777777600","[ES=1]");
		myHCountry.put("636723777777600","[ES=1]");
		myHCountry.put("636763677777600","[BR=1]");
		myHCountry.put("640003737777600","[GB=1]");
		myHCountry.put("640003777757600","[TN=1]");
		myHCountry.put("640023777775600","[MX=1]");
		myHCountry.put("640023777777600","[MX=1]");
		myHCountry.put("640033377777600","[MX=1]");
		myHCountry.put("640033777776600","[MX=1]");
		myHCountry.put("640033777777600","[MX=1]");
		myHCountry.put("640073774037600","[IE=1]");
		myHCountry.put("640203777777600","[MX=1]");
		myHCountry.put("640773677777600","[ES=1]");
		myHCountry.put("642207777777600","[MX=1]");
		myHCountry.put("642501777177600","[BR=1]");
		myHCountry.put("644043777777600","[MX=1]");
		myHCountry.put("644063777700600","[MX=1]");
		myHCountry.put("644063777777600","[MX=1]");
		myHCountry.put("644073777677600","[AR=1]");
		myHCountry.put("646763777777600","[DK=1]");
		myHCountry.put("654343677776600","[BR=1]");
		myHCountry.put("654573777767600","[NL=1]");
		myHCountry.put("654573777777600","[AR=1]");
		myHCountry.put("656343776677600","[NG=1]");
		myHCountry.put("656343777677600","[NG=1]");
		myHCountry.put("656563377777600","[GP=1]");
		myHCountry.put("660033777777600","[AR=1]");
		myHCountry.put("660073777777600","[DE=1]");
		myHCountry.put("662043777776200","[BR=1]");
		myHCountry.put("662763777417600","[FR=1]");
		myHCountry.put("662763777777600","[IT=1]");
		myHCountry.put("664000177777600","[AR=1]");
		myHCountry.put("664000377777600","[IE=1]");
		myHCountry.put("664000777777600","[BR=1]");
		myHCountry.put("664033774037600","[GB=1]");
		myHCountry.put("664041777777600","[IE=1]");
		myHCountry.put("664043774037600","[IE=1]");
		myHCountry.put("664053777775600","[IE=1]");
		myHCountry.put("664060767777600","[AR=1]");
		myHCountry.put("664061777777600","[BR=1]");
		myHCountry.put("664063077777600","[GB=1]");
		myHCountry.put("664063377777600","[GB=1]");
		myHCountry.put("664063774037600","[GB=1]");
		myHCountry.put("664063777747600","[ES=1]");
		myHCountry.put("664063777767600","[GB=1]");
		myHCountry.put("664063777775600","[IE=1]");
		myHCountry.put("664063777777200","[IE=1]");
		myHCountry.put("664063777777400","[IE=1]");
		myHCountry.put("664077777777200","[GB=1]");
		myHCountry.put("664077777777600","[GB=1]");
		myHCountry.put("664273777776600","[MX=1]");
		myHCountry.put("666073677777600","[ES=1]");
		myHCountry.put("666743677777600","[NL=1]");
		myHCountry.put("666763777777600","[IT=1]");
		myHCountry.put("666770777677600","[ML=1]");
		myHCountry.put("670041777767600","[NL=1]");
		myHCountry.put("670773677777600","[BR=1]");
		myHCountry.put("672763777177600","[TN=1]");
		myHCountry.put("672763777377600","[ES=1]");
		myHCountry.put("674000674177600","[ES=1]");
		myHCountry.put("674003674037600","[ES=1]");
		myHCountry.put("674063777777600","[GB=1]");
		myHCountry.put("674173677077600","[ES=1]");
		myHCountry.put("674173677777600","[ES=1]");
		myHCountry.put("674243676763600","[ES=1]");
		myHCountry.put("674363777776000","[MX=1]");
		myHCountry.put("674373777677600","[NG=1]");
		myHCountry.put("674573777743600","[NL=1]");
		myHCountry.put("674753677777600","[BR=1]");
		myHCountry.put("674763777077600","[MX=1]");
		myHCountry.put("674763777777600","[IT=1]");
		myHCountry.put("674773675777200","[AR=1]");
		myHCountry.put("674773677777600","[ES=1]");
		myHCountry.put("674773777777400","[ES=1]");
		myHCountry.put("674773777777600","[IT=1]");
		myHCountry.put("676000757777600","[TN=1]");
		myHCountry.put("676000777777600","[DK=1]");
		myHCountry.put("676033677777200","[FR=1]");
		myHCountry.put("676073777767600","[FR=1]");
		myHCountry.put("676143677477600","[ES=1]");
		myHCountry.put("676277777700600","[MX=1]");
		myHCountry.put("676303777777600","[IT=1]");
		myHCountry.put("676341757777600","[IR=1]");
		myHCountry.put("676343676037600","[BR=1]");
		myHCountry.put("676343777776600","[BR=1]");
		myHCountry.put("676353677777600","[AR=1]");
		myHCountry.put("676363577776600","[BR=1]");
		myHCountry.put("676373674177600","[ES=1]");
		myHCountry.put("676373777777600","[FR=1]");
		myHCountry.put("676400377777600","[AR=1]");
		myHCountry.put("676463777777600","[IE=1]");
		myHCountry.put("676537777777600","[US=1]");
		myHCountry.put("676562077777600","[ES=1]");
		myHCountry.put("676563777776600","[MX=1]");
		myHCountry.put("676600077777600","[FR=1]");
		myHCountry.put("676600377777600","[FR=1]");
		myHCountry.put("676603777777600","[IT=1]");
		myHCountry.put("676663677777600","[ES=1]");
		myHCountry.put("676663777777600","[DK=1]");
		myHCountry.put("676700677777600","[ES=1]");
		myHCountry.put("676701777767600","[FR=1]");
		myHCountry.put("676703777777600","[DE=1]");
		myHCountry.put("676723777777600","[ES=1]");
		myHCountry.put("676741002037600","[ES=1]");
		myHCountry.put("676741077757600","[ES=1]");
		myHCountry.put("676741377756600","[ES=1]");
		myHCountry.put("676741674177600","[ES=1]");
		myHCountry.put("676741777777600","[FR=1]");
		myHCountry.put("676743377757600","[ES=1]");
		myHCountry.put("676743676177600","[NL=1]");
		myHCountry.put("676743776677600","[NG=1]");
		myHCountry.put("676760077775600","[FR=1]");
		myHCountry.put("676760747777200","[BR=1]");
		myHCountry.put("676761037777600","[SE=1]");
		myHCountry.put("676762037777600","[ES=1]");
		myHCountry.put("676763277777600","[BR=1]");
		myHCountry.put("676763477777600","[ES=1]");
		myHCountry.put("676763675777200","[ES=1]");
		myHCountry.put("676763677757600","[ES=1]");
		myHCountry.put("676763677775200","[ES=1]");
		myHCountry.put("676763677775600","[ES=1]");
		myHCountry.put("676763776437600","[NG=1]");
		myHCountry.put("676763776677600","[NG=1]");
		myHCountry.put("676763777743600","[TN=1]");
		myHCountry.put("676763777774600","[DK=1]");
		myHCountry.put("676763777777400","[SA=1]");
		myHCountry.put("676767777777600","[DK=1]");
		myHCountry.put("676770777777600","[SE=1]");
		myHCountry.put("676771607177600","[SE=1]");
		myHCountry.put("676771757777600","[BR=1]");
		myHCountry.put("676771777774600","[NL=1]");
		myHCountry.put("676771777777600","[SE=1]");
		myHCountry.put("676776777777600","[SA=1]");
		myHCountry.put("676777673777600","[BE=1]");
		myHCountry.put("676777677777200","[NL=1]");
		myHCountry.put("676777757777600","[GB=1]");
		myHCountry.put("262773777756600","[UG=1]");
		myHCountry.put("272773777777600","[ES=1]");
		myHCountry.put("276573737777600","[FR=1]");
		myHCountry.put("602573777777600","[TZ=1]");
		myHCountry.put("602673677777600","[ES=1]");
		myHCountry.put("602772037777600","[UG=1]");
		myHCountry.put("602773677777200","[ES=1]");
		myHCountry.put("602773707777600","[ES=1]");
		myHCountry.put("602773774777600","[ET=1]");
		myHCountry.put("602773776000200","[FR=1]");
		myHCountry.put("602773777753600","[ET=1]");
		myHCountry.put("602773777773200","[IT=1]");
		myHCountry.put("606673777777600","[FR=1]");
		myHCountry.put("606773674177600","[ES=1]");
		myHCountry.put("606773777775600","[FR=1]");
		myHCountry.put("612773677777600","[ES=1]");
		myHCountry.put("616673777777600","[FR=1]");
		myHCountry.put("616772776377600","[FR=1]");
		myHCountry.put("616772777677600","[BE=1]");
		myHCountry.put("616773037777600","[IT=1]");
		myHCountry.put("616773402000000","[DZ=1]");
		myHCountry.put("616773677677600","[ES=1]");
		myHCountry.put("616773774777600","[GB=1]");
		myHCountry.put("616773776037600","[ES=1]");
		myHCountry.put("616773776163600","[FR=1]");
		myHCountry.put("616773776775600","[DZ=1]");
		myHCountry.put("616773777063600","[FR=1]");
		myHCountry.put("616773777177200","[DE=1]");
		myHCountry.put("616773777437200","[FR=1]");
		myHCountry.put("616773777570600","[FR=1]");
		myHCountry.put("616773777771600","[NL=1]");
		myHCountry.put("616773777777000","[ES=1]");
		myHCountry.put("636573637777600","[ES=1]");
		myHCountry.put("636573677777600","[ES=1]");
		myHCountry.put("636733777757600","[ES=1]");
		myHCountry.put("636773377757600","[ES=1]");
		myHCountry.put("636773777743600","[FR=1]");
		myHCountry.put("642773677777200","[ES=1]");
		myHCountry.put("642773677777600","[BE=1]");
		myHCountry.put("646773777667600","[ES=1]");
		myHCountry.put("656473637777600","[FR=1]");
		myHCountry.put("656473777743600","[MX=1]");
		myHCountry.put("656573237777200","[AR=1]");
		myHCountry.put("656573277777600","[US=1]");
		myHCountry.put("656772777777600","[BR=1]");
		myHCountry.put("656773674177600","[ES=1]");
		myHCountry.put("656773677777200","[ES=1]");
		myHCountry.put("656773677777600","[ES=1]");
		myHCountry.put("656773777677600","[TD=1]");
		myHCountry.put("656773777767600","[ES=1]");
		myHCountry.put("662773777477600","[IT=1]");
		myHCountry.put("666673677777600","[ES=1]");
		myHCountry.put("666772057777200","[EG=1]");
		myHCountry.put("666772377677600","[ML=1]");
		myHCountry.put("666773674177600","[ES=1]");
		myHCountry.put("666773677737600","[NL=1]");
		myHCountry.put("666773777643600","[BE=1]");
		myHCountry.put("666773777777600","[ES=1]");
		myHCountry.put("672733777777600","[ES=1]");
		myHCountry.put("672773677777200","[ES=1]");
		myHCountry.put("672773776377600","[FR=1]");
		myHCountry.put("672773777771600","[ES=1]");
		myHCountry.put("676413677777200","[ES=1]");
		myHCountry.put("676413777777600","[ES=1]");
		myHCountry.put("676473737777600","[MX=1]");
		myHCountry.put("676473776003600","[IE=1]");
		myHCountry.put("676473777757600","[IE=1]");
		myHCountry.put("676533777777600","[ES=1]");
		myHCountry.put("676573677577600","[ES=1]");
		myHCountry.put("676573774007600","[ES=1]");
		myHCountry.put("676573777577600","[AR=1]");
		myHCountry.put("676573777740600","[ES=1]");
		myHCountry.put("676573777743600","[BR=1]");
		myHCountry.put("676573777776200","[NZ=1]");
		myHCountry.put("676573777776600","[GB=1]");
		myHCountry.put("676612677777600","[ES=1]");
		myHCountry.put("676613657777600","[ES=1]");
		myHCountry.put("676633677777200","[DE=1]");
		myHCountry.put("676633777777000","[IT=1]");
		myHCountry.put("676733617747600","[ES=1]");
		myHCountry.put("676733776003600","[FR=1]");
		myHCountry.put("676733777776600","[ZA=1]");
		myHCountry.put("676733777777000","[ES=1]");
		myHCountry.put("676773000017600","[FR=1]");
		myHCountry.put("676773002017600","[TN=1]");
		myHCountry.put("676773357657600","[CM=1]");
		myHCountry.put("676773402017600","[FR=1]");
		myHCountry.put("676773406177600","[ES=1]");
		myHCountry.put("676773474177600","[ES=1]");
		myHCountry.put("676773617757600","[ES=1]");
		myHCountry.put("676773617777600","[FR=1]");
		myHCountry.put("676773657637600","[ES=1]");
		myHCountry.put("676773657747200","[ES=1]");
		myHCountry.put("676773657774600","[ES=1]");
		myHCountry.put("676773672000000","[DE=1]");
		myHCountry.put("676773674000200","[ES=1]");
		myHCountry.put("676773674037200","[BR=1]");
		myHCountry.put("676773674037400","[AR=1]");
		myHCountry.put("676773674143600","[ES=1]");
		myHCountry.put("676773674177400","[ES=1]");
		myHCountry.put("676773675767400","[DE=1]");
		myHCountry.put("676773677177600","[BR=1]");
		myHCountry.put("676773677407200","[ES=1]");
		myHCountry.put("676773677577200","[ES=1]");
		myHCountry.put("676773677600600","[BR=1]");
		myHCountry.put("676773677637600","[ES=1]");
		myHCountry.put("676773677677200","[ES=1]");
		myHCountry.put("676773677770600","[ES=1]");
		myHCountry.put("676773677774200","[ES=1]");
		myHCountry.put("676773677776200","[ES=1]");
		myHCountry.put("676773707777600","[]");
		myHCountry.put("676773757767600","[FR=1]");
		myHCountry.put("676773772000600","[BE=1]");
		myHCountry.put("676773774077600","[ES=1]");
		myHCountry.put("676773774377600","[BR=1]");
		myHCountry.put("676773775777200","[BR=1]");
		myHCountry.put("676773776637600","[NG=1]");
		myHCountry.put("676773777045600","[FR=1]");
		myHCountry.put("676773777437600","[FR=1]");
		myHCountry.put("676773777477600","[ES=1]");
		myHCountry.put("676773777637200","[TD=1]");
		myHCountry.put("676773777673600","[TD=1]");
		myHCountry.put("676773777674600","[TD=1]");
		myHCountry.put("676773777717600","[FR=1]");
		myHCountry.put("676773777740200","[FR=1]");
		myHCountry.put("676773777747000","[FR=1]");
		myHCountry.put("676773777757600","[IT=1]");
		myHCountry.put("676773777764600","[DK=1]");
		myHCountry.put("676773777767400","[FR=1]");
		myHCountry.put("676773777774600","[ES=1]");
		myHCountry.put("676773777776200","[IT=1]");
		myHCountry.put("664073017777600","[BR=1]");
		myHCountry.put("664073377077600","[FR=1]");
		myHCountry.put("664073477777600","[CL=1]");
		myHCountry.put("664073617700600","[US=1]");
		myHCountry.put("664073677563600","[IE=1]");
		myHCountry.put("664073757777600","[AU=1]");
		myHCountry.put("664073767777600","[NZ=1]");
		myHCountry.put("664073775777600","[IE=1]");
		myHCountry.put("664073776333600","[AU=1]");
		myHCountry.put("664073776373600","[AU=1]");
		myHCountry.put("664073776777600","[CL=1]");
		myHCountry.put("664073777000600","[ES=1]");
		myHCountry.put("664073777073600","[AU=1]");
		myHCountry.put("664073777377600","[BR=1]");
		myHCountry.put("664073777400600","[GB=1]");
		myHCountry.put("664073777437600","[AU=1]");
		myHCountry.put("664073777600200","[IE=1]");
		myHCountry.put("664073777623600","[GB=1]");
		myHCountry.put("664073777637600","[AU=1]");
		myHCountry.put("664073777643600","[CL=1]");
		myHCountry.put("664073777763200","[IE=1]");
		myHCountry.put("664073777771600","[IE=1]");
		myHCountry.put("664073777774600","[AR=1]");
		myHCountry.put("664073777775600","[CL=1]");
		myHCountry.put("664073777776200","[BR=1]");
		myHCountry.put("640012017777600","[MX=1]");
		myHCountry.put("640013677777600","[GB=1]");
		myHCountry.put("640013776377600","[US=1]");
		myHCountry.put("640013777776600","[IE=1]");
		myHCountry.put("000000170000000","[US=1]");
		myHCountry.put("000000400200000","[GR=1]");
		myHCountry.put("000000717207600","[ES=1]");
		myHCountry.put("000001777207600","[AT=1]");
		myHCountry.put("000140000000000","[BD=1]");
		myHCountry.put("000200000002000","[BD=1]");
		myHCountry.put("000220000000000","[TW=1]");
		myHCountry.put("001020000000000","[MY=1]");
		myHCountry.put("002002007000000","[FR=1]");
		myHCountry.put("064263777046600","[MX=1]");
		myHCountry.put("074002007177200","[GB=1]");
		myHCountry.put("200000017377600","[IT=1]");
		myHCountry.put("200000077377600","[SE=1]");
		myHCountry.put("200003317207600","[ES=1]");
		myHCountry.put("200003377205200","[ES=1]");
		myHCountry.put("200003617207600","[ES=1]");
		myHCountry.put("200003637343600","[DE=1]");
		myHCountry.put("200003677376600","[IT=1]");
		myHCountry.put("200003717204600","[ES=1]");
		myHCountry.put("200003717207400","[DE=1]");
		myHCountry.put("200003757377600","[ES=1]");
		myHCountry.put("200003773377600","[DE=1]");
		myHCountry.put("200003776007600","[DE=1]");
		myHCountry.put("200003776207600","[DE=1]");
		myHCountry.put("200003777203600","[FR=1]");
		myHCountry.put("200003777237600","[FR=1]");
		myHCountry.put("200003777357000","[AT=1]");
		myHCountry.put("200003777361600","[ES=1]");
		myHCountry.put("200003777373600","[DE=1]");
		myHCountry.put("200003777375600","[AT=1]");
		myHCountry.put("200023717207600","[ES=1]");
		myHCountry.put("200023777237600","[FR=1]");
		myHCountry.put("200023777377600","[NL=1]");
		myHCountry.put("204773674017600","[ES=1]");
		myHCountry.put("037637743760771","[TR=1]");
		myHCountry.put("114605743440220","[RO=1]");
		myHCountry.put("114671743740220","[RO=1]");
		myHCountry.put("371767643760651","[FR=1]");
		myHCountry.put("400023743760771","[BE=1]");
		myHCountry.put("477777743760771","[NG=1]");
		myHCountry.put("600003143140011","[GB=1]");
		myHCountry.put("600003743760771","[IT=1]");
		myHCountry.put("677777743460771","[US=1]");
		myHCountry.put("701777743760751","[CM=1]");
		myHCountry.put("717777742360771","[HT=1]");
		myHCountry.put("760003743760771","[FR=1]");
		myHCountry.put("770177743760751","[SD=1]");
		myHCountry.put("773777743760731","[BE=1]");
		myHCountry.put("774077743760771","[US=1]");
		myHCountry.put("775547743760671","[NG=1]");
		myHCountry.put("777347742140771","[TZ=1]");
		myHCountry.put("777367743760661","[ES=1]");
		myHCountry.put("777707743760771","[FR=1]");
		myHCountry.put("777747743760771","[NG=1]");
		myHCountry.put("777760343740771","[NG=1]");
		myHCountry.put("777767743460771","[FR=1]");
		myHCountry.put("777771343760771","[NG=1]");
		myHCountry.put("777773743760771","[ZA=1]");
		myHCountry.put("777775343760771","[AT=1]");
		myHCountry.put("777775743760771","[NG=1]");
		myHCountry.put("777777643560771","[BR=1]");
		myHCountry.put("777777743360771","[US=1]");
		myHCountry.put("777777743460731","[NG=1]");
		myHCountry.put("777777743760371","[FR=1]");
		myHCountry.put("101067740000071","[PK=1]");
		myHCountry.put("101777740003771","[ET=1]");
		myHCountry.put("103741000000760","[IR=1]");
		myHCountry.put("103777600001771","[IN=1]");
		myHCountry.put("103777740000371","[US=1]");
		myHCountry.put("103777740000771","[PK=1]");
		myHCountry.put("303741000000741","[PK=1]");
		myHCountry.put("303741000000760","[PK=1]");
		myHCountry.put("303777400003771","[GB=1]");
		myHCountry.put("500000000000000","[IQ=1]");
		myHCountry.put("500000000003771","[IN=1]");
		myHCountry.put("500207000003671","[IN=1]");
		myHCountry.put("500377600003771","[IN=1]");
		myHCountry.put("500506000003771","[IN=1]");
		myHCountry.put("501347740003661","[IN=1]");
		myHCountry.put("501367000003261","[PK=1]");
		myHCountry.put("501727740003071","[IQ=1]");
		myHCountry.put("501767740003071","[IQ=1]");
		myHCountry.put("501767740003131","[IN=1]");
		myHCountry.put("501767740003671","[IN=1]");
		myHCountry.put("503377740000371","[TZ=1]");
		myHCountry.put("503667700003771","[IQ=1]");
		myHCountry.put("503707700000771","[IN=1]");
		myHCountry.put("503767400001771","[IQ=1]");
		myHCountry.put("503767600003771","[OM=1]");
		myHCountry.put("503767700003771","[ZA=1]");
		myHCountry.put("503767740000671","[IN=1]");
		myHCountry.put("503776000003771","[GB=1]");
		myHCountry.put("503777000003171","[IN=1]");
		myHCountry.put("503777400003771","[IR=1]");
		myHCountry.put("503777700003771","[IN=1]");
		myHCountry.put("503777740000171","[IN=1]");
		myHCountry.put("700000000000011","[MY=1]");
		myHCountry.put("700000000000031","[SA=1]");
		myHCountry.put("700000000000071","[BE=1]");
		myHCountry.put("700000400000171","[BD=1]");
		myHCountry.put("700001400000061","[ES=1]");
		myHCountry.put("700002000000000","[FR=1]");
		myHCountry.put("700007400003771","[IN=1]");
		myHCountry.put("700017740000000","[CI=1]");
		myHCountry.put("700077740000371","[OM=1]");
		myHCountry.put("700337400003771","[MG=1]");
		myHCountry.put("700377400003771","[IR=1]");
		myHCountry.put("700377640001771","[IN=1]");
		myHCountry.put("700377700000031","[DE=1]");
		myHCountry.put("700377700000771","[IN=1]");
		myHCountry.put("700700000003771","[IN=1]");
		myHCountry.put("700737740003571","[IN=1]");
		myHCountry.put("700737740003771","[IN=1]");
		myHCountry.put("700777700003771","[IN=1]");
		myHCountry.put("700777740003171","[ET=1]");
		myHCountry.put("701060400003230","[SD=1]");
		myHCountry.put("701341000000771","[ZA=1]");
		myHCountry.put("701347000000771","[TZ=1]");
		myHCountry.put("701367740003671","[IN=1]");
		myHCountry.put("701467740003671","[PK=1]");
		myHCountry.put("701573600001771","[TZ=1]");
		myHCountry.put("701721000003771","[PK=1]");
		myHCountry.put("701737740003771","[BE=1]");
		myHCountry.put("701767600003671","[PK=1]");
		myHCountry.put("701767700003071","[IQ=1]");
		myHCountry.put("701767740003070","[PK=1]");
		myHCountry.put("701773740003771","[GB=1]");
		myHCountry.put("701776000000001","[CN=1]");
		myHCountry.put("701777440003771","[BD=1]");
		myHCountry.put("701777600001771","[ZA=1]");
		myHCountry.put("701777640003771","[JP=1]");
		myHCountry.put("701777700000371","[NL=1]");
		myHCountry.put("701777740003171","[BE=1]");
		myHCountry.put("702000000000771","[IN=1]");
		myHCountry.put("702077400001771","[NL=1]");
		myHCountry.put("702377400001771","[SE=1]");
		myHCountry.put("702777000003771","[IR=1]");
		myHCountry.put("703000000000771","[IN=1]");
		myHCountry.put("703000000003171","[ET=1]");
		myHCountry.put("703337740000771","[IN=1]");
		myHCountry.put("703370000000000","[MG=1]");
		myHCountry.put("703377200001771","[BE=1]");
		myHCountry.put("703377700001771","[TZ=1]");
		myHCountry.put("703377700003171","[CN=1]");
		myHCountry.put("703377740000771","[OM=1]");
		myHCountry.put("703600000003771","[MZ=1]");
		myHCountry.put("703601000003771","[IN=1]");
		myHCountry.put("703627400003771","[IQ=1]");
		myHCountry.put("703637400000000","[MM=1]");
		myHCountry.put("703637400003771","[IQ=1]");
		myHCountry.put("703637700003771","[IN=1]");
		myHCountry.put("703637740000771","[IN=1]");
		myHCountry.put("703677400003771","[SA=1]");
		myHCountry.put("703677740000371","[SA=1]");
		myHCountry.put("703700000000760","[IN=1]");
		myHCountry.put("703701000000000","[IN=1]");
		myHCountry.put("703701000000760","[IN=1]");
		myHCountry.put("703703700003771","[IN=1]");
		myHCountry.put("703707740001771","[US=1]");
		myHCountry.put("703717700000371","[US=1]");
		myHCountry.put("703721000000371","[ZA=1]");
		myHCountry.put("703727400001771","[OM=1]");
		myHCountry.put("703737000003371","[IN=1]");
		myHCountry.put("703737740000000","[IN=1]");
		myHCountry.put("703740000000700","[SR=1]");
		myHCountry.put("703740000000771","[US=1]");
		myHCountry.put("703740000003171","[CN=1]");
		myHCountry.put("703740000003731","[IN=1]");
		myHCountry.put("703740000003760","[US=1]");
		myHCountry.put("703741000000331","[PK=1]");
		myHCountry.put("703741000000700","[OM=1]");
		myHCountry.put("703741000000770","[IT=1]");
		myHCountry.put("703741600003771","[GB=1]");
		myHCountry.put("703755740000171","[AT=1]");
		myHCountry.put("703757740000371","[US=1]");
		myHCountry.put("703760000000000","[IN=1]");
		myHCountry.put("703761000000371","[IN=1]");
		myHCountry.put("703761000000471","[PK=1]");
		myHCountry.put("703761000003771","[IN=1]");
		myHCountry.put("703763500000771","[IN=1]");
		myHCountry.put("703763740001171","[SA=1]");
		myHCountry.put("703767400003070","[SD=1]");
		myHCountry.put("703767400003071","[IQ=1]");
		myHCountry.put("703767700003771","[SA=1]");
		myHCountry.put("703767740000171","[IQ=1]");
		myHCountry.put("703770000000771","[IN=1]");
		myHCountry.put("703773700003771","[GB=1]");
		myHCountry.put("703774000000031","[SA=1]");
		myHCountry.put("703774000000771","[SA=1]");
		myHCountry.put("703774000003171","[US=1]");
		myHCountry.put("703774000003731","[BD=1]");
		myHCountry.put("703775000000171","[IN=1]");
		myHCountry.put("703775000000371","[OM=1]");
		myHCountry.put("703775400000171","[NL=1]");
		myHCountry.put("703775700000171","[FR=1]");
		myHCountry.put("703775700003771","[MM=1]");
		myHCountry.put("703775740000771","[IN=1]");
		myHCountry.put("703776000003771","[US=1]");
		myHCountry.put("703776400001771","[YE=1]");
		myHCountry.put("703776700003771","[GB=1]");
		myHCountry.put("703777000000031","[US=1]");
		myHCountry.put("703777000000071","[ET=1]");
		myHCountry.put("703777000001771","[IR=1]");
		myHCountry.put("703777200000371","[KR=1]");
		myHCountry.put("703777300000771","[IN=1]");
		myHCountry.put("703777400003111","[IR=1]");
		myHCountry.put("703777400003471","[SA=1]");
		myHCountry.put("703777500003771","[IN=1]");
		myHCountry.put("703777600002771","[IN=1]");
		myHCountry.put("703777600003071","[NL=1]");
		myHCountry.put("703777600003151","[SA=1]");
		myHCountry.put("703777600003271","[IT=1]");
		myHCountry.put("703777600003760","[PK=1]");
		myHCountry.put("703777700000340","[IN=1]");
		myHCountry.put("703777700001771","[NP=1]");
		myHCountry.put("703777700003471","[SD=1]");
		myHCountry.put("703777740000011","[US=1]");
		myHCountry.put("703777740000271","[PK=1]");
		myHCountry.put("703777740000370","[SA=1]");
		myHCountry.put("703777740000471","[PK=1]");
		myHCountry.put("703777740000671","[PK=1]");
		myHCountry.put("703777740000730","[TZ=1]");
		myHCountry.put("703777740000760","[PK=1]");
		myHCountry.put("703777740001571","[IN=1]");
		myHCountry.put("703777740001700","[GB=1]");
		myHCountry.put("703777740001751","[NL=1]");
		myHCountry.put("102000340003171","[PK=1]");
		myHCountry.put("103767740003171","[IN=1]");
		myHCountry.put("103767740003771","[IN=1]");
		myHCountry.put("103777740003071","[MM=1]");
		myHCountry.put("303777740003731","[GB=1]");
		myHCountry.put("503367740003171","[SA=1]");
		myHCountry.put("503767640003021","[IN=1]");
		myHCountry.put("503767740003000","[IQ=1]");
		myHCountry.put("503767740003761","[IQ=1]");
		myHCountry.put("503777740003071","[IN=1]");
		myHCountry.put("503777740003171","[ET=1]");
		myHCountry.put("702077740003771","[IN=1]");
		myHCountry.put("702767740003771","[MZ=1]");
		myHCountry.put("702777340003771","[TN=1]");
		myHCountry.put("702777740003731","[OM=1]");
		myHCountry.put("703000740003771","[IN=1]");
		myHCountry.put("703001740003771","[IN=1]");
		myHCountry.put("703177740003031","[SD=1]");
		myHCountry.put("703237440003771","[IN=1]");
		myHCountry.put("703307740003771","[IN=1]");
		myHCountry.put("703347740003671","[ET=1]");
		myHCountry.put("703347740003771","[IN=1]");
		myHCountry.put("703367740003771","[OM=1]");
		myHCountry.put("703377740003661","[PK=1]");
		myHCountry.put("703417740003071","[US=1]");
		myHCountry.put("703457740003771","[IN=1]");
		myHCountry.put("703577640003771","[IN=1]");
		myHCountry.put("703577740003171","[SD=1]");
		myHCountry.put("703637740003731","[IN=1]");
		myHCountry.put("703676340003171","[ET=1]");
		myHCountry.put("703677540003771","[BD=1]");
		myHCountry.put("703700340003771","[US=1]");
		myHCountry.put("703701740003771","[IN=1]");
		myHCountry.put("703707740003471","[IN=1]");
		myHCountry.put("703707740003571","[IN=1]");
		myHCountry.put("703707740003731","[GB=1]");
		myHCountry.put("703707740003770","[OM=1]");
		myHCountry.put("703717740003771","[IN=1]");
		myHCountry.put("703721740003471","[SA=1]");
		myHCountry.put("703723740003771","[PK=1]");
		myHCountry.put("703727740003071","[PK=1]");
		myHCountry.put("703727740003111","[IQ=1]");
		myHCountry.put("703727740003771","[IN=1]");
		myHCountry.put("703736140003171","[SD=1]");
		myHCountry.put("703737740003371","[PL=1]");
		myHCountry.put("703743740003471","[US=1]");
		myHCountry.put("703747640003771","[IN=1]");
		myHCountry.put("703747740003471","[IN=1]");
		myHCountry.put("703747740003771","[IN=1]");
		myHCountry.put("703760040003071","[IN=1]");
		myHCountry.put("703760040003471","[IN=1]");
		myHCountry.put("703763740003471","[US=1]");
		myHCountry.put("703767740003071","[IQ=1]");
		myHCountry.put("703767740003371","[SA=1]");
		myHCountry.put("703767740003470","[SA=1]");
		myHCountry.put("703767740003661","[IQ=1]");
		myHCountry.put("703767740003731","[IN=1]");
		myHCountry.put("703767740003751","[OM=1]");
		myHCountry.put("703770740003771","[PK=1]");
		myHCountry.put("703771240003771","[SR=1]");
		myHCountry.put("703773740003171","[SA=1]");
		myHCountry.put("703774740003771","[MW=1]");
		myHCountry.put("703775640003771","[OM=1]");
		myHCountry.put("703775740003171","[US=1]");
		myHCountry.put("703775740003571","[PK=1]");
		myHCountry.put("703776740003771","[US=1]");
		myHCountry.put("703777340003171","[ET=1]");
		myHCountry.put("703777540003171","[SA=1]");
		myHCountry.put("703777540003771","[GB=1]");
		myHCountry.put("703777740002000","[OM=1]");
		myHCountry.put("703777740002031","[IN=1]");
		myHCountry.put("703777740003010","[US=1]");
		myHCountry.put("703777740003051","[US=1]");
		myHCountry.put("703777740003161","[IQ=1]");
		myHCountry.put("703777740003170","[SA=1]");
		myHCountry.put("703777740003631","[PK=1]");
		myHCountry.put("703777740003661","[PK=1]");
		myHCountry.put("703777740003700","[IN=1]");
		myHCountry.put("703777740003721","[AU=1]");
		myHCountry.put("703777740003741","[PK=1]");
		myHCountry.put("103377400001771","[ET=1]");
		myHCountry.put("703377400001471","[TZ=1]");
		myHCountry.put("703377400001731","[ZA=1]");
		myHCountry.put("500000740003671","[IN=1]");
		myHCountry.put("500003740003771","[BD=1]");
		myHCountry.put("500367640003671","[IN=1]");
		myHCountry.put("500377740003771","[IN=1]");
		myHCountry.put("700000740003711","[GB=1]");
		myHCountry.put("700000740003771","[IN=1]");
		myHCountry.put("700277740003711","[IN=1]");
		myHCountry.put("700277740003771","[NL=1]");
		myHCountry.put("700337740003631","[IN=1]");
		myHCountry.put("700343740003771","[OM=1]");
		myHCountry.put("700367740003771","[US=1]");
		myHCountry.put("700376340003371","[GB=1]");
		myHCountry.put("700377040003771","[IN=1]");
		myHCountry.put("700377740003671","[PK=1]");
		myHCountry.put("700377740003731","[NL=1]");
		myHCountry.put("077777747413731","[DK=1]");
		myHCountry.put("537776755413731","[NO=1]");
		myHCountry.put("577777755413331","[BD=1]");
		myHCountry.put("700777747413731","[MZ=1]");
		myHCountry.put("775777757413331","[BD=1]");
		myHCountry.put("777377757413331","[BD=1]");
		myHCountry.put("777767743413331","[BD=1]");
		myHCountry.put("777777341413331","[BD=1]");
		myHCountry.put("777777441413331","[BD=1]");
		myHCountry.put("777777747413731","[US=1]");
		myHCountry.put("777777751413331","[BD=1]");
		myHCountry.put("777777755413331","[BD=1]");
		myHCountry.put("777777755413731","[GB=1]");
		myHCountry.put("777777757413731","[MZ=1]");
		myHCountry.put("000000007413731","[IN=1]");
		myHCountry.put("000006007413731","[IN=1]");
		myHCountry.put("077327776413731","[TZ=1]");
		myHCountry.put("077777777413730","[GB=1]");
		myHCountry.put("137777777413731","[GB=1]");
		myHCountry.put("175776677413131","[MY=1]");
		myHCountry.put("177777777013721","[GB=1]");
		myHCountry.put("177777777413131","[MY=1]");
		myHCountry.put("377777377413731","[MY=1]");
		myHCountry.put("436375777413731","[IN=1]");
		myHCountry.put("457767637413731","[MZ=1]");
		myHCountry.put("474377777413731","[SG=1]");
		myHCountry.put("507777677413731","[BE=1]");
		myHCountry.put("577777437413731","[MZ=1]");
		myHCountry.put("666755777413731","[NL=1]");
		myHCountry.put("671777707413731","[US=1]");
		myHCountry.put("677703777413731","[DK=1]");
		myHCountry.put("677767677413731","[FR=1]");
		myHCountry.put("700177777413731","[US=1]");
		myHCountry.put("701767607413731","[MZ=1]");
		myHCountry.put("701777777413731","[FR=1]");
		myHCountry.put("703777777413731","[IN=1]");
		myHCountry.put("707777777413731","[BD=1]");
		myHCountry.put("715757503413131","[ID=1]");
		myHCountry.put("717477777413731","[US=1]");
		myHCountry.put("717777775412731","[SG=1]");
		myHCountry.put("723777777413731","[SE=1]");
		myHCountry.put("734000050013331","[BD=1]");
		myHCountry.put("737000377413731","[OM=1]");
		myHCountry.put("737777760013731","[MM=1]");
		myHCountry.put("740767777413331","[ID=1]");
		myHCountry.put("754177777413731","[FR=1]");
		myHCountry.put("757427607413731","[MZ=1]");
		myHCountry.put("757677677413731","[MW=1]");
		myHCountry.put("757767677413731","[MZ=1]");
		myHCountry.put("757767777413731","[MZ=1]");
		myHCountry.put("757775777413731","[MZ=1]");
		myHCountry.put("757777577413731","[MZ=1]");
		myHCountry.put("757777677413731","[MZ=1]");
		myHCountry.put("774027777413731","[MZ=1]");
		myHCountry.put("774137777413731","[TH=1]");
		myHCountry.put("775736777411731","[NL=1]");
		myHCountry.put("775777777412731","[ID=1]");
		myHCountry.put("776177775413731","[US=1]");
		myHCountry.put("776177777413731","[BD=1]");
		myHCountry.put("776363777413731","[OM=1]");
		myHCountry.put("776777777412731","[ID=1]");
		myHCountry.put("776777777413731","[FR=1]");
		myHCountry.put("777037377413731","[IN=1]");
		myHCountry.put("777177777413731","[MM=1]");
		myHCountry.put("777377762013731","[LK=1]");
		myHCountry.put("777600377413731","[US=1]");
		myHCountry.put("777636777413731","[US=1]");
		myHCountry.put("777723775413731","[IT=1]");
		myHCountry.put("777727677413731","[OM=1]");
		myHCountry.put("777727777413731","[MZ=1]");
		myHCountry.put("777737775413731","[MY=1]");
		myHCountry.put("777737777410731","[GB=1]");
		myHCountry.put("777747777413731","[US=1]");
		myHCountry.put("777760007413731","[MY=1]");
		myHCountry.put("777761777413731","[GB=1]");
		myHCountry.put("777763362413731","[OM=1]");
		myHCountry.put("777767717413731","[MZ=1]");
		myHCountry.put("777770037413731","[MY=1]");
		myHCountry.put("777774377413731","[US=1]");
		myHCountry.put("777776377412731","[BD=1]");
		myHCountry.put("777776777412131","[MY=1]");
		myHCountry.put("777776777412731","[AT=1]");
		myHCountry.put("777776777413131","[MY=1]");
		myHCountry.put("777777377413331","[ID=1]");
		myHCountry.put("777777603413731","[OM=1]");
		myHCountry.put("777777677413731","[NL=1]");
		myHCountry.put("777777702010331","[FR=1]");
		myHCountry.put("777777717413731","[MZ=1]");
		myHCountry.put("777777737413731","[IN=1]");
		myHCountry.put("777777760413731","[IN=1]");
		myHCountry.put("777777772013731","[OM=1]");
		myHCountry.put("777777772413731","[IN=1]");
		myHCountry.put("777777773413731","[ZW=1]");
		myHCountry.put("777777777413720","[IN=1]");
		myHCountry.put("657740007413771","[TW=1]");
		myHCountry.put("666000003413771","[BE=1]");
		myHCountry.put("677617407413371","[BD=1]");
		myHCountry.put("677740002013671","[FR=1]");
		myHCountry.put("677740017413771","[US=1]");
		myHCountry.put("677740077413771","[MX=1]");
		myHCountry.put("677760077413771","[US=1]");
		myHCountry.put("677767077413671","[MY=1]");
		myHCountry.put("677770003413771","[US=1]");
		myHCountry.put("677774000413771","[MY=1]");
		myHCountry.put("677776017413771","[MY=1]");
		myHCountry.put("677776037413771","[US=1]");
		myHCountry.put("677777077412771","[US=1]");
		myHCountry.put("677777077413771","[AU=1]");
		myHCountry.put("677777407413771","[US=1]");
		myHCountry.put("277775477413771","[US=1]");
		myHCountry.put("277777471413771","[SA=1]");
		myHCountry.put("640077477413771","[US=1]");
		myHCountry.put("647777477013771","[TW=1]");
		myHCountry.put("657777476413771","[US=1]");
		myHCountry.put("657777477413601","[MY=1]");
		myHCountry.put("657777477413751","[MY=1]");
		myHCountry.put("660177457413771","[IT=1]");
		myHCountry.put("661777477413771","[IT=1]");
		myHCountry.put("663777477413771","[US=1]");
		myHCountry.put("666003477413771","[MY=1]");
		myHCountry.put("667777477413771","[US=1]");
		myHCountry.put("673777477413470","[MY=1]");
		myHCountry.put("675757477413701","[US=1]");
		myHCountry.put("677417447413771","[SE=1]");
		myHCountry.put("677577457413771","[US=1]");
		myHCountry.put("677607477413731","[IT=1]");
		myHCountry.put("677637477413771","[US=1]");
		myHCountry.put("677677475413470","[MY=1]");
		myHCountry.put("677737470413771","[US=1]");
		myHCountry.put("677743477413771","[US=1]");
		myHCountry.put("677757477413771","[US=1]");
		myHCountry.put("677773477413771","[US=1]");
		myHCountry.put("677774477413771","[US=1]");
		myHCountry.put("677776477413471","[MY=1]");
		myHCountry.put("677777471413771","[US=1]");
		myHCountry.put("677777473413771","[US=1]");
		myHCountry.put("677777475413701","[MY=1]");
		myHCountry.put("677777475413771","[TW=1]");
		myHCountry.put("677777477410071","[US=1]");
		myHCountry.put("677777477413001","[US=1]");
		myHCountry.put("677777477413400","[PH=1]");
		myHCountry.put("677777477413470","[MY=1]");
		myHCountry.put("677777477413601","[US=1]");
		myHCountry.put("674000003413400","[MY=1]");
		myHCountry.put("674000003413710","[US=1]");
		myHCountry.put("674000003413731","[US=1]");
		myHCountry.put("674000003413770","[SA=1]");
		myHCountry.put("077417773413071","[IN=1]");
		myHCountry.put("400020277413071","[IN=1]");
		myHCountry.put("401767777413071","[DK=1]");
		myHCountry.put("401776777413071","[FR=1]");
		myHCountry.put("403767777413071","[DK=1]");
		myHCountry.put("443777777413071","[AT=1]");
		myHCountry.put("457767777413071","[US=1]");
		myHCountry.put("463777777413071","[NL=1]");
		myHCountry.put("467400777413071","[MY=1]");
		myHCountry.put("467677777413071","[IN=1]");
		myHCountry.put("473777777413070","[MY=1]");
		myHCountry.put("474003777413071","[NL=1]");
		myHCountry.put("474007777413071","[FR=1]");
		myHCountry.put("474017777413071","[MY=1]");
		myHCountry.put("474023777413071","[PK=1]");
		myHCountry.put("476037777413071","[GF=1]");
		myHCountry.put("477061777413071","[MM=1]");
		myHCountry.put("477407777413071","[GB=1]");
		myHCountry.put("477417777413071","[IN=1]");
		myHCountry.put("477467777413071","[PK=1]");
		myHCountry.put("477606317413071","[IN=1]");
		myHCountry.put("477701777413071","[MY=1]");
		myHCountry.put("477717777413071","[IN=1]");
		myHCountry.put("477723777413071","[IN=1]");
		myHCountry.put("477737477413071","[NL=1]");
		myHCountry.put("477747777413071","[US=1]");
		myHCountry.put("477770377413071","[US=1]");
		myHCountry.put("477773777413071","[US=1]");
		myHCountry.put("477774200013071","[US=1]");
		myHCountry.put("477775202013071","[NL=1]");
		myHCountry.put("477775342413071","[IN=1]");
		myHCountry.put("477775360013071","[IN=1]");
		myHCountry.put("477776037413071","[BD=1]");
		myHCountry.put("477777377413071","[NL=1]");
		myHCountry.put("477777402013071","[IN=1]");
		myHCountry.put("477777477413071","[TN=1]");
		myHCountry.put("477777767013071","[IN=1]");
		myHCountry.put("477777770413071","[IN=1]");
		myHCountry.put("477777771413071","[MY=1]");
		myHCountry.put("477777775413051","[SA=1]");
		myHCountry.put("477777775413071","[GB=1]");
		myHCountry.put("477777777013071","[IN=1]");
		myHCountry.put("477777777413060","[IN=1]");
		myHCountry.put("377777374413771","[US=1]");
		myHCountry.put("474377774413771","[FR=1]");
		myHCountry.put("501347674413671","[IN=1]");
		myHCountry.put("577601774413771","[DK=1]");
		myHCountry.put("577737774413771","[NL=1]");
		myHCountry.put("577767774413771","[DK=1]");
		myHCountry.put("577773774413771","[DK=1]");
		myHCountry.put("577777774413751","[VN=1]");
		myHCountry.put("700000774413771","[NL=1]");
		myHCountry.put("717777774413771","[VN=1]");
		myHCountry.put("767777774413771","[GM=1]");
		myHCountry.put("773777774412771","[AU=1]");
		myHCountry.put("773777774413731","[VN=1]");
		myHCountry.put("773777774413771","[FR=1]");
		myHCountry.put("774777774413771","[IT=1]");
		myHCountry.put("775777774413701","[US=1]");
		myHCountry.put("777417774413711","[VN=1]");
		myHCountry.put("777737754413771","[VN=1]");
		myHCountry.put("777757474413771","[DK=1]");
		myHCountry.put("777777734413771","[VN=1]");
		myHCountry.put("777777754413771","[US=1]");
		myHCountry.put("777777774410171","[US=1]");
		myHCountry.put("777777774411600","[TH=1]");
		myHCountry.put("777777774412771","[VN=1]");
		myHCountry.put("777777774413301","[SA=1]");
		myHCountry.put("777777774413471","[US=1]");
		myHCountry.put("777777774413700","[VN=1]");
		myHCountry.put("000000005413671","[MY=1]");
		myHCountry.put("000000077413671","[SG=1]");
		myHCountry.put("003777400010771","[SD=1]");
		myHCountry.put("003777775413031","[MY=1]");
		myHCountry.put("033777477413771","[PH=1]");
		myHCountry.put("037777740013771","[IN=1]");
		myHCountry.put("037777777413700","[GB=1]");
		myHCountry.put("074000003413771","[SA=1]");
		myHCountry.put("077577477413750","[NL=1]");
		myHCountry.put("077737777413740","[US=1]");
		myHCountry.put("077757777413771","[GB=1]");
		myHCountry.put("077773477413771","[US=1]");
		myHCountry.put("077776117413771","[US=1]");
		myHCountry.put("077777077413771","[TZ=1]");
		myHCountry.put("077777474013171","[NL=1]");
		myHCountry.put("077777477412771","[US=1]");
		myHCountry.put("077777777413031","[OM=1]");
		myHCountry.put("100000001413031","[BD=1]");
		myHCountry.put("177767777413700","[IN=1]");
		myHCountry.put("177777177413771","[MY=1]");
		myHCountry.put("177777777413371","[US=1]");
		myHCountry.put("337777777412031","[SA=1]");
		myHCountry.put("374176001413771","[MY=1]");
		myHCountry.put("377600007413371","[FR=1]");
		myHCountry.put("377736777413430","[MY=1]");
		myHCountry.put("377777000413771","[GB=1]");
		myHCountry.put("400000077413671","[MG=1]");
		myHCountry.put("400007400413771","[DK=1]");
		myHCountry.put("400007637413671","[NO=1]");
		myHCountry.put("400007777413671","[OM=1]");
		myHCountry.put("400020077413771","[OM=1]");
		myHCountry.put("400034037413771","[OM=1]");
		myHCountry.put("400036777413771","[MX=1]");
		myHCountry.put("400037001413771","[OM=1]");
		myHCountry.put("400037777413700","[MZ=1]");
		myHCountry.put("400037777413711","[OM=1]");
		myHCountry.put("400037777413760","[MZ=1]");
		myHCountry.put("400037777413770","[NL=1]");
		myHCountry.put("401777777413771","[LK=1]");
		myHCountry.put("402767777413771","[MG=1]");
		myHCountry.put("403777377413771","[MG=1]");
		myHCountry.put("417777777413011","[IN=1]");
		myHCountry.put("417777777413771","[FR=1]");
		myHCountry.put("437737377413371","[SA=1]");
		myHCountry.put("437760377410771","[MW=1]");
		myHCountry.put("437777277413771","[GB=1]");
		myHCountry.put("437777777413771","[TH=1]");
		myHCountry.put("454000003413661","[TH=1]");
		myHCountry.put("454377777413771","[IN=1]");
		myHCountry.put("457767777413771","[SA=1]");
		myHCountry.put("457777777413031","[DK=1]");
		myHCountry.put("457777777413771","[OM=1]");
		myHCountry.put("460023777413771","[IN=1]");
		myHCountry.put("460037037413771","[PK=1]");
		myHCountry.put("461777777413771","[US=1]");
		myHCountry.put("467377777413771","[OM=1]");
		myHCountry.put("470000377413031","[BR=1]");
		myHCountry.put("470377777413771","[MM=1]");
		myHCountry.put("473776077411771","[US=1]");
		myHCountry.put("473777077413771","[IN=1]");
		myHCountry.put("473777776013771","[OM=1]");
		myHCountry.put("474000001413671","[MY=1]");
		myHCountry.put("474000003413771","[US=1]");
		myHCountry.put("474000077413771","[IN=1]");
		myHCountry.put("474360377413771","[IN=1]");
		myHCountry.put("474361777413771","[IT=1]");
		myHCountry.put("474367767413771","[SA=1]");
		myHCountry.put("474367777413771","[US=1]");
		myHCountry.put("474377707413771","[MM=1]");
		myHCountry.put("474377777412771","[US=1]");
		myHCountry.put("474377777413671","[GB=1]");
		myHCountry.put("475777677413771","[US=1]");
		myHCountry.put("475777777413771","[IN=1]");
		myHCountry.put("476003477413771","[MY=1]");
		myHCountry.put("476177400010171","[ES=1]");
		myHCountry.put("476767777413771","[MG=1]");
		myHCountry.put("476777777413031","[SG=1]");
		myHCountry.put("477347777413271","[OM=1]");
		myHCountry.put("477377077413671","[TZ=1]");
		myHCountry.put("477407777413771","[US=1]");
		myHCountry.put("477437777413771","[MY=1]");
		myHCountry.put("477673777410771","[MW=1]");
		myHCountry.put("477701777413771","[SE=1]");
		myHCountry.put("477703777413751","[NL=1]");
		myHCountry.put("477707477413771","[US=1]");
		myHCountry.put("477737737413771","[OM=1]");
		myHCountry.put("477737777413771","[GB=1]");
		myHCountry.put("477740037413771","[SE=1]");
		myHCountry.put("477763777413001","[IN=1]");
		myHCountry.put("477767037413771","[SA=1]");
		myHCountry.put("477767477413771","[DK=1]");
		myHCountry.put("477767677413771","[IN=1]");
		myHCountry.put("477771777413031","[IN=1]");
		myHCountry.put("477773777413760","[LK=1]");
		myHCountry.put("477773777413771","[IN=1]");
		myHCountry.put("477774377413031","[SG=1]");
		myHCountry.put("477774777413571","[SE=1]");
		myHCountry.put("477776374013771","[MG=1]");
		myHCountry.put("477776471413771","[MY=1]");
		myHCountry.put("477776777412771","[AU=1]");
		myHCountry.put("477776777413031","[US=1]");
		myHCountry.put("477777003413571","[SA=1]");
		myHCountry.put("477777003413771","[SA=1]");
		myHCountry.put("477777077411171","[TZ=1]");
		myHCountry.put("477777077411771","[FR=1]");
		myHCountry.put("477777437413771","[US=1]");
		myHCountry.put("477777475413470","[MY=1]");
		myHCountry.put("477777477410771","[VN=1]");
		myHCountry.put("477777477413671","[US=1]");
		myHCountry.put("477777477413751","[MY=1]");
		myHCountry.put("477777677413771","[IN=1]");
		myHCountry.put("477777774013771","[PK=1]");
		myHCountry.put("477777775413671","[IN=1]");
		myHCountry.put("477777777410031","[GB=1]");
		myHCountry.put("477777777410171","[DK=1]");
		myHCountry.put("477777777411770","[GB=1]");
		myHCountry.put("477777777411771","[IN=1]");
		myHCountry.put("477777777412001","[BE=1]");
		myHCountry.put("477777777412700","[SE=1]");
		myHCountry.put("477777777413001","[GB=1]");
		myHCountry.put("477777777413030","[MU=1]");
		myHCountry.put("477777777413371","[NL=1]");
		myHCountry.put("477777777413700","[GB=1]");
		myHCountry.put("477777777413701","[US=1]");
		myHCountry.put("541777777413071","[SG=1]");
		myHCountry.put("557767777413700","[US=1]");
		myHCountry.put("570077777413071","[MY=1]");
		myHCountry.put("574003477413771","[NL=1]");
		myHCountry.put("575777777413771","[DK=1]");
		myHCountry.put("577577777413371","[SE=1]");
		myHCountry.put("577707637413661","[IT=1]");
		myHCountry.put("577747777413471","[IN=1]");
		myHCountry.put("577747777413600","[DK=1]");
		myHCountry.put("577757600413771","[BD=1]");
		myHCountry.put("577763777413071","[IN=1]");
		myHCountry.put("577767750013771","[TZ=1]");
		myHCountry.put("577775777413700","[NL=1]");
		myHCountry.put("577776001413771","[BD=1]");
		myHCountry.put("577777003413771","[BD=1]");
		myHCountry.put("577777077411171","[RE=1]");
		myHCountry.put("577777777412071","[IN=1]");
		myHCountry.put("577777777413371","[MM=1]");
		myHCountry.put("577777777413631","[IN=1]");
		myHCountry.put("577777777413671","[MM=1]");
		myHCountry.put("600001740013171","[SD=1]");
		myHCountry.put("600777477413771","[US=1]");
		myHCountry.put("601737477413771","[MY=1]");
		myHCountry.put("603777477413771","[US=1]");
		myHCountry.put("603777777413771","[FR=1]");
		myHCountry.put("637776477413771","[MY=1]");
		myHCountry.put("643777777413071","[IT=1]");
		myHCountry.put("674177777413751","[US=1]");
		myHCountry.put("675767377413671","[DK=1]");
		myHCountry.put("677367677413771","[ES=1]");
		myHCountry.put("677577777413071","[BE=1]");
		myHCountry.put("677737777413711","[VN=1]");
		myHCountry.put("677767377413771","[IQ=1]");
		myHCountry.put("677767677413771","[FR=1]");
		myHCountry.put("677775777413771","[GB=1]");
		myHCountry.put("677776777413071","[IN=1]");
		myHCountry.put("677777177413771","[MY=1]");
		myHCountry.put("677777377413771","[IQ=1]");
		myHCountry.put("677777577013771","[ES=1]");
		myHCountry.put("677777577413671","[IT=1]");
		myHCountry.put("677777577413771","[IT=1]");
		myHCountry.put("677777677413751","[MY=1]");
		myHCountry.put("677777757013371","[BE=1]");
		myHCountry.put("677777777412071","[IT=1]");
		myHCountry.put("677777777413631","[GW=1]");
		myHCountry.put("677777777413700","[BD=1]");
		myHCountry.put("700000001413371","[BD=1]");
		myHCountry.put("700000067413371","[US=1]");
		myHCountry.put("700001777413771","[MM=1]");
		myHCountry.put("700003600010000","[SD=1]");
		myHCountry.put("700077777413771","[MX=1]");
		myHCountry.put("700764007413771","[MG=1]");
		myHCountry.put("700765744013671","[ZM=1]");
		myHCountry.put("700775746013771","[MW=1]");
		myHCountry.put("700775776013771","[MW=1]");
		myHCountry.put("700777777413771","[US=1]");
		myHCountry.put("702337740013171","[SD=1]");
		myHCountry.put("703377400011771","[SD=1]");
		myHCountry.put("703717740013171","[SD=1]");
		myHCountry.put("703737740011571","[SD=1]");
		myHCountry.put("703763740013031","[SD=1]");
		myHCountry.put("703777700010771","[ES=1]");
		myHCountry.put("703777700013771","[SD=1]");
		myHCountry.put("703777740010000","[SD=1]");
		myHCountry.put("703777740013031","[SD=1]");
		myHCountry.put("703777740013071","[SD=1]");
		myHCountry.put("703777777013741","[SA=1]");
		myHCountry.put("707777777413700","[BD=1]");
		myHCountry.put("715777777413771","[TH=1]");
		myHCountry.put("717001777413771","[OM=1]");
		myHCountry.put("717777776010371","[BR=1]");
		myHCountry.put("717777777412371","[IN=1]");
		myHCountry.put("717777777413631","[IN=1]");
		myHCountry.put("717777777413700","[US=1]");
		myHCountry.put("717777777413771","[FR=1]");
		myHCountry.put("730017777411631","[TH=1]");
		myHCountry.put("730177777413631","[MM=1]");
		myHCountry.put("734177777413671","[MM=1]");
		myHCountry.put("737773637413771","[TH=1]");
		myHCountry.put("737775677413671","[AU=1]");
		myHCountry.put("737777707413171","[ID=1]");
		myHCountry.put("737777776413771","[SE=1]");
		myHCountry.put("737777777413700","[IN=1]");
		myHCountry.put("741777777413700","[IN=1]");
		myHCountry.put("743777777413030","[MY=1]");
		myHCountry.put("743777777413771","[MM=1]");
		myHCountry.put("747777777413071","[DE=1]");
		myHCountry.put("747777777413700","[IN=1]");
		myHCountry.put("754177777413771","[TH=1]");
		myHCountry.put("757037777413700","[SA=1]");
		myHCountry.put("757777477413771","[US=1]");
		myHCountry.put("757777607412401","[ZM=1]");
		myHCountry.put("757777737410031","[MW=1]");
		myHCountry.put("757777737410071","[MW=1]");
		myHCountry.put("757777776010771","[IN=1]");
		myHCountry.put("757777777413631","[TH=1]");
		myHCountry.put("757777777413671","[NL=1]");
		myHCountry.put("757777777413771","[TN=1]");
		myHCountry.put("760177777413771","[MY=1]");
		myHCountry.put("761777777413700","[GB=1]");
		myHCountry.put("763777777413371","[US=1]");
		myHCountry.put("765777777413771","[US=1]");
		myHCountry.put("770177777413771","[MY=1]");
		myHCountry.put("771777740013700","[IN=1]");
		myHCountry.put("773177777411631","[MM=1]");
		myHCountry.put("773700777413700","[MM=1]");
		myHCountry.put("773757777413471","[FR=1]");
		myHCountry.put("773767777413671","[MY=1]");
		myHCountry.put("773777677413031","[MY=1]");
		myHCountry.put("774000017413771","[MM=1]");
		myHCountry.put("774147775413470","[MY=1]");
		myHCountry.put("774177077413771","[MY=1]");
		myHCountry.put("774177777413671","[MY=1]");
		myHCountry.put("774377776413771","[US=1]");
		myHCountry.put("774377777413771","[MM=1]");
		myHCountry.put("774777477411771","[US=1]");
		myHCountry.put("775777777413371","[IN=1]");
		myHCountry.put("775777777413700","[MY=1]");
		myHCountry.put("776163607010171","[ES=1]");
		myHCountry.put("776177400010371","[ES=1]");
		myHCountry.put("776177400010771","[ES=1]");
		myHCountry.put("776177777013700","[LK=1]");
		myHCountry.put("776177777413771","[MM=1]");
		myHCountry.put("776237777413400","[IN=1]");
		myHCountry.put("776374007413771","[JP=1]");
		myHCountry.put("777003777413771","[US=1]");
		myHCountry.put("777037777413631","[IN=1]");
		myHCountry.put("777176707013771","[IN=1]");
		myHCountry.put("777177777413771","[SA=1]");
		myHCountry.put("777347777413600","[IN=1]");
		myHCountry.put("777347777413671","[TH=1]");
		myHCountry.put("777377777413771","[US=1]");
		myHCountry.put("777537777413371","[VN=1]");
		myHCountry.put("777577777013371","[US=1]");
		myHCountry.put("777577777413071","[VN=1]");
		myHCountry.put("777577777413231","[VN=1]");
		myHCountry.put("777600005413371","[GP=1]");
		myHCountry.put("777600007413771","[BR=1]");
		myHCountry.put("777601776413771","[US=1]");
		myHCountry.put("777617705413371","[BR=1]");
		myHCountry.put("777617777413771","[VN=1]");
		myHCountry.put("777637777410071","[NL=1]");
		myHCountry.put("777637777410171","[US=1]");
		myHCountry.put("777677740013371","[IN=1]");
		myHCountry.put("777677777413431","[FR=1]");
		myHCountry.put("777677777413671","[TH=1]");
		myHCountry.put("777700076413771","[US=1]");
		myHCountry.put("777701777413700","[SR=1]");
		myHCountry.put("777707777413771","[MY=1]");
		myHCountry.put("777717777413671","[MY=1]");
		myHCountry.put("777737707413771","[DZ=1]");
		myHCountry.put("777737767413631","[MY=1]");
		myHCountry.put("777737777410171","[NL=1]");
		myHCountry.put("777737777413671","[GB=1]");
		myHCountry.put("777737777413700","[US=1]");
		myHCountry.put("777740017413771","[IT=1]");
		myHCountry.put("777761777413700","[BD=1]");
		myHCountry.put("777763777413500","[GB=1]");
		myHCountry.put("777763777413771","[IN=1]");
		myHCountry.put("777767477413671","[MX=1]");
		myHCountry.put("777767776413771","[FR=1]");
		myHCountry.put("777771477413671","[SE=1]");
		myHCountry.put("777771677413671","[NL=1]");
		myHCountry.put("777771775413671","[SG=1]");
		myHCountry.put("777771777413671","[MY=1]");
		myHCountry.put("777775007413671","[LK=1]");
		myHCountry.put("777776001413771","[BD=1]");
		myHCountry.put("777776077413771","[MY=1]");
		myHCountry.put("777776775413470","[MY=1]");
		myHCountry.put("777776777413101","[MY=1]");
		myHCountry.put("777776777413771","[LC=1]");
		myHCountry.put("777777003013311","[BD=1]");
		myHCountry.put("777777003413771","[SE=1]");
		myHCountry.put("777777350010371","[BD=1]");
		myHCountry.put("777777376413771","[US=1]");
		myHCountry.put("777777377013771","[US=1]");
		myHCountry.put("777777377413511","[BD=1]");
		myHCountry.put("777777377413711","[IT=1]");
		myHCountry.put("777777401013371","[BD=1]");
		myHCountry.put("777777401413371","[MM=1]");
		myHCountry.put("777777477410001","[ES=1]");
		myHCountry.put("777777477410071","[ES=1]");
		myHCountry.put("777777477413671","[IT=1]");
		myHCountry.put("777777477413701","[SA=1]");
		myHCountry.put("777777477413751","[AU=1]");
		myHCountry.put("777777637413071","[GP=1]");
		myHCountry.put("777777637413771","[US=1]");
		myHCountry.put("777777717413371","[IN=1]");
		myHCountry.put("777777747013371","[BD=1]");
		myHCountry.put("777777757013171","[GB=1]");
		myHCountry.put("777777770413771","[SE=1]");
		myHCountry.put("777777771413470","[SG=1]");
		myHCountry.put("777777773410771","[MY=1]");
		myHCountry.put("777777774013031","[BD=1]");
		myHCountry.put("777777775411031","[MY=1]");
		myHCountry.put("777777775413700","[US=1]");
		myHCountry.put("777777775413771","[FR=1]");
		myHCountry.put("777777776013771","[DK=1]");
		myHCountry.put("777777776413711","[VN=1]");
		myHCountry.put("777777777013071","[AU=1]");
		myHCountry.put("777777777013700","[OM=1]");
		myHCountry.put("777777777410000","[US=1]");
		myHCountry.put("777777777410030","[VN=1]");
		myHCountry.put("777777777410031","[MY=1]");
		myHCountry.put("777777777410700","[IN=1]");
		myHCountry.put("777777777410770","[ID=1]");
		myHCountry.put("777777777412771","[US=1]");
		myHCountry.put("777777777413020","[MM=1]");
		myHCountry.put("777777777413271","[MM=1]");
		myHCountry.put("777777777413301","[US=1]");
		myHCountry.put("777777777413400","[MY=1]");
		myHCountry.put("777777777413571","[US=1]");
		myHCountry.put("777777777413600","[IT=1]");
		myHCountry.put("777777777413611","[MM=1]");
		myHCountry.put("777777777413641","[TH=1]");
		myHCountry.put("777777777413741","[VN=1]");
		myHCountry.put("777777777413761","[ID=1]");
		myHCountry.put("777777777413770","[US=1]");
		myHCountry.put("077777747413771","[OM=1]");
		myHCountry.put("100775747413771","[ZW=1]");
		myHCountry.put("177774747413771","[BD=1]");
		myHCountry.put("300767747413771","[MG=1]");
		myHCountry.put("300777747413771","[MW=1]");
		myHCountry.put("377776757413371","[MY=1]");
		myHCountry.put("377777747413771","[SE=1]");
		myHCountry.put("400037557413771","[IN=1]");
		myHCountry.put("417777757413371","[BD=1]");
		myHCountry.put("470377747413771","[DK=1]");
		myHCountry.put("477777757413671","[IN=1]");
		myHCountry.put("500775747413771","[MZ=1]");
		myHCountry.put("517767757413771","[IN=1]");
		myHCountry.put("537777757413771","[SR=1]");
		myHCountry.put("573775757413771","[BD=1]");
		myHCountry.put("575776347413771","[MY=1]");
		myHCountry.put("577437757413771","[US=1]");
		myHCountry.put("577761757413771","[BD=1]");
		myHCountry.put("577775747413771","[BD=1]");
		myHCountry.put("577775757411371","[BD=1]");
		myHCountry.put("577775757412771","[BD=1]");
		myHCountry.put("577777741413771","[BD=1]");
		myHCountry.put("577777757410371","[BD=1]");
		myHCountry.put("577777757413371","[AU=1]");
		myHCountry.put("600367747413771","[MZ=1]");
		myHCountry.put("677757747413071","[ID=1]");
		myHCountry.put("677777750413371","[BD=1]");
		myHCountry.put("700005747413771","[NL=1]");
		myHCountry.put("700677747413771","[BR=1]");
		myHCountry.put("700774747413771","[MW=1]");
		myHCountry.put("700776747413771","[BR=1]");
		myHCountry.put("700777747410771","[TZ=1]");
		myHCountry.put("701775747413771","[MW=1]");
		myHCountry.put("703777741413771","[ID=1]");
		myHCountry.put("704237746413271","[BD=1]");
		myHCountry.put("704377747413371","[US=1]");
		myHCountry.put("707777757413031","[NL=1]");
		myHCountry.put("717603757410371","[MZ=1]");
		myHCountry.put("717777757413671","[IT=1]");
		myHCountry.put("737675757413371","[OM=1]");
		myHCountry.put("737700357412071","[IN=1]");
		myHCountry.put("737775757413771","[GB=1]");
		myHCountry.put("737777757413151","[GB=1]");
		myHCountry.put("747777747413771","[US=1]");
		myHCountry.put("747777757413771","[FR=1]");
		myHCountry.put("757775747413771","[IN=1]");
		myHCountry.put("757777753413771","[FR=1]");
		myHCountry.put("757777757413371","[SG=1]");
		myHCountry.put("761737747413771","[MY=1]");
		myHCountry.put("767777743413771","[MZ=1]");
		myHCountry.put("767777747413771","[DK=1]");
		myHCountry.put("767777755413371","[BD=1]");
		myHCountry.put("770777747413771","[MZ=1]");
		myHCountry.put("772375747413771","[MY=1]");
		myHCountry.put("773377557413371","[SA=1]");
		myHCountry.put("773773757413371","[BD=1]");
		myHCountry.put("773774745413470","[MY=1]");
		myHCountry.put("773777751413771","[MY=1]");
		myHCountry.put("773777757413771","[IN=1]");
		myHCountry.put("774177757413771","[MY=1]");
		myHCountry.put("775775757413771","[TH=1]");
		myHCountry.put("775777757413771","[MM=1]");
		myHCountry.put("776377747413771","[NL=1]");
		myHCountry.put("776775747413771","[ID=1]");
		myHCountry.put("777347757413771","[BD=1]");
		myHCountry.put("777437757413371","[BD=1]");
		myHCountry.put("777477757413771","[SA=1]");
		myHCountry.put("777576757413771","[GB=1]");
		myHCountry.put("777601757413371","[MZ=1]");
		myHCountry.put("777617757413371","[US=1]");
		myHCountry.put("777635757413371","[MZ=1]");
		myHCountry.put("777672757413771","[IN=1]");
		myHCountry.put("777675757413370","[GB=1]");
		myHCountry.put("777677757412371","[NL=1]");
		myHCountry.put("777677757413171","[BD=1]");
		myHCountry.put("777731757413671","[MY=1]");
		myHCountry.put("777735747413771","[MY=1]");
		myHCountry.put("777736747413771","[MY=1]");
		myHCountry.put("777737747413701","[US=1]");
		myHCountry.put("777737755413271","[MM=1]");
		myHCountry.put("777737757410071","[MM=1]");
		myHCountry.put("777737757412371","[US=1]");
		myHCountry.put("777737757413271","[MM=1]");
		myHCountry.put("777737757413771","[IN=1]");
		myHCountry.put("777741757413371","[MY=1]");
		myHCountry.put("777743757413371","[MM=1]");
		myHCountry.put("777755047413371","[BD=1]");
		myHCountry.put("777755747413671","[ID=1]");
		myHCountry.put("777757747413771","[ID=1]");
		myHCountry.put("777771751413470","[SG=1]");
		myHCountry.put("777775745413771","[NL=1]");
		myHCountry.put("777776657413771","[GB=1]");
		myHCountry.put("777776757412771","[US=1]");
		myHCountry.put("777776757413671","[IN=1]");
		myHCountry.put("777777047413371","[BD=1]");
		myHCountry.put("777777457413771","[MY=1]");
		myHCountry.put("777777557410371","[BD=1]");
		myHCountry.put("777777557413771","[BD=1]");
		myHCountry.put("777777656413371","[BD=1]");
		myHCountry.put("777777741413371","[BD=1]");
		myHCountry.put("777777742413371","[JM=1]");
		myHCountry.put("777777743413311","[US=1]");
		myHCountry.put("777777747411371","[US=1]");
		myHCountry.put("777777747413671","[SA=1]");
		myHCountry.put("777777750413371","[BD=1]");
		myHCountry.put("777777750413400","[SG=1]");
		myHCountry.put("777777750413771","[NL=1]");
		myHCountry.put("777777751413671","[BD=1]");
		myHCountry.put("777777751413771","[SG=1]");
		myHCountry.put("777777752413371","[BD=1]");
		myHCountry.put("777777755413371","[BD=1]");
		myHCountry.put("777777757413350","[GB=1]");
		myHCountry.put("774777750003331","[BD=1]");
		myHCountry.put("777376750003271","[BD=1]");
		myHCountry.put("777767750003271","[GF=1]");
		myHCountry.put("777777750003331","[BD=1]");
		myHCountry.put("077775377413771","[MG=1]");
		myHCountry.put("467777277413771","[DK=1]");
		myHCountry.put("473777377413771","[FR=1]");
		myHCountry.put("477347277413671","[DK=1]");
		myHCountry.put("477357277413771","[ET=1]");
		myHCountry.put("477707377413771","[MG=1]");
		myHCountry.put("477747377413771","[MG=1]");
		myHCountry.put("477757277410771","[MG=1]");
		myHCountry.put("477757377413751","[MG=1]");
		myHCountry.put("477757377413771","[MG=1]");
		myHCountry.put("477767377410771","[FR=1]");
		myHCountry.put("477771377413671","[MG=1]");
		myHCountry.put("477777277410771","[MG=1]");
		myHCountry.put("477777277413671","[DK=1]");
		myHCountry.put("477777357413771","[MG=1]");
		myHCountry.put("477777377013771","[MG=1]");
		myHCountry.put("477777377413731","[MQ=1]");
		myHCountry.put("477777377413741","[MG=1]");
		myHCountry.put("000000074020771","[DK=1]");
		myHCountry.put("000002004020731","[FR=1]");
		myHCountry.put("000047414020771","[ES=1]");
		myHCountry.put("000077774020771","[FR=1]");
		myHCountry.put("003777774020771","[TR=1]");
		myHCountry.put("017777774020771","[AL=1]");
		myHCountry.put("037777774020771","[SE=1]");
		myHCountry.put("077777774020771","[IT=1]");
		myHCountry.put("100000004020771","[US=1]");
		myHCountry.put("157777574020571","[IT=1]");
		myHCountry.put("177777774020731","[TR=1]");
		myHCountry.put("377737774020731","[US=1]");
		myHCountry.put("377774374020771","[AT=1]");
		myHCountry.put("377777774020731","[GM=1]");
		myHCountry.put("476077774020771","[OM=1]");
		myHCountry.put("477700204020771","[MY=1]");
		myHCountry.put("477776774020771","[ID=1]");
		myHCountry.put("477777764020771","[ID=1]");
		myHCountry.put("500000004020771","[BR=1]");
		myHCountry.put("500771744020731","[SE=1]");
		myHCountry.put("557400004020771","[FR=1]");
		myHCountry.put("577767774020671","[FR=1]");
		myHCountry.put("577777634020771","[GW=1]");
		myHCountry.put("577777674020771","[BR=1]");
		myHCountry.put("577777774020731","[BG=1]");
		myHCountry.put("600000004020731","[ES=1]");
		myHCountry.put("600003774020771","[MZ=1]");
		myHCountry.put("600377774020771","[ES=1]");
		myHCountry.put("607777774020771","[PL=1]");
		myHCountry.put("617777774020760","[TR=1]");
		myHCountry.put("700000074020771","[NL=1]");
		myHCountry.put("700003774020771","[FR=1]");
		myHCountry.put("700177774020731","[FR=1]");
		myHCountry.put("717717774020771","[BR=1]");
		myHCountry.put("717737774020731","[CO=1]");
		myHCountry.put("737777404020771","[BR=1]");
		myHCountry.put("737777474020771","[TR=1]");
		myHCountry.put("737777704020771","[BR=1]");
		myHCountry.put("737777764020771","[RU=1]");
		myHCountry.put("737777774020731","[CO=1]");
		myHCountry.put("737777774020751","[ES=1]");
		myHCountry.put("741777774020771","[SE=1]");
		myHCountry.put("747777774020731","[SN=1]");
		myHCountry.put("754777774020621","[BE=1]");
		myHCountry.put("757000004020731","[BG=1]");
		myHCountry.put("757737774020771","[NL=1]");
		myHCountry.put("757777474020771","[LV=1]");
		myHCountry.put("757777774020471","[OM=1]");
		myHCountry.put("757777774020770","[GB=1]");
		myHCountry.put("760000004020771","[US=1]");
		myHCountry.put("767777744020750","[CU=1]");
		myHCountry.put("767777774020571","[OM=1]");
		myHCountry.put("770000704020771","[IT=1]");
		myHCountry.put("770163774020771","[RU=1]");
		myHCountry.put("770377774020771","[FR=1]");
		myHCountry.put("773777674020771","[MA=1]");
		myHCountry.put("774000174020771","[CN=1]");
		myHCountry.put("774007774020771","[US=1]");
		myHCountry.put("774177774020731","[FR=1]");
		myHCountry.put("774177774020771","[FR=1]");
		myHCountry.put("774377774020771","[DE=1]");
		myHCountry.put("774777374020771","[US=1]");
		myHCountry.put("775567774020631","[FR=1]");
		myHCountry.put("775767034020671","[FR=1]");
		myHCountry.put("775767604020651","[GP=1]");
		myHCountry.put("775767774020771","[TR=1]");
		myHCountry.put("776000004020771","[NL=1]");
		myHCountry.put("776077774020671","[IT=1]");
		myHCountry.put("776077774020771","[IT=1]");
		myHCountry.put("776367774020771","[BR=1]");
		myHCountry.put("777003704020771","[FR=1]");
		myHCountry.put("777007774020771","[RU=1]");
		myHCountry.put("777340004020771","[MX=1]");
		myHCountry.put("777356774020661","[BE=1]");
		myHCountry.put("777367774020661","[ES=1]");
		myHCountry.put("777367774020771","[ES=1]");
		myHCountry.put("777377774020621","[ZA=1]");
		myHCountry.put("777400004020771","[SE=1]");
		myHCountry.put("777437774020771","[BR=1]");
		myHCountry.put("777477764020771","[FR=1]");
		myHCountry.put("777477774020731","[FI=1]");
		myHCountry.put("777577774020771","[JM=1]");
		myHCountry.put("777600674020771","[FR=1]");
		myHCountry.put("777637774020731","[CO=1]");
		myHCountry.put("777637774020771","[TR=1]");
		myHCountry.put("777677704020751","[GM=1]");
		myHCountry.put("777731374020771","[US=1]");
		myHCountry.put("777737404020771","[US=1]");
		myHCountry.put("777737754020771","[FR=1]");
		myHCountry.put("777741014020771","[MA=1]");
		myHCountry.put("777741764020771","[TW=1]");
		myHCountry.put("777760004020731","[US=1]");
		myHCountry.put("777760014020771","[BR=1]");
		myHCountry.put("777761004020731","[ZA=1]");
		myHCountry.put("777761004020771","[FR=1]");
		myHCountry.put("777761374020771","[IT=1]");
		myHCountry.put("777763004020771","[FR=1]");
		myHCountry.put("777767004020771","[GM=1]");
		myHCountry.put("777767374020771","[US=1]");
		myHCountry.put("777767604020771","[RO=1]");
		myHCountry.put("777767674020731","[FR=1]");
		myHCountry.put("777767714020771","[AT=1]");
		myHCountry.put("777767754020731","[GP=1]");
		myHCountry.put("777767764020771","[DE=1]");
		myHCountry.put("777767774020571","[AR=1]");
		myHCountry.put("777767774020631","[MG=1]");
		myHCountry.put("777767774020671","[BG=1]");
		myHCountry.put("777770014020771","[AT=1]");
		myHCountry.put("777770374020771","[US=1]");
		myHCountry.put("777773774020771","[ES=1]");
		myHCountry.put("777774004020771","[US=1]");
		myHCountry.put("777774674020771","[DE=1]");
		myHCountry.put("777777014020771","[FR=1]");
		myHCountry.put("777777174020731","[FR=1]");
		myHCountry.put("777777174020771","[FR=1]");
		myHCountry.put("777777204020771","[VE=1]");
		myHCountry.put("777777274020771","[TN=1]");
		myHCountry.put("777777354020771","[SE=1]");
		myHCountry.put("777777364020771","[TN=1]");
		myHCountry.put("777777374020731","[CO=1]");
		myHCountry.put("777777404020731","[DZ=1]");
		myHCountry.put("777777414020731","[ZW=1]");
		myHCountry.put("777777474020731","[AT=1]");
		myHCountry.put("777777644020771","[RU=1]");
		myHCountry.put("777777674020371","[US=1]");
		myHCountry.put("777777734020571","[AT=1]");
		myHCountry.put("777777764020631","[MY=1]");
		myHCountry.put("777777764020740","[IT=1]");
		myHCountry.put("777777774020131","[BE=1]");
		myHCountry.put("777777774020660","[BR=1]");
		myHCountry.put("777777774020700","[US=1]");
		myHCountry.put("777777774020720","[FR=1]");
		myHCountry.put("777777774020730","[GF=1]");
		myHCountry.put("000000004020721","[VN=1]");
		myHCountry.put("000000004320771","[MG=1]");
		myHCountry.put("000000004620731","[US=1]");
		myHCountry.put("000000007620671","[FR=1]");
		myHCountry.put("000000007720701","[BR=1]");
		myHCountry.put("000000377620771","[IT=1]");
		myHCountry.put("000027607020671","[FR=1]");
		myHCountry.put("037637477720771","[TR=1]");
		myHCountry.put("037637777720771","[TR=1]");
		myHCountry.put("037667500020771","[AR=1]");
		myHCountry.put("037667600020771","[FR=1]");
		myHCountry.put("037674777620771","[SE=1]");
		myHCountry.put("037677500020731","[FR=1]");
		myHCountry.put("037677777620771","[IT=1]");
		myHCountry.put("037677777720731","[FR=1]");
		myHCountry.put("037777775720731","[CN=1]");
		myHCountry.put("037777777520771","[US=1]");
		myHCountry.put("037777777720671","[ES=1]");
		myHCountry.put("037777777720731","[IT=1]");
		myHCountry.put("037777777720771","[AU=1]");
		myHCountry.put("077677777720771","[NL=1]");
		myHCountry.put("077777700020771","[SE=1]");
		myHCountry.put("104244001020641","[GR=1]");
		myHCountry.put("177727777720771","[IT=1]");
		myHCountry.put("177737777720771","[IT=1]");
		myHCountry.put("277777607620771","[SE=1]");
		myHCountry.put("357777607720771","[SE=1]");
		myHCountry.put("377377777720771","[TN=1]");
		myHCountry.put("377737377720771","[ET=1]");
		myHCountry.put("377757607620771","[ID=1]");
		myHCountry.put("377767737720771","[AR=1]");
		myHCountry.put("377777037720771","[US=1]");
		myHCountry.put("377777607020771","[BR=1]");
		myHCountry.put("377777607620771","[ID=1]");
		myHCountry.put("377777737720771","[TN=1]");
		myHCountry.put("377777770020731","[CM=1]");
		myHCountry.put("400000000020771","[CN=1]");
		myHCountry.put("400000010020771","[MA=1]");
		myHCountry.put("403777777720771","[GF=1]");
		myHCountry.put("417777777720731","[FI=1]");
		myHCountry.put("417777777720771","[AT=1]");
		myHCountry.put("437677770020771","[AT=1]");
		myHCountry.put("437767777720771","[IN=1]");
		myHCountry.put("437777007720731","[US=1]");
		myHCountry.put("437777777720731","[US=1]");
		myHCountry.put("460376775720470","[MY=1]");
		myHCountry.put("460377777720771","[MY=1]");
		myHCountry.put("473144404620000","[GR=1]");
		myHCountry.put("476377777720771","[US=1]");
		myHCountry.put("477577777720771","[ID=1]");
		myHCountry.put("477677610020771","[NL=1]");
		myHCountry.put("477677777720771","[GR=1]");
		myHCountry.put("477767777720731","[IN=1]");
		myHCountry.put("477775777720771","[SA=1]");
		myHCountry.put("531044224120431","[ET=1]");
		myHCountry.put("567777777020171","[IR=1]");
		myHCountry.put("577003377720771","[TH=1]");
		myHCountry.put("577677777720771","[BR=1]");
		myHCountry.put("577737777520771","[IQ=1]");
		myHCountry.put("577737777720771","[CN=1]");
		myHCountry.put("577743703720471","[BR=1]");
		myHCountry.put("577767607720771","[BR=1]");
		myHCountry.put("577767675720671","[FR=1]");
		myHCountry.put("577767737720771","[VN=1]");
		myHCountry.put("577767745720771","[FR=1]");
		myHCountry.put("577767777320771","[US=1]");
		myHCountry.put("577767777520771","[IQ=1]");
		myHCountry.put("577767777720671","[IT=1]");
		myHCountry.put("577767777720771","[US=1]");
		myHCountry.put("577777477720771","[TH=1]");
		myHCountry.put("577777770020731","[CN=1]");
		myHCountry.put("577777774220731","[IT=1]");
		myHCountry.put("577777777220771","[IT=1]");
		myHCountry.put("577777777520771","[IQ=1]");
		myHCountry.put("577777777720711","[PL=1]");
		myHCountry.put("600000377720751","[SE=1]");
		myHCountry.put("600377777720771","[SE=1]");
		myHCountry.put("603777677520771","[US=1]");
		myHCountry.put("617737400020771","[AT=1]");
		myHCountry.put("617737777720771","[PE=1]");
		myHCountry.put("617777600020771","[CN=1]");
		myHCountry.put("637777737720771","[CO=1]");
		myHCountry.put("677367776020661","[ES=1]");
		myHCountry.put("677737605620000","[BR=1]");
		myHCountry.put("677737607620760","[VE=1]");
		myHCountry.put("677757777720771","[AT=1]");
		myHCountry.put("677777717720771","[BG=1]");
		myHCountry.put("677777770020771","[US=1]");
		myHCountry.put("677777776620571","[ET=1]");
		myHCountry.put("677777777620771","[IT=1]");
		myHCountry.put("677777777720631","[IT=1]");
		myHCountry.put("700000007720771","[US=1]");
		myHCountry.put("700037777720771","[TR=1]");
		myHCountry.put("700777600020771","[MG=1]");
		myHCountry.put("701774777720771","[IN=1]");
		myHCountry.put("717777610020771","[BR=1]");
		myHCountry.put("717777777620611","[AT=1]");
		myHCountry.put("717777777720771","[FR=1]");
		myHCountry.put("736300000020771","[US=1]");
		myHCountry.put("737737770020771","[TW=1]");
		myHCountry.put("737737777720731","[MQ=1]");
		myHCountry.put("737777607720771","[BR=1]");
		myHCountry.put("737777737720771","[AR=1]");
		myHCountry.put("737777742020771","[BR=1]");
		myHCountry.put("737777772020771","[BE=1]");
		myHCountry.put("737777774220771","[FI=1]");
		myHCountry.put("737777775720771","[US=1]");
		myHCountry.put("737777777520771","[ES=1]");
		myHCountry.put("737777777720031","[US=1]");
		myHCountry.put("737777777720671","[AM=1]");
		myHCountry.put("747774077020771","[PE=1]");
		myHCountry.put("747777777720771","[IT=1]");
		myHCountry.put("756377777720771","[FI=1]");
		myHCountry.put("757367607320631","[ZA=1]");
		myHCountry.put("757377704320771","[AL=1]");
		myHCountry.put("757734777720730","[US=1]");
		myHCountry.put("757740005720771","[US=1]");
		myHCountry.put("757761777720771","[US=1]");
		myHCountry.put("757767777720771","[IQ=1]");
		myHCountry.put("757776740020771","[TW=1]");
		myHCountry.put("757777400020771","[NL=1]");
		myHCountry.put("757777607020771","[BR=1]");
		myHCountry.put("757777607720771","[BR=1]");
		myHCountry.put("757777677320771","[DO=1]");
		myHCountry.put("757777741720771","[CN=1]");
		myHCountry.put("757777770020731","[ID=1]");
		myHCountry.put("757777776520250","[ES=1]");
		myHCountry.put("757777776720771","[NL=1]");
		myHCountry.put("757777777020771","[ES=1]");
		myHCountry.put("757777777620771","[AT=1]");
		myHCountry.put("757777777720700","[FR=1]");
		myHCountry.put("757777777720751","[IT=1]");
		myHCountry.put("760000005720771","[US=1]");
		myHCountry.put("760001607720171","[PT=1]");
		myHCountry.put("760017777720771","[BG=1]");
		myHCountry.put("760147777720771","[PY=1]");
		myHCountry.put("760377403720771","[TW=1]");
		myHCountry.put("760777777720771","[BE=1]");
		myHCountry.put("767717601720771","[CN=1]");
		myHCountry.put("767737777720771","[US=1]");
		myHCountry.put("767777570020731","[NL=1]");
		myHCountry.put("767777707720771","[BR=1]");
		myHCountry.put("767777767720771","[CO=1]");
		myHCountry.put("767777770020771","[CN=1]");
		myHCountry.put("767777777620771","[ZA=1]");
		myHCountry.put("767777777720131","[DE=1]");
		myHCountry.put("770000007720771","[FR=1]");
		myHCountry.put("770000740020771","[US=1]");
		myHCountry.put("770000777620731","[IT=1]");
		myHCountry.put("770000777620771","[IT=1]");
		myHCountry.put("770037000020700","[AR=1]");
		myHCountry.put("770377777720071","[AT=1]");
		myHCountry.put("770777777720771","[TW=1]");
		myHCountry.put("771777777720771","[US=1]");
		myHCountry.put("773367777720661","[ES=1]");
		myHCountry.put("773676775720470","[MY=1]");
		myHCountry.put("773736777720771","[AT=1]");
		myHCountry.put("773777377720771","[PE=1]");
		myHCountry.put("774000007720771","[GF=1]");
		myHCountry.put("774000370020611","[VN=1]");
		myHCountry.put("774017770020771","[YE=1]");
		myHCountry.put("774777400020771","[RU=1]");
		myHCountry.put("774777777020771","[PT=1]");
		myHCountry.put("774777777520771","[AT=1]");
		myHCountry.put("775567677720671","[AR=1]");
		myHCountry.put("775737777720771","[AT=1]");
		myHCountry.put("776066663020771","[HN=1]");
		myHCountry.put("776077747720671","[SE=1]");
		myHCountry.put("776137600020771","[MA=1]");
		myHCountry.put("776176600020771","[US=1]");
		myHCountry.put("776177400020171","[PT=1]");
		myHCountry.put("776177400020771","[GF=1]");
		myHCountry.put("776177607720771","[ES=1]");
		myHCountry.put("776375767720771","[IT=1]");
		myHCountry.put("776377740020731","[FR=1]");
		myHCountry.put("776377745720771","[AU=1]");
		myHCountry.put("776377760020771","[DZ=1]");
		myHCountry.put("776377770020771","[IT=1]");
		myHCountry.put("776377775720071","[IT=1]");
		myHCountry.put("776377777720271","[IT=1]");
		myHCountry.put("776767776020771","[GP=1]");
		myHCountry.put("776777777720731","[FI=1]");
		myHCountry.put("777000000020571","[SE=1]");
		myHCountry.put("777000007720771","[NL=1]");
		myHCountry.put("777000020020771","[DE=1]");
		myHCountry.put("777000377720771","[ET=1]");
		myHCountry.put("777001745720771","[MA=1]");
		myHCountry.put("777077037720771","[US=1]");
		myHCountry.put("777077777720771","[ES=1]");
		myHCountry.put("777170777520771","[ES=1]");
		myHCountry.put("777177000020771","[US=1]");
		myHCountry.put("777327777720771","[ES=1]");
		myHCountry.put("777347707720771","[BR=1]");
		myHCountry.put("777347777720671","[DK=1]");
		myHCountry.put("777357777020771","[CO=1]");
		myHCountry.put("777357777720771","[BR=1]");
		myHCountry.put("777367606020661","[ES=1]");
		myHCountry.put("777367776020771","[ES=1]");
		myHCountry.put("777377766320361","[GR=1]");
		myHCountry.put("777377777620731","[ES=1]");
		myHCountry.put("777400004120771","[SE=1]");
		myHCountry.put("777400007720771","[SE=1]");
		myHCountry.put("777400017720771","[CN=1]");
		myHCountry.put("777417760020611","[US=1]");
		myHCountry.put("777477770020771","[IT=1]");
		myHCountry.put("777477777720771","[IT=1]");
		myHCountry.put("777600000020771","[SE=1]");
		myHCountry.put("777607777720771","[BE=1]");
		myHCountry.put("777617770020771","[IT=1]");
		myHCountry.put("777627777720771","[PK=1]");
		myHCountry.put("777637600020771","[CN=1]");
		myHCountry.put("777637777720760","[US=1]");
		myHCountry.put("777640007720731","[CN=1]");
		myHCountry.put("777657777720771","[US=1]");
		myHCountry.put("777677700020771","[ES=1]");
		myHCountry.put("777677757720771","[AT=1]");
		myHCountry.put("777677777720731","[SE=1]");
		myHCountry.put("777701377720771","[SE=1]");
		myHCountry.put("777703600020770","[PY=1]");
		myHCountry.put("777703677720771","[JP=1]");
		myHCountry.put("777703777720771","[TN=1]");
		myHCountry.put("777707737720771","[AT=1]");
		myHCountry.put("777707760020771","[NG=1]");
		myHCountry.put("777707767020771","[ET=1]");
		myHCountry.put("777707777620671","[MY=1]");
		myHCountry.put("777707777720771","[AR=1]");
		myHCountry.put("777717607020771","[IT=1]");
		myHCountry.put("777720000020771","[CN=1]");
		myHCountry.put("777721777720731","[NG=1]");
		myHCountry.put("777724234320771","[NL=1]");
		myHCountry.put("777737177720771","[FR=1]");
		myHCountry.put("777737377720700","[ET=1]");
		myHCountry.put("777737677720731","[US=1]");
		myHCountry.put("777737700020731","[US=1]");
		myHCountry.put("777737740020771","[DE=1]");
		myHCountry.put("777737747720771","[SE=1]");
		myHCountry.put("777737757720771","[CN=1]");
		myHCountry.put("777737776720771","[CN=1]");
		myHCountry.put("777737777620000","[SR=1]");
		myHCountry.put("777737777720671","[ID=1]");
		myHCountry.put("777740000020771","[US=1]");
		myHCountry.put("777740070020771","[CN=1]");
		myHCountry.put("777740177620771","[BR=1]");
		myHCountry.put("777741000020601","[NL=1]");
		myHCountry.put("777741007720731","[FI=1]");
		myHCountry.put("777741177720771","[CO=1]");
		myHCountry.put("777741767720771","[FR=1]");
		myHCountry.put("777741777720671","[BE=1]");
		myHCountry.put("777743777320771","[AT=1]");
		myHCountry.put("777743777720431","[ID=1]");
		myHCountry.put("777747777720671","[HT=1]");
		myHCountry.put("777747777720771","[AR=1]");
		myHCountry.put("777757767720771","[ET=1]");
		myHCountry.put("777757777020771","[CO=1]");
		myHCountry.put("777757777520771","[BE=1]");
		myHCountry.put("777757777720671","[HT=1]");
		myHCountry.put("777760000020611","[MX=1]");
		myHCountry.put("777760037720771","[DE=1]");
		myHCountry.put("777760040020771","[GB=1]");
		myHCountry.put("777760377720771","[US=1]");
		myHCountry.put("777760770020771","[NL=1]");
		myHCountry.put("777761007720771","[HT=1]");
		myHCountry.put("777761077620771","[BR=1]");
		myHCountry.put("777761675720771","[FR=1]");
		myHCountry.put("777763004120771","[FR=1]");
		myHCountry.put("777763670020771","[BR=1]");
		myHCountry.put("777763740020771","[TW=1]");
		myHCountry.put("777763770020731","[FR=1]");
		myHCountry.put("777763770020771","[TW=1]");
		myHCountry.put("777763777720671","[FR=1]");
		myHCountry.put("777764040020771","[AR=1]");
		myHCountry.put("777767037720771","[GF=1]");
		myHCountry.put("777767604320771","[AR=1]");
		myHCountry.put("777767677720771","[ES=1]");
		myHCountry.put("777767717720771","[ID=1]");
		myHCountry.put("777767747720771","[FR=1]");
		myHCountry.put("777767766020761","[SA=1]");
		myHCountry.put("777767770720671","[ID=1]");
		myHCountry.put("777767774720731","[IT=1]");
		myHCountry.put("777767776020771","[SA=1]");
		myHCountry.put("777767777320771","[FR=1]");
		myHCountry.put("777767777620731","[FR=1]");
		myHCountry.put("777767777620771","[EG=1]");
		myHCountry.put("777767777720751","[AT=1]");
		myHCountry.put("777770770020771","[TW=1]");
		myHCountry.put("777771000020771","[NG=1]");
		myHCountry.put("777771004720771","[FR=1]");
		myHCountry.put("777771006020731","[ZM=1]");
		myHCountry.put("777771777720631","[OM=1]");
		myHCountry.put("777773377720771","[FR=1]");
		myHCountry.put("777773770020771","[TW=1]");
		myHCountry.put("777773777720731","[SD=1]");
		myHCountry.put("777774007720771","[US=1]");
		myHCountry.put("777774344120771","[AR=1]");
		myHCountry.put("777774777720771","[TW=1]");
		myHCountry.put("777775377720771","[GF=1]");
		myHCountry.put("777775770020771","[ZW=1]");
		myHCountry.put("777776000120771","[CN=1]");
		myHCountry.put("777776001720771","[CN=1]");
		myHCountry.put("777776774720171","[GB=1]");
		myHCountry.put("777776777720671","[IT=1]");
		myHCountry.put("777777000020731","[US=1]");
		myHCountry.put("777777005720771","[AT=1]");
		myHCountry.put("777777020020671","[IT=1]");
		myHCountry.put("777777037720471","[NL=1]");
		myHCountry.put("777777077720771","[FR=1]");
		myHCountry.put("777777177720731","[FR=1]");
		myHCountry.put("777777263720771","[DE=1]");
		myHCountry.put("777777340020771","[US=1]");
		myHCountry.put("777777357720771","[BS=1]");
		myHCountry.put("777777370020751","[CN=1]");
		myHCountry.put("777777377120771","[SE=1]");
		myHCountry.put("777777377520771","[ET=1]");
		myHCountry.put("777777400020471","[IT=1]");
		myHCountry.put("777777400020731","[FR=1]");
		myHCountry.put("777777404320731","[US=1]");
		myHCountry.put("777777404620771","[BG=1]");
		myHCountry.put("777777414220771","[BR=1]");
		myHCountry.put("777777437720771","[US=1]");
		myHCountry.put("777777560020611","[MX=1]");
		myHCountry.put("777777577720671","[EG=1]");
		myHCountry.put("777777577720731","[ES=1]");
		myHCountry.put("777777600020371","[TW=1]");
		myHCountry.put("777777606020771","[ES=1]");
		myHCountry.put("777777607020731","[FR=1]");
		myHCountry.put("777777637720751","[PT=1]");
		myHCountry.put("777777640020711","[US=1]");
		myHCountry.put("777777660020771","[NL=1]");
		myHCountry.put("777777663520771","[NL=1]");
		myHCountry.put("777777663720771","[MY=1]");
		myHCountry.put("777777666020771","[MW=1]");
		myHCountry.put("777777670020771","[AU=1]");
		myHCountry.put("777777674720771","[FR=1]");
		myHCountry.put("777777700020571","[AT=1]");
		myHCountry.put("777777700020611","[FR=1]");
		myHCountry.put("777777700020731","[RU=1]");
		myHCountry.put("777777702020771","[US=1]");
		myHCountry.put("777777704720771","[US=1]");
		myHCountry.put("777777710020771","[CN=1]");
		myHCountry.put("777777717620731","[FR=1]");
		myHCountry.put("777777717720731","[US=1]");
		myHCountry.put("777777740020731","[US=1]");
		myHCountry.put("777777742120771","[MA=1]");
		myHCountry.put("777777743720751","[CM=1]");
		myHCountry.put("777777743720771","[FR=1]");
		myHCountry.put("777777745720771","[FR=1]");
		myHCountry.put("777777746020771","[BE=1]");
		myHCountry.put("777777747720731","[FI=1]");
		myHCountry.put("777777750020771","[FR=1]");
		myHCountry.put("777777755020771","[US=1]");
		myHCountry.put("777777757620370","[EG=1]");
		myHCountry.put("777777760020751","[CN=1]");
		myHCountry.put("777777760120731","[FR=1]");
		myHCountry.put("777777760120771","[NL=1]");
		myHCountry.put("777777770020751","[US=1]");
		myHCountry.put("777777771720771","[CN=1]");
		myHCountry.put("777777774620771","[IT=1]");
		myHCountry.put("777777775020371","[PE=1]");
		myHCountry.put("777777775020711","[BR=1]");
		myHCountry.put("777777775720471","[PE=1]");
		myHCountry.put("777777776020371","[BR=1]");
		myHCountry.put("777777776020731","[CO=1]");
		myHCountry.put("777777776220771","[BR=1]");
		myHCountry.put("777777777020700","[IT=1]");
		myHCountry.put("777777777620031","[NL=1]");
		myHCountry.put("777777777620760","[IT=1]");
		myHCountry.put("777777777720031","[AT=1]");
		myHCountry.put("777777777720531","[SE=1]");
		myHCountry.put("777777777720600","[SE=1]");
		myHCountry.put("777777777720601","[FR=1]");
		myHCountry.put("777777777720741","[IT=1]");
		myHCountry.put("777777777720761","[NG=1]");
		myHCountry.put("007777607760711","[PT=1]");
		myHCountry.put("037377607760731","[BR=1]");
		myHCountry.put("074367607760700","[VE=1]");
		myHCountry.put("077777607760731","[US=1]");
		myHCountry.put("573737607760700","[BR=1]");
		myHCountry.put("637777607760431","[US=1]");
		myHCountry.put("637777607760731","[HT=1]");
		myHCountry.put("675777607740331","[MZ=1]");
		myHCountry.put("675777607760731","[FR=1]");
		myHCountry.put("677774207760711","[US=1]");
		myHCountry.put("677777207760731","[ZA=1]");
		myHCountry.put("677777607060731","[US=1]");
		myHCountry.put("677777607660001","[IT=1]");
		myHCountry.put("677777607660611","[BR=1]");
		myHCountry.put("677777607660711","[FI=1]");
		myHCountry.put("677777607760701","[ES=1]");
		myHCountry.put("677777607760730","[SE=1]");
		myHCountry.put("737737607760400","[BR=1]");
		myHCountry.put("754337607760700","[CU=1]");
		myHCountry.put("777614007760731","[GB=1]");
		myHCountry.put("777737607760700","[BR=1]");
		myHCountry.put("000777607760771","[HT=1]");
		myHCountry.put("011340607740661","[EG=1]");
		myHCountry.put("037676004360371","[AT=1]");
		myHCountry.put("037777407760731","[PE=1]");
		myHCountry.put("057777607760771","[FR=1]");
		myHCountry.put("077757607760771","[US=1]");
		myHCountry.put("407777607760770","[GE=1]");
		myHCountry.put("417763607760771","[US=1]");
		myHCountry.put("457347607760671","[ZW=1]");
		myHCountry.put("457777407760771","[BR=1]");
		myHCountry.put("477347607760661","[MW=1]");
		myHCountry.put("477767607760771","[IQ=1]");
		myHCountry.put("637677607760771","[FR=1]");
		myHCountry.put("647777607760771","[PT=1]");
		myHCountry.put("657677607760771","[BR=1]");
		myHCountry.put("657777607160771","[US=1]");
		myHCountry.put("657777607360761","[BE=1]");
		myHCountry.put("661777607760771","[PT=1]");
		myHCountry.put("665777607760771","[BR=1]");
		myHCountry.put("674377407760771","[BR=1]");
		myHCountry.put("675567607760671","[PT=1]");
		myHCountry.put("675766607760661","[BR=1]");
		myHCountry.put("676377607760771","[BR=1]");
		myHCountry.put("677347607760771","[MW=1]");
		myHCountry.put("677367607760671","[CO=1]");
		myHCountry.put("677377607740771","[VE=1]");
		myHCountry.put("677467607760771","[AR=1]");
		myHCountry.put("677567607760670","[PT=1]");
		myHCountry.put("677740204060771","[IT=1]");
		myHCountry.put("677747607760771","[AR=1]");
		myHCountry.put("677767607740771","[PT=1]");
		myHCountry.put("677773607760771","[HT=1]");
		myHCountry.put("677777407760771","[BR=1]");
		myHCountry.put("677777604740771","[BR=1]");
		myHCountry.put("677777607760171","[US=1]");
		myHCountry.put("677777607760741","[PL=1]");
		myHCountry.put("677777607760751","[BR=1]");
		myHCountry.put("000777606060771","[MW=1]");
		myHCountry.put("001777606060771","[ZM=1]");
		myHCountry.put("016777606060771","[BE=1]");
		myHCountry.put("057677606060771","[BR=1]");
		myHCountry.put("061777606060671","[ZM=1]");
		myHCountry.put("077347606060661","[ZM=1]");
		myHCountry.put("177757606060771","[IT=1]");
		myHCountry.put("177777606060471","[ZM=1]");
		myHCountry.put("373737606060771","[BE=1]");
		myHCountry.put("377577606060771","[ZM=1]");
		myHCountry.put("377777606060751","[BE=1]");
		myHCountry.put("400177606060771","[MZ=1]");
		myHCountry.put("437777606060771","[TZ=1]");
		myHCountry.put("477777606060771","[ZA=1]");
		myHCountry.put("577347606060631","[ZW=1]");
		myHCountry.put("577677606060771","[US=1]");
		myHCountry.put("577717606060731","[ZW=1]");
		myHCountry.put("577757606060731","[ZW=1]");
		myHCountry.put("577777606060731","[MZ=1]");
		myHCountry.put("600000606060631","[MZ=1]");
		myHCountry.put("601357606060731","[MW=1]");
		myHCountry.put("601777606060731","[ZW=1]");
		myHCountry.put("603777606060771","[ZA=1]");
		myHCountry.put("637777606060771","[ZA=1]");
		myHCountry.put("657777406060771","[BR=1]");
		myHCountry.put("677777606060731","[TZ=1]");
		myHCountry.put("717777606060771","[ZM=1]");
		myHCountry.put("733757606060771","[BE=1]");
		myHCountry.put("737777606060771","[MW=1]");
		myHCountry.put("743417606060671","[ZM=1]");
		myHCountry.put("743777606060731","[ZM=1]");
		myHCountry.put("747777606060731","[ZM=1]");
		myHCountry.put("757757606060771","[ZW=1]");
		myHCountry.put("767777606060731","[ZM=1]");
		myHCountry.put("773777606060731","[ZW=1]");
		myHCountry.put("773777606060771","[BE=1]");
		myHCountry.put("774017606060731","[ZW=1]");
		myHCountry.put("776003606060731","[ZA=1]");
		myHCountry.put("777177606060771","[ZM=1]");
		myHCountry.put("777337606060731","[BE=1]");
		myHCountry.put("777337606060771","[MW=1]");
		myHCountry.put("777357606060571","[MW=1]");
		myHCountry.put("777357606060671","[BE=1]");
		myHCountry.put("777357606060731","[ZW=1]");
		myHCountry.put("777357606060771","[MW=1]");
		myHCountry.put("777367606060361","[TZ=1]");
		myHCountry.put("777367606060661","[ZA=1]");
		myHCountry.put("777377606060671","[ZM=1]");
		myHCountry.put("777377606060731","[BE=1]");
		myHCountry.put("777577606060771","[BE=1]");
		myHCountry.put("777603606060771","[MG=1]");
		myHCountry.put("777717606060771","[ZM=1]");
		myHCountry.put("777737206060731","[ZA=1]");
		myHCountry.put("777757606060771","[BE=1]");
		myHCountry.put("777764606060731","[TZ=1]");
		myHCountry.put("777765606060631","[MZ=1]");
		myHCountry.put("777767606060631","[MZ=1]");
		myHCountry.put("777767606060661","[ZM=1]");
		myHCountry.put("777767606060671","[ZM=1]");
		myHCountry.put("777767606060731","[ZM=1]");
		myHCountry.put("777774206060771","[FR=1]");
		myHCountry.put("777776206060771","[BE=1]");
		myHCountry.put("777777206060731","[GB=1]");
		myHCountry.put("777777206060771","[ZM=1]");
		myHCountry.put("777777606060470","[ZM=1]");
		myHCountry.put("777777606060621","[ZM=1]");
		myHCountry.put("777777606060701","[BE=1]");
		myHCountry.put("777777606060730","[ZM=1]");
		myHCountry.put("376017607760771","[ES=1]");
		myHCountry.put("676013607760771","[FR=1]");
		myHCountry.put("772017607760771","[ES=1]");
		myHCountry.put("776017607160771","[ES=1]");
		myHCountry.put("015707607740071","[VE=1]");
		myHCountry.put("077603405760471","[IT=1]");
		myHCountry.put("077603405760671","[IT=1]");
		myHCountry.put("077737604560771","[VE=1]");
		myHCountry.put("437737607760771","[VE=1]");
		myHCountry.put("455307407640071","[VE=1]");
		myHCountry.put("457727607740771","[AR=1]");
		myHCountry.put("457727607760761","[VE=1]");
		myHCountry.put("477617607760771","[IT=1]");
		myHCountry.put("477737607760751","[SN=1]");
		myHCountry.put("607737607760731","[VE=1]");
		myHCountry.put("637727607760771","[HN=1]");
		myHCountry.put("657636207760771","[US=1]");
		myHCountry.put("667737607760771","[AR=1]");
		myHCountry.put("671527607760671","[PT=1]");
		myHCountry.put("671737607760771","[HT=1]");
		myHCountry.put("675737607760711","[CO=1]");
		myHCountry.put("676737607760771","[BR=1]");
		myHCountry.put("677327607760661","[BR=1]");
		myHCountry.put("677337607660771","[BR=1]");
		myHCountry.put("677400607760671","[HT=1]");
		myHCountry.put("677637607760011","[BR=1]");
		myHCountry.put("677637607760711","[BR=1]");
		myHCountry.put("677707607760671","[MW=1]");
		myHCountry.put("677707607760711","[BR=1]");
		myHCountry.put("677707607760771","[MW=1]");
		myHCountry.put("677721607760271","[VE=1]");
		myHCountry.put("677727607760671","[VE=1]");
		myHCountry.put("677727607760751","[VE=1]");
		myHCountry.put("677730207760771","[BR=1]");
		myHCountry.put("677735607560771","[US=1]");
		myHCountry.put("677737606160731","[VE=1]");
		myHCountry.put("677737606760771","[US=1]");
		myHCountry.put("677737607560171","[VE=1]");
		myHCountry.put("677737607760011","[BR=1]");
		myHCountry.put("677737607760031","[BR=1]");
		myHCountry.put("677737607760351","[BR=1]");
		myHCountry.put("677737607760471","[BR=1]");
		myHCountry.put("677737607760631","[IN=1]");
		myHCountry.put("000000007560731","[MY=1]");
		myHCountry.put("000000007740071","[MA=1]");
		myHCountry.put("000000007740661","[AL=1]");
		myHCountry.put("000000007760331","[US=1]");
		myHCountry.put("000000007760661","[OM=1]");
		myHCountry.put("000000007760701","[IT=1]");
		myHCountry.put("000000007760761","[TZ=1]");
		myHCountry.put("000003607760771","[CO=1]");
		myHCountry.put("300000007760771","[GB=1]");
		myHCountry.put("336177607760771","[US=1]");
		myHCountry.put("376117607760771","[ZA=1]");
		myHCountry.put("376157607760661","[VE=1]");
		myHCountry.put("400000004160771","[TR=1]");
		myHCountry.put("400003607760731","[ES=1]");
		myHCountry.put("400037607760771","[BR=1]");
		myHCountry.put("400040007760771","[CN=1]");
		myHCountry.put("402000007760701","[ES=1]");
		myHCountry.put("416177607760751","[ZA=1]");
		myHCountry.put("536177607760771","[US=1]");
		myHCountry.put("540177607760771","[BE=1]");
		myHCountry.put("560000007760771","[US=1]");
		myHCountry.put("576167605760671","[FR=1]");
		myHCountry.put("576167607740771","[AR=1]");
		myHCountry.put("600000004360601","[FR=1]");
		myHCountry.put("600000004360611","[FR=1]");
		myHCountry.put("600003607760771","[MA=1]");
		myHCountry.put("600174207760771","[US=1]");
		myHCountry.put("600177607760771","[VE=1]");
		myHCountry.put("636167607760771","[MZ=1]");
		myHCountry.put("674037607760771","[PA=1]");
		myHCountry.put("676077607760371","[TZ=1]");
		myHCountry.put("700000007760771","[TR=1]");
		myHCountry.put("700000207760621","[US=1]");
		myHCountry.put("700000207760771","[US=1]");
		myHCountry.put("700027607760731","[HN=1]");
		myHCountry.put("700036007760771","[US=1]");
		myHCountry.put("706137607760771","[BR=1]");
		myHCountry.put("716137607760731","[ES=1]");
		myHCountry.put("720000007760731","[ZA=1]");
		myHCountry.put("720147606560771","[ID=1]");
		myHCountry.put("730001607560771","[BR=1]");
		myHCountry.put("736177607760711","[ES=1]");
		myHCountry.put("736177607760731","[ES=1]");
		myHCountry.put("736177607760741","[US=1]");
		myHCountry.put("740000007560771","[US=1]");
		myHCountry.put("740000607760731","[BR=1]");
		myHCountry.put("740003606760771","[MY=1]");
		myHCountry.put("740007606760771","[MY=1]");
		myHCountry.put("740017607760171","[BE=1]");
		myHCountry.put("760000004160471","[DE=1]");
		myHCountry.put("760000007760731","[US=1]");
		myHCountry.put("760000207560771","[BR=1]");
		myHCountry.put("760017607760771","[MA=1]");
		myHCountry.put("760177607760771","[BR=1]");
		myHCountry.put("762174607760771","[IT=1]");
		myHCountry.put("762177607760771","[ZA=1]");
		myHCountry.put("766137607760771","[BR=1]");
		myHCountry.put("766167607760771","[VE=1]");
		myHCountry.put("766177007760771","[ZA=1]");
		myHCountry.put("770037607760771","[US=1]");
		myHCountry.put("770077407760671","[ES=1]");
		myHCountry.put("770077407760771","[ES=1]");
		myHCountry.put("770077607560771","[US=1]");
		myHCountry.put("770167607760771","[MX=1]");
		myHCountry.put("772177606560771","[PT=1]");
		myHCountry.put("772177607760771","[ES=1]");
		myHCountry.put("774000007760731","[US=1]");
		myHCountry.put("774003607740071","[NG=1]");
		myHCountry.put("774003607760771","[DZ=1]");
		myHCountry.put("774137607560771","[MA=1]");
		myHCountry.put("774147607760271","[BR=1]");
		myHCountry.put("774177007760771","[ZA=1]");
		myHCountry.put("774177607760771","[ES=1]");
		myHCountry.put("776000606760771","[BR=1]");
		myHCountry.put("776021607760771","[BR=1]");
		myHCountry.put("776037606760771","[MA=1]");
		myHCountry.put("776057606560771","[ID=1]");
		myHCountry.put("776077207760771","[SE=1]");
		myHCountry.put("776077606760771","[ID=1]");
		myHCountry.put("776100207740771","[DO=1]");
		myHCountry.put("776103007760771","[ZA=1]");
		myHCountry.put("776123407760771","[ZA=1]");
		myHCountry.put("776137607760371","[IT=1]");
		myHCountry.put("776137607760531","[GB=1]");
		myHCountry.put("776137607760711","[ES=1]");
		myHCountry.put("776141607760731","[PE=1]");
		myHCountry.put("776147607360671","[AR=1]");
		myHCountry.put("776147607760661","[BR=1]");
		myHCountry.put("776147607760671","[VE=1]");
		myHCountry.put("776157007760771","[ZA=1]");
		myHCountry.put("776157607760731","[BR=1]");
		myHCountry.put("776157607760771","[AR=1]");
		myHCountry.put("776167607760071","[ES=1]");
		myHCountry.put("776167607760371","[AR=1]");
		myHCountry.put("776167607760401","[MZ=1]");
		myHCountry.put("776167607760611","[ES=1]");
		myHCountry.put("776167607760711","[HN=1]");
		myHCountry.put("776171607760771","[ES=1]");
		myHCountry.put("776174007760771","[BE=1]");
		myHCountry.put("776174207760771","[ZA=1]");
		myHCountry.put("776174407760771","[ZA=1]");
		myHCountry.put("776176207760740","[US=1]");
		myHCountry.put("776177007760771","[BE=1]");
		myHCountry.put("776177605760771","[FR=1]");
		myHCountry.put("776177606360771","[ZA=1]");
		myHCountry.put("776177607360731","[US=1]");
		myHCountry.put("776177607740711","[MA=1]");
		myHCountry.put("776177607740731","[US=1]");
		myHCountry.put("177777607760731","[MX=1]");
		myHCountry.put("357777607760731","[BR=1]");
		myHCountry.put("377477607760031","[FR=1]");
		myHCountry.put("377777607460731","[BR=1]");
		myHCountry.put("377777607760400","[BR=1]");
		myHCountry.put("555567607740731","[AR=1]");
		myHCountry.put("557367607740231","[IT=1]");
		myHCountry.put("557767607760011","[ES=1]");
		myHCountry.put("577757604040731","[ZW=1]");
		myHCountry.put("577777607740731","[GW=1]");
		myHCountry.put("737377607760731","[BR=1]");
		myHCountry.put("737777607760600","[ES=1]");
		myHCountry.put("737777607760601","[ES=1]");
		myHCountry.put("737777607760731","[MA=1]");
		myHCountry.put("757777607760611","[ES=1]");
		myHCountry.put("760777607760711","[PT=1]");
		myHCountry.put("761773607760731","[AT=1]");
		myHCountry.put("767767607760731","[FI=1]");
		myHCountry.put("767777607760631","[AT=1]");
		myHCountry.put("770777607760731","[IT=1]");
		myHCountry.put("771777607760731","[BR=1]");
		myHCountry.put("773777607760700","[US=1]");
		myHCountry.put("774377607760731","[BR=1]");
		myHCountry.put("776340007760731","[US=1]");
		myHCountry.put("776377607760100","[BR=1]");
		myHCountry.put("777357607760631","[ZW=1]");
		myHCountry.put("777377607760731","[US=1]");
		myHCountry.put("777740007760731","[CN=1]");
		myHCountry.put("777741007760511","[CN=1]");
		myHCountry.put("777741007760601","[ZA=1]");
		myHCountry.put("777747607740331","[BR=1]");
		myHCountry.put("777757607760400","[BD=1]");
		myHCountry.put("777757607760731","[ID=1]");
		myHCountry.put("777760607760700","[BR=1]");
		myHCountry.put("777767607540131","[MX=1]");
		myHCountry.put("777767607760700","[FI=1]");
		myHCountry.put("777767607760730","[ZA=1]");
		myHCountry.put("777771007760731","[NG=1]");
		myHCountry.put("777773607760731","[US=1]");
		myHCountry.put("777774007760731","[GB=1]");
		myHCountry.put("777774607760731","[ZA=1]");
		myHCountry.put("777776407760611","[US=1]");
		myHCountry.put("777777007760631","[IT=1]");
		myHCountry.put("777777007760731","[BR=1]");
		myHCountry.put("777777207760631","[BR=1]");
		myHCountry.put("777777207760711","[US=1]");
		myHCountry.put("777777407760720","[BR=1]");
		myHCountry.put("777777604060631","[GB=1]");
		myHCountry.put("777777607160731","[BR=1]");
		myHCountry.put("777777607360421","[MA=1]");
		myHCountry.put("777777607360731","[DO=1]");
		myHCountry.put("777777607460730","[BR=1]");
		myHCountry.put("777777607540410","[BR=1]");
		myHCountry.put("777777607540731","[BR=1]");
		myHCountry.put("777777607740011","[MX=1]");
		myHCountry.put("777777607760531","[BR=1]");
		myHCountry.put("777777607760601","[IT=1]");
		myHCountry.put("777777607760611","[ES=1]");
		myHCountry.put("777777607760621","[US=1]");
		myHCountry.put("177737607760771","[MM=1]");
		myHCountry.put("377717607760771","[BR=1]");
		myHCountry.put("377727607760771","[IN=1]");
		myHCountry.put("555307407640071","[VE=1]");
		myHCountry.put("577703607760771","[GW=1]");
		myHCountry.put("700623405760471","[ZA=1]");
		myHCountry.put("707727607760771","[VE=1]");
		myHCountry.put("717337607760771","[VE=1]");
		myHCountry.put("717727607760771","[VE=1]");
		myHCountry.put("717737607760771","[VE=1]");
		myHCountry.put("737603407760771","[AT=1]");
		myHCountry.put("743737607760771","[AR=1]");
		myHCountry.put("747737607760771","[US=1]");
		myHCountry.put("756737407760771","[US=1]");
		myHCountry.put("757703607760771","[CO=1]");
		myHCountry.put("757717607760771","[VE=1]");
		myHCountry.put("757737607760760","[AR=1]");
		myHCountry.put("767717607760771","[US=1]");
		myHCountry.put("767737607760771","[BR=1]");
		myHCountry.put("771603405760471","[ZA=1]");
		myHCountry.put("775337607760661","[VE=1]");
		myHCountry.put("775700007760771","[US=1]");
		myHCountry.put("776337407760771","[SA=1]");
		myHCountry.put("777017607760771","[US=1]");
		myHCountry.put("777037607760771","[BR=1]");
		myHCountry.put("777317607760671","[VE=1]");
		myHCountry.put("777327607760671","[ES=1]");
		myHCountry.put("777327607760771","[AR=1]");
		myHCountry.put("777337607660771","[BR=1]");
		myHCountry.put("777337607760771","[BR=1]");
		myHCountry.put("777401607760771","[RU=1]");
		myHCountry.put("777407607760771","[GW=1]");
		myHCountry.put("777537607760771","[VE=1]");
		myHCountry.put("777600007760771","[ES=1]");
		myHCountry.put("777600207760771","[BR=1]");
		myHCountry.put("777602004360471","[ZA=1]");
		myHCountry.put("777603404060471","[BR=1]");
		myHCountry.put("777603405760771","[ID=1]");
		myHCountry.put("777623405760771","[ZA=1]");
		myHCountry.put("777700607740771","[BR=1]");
		myHCountry.put("777701007760771","[EG=1]");
		myHCountry.put("777701607760770","[NL=1]");
		myHCountry.put("777703607760771","[GW=1]");
		myHCountry.put("777707607760771","[BR=1]");
		myHCountry.put("777717605760771","[BR=1]");
		myHCountry.put("777721007760771","[FR=1]");
		myHCountry.put("777727607160771","[VE=1]");
		myHCountry.put("777727607760761","[ET=1]");
		myHCountry.put("777727607760770","[AR=1]");
		myHCountry.put("777730004160771","[GB=1]");
		myHCountry.put("777735207760770","[VE=1]");
		myHCountry.put("777735207760771","[VE=1]");
		myHCountry.put("777737007760771","[US=1]");
		myHCountry.put("777737606360771","[BE=1]");
		myHCountry.put("777737607660471","[ES=1]");
		myHCountry.put("777737607660771","[CM=1]");
		myHCountry.put("777737607740771","[IT=1]");
		myHCountry.put("777737607760371","[US=1]");
		myHCountry.put("777737607760671","[AR=1]");
		myHCountry.put("777737607760760","[BR=1]");
		myHCountry.put("001777207560771","[FR=1]");
		myHCountry.put("017777607560371","[BR=1]");
		myHCountry.put("077777607560771","[DE=1]");
		myHCountry.put("177777607560740","[US=1]");
		myHCountry.put("377637607560771","[GF=1]");
		myHCountry.put("557767607560771","[BR=1]");
		myHCountry.put("557777607560771","[IT=1]");
		myHCountry.put("577327606560711","[TZ=1]");
		myHCountry.put("577747607560771","[BR=1]");
		myHCountry.put("607777607560771","[BR=1]");
		myHCountry.put("677747607560671","[NA=1]");
		myHCountry.put("701777607560771","[ES=1]");
		myHCountry.put("737377607560171","[BR=1]");
		myHCountry.put("737777607560731","[PE=1]");
		myHCountry.put("757777607560000","[PT=1]");
		myHCountry.put("763777607560760","[PY=1]");
		myHCountry.put("770777607560771","[ZM=1]");
		myHCountry.put("774337606560771","[BR=1]");
		myHCountry.put("775767607560671","[BR=1]");
		myHCountry.put("776377607560771","[BR=1]");
		myHCountry.put("776777607560771","[BR=1]");
		myHCountry.put("777077607560771","[PK=1]");
		myHCountry.put("777337607560771","[ES=1]");
		myHCountry.put("777347606560671","[CU=1]");
		myHCountry.put("777357607560731","[BR=1]");
		myHCountry.put("777367607560771","[BR=1]");
		myHCountry.put("777376607560771","[BR=1]");
		myHCountry.put("777377607560761","[BR=1]");
		myHCountry.put("777700007560771","[BR=1]");
		myHCountry.put("777701607560731","[ES=1]");
		myHCountry.put("777707607560771","[BR=1]");
		myHCountry.put("777717607560771","[BR=1]");
		myHCountry.put("777737607560570","[BR=1]");
		myHCountry.put("777737607560760","[BR=1]");
		myHCountry.put("777747606560771","[ID=1]");
		myHCountry.put("777747607560671","[BR=1]");
		myHCountry.put("777761007560771","[BR=1]");
		myHCountry.put("777767407560731","[AR=1]");
		myHCountry.put("777767607560671","[BR=1]");
		myHCountry.put("777767607560731","[BR=1]");
		myHCountry.put("777775607560771","[BR=1]");
		myHCountry.put("777776007560771","[CO=1]");
		myHCountry.put("777776407560771","[BR=1]");
		myHCountry.put("777776607560771","[US=1]");
		myHCountry.put("777777007560771","[FR=1]");
		myHCountry.put("777777407560730","[ES=1]");
		myHCountry.put("777777606560731","[PE=1]");
		myHCountry.put("777777606560771","[ID=1]");
		myHCountry.put("777777607560471","[BR=1]");
		myHCountry.put("777777607560531","[MX=1]");
		myHCountry.put("777777607560570","[BR=1]");
		myHCountry.put("777777607560661","[BR=1]");
		myHCountry.put("777777607560721","[BR=1]");
		myHCountry.put("777777607560751","[BR=1]");
		myHCountry.put("777777607560760","[BR=1]");
		myHCountry.put("657777406760771","[BR=1]");
		myHCountry.put("677767606740371","[MX=1]");
		myHCountry.put("677767606760771","[BR=1]");
		myHCountry.put("717707606740771","[ID=1]");
		myHCountry.put("717747606760771","[ID=1]");
		myHCountry.put("747757606760771","[ID=1]");
		myHCountry.put("757777606760771","[BR=1]");
		myHCountry.put("771000006760771","[ZA=1]");
		myHCountry.put("776377606760711","[AU=1]");
		myHCountry.put("777377606760771","[IT=1]");
		myHCountry.put("777537606760771","[OM=1]");
		myHCountry.put("777717606760770","[ID=1]");
		myHCountry.put("777757206760771","[ID=1]");
		myHCountry.put("777757606760611","[ID=1]");
		myHCountry.put("777757606760771","[ID=1]");
		myHCountry.put("777777606760571","[GF=1]");
		myHCountry.put("777777606760631","[ID=1]");
		myHCountry.put("173767607760771","[MG=1]");
		myHCountry.put("173776607760771","[MG=1]");
		myHCountry.put("175477607760771","[MQ=1]");
		myHCountry.put("177177607760771","[OM=1]");
		myHCountry.put("337777607760771","[TW=1]");
		myHCountry.put("357777607760571","[MA=1]");
		myHCountry.put("357777607760771","[BR=1]");
		myHCountry.put("377767605660771","[FR=1]");
		myHCountry.put("377777605660771","[FR=1]");
		myHCountry.put("377777607660771","[FR=1]");
		myHCountry.put("377777607740371","[BR=1]");
		myHCountry.put("377777607760570","[BR=1]");
		myHCountry.put("377777607760571","[FR=1]");
		myHCountry.put("515047607760661","[RU=1]");
		myHCountry.put("517767607760661","[BR=1]");
		myHCountry.put("543777607760771","[US=1]");
		myHCountry.put("555047607760661","[RU=1]");
		myHCountry.put("557347607760761","[PY=1]");
		myHCountry.put("557747607740661","[IQ=1]");
		myHCountry.put("557767607540771","[US=1]");
		myHCountry.put("557777607760771","[US=1]");
		myHCountry.put("575777607760771","[SA=1]");
		myHCountry.put("577743607760771","[IT=1]");
		myHCountry.put("577747607760661","[VE=1]");
		myHCountry.put("577747607760771","[GW=1]");
		myHCountry.put("577777407760771","[BR=1]");
		myHCountry.put("577777607760571","[FR=1]");
		myHCountry.put("577777607760671","[CU=1]");
		myHCountry.put("703777607760771","[VE=1]");
		myHCountry.put("707777607760771","[US=1]");
		myHCountry.put("711041007740661","[RU=1]");
		myHCountry.put("711054206360771","[BE=1]");
		myHCountry.put("711762007760771","[EG=1]");
		myHCountry.put("737377607760671","[VE=1]");
		myHCountry.put("737377607760761","[MA=1]");
		myHCountry.put("737741007760771","[TN=1]");
		myHCountry.put("737777207760771","[BR=1]");
		myHCountry.put("737777606160771","[US=1]");
		myHCountry.put("741761007760771","[FR=1]");
		myHCountry.put("741777604760771","[ES=1]");
		myHCountry.put("743767607760671","[AR=1]");
		myHCountry.put("757740007760771","[RU=1]");
		myHCountry.put("757743607760771","[VE=1]");
		myHCountry.put("757767607760771","[BR=1]");
		myHCountry.put("757777607360340","[BR=1]");
		myHCountry.put("757777607540771","[BR=1]");
		myHCountry.put("757777607760760","[BR=1]");
		myHCountry.put("761777607760371","[CO=1]");
		myHCountry.put("767777605760671","[IT=1]");
		myHCountry.put("767777605760771","[CO=1]");
		myHCountry.put("770777607760771","[HN=1]");
		myHCountry.put("771077607760771","[PY=1]");
		myHCountry.put("771567607760671","[PT=1]");
		myHCountry.put("771767607760771","[BE=1]");
		myHCountry.put("773756607760771","[AR=1]");
		myHCountry.put("773777207760771","[US=1]");
		myHCountry.put("774767607760771","[IT=1]");
		myHCountry.put("774777607760771","[CO=1]");
		myHCountry.put("775167607760671","[CO=1]");
		myHCountry.put("775567607760771","[MZ=1]");
		myHCountry.put("776340007760771","[US=1]");
		myHCountry.put("776361607760771","[IT=1]");
		myHCountry.put("776367407760771","[IT=1]");
		myHCountry.put("776377607760570","[BR=1]");
		myHCountry.put("776377607760571","[BR=1]");
		myHCountry.put("777140007760771","[PL=1]");
		myHCountry.put("777157607760771","[BR=1]");
		myHCountry.put("777167607760671","[AR=1]");
		myHCountry.put("777340007760771","[RU=1]");
		myHCountry.put("777357607760671","[CO=1]");
		myHCountry.put("777377607760671","[PY=1]");
		myHCountry.put("777377607760761","[PY=1]");
		myHCountry.put("777467607760771","[RU=1]");
		myHCountry.put("777567607760771","[PT=1]");
		myHCountry.put("777640007760671","[US=1]");
		myHCountry.put("777663405760471","[ZA=1]");
		myHCountry.put("777667607760771","[ES=1]");
		myHCountry.put("777740007760371","[BE=1]");
		myHCountry.put("777740007760571","[CN=1]");
		myHCountry.put("777740207760771","[US=1]");
		myHCountry.put("777741207760771","[GR=1]");
		myHCountry.put("777747606040671","[ID=1]");
		myHCountry.put("777757607060671","[VE=1]");
		myHCountry.put("777757607760770","[ID=1]");
		myHCountry.put("777761007760651","[RU=1]");
		myHCountry.put("777764607760771","[ZA=1]");
		myHCountry.put("777766607760771","[IQ=1]");
		myHCountry.put("777767407760770","[EG=1]");
		myHCountry.put("777767607760671","[MX=1]");
		myHCountry.put("777767607760760","[BR=1]");
		myHCountry.put("777770606360771","[BR=1]");
		myHCountry.put("777771007760771","[TN=1]");
		myHCountry.put("777776007760471","[US=1]");
		myHCountry.put("777777005760471","[MA=1]");
		myHCountry.put("777777007660771","[FR=1]");
		myHCountry.put("777777007760671","[FR=1]");
		myHCountry.put("777777204760771","[TN=1]");
		myHCountry.put("777777205760771","[MX=1]");
		myHCountry.put("777777207760751","[BR=1]");
		myHCountry.put("777777405760571","[PL=1]");
		myHCountry.put("777777407060771","[BR=1]");
		myHCountry.put("777777604060771","[ZA=1]");
		myHCountry.put("777777605660771","[BR=1]");
		myHCountry.put("777777606340771","[US=1]");
		myHCountry.put("777777607440771","[BR=1]");
		myHCountry.put("777777607460471","[BR=1]");
		myHCountry.put("777777607540570","[BR=1]");
		myHCountry.put("777777607660671","[FR=1]");
		myHCountry.put("777777607740171","[US=1]");
		myHCountry.put("777777607740271","[BR=1]");
		myHCountry.put("777777607760271","[IT=1]");
		myHCountry.put("400037377613771","[NO=1]");
		myHCountry.put("400037777433771","[SA=1]");
		myHCountry.put("400037777513771","[GB=1]");
		myHCountry.put("474377777773771","[MY=1]");
		myHCountry.put("477777277613771","[NO=1]");
		myHCountry.put("477777777433071","[IN=1]");
		myHCountry.put("477777777453071","[SA=1]");
		myHCountry.put("535745777772771","[ET=1]");
		myHCountry.put("577777776773771","[IN=1]");
		myHCountry.put("577777777433771","[IN=1]");
		myHCountry.put("674000001613771","[US=1]");
		myHCountry.put("677727655772671","[BR=1]");
		myHCountry.put("677737477573771","[GB=1]");
		myHCountry.put("677760077753271","[ID=1]");
		myHCountry.put("677776777773671","[IN=1]");
		myHCountry.put("677777277473771","[SA=1]");
		myHCountry.put("677777777773771","[MM=1]");
		myHCountry.put("700775747473771","[ZW=1]");
		myHCountry.put("701357777773771","[MW=1]");
		myHCountry.put("703777707773771","[SA=1]");
		myHCountry.put("703777747773771","[IN=1]");
		myHCountry.put("704175417672360","[IT=1]");
		myHCountry.put("714115777672360","[IT=1]");
		myHCountry.put("714175754072360","[IT=1]");
		myHCountry.put("717367777513621","[ID=1]");
		myHCountry.put("733737477753771","[JP=1]");
		myHCountry.put("737777272772271","[ET=1]");
		myHCountry.put("741777777613771","[NO=1]");
		myHCountry.put("747777777553771","[SA=1]");
		myHCountry.put("754135757672360","[IT=1]");
		myHCountry.put("754175617672360","[IT=1]");
		myHCountry.put("757777777713771","[ZA=1]");
		myHCountry.put("760377757773771","[BD=1]");
		myHCountry.put("770637777773771","[LK=1]");
		myHCountry.put("771777767613770","[IN=1]");
		myHCountry.put("775357777772671","[ET=1]");
		myHCountry.put("776177607773771","[US=1]");
		myHCountry.put("776763777773660","[EG=1]");
		myHCountry.put("777347777433671","[IN=1]");
		myHCountry.put("777377777573771","[BR=1]");
		myHCountry.put("777417740773771","[IN=1]");
		myHCountry.put("777603777713771","[SA=1]");
		myHCountry.put("777661043333771","[IN=1]");
		myHCountry.put("777737357773771","[FR=1]");
		myHCountry.put("777737747753771","[FR=1]");
		myHCountry.put("777737776613771","[NO=1]");
		myHCountry.put("777757577772771","[ET=1]");
		myHCountry.put("777761607473771","[TZ=1]");
		myHCountry.put("777767777553771","[TZ=1]");
		myHCountry.put("777767777773771","[MG=1]");
		myHCountry.put("777773777513761","[ID=1]");
		myHCountry.put("777777003473771","[NL=1]");
		myHCountry.put("777777007113771","[PK=1]");
		myHCountry.put("777777407773771","[NL=1]");
		myHCountry.put("777777477753771","[US=1]");
		myHCountry.put("777777577733751","[SA=1]");
		myHCountry.put("777777577773771","[SA=1]");
		myHCountry.put("777777743773771","[JM=1]");
		myHCountry.put("777777744533771","[SD=1]");
		myHCountry.put("777777744773771","[SD=1]");
		myHCountry.put("777777747473771","[ZW=1]");
		myHCountry.put("777777747613371","[US=1]");
		myHCountry.put("777777747773771","[SD=1]");
		myHCountry.put("777777754613771","[NO=1]");
		myHCountry.put("777777757733771","[MM=1]");
		myHCountry.put("777777775733771","[SD=1]");
		myHCountry.put("777777776613771","[NO=1]");
		myHCountry.put("777777776773771","[SD=1]");
		myHCountry.put("777777777433771","[IN=1]");
		myHCountry.put("777777777513731","[GB=1]");
		myHCountry.put("777777777573731","[IN=1]");
		myHCountry.put("777777777613071","[NO=1]");
		myHCountry.put("777777777613771","[NO=1]");
		myHCountry.put("777777777633771","[BR=1]");
		myHCountry.put("777777777733671","[ID=1]");
		myHCountry.put("777777777753731","[VN=1]");
		myHCountry.put("777777777753771","[IN=1]");
		myHCountry.put("777777777772771","[IN=1]");
		myHCountry.put("777777777773071","[SA=1]");
		myHCountry.put("000000000363771","[IN=1]");
		myHCountry.put("000000007763771","[RU=1]");
		myHCountry.put("103777777563771","[IN=1]");
		myHCountry.put("300036777763771","[US=1]");
		myHCountry.put("377377777723771","[TR=1]");
		myHCountry.put("437777647763771","[IT=1]");
		myHCountry.put("437777777423771","[IN=1]");
		myHCountry.put("477737607763771","[IN=1]");
		myHCountry.put("477777777423771","[IN=1]");
		myHCountry.put("477777777763771","[CN=1]");
		myHCountry.put("515347476763771","[IN=1]");
		myHCountry.put("517777637763771","[CN=1]");
		myHCountry.put("535743737723771","[VN=1]");
		myHCountry.put("557367677763671","[GW=1]");
		myHCountry.put("575667635162631","[FR=1]");
		myHCountry.put("577347777763661","[IN=1]");
		myHCountry.put("577737777763761","[CN=1]");
		myHCountry.put("577757601763771","[CN=1]");
		myHCountry.put("577767776763771","[VN=1]");
		myHCountry.put("577767777763731","[TW=1]");
		myHCountry.put("577777676763731","[AU=1]");
		myHCountry.put("577777676763771","[VN=1]");
		myHCountry.put("577777677763771","[AR=1]");
		myHCountry.put("600776777763771","[NP=1]");
		myHCountry.put("617774077763771","[MZ=1]");
		myHCountry.put("637767607763771","[US=1]");
		myHCountry.put("637775777763770","[ZW=1]");
		myHCountry.put("637777747762771","[UG=1]");
		myHCountry.put("670000777763771","[GE=1]");
		myHCountry.put("677377607763771","[ES=1]");
		myHCountry.put("677737607763771","[MM=1]");
		myHCountry.put("677767637563771","[IQ=1]");
		myHCountry.put("677775607763771","[CN=1]");
		myHCountry.put("677777607763771","[US=1]");
		myHCountry.put("700036777763771","[US=1]");
		myHCountry.put("700066777763771","[ZA=1]");
		myHCountry.put("702777707763771","[ZA=1]");
		myHCountry.put("703377777763771","[EG=1]");
		myHCountry.put("703777740423171","[PK=1]");
		myHCountry.put("703777747763371","[IN=1]");
		myHCountry.put("703777777763771","[IN=1]");
		myHCountry.put("707777777763771","[RU=1]");
		myHCountry.put("710157777763771","[DK=1]");
		myHCountry.put("711054776763771","[IN=1]");
		myHCountry.put("713777777563771","[IN=1]");
		myHCountry.put("713777777763171","[IQ=1]");
		myHCountry.put("730076777763771","[ZA=1]");
		myHCountry.put("730777777723600","[NL=1]");
		myHCountry.put("731044606763771","[MX=1]");
		myHCountry.put("737777740023771","[ZA=1]");
		myHCountry.put("737777756763771","[IN=1]");
		myHCountry.put("737777777723771","[CN=1]");
		myHCountry.put("737777777763771","[DE=1]");
		myHCountry.put("741737777763771","[JP=1]");
		myHCountry.put("743777740023671","[IN=1]");
		myHCountry.put("743777740023771","[IN=1]");
		myHCountry.put("756177607763771","[ZA=1]");
		myHCountry.put("757377377763771","[EG=1]");
		myHCountry.put("757767637763771","[IQ=1]");
		myHCountry.put("757767777723671","[IN=1]");
		myHCountry.put("757777740763771","[IQ=1]");
		myHCountry.put("757777777763771","[IN=1]");
		myHCountry.put("763777777763771","[SA=1]");
		myHCountry.put("767777777722071","[SA=1]");
		myHCountry.put("767777777763771","[OM=1]");
		myHCountry.put("770377777762771","[CN=1]");
		myHCountry.put("770377777763771","[CN=1]");
		myHCountry.put("771077777363771","[IN=1]");
		myHCountry.put("773377777763671","[AR=1]");
		myHCountry.put("773777607763771","[NL=1]");
		myHCountry.put("773777757563771","[SA=1]");
		myHCountry.put("774000007763771","[US=1]");
		myHCountry.put("774375776563771","[KR=1]");
		myHCountry.put("774777737423731","[RU=1]");
		myHCountry.put("775377757763771","[ES=1]");
		myHCountry.put("775743003763771","[RU=1]");
		myHCountry.put("776017607762771","[NL=1]");
		myHCountry.put("776177606063771","[TZ=1]");
		myHCountry.put("776177607763731","[CO=1]");
		myHCountry.put("776777647763771","[ZA=1]");
		myHCountry.put("777347777763671","[US=1]");
		myHCountry.put("777347777763771","[CN=1]");
		myHCountry.put("777367677763771","[ZA=1]");
		myHCountry.put("777367757763771","[ES=1]");
		myHCountry.put("777376677763771","[ZA=1]");
		myHCountry.put("777377346763771","[ZA=1]");
		myHCountry.put("777637777723771","[DE=1]");
		myHCountry.put("777677003423771","[RU=1]");
		myHCountry.put("777677637763771","[RU=1]");
		myHCountry.put("777707677763771","[IN=1]");
		myHCountry.put("777741007763771","[CN=1]");
		myHCountry.put("777747637763771","[CN=1]");
		myHCountry.put("777747763762771","[NG=1]");
		myHCountry.put("777754007763771","[GE=1]");
		myHCountry.put("777757677763771","[CN=1]");
		myHCountry.put("777760003763771","[RU=1]");
		myHCountry.put("777761007763771","[RU=1]");
		myHCountry.put("777761777763771","[CN=1]");
		myHCountry.put("777763007763771","[RU=1]");
		myHCountry.put("777767607763771","[RU=1]");
		myHCountry.put("777767677763771","[IN=1]");
		myHCountry.put("777767743763771","[FR=1]");
		myHCountry.put("777767777722771","[IQ=1]");
		myHCountry.put("777767777762771","[TZ=1]");
		myHCountry.put("777776761763771","[TW=1]");
		myHCountry.put("777776771763771","[TW=1]");
		myHCountry.put("777777037723771","[HN=1]");
		myHCountry.put("777777577363771","[TH=1]");
		myHCountry.put("777777604063771","[ZA=1]");
		myHCountry.put("777777607563771","[BE=1]");
		myHCountry.put("777777617762771","[BR=1]");
		myHCountry.put("777777637763771","[GW=1]");
		myHCountry.put("777777674063771","[ZW=1]");
		myHCountry.put("777777677723600","[US=1]");
		myHCountry.put("777777700023771","[ZA=1]");
		myHCountry.put("777777704022731","[IN=1]");
		myHCountry.put("777777740023771","[CN=1]");
		myHCountry.put("777777747762771","[UG=1]");
		myHCountry.put("777777757762771","[NG=1]");
		myHCountry.put("777777767763771","[VN=1]");
		myHCountry.put("777777770023771","[US=1]");
		myHCountry.put("777777773763771","[RU=1]");
		myHCountry.put("777777775723771","[MY=1]");
		myHCountry.put("777777777423771","[MG=1]");
		myHCountry.put("777777777463731","[MZ=1]");
		myHCountry.put("777777777623771","[MQ=1]");
		myHCountry.put("777777777722771","[CN=1]");
		myHCountry.put("777777777763571","[US=1]");
		myHCountry.put("777777777763731","[SD=1]");
		myHCountry.put("777777777763751","[TH=1]");
		myHCountry.put("777777777763761","[MY=1]");
		myHCountry.put("077777777770771","[ES=1]");
		myHCountry.put("203067777670671","[EG=1]");
		myHCountry.put("300067777670531","[ES=1]");
		myHCountry.put("376157767730771","[VE=1]");
		myHCountry.put("477377777730771","[SD=1]");
		myHCountry.put("520132417570761","[BR=1]");
		myHCountry.put("534357377750771","[ET=1]");
		myHCountry.put("575367577770671","[ES=1]");
		myHCountry.put("577347477750671","[ET=1]");
		myHCountry.put("577757777750771","[BR=1]");
		myHCountry.put("577773777430671","[IR=1]");
		myHCountry.put("677737607770731","[ES=1]");
		myHCountry.put("677737607770771","[ES=1]");
		myHCountry.put("677777607670771","[ES=1]");
		myHCountry.put("677777757770771","[ES=1]");
		myHCountry.put("677777777730671","[ES=1]");
		myHCountry.put("677777777730770","[ES=1]");
		myHCountry.put("677777777770771","[ES=1]");
		myHCountry.put("702003347770471","[ES=1]");
		myHCountry.put("703767577750751","[SA=1]");
		myHCountry.put("737737777770731","[IT=1]");
		myHCountry.put("737777607030771","[ZW=1]");
		myHCountry.put("747740007770771","[IT=1]");
		myHCountry.put("756177607770771","[ES=1]");
		myHCountry.put("757547477770771","[ET=1]");
		myHCountry.put("767776777770771","[GB=1]");
		myHCountry.put("773767777730771","[ES=1]");
		myHCountry.put("774777777430771","[IT=1]");
		myHCountry.put("775777777770771","[ES=1]");
		myHCountry.put("776077607770771","[ES=1]");
		myHCountry.put("776177607770670","[ES=1]");
		myHCountry.put("776177637570771","[ES=1]");
		myHCountry.put("776177777770771","[ES=1]");
		myHCountry.put("776377774030771","[ES=1]");
		myHCountry.put("776377777770771","[ES=1]");
		myHCountry.put("776777777770730","[SD=1]");
		myHCountry.put("777000177770771","[NO=1]");
		myHCountry.put("777357677770761","[US=1]");
		myHCountry.put("777677677770771","[BR=1]");
		myHCountry.put("777677777770771","[BR=1]");
		myHCountry.put("777707607770771","[ES=1]");
		myHCountry.put("777721607570731","[ES=1]");
		myHCountry.put("777721607570771","[ES=1]");
		myHCountry.put("777735777450771","[ES=1]");
		myHCountry.put("777736077570771","[PA=1]");
		myHCountry.put("777737557770771","[ES=1]");
		myHCountry.put("777737607770771","[ES=1]");
		myHCountry.put("777741607730771","[ES=1]");
		myHCountry.put("777767477770771","[ET=1]");
		myHCountry.put("777767607570771","[NG=1]");
		myHCountry.put("777776077570771","[PA=1]");
		myHCountry.put("777776777650771","[US=1]");
		myHCountry.put("777777343770771","[SD=1]");
		myHCountry.put("777777477670400","[ES=1]");
		myHCountry.put("777777557770671","[ES=1]");
		myHCountry.put("777777577770771","[ES=1]");
		myHCountry.put("777777604030771","[ES=1]");
		myHCountry.put("777777605770471","[ES=1]");
		myHCountry.put("777777607670771","[ES=1]");
		myHCountry.put("777777707770771","[BD=1]");
		myHCountry.put("777777740070771","[ES=1]");
		myHCountry.put("777777744030771","[ES=1]");
		myHCountry.put("777777747770471","[ES=1]");
		myHCountry.put("777777754030670","[ES=1]");
		myHCountry.put("777777754030771","[ES=1]");
		myHCountry.put("777777760070771","[ES=1]");
		myHCountry.put("777777764130771","[ES=1]");
		myHCountry.put("777777767770731","[IT=1]");
		myHCountry.put("777777774030771","[ES=1]");
		myHCountry.put("777777774050771","[TN=1]");
		myHCountry.put("777777777530771","[ES=1]");
		myHCountry.put("777777777730731","[ES=1]");
		myHCountry.put("777777777750771","[ES=1]");
		myHCountry.put("777777777770471","[ES=1]");
		myHCountry.put("777777777770711","[IN=1]");
		myHCountry.put("000000000100200","[GR=1]");
		myHCountry.put("000000000140000","[TW=1]");
		myHCountry.put("000000024000000","[GB=1]");
		myHCountry.put("020000000000400","[MY=1]");
		myHCountry.put("070000000003600","[GB=1]");
		myHCountry.put("074000007777600","[GB=1]");
		myHCountry.put("074000026037600","[FR=1]");
		myHCountry.put("074000030000400","[GB=1]");
		myHCountry.put("074000032000400","[GB=1]");
		myHCountry.put("074000037377600","[GB=1]");
		myHCountry.put("000000004000000","[GR=1]");
		myHCountry.put("000000004367600","[FR=1]");
		myHCountry.put("000000007377600","[IT=1]");
		myHCountry.put("000000007776200","[BE=1]");
		myHCountry.put("006363777760771","[IT=1]");
		myHCountry.put("016337777760731","[FR=1]");
		myHCountry.put("076357777760771","[BR=1]");
		myHCountry.put("076377737760771","[PL=1]");
		myHCountry.put("176377777760771","[IT=1]");
		myHCountry.put("276377776060731","[CA=1]");
		myHCountry.put("376367777760771","[FR=1]");
		myHCountry.put("376377777760731","[IT=1]");
		myHCountry.put("476367737760731","[AL=1]");
		myHCountry.put("516347777760771","[EG=1]");
		myHCountry.put("566377777760771","[MX=1]");
		myHCountry.put("576360777760771","[DE=1]");
		myHCountry.put("576367377660771","[BE=1]");
		myHCountry.put("576377703740731","[CO=1]");
		myHCountry.put("576377767760771","[AU=1]");
		myHCountry.put("616377707560771","[BR=1]");
		myHCountry.put("616377777760771","[FR=1]");
		myHCountry.put("636377777760731","[ZA=1]");
		myHCountry.put("676267737760771","[AR=1]");
		myHCountry.put("676277757760771","[AR=1]");
		myHCountry.put("676377700140771","[MX=1]");
		myHCountry.put("676377717760771","[ES=1]");
		myHCountry.put("676377737760711","[IT=1]");
		myHCountry.put("676377737760731","[AT=1]");
		myHCountry.put("676377776060731","[CA=1]");
		myHCountry.put("676377777760731","[GB=1]");
		myHCountry.put("702371001760771","[ZA=1]");
		myHCountry.put("702377777760771","[EG=1]");
		myHCountry.put("706377777760731","[ES=1]");
		myHCountry.put("716376777760771","[TR=1]");
		myHCountry.put("736377737560731","[BE=1]");
		myHCountry.put("736377737760731","[IT=1]");
		myHCountry.put("746377775760771","[FI=1]");
		myHCountry.put("756367777760771","[FR=1]");
		myHCountry.put("766377777760731","[IT=1]");
		myHCountry.put("772202777760771","[PL=1]");
		myHCountry.put("772377777760771","[US=1]");
		myHCountry.put("776200000060771","[RU=1]");
		myHCountry.put("776247777760571","[MX=1]");
		myHCountry.put("776303177760771","[FR=1]");
		myHCountry.put("776307777760770","[VE=1]");
		myHCountry.put("776307777760771","[ES=1]");
		myHCountry.put("776317777760771","[BE=1]");
		myHCountry.put("776333760360771","[IT=1]");
		myHCountry.put("776333777760771","[GR=1]");
		myHCountry.put("776335777760771","[IT=1]");
		myHCountry.put("776337403760771","[BR=1]");
		myHCountry.put("776337677760771","[IT=1]");
		myHCountry.put("776337700760771","[BR=1]");
		myHCountry.put("776337777360571","[BE=1]");
		myHCountry.put("776337777660731","[IT=1]");
		myHCountry.put("776347757760771","[CO=1]");
		myHCountry.put("776347777760661","[BR=1]");
		myHCountry.put("776357777760571","[MX=1]");
		myHCountry.put("776357777760771","[US=1]");
		myHCountry.put("776361741760771","[GP=1]");
		myHCountry.put("776363677760771","[BE=1]");
		myHCountry.put("776363770040731","[BR=1]");
		myHCountry.put("776367777660771","[FR=1]");
		myHCountry.put("776373777760731","[TR=1]");
		myHCountry.put("776376037760771","[IT=1]");
		myHCountry.put("776376137760771","[IT=1]");
		myHCountry.put("776376177760771","[NL=1]");
		myHCountry.put("776376737560731","[IT=1]");
		myHCountry.put("776376777760731","[MX=1]");
		myHCountry.put("776377077760771","[US=1]");
		myHCountry.put("776377177760771","[BR=1]");
		myHCountry.put("776377377760631","[ES=1]");
		myHCountry.put("776377577760731","[CA=1]");
		myHCountry.put("776377637760771","[ES=1]");
		myHCountry.put("776377700360771","[ZA=1]");
		myHCountry.put("776377701740731","[CO=1]");
		myHCountry.put("776377733760771","[ZA=1]");
		myHCountry.put("776377737760711","[BG=1]");
		myHCountry.put("776377740760771","[IT=1]");
		myHCountry.put("776377741760771","[SE=1]");
		myHCountry.put("776377747760771","[IT=1]");
		myHCountry.put("776377771760771","[FR=1]");
		myHCountry.put("776377776360771","[OM=1]");
		myHCountry.put("776377777740000","[SE=1]");
		myHCountry.put("776377777760741","[IT=1]");
		myHCountry.put("776377777760760","[US=1]");
		myHCountry.put("177727001760731","[GF=1]");
		myHCountry.put("377737777760731","[BR=1]");
		myHCountry.put("437737777760731","[FR=1]");
		myHCountry.put("537737777760731","[FI=1]");
		myHCountry.put("577737377760771","[CN=1]");
		myHCountry.put("577737757760771","[CN=1]");
		myHCountry.put("577777377760731","[CN=1]");
		myHCountry.put("607737777760731","[BR=1]");
		myHCountry.put("617737377760601","[US=1]");
		myHCountry.put("633774477760730","[TZ=1]");
		myHCountry.put("677777377760731","[NL=1]");
		myHCountry.put("706003357760671","[IT=1]");
		myHCountry.put("707737307760431","[NL=1]");
		myHCountry.put("737727777760731","[IT=1]");
		myHCountry.put("747737757760771","[FR=1]");
		myHCountry.put("747737777760731","[US=1]");
		myHCountry.put("761737777760731","[FR=1]");
		myHCountry.put("770737777760731","[ET=1]");
		myHCountry.put("773737777760731","[HT=1]");
		myHCountry.put("774337757660771","[VE=1]");
		myHCountry.put("776737377760700","[ET=1]");
		myHCountry.put("777137757760771","[DE=1]");
		myHCountry.put("777177377760731","[CN=1]");
		myHCountry.put("777327777760731","[OM=1]");
		myHCountry.put("777377677760671","[PE=1]");
		myHCountry.put("777723777760731","[ID=1]");
		myHCountry.put("777727757760771","[ET=1]");
		myHCountry.put("777727775540731","[ET=1]");
		myHCountry.put("777731777760731","[MX=1]");
		myHCountry.put("777733777760731","[PE=1]");
		myHCountry.put("777737347760771","[HN=1]");
		myHCountry.put("777737377460771","[CN=1]");
		myHCountry.put("777737663560731","[OM=1]");
		myHCountry.put("777737676740131","[ID=1]");
		myHCountry.put("777737717760731","[IT=1]");
		myHCountry.put("777737760160731","[IT=1]");
		myHCountry.put("777737774760731","[IT=1]");
		myHCountry.put("777737777460731","[IT=1]");
		myHCountry.put("777737777560721","[BR=1]");
		myHCountry.put("777737777660731","[FR=1]");
		myHCountry.put("777737777740331","[BE=1]");
		myHCountry.put("777737777740731","[US=1]");
		myHCountry.put("777737777760131","[BG=1]");
		myHCountry.put("777771350060771","[SE=1]");
		myHCountry.put("777777357760700","[IT=1]");
		myHCountry.put("777777357760771","[US=1]");
		myHCountry.put("000000000060771","[AT=1]");
		myHCountry.put("000000000160771","[BG=1]");
		myHCountry.put("000000000360771","[TR=1]");
		myHCountry.put("000000000760771","[IT=1]");
		myHCountry.put("000000001560771","[FR=1]");
		myHCountry.put("000000001760771","[MZ=1]");
		myHCountry.put("000000017740771","[AU=1]");
		myHCountry.put("000000076760631","[FR=1]");
		myHCountry.put("000000077560771","[PY=1]");
		myHCountry.put("000000176760771","[SE=1]");
		myHCountry.put("000003777760771","[CA=1]");
		myHCountry.put("000007760060771","[AT=1]");
		myHCountry.put("000037777760771","[MZ=1]");
		myHCountry.put("000177717760771","[SE=1]");
		myHCountry.put("000177777760771","[ES=1]");
		myHCountry.put("000367436560771","[TZ=1]");
		myHCountry.put("000377601560771","[FR=1]");
		myHCountry.put("000377777560771","[PY=1]");
		myHCountry.put("000377777760771","[KZ=1]");
		myHCountry.put("000667777760771","[HT=1]");
		myHCountry.put("000777774560460","[TZ=1]");
		myHCountry.put("000777777760771","[IT=1]");
		myHCountry.put("001741760340071","[IN=1]");
		myHCountry.put("001777777760771","[CI=1]");
		myHCountry.put("003774777760771","[NL=1]");
		myHCountry.put("007777777760771","[FR=1]");
		myHCountry.put("011203777760671","[EG=1]");
		myHCountry.put("015747600160760","[IN=1]");
		myHCountry.put("017467576140771","[ID=1]");
		myHCountry.put("017677777760771","[FR=1]");
		myHCountry.put("017703777760771","[US=1]");
		myHCountry.put("017774001560371","[BR=1]");
		myHCountry.put("017775001560371","[BR=1]");
		myHCountry.put("017777417760771","[AR=1]");
		myHCountry.put("017777777760760","[FR=1]");
		myHCountry.put("037636377760771","[TR=1]");
		myHCountry.put("037637037760771","[PS=1]");
		myHCountry.put("037637717760771","[SE=1]");
		myHCountry.put("037640367760771","[IT=1]");
		myHCountry.put("037677707760771","[TR=1]");
		myHCountry.put("037767777760471","[DE=1]");
		myHCountry.put("037777770060771","[US=1]");
		myHCountry.put("076777737760771","[ET=1]");
		myHCountry.put("077307637740771","[BE=1]");
		myHCountry.put("077767777760771","[FR=1]");
		myHCountry.put("077777774360771","[TR=1]");
		myHCountry.put("077777777560771","[SE=1]");
		myHCountry.put("077777777760031","[US=1]");
		myHCountry.put("115347576360661","[IN=1]");
		myHCountry.put("117777777560771","[IN=1]");
		myHCountry.put("137347677560631","[IN=1]");
		myHCountry.put("137637777760771","[IR=1]");
		myHCountry.put("153777777760771","[IT=1]");
		myHCountry.put("155747437740771","[AL=1]");
		myHCountry.put("177757677760771","[IT=1]");
		myHCountry.put("177760603760771","[HT=1]");
		myHCountry.put("177767623760661","[AL=1]");
		myHCountry.put("177767737740771","[AL=1]");
		myHCountry.put("177767737760771","[GF=1]");
		myHCountry.put("177776003760771","[MQ=1]");
		myHCountry.put("177777617760771","[GE=1]");
		myHCountry.put("177777637760771","[MX=1]");
		myHCountry.put("177777677760171","[US=1]");
		myHCountry.put("177777737760771","[PL=1]");
		myHCountry.put("177777777760601","[PT=1]");
		myHCountry.put("200017777760771","[FR=1]");
		myHCountry.put("200037777760771","[FR=1]");
		myHCountry.put("203777763760771","[TR=1]");
		myHCountry.put("277757577760771","[AR=1]");
		myHCountry.put("277777603560771","[BR=1]");
		myHCountry.put("300077777760771","[GB=1]");
		myHCountry.put("300777663660711","[PY=1]");
		myHCountry.put("317777777760771","[TR=1]");
		myHCountry.put("337637777760771","[SA=1]");
		myHCountry.put("357776377760771","[FR=1]");
		myHCountry.put("357777601560711","[BR=1]");
		myHCountry.put("357777777760771","[FR=1]");
		myHCountry.put("363777600560771","[PK=1]");
		myHCountry.put("373767744760771","[SA=1]");
		myHCountry.put("373767777760771","[SA=1]");
		myHCountry.put("374377602760771","[NG=1]");
		myHCountry.put("375367423560671","[SA=1]");
		myHCountry.put("376177600360771","[HN=1]");
		myHCountry.put("376776337760771","[DK=1]");
		myHCountry.put("377367767760771","[ES=1]");
		myHCountry.put("377677777760771","[SE=1]");
		myHCountry.put("377703777760771","[ID=1]");
		myHCountry.put("377707777760771","[AR=1]");
		myHCountry.put("377740777760771","[US=1]");
		myHCountry.put("377743774360771","[CN=1]");
		myHCountry.put("377757777760771","[BE=1]");
		myHCountry.put("377767677760771","[TR=1]");
		myHCountry.put("377767777760761","[AT=1]");
		myHCountry.put("377773775760771","[SA=1]");
		myHCountry.put("377774077560771","[VE=1]");
		myHCountry.put("377777637760771","[BR=1]");
		myHCountry.put("377777677760671","[VE=1]");
		myHCountry.put("377777677760771","[BR=1]");
		myHCountry.put("377777707760771","[BR=1]");
		myHCountry.put("377777737560771","[FR=1]");
		myHCountry.put("377777763660771","[TR=1]");
		myHCountry.put("377777770060771","[FR=1]");
		myHCountry.put("377777775760771","[FR=1]");
		myHCountry.put("377777776760771","[CN=1]");
		myHCountry.put("377777777740071","[CN=1]");
		myHCountry.put("377777777740771","[DE=1]");
		myHCountry.put("377777777760601","[BR=1]");
		myHCountry.put("400000000360771","[NL=1]");
		myHCountry.put("400001777760771","[BG=1]");
		myHCountry.put("400067777760671","[MX=1]");
		myHCountry.put("400177777760771","[US=1]");
		myHCountry.put("407760077660771","[NL=1]");
		myHCountry.put("415347434640261","[IT=1]");
		myHCountry.put("417177777760771","[ES=1]");
		myHCountry.put("417777677760771","[BR=1]");
		myHCountry.put("417777777660771","[FR=1]");
		myHCountry.put("417777777760700","[GP=1]");
		myHCountry.put("417777777760771","[FR=1]");
		myHCountry.put("437767777760771","[EG=1]");
		myHCountry.put("437777037760771","[EG=1]");
		myHCountry.put("437777677760771","[FR=1]");
		myHCountry.put("437777777760011","[JP=1]");
		myHCountry.put("437777777760601","[GB=1]");
		myHCountry.put("457767621740771","[OM=1]");
		myHCountry.put("457777777760771","[CN=1]");
		myHCountry.put("467777777760471","[BR=1]");
		myHCountry.put("467777777760711","[VN=1]");
		myHCountry.put("471777777760631","[VN=1]");
		myHCountry.put("477067777760771","[FR=1]");
		myHCountry.put("477077777760771","[FR=1]");
		myHCountry.put("477617677760771","[CN=1]");
		myHCountry.put("477777577760771","[BR=1]");
		myHCountry.put("477777637760771","[IT=1]");
		myHCountry.put("477777767760771","[NG=1]");
		myHCountry.put("477777777760011","[JP=1]");
		myHCountry.put("477777777760631","[VN=1]");
		myHCountry.put("500044674740671","[IN=1]");
		myHCountry.put("500757677760571","[CU=1]");
		myHCountry.put("511367734160600","[IT=1]");
		myHCountry.put("511760047760371","[IN=1]");
		myHCountry.put("514347776560761","[EG=1]");
		myHCountry.put("515047636760661","[RU=1]");
		myHCountry.put("515247400640261","[IT=1]");
		myHCountry.put("515247634160661","[IN=1]");
		myHCountry.put("515347420040261","[IT=1]");
		myHCountry.put("515347436640261","[IT=1]");
		myHCountry.put("517347636060661","[MW=1]");
		myHCountry.put("517367637760771","[MX=1]");
		myHCountry.put("517757777760771","[IT=1]");
		myHCountry.put("517767677560771","[IN=1]");
		myHCountry.put("517777637760771","[BR=1]");
		myHCountry.put("530027637560600","[NL=1]");
		myHCountry.put("531255676760771","[ZA=1]");
		myHCountry.put("534307637760771","[EG=1]");
		myHCountry.put("535767777560671","[VN=1]");
		myHCountry.put("537637777760771","[IQ=1]");
		myHCountry.put("537777777760711","[IT=1]");
		myHCountry.put("541777777760771","[IN=1]");
		myHCountry.put("555147437740771","[AL=1]");
		myHCountry.put("555307437740771","[AL=1]");
		myHCountry.put("555347435640271","[BR=1]");
		myHCountry.put("555357637740661","[AL=1]");
		myHCountry.put("555567777740771","[AR=1]");
		myHCountry.put("555767777740771","[AR=1]");
		myHCountry.put("557163777760671","[ZW=1]");
		myHCountry.put("557347777760661","[CO=1]");
		myHCountry.put("557367600740771","[TZ=1]");
		myHCountry.put("557767677760771","[BR=1]");
		myHCountry.put("557767703760771","[BR=1]");
		myHCountry.put("557767763660771","[US=1]");
		myHCountry.put("557767777740611","[IN=1]");
		myHCountry.put("557777777740771","[IN=1]");
		myHCountry.put("557777777760771","[MX=1]");
		myHCountry.put("560003777560770","[IT=1]");
		myHCountry.put("560777777560631","[SA=1]");
		myHCountry.put("570177677740671","[GW=1]");
		myHCountry.put("571347677760671","[IN=1]");
		myHCountry.put("571777777760771","[IT=1]");
		myHCountry.put("573777577760771","[ET=1]");
		myHCountry.put("574777777760771","[CN=1]");
		myHCountry.put("575167777760671","[MY=1]");
		myHCountry.put("575207737740771","[ES=1]");
		myHCountry.put("575347774760661","[IN=1]");
		myHCountry.put("575367777560460","[TZ=1]");
		myHCountry.put("575767737760671","[IQ=1]");
		myHCountry.put("575767777760771","[BR=1]");
		myHCountry.put("576177417760771","[ZA=1]");
		myHCountry.put("577000177760771","[ET=1]");
		myHCountry.put("577347777760671","[DK=1]");
		myHCountry.put("577367603740661","[BR=1]");
		myHCountry.put("577577777760711","[ES=1]");
		myHCountry.put("577603777760700","[IT=1]");
		myHCountry.put("577637417560771","[BR=1]");
		myHCountry.put("577667777760771","[FR=1]");
		myHCountry.put("577677777760700","[IT=1]");
		myHCountry.put("577700777760771","[US=1]");
		myHCountry.put("577701777760771","[SN=1]");
		myHCountry.put("577707777760671","[GF=1]");
		myHCountry.put("577717417560771","[BR=1]");
		myHCountry.put("577717777760771","[CN=1]");
		myHCountry.put("577740517740771","[GW=1]");
		myHCountry.put("577743677660671","[EG=1]");
		myHCountry.put("577743777760771","[US=1]");
		myHCountry.put("577747437740771","[AL=1]");
		myHCountry.put("577757677760671","[IN=1]");
		myHCountry.put("577757677760771","[CN=1]");
		myHCountry.put("577767076640651","[US=1]");
		myHCountry.put("577767600760771","[IQ=1]");
		myHCountry.put("577767637760771","[IT=1]");
		myHCountry.put("577767663760771","[AR=1]");
		myHCountry.put("577767675760771","[FR=1]");
		myHCountry.put("577767676760631","[IN=1]");
		myHCountry.put("577767677760671","[BR=1]");
		myHCountry.put("577767775760571","[IN=1]");
		myHCountry.put("577767777760600","[AR=1]");
		myHCountry.put("577775775760771","[IN=1]");
		myHCountry.put("577776377760601","[GB=1]");
		myHCountry.put("577777102060771","[IT=1]");
		myHCountry.put("577777177760771","[FR=1]");
		myHCountry.put("577777437740771","[AL=1]");
		myHCountry.put("577777437760771","[EG=1]");
		myHCountry.put("577777603760771","[ZW=1]");
		myHCountry.put("577777637760771","[BR=1]");
		myHCountry.put("577777663560711","[IN=1]");
		myHCountry.put("577777677760771","[US=1]");
		myHCountry.put("577777707760771","[NL=1]");
		myHCountry.put("577777774160771","[GB=1]");
		myHCountry.put("577777776360771","[FR=1]");
		myHCountry.put("577777777060771","[US=1]");
		myHCountry.put("577777777660471","[ZA=1]");
		myHCountry.put("577777777760471","[BR=1]");
		myHCountry.put("577777777760600","[IT=1]");
		myHCountry.put("577777777760621","[IN=1]");
		myHCountry.put("600000377760771","[IT=1]");
		myHCountry.put("600001777760771","[MY=1]");
		myHCountry.put("600017777760771","[US=1]");
		myHCountry.put("600167700060771","[IT=1]");
		myHCountry.put("600177777760751","[FR=1]");
		myHCountry.put("600377777760771","[TZ=1]");
		myHCountry.put("601777677760771","[NL=1]");
		myHCountry.put("603777777760771","[SE=1]");
		myHCountry.put("607677677560771","[BR=1]");
		myHCountry.put("607774077560771","[FI=1]");
		myHCountry.put("607777637760771","[US=1]");
		myHCountry.put("607777777740771","[GR=1]");
		myHCountry.put("607777777760771","[US=1]");
		myHCountry.put("617400003760771","[ES=1]");
		myHCountry.put("617774077760601","[MU=1]");
		myHCountry.put("637603747760771","[ID=1]");
		myHCountry.put("637637777760771","[AT=1]");
		myHCountry.put("637703777760571","[AU=1]");
		myHCountry.put("637740000060771","[FR=1]");
		myHCountry.put("637757637760771","[US=1]");
		myHCountry.put("637777577360771","[BR=1]");
		myHCountry.put("637777601560771","[FR=1]");
		myHCountry.put("637777637760031","[HN=1]");
		myHCountry.put("637777707760771","[ES=1]");
		myHCountry.put("657777777760671","[NL=1]");
		myHCountry.put("660000000060771","[ES=1]");
		myHCountry.put("660777777660771","[CN=1]");
		myHCountry.put("667777776360771","[CN=1]");
		myHCountry.put("667777777760471","[BR=1]");
		myHCountry.put("670000000160771","[PE=1]");
		myHCountry.put("676037777740771","[FR=1]");
		myHCountry.put("677377747760771","[MW=1]");
		myHCountry.put("677477777760771","[CN=1]");
		myHCountry.put("677677417760771","[MZ=1]");
		myHCountry.put("677717777760771","[BR=1]");
		myHCountry.put("677761777760771","[SA=1]");
		myHCountry.put("677767617760771","[VE=1]");
		myHCountry.put("677767776160771","[ES=1]");
		myHCountry.put("677767777760071","[US=1]");
		myHCountry.put("677776377760571","[AL=1]");
		myHCountry.put("677777037760771","[FR=1]");
		myHCountry.put("677777400060571","[US=1]");
		myHCountry.put("677777423560771","[US=1]");
		myHCountry.put("677777601760771","[BR=1]");
		myHCountry.put("677777617760371","[BR=1]");
		myHCountry.put("677777637760671","[IT=1]");
		myHCountry.put("677777703760771","[BR=1]");
		myHCountry.put("677777707760371","[ZM=1]");
		myHCountry.put("677777735740771","[IT=1]");
		myHCountry.put("677777771760771","[US=1]");
		myHCountry.put("677777773760700","[CN=1]");
		myHCountry.put("677777775760771","[US=1]");
		myHCountry.put("677777777360371","[SE=1]");
		myHCountry.put("677777777760411","[IT=1]");
		myHCountry.put("677777777760700","[NL=1]");
		myHCountry.put("700000077740771","[CU=1]");
		myHCountry.put("700017770360771","[BR=1]");
		myHCountry.put("700027177760771","[GF=1]");
		myHCountry.put("700037717560600","[IN=1]");
		myHCountry.put("700037776360771","[US=1]");
		myHCountry.put("700037777560771","[ES=1]");
		myHCountry.put("700060777760771","[ZA=1]");
		myHCountry.put("700074777760771","[BD=1]");
		myHCountry.put("700075717760700","[GP=1]");
		myHCountry.put("700075777760771","[BE=1]");
		myHCountry.put("700077777760700","[SA=1]");
		myHCountry.put("700177777760771","[BR=1]");
		myHCountry.put("700377777760771","[TR=1]");
		myHCountry.put("701767774760071","[PK=1]");
		myHCountry.put("703767777760771","[AR=1]");
		myHCountry.put("703777747760371","[PK=1]");
		myHCountry.put("706637777760771","[ES=1]");
		myHCountry.put("706777777760771","[IT=1]");
		myHCountry.put("707777777760571","[IT=1]");
		myHCountry.put("707777777760711","[PL=1]");
		myHCountry.put("711041003760661","[RU=1]");
		myHCountry.put("715767777760771","[BR=1]");
		myHCountry.put("716000003760771","[PL=1]");
		myHCountry.put("717357776760771","[BD=1]");
		myHCountry.put("717757777760371","[BR=1]");
		myHCountry.put("717777777660771","[BG=1]");
		myHCountry.put("717777777760601","[GM=1]");
		myHCountry.put("731047436540661","[EG=1]");
		myHCountry.put("733357777760770","[BR=1]");
		myHCountry.put("733777623760771","[GB=1]");
		myHCountry.put("733777777060770","[SA=1]");
		myHCountry.put("733777777060771","[SA=1]");
		myHCountry.put("736177777760771","[ZM=1]");
		myHCountry.put("737377776760771","[BR=1]");
		myHCountry.put("737637417760771","[AT=1]");
		myHCountry.put("737677777560611","[BR=1]");
		myHCountry.put("737753777760771","[IR=1]");
		myHCountry.put("737765637760771","[BR=1]");
		myHCountry.put("737767401760630","[ZA=1]");
		myHCountry.put("737767777740771","[ID=1]");
		myHCountry.put("737767777760771","[GW=1]");
		myHCountry.put("737775777760771","[GR=1]");
		myHCountry.put("737777637760771","[US=1]");
		myHCountry.put("737777777660771","[YE=1]");
		myHCountry.put("737777777740771","[TR=1]");
		myHCountry.put("737777777760031","[MX=1]");
		myHCountry.put("737777777760371","[RU=1]");
		myHCountry.put("737777777760571","[US=1]");
		myHCountry.put("737777777760631","[US=1]");
		myHCountry.put("740000021760771","[CN=1]");
		myHCountry.put("740000370360771","[IT=1]");
		myHCountry.put("740037777760700","[TN=1]");
		myHCountry.put("740037777760771","[FR=1]");
		myHCountry.put("740377777740000","[SE=1]");
		myHCountry.put("740377777760771","[CN=1]");
		myHCountry.put("740777776360771","[TR=1]");
		myHCountry.put("740777777760700","[PL=1]");
		myHCountry.put("741407677560771","[JP=1]");
		myHCountry.put("741407777760771","[JP=1]");
		myHCountry.put("741427777760771","[JP=1]");
		myHCountry.put("741703577760771","[BR=1]");
		myHCountry.put("741703775760771","[BR=1]");
		myHCountry.put("741757706060771","[BE=1]");
		myHCountry.put("741765047560771","[HN=1]");
		myHCountry.put("743777417740771","[BJ=1]");
		myHCountry.put("743777700060771","[OM=1]");
		myHCountry.put("747740017760771","[IT=1]");
		myHCountry.put("747777647560771","[BR=1]");
		myHCountry.put("747777717760771","[BR=1]");
		myHCountry.put("747777737760771","[VE=1]");
		myHCountry.put("747777770060771","[RU=1]");
		myHCountry.put("750037747760471","[US=1]");
		myHCountry.put("751767777760771","[BR=1]");
		myHCountry.put("753347777760771","[EG=1]");
		myHCountry.put("753767776760771","[VE=1]");
		myHCountry.put("756171777760771","[IT=1]");
		myHCountry.put("756777777760771","[IT=1]");
		myHCountry.put("757177777760771","[FR=1]");
		myHCountry.put("757367777760771","[ES=1]");
		myHCountry.put("757400037760771","[IT=1]");
		myHCountry.put("757600377760771","[FR=1]");
		myHCountry.put("757703777760771","[MY=1]");
		myHCountry.put("757717637760771","[HN=1]");
		myHCountry.put("757740777760771","[PE=1]");
		myHCountry.put("757747667760771","[BR=1]");
		myHCountry.put("757767777740771","[BR=1]");
		myHCountry.put("757767777760671","[OM=1]");
		myHCountry.put("757767777760700","[FR=1]");
		myHCountry.put("757775777760771","[IT=1]");
		myHCountry.put("757777001760771","[FR=1]");
		myHCountry.put("757777037760771","[BE=1]");
		myHCountry.put("757777077760771","[IT=1]");
		myHCountry.put("757777616760631","[SA=1]");
		myHCountry.put("757777717760671","[IT=1]");
		myHCountry.put("757777767760631","[MM=1]");
		myHCountry.put("757777777360771","[MA=1]");
		myHCountry.put("757777777660771","[AR=1]");
		myHCountry.put("757777777740771","[IT=1]");
		myHCountry.put("757777777760631","[FR=1]");
		myHCountry.put("757777777760751","[US=1]");
		myHCountry.put("757777777760760","[IT=1]");
		myHCountry.put("760000000060751","[US=1]");
		myHCountry.put("760000000340371","[TR=1]");
		myHCountry.put("760000000760771","[ES=1]");
		myHCountry.put("760000060160771","[US=1]");
		myHCountry.put("760000077760771","[US=1]");
		myHCountry.put("760000777760771","[MX=1]");
		myHCountry.put("760002001760771","[FR=1]");
		myHCountry.put("760177777760771","[PY=1]");
		myHCountry.put("760377637760701","[AR=1]");
		myHCountry.put("763740777760771","[PE=1]");
		myHCountry.put("763761017760771","[IT=1]");
		myHCountry.put("763777677560771","[BR=1]");
		myHCountry.put("763777777760371","[MX=1]");
		myHCountry.put("763777777760700","[DZ=1]");
		myHCountry.put("763777777760711","[EE=1]");
		myHCountry.put("765677777760771","[GR=1]");
		myHCountry.put("767367777760661","[ES=1]");
		myHCountry.put("767701003740771","[CN=1]");
		myHCountry.put("767740003760771","[CN=1]");
		myHCountry.put("767757777760771","[CN=1]");
		myHCountry.put("767774077560771","[US=1]");
		myHCountry.put("767776037760771","[US=1]");
		myHCountry.put("767777577740771","[SA=1]");
		myHCountry.put("767777663760371","[DE=1]");
		myHCountry.put("767777777740771","[TR=1]");
		myHCountry.put("767777777760471","[US=1]");
		myHCountry.put("767777777760571","[US=1]");
		myHCountry.put("767777777760611","[US=1]");
		myHCountry.put("767777777760631","[MA=1]");
		myHCountry.put("767777777760761","[FR=1]");
		myHCountry.put("770000001760771","[FR=1]");
		myHCountry.put("770000103760771","[FR=1]");
		myHCountry.put("770001777760771","[TN=1]");
		myHCountry.put("770003777760771","[NL=1]");
		myHCountry.put("770011777760771","[GR=1]");
		myHCountry.put("770014777760771","[GE=1]");
		myHCountry.put("770017037760711","[US=1]");
		myHCountry.put("770037777740751","[US=1]");
		myHCountry.put("770037777760700","[FI=1]");
		myHCountry.put("770077777760771","[FR=1]");
		myHCountry.put("770171777740771","[GR=1]");
		myHCountry.put("770177777740071","[AR=1]");
		myHCountry.put("770377037760771","[FR=1]");
		myHCountry.put("770377777760711","[FR=1]");
		myHCountry.put("770777770360771","[TN=1]");
		myHCountry.put("770777777760771","[CU=1]");
		myHCountry.put("771767677560671","[FR=1]");
		myHCountry.put("771777717760771","[RE=1]");
		myHCountry.put("773367777760771","[ES=1]");
		myHCountry.put("773377777760771","[GR=1]");
		myHCountry.put("773567675560671","[PT=1]");
		myHCountry.put("773763777760771","[US=1]");
		myHCountry.put("773767666560671","[FR=1]");
		myHCountry.put("773773745760771","[BE=1]");
		myHCountry.put("773777037760771","[SE=1]");
		myHCountry.put("773777137760771","[IT=1]");
		myHCountry.put("773777663760771","[BR=1]");
		myHCountry.put("773777676760771","[ID=1]");
		myHCountry.put("773777677760611","[NL=1]");
		myHCountry.put("773777707760771","[IN=1]");
		myHCountry.put("773777773640400","[MY=1]");
		myHCountry.put("773777776760771","[TR=1]");
		myHCountry.put("773777777360771","[US=1]");
		myHCountry.put("773777777740171","[EG=1]");
		myHCountry.put("773777777760031","[US=1]");
		myHCountry.put("774000777740771","[IT=1]");
		myHCountry.put("774002000760771","[RU=1]");
		myHCountry.put("774017477740771","[US=1]");
		myHCountry.put("774037603760700","[FR=1]");
		myHCountry.put("774037703760700","[FR=1]");
		myHCountry.put("774037707560700","[ZA=1]");
		myHCountry.put("774037707760771","[ZA=1]");
		myHCountry.put("774037777740700","[GP=1]");
		myHCountry.put("774037777760771","[NL=1]");
		myHCountry.put("774047777760661","[ES=1]");
		myHCountry.put("774377776360751","[PE=1]");
		myHCountry.put("774777777460771","[JM=1]");
		myHCountry.put("775347437740261","[AL=1]");
		myHCountry.put("775357777760771","[CO=1]");
		myHCountry.put("775567677760771","[MZ=1]");
		myHCountry.put("775740003760711","[RU=1]");
		myHCountry.put("775741000760571","[AR=1]");
		myHCountry.put("775767600160771","[BR=1]");
		myHCountry.put("775767600760771","[IQ=1]");
		myHCountry.put("775767760060771","[PY=1]");
		myHCountry.put("775767777760471","[BR=1]");
		myHCountry.put("775777676740600","[IN=1]");
		myHCountry.put("775777717760771","[ZA=1]");
		myHCountry.put("775777777760221","[PK=1]");
		myHCountry.put("775777777760601","[CU=1]");
		myHCountry.put("776000017760771","[IT=1]");
		myHCountry.put("776000377760771","[CN=1]");
		myHCountry.put("776017777760571","[FR=1]");
		myHCountry.put("776023777760771","[BR=1]");
		myHCountry.put("776037777760571","[CO=1]");
		myHCountry.put("776077737760771","[AR=1]");
		myHCountry.put("776077777760671","[ES=1]");
		myHCountry.put("776167777760171","[ES=1]");
		myHCountry.put("776167777760621","[ES=1]");
		myHCountry.put("776167777760671","[MG=1]");
		myHCountry.put("776167777760751","[ET=1]");
		myHCountry.put("776173777760771","[IT=1]");
		myHCountry.put("776174000060771","[US=1]");
		myHCountry.put("776175770760771","[AR=1]");
		myHCountry.put("776177403760771","[BE=1]");
		myHCountry.put("776177577760771","[EG=1]");
		myHCountry.put("776177600160771","[HT=1]");
		myHCountry.put("776177600560771","[PE=1]");
		myHCountry.put("776177707760771","[BR=1]");
		myHCountry.put("776177747760771","[BR=1]");
		myHCountry.put("776177777760751","[IT=1]");
		myHCountry.put("776637777760771","[PE=1]");
		myHCountry.put("776717777760771","[FR=1]");
		myHCountry.put("776777737760771","[IT=1]");
		myHCountry.put("776777770060771","[BE=1]");
		myHCountry.put("777000000060771","[MZ=1]");
		myHCountry.put("777000777760771","[GB=1]");
		myHCountry.put("777001745760771","[MA=1]");
		myHCountry.put("777007777760771","[ET=1]");
		myHCountry.put("777017777760771","[SD=1]");
		myHCountry.put("777033777760771","[NL=1]");
		myHCountry.put("777067777760771","[GM=1]");
		myHCountry.put("777155774760761","[GR=1]");
		myHCountry.put("777157777760671","[BE=1]");
		myHCountry.put("777176003760771","[FR=1]");
		myHCountry.put("777177616040771","[MX=1]");
		myHCountry.put("777177677760771","[MX=1]");
		myHCountry.put("777177717760571","[NL=1]");
		myHCountry.put("777177776160771","[BR=1]");
		myHCountry.put("777177777460771","[SA=1]");
		myHCountry.put("777300003760771","[GE=1]");
		myHCountry.put("777317677760771","[ET=1]");
		myHCountry.put("777341017760771","[AR=1]");
		myHCountry.put("777347777760671","[AR=1]");
		myHCountry.put("777357777760611","[GR=1]");
		myHCountry.put("777357777760631","[VE=1]");
		myHCountry.put("777357777760671","[AR=1]");
		myHCountry.put("777365637760771","[BR=1]");
		myHCountry.put("777367623540771","[MX=1]");
		myHCountry.put("777367626760771","[MX=1]");
		myHCountry.put("777367761760771","[BR=1]");
		myHCountry.put("777367776760771","[TZ=1]");
		myHCountry.put("777367777560661","[ES=1]");
		myHCountry.put("777367777740771","[MX=1]");
		myHCountry.put("777367777760661","[ES=1]");
		myHCountry.put("777367777760761","[RU=1]");
		myHCountry.put("777375777760771","[GR=1]");
		myHCountry.put("777376077560771","[VE=1]");
		myHCountry.put("777377647760771","[ZA=1]");
		myHCountry.put("777377663760771","[AR=1]");
		myHCountry.put("777377717760671","[IT=1]");
		myHCountry.put("777377767560761","[BR=1]");
		myHCountry.put("777377777740771","[IT=1]");
		myHCountry.put("777377777760601","[TR=1]");
		myHCountry.put("777377777760661","[SA=1]");
		myHCountry.put("777377777760700","[VE=1]");
		myHCountry.put("777400002760771","[NL=1]");
		myHCountry.put("777400377760771","[FR=1]");
		myHCountry.put("777437777740371","[JP=1]");
		myHCountry.put("777437777760671","[IT=1]");
		myHCountry.put("777477776760770","[US=1]");
		myHCountry.put("777477777740371","[US=1]");
		myHCountry.put("777557733760771","[US=1]");
		myHCountry.put("777557777760431","[FR=1]");
		myHCountry.put("777567777760671","[SA=1]");
		myHCountry.put("777567777760771","[DK=1]");
		myHCountry.put("777577737760771","[FR=1]");
		myHCountry.put("777577747760471","[NL=1]");
		myHCountry.put("777577747760771","[DK=1]");
		myHCountry.put("777577777460771","[SA=1]");
		myHCountry.put("777577777740771","[DE=1]");
		myHCountry.put("777600000760711","[SE=1]");
		myHCountry.put("777600001760711","[SE=1]");
		myHCountry.put("777600001760771","[MG=1]");
		myHCountry.put("777600077760771","[CN=1]");
		myHCountry.put("777600377760771","[JP=1]");
		myHCountry.put("777600777760700","[US=1]");
		myHCountry.put("777600777760771","[FR=1]");
		myHCountry.put("777603017760631","[TN=1]");
		myHCountry.put("777603415760771","[ES=1]");
		myHCountry.put("777603427760771","[ID=1]");
		myHCountry.put("777603677760701","[CN=1]");
		myHCountry.put("777603717760771","[AR=1]");
		myHCountry.put("777607777760671","[MG=1]");
		myHCountry.put("777617707760771","[AT=1]");
		myHCountry.put("777621777760771","[JP=1]");
		myHCountry.put("777623777760771","[ID=1]");
		myHCountry.put("777637002060771","[SE=1]");
		myHCountry.put("777637077760771","[US=1]");
		myHCountry.put("777637603760771","[IN=1]");
		myHCountry.put("777637777740771","[US=1]");
		myHCountry.put("777637777760071","[CN=1]");
		myHCountry.put("777637777760471","[BE=1]");
		myHCountry.put("777637777760711","[BD=1]");
		myHCountry.put("777640037760771","[GB=1]");
		myHCountry.put("777640107760771","[US=1]");
		myHCountry.put("777657777760771","[CN=1]");
		myHCountry.put("777667777740771","[FR=1]");
		myHCountry.put("777675777760600","[BR=1]");
		myHCountry.put("777677677760771","[PL=1]");
		myHCountry.put("777677737760771","[BR=1]");
		myHCountry.put("777677741760771","[SA=1]");
		myHCountry.put("777677747760471","[BR=1]");
		myHCountry.put("777677770060771","[IT=1]");
		myHCountry.put("777677777760471","[BR=1]");
		myHCountry.put("777677777760750","[US=1]");
		myHCountry.put("777700000360771","[IT=1]");
		myHCountry.put("777700000760751","[IT=1]");
		myHCountry.put("777700003760000","[IT=1]");
		myHCountry.put("777700003760631","[IT=1]");
		myHCountry.put("777700003760661","[GE=1]");
		myHCountry.put("777700017560771","[MX=1]");
		myHCountry.put("777700037760771","[NP=1]");
		myHCountry.put("777700077760600","[US=1]");
		myHCountry.put("777700637760771","[BR=1]");
		myHCountry.put("777700777760601","[BR=1]");
		myHCountry.put("777701000160771","[IT=1]");
		myHCountry.put("777701001760771","[FI=1]");
		myHCountry.put("777701003760771","[RU=1]");
		myHCountry.put("777701017760771","[MA=1]");
		myHCountry.put("777701037760771","[FR=1]");
		myHCountry.put("777703000760771","[ES=1]");
		myHCountry.put("777703747760771","[ID=1]");
		myHCountry.put("777703776760771","[FR=1]");
		myHCountry.put("777703777560771","[NL=1]");
		myHCountry.put("777703777760471","[MY=1]");
		myHCountry.put("777703777760770","[ET=1]");
		myHCountry.put("777707637740771","[AL=1]");
		myHCountry.put("777707776360751","[ES=1]");
		myHCountry.put("777717777660771","[CN=1]");
		myHCountry.put("777717777760571","[FR=1]");
		myHCountry.put("777717777760671","[AL=1]");
		myHCountry.put("777717777760770","[IT=1]");
		myHCountry.put("777740001560771","[BR=1]");
		myHCountry.put("777740037760771","[AT=1]");
		myHCountry.put("777740077560771","[BR=1]");
		myHCountry.put("777740167760771","[ZA=1]");
		myHCountry.put("777740177760471","[US=1]");
		myHCountry.put("777740177760771","[FR=1]");
		myHCountry.put("777740360060771","[NL=1]");
		myHCountry.put("777740617760771","[US=1]");
		myHCountry.put("777740677760771","[PE=1]");
		myHCountry.put("777741000060771","[FR=1]");
		myHCountry.put("777741000160771","[FR=1]");
		myHCountry.put("777741000360771","[AR=1]");
		myHCountry.put("777741000760771","[AR=1]");
		myHCountry.put("777741001760771","[NL=1]");
		myHCountry.put("777741637760771","[ES=1]");
		myHCountry.put("777743767760700","[CN=1]");
		myHCountry.put("777747637740771","[AL=1]");
		myHCountry.put("777747637760771","[EG=1]");
		myHCountry.put("777747663740771","[ID=1]");
		myHCountry.put("777747666760771","[ID=1]");
		myHCountry.put("777747717760771","[CN=1]");
		myHCountry.put("777747777740771","[AL=1]");
		myHCountry.put("777747777760671","[MX=1]");
		myHCountry.put("777757602760771","[ID=1]");
		myHCountry.put("777757603760771","[ID=1]");
		myHCountry.put("777757617760771","[AR=1]");
		myHCountry.put("777757633740371","[CN=1]");
		myHCountry.put("777757662760771","[ID=1]");
		myHCountry.put("777757677760770","[AR=1]");
		myHCountry.put("777757760160771","[CM=1]");
		myHCountry.put("777757777760471","[BR=1]");
		myHCountry.put("777757777760700","[ET=1]");
		myHCountry.put("777760037560771","[CO=1]");
		myHCountry.put("777760037760771","[IT=1]");
		myHCountry.put("777760047760071","[US=1]");
		myHCountry.put("777760063560771","[BR=1]");
		myHCountry.put("777760077760740","[GB=1]");
		myHCountry.put("777760147760771","[GB=1]");
		myHCountry.put("777760177760771","[US=1]");
		myHCountry.put("777760577760771","[CO=1]");
		myHCountry.put("777761003760771","[FR=1]");
		myHCountry.put("777761017760771","[ZA=1]");
		myHCountry.put("777761047760371","[IN=1]");
		myHCountry.put("777761077760770","[NL=1]");
		myHCountry.put("777761707760771","[FR=1]");
		myHCountry.put("777761777760601","[GW=1]");
		myHCountry.put("777762047760771","[BE=1]");
		myHCountry.put("777763017760771","[ZA=1]");
		myHCountry.put("777763417760771","[BR=1]");
		myHCountry.put("777764037560771","[US=1]");
		myHCountry.put("777764077760771","[IQ=1]");
		myHCountry.put("777764777760771","[JP=1]");
		myHCountry.put("777765777760771","[AT=1]");
		myHCountry.put("777766377760601","[US=1]");
		myHCountry.put("777767417760771","[SA=1]");
		myHCountry.put("777767437740761","[SA=1]");
		myHCountry.put("777767477740670","[ID=1]");
		myHCountry.put("777767504760771","[SA=1]");
		myHCountry.put("777767577660671","[SA=1]");
		myHCountry.put("777767601760771","[FR=1]");
		myHCountry.put("777767617760771","[BR=1]");
		myHCountry.put("777767636760771","[IQ=1]");
		myHCountry.put("777767637760371","[BR=1]");
		myHCountry.put("777767675760771","[FR=1]");
		myHCountry.put("777767677760631","[AR=1]");
		myHCountry.put("777767677760671","[MG=1]");
		myHCountry.put("777767706760711","[ID=1]");
		myHCountry.put("777767707760771","[AR=1]");
		myHCountry.put("777767717760711","[BR=1]");
		myHCountry.put("777767717760771","[IT=1]");
		myHCountry.put("777767737760621","[MG=1]");
		myHCountry.put("777767747760471","[AT=1]");
		myHCountry.put("777767760060771","[FR=1]");
		myHCountry.put("777767760360371","[HT=1]");
		myHCountry.put("777767767760661","[ZA=1]");
		myHCountry.put("777767774060771","[TR=1]");
		myHCountry.put("777767776160601","[PT=1]");
		myHCountry.put("777767776760771","[TR=1]");
		myHCountry.put("777767777460611","[BR=1]");
		myHCountry.put("777767777460771","[BR=1]");
		myHCountry.put("777767777740051","[US=1]");
		myHCountry.put("777767777740671","[IN=1]");
		myHCountry.put("777767777740761","[TZ=1]");
		myHCountry.put("777767777760171","[PT=1]");
		myHCountry.put("777767777760410","[AT=1]");
		myHCountry.put("777767777760611","[PT=1]");
		myHCountry.put("777767777760701","[AR=1]");
		myHCountry.put("777767777760711","[MG=1]");
		myHCountry.put("777770017760771","[GR=1]");
		myHCountry.put("777770077760601","[FR=1]");
		myHCountry.put("777770367760771","[NG=1]");
		myHCountry.put("777770377760771","[IT=1]");
		myHCountry.put("777773773760771","[DE=1]");
		myHCountry.put("777773774060371","[BR=1]");
		myHCountry.put("777773777760371","[PL=1]");
		myHCountry.put("777774000060771","[HN=1]");
		myHCountry.put("777774003660771","[IT=1]");
		myHCountry.put("777774037760771","[LY=1]");
		myHCountry.put("777774077560711","[PE=1]");
		myHCountry.put("777774077560751","[BR=1]");
		myHCountry.put("777774077760771","[US=1]");
		myHCountry.put("777774177760771","[HT=1]");
		myHCountry.put("777774777760751","[VN=1]");
		myHCountry.put("777775037560771","[NL=1]");
		myHCountry.put("777775074060771","[ES=1]");
		myHCountry.put("777775177760571","[MQ=1]");
		myHCountry.put("777775677560771","[ES=1]");
		myHCountry.put("777775747760471","[SE=1]");
		myHCountry.put("777775777760431","[GB=1]");
		myHCountry.put("777776000160771","[RU=1]");
		myHCountry.put("777776001760771","[US=1]");
		myHCountry.put("777776003760771","[AR=1]");
		myHCountry.put("777776017760771","[HT=1]");
		myHCountry.put("777776047560771","[BR=1]");
		myHCountry.put("777776077760711","[US=1]");
		myHCountry.put("777776201560771","[FR=1]");
		myHCountry.put("777776374360711","[US=1]");
		myHCountry.put("777776377060771","[NL=1]");
		myHCountry.put("777776377760760","[AR=1]");
		myHCountry.put("777777000160771","[TR=1]");
		myHCountry.put("777777001660771","[BR=1]");
		myHCountry.put("777777001760771","[ET=1]");
		myHCountry.put("777777003760771","[AR=1]");
		myHCountry.put("777777037740071","[HN=1]");
		myHCountry.put("777777037760031","[GF=1]");
		myHCountry.put("777777071760771","[ES=1]");
		myHCountry.put("777777076760771","[LY=1]");
		myHCountry.put("777777077660771","[FR=1]");
		myHCountry.put("777777177760571","[FR=1]");
		myHCountry.put("777777400740371","[TR=1]");
		myHCountry.put("777777414760771","[NL=1]");
		myHCountry.put("777777417740371","[BR=1]");
		myHCountry.put("777777437760541","[VN=1]");
		myHCountry.put("777777500760771","[FR=1]");
		myHCountry.put("777777504760771","[FR=1]");
		myHCountry.put("777777507760771","[GW=1]");
		myHCountry.put("777777547760771","[ES=1]");
		myHCountry.put("777777577560771","[BG=1]");
		myHCountry.put("777777577660071","[CN=1]");
		myHCountry.put("777777577760770","[VN=1]");
		myHCountry.put("777777600040771","[US=1]");
		myHCountry.put("777777600360771","[NL=1]");
		myHCountry.put("777777601560711","[BR=1]");
		myHCountry.put("777777601740771","[BR=1]");
		myHCountry.put("777777602060371","[SE=1]");
		myHCountry.put("777777602760771","[MY=1]");
		myHCountry.put("777777603760631","[US=1]");
		myHCountry.put("777777626060771","[GB=1]");
		myHCountry.put("777777637760751","[PT=1]");
		myHCountry.put("777777640760601","[AU=1]");
		myHCountry.put("777777641560771","[GF=1]");
		myHCountry.put("777777647660761","[BR=1]");
		myHCountry.put("777777660060771","[FR=1]");
		myHCountry.put("777777661760771","[PY=1]");
		myHCountry.put("777777663360771","[SR=1]");
		myHCountry.put("777777670160771","[MX=1]");
		myHCountry.put("777777673760771","[CN=1]");
		myHCountry.put("777777674760771","[ID=1]");
		myHCountry.put("777777675760771","[FR=1]");
		myHCountry.put("777777676060771","[ZW=1]");
		myHCountry.put("777777676740031","[IN=1]");
		myHCountry.put("777777677560371","[BR=1]");
		myHCountry.put("777777677660771","[FR=1]");
		myHCountry.put("777777677760000","[BR=1]");
		myHCountry.put("777777677760601","[ES=1]");
		myHCountry.put("777777677760711","[CN=1]");
		myHCountry.put("777777700160771","[VE=1]");
		myHCountry.put("777777701760771","[AT=1]");
		myHCountry.put("777777706360771","[JM=1]");
		myHCountry.put("777777707740371","[AU=1]");
		myHCountry.put("777777707740771","[IT=1]");
		myHCountry.put("777777707760351","[TR=1]");
		myHCountry.put("777777707760711","[BE=1]");
		myHCountry.put("777777717560771","[BR=1]");
		myHCountry.put("777777717760770","[IN=1]");
		myHCountry.put("777777727760771","[NG=1]");
		myHCountry.put("777777736760771","[SA=1]");
		myHCountry.put("777777737660771","[US=1]");
		myHCountry.put("777777737760571","[BG=1]");
		myHCountry.put("777777737760700","[IR=1]");
		myHCountry.put("777777737760751","[FR=1]");
		myHCountry.put("777777740060771","[ES=1]");
		myHCountry.put("777777740160771","[AT=1]");
		myHCountry.put("777777740360771","[US=1]");
		myHCountry.put("777777740660771","[US=1]");
		myHCountry.put("777777746060771","[ZM=1]");
		myHCountry.put("777777747760411","[SE=1]");
		myHCountry.put("777777747760671","[SA=1]");
		myHCountry.put("777777760160771","[FR=1]");
		myHCountry.put("777777760340000","[RU=1]");
		myHCountry.put("777777761740771","[FR=1]");
		myHCountry.put("777777767760371","[BR=1]");
		myHCountry.put("777777767760761","[ZA=1]");
		myHCountry.put("777777770060571","[FR=1]");
		myHCountry.put("777777770760671","[AT=1]");
		myHCountry.put("777777770760760","[CN=1]");
		myHCountry.put("777777774060671","[KE=1]");
		myHCountry.put("777777774160771","[MX=1]");
		myHCountry.put("777777774560771","[PK=1]");
		myHCountry.put("777777775760371","[AT=1]");
		myHCountry.put("777777775760571","[SA=1]");
		myHCountry.put("777777776360711","[US=1]");
		myHCountry.put("777777776760600","[IN=1]");
		myHCountry.put("777777777160771","[US=1]");
		myHCountry.put("777777777360200","[SD=1]");
		myHCountry.put("777777777360371","[SE=1]");
		myHCountry.put("777777777360411","[SE=1]");
		myHCountry.put("777777777460631","[VE=1]");
		myHCountry.put("777777777460760","[TR=1]");
		myHCountry.put("777777777560611","[IT=1]");
		myHCountry.put("777777777560740","[SA=1]");
		myHCountry.put("777777777640171","[SA=1]");
		myHCountry.put("777777777740000","[MM=1]");
		myHCountry.put("777777777740011","[ET=1]");
		myHCountry.put("777777777740411","[FI=1]");
		myHCountry.put("777777777740711","[US=1]");
		myHCountry.put("777777777760151","[BE=1]");
		myHCountry.put("777777777760370","[ID=1]");
		myHCountry.put("777777777760410","[BR=1]");
		myHCountry.put("777777777760500","[AT=1]");
		myHCountry.put("777777777760570","[ES=1]");
		myHCountry.put("777777777760661","[ZM=1]");
		myHCountry.put("370000777760771","[RU=1]");
		myHCountry.put("570000636760601","[RU=1]");
		myHCountry.put("570000675760771","[FR=1]");
		myHCountry.put("570000777760771","[FR=1]");
		myHCountry.put("670000775760731","[GE=1]");
		myHCountry.put("770000607760731","[SE=1]");
		myHCountry.put("770000643760771","[SE=1]");
		myHCountry.put("770000675760771","[FR=1]");
		myHCountry.put("770000677760700","[NL=1]");
		myHCountry.put("770000677760731","[SE=1]");
		myHCountry.put("770000677760771","[NL=1]");
		myHCountry.put("770000707760771","[RU=1]");
		myHCountry.put("770000737760771","[RU=1]");
		myHCountry.put("770000757760711","[FR=1]");
		myHCountry.put("770000771760771","[BE=1]");
		myHCountry.put("770000777760701","[RU=1]");
		myHCountry.put("000000000060731","[US=1]");
		myHCountry.put("000000077560731","[US=1]");
		myHCountry.put("000000107760731","[]");
		myHCountry.put("000000577760731","[TN=1]");
		myHCountry.put("000003777760731","[CA=1]");
		myHCountry.put("003777777760731","[US=1]");
		myHCountry.put("014177775740721","[IT=1]");
		myHCountry.put("017777777760731","[IT=1]");
		myHCountry.put("037741403760731","[FR=1]");
		myHCountry.put("077777707760731","[US=1]");
		myHCountry.put("157777777760731","[IT=1]");
		myHCountry.put("176177777760731","[CN=1]");
		myHCountry.put("177767777760731","[CM=1]");
		myHCountry.put("177775777760731","[IT=1]");
		myHCountry.put("337777777760731","[ZA=1]");
		myHCountry.put("343775777560731","[US=1]");
		myHCountry.put("367777770060731","[SA=1]");
		myHCountry.put("377703777760731","[FR=1]");
		myHCountry.put("377767777760731","[CM=1]");
		myHCountry.put("377777600160731","[BR=1]");
		myHCountry.put("377777707760731","[BR=1]");
		myHCountry.put("377777777560731","[CN=1]");
		myHCountry.put("377777777660731","[CN=1]");
		myHCountry.put("377777777740331","[GB=1]");
		myHCountry.put("400000037760731","[MZ=1]");
		myHCountry.put("417177777760731","[FR=1]");
		myHCountry.put("437767777760731","[CM=1]");
		myHCountry.put("457777777760731","[CN=1]");
		myHCountry.put("477477777760731","[CN=1]");
		myHCountry.put("477703777760731","[MY=1]");
		myHCountry.put("477777717760731","[PE=1]");
		myHCountry.put("535767777740731","[AR=1]");
		myHCountry.put("577265637740721","[TZ=1]");
		myHCountry.put("577777403760731","[CN=1]");
		myHCountry.put("577777777360731","[BE=1]");
		myHCountry.put("600017777760731","[IT=1]");
		myHCountry.put("601357636060731","[MW=1]");
		myHCountry.put("603757414140731","[TZ=1]");
		myHCountry.put("607777677560731","[BR=1]");
		myHCountry.put("637764777760731","[FR=1]");
		myHCountry.put("637777777760731","[IT=1]");
		myHCountry.put("640000000060731","[BR=1]");
		myHCountry.put("677377777760731","[IT=1]");
		myHCountry.put("677763777760731","[CN=1]");
		myHCountry.put("677774764060731","[NG=1]");
		myHCountry.put("677775764060731","[NG=1]");
		myHCountry.put("677777747760731","[US=1]");
		myHCountry.put("677777770060731","[CN=1]");
		myHCountry.put("700777777760731","[IT=1]");
		myHCountry.put("703777777760731","[DO=1]");
		myHCountry.put("717777177760731","[RU=1]");
		myHCountry.put("717777663560731","[NL=1]");
		myHCountry.put("733777777760731","[SE=1]");
		myHCountry.put("737377771760731","[FR=1]");
		myHCountry.put("737377777760731","[SA=1]");
		myHCountry.put("737767777760731","[FR=1]");
		myHCountry.put("737777677460731","[NG=1]");
		myHCountry.put("737777777760531","[IN=1]");
		myHCountry.put("737777777760720","[SE=1]");
		myHCountry.put("740077777760731","[FR=1]");
		myHCountry.put("740377773740731","[BR=1]");
		myHCountry.put("740617777760731","[NL=1]");
		myHCountry.put("740767777760731","[AR=1]");
		myHCountry.put("742177777760731","[CN=1]");
		myHCountry.put("743577777760731","[CN=1]");
		myHCountry.put("746000076360731","[TW=1]");
		myHCountry.put("747777737760731","[FR=1]");
		myHCountry.put("747777774160731","[CN=1]");
		myHCountry.put("753777777760531","[CN=1]");
		myHCountry.put("756177777760731","[TW=1]");
		myHCountry.put("757037777760731","[US=1]");
		myHCountry.put("757367777760731","[DE=1]");
		myHCountry.put("757377777760731","[BR=1]");
		myHCountry.put("757703777760731","[US=1]");
		myHCountry.put("757743747760731","[SE=1]");
		myHCountry.put("757761774760731","[IT=1]");
		myHCountry.put("757776377760731","[IT=1]");
		myHCountry.put("757777403760731","[FI=1]");
		myHCountry.put("757777601760731","[FI=1]");
		myHCountry.put("757777767760731","[SE=1]");
		myHCountry.put("757777770760731","[CN=1]");
		myHCountry.put("760002377760731","[HT=1]");
		myHCountry.put("760377777760731","[FR=1]");
		myHCountry.put("760767777760731","[CM=1]");
		myHCountry.put("763777777760731","[US=1]");
		myHCountry.put("770003777760731","[IT=1]");
		myHCountry.put("770077637760731","[US=1]");
		myHCountry.put("774177777760731","[CN=1]");
		myHCountry.put("774377777660731","[ES=1]");
		myHCountry.put("774377777760731","[IT=1]");
		myHCountry.put("776177777760731","[CN=1]");
		myHCountry.put("776777706060731","[ZM=1]");
		myHCountry.put("777000003760731","[BD=1]");
		myHCountry.put("777000010060731","[US=1]");
		myHCountry.put("777020777760731","[ZA=1]");
		myHCountry.put("777317777760731","[US=1]");
		myHCountry.put("777377706060731","[BR=1]");
		myHCountry.put("777400000760731","[US=1]");
		myHCountry.put("777577777760331","[CN=1]");
		myHCountry.put("777601777760731","[JP=1]");
		myHCountry.put("777617777760731","[VN=1]");
		myHCountry.put("777623777760731","[FR=1]");
		myHCountry.put("777637777760731","[IT=1]");
		myHCountry.put("777677723760731","[CN=1]");
		myHCountry.put("777707777760731","[RU=1]");
		myHCountry.put("777717776760731","[KR=1]");
		myHCountry.put("777740000760531","[MM=1]");
		myHCountry.put("777740017760720","[IT=1]");
		myHCountry.put("777741017760731","[AR=1]");
		myHCountry.put("777743777760731","[TW=1]");
		myHCountry.put("777747777760731","[ES=1]");
		myHCountry.put("777757663760731","[ID=1]");
		myHCountry.put("777757777560731","[BR=1]");
		myHCountry.put("777757777760531","[CN=1]");
		myHCountry.put("777767637760321","[AL=1]");
		myHCountry.put("777767677760731","[AR=1]");
		myHCountry.put("777767777460731","[FR=1]");
		myHCountry.put("777767777560731","[GW=1]");
		myHCountry.put("777767777760721","[IN=1]");
		myHCountry.put("777770017760731","[IT=1]");
		myHCountry.put("777773637560731","[BR=1]");
		myHCountry.put("777774077760731","[RU=1]");
		myHCountry.put("777774377760131","[CN=1]");
		myHCountry.put("777774602760731","[US=1]");
		myHCountry.put("777775177760731","[NL=1]");
		myHCountry.put("777776177760731","[IT=1]");
		myHCountry.put("777776376360731","[CN=1]");
		myHCountry.put("777777000760731","[US=1]");
		myHCountry.put("777777003760731","[NL=1]");
		myHCountry.put("777777017760731","[FR=1]");
		myHCountry.put("777777400740731","[US=1]");
		myHCountry.put("777777400760731","[HN=1]");
		myHCountry.put("777777403760731","[ZA=1]");
		myHCountry.put("777777601560731","[US=1]");
		myHCountry.put("777777617760731","[IT=1]");
		myHCountry.put("777777627760731","[MW=1]");
		myHCountry.put("777777637760731","[MA=1]");
		myHCountry.put("777777677560731","[BR=1]");
		myHCountry.put("777777701760731","[IT=1]");
		myHCountry.put("777777702760731","[CN=1]");
		myHCountry.put("777777706660731","[VE=1]");
		myHCountry.put("777777760760731","[VN=1]");
		myHCountry.put("777777763560731","[US=1]");
		myHCountry.put("777777774160731","[BR=1]");
		myHCountry.put("777777776360731","[NL=1]");
		myHCountry.put("777777777460731","[SA=1]");
		myHCountry.put("777777777740331","[BE=1]");
		myHCountry.put("777777777760720","[CN=1]");
		myHCountry.put("000000017760730","[NL=1]");
		myHCountry.put("077777417760730","[AR=1]");
		myHCountry.put("237760774360330","[TZ=1]");
		myHCountry.put("237775777760730","[ZM=1]");
		myHCountry.put("437764777540730","[DE=1]");
		myHCountry.put("437774777760730","[DE=1]");
		myHCountry.put("623774777760730","[DE=1]");
		myHCountry.put("624775777760730","[ZA=1]");
		myHCountry.put("637344777740130","[TZ=1]");
		myHCountry.put("637574777760730","[UG=1]");
		myHCountry.put("637735777760730","[NL=1]");
		myHCountry.put("637764777760730","[US=1]");
		myHCountry.put("637765777760730","[DE=1]");
		myHCountry.put("637767777760730","[ZA=1]");
		myHCountry.put("637774777560730","[SE=1]");
		myHCountry.put("637775777740130","[BE=1]");
		myHCountry.put("637776777760730","[BE=1]");
		myHCountry.put("677773777760730","[GB=1]");
		myHCountry.put("737337777760730","[BR=1]");
		myHCountry.put("737737777760730","[BR=1]");
		myHCountry.put("737777603760730","[ZA=1]");
		myHCountry.put("757777403760730","[GB=1]");
		myHCountry.put("767777777760530","[ZA=1]");
		myHCountry.put("777037777760730","[UG=1]");
		myHCountry.put("777535777760730","[DE=1]");
		myHCountry.put("777577777760730","[ZA=1]");
		myHCountry.put("777707777760730","[DE=1]");
		myHCountry.put("777737777760730","[ZA=1]");
		myHCountry.put("777777603760730","[ZA=1]");
		myHCountry.put("037727777760771","[IQ=1]");
		myHCountry.put("076737737760771","[ET=1]");
		myHCountry.put("177737600160771","[BR=1]");
		myHCountry.put("337737777760771","[ET=1]");
		myHCountry.put("373727777760771","[SA=1]");
		myHCountry.put("377723777760671","[ZA=1]");
		myHCountry.put("377734077760771","[HT=1]");
		myHCountry.put("377737677760771","[DO=1]");
		myHCountry.put("377737777760700","[ID=1]");
		myHCountry.put("403737636040401","[FR=1]");
		myHCountry.put("437723777760771","[BD=1]");
		myHCountry.put("437727777760771","[IQ=1]");
		myHCountry.put("437737777760771","[FR=1]");
		myHCountry.put("457733577760771","[IR=1]");
		myHCountry.put("477737637760711","[BR=1]");
		myHCountry.put("477737637760771","[BR=1]");
		myHCountry.put("517727403540061","[BR=1]");
		myHCountry.put("537737777760771","[AL=1]");
		myHCountry.put("577537777760771","[PE=1]");
		myHCountry.put("577727437160760","[SA=1]");
		myHCountry.put("577737577760771","[SA=1]");
		myHCountry.put("577737777760761","[CN=1]");
		myHCountry.put("600137777760771","[JP=1]");
		myHCountry.put("637737777760771","[IQ=1]");
		myHCountry.put("677737600060771","[GP=1]");
		myHCountry.put("677737600760771","[VE=1]");
		myHCountry.put("677737603760771","[CO=1]");
		myHCountry.put("677737707760711","[BR=1]");
		myHCountry.put("677737777460771","[CN=1]");
		myHCountry.put("701727677760771","[JP=1]");
		myHCountry.put("701737777740471","[BR=1]");
		myHCountry.put("703737777760771","[OM=1]");
		myHCountry.put("737723777760771","[ID=1]");
		myHCountry.put("737737737560771","[CN=1]");
		myHCountry.put("737737777740071","[CN=1]");
		myHCountry.put("737737777740371","[HU=1]");
		myHCountry.put("754337777760771","[CN=1]");
		myHCountry.put("757723777760771","[JP=1]");
		myHCountry.put("767737777760771","[PE=1]");
		myHCountry.put("771737777760771","[ES=1]");
		myHCountry.put("773737003760771","[FR=1]");
		myHCountry.put("775737777760771","[PL=1]");
		myHCountry.put("776137600760771","[ZA=1]");
		myHCountry.put("776137777760771","[PT=1]");
		myHCountry.put("776737437760771","[ET=1]");
		myHCountry.put("776737617760771","[ET=1]");
		myHCountry.put("776737777760751","[ES=1]");
		myHCountry.put("777123777760771","[JP=1]");
		myHCountry.put("777323777760771","[ES=1]");
		myHCountry.put("777336377760771","[IR=1]");
		myHCountry.put("777720777760671","[IT=1]");
		myHCountry.put("777720777760771","[IT=1]");
		myHCountry.put("777721777760771","[ET=1]");
		myHCountry.put("777723717760771","[ID=1]");
		myHCountry.put("777723737760771","[ID=1]");
		myHCountry.put("777723777360771","[ES=1]");
		myHCountry.put("777727637760771","[EG=1]");
		myHCountry.put("777727737760760","[FR=1]");
		myHCountry.put("777727761760771","[FR=1]");
		myHCountry.put("777727777760071","[PT=1]");
		myHCountry.put("777730777760771","[US=1]");
		myHCountry.put("777734047560771","[US=1]");
		myHCountry.put("777735777760771","[SE=1]");
		myHCountry.put("777737637760071","[BR=1]");
		myHCountry.put("777737637760771","[VE=1]");
		myHCountry.put("777737667760771","[GE=1]");
		myHCountry.put("777737676760771","[MY=1]");
		myHCountry.put("777737677560771","[BR=1]");
		myHCountry.put("777737677760671","[VE=1]");
		myHCountry.put("777737707760711","[BR=1]");
		myHCountry.put("777737737760371","[OM=1]");
		myHCountry.put("777737737760571","[OM=1]");
		myHCountry.put("777737737760600","[US=1]");
		myHCountry.put("777737747760411","[CN=1]");
		myHCountry.put("777737747760771","[CN=1]");
		myHCountry.put("777737767760711","[NL=1]");
		myHCountry.put("777737767760771","[US=1]");
		myHCountry.put("777737770760771","[FR=1]");
		myHCountry.put("777737774060671","[OM=1]");
		myHCountry.put("777737776360771","[BE=1]");
		myHCountry.put("777737777560771","[ET=1]");
		myHCountry.put("777737777660700","[MA=1]");
		myHCountry.put("777737777740171","[NL=1]");
		myHCountry.put("777737777740371","[IN=1]");
		myHCountry.put("777737777740770","[PK=1]");
		myHCountry.put("777737777760000","[DE=1]");
		myHCountry.put("777737777760031","[US=1]");
		myHCountry.put("777737777760500","[US=1]");
		myHCountry.put("777737777760761","[CN=1]");
		myHCountry.put("515000236740261","[ET=1]");
		myHCountry.put("577000237760671","[ET=1]");
		myHCountry.put("577000377740771","[NO=1]");
		myHCountry.put("577000377760771","[FR=1]");
		myHCountry.put("737000377760771","[IT=1]");
		myHCountry.put("777000206060771","[MW=1]");
		myHCountry.put("777000317760771","[FR=1]");
		myHCountry.put("777000376760671","[ET=1]");
		myHCountry.put("777000377740071","[GB=1]");
		myHCountry.put("777000377760671","[SA=1]");
		myHCountry.put("777000377760751","[NL=1]");
		myHCountry.put("741727757560771","[JP=1]");
		myHCountry.put("741737177760771","[JP=1]");
		myHCountry.put("741737437760771","[JP=1]");
		myHCountry.put("741737577760731","[JP=1]");
		myHCountry.put("741737577760771","[JP=1]");
		myHCountry.put("741737717760771","[JP=1]");
		myHCountry.put("417777377760771","[GM=1]");
		myHCountry.put("515047236760661","[RU=1]");
		myHCountry.put("574367375760671","[FR=1]");
		myHCountry.put("575777307760771","[IT=1]");
		myHCountry.put("577767377740771","[NO=1]");
		myHCountry.put("577777377460771","[SA=1]");
		myHCountry.put("600017377760771","[HT=1]");
		myHCountry.put("617623377760771","[IN=1]");
		myHCountry.put("617777377760601","[US=1]");
		myHCountry.put("617777377760771","[US=1]");
		myHCountry.put("700037377760711","[ZA=1]");
		myHCountry.put("700077377760671","[US=1]");
		myHCountry.put("704003347760400","[PE=1]");
		myHCountry.put("707777377360771","[ET=1]");
		myHCountry.put("757777277760771","[IT=1]");
		myHCountry.put("767777377760771","[FR=1]");
		myHCountry.put("771777377760771","[KG=1]");
		myHCountry.put("774037377760700","[MA=1]");
		myHCountry.put("774177374760771","[FR=1]");
		myHCountry.put("777071377760771","[SA=1]");
		myHCountry.put("777361377760671","[CO=1]");
		myHCountry.put("777623377760771","[BE=1]");
		myHCountry.put("777677377760771","[TR=1]");
		myHCountry.put("777707377760771","[BR=1]");
		myHCountry.put("777743377760771","[TW=1]");
		myHCountry.put("777747237760671","[NO=1]");
		myHCountry.put("777747337760771","[NL=1]");
		myHCountry.put("777757377760601","[ZA=1]");
		myHCountry.put("777763377760771","[ES=1]");
		myHCountry.put("777767377760771","[IT=1]");
		myHCountry.put("777771377760771","[FR=1]");
		myHCountry.put("777773377760771","[US=1]");
		myHCountry.put("777775237560771","[IT=1]");
		myHCountry.put("777775377660771","[FR=1]");
		myHCountry.put("777775377760571","[AT=1]");
		myHCountry.put("777777263760771","[RU=1]");
		myHCountry.put("777777307760771","[BG=1]");
		myHCountry.put("777777317760571","[US=1]");
		myHCountry.put("777777377660771","[PS=1]");
		myHCountry.put("777777377760371","[AT=1]");
		myHCountry.put("777777377760760","[AR=1]");
		myHCountry.put("533777347560471","[CZ=1]");
		myHCountry.put("703777347760471","[US=1]");
		myHCountry.put("704003347560471","[US=1]");
		myHCountry.put("704023345760471","[PA=1]");
		myHCountry.put("707777344560471","[SE=1]");
		myHCountry.put("707777347760451","[CA=1]");
		myHCountry.put("757777347760471","[FR=1]");
		myHCountry.put("776777347760471","[FR=1]");
		myHCountry.put("777377347760471","[ZA=1]");
		myHCountry.put("777603347760471","[US=1]");
		myHCountry.put("777637347760471","[DE=1]");
		myHCountry.put("777777347160471","[AT=1]");
		myHCountry.put("000000157760771","[US=1]");
		myHCountry.put("531357657560771","[NL=1]");
		myHCountry.put("577760057760671","[IN=1]");
		myHCountry.put("577767657760771","[CM=1]");
		myHCountry.put("653700357760771","[AT=1]");
		myHCountry.put("675767757760671","[AT=1]");
		myHCountry.put("703777757760371","[IN=1]");
		myHCountry.put("715777757760771","[DE=1]");
		myHCountry.put("747777757660771","[VE=1]");
		myHCountry.put("757777757760711","[SE=1]");
		myHCountry.put("765777757760771","[DZ=1]");
		myHCountry.put("770037757760771","[SA=1]");
		myHCountry.put("771567757760671","[MZ=1]");
		myHCountry.put("771777757760771","[NL=1]");
		myHCountry.put("773777757760771","[ES=1]");
		myHCountry.put("775707757760771","[IT=1]");
		myHCountry.put("775743757760771","[AT=1]");
		myHCountry.put("775767756760771","[AR=1]");
		myHCountry.put("775777653760771","[GW=1]");
		myHCountry.put("775777757760760","[AR=1]");
		myHCountry.put("777000057760771","[FR=1]");
		myHCountry.put("777037757760771","[ET=1]");
		myHCountry.put("777345650760661","[AR=1]");
		myHCountry.put("777367757760771","[ES=1]");
		myHCountry.put("777701757760771","[CN=1]");
		myHCountry.put("777760053760771","[IN=1]");
		myHCountry.put("777760057740771","[IN=1]");
		myHCountry.put("777761757760771","[IN=1]");
		myHCountry.put("777767750060771","[SE=1]");
		myHCountry.put("777767757760671","[IT=1]");
		myHCountry.put("777777657760071","[BR=1]");
		myHCountry.put("777777750060671","[FI=1]");
		myHCountry.put("777777754360771","[IT=1]");
		myHCountry.put("777777755460771","[BR=1]");
		myHCountry.put("777777757460771","[SE=1]");
		myHCountry.put("777777757560771","[PY=1]");
		myHCountry.put("777777757760571","[RU=1]");
		myHCountry.put("777777757760611","[US=1]");
		myHCountry.put("777777757760671","[ZA=1]");
		myHCountry.put("777777757760760","[FR=1]");
		myHCountry.put("777777757760770","[GW=1]");
		myHCountry.put("637777557760771","[ES=1]");
		myHCountry.put("775777557760771","[BR=1]");
		myHCountry.put("777337557560771","[ES=1]");
		myHCountry.put("777347557760661","[BR=1]");
		myHCountry.put("777757557760771","[AR=1]");
		myHCountry.put("777763557760771","[BR=1]");
		myHCountry.put("777767557760671","[AR=1]");
		myHCountry.put("777775557760771","[AR=1]");
		myHCountry.put("777777556360771","[ES=1]");
		myHCountry.put("777777557760631","[FR=1]");
		myHCountry.put("000020007760731","[BR=1]");
		myHCountry.put("157760000060771","[CN=1]");
		myHCountry.put("377760007760771","[IT=1]");
		myHCountry.put("677760007760771","[US=1]");
		myHCountry.put("757760007760771","[RU=1]");
		myHCountry.put("770160007740771","[US=1]");
		myHCountry.put("773760007760771","[TR=1]");
		myHCountry.put("777460003760771","[RU=1]");
		myHCountry.put("777660007760771","[LV=1]");
		myHCountry.put("777760001560771","[RU=1]");
		myHCountry.put("777760005760771","[GE=1]");
		myHCountry.put("777760007560571","[BR=1]");
		myHCountry.put("777760007760571","[RU=1]");
		myHCountry.put("777760007760600","[RU=1]");
		myHCountry.put("777760007760671","[RU=1]");
		myHCountry.put("037777477760711","[FI=1]");
		myHCountry.put("377777477760771","[BR=1]");
		myHCountry.put("555346477760671","[EG=1]");
		myHCountry.put("577767477760671","[IN=1]");
		myHCountry.put("577767477760771","[GB=1]");
		myHCountry.put("677777477060771","[MX=1]");
		myHCountry.put("712517477760631","[IN=1]");
		myHCountry.put("776357467760671","[NG=1]");
		myHCountry.put("777177474760631","[GR=1]");
		myHCountry.put("777657477760771","[CN=1]");
		myHCountry.put("777667477760771","[IR=1]");
		myHCountry.put("777737467760771","[EG=1]");
		myHCountry.put("777737477760751","[VN=1]");
		myHCountry.put("777737477760771","[GB=1]");
		myHCountry.put("777776477760771","[MX=1]");
		myHCountry.put("777777467760771","[ES=1]");
		myHCountry.put("777777474760771","[TR=1]");
		myHCountry.put("777777477760411","[IT=1]");
		myHCountry.put("377760017760601","[MG=1]");
		myHCountry.put("477720017760771","[IT=1]");
		myHCountry.put("747760017760771","[IT=1]");
		myHCountry.put("757760017760771","[ES=1]");
		myHCountry.put("775760017760771","[AT=1]");
		myHCountry.put("777660017760771","[IT=1]");
		myHCountry.put("777760017760601","[MG=1]");
		myHCountry.put("000740004460671","[BD=1]");
		myHCountry.put("400000004760771","[US=1]");
		myHCountry.put("517347404540621","[IT=1]");
		myHCountry.put("577347404760471","[DK=1]");
		myHCountry.put("577707404760771","[AL=1]");
		myHCountry.put("577747404560671","[IT=1]");
		myHCountry.put("577777404760771","[TR=1]");
		myHCountry.put("717777404760771","[TR=1]");
		myHCountry.put("757777404760771","[GR=1]");
		myHCountry.put("761777404760771","[TR=1]");
		myHCountry.put("773777404760771","[TR=1]");
		myHCountry.put("775567404760771","[TR=1]");
		myHCountry.put("776007404760771","[DK=1]");
		myHCountry.put("776037404760771","[TR=1]");
		myHCountry.put("776777404760771","[TR=1]");
		myHCountry.put("777617404760771","[US=1]");
		myHCountry.put("777727404760771","[TR=1]");
		myHCountry.put("777747404740771","[TR=1]");
		myHCountry.put("777757404760771","[TR=1]");
		myHCountry.put("777777004760771","[ET=1]");
		myHCountry.put("777777404760411","[SA=1]");
		myHCountry.put("777777404760711","[BG=1]");
		myHCountry.put("000000000007771","[CN=1]");
		myHCountry.put("000000000017771","[PH=1]");
		myHCountry.put("000000000503771","[RU=1]");
		myHCountry.put("000000000730371","[SD=1]");
		myHCountry.put("000000003737031","[TN=1]");
		myHCountry.put("000000004000011","[US=1]");
		myHCountry.put("000000004000771","[HT=1]");
		myHCountry.put("000000004003771","[IT=1]");
		myHCountry.put("000000004024771","[IT=1]");
		myHCountry.put("000000006000011","[HT=1]");
		myHCountry.put("000000006003771","[US=1]");
		myHCountry.put("000000007600771","[BR=1]");
		myHCountry.put("000000007700021","[AL=1]");
		myHCountry.put("000000007777711","[US=1]");
		myHCountry.put("000000034003771","[IN=1]");
		myHCountry.put("000000036377371","[ID=1]");
		myHCountry.put("000000077403771","[MG=1]");
		myHCountry.put("000000077777771","[US=1]");
		myHCountry.put("000000100003771","[FR=1]");
		myHCountry.put("000000377000010","[FR=1]");
		myHCountry.put("000000377400771","[AZ=1]");
		myHCountry.put("000000674364771","[IT=1]");
		myHCountry.put("000000757670340","[IT=1]");
		myHCountry.put("000000760003771","[IN=1]");
		myHCountry.put("000002000000111","[PL=1]");
		myHCountry.put("000003770000731","[CO=1]");
		myHCountry.put("000007077700020","[IN=1]");
		myHCountry.put("000017600003771","[PL=1]");
		myHCountry.put("000020000003771","[FR=1]");
		myHCountry.put("000022000003771","[ET=1]");
		myHCountry.put("000037774001771","[MG=1]");
		myHCountry.put("000147620000771","[AL=1]");
		myHCountry.put("000200000000000","[US=1]");
		myHCountry.put("000320000003771","[MY=1]");
		myHCountry.put("001767000003071","[PK=1]");
		myHCountry.put("001777600000000","[PK=1]");
		myHCountry.put("002002000003371","[FR=1]");
		myHCountry.put("002777777400771","[IR=1]");
		myHCountry.put("003000001400020","[FR=1]");
		myHCountry.put("003767700003771","[IN=1]");
		myHCountry.put("003777400000771","[US=1]");
		myHCountry.put("006563616000771","[AL=1]");
		myHCountry.put("007757606000031","[NL=1]");
		myHCountry.put("007764057777771","[VN=1]");
		myHCountry.put("007777777741371","[IT=1]");
		myHCountry.put("017700007702400","[US=1]");
		myHCountry.put("017777600000771","[BR=1]");
		myHCountry.put("017777777700000","[AT=1]");
		myHCountry.put("035777776400000","[IN=1]");
		myHCountry.put("037174002000071","[US=1]");
		myHCountry.put("037317607000271","[VE=1]");
		myHCountry.put("037621703774371","[SD=1]");
		myHCountry.put("037637377770371","[SA=1]");
		myHCountry.put("037657777741770","[IQ=1]");
		myHCountry.put("037676000000331","[FR=1]");
		myHCountry.put("037676000000360","[FR=1]");
		myHCountry.put("037677760000171","[AT=1]");
		myHCountry.put("037677777607771","[SA=1]");
		myHCountry.put("037777606000031","[AR=1]");
		myHCountry.put("037777740003771","[PK=1]");
		myHCountry.put("040077200000370","[TZ=1]");
		myHCountry.put("071777770000021","[PK=1]");
		myHCountry.put("075747637403631","[NO=1]");
		myHCountry.put("077264577775270","[TH=1]");
		myHCountry.put("077577757000371","[BD=1]");
		myHCountry.put("077577757000771","[BD=1]");
		myHCountry.put("077577767403371","[BD=1]");
		myHCountry.put("077637770000671","[PK=1]");
		myHCountry.put("077701002001731","[IN=1]");
		myHCountry.put("077737777403771","[PK=1]");
		myHCountry.put("077776777403671","[SE=1]");
		myHCountry.put("077777277403771","[MG=1]");
		myHCountry.put("077777377400771","[MG=1]");
		myHCountry.put("077777377417771","[MG=1]");
		myHCountry.put("077777777000071","[IN=1]");
		myHCountry.put("077777777000371","[AT=1]");
		myHCountry.put("077777777777771","[TW=1]");
		myHCountry.put("100240204004621","[BD=1]");
		myHCountry.put("157777777004340","[IT=1]");
		myHCountry.put("160000000003771","[IT=1]");
		myHCountry.put("160001400000171","[FR=1]");
		myHCountry.put("161777776003771","[IT=1]");
		myHCountry.put("174132417537741","[BR=1]");
		myHCountry.put("176160000000071","[ES=1]");
		myHCountry.put("177377777777771","[US=1]");
		myHCountry.put("177767377400671","[IQ=1]");
		myHCountry.put("177767777777731","[AU=1]");
		myHCountry.put("177770000000731","[MY=1]");
		myHCountry.put("177770200000720","[NL=1]");
		myHCountry.put("177775757443771","[SA=1]");
		myHCountry.put("177777377000371","[IT=1]");
		myHCountry.put("177777606400371","[US=1]");
		myHCountry.put("177777677766171","[US=1]");
		myHCountry.put("177777750000000","[ID=1]");
		myHCountry.put("177777770000000","[US=1]");
		myHCountry.put("177777777703560","[IT=1]");
		myHCountry.put("200023777377601","[TR=1]");
		myHCountry.put("203777140000771","[SA=1]");
		myHCountry.put("267013775777600","[BR=1]");
		myHCountry.put("277677606000171","[BR=1]");
		myHCountry.put("300000007177771","[ET=1]");
		myHCountry.put("300076760000011","[CU=1]");
		myHCountry.put("303777775003171","[SA=1]");
		myHCountry.put("306777100003361","[PK=1]");
		myHCountry.put("331440640000000","[IN=1]");
		myHCountry.put("341357000001661","[BE=1]");
		myHCountry.put("341777600001771","[BE=1]");
		myHCountry.put("363777777403771","[FR=1]");
		myHCountry.put("376160000000071","[BR=1]");
		myHCountry.put("377577777003731","[GY=1]");
		myHCountry.put("377700000001331","[ID=1]");
		myHCountry.put("377737670000000","[RU=1]");
		myHCountry.put("377777377000371","[GB=1]");
		myHCountry.put("377777400000771","[US=1]");
		myHCountry.put("377777606000031","[PE=1]");
		myHCountry.put("377777703770351","[BR=1]");
		myHCountry.put("377777760000000","[BR=1]");
		myHCountry.put("377777760700000","[BR=1]");
		myHCountry.put("377777770000000","[BE=1]");
		myHCountry.put("377777776000771","[NL=1]");
		myHCountry.put("377777777607771","[SA=1]");
		myHCountry.put("377777777700000","[FR=1]");
		myHCountry.put("377777777741771","[IR=1]");
		myHCountry.put("400000000000001","[IT=1]");
		myHCountry.put("400000000002771","[CN=1]");
		myHCountry.put("400000000403771","[IN=1]");
		myHCountry.put("400000020000001","[AL=1]");
		myHCountry.put("400377740003771","[IN=1]");
		myHCountry.put("403377777403071","[PK=1]");
		myHCountry.put("403567740001320","[US=1]");
		myHCountry.put("403767400001771","[IQ=1]");
		myHCountry.put("403777740000000","[ET=1]");
		myHCountry.put("403777740003471","[ET=1]");
		myHCountry.put("407777606000031","[PE=1]");
		myHCountry.put("407777740003171","[US=1]");
		myHCountry.put("410222000000031","[MX=1]");
		myHCountry.put("414007004600001","[IN=1]");
		myHCountry.put("414023417403431","[IN=1]");
		myHCountry.put("416163400000171","[ES=1]");
		myHCountry.put("417777437402071","[IN=1]");
		myHCountry.put("417777777403071","[US=1]");
		myHCountry.put("435247676403070","[IN=1]");
		myHCountry.put("437777777721771","[US=1]");
		myHCountry.put("441737777761771","[IN=1]");
		myHCountry.put("455367776403071","[PK=1]");
		myHCountry.put("457767637403071","[OM=1]");
		myHCountry.put("460000000000071","[IN=1]");
		myHCountry.put("466177400000171","[PA=1]");
		myHCountry.put("467767677403031","[IN=1]");
		myHCountry.put("473777377403771","[MG=1]");
		myHCountry.put("474077607177031","[TN=1]");
		myHCountry.put("474077777403031","[MY=1]");
		myHCountry.put("474377774000771","[AU=1]");
		myHCountry.put("475347777403621","[IN=1]");
		myHCountry.put("475367677003071","[IN=1]");
		myHCountry.put("475740002000071","[IN=1]");
		myHCountry.put("475747637403071","[NO=1]");
		myHCountry.put("475777774000031","[IN=1]");
		myHCountry.put("476000000000771","[FI=1]");
		myHCountry.put("476377774000371","[AT=1]");
		myHCountry.put("476773777777771","[IN=1]");
		myHCountry.put("477367036003071","[SA=1]");
		myHCountry.put("477417760000771","[PK=1]");
		myHCountry.put("477576770000000","[MX=1]");
		myHCountry.put("477623030000071","[PK=1]");
		myHCountry.put("477623760003031","[PK=1]");
		myHCountry.put("477637377417771","[MG=1]");
		myHCountry.put("477677376203741","[NO=1]");
		myHCountry.put("477707077403771","[MG=1]");
		myHCountry.put("477737357403771","[MG=1]");
		myHCountry.put("477737377403771","[MG=1]");
		myHCountry.put("477737377417771","[MG=1]");
		myHCountry.put("477767376000071","[MG=1]");
		myHCountry.put("477767740003071","[PK=1]");
		myHCountry.put("477767774000771","[ID=1]");
		myHCountry.put("477767777403001","[NO=1]");
		myHCountry.put("477767777403671","[IN=1]");
		myHCountry.put("477775777403771","[GW=1]");
		myHCountry.put("477776630000000","[GW=1]");
		myHCountry.put("477776770000000","[US=1]");
		myHCountry.put("477777077403771","[MG=1]");
		myHCountry.put("477777277005771","[BE=1]");
		myHCountry.put("477777277400771","[MG=1]");
		myHCountry.put("477777374003771","[US=1]");
		myHCountry.put("477777477403701","[MX=1]");
		myHCountry.put("477777740000011","[BR=1]");
		myHCountry.put("477777740000260","[PK=1]");
		myHCountry.put("477777770000001","[SG=1]");
		myHCountry.put("477777774000061","[PK=1]");
		myHCountry.put("477777774605431","[SD=1]");
		myHCountry.put("477777777000071","[IN=1]");
		myHCountry.put("477777777003171","[IN=1]");
		myHCountry.put("477777777400001","[FR=1]");
		myHCountry.put("477777777400771","[IN=1]");
		myHCountry.put("477777777401071","[TZ=1]");
		myHCountry.put("477777777401771","[TZ=1]");
		myHCountry.put("477777777426071","[IN=1]");
		myHCountry.put("500000004000771","[BR=1]");
		myHCountry.put("500173760003771","[IN=1]");
		myHCountry.put("501777770000001","[PK=1]");
		myHCountry.put("503767774003771","[IN=1]");
		myHCountry.put("503767777400671","[IR=1]");
		myHCountry.put("506044034000461","[RU=1]");
		myHCountry.put("510247600003771","[ID=1]");
		myHCountry.put("514132775577741","[BR=1]");
		myHCountry.put("515247004402261","[BD=1]");
		myHCountry.put("515347777766271","[ET=1]");
		myHCountry.put("517345777756461","[ET=1]");
		myHCountry.put("517367636700771","[MX=1]");
		myHCountry.put("517747637743771","[CN=1]");
		myHCountry.put("517777777743771","[IN=1]");
		myHCountry.put("535347777777661","[ET=1]");
		myHCountry.put("535357757403631","[IN=1]");
		myHCountry.put("535757777776671","[ET=1]");
		myHCountry.put("547777777600171","[MG=1]");
		myHCountry.put("555347434000771","[AL=1]");
		myHCountry.put("555347637743771","[BE=1]");
		myHCountry.put("555707437000261","[AL=1]");
		myHCountry.put("555727604643071","[IQ=1]");
		myHCountry.put("555747420000661","[AL=1]");
		myHCountry.put("555747434000771","[AL=1]");
		myHCountry.put("555767434000061","[AL=1]");
		myHCountry.put("555767774000000","[FR=1]");
		myHCountry.put("557347437700771","[AL=1]");
		myHCountry.put("557367617000761","[BR=1]");
		myHCountry.put("557707777000271","[AL=1]");
		myHCountry.put("557727777400771","[IQ=1]");
		myHCountry.put("557747634000771","[AL=1]");
		myHCountry.put("557747637743771","[CN=1]");
		myHCountry.put("557747737000271","[AL=1]");
		myHCountry.put("557767764000771","[BR=1]");
		myHCountry.put("557767777743771","[IQ=1]");
		myHCountry.put("560000000700011","[AT=1]");
		myHCountry.put("560060000000771","[FI=1]");
		myHCountry.put("571776770000000","[FR=1]");
		myHCountry.put("574077607777031","[TN=1]");
		myHCountry.put("574077607777771","[NG=1]");
		myHCountry.put("575067674000071","[CM=1]");
		myHCountry.put("575747637700631","[BR=1]");
		myHCountry.put("575766437700771","[ES=1]");
		myHCountry.put("575777777403731","[GF=1]");
		myHCountry.put("576177400000171","[VE=1]");
		myHCountry.put("576177637000371","[IT=1]");
		myHCountry.put("576377774300371","[IT=1]");
		myHCountry.put("577000007700761","[BR=1]");
		myHCountry.put("577347774000221","[IN=1]");
		myHCountry.put("577347777000261","[AL=1]");
		myHCountry.put("577367056403011","[TZ=1]");
		myHCountry.put("577367777403431","[TZ=1]");
		myHCountry.put("577377600000171","[BR=1]");
		myHCountry.put("577400000000171","[FR=1]");
		myHCountry.put("577707677403431","[MY=1]");
		myHCountry.put("577737770000771","[FR=1]");
		myHCountry.put("577737777400071","[OM=1]");
		myHCountry.put("577747620000771","[AL=1]");
		myHCountry.put("577747636700771","[BR=1]");
		myHCountry.put("577764000000061","[GF=1]");
		myHCountry.put("577767477400001","[ES=1]");
		myHCountry.put("577767637000271","[AL=1]");
		myHCountry.put("577767677700771","[ES=1]");
		myHCountry.put("577767775400031","[IN=1]");
		myHCountry.put("577767777700160","[AL=1]");
		myHCountry.put("577767777700771","[BR=1]");
		myHCountry.put("577775750003771","[BD=1]");
		myHCountry.put("577777577374771","[ET=1]");
		myHCountry.put("577777600000171","[FR=1]");
		myHCountry.put("577777607700771","[BR=1]");
		myHCountry.put("577777607741771","[BR=1]");
		myHCountry.put("577777720003771","[BD=1]");
		myHCountry.put("577777750003171","[BD=1]");
		myHCountry.put("577777757401371","[VN=1]");
		myHCountry.put("577777772000000","[MZ=1]");
		myHCountry.put("577777774000371","[IT=1]");
		myHCountry.put("577777774000771","[GW=1]");
		myHCountry.put("577777774003731","[BD=1]");
		myHCountry.put("577777776030371","[BR=1]");
		myHCountry.put("577777777400000","[CN=1]");
		myHCountry.put("577777777400371","[DE=1]");
		myHCountry.put("577777777400771","[IN=1]");
		myHCountry.put("577777777403771","[IN=1]");
		myHCountry.put("577777777743771","[CN=1]");
		myHCountry.put("577777777777771","[ET=1]");
		myHCountry.put("600000000000000","[US=1]");
		myHCountry.put("600000000000011","[CN=1]");
		myHCountry.put("600000003761771","[AZ=1]");
		myHCountry.put("600000360000371","[AL=1]");
		myHCountry.put("600001740003171","[SD=1]");
		myHCountry.put("600021740003171","[SE=1]");
		myHCountry.put("600273740003771","[IN=1]");
		myHCountry.put("600377740003771","[IN=1]");
		myHCountry.put("602377740003771","[OM=1]");
		myHCountry.put("603377400001771","[IT=1]");
		myHCountry.put("603777400001771","[US=1]");
		myHCountry.put("603777700000771","[SA=1]");
		myHCountry.put("603777740000071","[SA=1]");
		myHCountry.put("603777740003371","[OM=1]");
		myHCountry.put("603777740003731","[BE=1]");
		myHCountry.put("603777747770371","[IR=1]");
		myHCountry.put("607747521100031","[ID=1]");
		myHCountry.put("607777700003771","[ZA=1]");
		myHCountry.put("607777776000371","[US=1]");
		myHCountry.put("617736000000000","[IT=1]");
		myHCountry.put("617777777700771","[PE=1]");
		myHCountry.put("636177400000171","[PE=1]");
		myHCountry.put("636377770000731","[ES=1]");
		myHCountry.put("637774740000130","[US=1]");
		myHCountry.put("640033777777710","[MX=1]");
		myHCountry.put("643677607741771","[BR=1]");
		myHCountry.put("647677607741771","[BR=1]");
		myHCountry.put("657777607541771","[BR=1]");
		myHCountry.put("660001400000171","[FR=1]");
		myHCountry.put("660003400000171","[FR=1]");
		myHCountry.put("663600007400071","[US=1]");
		myHCountry.put("667617777777600","[BR=1]");
		myHCountry.put("667777470000000","[AU=1]");
		myHCountry.put("667777777777771","[TZ=1]");
		myHCountry.put("674000003417771","[BE=1]");
		myHCountry.put("674377777403771","[TH=1]");
		myHCountry.put("674573777777601","[US=1]");
		myHCountry.put("674777470003771","[PH=1]");
		myHCountry.put("676000000000171","[BR=1]");
		myHCountry.put("676177400000371","[IT=1]");
		myHCountry.put("676177770000010","[FR=1]");
		myHCountry.put("676357760000171","[ID=1]");
		myHCountry.put("676377736000131","[US=1]");
		myHCountry.put("676377760000171","[GF=1]");
		myHCountry.put("676773677777700","[NL=1]");
		myHCountry.put("676773777777710","[MX=1]");
		myHCountry.put("676777477401771","[ES=1]");
		myHCountry.put("676777677777601","[ES=1]");
		myHCountry.put("677001607700771","[VE=1]");
		myHCountry.put("677367776177661","[ES=1]");
		myHCountry.put("677377776000271","[IT=1]");
		myHCountry.put("677400000001771","[US=1]");
		myHCountry.put("677477776000331","[ES=1]");
		myHCountry.put("677607477403771","[ES=1]");
		myHCountry.put("677637607600740","[BR=1]");
		myHCountry.put("677677417741671","[IN=1]");
		myHCountry.put("677677607741771","[BR=1]");
		myHCountry.put("677707607700771","[VE=1]");
		myHCountry.put("677723607400000","[BR=1]");
		myHCountry.put("677737607400731","[CU=1]");
		myHCountry.put("677737607600740","[BR=1]");
		myHCountry.put("677737607600771","[BR=1]");
		myHCountry.put("677737607700011","[ES=1]");
		myHCountry.put("677742000001771","[TW=1]");
		myHCountry.put("677767477403631","[US=1]");
		myHCountry.put("677767477403771","[OM=1]");
		myHCountry.put("677767676003771","[JP=1]");
		myHCountry.put("677767774000371","[BG=1]");
		myHCountry.put("677770000000771","[AU=1]");
		myHCountry.put("677775000001771","[US=1]");
		myHCountry.put("677777000000331","[FR=1]");
		myHCountry.put("677777377700771","[ID=1]");
		myHCountry.put("677777402003771","[US=1]");
		myHCountry.put("677777440003770","[GB=1]");
		myHCountry.put("677777470000771","[US=1]");
		myHCountry.put("677777470003731","[US=1]");
		myHCountry.put("677777470003771","[GB=1]");
		myHCountry.put("677777472000071","[SA=1]");
		myHCountry.put("677777475000051","[ID=1]");
		myHCountry.put("677777477003771","[OM=1]");
		myHCountry.put("677777477417771","[SA=1]");
		myHCountry.put("677777600000000","[MX=1]");
		myHCountry.put("677777600000371","[IT=1]");
		myHCountry.put("677777607700031","[HT=1]");
		myHCountry.put("677777607700071","[LV=1]");
		myHCountry.put("677777607700371","[BR=1]");
		myHCountry.put("677777607741771","[BR=1]");
		myHCountry.put("677777770000000","[IT=1]");
		myHCountry.put("677777777003771","[GW=1]");
		myHCountry.put("677777777400000","[IN=1]");
		myHCountry.put("677777777403701","[GW=1]");
		myHCountry.put("677777777757771","[SA=1]");
		myHCountry.put("677777777777600","[TN=1]");
		myHCountry.put("700000000403371","[BD=1]");
		myHCountry.put("700000004037771","[ET=1]");
		myHCountry.put("700003760000171","[IN=1]");
		myHCountry.put("700007777770371","[FR=1]");
		myHCountry.put("700036774000011","[US=1]");
		myHCountry.put("700066777400771","[MZ=1]");
		myHCountry.put("700076777700771","[US=1]");
		myHCountry.put("700137617000000","[ES=1]");
		myHCountry.put("700167760001771","[IN=1]");
		myHCountry.put("700177760000171","[OM=1]");
		myHCountry.put("700775745403771","[MG=1]");
		myHCountry.put("700775747417771","[GF=1]");
		myHCountry.put("700777770403731","[PK=1]");
		myHCountry.put("701347604001661","[ZM=1]");
		myHCountry.put("701740007774671","[US=1]");
		myHCountry.put("701773760003171","[EG=1]");
		myHCountry.put("701777777403771","[IN=1]");
		myHCountry.put("703227660003771","[PK=1]");
		myHCountry.put("703363464001771","[ET=1]");
		myHCountry.put("703367607543771","[PK=1]");
		myHCountry.put("703377420001771","[GB=1]");
		myHCountry.put("703377740005471","[SD=1]");
		myHCountry.put("703377777400371","[IR=1]");
		myHCountry.put("703377777761771","[ZA=1]");
		myHCountry.put("703637745770371","[PK=1]");
		myHCountry.put("703707730003771","[SA=1]");
		myHCountry.put("703737770003171","[SD=1]");
		myHCountry.put("703741444002371","[IN=1]");
		myHCountry.put("703747740006171","[PK=1]");
		myHCountry.put("703747777777421","[BE=1]");
		myHCountry.put("703757760001771","[SE=1]");
		myHCountry.put("703760020000331","[GB=1]");
		myHCountry.put("703767434001671","[IR=1]");
		myHCountry.put("703767740201371","[FR=1]");
		myHCountry.put("703767770003771","[SA=1]");
		myHCountry.put("703767774003571","[SA=1]");
		myHCountry.put("703767777003771","[IR=1]");
		myHCountry.put("703771747777771","[SD=1]");
		myHCountry.put("703777020003771","[GB=1]");
		myHCountry.put("703777640007760","[IT=1]");
		myHCountry.put("703777700004071","[SD=1]");
		myHCountry.put("703777700005471","[SD=1]");
		myHCountry.put("703777740007770","[IT=1]");
		myHCountry.put("703777742003771","[SA=1]");
		myHCountry.put("703777744034771","[US=1]");
		myHCountry.put("703777747570371","[PK=1]");
		myHCountry.put("703777747670371","[GB=1]");
		myHCountry.put("703777747770171","[IN=1]");
		myHCountry.put("703777747770271","[OM=1]");
		myHCountry.put("703777756002771","[IR=1]");
		myHCountry.put("703777757403771","[FR=1]");
		myHCountry.put("703777760000171","[GB=1]");
		myHCountry.put("703777760000371","[BD=1]");
		myHCountry.put("703777760003010","[GB=1]");
		myHCountry.put("703777760003131","[SE=1]");
		myHCountry.put("703777760003171","[GB=1]");
		myHCountry.put("703777774000171","[GB=1]");
		myHCountry.put("703777774003771","[IN=1]");
		myHCountry.put("703777776003471","[IN=1]");
		myHCountry.put("703777777403771","[IN=1]");
		myHCountry.put("703777777747771","[IN=1]");
		myHCountry.put("703777777770371","[IN=1]");
		myHCountry.put("706377710000671","[SA=1]");
		myHCountry.put("707377776003771","[SA=1]");
		myHCountry.put("707706000000000","[DE=1]");
		myHCountry.put("707757741603131","[ID=1]");
		myHCountry.put("707761740003771","[GB=1]");
		myHCountry.put("707777740003000","[MY=1]");
		myHCountry.put("707777740003771","[CN=1]");
		myHCountry.put("707777742037171","[SA=1]");
		myHCountry.put("707777760000740","[GB=1]");
		myHCountry.put("710003600077761","[IT=1]");
		myHCountry.put("710777777777631","[GM=1]");
		myHCountry.put("711022103370371","[MX=1]");
		myHCountry.put("711067777706771","[MX=1]");
		myHCountry.put("711767435002430","[ID=1]");
		myHCountry.put("711777640003771","[NL=1]");
		myHCountry.put("712366674303000","[ID=1]");
		myHCountry.put("713757777743771","[IN=1]");
		myHCountry.put("714140377776741","[IT=1]");
		myHCountry.put("714566703670340","[BR=1]");
		myHCountry.put("716077670000000","[ID=1]");
		myHCountry.put("716763677756200","[BR=1]");
		myHCountry.put("717337706001371","[BD=1]");
		myHCountry.put("717603752000000","[MZ=1]");
		myHCountry.put("717727777764671","[IT=1]");
		myHCountry.put("717747761700771","[ID=1]");
		myHCountry.put("717747764000771","[ID=1]");
		myHCountry.put("717767730000471","[ID=1]");
		myHCountry.put("717767740003771","[IN=1]");
		myHCountry.put("717767777503771","[JP=1]");
		myHCountry.put("717777757777771","[US=1]");
		myHCountry.put("717777777400771","[AT=1]");
		myHCountry.put("717777777703771","[CN=1]");
		myHCountry.put("727777777000171","[LV=1]");
		myHCountry.put("730377600003771","[ZW=1]");
		myHCountry.put("735777774001740","[IN=1]");
		myHCountry.put("736000000007771","[SA=1]");
		myHCountry.put("737003400700771","[ID=1]");
		myHCountry.put("737377606061771","[MW=1]");
		myHCountry.put("737377777761771","[IN=1]");
		myHCountry.put("737637770000731","[TW=1]");
		myHCountry.put("737736770000000","[BR=1]");
		myHCountry.put("737747606100000","[ID=1]");
		myHCountry.put("737767770000000","[ID=1]");
		myHCountry.put("737777677670371","[ZW=1]");
		myHCountry.put("737777710000071","[DE=1]");
		myHCountry.put("737777740003771","[IR=1]");
		myHCountry.put("737777777401761","[US=1]");
		myHCountry.put("737777777403771","[US=1]");
		myHCountry.put("737777777700000","[AT=1]");
		myHCountry.put("737777777700171","[IT=1]");
		myHCountry.put("737777777761771","[TZ=1]");
		myHCountry.put("737777777767771","[JP=1]");
		myHCountry.put("740000000000031","[SE=1]");
		myHCountry.put("740000004024771","[IT=1]");
		myHCountry.put("740177400000171","[IT=1]");
		myHCountry.put("740376770000000","[GR=1]");
		myHCountry.put("740377740003771","[ZA=1]");
		myHCountry.put("740657600003571","[ID=1]");
		myHCountry.put("740737777777771","[SN=1]");
		myHCountry.put("740757600000171","[ID=1]");
		myHCountry.put("740777674000011","[GF=1]");
		myHCountry.put("740777757777771","[US=1]");
		myHCountry.put("741737777400771","[JP=1]");
		myHCountry.put("741777740003771","[ZA=1]");
		myHCountry.put("742161047700771","[ZA=1]");
		myHCountry.put("742301040003771","[IN=1]");
		myHCountry.put("743077740003171","[IN=1]");
		myHCountry.put("743567777401731","[BE=1]");
		myHCountry.put("743717670000731","[SD=1]");
		myHCountry.put("743737740003171","[US=1]");
		myHCountry.put("743767740003171","[IQ=1]");
		myHCountry.put("743777040003771","[IN=1]");
		myHCountry.put("743777600003771","[IN=1]");
		myHCountry.put("743777760003771","[IN=1]");
		myHCountry.put("747763600000000","[FR=1]");
		myHCountry.put("747777577743771","[SA=1]");
		myHCountry.put("747777600000771","[BR=1]");
		myHCountry.put("747777634000000","[ES=1]");
		myHCountry.put("747777776001731","[BE=1]");
		myHCountry.put("753765247703771","[ZA=1]");
		myHCountry.put("754100777676701","[IT=1]");
		myHCountry.put("755567677402631","[MZ=1]");
		myHCountry.put("756377770000731","[CO=1]");
		myHCountry.put("756377774000731","[US=1]");
		myHCountry.put("757357777777371","[ET=1]");
		myHCountry.put("757430000000371","[FR=1]");
		myHCountry.put("757700000003371","[US=1]");
		myHCountry.put("757737777403731","[ZA=1]");
		myHCountry.put("757741477776761","[ET=1]");
		myHCountry.put("757747524000771","[ID=1]");
		myHCountry.put("757757764000771","[ID=1]");
		myHCountry.put("757767400741771","[IQ=1]");
		myHCountry.put("757767740003171","[IQ=1]");
		myHCountry.put("757767774000671","[ID=1]");
		myHCountry.put("757767777300771","[GP=1]");
		myHCountry.put("757767777743771","[IQ=1]");
		myHCountry.put("757777600000331","[BR=1]");
		myHCountry.put("757777600003371","[BD=1]");
		myHCountry.put("757777700000771","[IT=1]");
		myHCountry.put("757777744743771","[IQ=1]");
		myHCountry.put("757777746000771","[US=1]");
		myHCountry.put("757777770000000","[FR=1]");
		myHCountry.put("757777774000000","[IT=1]");
		myHCountry.put("757777774000771","[DE=1]");
		myHCountry.put("757777776000761","[BE=1]");
		myHCountry.put("757777777700331","[CN=1]");
		myHCountry.put("757777777700371","[IT=1]");
		myHCountry.put("760000000000000","[IT=1]");
		myHCountry.put("760000000000031","[IT=1]");
		myHCountry.put("760000000000771","[HN=1]");
		myHCountry.put("760000000003771","[GP=1]");
		myHCountry.put("760000171027771","[US=1]");
		myHCountry.put("760001500000171","[BR=1]");
		myHCountry.put("760001600000371","[BR=1]");
		myHCountry.put("760003400000171","[FR=1]");
		myHCountry.put("760760007700771","[ZA=1]");
		myHCountry.put("760770047700771","[TN=1]");
		myHCountry.put("761760000000000","[ID=1]");
		myHCountry.put("761767776000371","[EG=1]");
		myHCountry.put("761777607761771","[US=1]");
		myHCountry.put("761777777600171","[IT=1]");
		myHCountry.put("763000077757000","[SA=1]");
		myHCountry.put("763000077757031","[SA=1]");
		myHCountry.put("763000077777031","[TW=1]");
		myHCountry.put("767717737757000","[IT=1]");
		myHCountry.put("767757740000000","[ID=1]");
		myHCountry.put("767757740000771","[ID=1]");
		myHCountry.put("767767740000000","[SA=1]");
		myHCountry.put("767776377003731","[US=1]");
		myHCountry.put("767776777770371","[GB=1]");
		myHCountry.put("770000000003771","[RU=1]");
		myHCountry.put("770000700000000","[RU=1]");
		myHCountry.put("770000740000331","[BG=1]");
		myHCountry.put("770000777600031","[DK=1]");
		myHCountry.put("770003606377031","[IT=1]");
		myHCountry.put("770077757400771","[MX=1]");
		myHCountry.put("770141000000000","[IT=1]");
		myHCountry.put("770337740000177","[IT=1]");
		myHCountry.put("770363777777600","[IT=1]");
		myHCountry.put("770377756003771","[IN=1]");
		myHCountry.put("770777747777771","[NG=1]");
		myHCountry.put("770777770000000","[CM=1]");
		myHCountry.put("770777777000000","[GW=1]");
		myHCountry.put("770777777770371","[OM=1]");
		myHCountry.put("770777777777600","[GW=1]");
		myHCountry.put("770777777777611","[GW=1]");
		myHCountry.put("771375600000771","[BR=1]");
		myHCountry.put("771773777403600","[MY=1]");
		myHCountry.put("771777600000000","[BR=1]");
		myHCountry.put("772000007775671","[FR=1]");
		myHCountry.put("772600000000671","[BE=1]");
		myHCountry.put("772607777777671","[GW=1]");
		myHCountry.put("772700000003671","[NL=1]");
		myHCountry.put("772700200003671","[IT=1]");
		myHCountry.put("772701000003631","[NL=1]");
		myHCountry.put("772777700000000","[BE=1]");
		myHCountry.put("772777777777671","[FR=1]");
		myHCountry.put("773377740003771","[IN=1]");
		myHCountry.put("773377777761771","[ET=1]");
		myHCountry.put("773767477770371","[ET=1]");
		myHCountry.put("773777577503771","[SA=1]");
		myHCountry.put("773777600000771","[US=1]");
		myHCountry.put("773777676000771","[ET=1]");
		myHCountry.put("773777740003171","[SD=1]");
		myHCountry.put("773777740003471","[IN=1]");
		myHCountry.put("773777747403771","[SA=1]");
		myHCountry.put("773777760000771","[JP=1]");
		myHCountry.put("773777770000000","[CN=1]");
		myHCountry.put("773777770001771","[SA=1]");
		myHCountry.put("774000000000371","[KG=1]");
		myHCountry.put("774000003770331","[MY=1]");
		myHCountry.put("774000730000000","[IT=1]");
		myHCountry.put("774037600000000","[FR=1]");
		myHCountry.put("774037750000371","[FR=1]");
		myHCountry.put("774040070000001","[FR=1]");
		myHCountry.put("774040077777031","[TN=1]");
		myHCountry.put("774063777777600","[GB=1]");
		myHCountry.put("774067400603031","[FR=1]");
		myHCountry.put("774067402603031","[FR=1]");
		myHCountry.put("774067407777031","[TN=1]");
		myHCountry.put("774067607777031","[CM=1]");
		myHCountry.put("774076000003031","[NG=1]");
		myHCountry.put("774077607777000","[US=1]");
		myHCountry.put("774077772001031","[FR=1]");
		myHCountry.put("774167777717771","[SA=1]");
		myHCountry.put("774177747757771","[SA=1]");
		myHCountry.put("774177773717700","[SA=1]");
		myHCountry.put("774177777357771","[SA=1]");
		myHCountry.put("774177777707740","[SA=1]");
		myHCountry.put("774177777717771","[SA=1]");
		myHCountry.put("774737777737771","[IN=1]");
		myHCountry.put("774777774000771","[RU=1]");
		myHCountry.put("775147677700000","[RU=1]");
		myHCountry.put("775345437774271","[ET=1]");
		myHCountry.put("775367637700771","[BR=1]");
		myHCountry.put("775375777777601","[ET=1]");
		myHCountry.put("775377777776771","[ET=1]");
		myHCountry.put("775577747777671","[NG=1]");
		myHCountry.put("775706767703671","[MY=1]");
		myHCountry.put("775757777776771","[ET=1]");
		myHCountry.put("775777770000000","[FR=1]");
		myHCountry.put("775777777401771","[TH=1]");
		myHCountry.put("775777777403771","[US=1]");
		myHCountry.put("775777777777471","[NL=1]");
		myHCountry.put("776000100000171","[ID=1]");
		myHCountry.put("776017400000171","[US=1]");
		myHCountry.put("776037400000771","[NL=1]");
		myHCountry.put("776067600000771","[PT=1]");
		myHCountry.put("776077703770371","[FR=1]");
		myHCountry.put("776107400000171","[BR=1]");
		myHCountry.put("776127400000171","[ES=1]");
		myHCountry.put("776141607761771","[US=1]");
		myHCountry.put("776157400000171","[BD=1]");
		myHCountry.put("776161004000071","[ES=1]");
		myHCountry.put("776167400000171","[ES=1]");
		myHCountry.put("776173400000071","[ES=1]");
		myHCountry.put("776174000000071","[ES=1]");
		myHCountry.put("776177404000171","[IT=1]");
		myHCountry.put("776177600000771","[PT=1]");
		myHCountry.put("776177607000371","[IT=1]");
		myHCountry.put("776177607000771","[HN=1]");
		myHCountry.put("776177607770371","[ES=1]");
		myHCountry.put("776177677500071","[ES=1]");
		myHCountry.put("776177760000000","[TR=1]");
		myHCountry.put("776177770000731","[ES=1]");
		myHCountry.put("776177770000771","[TR=1]");
		myHCountry.put("776177774030171","[ES=1]");
		myHCountry.put("776217777777771","[MY=1]");
		myHCountry.put("776337777000071","[CU=1]");
		myHCountry.put("776340000000071","[BR=1]");
		myHCountry.put("776357777700371","[MX=1]");
		myHCountry.put("776363400000171","[ES=1]");
		myHCountry.put("776367770000731","[CO=1]");
		myHCountry.put("776377560000731","[ES=1]");
		myHCountry.put("776377600000371","[IT=1]");
		myHCountry.put("776377607600371","[ES=1]");
		myHCountry.put("776377700000001","[IT=1]");
		myHCountry.put("776377700000611","[FR=1]");
		myHCountry.put("776377740000031","[ZA=1]");
		myHCountry.put("776377760000371","[US=1]");
		myHCountry.put("776377760000731","[US=1]");
		myHCountry.put("776377770200771","[BR=1]");
		myHCountry.put("776377774000371","[FR=1]");
		myHCountry.put("776377777000771","[IT=1]");
		myHCountry.put("776377777730371","[FR=1]");
		myHCountry.put("776460647015740","[GB=1]");
		myHCountry.put("776541402711340","[GB=1]");
		myHCountry.put("776573777777600","[US=1]");
		myHCountry.put("776737607770371","[BR=1]");
		myHCountry.put("776767777777771","[BR=1]");
		myHCountry.put("776777400000001","[BR=1]");
		myHCountry.put("776777600000720","[AU=1]");
		myHCountry.put("776777604000771","[GM=1]");
		myHCountry.put("776777770000731","[ES=1]");
		myHCountry.put("776777776000271","[ES=1]");
		myHCountry.put("776777777777671","[ES=1]");
		myHCountry.put("777000000000771","[RU=1]");
		myHCountry.put("777000002000000","[SE=1]");
		myHCountry.put("777000200000000","[FR=1]");
		myHCountry.put("777000323506471","[SA=1]");
		myHCountry.put("777007773757700","[SA=1]");
		myHCountry.put("777077700003371","[GE=1]");
		myHCountry.put("777177774000771","[BR=1]");
		myHCountry.put("777177777510171","[ES=1]");
		myHCountry.put("777207773757741","[SA=1]");
		myHCountry.put("777327677774771","[ET=1]");
		myHCountry.put("777347677403671","[IN=1]");
		myHCountry.put("777357677774671","[ET=1]");
		myHCountry.put("777357777542771","[IN=1]");
		myHCountry.put("777357777777251","[ET=1]");
		myHCountry.put("777357777777771","[ET=1]");
		myHCountry.put("777367400003771","[TZ=1]");
		myHCountry.put("777367477400771","[ES=1]");
		myHCountry.put("777367756403671","[BD=1]");
		myHCountry.put("777377577771771","[SA=1]");
		myHCountry.put("777377677777771","[ZA=1]");
		myHCountry.put("777377702003371","[GF=1]");
		myHCountry.put("777377706001351","[BD=1]");
		myHCountry.put("777377706001371","[US=1]");
		myHCountry.put("777377740003371","[GF=1]");
		myHCountry.put("777377770000000","[IT=1]");
		myHCountry.put("777377770000131","[CN=1]");
		myHCountry.put("777377770000671","[ES=1]");
		myHCountry.put("777377777757771","[SA=1]");
		myHCountry.put("777377777770371","[BR=1]");
		myHCountry.put("777377777777771","[ET=1]");
		myHCountry.put("777400037770371","[FR=1]");
		myHCountry.put("777402000000171","[RU=1]");
		myHCountry.put("777407773757741","[SA=1]");
		myHCountry.put("777417600000000","[US=1]");
		myHCountry.put("777417700000000","[GW=1]");
		myHCountry.put("777417707700000","[FR=1]");
		myHCountry.put("777417730000000","[US=1]");
		myHCountry.put("777437707700000","[GM=1]");
		myHCountry.put("777467607500671","[LV=1]");
		myHCountry.put("777477777407371","[SA=1]");
		myHCountry.put("777557757003331","[US=1]");
		myHCountry.put("777557777003731","[US=1]");
		myHCountry.put("777577604000771","[VE=1]");
		myHCountry.put("777577757003300","[BD=1]");
		myHCountry.put("777577767777771","[FR=1]");
		myHCountry.put("777577777001731","[BD=1]");
		myHCountry.put("777577777003771","[BD=1]");
		myHCountry.put("777577777403171","[VN=1]");
		myHCountry.put("777577777417071","[VN=1]");
		myHCountry.put("777600000000031","[RU=1]");
		myHCountry.put("777600000000171","[IT=1]");
		myHCountry.put("777600007400261","[BR=1]");
		myHCountry.put("777601760000171","[US=1]");
		myHCountry.put("777603607400011","[US=1]");
		myHCountry.put("777607770000731","[ID=1]");
		myHCountry.put("777617017777771","[TW=1]");
		myHCountry.put("777617600000000","[BR=1]");
		myHCountry.put("777617777700000","[IT=1]");
		myHCountry.put("777637600500771","[PK=1]");
		myHCountry.put("777637770000000","[CN=1]");
		myHCountry.put("777637777777771","[JP=1]");
		myHCountry.put("777660040000071","[IN=1]");
		myHCountry.put("777667777700000","[ID=1]");
		myHCountry.put("777677207600771","[BR=1]");
		myHCountry.put("777677647403371","[MG=1]");
		myHCountry.put("777677700041771","[TW=1]");
		myHCountry.put("777677774014351","[SD=1]");
		myHCountry.put("777700000000011","[US=1]");
		myHCountry.put("777700000000071","[US=1]");
		myHCountry.put("777700000000671","[FR=1]");
		myHCountry.put("777700000001031","[OM=1]");
		myHCountry.put("777700000001631","[US=1]");
		myHCountry.put("777700002001731","[US=1]");
		myHCountry.put("777700043700171","[IT=1]");
		myHCountry.put("777700654000000","[BR=1]");
		myHCountry.put("777700700000611","[FR=1]");
		myHCountry.put("777700770000071","[US=1]");
		myHCountry.put("777701000000431","[FR=1]");
		myHCountry.put("777701002001431","[OM=1]");
		myHCountry.put("777701400000011","[VN=1]");
		myHCountry.put("777701400000300","[FR=1]");
		myHCountry.put("777702004001731","[IN=1]");
		myHCountry.put("777702017400771","[DK=1]");
		myHCountry.put("777703700003371","[US=1]");
		myHCountry.put("777703757761771","[US=1]");
		myHCountry.put("777703774000011","[BR=1]");
		myHCountry.put("777703777000071","[IT=1]");
		myHCountry.put("777703777400371","[CM=1]");
		myHCountry.put("777707737777771","[MY=1]");
		myHCountry.put("777707760000171","[US=1]");
		myHCountry.put("777717630000000","[CN=1]");
		myHCountry.put("777721000001731","[IN=1]");
		myHCountry.put("777721002001431","[IN=1]");
		myHCountry.put("777723002001731","[IN=1]");
		myHCountry.put("777727000000471","[PK=1]");
		myHCountry.put("777727400000771","[FR=1]");
		myHCountry.put("777727770000000","[ET=1]");
		myHCountry.put("777727770000071","[ZA=1]");
		myHCountry.put("777727770000771","[ID=1]");
		myHCountry.put("777727777000771","[IQ=1]");
		myHCountry.put("777727777770351","[BR=1]");
		myHCountry.put("777731603770371","[FR=1]");
		myHCountry.put("777734277700771","[US=1]");
		myHCountry.put("777736570000000","[GR=1]");
		myHCountry.put("777736600000000","[VE=1]");
		myHCountry.put("777737240000031","[MX=1]");
		myHCountry.put("777737340000371","[SE=1]");
		myHCountry.put("777737376000171","[BG=1]");
		myHCountry.put("777737400000011","[BE=1]");
		myHCountry.put("777737400003771","[DK=1]");
		myHCountry.put("777737405777731","[US=1]");
		myHCountry.put("777737407777731","[US=1]");
		myHCountry.put("777737477400001","[ES=1]");
		myHCountry.put("777737570000000","[IT=1]");
		myHCountry.put("777737600000171","[US=1]");
		myHCountry.put("777737603774351","[SD=1]");
		myHCountry.put("777737605700771","[BE=1]");
		myHCountry.put("777737606000171","[VE=1]");
		myHCountry.put("777737607400031","[BG=1]");
		myHCountry.put("777737607700351","[US=1]");
		myHCountry.put("777737607700371","[TN=1]");
		myHCountry.put("777737637400071","[AL=1]");
		myHCountry.put("777737700000001","[SE=1]");
		myHCountry.put("777737700000171","[FR=1]");
		myHCountry.put("777737703605751","[IN=1]");
		myHCountry.put("777737710000131","[AT=1]");
		myHCountry.put("777737737741771","[OM=1]");
		myHCountry.put("777737740000071","[AT=1]");
		myHCountry.put("777737747001771","[BE=1]");
		myHCountry.put("777737760177731","[CN=1]");
		myHCountry.put("777737770000371","[IR=1]");
		myHCountry.put("777737770770041","[SA=1]");
		myHCountry.put("777737774403431","[US=1]");
		myHCountry.put("777737777600011","[PL=1]");
		myHCountry.put("777737777600171","[AT=1]");
		myHCountry.put("777737777700011","[IT=1]");
		myHCountry.put("777737777721771","[GB=1]");
		myHCountry.put("777737777754771","[NL=1]");
		myHCountry.put("777737777761711","[BD=1]");
		myHCountry.put("777737777770351","[BR=1]");
		myHCountry.put("777740000000011","[AT=1]");
		myHCountry.put("777740000000071","[GE=1]");
		myHCountry.put("777740017000331","[IT=1]");
		myHCountry.put("777741000000000","[MA=1]");
		myHCountry.put("777741343700000","[NG=1]");
		myHCountry.put("777741370000731","[BG=1]");
		myHCountry.put("777742011003671","[IT=1]");
		myHCountry.put("777747376776771","[ET=1]");
		myHCountry.put("777747434000371","[AL=1]");
		myHCountry.put("777747761700771","[ID=1]");
		myHCountry.put("777747770000771","[BR=1]");
		myHCountry.put("777747777777771","[US=1]");
		myHCountry.put("777753777742070","[CZ=1]");
		myHCountry.put("777754377777730","[ET=1]");
		myHCountry.put("777757360000171","[MX=1]");
		myHCountry.put("777757377000371","[FR=1]");
		myHCountry.put("777757377607771","[FR=1]");
		myHCountry.put("777757700003371","[BD=1]");
		myHCountry.put("777757776764661","[ET=1]");
		myHCountry.put("777757777403771","[TH=1]");
		myHCountry.put("777757777700000","[ID=1]");
		myHCountry.put("777760000000771","[AU=1]");
		myHCountry.put("777760007600771","[ES=1]");
		myHCountry.put("777760044000771","[IN=1]");
		myHCountry.put("777760047500771","[ZA=1]");
		myHCountry.put("777760070000771","[CO=1]");
		myHCountry.put("777760370000000","[ET=1]");
		myHCountry.put("777761000000031","[PT=1]");
		myHCountry.put("777761000000171","[SE=1]");
		myHCountry.put("777761000001771","[IN=1]");
		myHCountry.put("777761007600771","[ES=1]");
		myHCountry.put("777762047700771","[ZA=1]");
		myHCountry.put("777763047700771","[ZA=1]");
		myHCountry.put("777763477400001","[NL=1]");
		myHCountry.put("777763777700000","[IT=1]");
		myHCountry.put("777763777777771","[NL=1]");
		myHCountry.put("777767477342671","[IN=1]");
		myHCountry.put("777767477403671","[MX=1]");
		myHCountry.put("777767600000000","[BR=1]");
		myHCountry.put("777767600000011","[ES=1]");
		myHCountry.put("777767600000071","[LY=1]");
		myHCountry.put("777767600000731","[BR=1]");
		myHCountry.put("777767607700771","[BR=1]");
		myHCountry.put("777767637741771","[MX=1]");
		myHCountry.put("777767677403631","[IN=1]");
		myHCountry.put("777767702003371","[OM=1]");
		myHCountry.put("777767740000371","[JP=1]");
		myHCountry.put("777767747743540","[GB=1]");
		myHCountry.put("777767760000371","[FR=1]");
		myHCountry.put("777767770000771","[IQ=1]");
		myHCountry.put("777767774000671","[ID=1]");
		myHCountry.put("777767776000271","[MY=1]");
		myHCountry.put("777767777400771","[MZ=1]");
		myHCountry.put("777767777403671","[ID=1]");
		myHCountry.put("777767777777771","[JP=1]");
		myHCountry.put("777770007600771","[ES=1]");
		myHCountry.put("777770077700771","[US=1]");
		myHCountry.put("777771000000071","[PE=1]");
		myHCountry.put("777771000000771","[FR=1]");
		myHCountry.put("777771340000071","[SE=1]");
		myHCountry.put("777773000000071","[FR=1]");
		myHCountry.put("777773000000731","[FR=1]");
		myHCountry.put("777773002000011","[VN=1]");
		myHCountry.put("777773374000000","[DK=1]");
		myHCountry.put("777773677777600","[ES=1]");
		myHCountry.put("777773740000031","[FR=1]");
		myHCountry.put("777773740000071","[US=1]");
		myHCountry.put("777773750000000","[IT=1]");
		myHCountry.put("777773760000171","[US=1]");
		myHCountry.put("777773773742070","[CZ=1]");
		myHCountry.put("777773776000371","[SA=1]");
		myHCountry.put("777774000000371","[US=1]");
		myHCountry.put("777774005777731","[CN=1]");
		myHCountry.put("777774377403400","[SA=1]");
		myHCountry.put("777775000000000","[IN=1]");
		myHCountry.put("777775000000371","[FR=1]");
		myHCountry.put("777775003770351","[SD=1]");
		myHCountry.put("777775070000171","[ES=1]");
		myHCountry.put("777775376000711","[IT=1]");
		myHCountry.put("777775606000731","[MZ=1]");
		myHCountry.put("777775607700771","[ZA=1]");
		myHCountry.put("777775750003771","[BD=1]");
		myHCountry.put("777775777577571","[SA=1]");
		myHCountry.put("777776000000071","[KG=1]");
		myHCountry.put("777776000000771","[ID=1]");
		myHCountry.put("777776700000000","[CM=1]");
		myHCountry.put("777776757403771","[IT=1]");
		myHCountry.put("777776774000000","[IT=1]");
		myHCountry.put("777776776000601","[US=1]");
		myHCountry.put("777776777400731","[IN=1]");
		myHCountry.put("777776777600001","[US=1]");
		myHCountry.put("777776777761771","[US=1]");
		myHCountry.put("777776777767711","[MX=1]");
		myHCountry.put("777776777777771","[TW=1]");
		myHCountry.put("777777000000001","[BD=1]");
		myHCountry.put("777777000000071","[FR=1]");
		myHCountry.put("777777000000371","[US=1]");
		myHCountry.put("777777000000771","[FI=1]");
		myHCountry.put("777777007600331","[BR=1]");
		myHCountry.put("777777037700771","[FR=1]");
		myHCountry.put("777777077400001","[MA=1]");
		myHCountry.put("777777207400171","[BR=1]");
		myHCountry.put("777777207600371","[BR=1]");
		myHCountry.put("777777300000171","[TN=1]");
		myHCountry.put("777777347400071","[MQ=1]");
		myHCountry.put("777777350000171","[BD=1]");
		myHCountry.put("777777350000371","[BD=1]");
		myHCountry.put("777777373761631","[ID=1]");
		myHCountry.put("777777377000771","[ZA=1]");
		myHCountry.put("777777377700771","[IT=1]");
		myHCountry.put("777777377766771","[IT=1]");
		myHCountry.put("777777400000000","[HT=1]");
		myHCountry.put("777777400000731","[US=1]");
		myHCountry.put("777777400000771","[IT=1]");
		myHCountry.put("777777400400071","[BD=1]");
		myHCountry.put("777777402000371","[FR=1]");
		myHCountry.put("777777402374371","[SD=1]");
		myHCountry.put("777777407700071","[GB=1]");
		myHCountry.put("777777407777771","[TH=1]");
		myHCountry.put("777777414141571","[GB=1]");
		myHCountry.put("777777414147771","[FR=1]");
		myHCountry.put("777777417300771","[BR=1]");
		myHCountry.put("777777437500000","[FI=1]");
		myHCountry.put("777777440003771","[US=1]");
		myHCountry.put("777777470000771","[GE=1]");
		myHCountry.put("777777476000001","[CU=1]");
		myHCountry.put("777777477700731","[US=1]");
		myHCountry.put("777777477777771","[IR=1]");
		myHCountry.put("777777557700771","[PE=1]");
		myHCountry.put("777777557777771","[CN=1]");
		myHCountry.put("777777576000000","[CM=1]");
		myHCountry.put("777777600000200","[KR=1]");
		myHCountry.put("777777600000611","[JP=1]");
		myHCountry.put("777777600000711","[DE=1]");
		myHCountry.put("777777600003371","[BD=1]");
		myHCountry.put("777777600003771","[RU=1]");
		myHCountry.put("777777600500771","[PK=1]");
		myHCountry.put("777777602003771","[MX=1]");
		myHCountry.put("777777603401700","[OM=1]");
		myHCountry.put("777777604000171","[ZW=1]");
		myHCountry.put("777777605761771","[BR=1]");
		myHCountry.put("777777606000171","[FR=1]");
		myHCountry.put("777777606000371","[GF=1]");
		myHCountry.put("777777606041771","[OM=1]");
		myHCountry.put("777777607000000","[BR=1]");
		myHCountry.put("777777607000031","[BR=1]");
		myHCountry.put("777777607200771","[BR=1]");
		myHCountry.put("777777607400071","[RU=1]");
		myHCountry.put("777777607403171","[EE=1]");
		myHCountry.put("777777607500771","[IT=1]");
		myHCountry.put("777777607610031","[SD=1]");
		myHCountry.put("777777607700001","[BE=1]");
		myHCountry.put("777777607703771","[ZA=1]");
		myHCountry.put("777777607761771","[US=1]");
		myHCountry.put("777777607777771","[BE=1]");
		myHCountry.put("777777620000000","[IT=1]");
		myHCountry.put("777777637400071","[FR=1]");
		myHCountry.put("777777640000000","[US=1]");
		myHCountry.put("777777640000271","[IT=1]");
		myHCountry.put("777777640000771","[IN=1]");
		myHCountry.put("777777660000131","[BR=1]");
		myHCountry.put("777777663600171","[KG=1]");
		myHCountry.put("777777666000771","[ES=1]");
		myHCountry.put("777777666003771","[IN=1]");
		myHCountry.put("777777676561771","[FR=1]");
		myHCountry.put("777777676761771","[IN=1]");
		myHCountry.put("777777677577771","[JP=1]");
		myHCountry.put("777777677777671","[US=1]");
		myHCountry.put("777777700000001","[ID=1]");
		myHCountry.put("777777700000611","[BR=1]");
		myHCountry.put("777777700003070","[MY=1]");
		myHCountry.put("777777700003341","[OM=1]");
		myHCountry.put("777777702003531","[OM=1]");
		myHCountry.put("777777702003771","[IT=1]");
		myHCountry.put("777777704303771","[CN=1]");
		myHCountry.put("777777707700371","[AT=1]");
		myHCountry.put("777777707741770","[TN=1]");
		myHCountry.put("777777707741771","[IT=1]");
		myHCountry.put("777777720000171","[DE=1]");
		myHCountry.put("777777730000771","[NL=1]");
		myHCountry.put("777777740000011","[NL=1]");
		myHCountry.put("777777740000031","[IT=1]");
		myHCountry.put("777777740000071","[FR=1]");
		myHCountry.put("777777740003171","[US=1]");
		myHCountry.put("777777740003371","[BD=1]");
		myHCountry.put("777777740403371","[BD=1]");
		myHCountry.put("777777742000071","[FI=1]");
		myHCountry.put("777777742003371","[GF=1]");
		myHCountry.put("777777742003771","[OM=1]");
		myHCountry.put("777777744003671","[IN=1]");
		myHCountry.put("777777747700771","[SE=1]");
		myHCountry.put("777777747743771","[SA=1]");
		myHCountry.put("777777747764771","[NG=1]");
		myHCountry.put("777777750000371","[GE=1]");
		myHCountry.put("777777750000471","[IT=1]");
		myHCountry.put("777777750400371","[US=1]");
		myHCountry.put("777777750403371","[MM=1]");
		myHCountry.put("777777752000071","[US=1]");
		myHCountry.put("777777752003371","[OM=1]");
		myHCountry.put("777777752403371","[ES=1]");
		myHCountry.put("777777753400671","[MM=1]");
		myHCountry.put("777777757003771","[BD=1]");
		myHCountry.put("777777757400000","[NL=1]");
		myHCountry.put("777777757400011","[ZA=1]");
		myHCountry.put("777777757400071","[BD=1]");
		myHCountry.put("777777757403711","[JP=1]");
		myHCountry.put("777777757600000","[EE=1]");
		myHCountry.put("777777757761771","[CM=1]");
		myHCountry.put("777777760000011","[US=1]");
		myHCountry.put("777777760000131","[MX=1]");
		myHCountry.put("777777760000731","[TW=1]");
		myHCountry.put("777777761700000","[AT=1]");
		myHCountry.put("777777763000371","[AT=1]");
		myHCountry.put("777777766000771","[ES=1]");
		myHCountry.put("777777767400071","[US=1]");
		myHCountry.put("777777767600000","[US=1]");
		myHCountry.put("777777767605571","[MY=1]");
		myHCountry.put("777777767777771","[TW=1]");
		myHCountry.put("777777770000011","[GF=1]");
		myHCountry.put("777777770000311","[ES=1]");
		myHCountry.put("777777770000631","[FR=1]");
		myHCountry.put("777777770000711","[BR=1]");
		myHCountry.put("777777770003171","[US=1]");
		myHCountry.put("777777770003731","[AU=1]");
		myHCountry.put("777777770200771","[BR=1]");
		myHCountry.put("777777771700000","[ID=1]");
		myHCountry.put("777777773774731","[IN=1]");
		myHCountry.put("777777774000031","[FR=1]");
		myHCountry.put("777777774000131","[IT=1]");
		myHCountry.put("777777774000731","[LK=1]");
		myHCountry.put("777777774003771","[LK=1]");
		myHCountry.put("777777774004771","[ZA=1]");
		myHCountry.put("777777774021771","[MG=1]");
		myHCountry.put("777777774025471","[SA=1]");
		myHCountry.put("777777774400771","[FR=1]");
		myHCountry.put("777777774600771","[AL=1]");
		myHCountry.put("777777775000011","[CI=1]");
		myHCountry.put("777777775403731","[AU=1]");
		myHCountry.put("777777775600000","[BR=1]");
		myHCountry.put("777777776000171","[HU=1]");
		myHCountry.put("777777776000731","[FI=1]");
		myHCountry.put("777777776001700","[IN=1]");
		myHCountry.put("777777776003731","[TH=1]");
		myHCountry.put("777777776003771","[US=1]");
		myHCountry.put("777777776403731","[US=1]");
		myHCountry.put("777777776700000","[US=1]");
		myHCountry.put("777777776761771","[SA=1]");
		myHCountry.put("777777777000000","[VE=1]");
		myHCountry.put("777777777000031","[VE=1]");
		myHCountry.put("777777777000171","[SE=1]");
		myHCountry.put("777777777000700","[NL=1]");
		myHCountry.put("777777777003700","[LK=1]");
		myHCountry.put("777777777003731","[NL=1]");
		myHCountry.put("777777777300471","[ID=1]");
		myHCountry.put("777777777402077","[IT=1]");
		myHCountry.put("777777777403700","[IN=1]");
		myHCountry.put("777777777437771","[MG=1]");
		myHCountry.put("777777777503771","[IN=1]");
		myHCountry.put("777777777543771","[IN=1]");
		myHCountry.put("777777777567771","[JP=1]");
		myHCountry.put("777777777600000","[DE=1]");
		myHCountry.put("777777777600011","[US=1]");
		myHCountry.put("777777777600031","[PL=1]");
		myHCountry.put("777777777600071","[US=1]");
		myHCountry.put("777777777600171","[NL=1]");
		myHCountry.put("777777777600471","[US=1]");
		myHCountry.put("777777777600700","[BR=1]");
		myHCountry.put("777777777600760","[US=1]");
		myHCountry.put("777777777601731","[MY=1]");
		myHCountry.put("777777777641771","[MY=1]");
		myHCountry.put("777777777700001","[DE=1]");
		myHCountry.put("777777777700160","[AL=1]");
		myHCountry.put("777777777700231","[FR=1]");
		myHCountry.put("777777777700331","[HU=1]");
		myHCountry.put("777777777700571","[TR=1]");
		myHCountry.put("777777777700760","[AL=1]");
		myHCountry.put("777777777703761","[CN=1]");
		myHCountry.put("777777777703771","[BR=1]");
		myHCountry.put("777777777730351","[SD=1]");
		myHCountry.put("777777777731771","[SD=1]");
		myHCountry.put("777777777741671","[AT=1]");
		myHCountry.put("777777777741771","[MY=1]");
		myHCountry.put("777777777765771","[BD=1]");
		myHCountry.put("777777777766771","[NG=1]");
		myHCountry.put("777777777770000","[RU=1]");
		myHCountry.put("777777777770031","[IN=1]");
		myHCountry.put("777777777770231","[BR=1]");
		myHCountry.put("777777777770331","[US=1]");
		myHCountry.put("777777777770370","[AT=1]");
		myHCountry.put("777777777770371","[SD=1]");
		myHCountry.put("777777777774731","[NG=1]");
		myHCountry.put("777777777775771","[CN=1]");
		myHCountry.put("777777777776071","[SA=1]");
		myHCountry.put("777777777777600","[DK=1]");
		myHCountry.put("777777777777601","[US=1]");
		myHCountry.put("777777777777751","[VN=1]");
		myHCountry.put("374777777420771","[BG=1]");
		myHCountry.put("376377777420770","[US=1]");
		myHCountry.put("377737777420771","[FI=1]");
		myHCountry.put("673777777420731","[ET=1]");
		myHCountry.put("677737777420771","[FR=1]");
		myHCountry.put("711007676420621","[RU=1]");
		myHCountry.put("731737777420771","[CN=1]");
		myHCountry.put("733737673420771","[ES=1]");
		myHCountry.put("740003777420771","[FR=1]");
		myHCountry.put("740007777420771","[RU=1]");
		myHCountry.put("754777777420771","[IT=1]");
		myHCountry.put("757737777420771","[RU=1]");
		myHCountry.put("757740007420771","[US=1]");
		myHCountry.put("757777777420771","[US=1]");
		myHCountry.put("760337777420771","[RU=1]");
		myHCountry.put("763737777420771","[LV=1]");
		myHCountry.put("763777777420771","[ET=1]");
		myHCountry.put("764777777420771","[US=1]");
		myHCountry.put("767777777420731","[ET=1]");
		myHCountry.put("767777777420771","[IR=1]");
		myHCountry.put("770017177420771","[RU=1]");
		myHCountry.put("770337777420771","[FR=1]");
		myHCountry.put("770777777420771","[BE=1]");
		myHCountry.put("771777777420771","[EE=1]");
		myHCountry.put("774776777420771","[IR=1]");
		myHCountry.put("774777000420771","[NL=1]");
		myHCountry.put("774777207420771","[KG=1]");
		myHCountry.put("774777600420731","[RU=1]");
		myHCountry.put("774777607420771","[TN=1]");
		myHCountry.put("774777775420771","[TR=1]");
		myHCountry.put("774777776420771","[RU=1]");
		myHCountry.put("774777777420700","[TR=1]");
		myHCountry.put("776777777420771","[IR=1]");
		myHCountry.put("777237777420771","[GE=1]");
		myHCountry.put("777353777420331","[BE=1]");
		myHCountry.put("777417777420771","[AT=1]");
		myHCountry.put("777727777420771","[IQ=1]");
		myHCountry.put("777735777420771","[RU=1]");
		myHCountry.put("777736777420771","[SE=1]");
		myHCountry.put("777737017420771","[BE=1]");
		myHCountry.put("777737377420771","[AT=1]");
		myHCountry.put("777737577420771","[SE=1]");
		myHCountry.put("777737703420771","[FR=1]");
		myHCountry.put("777737775420771","[DE=1]");
		myHCountry.put("777737777420371","[RU=1]");
		myHCountry.put("777737777420711","[US=1]");
		myHCountry.put("777737777420770","[KZ=1]");
		myHCountry.put("777757777420611","[BR=1]");
		myHCountry.put("777767777420731","[ET=1]");
		myHCountry.put("777777377420771","[ET=1]");
		myHCountry.put("777777407420771","[IN=1]");
		myHCountry.put("777777747420771","[ET=1]");
		myHCountry.put("777777763420771","[ET=1]");
		myHCountry.put("777777777420311","[ET=1]");
		myHCountry.put("777777777420371","[RU=1]");
		myHCountry.put("777777777420601","[OM=1]");
		myHCountry.put("777777777420631","[BE=1]");
		myHCountry.put("777777777420770","[ET=1]");
		myHCountry.put("000000007420731","[SE=1]");
		myHCountry.put("000000007420771","[IR=1]");
		myHCountry.put("000377777420771","[IR=1]");
		myHCountry.put("003777777420771","[NL=1]");
		myHCountry.put("015777777420771","[NL=1]");
		myHCountry.put("034777777420771","[BE=1]");
		myHCountry.put("077737777420771","[RU=1]");
		myHCountry.put("077777777420771","[RU=1]");
		myHCountry.put("130257677420771","[GE=1]");
		myHCountry.put("177767377420771","[IR=1]");
		myHCountry.put("177777777420671","[IQ=1]");
		myHCountry.put("417777377420771","[GB=1]");
		myHCountry.put("434777747420771","[AT=1]");
		myHCountry.put("477777777420711","[IR=1]");
		myHCountry.put("507777777420771","[IR=1]");
		myHCountry.put("557777777420731","[IR=1]");
		myHCountry.put("573767777420771","[SA=1]");
		myHCountry.put("575761777420771","[CN=1]");
		myHCountry.put("576177777420771","[NL=1]");
		myHCountry.put("577017777420771","[IR=1]");
		myHCountry.put("577077777420771","[IR=1]");
		myHCountry.put("577600017420771","[IN=1]");
		myHCountry.put("577637677420771","[RU=1]");
		myHCountry.put("577727777420771","[IQ=1]");
		myHCountry.put("577737775420771","[PK=1]");
		myHCountry.put("577737777420571","[US=1]");
		myHCountry.put("577757777420771","[IR=1]");
		myHCountry.put("577767777420731","[ET=1]");
		myHCountry.put("577773776420771","[IR=1]");
		myHCountry.put("577773777420771","[IR=1]");
		myHCountry.put("577774777420771","[US=1]");
		myHCountry.put("577777717420771","[KG=1]");
		myHCountry.put("577777737420771","[CN=1]");
		myHCountry.put("577777776420731","[FR=1]");
		myHCountry.put("577777777420611","[AT=1]");
		myHCountry.put("000016777760671","[US=1]");
		myHCountry.put("000076775760671","[MY=1]");
		myHCountry.put("001776777760771","[US=1]");
		myHCountry.put("003776777760771","[IT=1]");
		myHCountry.put("007776777760771","[US=1]");
		myHCountry.put("077776777760731","[GB=1]");
		myHCountry.put("117756777760771","[US=1]");
		myHCountry.put("377776777760571","[BG=1]");
		myHCountry.put("377776777760771","[NL=1]");
		myHCountry.put("417176777760731","[IT=1]");
		myHCountry.put("417336777760731","[VE=1]");
		myHCountry.put("555346437740061","[AL=1]");
		myHCountry.put("557346637740261","[AL=1]");
		myHCountry.put("577366737560621","[SA=1]");
		myHCountry.put("577776744760771","[BR=1]");
		myHCountry.put("577776777760771","[IT=1]");
		myHCountry.put("617772777760771","[BE=1]");
		myHCountry.put("637676777760131","[AT=1]");
		myHCountry.put("637776777760771","[AT=1]");
		myHCountry.put("657776777760771","[US=1]");
		myHCountry.put("677336777760771","[FR=1]");
		myHCountry.put("677576777760571","[AT=1]");
		myHCountry.put("677776777560771","[ZA=1]");
		myHCountry.put("677776777760571","[SE=1]");
		myHCountry.put("700736777760771","[MX=1]");
		myHCountry.put("707776777760771","[US=1]");
		myHCountry.put("720036777760731","[US=1]");
		myHCountry.put("730036777760731","[VE=1]");
		myHCountry.put("730036777760771","[NL=1]");
		myHCountry.put("737376777760771","[BR=1]");
		myHCountry.put("737776677760771","[DZ=1]");
		myHCountry.put("740376777760771","[GB=1]");
		myHCountry.put("760002777760771","[CO=1]");
		myHCountry.put("760356777760701","[AR=1]");
		myHCountry.put("770016777760771","[US=1]");
		myHCountry.put("770346627740371","[MX=1]");
		myHCountry.put("770376777760701","[BR=1]");
		myHCountry.put("771776777660771","[MX=1]");
		myHCountry.put("773776777760771","[US=1]");
		myHCountry.put("774036777760700","[MG=1]");
		myHCountry.put("776776777360771","[US=1]");
		myHCountry.put("777036777760771","[AR=1]");
		myHCountry.put("777076777760731","[TZ=1]");
		myHCountry.put("777406777760771","[US=1]");
		myHCountry.put("777436777760731","[MX=1]");
		myHCountry.put("777566777760771","[MZ=1]");
		myHCountry.put("777606777760771","[US=1]");
		myHCountry.put("777616777760771","[RU=]");
		myHCountry.put("777736707760771","[US=1]");
		myHCountry.put("777736777360771","[US=1]");
		myHCountry.put("777736777560731","[US=1]");
		myHCountry.put("777736777760571","[TR=1]");
		myHCountry.put("777736777760751","[ET=1]");
		myHCountry.put("777742777760671","[US=1]");
		myHCountry.put("777756777760301","[BE=1]");
		myHCountry.put("777756777760671","[BE=1]");
		myHCountry.put("777756777760731","[US=1]");
		myHCountry.put("777766637760771","[IQ=1]");
		myHCountry.put("777766737760571","[DE=1]");
		myHCountry.put("777766777060771","[IN=1]");
		myHCountry.put("777766777760071","[ES=1]");
		myHCountry.put("777766777760631","[GP=1]");
		myHCountry.put("777766777760731","[ES=1]");
		myHCountry.put("777776417760771","[US=1]");
		myHCountry.put("777776437760771","[US=1]");
		myHCountry.put("777776617560771","[ZA=1]");
		myHCountry.put("777776677760711","[NL=1]");
		myHCountry.put("777776707760771","[MQ=1]");
		myHCountry.put("777776727760771","[NL=1]");
		myHCountry.put("777776737560771","[ZA=1]");
		myHCountry.put("777776756360711","[US=1]");
		myHCountry.put("777776767760571","[IT=1]");
		myHCountry.put("777776775760571","[US=1]");
		myHCountry.put("777776775760700","[PE=1]");
		myHCountry.put("777776775760731","[US=1]");
		myHCountry.put("777776777160771","[AT=1]");
		myHCountry.put("777776777740771","[GB=1]");
		myHCountry.put("777776777760351","[BE=1]");
		myHCountry.put("777776777760631","[US=1]");
		myHCountry.put("777776777760641","[BR=1]");
		myHCountry.put("777776777760661","[LV=1]");
		myHCountry.put("037766677760601","[US=1]");
		myHCountry.put("177776407760401","[US=1]");
		myHCountry.put("437776777760401","[GB=1]");
		myHCountry.put("515346436640201","[IT=1]");
		myHCountry.put("517346636540201","[IT=1]");
		myHCountry.put("577746616540601","[IT=1]");
		myHCountry.put("577746617760601","[IT=1]");
		myHCountry.put("577756777760601","[ZW=1]");
		myHCountry.put("577766777760601","[IN=1]");
		myHCountry.put("577776774360601","[US=1]");
		myHCountry.put("616762777560600","[FR=1]");
		myHCountry.put("676776777760601","[BR=1]");
		myHCountry.put("701776617760601","[US=1]");
		myHCountry.put("737766777760601","[ZA=1]");
		myHCountry.put("747776777760601","[IN=1]");
		myHCountry.put("770056676160000","[BR=1]");
		myHCountry.put("773776407760601","[US=1]");
		myHCountry.put("776776707760601","[US=1]");
		myHCountry.put("777236777760601","[ZA=1]");
		myHCountry.put("777356777760601","[GR=1]");
		myHCountry.put("777376776360601","[US=1]");
		myHCountry.put("777376777560601","[VE=1]");
		myHCountry.put("777676776360601","[US=1]");
		myHCountry.put("777736407760601","[US=1]");
		myHCountry.put("777746777760601","[US=1]");
		myHCountry.put("777762777760601","[GB=1]");
		myHCountry.put("777766407760601","[US=1]");
		myHCountry.put("777766777760600","[FR=1]");
		myHCountry.put("777776404160601","[US=1]");
		myHCountry.put("777776477760601","[ES=1]");
		myHCountry.put("777776577760601","[BR=1]");
		myHCountry.put("777776675760601","[FR=1]");
		myHCountry.put("777776775760600","[PE=1]");
		myHCountry.put("777776776160601","[ES=1]");
		myHCountry.put("777776777740001","[US=1]");
		myHCountry.put("777776777740601","[US=1]");
		myHCountry.put("777776777760401","[US=1]");
		myHCountry.put("100036677760771","[PE=1]");
		myHCountry.put("500006636540621","[IT=1]");
		myHCountry.put("500006677560671","[IT=1]");
		myHCountry.put("500016477740671","[IT=1]");
		myHCountry.put("500046637740661","[IN=1]");
		myHCountry.put("500066477760611","[AR=1]");
		myHCountry.put("500066777740671","[AR=1]");
		myHCountry.put("700036477760731","[PE=1]");
		myHCountry.put("700036775560771","[PE=1]");
		myHCountry.put("700036775760771","[CO=1]");
		myHCountry.put("700036777560471","[US=1]");
		myHCountry.put("700036777760631","[US=1]");
		myHCountry.put("700036777760711","[US=1]");
		myHCountry.put("700046677760671","[ZW=1]");
		myHCountry.put("700056777740371","[BE=1]");
		myHCountry.put("700056777760771","[ZA=1]");
		myHCountry.put("700066777760661","[ZM=1]");
		myHCountry.put("700066777760700","[FR=1]");
		myHCountry.put("700076657760771","[US=1]");
		myHCountry.put("700076677760771","[FR=1]");
		myHCountry.put("700076717760700","[LC=1]");
		myHCountry.put("700076747760771","[ZA=1]");
		myHCountry.put("700076777740000","[US=1]");
		myHCountry.put("700076777740271","[MX=1]");
		myHCountry.put("700076777760760","[IT=1]");
		myHCountry.put("704036776360771","[PE=1]");
		myHCountry.put("177777000000011","[NL=1]");
		myHCountry.put("741777000000011","[VN=1]");

		
		if(!myHCountry.containsKey(spoligotype)){
			return "ND";
		}
		else{
			return myHCountry.get(spoligotype);
		}
	}
}